@@ -35,7 +35,7 @@
 #define VER_TYPE	VER_BD
 #define VERSION		"0.9.1"
 #define AUTORS		"Roman Savochenko"
-#define DESCRIPTION	"DB modul. Allow support of the DB FireBird."
+#define DESCRIPTION	"DB module. Provides support of the DB FireBird."
 #define LICENSE		"GPL"
 //******************************************************************************
 
@@ -86,42 +86,11 @@ TBD *BDMod::openBD( const string &iid )
     return new MBD(iid,&owner().openDB_E());
 }
 
-string BDMod::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n"
-	"\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void BDMod::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
-
-    //- Load parameters from config file -=
+    //> Load parameters from config file
 
 }
 
