@@ -23,6 +23,7 @@
 #include <string>
 #include <mysql/mysql.h>
 #include <mysql/errmsg.h>
+#include <mysql/mysqld_error.h>
 
 #include <tsys.h>
 #include <tmess.h>
@@ -35,9 +36,9 @@
 #define MOD_NAME	"DB MySQL"
 #define MOD_TYPE	"BD"
 #define VER_TYPE	VER_BD
-#define VERSION		"1.6.0"
+#define VERSION		"1.6.1"
 #define AUTORS		"Roman Savochenko"
-#define DESCRIPTION	"BD modul. Allow support of the BD MySQL."
+#define DESCRIPTION	"BD module. Provides support of the BD MySQL."
 #define MOD_LICENSE	"GPL"
 //************************************************
 
@@ -88,39 +89,10 @@ TBD *BDMod::openBD( const string &name )
     return new MBD(name,&owner().openDB_E());
 }
 
-string BDMod::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void BDMod::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 }
 
 //************************************************
@@ -167,6 +139,7 @@ void MBD::enable( )
     bd   = TSYS::strSepParse(addr(),3,';');
     port = atoi(TSYS::strSepParse(addr(),4,';').c_str());
     u_sock = TSYS::strSepParse(addr(),5,';');
+    names = TSYS::strSepParse(addr(),6,';');
     cd_pg  = codePage().size()?codePage():Mess->charset();
 
     if(!mysql_init(&connect))
@@ -177,8 +150,8 @@ void MBD::enable( )
 
     TBD::enable( );
 
-    string req = "CREATE DATABASE IF NOT EXISTS `"+TSYS::strEncode(bd,TSYS::SQL)+"`";
-    sqlReq(req);
+    sqlReq("CREATE DATABASE IF NOT EXISTS `"+TSYS::strEncode(bd,TSYS::SQL)+"`");
+    if( !names.empty() ) sqlReq("SET NAMES '"+names+"'");
 }
 
 void MBD::disable( )
@@ -221,6 +194,7 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
     ResAlloc resource(conn_res,true);
 
     int irez;
+    rep:
     if( irez = mysql_real_query(&connect,req.c_str(),req.size()) )
     {
 	if( irez == CR_SERVER_GONE_ERROR || irez == CR_SERVER_LOST )
@@ -233,6 +207,13 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
 	}
 	if( irez )
 	{
+	    if( mysql_errno(&connect) == ER_NO_DB_ERROR )
+	    {
+		resource.release();
+		sqlReq("USE `"+TSYS::strEncode(bd,TSYS::SQL)+"`");
+		resource.request(true);
+		goto rep;
+	    }
 	    //mess_debug(nodePath().c_str(),_("Query <%s> is error."),ireq.c_str());
 	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Query to DB error %d: %s"),irez,mysql_error(&connect));
 	}
@@ -248,7 +229,7 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
 
 	vector<string> fld;
 	//- Add head -
-	for(int i=0; i < num_fields; i++)
+	for( int i=0; i < num_fields; i++ )
 	    fld.push_back(mysql_fetch_field_direct(res,i)->name);
 	tbl->push_back(fld);
 	//- Add data -
@@ -272,15 +253,16 @@ void MBD::cntrCmdProc( XMLNode *opt )
 	TBD::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",2,
 	    "tp","str","help",
-	    _("MySQL DB address must be written as: [<host>;<user>;<pass>;<db>;<port>;<u_sock>].\n"
+	    _("MySQL DB address must be written as: [<host>;<user>;<pass>;<db>;<port>;<u_sock>;<names>].\n"
 	      "Where:\n"
 	      "  host - MySQL server hostname;\n"
 	      "  user - DB user name;\n"
 	      "  pass - user's password for DB access;\n"
 	      "  db - DB name;\n"
 	      "  port - DB server port (default 3306);\n"
-	      "  u_sock - UNIX-socket name, for local access to DB (/var/lib/mysql/mysql.sock).\n"
-	      "For local DB: [;roman;123456;OpenSCADA;;/var/lib/mysql/mysql.sock].\n"
+	      "  u_sock - UNIX-socket name, for local access to DB (/var/lib/mysql/mysql.sock);\n"
+	      "  names - MySQL SET NAMES charset.\n"
+	      "For local DB: [;roman;123456;OpenSCADA;;/var/lib/mysql/mysql.sock;utf8].\n"
 	      "For remote DB: [server.nm.org;roman;123456;OpenSCADA;3306]."));
 	return;
     }
@@ -314,13 +296,6 @@ MTable::~MTable(  )
 
 }
 
-int MTable::fieldLen( int len )
-{
-    if( owner().cd_pg.find("UTF") != string::npos || owner().cd_pg.find("utf") != string::npos )
-	return len*2;
-    return len;
-}
-
 void MTable::postDisable(int flag)
 {
     if( flag )
@@ -374,22 +349,20 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
     mLstUse = time(NULL);
 
     string sid;
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
     //> Make SELECT and WHERE
     string req = "SELECT ";
     string req_where = "WHERE ";
     //> Add use keys to list
-    bool first_sel = true;
-    bool next = false;
+    bool first_sel = true, next = false, trPresent = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
 	sid = tblStrct[i_fld][0];
 	TCfg *u_cfg = cfg.at(sid,true);
-	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
+	if( !cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
-	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText) ) continue;
+	    trPresent = true;
 	}
 	if( !u_cfg ) continue;
 
@@ -416,7 +389,7 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
 	sid = tbl[0][i_fld];
 	TCfg *u_cfg = cfg.at(sid,true);
 	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
-	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
+	else if( trPresent && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") && tbl[1][i_fld].size() )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
 	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
@@ -434,22 +407,20 @@ void MTable::fieldGet( TConfig &cfg )
     mLstUse = time(NULL);
 
     string sid;
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
     //> Prepare request
     string req = "SELECT ";
     string req_where;
     //>> Add fields list to queue
-    bool first_sel = true;
-    bool next_wr = false;
+    bool first_sel = true, next_wr = false, trPresent = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
 	sid = tblStrct[i_fld][0];
 	TCfg *u_cfg = cfg.at(sid,true);
-	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
+	if( !cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
-	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText) ) continue;
+	    trPresent = true;
 	}
 	if( !u_cfg ) continue;
 
@@ -467,7 +438,6 @@ void MTable::fieldGet( TConfig &cfg )
     req = req + " FROM `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` WHERE " + req_where;
 
     //> Query
-    //printf("TEST 01: query: <%s>\n",req.c_str());
     owner().sqlReq( req, &tbl );
     if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row is not present!"));
 
@@ -477,7 +447,7 @@ void MTable::fieldGet( TConfig &cfg )
 	sid = tbl[0][i_fld];
 	TCfg *u_cfg = cfg.at(sid,true);
 	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
-	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
+	else if( trPresent && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") && tbl[1][i_fld].size() )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
 	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
@@ -493,12 +463,25 @@ void MTable::fieldSet( TConfig &cfg )
     mLstUse = time(NULL);
 
     string sid, sval;
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && !cfg.noTransl() && Mess->lang2Code() != Mess->lang2CodeBase());
     //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
+    //> Check for translation present
+    bool trPresent = isVarTextTransl, trDblDef = false;
+    for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
+    {
+	if( (trPresent || cfg.noTransl()) && (!isVarTextTransl || trDblDef) ) break;
+	sid = tblStrct[i_fld][0];
+	if( sid.size() > 3 )
+	{
+	    if( !trPresent && sid.substr(0,3) == (Mess->lang2Code()+"#") ) trPresent = true;
+	    if( isVarTextTransl && !trDblDef && sid.substr(0,3) == (Mess->lang2CodeBase()+"#") ) trDblDef = true;
+	}
+    }
+    if( trDblDef ) fieldFix(cfg);
+
     //> Get present fields list
     string req_where = "WHERE ";
     //>> Add key list to query
@@ -527,11 +510,12 @@ void MTable::fieldSet( TConfig &cfg )
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( !(u_cfg.fld().flg()&TCfg::Key) && !u_cfg.view() ) continue;
 
-	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
-	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
-	    ins_name = ins_name + (next?",`":"`") + TSYS::strEncode(sid,TSYS::SQL) + "` ";
-	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
-	    ins_value = ins_value + (next?",'":"'") + TSYS::strEncode(sval,TSYS::SQL) + "' ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && trPresent && !u_cfg.noTransl());
+	    ins_name = ins_name + (next?",`":"`") + TSYS::strEncode(cf_el[i_el],TSYS::SQL) + "` " +
+		( isTransl ? (",`" + TSYS::strEncode(Mess->lang2Code()+"#"+cf_el[i_el],TSYS::SQL) + "` ") : "" );
+	    sval = getVal(u_cfg);
+	    ins_value = ins_value + (next?",'":"'") + TSYS::strEncode(sval,TSYS::SQL) + "' " +
+		( isTransl ? (",'" + TSYS::strEncode(sval,TSYS::SQL) + "' ") : "" );
 	    next = true;
 	}
 	req = req + "("+ins_name+") VALUES ("+ins_value+")";
@@ -546,9 +530,9 @@ void MTable::fieldSet( TConfig &cfg )
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( u_cfg.fld().flg()&TCfg::Key || !u_cfg.view() ) continue;
 
-	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
-	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
-	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && trPresent && !u_cfg.noTransl());
+	    sid = isTransl ? (Mess->lang2Code()+"#"+cf_el[i_el]) : cf_el[i_el];
+	    sval = getVal(u_cfg);
 	    req = req + (next?",`":"`") + TSYS::strEncode(sid,TSYS::SQL) + "`='" + TSYS::strEncode(sval,TSYS::SQL) + "' ";
 	    next = true;
 	}
@@ -556,7 +540,6 @@ void MTable::fieldSet( TConfig &cfg )
     }
 
     //> Query
-    //printf("TEST 02: query: <%s>\n",req.c_str());
     try{ owner().sqlReq( req ); }
     catch(TError err)	{ fieldFix(cfg); owner().sqlReq( req ); }
 }
@@ -593,8 +576,8 @@ void MTable::fieldFix( TConfig &cfg )
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty!"));
 
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //string lang2Code = cfg.noTransl( ) ? "" : Mess->lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && !cfg.noTransl() && Mess->lang2Code() != Mess->lang2CodeBase());
     //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
@@ -628,9 +611,9 @@ void MTable::fieldFix( TConfig &cfg )
 	{
 	    switch(u_cfg.fld().type())
 	    {
-		    if( fieldLen(u_cfg.fld().len()) < 256 || u_cfg.fld().flg()&TCfg::Key )
-			f_tp = "varchar("+TSYS::int2str(vmax(1,vmin((u_cfg.fld().flg()&TCfg::Key)?200:255,fieldLen(u_cfg.fld().len()))))+")";
-		    else if( fieldLen(u_cfg.fld().len()) < 65536 )
+		    if( u_cfg.fld().len() < 256 || u_cfg.fld().flg()&TCfg::Key )
+			f_tp = "varchar("+TSYS::int2str(vmax(1,vmin((u_cfg.fld().flg()&TCfg::Key)?200:255,u_cfg.fld().len())))+")";
+		    else if( u_cfg.fld().len() < 65536 )
 			f_tp = "text";
 		    else f_tp = "mediumtext";
 		    break;
@@ -664,7 +647,7 @@ void MTable::fieldFix( TConfig &cfg )
 	{
 	    bool col_cur = false;
 	    for( int i_c = i_fld; i_c < tblStrct.size(); i_c++ )
-		if( tblStrct[i_c][0].size() > 3 && tblStrct[i_c][0].substr(2) == ("#"+cf_el[i_cf]) )
+		if( tblStrct[i_c][0].size() > 3 && tblStrct[i_c][0].substr(2) == ("#"+cf_el[i_cf]) && tblStrct[i_c][0].substr(0,2) != Mess->lang2CodeBase() )
 		{
 		    if( tblStrct[i_c][1] != f_tp )
 		    {
@@ -672,11 +655,11 @@ void MTable::fieldFix( TConfig &cfg )
 			next = true;
 			fieldPrmSet(u_cfg,(i_cf>0)?cf_el[i_cf-1]:"",req);
 		    }
-		    if( tblStrct[i_c][0].substr(0,2) == lang2Code ) col_cur = true;
+		    if( tblStrct[i_c][0].substr(0,2) == Mess->lang2Code() ) col_cur = true;
 		}
 	    if( !col_cur && isVarTextTransl )
 	    {
-		req = req + (next?",ADD `":"ADD `") + TSYS::strEncode(lang2Code+"#"+cf_el[i_cf],TSYS::SQL) + "` ";
+		req = req + (next?",ADD `":"ADD `") + TSYS::strEncode(Mess->lang2Code()+"#"+cf_el[i_cf],TSYS::SQL) + "` ";
 		next = true;
 		fieldPrmSet(u_cfg,(i_cf>0)?cf_el[i_cf-1]:"",req);
 	    }
@@ -688,7 +671,8 @@ void MTable::fieldFix( TConfig &cfg )
 	int i_cf;
 	for( i_cf = 0; i_cf < cf_el.size(); i_cf++ )
 	    if( cf_el[i_cf] == tblStrct[i_fld][0] ||
-		    (cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::TransltText && tblStrct[i_fld][0].size() > 3 && tblStrct[i_fld][0].substr(2) == ("#"+cf_el[i_cf])) )
+		    (cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::TransltText && tblStrct[i_fld][0].size() > 3 && 
+		    tblStrct[i_fld][0].substr(2) == ("#"+cf_el[i_cf]) && tblStrct[i_fld][0].substr(0,2) != Mess->lang2CodeBase()) )
 		break;
 	if( i_cf >= cf_el.size() )
 	{
@@ -713,10 +697,10 @@ void MTable::fieldPrmSet( TCfg &cfg, const string &last, string &req )
     switch(cfg.fld().type())
     {
 	case TFld::String:
-	    if( fieldLen(cfg.fld().len()) < 256 || cfg.fld().flg()&TCfg::Key )
-		req=req+"varchar("+SYS->int2str(vmax(1,vmin((cfg.fld().flg()&TCfg::Key)?200:255,fieldLen(cfg.fld().len()))))+") "+
+	    if( cfg.fld().len() < 256 || cfg.fld().flg()&TCfg::Key )
+		req=req+"varchar("+SYS->int2str(vmax(1,vmin((cfg.fld().flg()&TCfg::Key)?200:255,cfg.fld().len())))+") "+
 			((cfg.fld().flg()&TCfg::Key)?"BINARY":"")+" NOT NULL DEFAULT '"+cfg.fld().def()+"' ";
-	    else if( fieldLen(cfg.fld().len()) < 65536 )
+	    else if( cfg.fld().len() < 65536 )
 		req=req+"text NOT NULL ";// DEFAULT '"+cfg.fld().def()+"' ";
 	    else req=req+"mediumtext NOT NULL ";// DEFAULT '"+cfg.fld().def()+"' ";
 	    break;
