@@ -35,9 +35,9 @@
 #define MOD_NAME	"DB SQLite"
 #define MOD_TYPE	"BD"
 #define VER_TYPE	VER_BD
-#define VERSION		"1.6.0"
+#define VERSION		"1.6.1"
 #define AUTORS		"Roman Savochenko"
-#define DESCRIPTION	"BD modul. Allow support of the BD SQLite."
+#define DESCRIPTION	"BD module. Provides support of the BD SQLite."
 #define LICENSE		"GPL"
 //******************************************************************************
 
@@ -88,42 +88,11 @@ TBD *BDMod::openBD( const string &iid )
     return new MBD(iid,&owner().openDB_E());
 }
 
-string BDMod::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n"
-	"\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void BDMod::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
-
-    //- Load parameters from config file -=
+    //> Load parameters from config file
 
 }
 
@@ -180,12 +149,12 @@ void MBD::disable( )
 {
     if( !enableStat() )  return;
 
-    //- Last commit -
+    //> Last commit
     if(commCnt) { commCnt = trans_reqs; sqlReq(""); }
 
     TBD::disable( );
 
-    //- Close DB -
+    //> Close DB
     ResAlloc res(conn_res,true);
     sqlite3_close(m_db);
 }
@@ -217,8 +186,7 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
     if( tbl ) tbl->clear();
     if(!enableStat())	return;
 
-    //printf("TEST 03: query: <%s>\n",req.c_str());
-    //- Commit set -
+    //> Commit set
     string req = ireq;
 
     ResAlloc res(conn_res,true);
@@ -231,22 +199,22 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
 	    commCnt=0;
 	}
     }
-    //- Put request -
+    //> Put request
     rc = sqlite3_get_table(m_db,Mess->codeConvOut(cd_pg.c_str(),req).c_str(),&result, &nrow, &ncol, &zErrMsg );
     if( rc != SQLITE_OK )
     {
-	//-- Fix transaction --
+	//>> Fix transaction
 	if( trans_reqs>1 && (commCnt-1)<0 )	commCnt=trans_reqs;
+	//mess_err(nodePath().c_str(),_("Request error: %s"),req.c_str());
 	throw TError(100+rc,nodePath().c_str(),_("Getting table error: %s"),zErrMsg);
     }
     if( tbl && ncol > 0 )
     {
 	vector<string> row;
-	//-- Add head --
-	for(int i=0; i < ncol; i++)
-	    row.push_back(result[i]?result[i]:"");
+	//>> Add head
+	for( int i=0; i < ncol; i++ ) row.push_back(result[i]?result[i]:"");
 	tbl->push_back(row);
-	//-- Add data --
+	//>> Add data
 	for(int i=0; i < nrow; i++)
 	{
 	    row.clear();
@@ -260,7 +228,7 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
 
 void MBD::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TBD::cntrCmdProc(opt);
@@ -276,7 +244,7 @@ void MBD::cntrCmdProc( XMLNode *opt )
     }
     //- Process command to page -
     string a_path = opt->attr("path");
-    if( a_path == "/prm/st/end_tr" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) && commCnt )
+    if( a_path == "/prm/st/end_tr" && ctrChkNode(opt,"set",0660,"root","root",SEC_WR) && commCnt )
     { commCnt = trans_reqs; sqlReq(""); }
     else TBD::cntrCmdProc(opt);
 }
@@ -342,22 +310,20 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
     mLstUse = time(NULL);
 
     string sid;
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
     //> Make WHERE
     string req = "SELECT ";
     string req_where = "WHERE ";
     //>> Add use keys to list
-    bool first_sel = true;
-    bool next = false;
+    bool first_sel = true, next = false, trPresent = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
 	sid = tblStrct[i_fld][1];
 	TCfg *u_cfg = cfg.at(sid,true);
-	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
+	if( !cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
-	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText) ) continue;
+	    trPresent = true;
 	}
 	if( !u_cfg ) continue;
 
@@ -384,7 +350,7 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
 	sid = tbl[0][i_fld];
 	TCfg *u_cfg = cfg.at(sid,true);
 	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
-	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
+	else if( trPresent && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") && tbl[1][i_fld].size() )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
 	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
@@ -402,22 +368,20 @@ void MTable::fieldGet( TConfig &cfg )
     mLstUse = time(NULL);
 
     string sid;
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
     //> Prepare request
     string req = "SELECT ";
     string req_where;
     //>> Add fields list to queue
-    bool first_sel = true;
-    bool next_wr = false;
+    bool first_sel = true, next_wr = false, trPresent = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
 	sid = tblStrct[i_fld][1];
 	TCfg *u_cfg = cfg.at(sid,true);
-	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
+	if( !cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
-	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText) ) continue;
+	    trPresent = true;
 	}
 	if( !u_cfg ) continue;
 
@@ -444,7 +408,7 @@ void MTable::fieldGet( TConfig &cfg )
 	sid = tbl[0][i_fld];
 	TCfg *u_cfg = cfg.at(sid,true);
 	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
-	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
+	else if( trPresent && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") && tbl[1][i_fld].size() )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
 	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
@@ -460,12 +424,25 @@ void MTable::fieldSet( TConfig &cfg )
     mLstUse = time(NULL);
 
     string sid, sval;
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && !cfg.noTransl() && Mess->lang2Code() != Mess->lang2CodeBase());
     //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
+    //> Check for translation present
+    bool trPresent = isVarTextTransl, trDblDef = false;
+    for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
+    {
+	if( (trPresent || cfg.noTransl()) && (!isVarTextTransl || trDblDef) ) break;
+	sid = tblStrct[i_fld][1];
+	if( sid.size() > 3 )
+	{
+	    if( !trPresent && sid.substr(0,3) == (Mess->lang2Code()+"#") ) trPresent = true;
+	    if( isVarTextTransl && !trDblDef && sid.substr(0,3) == (Mess->lang2CodeBase()+"#") ) trDblDef = true;
+	}
+    }
+    if( trDblDef ) fieldFix(cfg);
+
     //> Get present fields list
     string req_where = "WHERE ";
     //>> Add key list to queue
@@ -497,11 +474,12 @@ void MTable::fieldSet( TConfig &cfg )
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( !(u_cfg.fld().flg()&TCfg::Key) && !u_cfg.view() ) continue;
 
-	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
-	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
-	    ins_name = ins_name + (next?",\"":"\"") + mod->sqlReqCode(sid,'"') + "\" ";
-	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
-	    ins_value = ins_value + (next?",'":"'") + mod->sqlReqCode(sval) + "' ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && trPresent && !u_cfg.noTransl());
+	    ins_name = ins_name + (next?",\"":"\"") + mod->sqlReqCode(cf_el[i_el],'"') + "\" " +
+		(isTransl ? (",\"" + mod->sqlReqCode(Mess->lang2Code()+"#"+cf_el[i_el],'"') + "\" ") : "");
+	    sval = getVal(u_cfg);
+	    ins_value = ins_value + (next?",'":"'") + mod->sqlReqCode(sval) + "' " + 
+		(isTransl ? (",'" + mod->sqlReqCode(sval) + "' ") : "");
 	    next = true;
 	}
 	req = req + "("+ins_name+") VALUES ("+ins_value+")";
@@ -516,9 +494,9 @@ void MTable::fieldSet( TConfig &cfg )
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( u_cfg.fld().flg()&TCfg::Key || !u_cfg.view() ) continue;
 
-	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
-	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
-	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && trPresent && !u_cfg.noTransl());
+	    sid = isTransl ? (Mess->lang2Code()+"#"+cf_el[i_el]) : cf_el[i_el];
+	    sval = getVal(u_cfg);
 	    req = req + (next?",\"":"\"") + mod->sqlReqCode(sid,'"') + "\"='" + mod->sqlReqCode(sval) + "' ";
 	    next = true;
 	}
@@ -527,14 +505,12 @@ void MTable::fieldSet( TConfig &cfg )
     req += ";";
 
     //> Query
-    //printf("TEST 02: query: <%s>\n",req.c_str());
-    try{ owner().sqlReq( req ); }
+    try { owner().sqlReq( req ); }
     catch(TError err)
     {
 	if( (err.cod-100) == SQLITE_READONLY )	return;
 	fieldFix(cfg); owner().sqlReq( req );
     }
-    //printf("TEST 01b: End from set\n");
 }
 
 void MTable::fieldDel( TConfig &cfg )
@@ -576,8 +552,7 @@ void MTable::fieldFix( TConfig &cfg )
 
     string req;
 
-    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
-    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && !cfg.noTransl() && Mess->lang2Code() != Mess->lang2CodeBase());
     //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
@@ -610,10 +585,10 @@ void MTable::fieldFix( TConfig &cfg )
 		    {
 			bool col_cur = false;
 			for( int i_c = i_fld; i_c < tblStrct.size(); i_c++ )
-			    if( tblStrct[i_c][1].size() > 3 && tblStrct[i_c][1].substr(2) == ("#"+cf_el[i_cf]) )
+			    if( tblStrct[i_c][1].size() > 3 && tblStrct[i_c][1].substr(2) == ("#"+cf_el[i_cf]) && tblStrct[i_c][1].substr(0,2) != Mess->lang2Code() )
 			    {
 				all_flds = all_flds + ",\"" + mod->sqlReqCode(tblStrct[i_c][1],'"') + "\"";
-				if( tblStrct[i_c][1].substr(0,2) == lang2Code ) col_cur = true;
+				if( tblStrct[i_c][1].substr(0,2) == Mess->lang2Code() ) col_cur = true;
 			    }
 			if( !col_cur && isVarTextTransl ) fix = true;
 		    }
@@ -627,7 +602,8 @@ void MTable::fieldFix( TConfig &cfg )
 	    int i_cf;
 	    for( i_cf = 0; i_cf < cf_el.size(); i_cf++ )
 		if( cf_el[i_cf] == tblStrct[i_fld][1] ||
-			(cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::TransltText && tblStrct[i_fld][1].size() > 3 && tblStrct[i_fld][1].substr(2) == ("#"+cf_el[i_cf])) )
+			(cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::TransltText && tblStrct[i_fld][1].size() > 3 && 
+			tblStrct[i_fld][1].substr(2) == ("#"+cf_el[i_cf]) && tblStrct[i_fld][1].substr(0,2) != Mess->lang2Code()) )
 		    break;
 	    if( i_cf >= cf_el.size() ) fix = true;
 	}
@@ -666,13 +642,13 @@ void MTable::fieldFix( TConfig &cfg )
 	{
 	    bool col_cur = false;
 	    for( int i_c = 1; i_c < tblStrct.size(); i_c++ )
-		if( tblStrct[i_c][1].size() > 3 && tblStrct[i_c][1].substr(2) == ("#"+cf_el[i_cf]) )
+		if( tblStrct[i_c][1].size() > 3 && tblStrct[i_c][1].substr(2) == ("#"+cf_el[i_cf]) && tblStrct[i_c][1].substr(0,2) != Mess->lang2Code() )
 		{
 		    req = req + ",\"" + mod->sqlReqCode(tblStrct[i_c][1],'"') + "\" "+tpCfg;
-		    if( tblStrct[i_c][1].substr(0,2) == lang2Code ) col_cur = true;
+		    if( tblStrct[i_c][1].substr(0,2) == Mess->lang2Code() ) col_cur = true;
 		}
 	    if( !col_cur && isVarTextTransl )
-		req = req + ",\"" + mod->sqlReqCode(lang2Code+"#"+cf_el[i_cf],'"') + "\" "+tpCfg;
+		req = req + ",\"" + mod->sqlReqCode(Mess->lang2Code()+"#"+cf_el[i_cf],'"') + "\" "+tpCfg;
 	}
 	//>> Primary key
 	else if( cf.fld().flg()&TCfg::Key )
@@ -682,16 +658,13 @@ void MTable::fieldFix( TConfig &cfg )
 	}
     }
     req += ", PRIMARY KEY ("+pr_keys+"));";
-    //printf("TEST 03: query: <%s>\n",req.c_str());
     owner().sqlReq( req );
 
-    //printf("TEST 01: %d\n",fix);
     //- Copy data from temporary DB -
     if( fix )
     {
 	req = "INSERT INTO '"+mod->sqlReqCode(name())+"'("+all_flds+") SELECT "+all_flds+" FROM 'temp_"+mod->sqlReqCode(name())+"';"
 	    "DROP TABLE 'temp_"+mod->sqlReqCode(name())+"';";
-	//printf("TEST 02: %s\n",req.c_str());
 	owner().sqlReq( req );
     }
 
