@@ -36,7 +36,7 @@ Block::Block( const string &iid, Contr *iown ) :
     TCntrNode(iown), TConfig( &((TipContr &)iown->owner()).blockE() ), TValFunc(iid+"_block",NULL),
     m_enable(false), m_process(false), id_freq(-1), id_start(-1), id_stop(-1),
     m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()), m_descr(cfg("DESCR").getSd()),
-    m_func(cfg("FUNC").getSd()), m_to_en(cfg("EN").getBd()), m_to_prc(cfg("PROC").getBd())
+    m_func(cfg("FUNC").getSd()), m_to_en(cfg("EN").getBd()), m_to_prc(cfg("PROC").getBd()), m_prior(cfg("PRIOR").getSd())
 {
     m_id = iid;
 }
@@ -51,12 +51,12 @@ TCntrNode &Block::operator=( TCntrNode &node )
     Block *src_n = dynamic_cast<Block*>(&node);
     if( !src_n ) return *this;
 
-    //- Copy parameters -
+    //> Copy parameters
     string prev_id = m_id;
     *(TConfig *)this = *(TConfig*)src_n;
     m_id = prev_id;
 
-    //- Copy IO and links -
+    //> Copy IO and links
     if( src_n->enable() )
     {
 	setEnable(true);
@@ -102,7 +102,7 @@ void Block::load_( )
     string bd = owner().DB()+"."+owner().cfg("BLOCK_SH").getS();
     SYS->db().at().dataGet(bd,mod->nodePath()+owner().cfg("BLOCK_SH").getS(),*this);
 
-    //- Load IO config -
+    //> Load IO config
     loadIO();
 }
 
@@ -111,7 +111,7 @@ void Block::save_( )
     string bd = owner().DB()+"."+owner().cfg("BLOCK_SH").getS();
     SYS->db().at().dataSet(bd,mod->nodePath()+owner().cfg("BLOCK_SH").getS(),*this);
 
-    //- Save IO config -
+    //> Save IO config
     saveIO();
 }
 
@@ -133,7 +133,7 @@ void Block::loadIO( const string &blk_db, const string &blk_id )
 	bd_tbl = TSYS::strSepParse(bd,2,'.');
     }
 
-    for( int i_ln = 0; i_ln < m_val.size(); i_ln++ )
+    for( int i_ln = 0; i_ln < ioSize(); i_ln++ )
     {
 	if( i_ln >= m_lnk.size() )
 	{
@@ -146,7 +146,7 @@ void Block::loadIO( const string &blk_db, const string &blk_id )
 	//> Value
 	setS(i_ln,cfg.cfg("VAL").getS());
 	//> Config of link
-	setLink(i_ln,SET,(LnkT)cfg.cfg("TLNK").getI(),cfg.cfg("LNK").getS());
+	setLink( i_ln, SET, (LnkT)cfg.cfg("TLNK").getI(), cfg.cfg("LNK").getS() );
     }
 }
 
@@ -178,7 +178,7 @@ void Block::saveIO( )
 
 void Block::setEnable( bool val )
 {
-    //- Enable -
+    //> Enable
     if( val && !m_enable )
     {
 	if( !func() )
@@ -186,27 +186,27 @@ void Block::setEnable( bool val )
 	    if( !dynamic_cast<TFunction *>(&SYS->nodeAt(wFunc(),0,'.').at()) )
 		throw TError(nodePath().c_str(),_("Node '%s' is not function."),wFunc().c_str());
 	    setFunc( (TFunction *)&SYS->nodeAt(wFunc(),0,'.').at() );
-	    //-- Init system attributes identifiers --
+	    //>> Init system attributes identifiers
 	    id_freq  = func()->ioId("f_frq");
 	    id_start = func()->ioId("f_start");
 	    id_stop  = func()->ioId("f_stop");
 	}
-	//-- Init links --
+	//>> Init links
 	loadIO( );
     }
-    //- Disable -
+    //> Disable
     else if( !val && m_enable )
     {
 	if( process() ) setProcess(false);
-	//-- Save IO config --
+	//>> Save IO config
 	//saveIO();
 
-	//-- Clean IO --
+	//>> Clean IO
 	for( unsigned i_ln = 0; i_ln < m_lnk.size(); i_ln++ )
-	    setLink(i_ln,SET,FREE);
+	    setLink( i_ln, SET, FREE );
 	m_lnk.clear();
 
-	//-- Free func --
+	//>> Free func
 	setFunc(NULL);
 	id_freq=id_start=id_stop=-1;
     }
@@ -217,14 +217,14 @@ void Block::setProcess( bool val )
 {
     if( val && !enable() ) setEnable(true);
 
-    //- Connect links -
+    //> Connect links
     if( val && !process() )
     {
 	for( int i_ln = 0; i_ln < m_lnk.size(); i_ln++ )
 	    setLink( i_ln, INIT );
 	owner().blkProc( id(), val );
     }
-    //- Disconnect links -
+    //> Disconnect links
     if( !val && process() )
     {
 	for( int i_ln = 0; i_ln < m_lnk.size(); i_ln++ )
@@ -260,16 +260,15 @@ bool Block::linkActive( unsigned iid )
 void Block::setLink( unsigned iid, LnkCmd cmd, LnkT lnk, const string &vlnk )
 {
     ResAlloc res(lnk_res,true);
-    if( iid >= m_lnk.size() )
-	throw TError(nodePath().c_str(),_("Link %d error!"),iid);
+    if( iid >= m_lnk.size() )	throw TError(nodePath().c_str(),_("Link %d error!"),iid);
 
-    //- Change type link -
+    //> Change type link
     if( cmd == SET )
     {
 	if( lnk != m_lnk[iid].tp )
 	{
-	    //-- Free old structures --
-	    switch(m_lnk[iid].tp)
+	    //>> Free old structures
+	    switch( m_lnk[iid].tp )
 	    {
 		case I_LOC: case I_GLB:	case O_LOC: case O_GLB:
 		    delete m_lnk[iid].iblk;	break;
@@ -277,8 +276,8 @@ void Block::setLink( unsigned iid, LnkCmd cmd, LnkT lnk, const string &vlnk )
 		    delete m_lnk[iid].aprm;	break;
 	    }
 
-	    //-- Make new structures --
-	    switch(lnk)
+	    //>> Make new structures
+	    switch( lnk )
 	    {
 		case I_LOC: case I_GLB:	case O_LOC: case O_GLB:
 		    m_lnk[iid].iblk = new SLIBlk;	break;
@@ -289,7 +288,7 @@ void Block::setLink( unsigned iid, LnkCmd cmd, LnkT lnk, const string &vlnk )
 	}
 	m_lnk[iid].lnk = vlnk;
     }
-    //- Connect new link and init -
+    //> Connect new link and init
     if( cmd == INIT || (cmd == SET && process()) )
     {
 	string lo1 = TSYS::strSepParse(m_lnk[iid].lnk,0,'.');
@@ -319,12 +318,16 @@ void Block::setLink( unsigned iid, LnkCmd cmd, LnkT lnk, const string &vlnk )
 		break;
 	    case I_PRM: case O_PRM:
 		m_lnk[iid].aprm->free();
-		if( dynamic_cast<TVal *>(&SYS->nodeAt(m_lnk[iid].lnk,0,'.').at()) )
-		    *m_lnk[iid].aprm = SYS->nodeAt(m_lnk[iid].lnk,0,'.');
+		try { *m_lnk[iid].aprm = SYS->daq().at().at(lo1).at().at(lo2).at().at(lo3).at().vlAt(lo4); }
+		catch( TError err)
+		{
+		    try{ *m_lnk[iid].aprm = SYS->nodeAt(m_lnk[iid].lnk,0,'.'); }
+		    catch( TError err) { }
+		}
 		break;
 	}
     }
-    //- Disconnect -
+    //> Disconnect
     if( cmd == DEINIT )
 	switch(m_lnk[iid].tp)
 	{
@@ -337,16 +340,16 @@ void Block::setLink( unsigned iid, LnkCmd cmd, LnkT lnk, const string &vlnk )
 
 void Block::calc( bool first, bool last )
 {
-    //- Set fixed system attributes -
+    //> Set fixed system attributes
     if( id_freq>=0 )	setR(id_freq,(1000.*(double)owner().iterate())/(double)owner().period());
     if( id_start>=0 )	setB(id_start,first);
     if( id_stop>=0 )	setB(id_stop,last);
 
-    //- Get values from input links -
+    //> Get values from input links
     lnk_res.resRequestR( );
     try
     {
-	//-- Get input links --
+	//>> Get input links
 	for( unsigned i_ln=0; i_ln < m_lnk.size(); i_ln++ )
 	    switch( m_lnk[i_ln].tp )
 	    {
@@ -379,7 +382,7 @@ void Block::calc( bool first, bool last )
     }
     lnk_res.resRelease( );
 
-    //- Calc function -
+    //> Calc function
     try
     {
 	TValFunc::calc( );
@@ -387,7 +390,7 @@ void Block::calc( bool first, bool last )
     }
     catch(TError err) { err_cnt++; throw; }
 
-    //- Put values to output links -
+    //> Put values to output links
     lnk_res.resRequestR( );
     try
     {
@@ -434,10 +437,10 @@ void Block::cntrCmdProc( XMLNode *opt )
 	if( a_path == "/serv/attr" )
 	{
 	    if( !enable() || !func() ) throw TError(nodePath().c_str(),_("Block disabled or error."));
-	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )
 		for( int i_a = 0; i_a < ioSize(); i_a++ )
 		    opt->childAdd("a")->setAttr("id",func()->io(i_a)->id())->setText(getS(i_a));
-	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )
 		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
 		{
 		    int io_id = -1;
@@ -469,6 +472,8 @@ void Block::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/blck/cfg/descr",_("Description"),0664,"root","root",3,"tp","str","cols","90","rows","4");
 		ctrMkNode("fld",opt,-1,"/blck/cfg/toen",_("To enable"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/blck/cfg/toprc",_("To process"),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/blck/cfg/prior",_("Prior block"),0664,"root","root",4,"tp","str","dest","sel_ed","select","/blck/cfg/blks",
+		    "help",_("Priority block(s) calced before this block. Blocks list is separated by symbol ';'."));
 		ctrMkNode("fld",opt,-1,"/blck/cfg/func",_("Function"),(!func())?0664:0444,"root","root",3,"tp","str","dest","sel_ed","select","/blck/cfg/fncs");
 		ctrMkNode("comm",opt,-1,"/blck/cfg/func_lnk",_("Go to function"),wFunc().empty()?0:0660,"root","root",1,"tp","lnk");
 	    }
@@ -508,14 +513,14 @@ void Block::cntrCmdProc( XMLNode *opt )
 		{
 		    vector<string> list;
 		    ioList(list);
-		    //-- Put IO links --
+		    //>> Put IO links
 		    for( int i_io = 0; i_io < list.size(); i_io++ )
 		    {
 			int id = ioId(list[i_io]);
 
 			if( ioHide(id) && !atoi(TBDS::genDBGet(owner().nodePath()+"showHide","0",opt->attr("user")).c_str()) ) continue;
 
-			//-- Add link's type --
+			//>> Add link's type
 			ctrMkNode("fld",opt,-1,(string("/lnk/io/1|")+list[i_io]).c_str(),
 			    func()->io(id)->name().c_str(),0664,"root","root",3,"tp","dec","dest","select","select",(ioFlg(id)&(IO::Output|IO::Return))?"/lnk/otp":"/lnk/itp");
 			if( m_lnk[id].tp != FREE )
@@ -529,41 +534,54 @@ void Block::cntrCmdProc( XMLNode *opt )
     //> Process command to page
     if( a_path == "/blck/st/en" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(enable()?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setEnable(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(enable()?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setEnable(atoi(opt->text().c_str()));
     }
     else if( a_path == "/blck/st/prc" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(process()?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setProcess(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(process()?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setProcess(atoi(opt->text().c_str()));
     }
     else if( a_path == "/blck/cfg/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/blck/cfg/name" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(name());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setName(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(name());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setName(opt->text());
     }
     else if( a_path == "/blck/cfg/descr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(descr());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setDescr(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(descr());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setDescr(opt->text());
     }
     else if( a_path == "/blck/cfg/toen" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( toEnable() ? "1" : "0" );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setToEnable( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText( toEnable() ? "1" : "0" );
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setToEnable( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/blck/cfg/toprc" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( toProcess() ? "1" : "0" );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setToProcess( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText( toProcess() ? "1" : "0" );
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setToProcess( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/blck/cfg/prior" )
+    {
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText( prior() );
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setPrior( opt->text() );
+    }
+    else if( a_path == "/blck/cfg/blks" && ctrChkNode(opt) )
+    {
+	vector<string> list;
+	owner().blkList(list);
+	opt->childAdd("el")->setText("");
+	for( int i_b = 0; i_b < list.size(); i_b++ )
+	    if( list[i_b] != id() ) opt->childAdd("el")->setText(list[i_b]);
     }
     else if( a_path == "/blck/cfg/func" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( wFunc() );
-	if( !func() && ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setWFunc( opt->text() );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText( wFunc() );
+	if( !func() && ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setWFunc( opt->text() );
     }
-    else if( a_path == "/blck/cfg/func_lnk" && ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )
+    else if( a_path == "/blck/cfg/func_lnk" && ctrChkNode(opt,"get",0440,"root","root",SEC_RD) )
 	opt->setText(TSYS::sepstr2path(wFunc(),'.'));
     else if( a_path == "/blck/cfg/fncs" && ctrChkNode(opt) )
     {
@@ -583,20 +601,20 @@ void Block::cntrCmdProc( XMLNode *opt )
     }
     else if( (a_path == "/lio/show/hide" || a_path == "/lnk/show/hide") && enable() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"showHide","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	TBDS::genDBSet(owner().nodePath()+"showHide",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"showHide","0",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	TBDS::genDBSet(owner().nodePath()+"showHide",opt->text(),opt->attr("user"));
     }
     else if( a_path.substr(0,7) == "/lio/io" && enable() )
     {
 	int id = ioId(TSYS::pathLev(a_path,2));
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	    opt->setText( (ioType(id)==IO::Real) ? TSYS::real2str(getR(id),6) : getS(id));
-	if( !linkActive(id) && ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( !linkActive(id) && ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{ setS(id,opt->text()); modif(); }
     }
     else if( a_path.substr(0,7) == "/lnk/io" && enable() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    vector<string> list;
 	    char lev = TSYS::pathLev(a_path,2)[0];
@@ -617,48 +635,68 @@ void Block::cntrCmdProc( XMLNode *opt )
 		    opt->childAdd("el")->setText(c_path);
 		}
 		if(c_lv) c_path+=".";
-		switch(m_lnk[id].tp)
+
+		string prm0 = TSYS::strSepParse( lnk, 0, '.' );
+		string prm1 = TSYS::strSepParse( lnk, 1, '.' );
+		string prm2 = TSYS::strSepParse( lnk, 2, '.' );
+
+		switch( m_lnk[id].tp )
 		{
 		    case I_LOC: case O_LOC:
-			switch(c_lv)
+			switch( c_lv )
 			{
 			    case 0: owner().blkList(list); break;
 			    case 1:
-				if( owner().blkPresent(TSYS::strSepParse(lnk,0,'.'))
-					&& owner().blkAt(TSYS::strSepParse(lnk,0,'.')).at().func() )
-				    owner().blkAt(TSYS::strSepParse(lnk,0,'.')).at().ioList(list);
+				if( owner().blkPresent(prm0) && owner().blkAt(prm0).at().func() )
+				    owner().blkAt(prm0).at().ioList(list);
 				break;
 			}
 			break;
 		    case I_GLB: case O_GLB:
-			switch(c_lv)
+			switch( c_lv )
 			{
 			    case 0: owner().owner().list(list); break;
 			    case 1:
-				if( owner().owner().present(TSYS::strSepParse(lnk,0,'.')) )
-				    ((Contr &)owner().owner().at(TSYS::strSepParse(lnk,0,'.')).at()).blkList(list);
+				if( owner().owner().present(prm0) )
+				    ((Contr &)owner().owner().at(prm0).at()).blkList(list);
 				break;
 			    case 2:
-				if( owner().owner().present(TSYS::strSepParse(lnk,0,'.')) &&
-					((Contr &)owner().owner().at(TSYS::strSepParse(lnk,0,'.')).at()).blkPresent(TSYS::strSepParse(lnk,1,'.')) )
-				    ((Contr &)owner().owner().at(TSYS::strSepParse(lnk,0,'.')).at()).blkAt(TSYS::strSepParse(lnk,1,'.')).at().ioList(list);
+				if( owner().owner().present(prm0) && ((Contr &)owner().owner().at(prm0).at()).blkPresent(prm1) )
+				    ((Contr&)owner().owner().at(prm0).at()).blkAt(prm1).at().ioList(list);
 				break;
 			}
 			break;
 		    case I_PRM: case O_PRM:
-		        SYS->nodeAt(lnk,0,'.').at().nodeList(list);
+			//SYS->nodeAt(lnk,0,'.').at().nodeList(list);
+			switch( c_lv )
+			{
+			    case 0:	SYS->daq().at().modList(list);	break;
+			    case 1:
+				if( SYS->daq().at().modPresent(prm0) )
+				    SYS->daq().at().at(prm0).at().list(list);
+				break;
+			    case 2:
+				 if( SYS->daq().at().modPresent(prm0) && SYS->daq().at().at(prm0).at().present(prm1) )
+				    SYS->daq().at().at(prm0).at().at(prm1).at().list(list);
+				break;
+			    case 3:
+				if( SYS->daq().at().modPresent(prm0) && SYS->daq().at().at(prm0).at().present(prm1) &&
+					SYS->daq().at().at(prm0).at().at(prm1).at().present(prm2) )
+				    SYS->daq().at().at(prm0).at().at(prm1).at().at(prm2).at().vlList(list);
+				break;
+			}
 			break;
 		}
 		for( unsigned i_a=0; i_a < list.size(); i_a++ )
 		    opt->childAdd("el")->setText(c_path+list[i_a]);
 	    }
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    char lev = TSYS::pathLev(a_path,2)[0];
 	    int id = ioId(TSYS::pathLev(a_path,2).substr(2));
-	    if( lev == '1' )		setLink(id,SET,(Block::LnkT)atoi(opt->text().c_str()));
-	    else if( lev == '2' )	setLink(id,SET,m_lnk[id].tp,opt->text());
+	    if( lev == '1' )		setLink( id, SET, (Block::LnkT)atoi(opt->text().c_str()) );
+	    else if( lev == '2' )	setLink( id, SET, m_lnk[id].tp,opt->text() );
 	    modif();
 	}
     }
@@ -672,9 +710,12 @@ void Block::cntrCmdProc( XMLNode *opt )
     else if( a_path == "/lnk/otp" && enable() && ctrChkNode(opt) )
     {
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::FREE))->setText(_("Free"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::O_LOC))->setText(_("Local"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::O_GLB))->setText(_("Global"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::O_PRM))->setText(_("Parameter"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::O_LOC))->setText(_("Local out"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::O_GLB))->setText(_("Global out"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::O_PRM))->setText(_("Parameter out"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::I_LOC))->setText(_("Local in"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::I_GLB))->setText(_("Global in"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(Block::I_PRM))->setText(_("Parameter in"));
     }
     else TCntrNode::cntrCmdProc(opt);
 }
