@@ -44,7 +44,7 @@
 #define MOD_NAME	"Block based calculator"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"1.2.0"
+#define VERSION		"1.3.1"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow block based calculator."
 #define LICENSE		"GPL"
@@ -73,7 +73,7 @@ using namespace Virtual;
 //************************************************
 //* TipContr - BlockCalc type controller         *
 //************************************************
-TipContr::TipContr( string name ) 
+TipContr::TipContr( string name )
 {
     mId		= MOD_ID;
     mName	= MOD_NAME;
@@ -92,40 +92,11 @@ TipContr::~TipContr()
     nodeDelAll();
 }
 
-string TipContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-    return(buf);
-}
-
 void TipContr::load_()
 {
-    //========== Load parameters from command line ============
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"      ,0,NULL,'h'},
-	{NULL        ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
-
-    //========== Load parameters from config file =============
+    //> Load parameters from config file
 }
 
 void TipContr::postEnable( int flag )
@@ -136,12 +107,12 @@ void TipContr::postEnable( int flag )
     fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"30","system") );
     fldAdd( new TFld("BLOCK_SH",_("Block's table"),TFld::String,TFld::NoFlag,"30","block") );
     fldAdd( new TFld("PERIOD",_("Calc period (ms)"),TFld::Integer,TFld::NoFlag,"5","1000","0;10000") );
-    fldAdd( new TFld("PRIOR",_("Calc task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
+    fldAdd( new TFld("PRIOR",_("Calc task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
     fldAdd( new TFld("ITER",_("Iteration number into calc period"),TFld::Integer,TFld::NoFlag,"2","1","0;99") );
 
     //Add parameter types
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
-    tpPrmAt(t_prm).fldAdd( new TFld("IO",_("Blocks' IOs"),TFld::String,TFld::FullText|TCfg::TransltText|TCfg::NoVal,"200") );
+    tpPrmAt(t_prm).fldAdd( new TFld("IO",_("Blocks' IOs"),TFld::String,TFld::FullText|TCfg::TransltText|TCfg::NoVal,"1000") );
 
     //Blok's db structure
     blk_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
@@ -150,6 +121,7 @@ void TipContr::postEnable( int flag )
     blk_el.fldAdd( new TFld("FUNC",_("Function"),TFld::String,TFld::NoFlag,"75") );
     blk_el.fldAdd( new TFld("EN",_("To enable"),TFld::Boolean,TFld::NoFlag,"1","0") );
     blk_el.fldAdd( new TFld("PROC",_("To process"),TFld::Boolean,TFld::NoFlag,"1","0") );
+    blk_el.fldAdd( new TFld("PRIOR",_("Prior block"),TFld::String,TFld::NoFlag,"200") );    
 
     //IO blok's db structure
     blkio_el.fldAdd( new TFld("BLK_ID",_("Blok's ID"),TFld::String,TCfg::Key,"20") );
@@ -176,21 +148,6 @@ TController *TipContr::ContrAttach( const string &name, const string &daq_db )
     return new Contr(name,daq_db,this);
 }
 
-void TipContr::cntrCmdProc( XMLNode *opt )
-{
-    //Get page info
-    if( opt->name() == "info" )
-    {
-	TTipDAQ::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
-    }
-
-    //Process command to page
-    string a_path = opt->attr("path");
-    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) ) opt->setText(optDescr());
-    else TTipDAQ::cntrCmdProc(opt);
-}
-
 //************************************************
 //* Contr - Blocks and parameters container      *
 //************************************************ 
@@ -215,12 +172,12 @@ TCntrNode &Contr::operator=( TCntrNode &node )
     Contr *src_n = dynamic_cast<Contr*>(&node);
     if( !src_n ) return *this;
 
-    //- Blocks copy -
+    //> Blocks copy
     if( src_n->enableStat( ) )
     {
 	if( !enableStat( ) )    enable();
 
-	//-- Blocks copy --
+	//>> Blocks copy
 	vector<string> ls;
 	src_n->blkList(ls);
 	for( int i_l = 0; i_l < ls.size(); i_l++ )
@@ -270,7 +227,7 @@ void Contr::load_( )
 
     TController::load_( );
 
-    //- Load block's configuration -
+    //> Load block's configuration
     TConfig c_el(&mod->blockE());
     c_el.cfgViewAll(false);
     string bd = DB()+"."+cfg("BLOCK_SH").getS();
@@ -319,7 +276,7 @@ void Contr::disable_( )
 
 void Contr::start_( )
 {
-    //- Make process all bloks -
+    //> Make process all bloks
     vector<string> lst;
     blkList(lst);
     for( int i_l = 0; i_l < lst.size(); i_l++ )
@@ -331,38 +288,38 @@ void Contr::start_( )
 	    mess_warning(nodePath().c_str(),_("Process block <%s> error."),lst[i_l].c_str());
 	}
 
-    //- Start the request and calc data task -
-    if( !prc_st )
+    //> Sort blocks
+    ResAlloc res(hd_res,true);
+    string pvl;
+    for( int i_be = 0, permCnt = 0, i_blk = 0; i_be < clc_blks.size() && permCnt < clc_blks.size()/2; i_be++ )
     {
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	if( m_prior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority=m_prior;
-	pthread_attr_setschedparam(&pthr_attr,&prior);
-
-	pthread_create(&calcPthr,&pthr_attr,Contr::Task,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
+	AutoHD<Block> cBlk = clc_blks[i_be];
+	for( int off = 0; (pvl=TSYS::strSepParse(cBlk.at().prior(),0,';',&off)).size(); )
+	{
+	    for( i_blk = i_be; i_blk < clc_blks.size(); i_blk++ )
+		if( pvl == clc_blks[i_blk].at().id() )
+		{
+		    clc_blks[i_be] = clc_blks[i_blk];
+		    clc_blks[i_blk] = cBlk;
+		    permCnt++;
+		    break;
+		}
+	    if( i_blk < clc_blks.size() ) break;
+	}
+	if( !pvl.empty() ) i_be = -1;
     }
+    res.release();
+
+    //> Start the request and calc data task
+    if( !prc_st ) SYS->taskCreate( nodePath('.',true), m_prior, Contr::Task, this, &prc_st );
 }
 
 void Contr::stop_( )
 {
-    //- Stop the request and calc data task -
-    if( prc_st )
-    {
-	endrun_req = true;
-	pthread_kill( calcPthr, SIGALRM );
-	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
-	pthread_join( calcPthr, NULL );
-    }
+    //> Stop the request and calc data task
+    if( prc_st ) SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
 
-    //- Make deprocess all blocks -
+    //> Make deprocess all blocks
     vector<string> lst;
     blkList(lst);
     for( int i_l = 0; i_l < lst.size(); i_l++ )
@@ -390,6 +347,7 @@ void *Contr::Task( void *icontr )
 	long long t_cnt = TSYS::curTime();
 
 	cntr.hd_res.resRequestR( );
+	ResAlloc sres(cntr.calcRes,true);
 	for( unsigned i_it = 0; i_it < cntr.m_iter && !cntr.redntUse(); i_it++ )
 	    for( unsigned i_blk = 0; i_blk < cntr.clc_blks.size(); i_blk++ )
 	    {
@@ -406,6 +364,7 @@ void *Contr::Task( void *icontr )
 		    cntr.hd_res.resRequestR( );
 		}
 	    }
+	sres.release();
 	cntr.hd_res.resRelease( );
 
 	cntr.tm_calc = TSYS::curTime()-t_cnt;
@@ -414,8 +373,8 @@ void *Contr::Task( void *icontr )
 
 	TSYS::taskSleep((long long)cntr.period()*1000000);
 
-	if(cntr.endrun_req) is_stop = true;
-	is_start = false;
+	if( cntr.endrun_req ) is_stop = true;
+	if( !cntr.redntUse() ) is_start = false;
     }
 
     cntr.prc_st = false;
@@ -423,9 +382,9 @@ void *Contr::Task( void *icontr )
     return NULL;
 }
 
-void Contr::redntDataUpdate( bool firstArchiveSync )
+void Contr::redntDataUpdate( )
 {
-    TController::redntDataUpdate(firstArchiveSync);
+    TController::redntDataUpdate( );
 
     vector<string> bls; blkList(bls);
 
@@ -438,8 +397,7 @@ void Contr::redntDataUpdate( bool firstArchiveSync )
     }
 
     //> Send request to first active station for this controller
-    try{ owner().owner().rdStRequest(workId(),req); }
-    catch(TError err) { return; }
+    if( owner().owner().rdStRequest(workId(),req).empty() ) return;
 
     //> Redirect respond to local parameters
     req.setAttr("path","/");
@@ -463,12 +421,12 @@ void Contr::blkAdd( const string &iid )
     chldAdd(m_bl,new Block( iid, this ));
 }
 
-void Contr::blkProc( const string & id, bool val )
+void Contr::blkProc( const string &id, bool val )
 {
     unsigned i_blk;
 
     ResAlloc res(hd_res,true);
-    for( i_blk = 0; i_blk < clc_blks.size(); i_blk++)
+    for( i_blk = 0; i_blk < clc_blks.size(); i_blk++ )
 	if( clc_blks[i_blk].at().id() == id ) break;
 
     if( val && i_blk >= clc_blks.size() ) clc_blks.push_back(blkAt(id));
@@ -491,21 +449,46 @@ void Contr::cntrCmdProc( XMLNode *opt )
     }
     //Process command to page
     string a_path = opt->attr("path");
-    if( a_path == "/br/blk_" || a_path == "/scheme/sch" )
+    if( a_path == "/br/blk_" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    vector<string> lst;
 	    blkList(lst);
 	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
 		opt->childAdd("el")->setAttr("id",lst[i_f])->setText(blkAt(lst[i_f]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    blkAdd(vid); blkAt(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	chldDel(m_bl,opt->attr("id"),-1,1);
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )	chldDel(m_bl,opt->attr("id"),-1,1);
+    }
+    else if( a_path == "/scheme/sch" )
+    {
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
+	{
+	    if( !startStat() )
+	    {
+		vector<string> lst;
+		blkList(lst);
+		for( unsigned i_f=0; i_f < lst.size(); i_f++ )
+		    opt->childAdd("el")->setAttr("id",lst[i_f])->setText(blkAt(lst[i_f]).at().name());
+	    }
+	    else
+	    {
+		ResAlloc sres(hd_res,false);
+		for( unsigned i_b=0; i_b < clc_blks.size(); i_b++ )
+		    opt->childAdd("el")->setAttr("id",clc_blks[i_b].at().id())->setText(clc_blks[i_b].at().name());
+	    }
+	}
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )
+	{
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
+	    blkAdd(vid); blkAt(vid).at().setName(opt->text());
+	}
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )	chldDel(m_bl,opt->attr("id"),-1,1);
     }
     else if( a_path == "/scheme/nmb" && ctrChkNode(opt) )
     {
@@ -632,9 +615,9 @@ void Prm::vlSet( TVal &val, const TVariant &pvl )
     //> Send to active reserve station
     if( owner().redntUse( ) )
     {
-	if( val.getS() == pvl.getS() ) return;
+	if( val.getS(0,true) == pvl.getS() ) return;
 	XMLNode req("set");
-	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS(0,true));
 	SYS->daq().at().rdStRequest(owner().workId(),req);
 	return;
     }
@@ -646,6 +629,8 @@ void Prm::vlSet( TVal &val, const TVariant &pvl )
 	int io_id = blk.at().ioId(TSYS::strSepParse(val.fld().reserve(),1,'.'));
 	if( io_id < 0 ) disable();
 	else
+	{
+	    ResAlloc sres(owner().calcRes,true);
 	    switch(val.fld().type())
 	    {
 		case TFld::String:	blk.at().setS(io_id,val.getS(0,true));	break;
@@ -653,6 +638,7 @@ void Prm::vlSet( TVal &val, const TVariant &pvl )
 		case TFld::Real:	blk.at().setR(io_id,val.getR(0,true));	break;
 		case TFld::Boolean:	blk.at().setB(io_id,val.getB(0,true));	break;
 	    }
+	}
     }catch(TError err) { disable(); }
 }
 
@@ -660,10 +646,8 @@ void Prm::vlGet( TVal &val )
 {
     if( val.name() == "err" )
     {
-	if( !owner().startStat() )
-	    val.setS(_("2:Controller is stoped"),0,true);
-	else if( !enableStat() )
-	    val.setS(_("1:Parameter is disabled"),0,true);
+	if( !enableStat() ) val.setS(_("1:Parameter is disabled."),0,true);
+	else if( !owner().startStat( ) ) val.setS(_("2:Controller is stoped."),0,true);
 	else val.setS("0",0,true);
 	return;
     }
