@@ -33,7 +33,7 @@
 #define MOD_NAME	"Data sources gate"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"0.7.0"
+#define VERSION		"0.9.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow to make gate data sources of remote OpenSCADA station to local OpenSCADA station."
 #define LICENSE		"GPL"
@@ -80,39 +80,10 @@ TTpContr::~TTpContr( )
 
 }
 
-string TTpContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TTpContr::load_( )
 {
     //> Load parameters from command line
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 }
 
 void TTpContr::postEnable( int flag )
@@ -121,9 +92,9 @@ void TTpContr::postEnable( int flag )
 
     //> Controler's DB structure
     fldAdd( new TFld("PERIOD",_("Gather data period (s)"),TFld::Real,TFld::NoFlag,"6.2","1","0;100") );
-    fldAdd( new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
+    fldAdd( new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
     fldAdd( new TFld("TM_REST",_("Restore timeout (s)"),TFld::Integer,TFld::NoFlag,"3","30","0;1000") );
-    fldAdd( new TFld("TM_REST_DT",_("Restore data depth time (hour)"),TFld::Real,TFld::NoFlag,"6.2","1","0.01;12") );
+    fldAdd( new TFld("TM_REST_DT",_("Restore data depth time (hour). Zero for disable archive access."),TFld::Real,TFld::NoFlag,"6.2","1","0;12") );
     fldAdd( new TFld("SYNCPER",_("Sync inter remote station period (s)"),TFld::Real,TFld::NoFlag,"6.2","60","0;1000") );
     fldAdd( new TFld("STATIONS",_("Remote stations list"),TFld::String,TFld::FullText,"100") );
     fldAdd( new TFld("CNTRPRM",_("Remote cotrollers and parameters list"),TFld::String,TFld::FullText,"200") );
@@ -164,9 +135,11 @@ string TMdContr::getStatus( )
     if( startStat( ) && !redntUse( ) )
     {
 	val += TSYS::strMess(_("Gather data time %.6g ms. "),tmGath);
-	for( map<string,float>::iterator sti = mStatWork.begin(); sti != mStatWork.end(); sti++ )
-	    if( sti->second > 0 ) val += TSYS::strMess(_("Station '%s' error, restoring in %.6g s."),sti->first.c_str(),sti->second);
-	    else val += TSYS::strMess(_("Requests to station '%s': %.6g."),sti->first.c_str(),-sti->second);
+	bool isWork = false;
+	for( int i_st = 0; i_st < mStatWork.size(); i_st++ )
+	    if( mStatWork[i_st].second > -1 ) val += TSYS::strMess(_("Station '%s' error, restoring in %.6g s."),mStatWork[i_st].first.c_str(),mStatWork[i_st].second);
+	    else { val += TSYS::strMess(_("Requests to station '%s': %.6g."),mStatWork[i_st].first.c_str(),-mStatWork[i_st].second); isWork = true; }
+	if( !isWork ) val.replace(0,1,"10");
     }
 
     return val;
@@ -180,16 +153,20 @@ TParamContr *TMdContr::ParamAttach( const string &name, int type )
 void TMdContr::enable_( )
 {
     string statv, cp_el, daqtp, cntrnm, prmnm, cntrpath, gPrmLs;
+    bool gAddPrm = false;
     vector<string> prm_ls;
     XMLNode req("list");
 
     bool en_err = false;
 
-    //> Remote station scaning
+    //> Clear present parameters configuration
+    list(prm_ls);
+    for( int i_p = 0; i_p < prm_ls.size(); i_p++ ) at(prm_ls[i_p]).at().setCntrAdr("");
+
+    //> Remote station scaning. Controllers and parameters scaning
     for( int st_off = 0; (statv=TSYS::strSepParse(mStations,0,'\n',&st_off)).size(); )
     {
-	mStatWork[statv] = 0;
-	//> Controllers and parameters scaning
+	if( !enableStat( ) ) mStatWork.push_back(pair<string,float>(statv,0));
 	for( int cp_off = 0; (cp_el=TSYS::strSepParse(mContrPrm,0,'\n',&cp_off)).size(); )
 	    try
 	    {
@@ -205,7 +182,7 @@ void TMdContr::enable_( )
 		{
 		    //>> Get attributes list
 		    req.clear()->setName("get")->setAttr("path",cntrpath+"%2fprm%2fprm");
-		    if( cntrIfCmd(req,true) || req.childSize() == 0 )	en_err = true;
+		    if( cntrIfCmd(req) ) throw TError(req.attr("mcat").c_str(),"%s",req.text().c_str());
 		    else for( int i_ch = 0; i_ch < req.childSize(); i_ch++ )
 			prm_ls.push_back(req.childGet(i_ch)->attr("id"));
 		}
@@ -218,30 +195,44 @@ void TMdContr::enable_( )
 		    {
 			//>>> Parameter name request and make new parameter object
 			req.clear()->setName("get")->setAttr("path",cntrpath+prm_ls[i_p]+"/%2fprm%2fcfg%2fNAME");
-			if( cntrIfCmd(req,true) ) { en_err = true; continue; }
+			if( cntrIfCmd(req) ) throw TError(req.attr("mcat").c_str(),"%s",req.text().c_str());
 			add(prm_ls[i_p],owner().tpPrmToId("std"));
 			at(prm_ls[i_p]).at().setName(req.text());
+			gAddPrm = true;
 		    }
 		    if( !at(prm_ls[i_p]).at().enableStat() ) at(prm_ls[i_p]).at().enable();
 		    at(prm_ls[i_p]).at().setCntrAdr(cntrpath);
 		    at(prm_ls[i_p]).at().load();
 		    gPrmLs += prm_ls[i_p]+";";
 		}
-	    }catch(TError err){ }
+	    }catch(TError err){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
     }
 
     //> Removing parameter's try
-    list(prm_ls);
-    for( int i_p = 0; i_p < prm_ls.size(); i_p++ )
-	if( gPrmLs.find(prm_ls[i_p]+";",0) == string::npos )
-	    try{ del(prm_ls[i_p]); }
-	    catch( TError err )
-	    {
-		mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		mess_err(nodePath().c_str(),_("Deletion parameter '%s' is error but it no present on configuration or remote station."),prm_ls[i_p].c_str());
-	    }
+    if( !enableStat( ) )
+    {
+	list(prm_ls);
+	for( int i_p = 0; i_p < prm_ls.size(); i_p++ )
+	    if( gPrmLs.find(prm_ls[i_p]+";",0) == string::npos )
+		try{ del(prm_ls[i_p]); }
+		catch( TError err )
+		{
+		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
+		    mess_err(nodePath().c_str(),_("Deletion parameter '%s' is error but it no present on configuration or remote station."),prm_ls[i_p].c_str());
+		}
+    }
 
-    if( en_err ) throw TError(nodePath().c_str(),_("Some enable errors are present"));
+    //> Update try for archives start
+    if( gAddPrm && SYS->archive().at().subStartStat() )
+    {
+	SYS->archive().at().valList(prm_ls);
+	for( int i_a = 0; i_a < prm_ls.size(); i_a++ )
+	{
+	    AutoHD<TVArchive> archt = SYS->archive().at().valAt(prm_ls[i_a]);
+	    if( archt.at().toStart() && !archt.at().startStat() )
+		try{ archt.at().start(); } catch(...) { }
+	}
+    }
 }
 
 void TMdContr::disable_( )
@@ -251,54 +242,21 @@ void TMdContr::disable_( )
 
 void TMdContr::start_( )
 {
-    if( !prcSt )
-    {
-	//> Start the gathering data task
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	if( mPrior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority = mPrior;
-	pthread_attr_setschedparam(&pthr_attr,&prior);
+    if( prcSt ) return;
 
-	pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prcSt, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Gathering task is not started!"));
+    //> Clear stations request counter
+    for( int i_st = 0; i_st < mStatWork.size(); i_st++ ) mStatWork[i_st].second = 0;
 
-	//> Fill stations list
-	string statv;
-	for( int st_off = 0; (statv=TSYS::strSepParse(mStations,0,'\n',&st_off)).size(); )
-	    mStatWork[statv] = 0;
-    }
+    //> Start the gathering data task
+    SYS->taskCreate( nodePath('.',true), mPrior, TMdContr::Task, this, &prcSt );
 }
 
 void TMdContr::stop_( )
 {
-    if( prcSt )
-    {
+    if( !prcSt ) return;
 
-	//> Stop the request and calc data task
-	endrunReq = true;
-	pthread_kill( procPthr, SIGALRM );
-	if( TSYS::eventWait(prcSt,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Gathering task is not stopped!"));
-	pthread_join( procPthr, NULL );
-    }
-}
-
-void TMdContr::prmEn( const string &id, bool val )
-{
-    int i_prm;
-
-    ResAlloc res(enRes,true);
-    for( i_prm = 0; i_prm < pHd.size(); i_prm++)
-	if( pHd[i_prm].at().id() == id ) break;
-
-    if( val && i_prm >= pHd.size() )	pHd.push_back(at(id));
-    if( !val && i_prm < pHd.size() )	pHd.erase(pHd.begin()+i_prm);
+    //> Stop the request and calc data task
+    SYS->taskDestroy( nodePath('.',true), &prcSt, &endrunReq );
 }
 
 void *TMdContr::Task( void *icntr )
@@ -313,65 +271,175 @@ void *TMdContr::Task( void *icntr )
     cntr.endrunReq = false;
     cntr.prcSt = true;
 
-    try
+    for( unsigned int it_cnt = 0; !cntr.endrunReq; it_cnt++ )
     {
-	for( unsigned int it_cnt = 0; !cntr.endrunReq; it_cnt++ )
+	if( cntr.redntUse( ) ) { usleep( STD_WAIT_DELAY*1000 ); continue; }
+
+	long long t_cnt = TSYS::curTime();
+
+	try
 	{
-	    long long t_cnt = TSYS::curTime();
+	    ResAlloc res(cntr.enRes,false);
 
-	    cntr.enRes.resRequestR( );
+	    //> Allow stations presenting
+	    bool isAccess = false;
+	    for( int i_st = 0; i_st < cntr.mStatWork.size(); i_st++ )
+		if( cntr.mStatWork[i_st].second > 0 ) cntr.mStatWork[i_st].second = vmax(0,cntr.mStatWork[i_st].second-cntr.period());
+		else isAccess = true;
 
-	    if( !cntr.redntUse( ) )
+	    if( isAccess )
 	    {
-		//> Allow stations presenting
-		bool isAccess = false;
-		for( sti = cntr.mStatWork.begin(); sti != cntr.mStatWork.end(); sti++ )
-		    if( sti->second > 0 ) sti->second = vmax(0,sti->second-cntr.period());
-		    else isAccess = true;
+		unsigned int div = (unsigned int)(cntr.mSync/cntr.period());
+		vector<string> pLS;
+		cntr.list(pLS);
+		AutoHD<TMdPrm> prm;
+		string scntr;
 
-		//> Update controller's data
-		for( int i_p=0; isAccess && i_p < cntr.pHd.size(); i_p++)
+		//> Parameters list update
+		if( (it_cnt > div && it_cnt%div == 0) || pLS.empty() )
+		    try { res.release( ); cntr.enable_( ); res.request(false); }
+		    catch(TError err) { }
+
+		//> Mark no process
+		for( int i_p=0; i_p < pLS.size(); i_p++)
 		{
-		    //>> Update parameter's values
-		    cntr.pHd[i_p].at().update();
-		    //>> Check for sync moment and sync parameter's structure
-		    unsigned int div = (unsigned int)(cntr.mSync/cntr.period());
-		    if( div && (it_cnt+i_p)%div == 0 ) cntr.pHd[i_p].at().load();
+		    if( it_cnt > div && (it_cnt+i_p)%div == 0 ) cntr.at(pLS[i_p]).at().load_();
+		    cntr.at(pLS[i_p]).at().isPrcOK = false;
+		}
+
+		//> Station's cycle
+		for( int i_st = 0; i_st < cntr.mStatWork.size(); i_st++ )
+		{
+		    if( cntr.mStatWork[i_st].second > 0 ) continue;
+		    XMLNode req("CntrReqs"); req.setAttr("path","/"+cntr.mStatWork[i_st].first+"/DAQ/");
+
+		    //> Put attributes reuests
+		    for( int i_p=0; i_p < pLS.size(); i_p++)
+		    {
+			prm = cntr.at(pLS[i_p]);
+			if( prm.at().isPrcOK ) continue;
+			for( int c_off = 0; (scntr=TSYS::strSepParse(prm.at().cntrAdr(),0,';',&c_off)).size(); )
+			{
+			    if( TSYS::pathLev(scntr,0) != cntr.mStatWork[i_st].first ) continue;
+
+			    XMLNode *prmNd = req.childAdd("get")->setAttr("path","/"+TSYS::pathLev(scntr,2)+"/"+TSYS::pathLev(scntr,3)+"/"+prm.at().id()+"/%2fserv%2fattr");
+			    prmNd->setAttr( "hostTm", !cntr.restDtTm() ? "1" : "0" );
+
+			    //>> Prepare individual attributes list
+			    bool sepReq = !prm.at().isEVAL && ((it_cnt+i_p)%div);
+			    prmNd->setAttr( "sepReq", sepReq ? "1" : "0" );
+			    if( !cntr.restDtTm() && !sepReq ) continue;
+
+			    vector<string> listV;
+			    prm.at().vlList(listV);
+			    int rC = 0;
+			    for( int iV = 0; iV < listV.size(); iV++ )
+			    {
+				AutoHD<TVal> vl = prm.at().vlAt(listV[iV]);
+				if( sepReq && (!vl.at().arch().freeStat() || vl.at().reqFlg()) ) { prmNd->childAdd("el")->setAttr("id",listV[iV]); rC++; }
+				if( !vl.at().arch().freeStat() )
+				    prmNd->childAdd("ael")->setAttr("id",listV[iV])->setAttr("tm",TSYS::ll2str(vmax(vl.at().arch().at().end(""),TSYS::curTime()-(long long)(3.6e9*cntr.restDtTm()))));
+			    }
+			    if( sepReq && rC > listV.size()/2 )
+			    {
+				prmNd->childClear("el");
+				prmNd->setAttr( "sepReq", "0" );
+			    }
+			}
+		    }
+		    if( !req.childSize() ) continue;
+
+		    //> Same request
+		    if( cntr.cntrIfCmd(req,true) ) { mess_err(req.attr("mcat").c_str(),"%s",req.text().c_str()); continue; }
+
+		    //> Result process
+		    for( int i_r = 0; i_r < req.childSize(); i_r++ )
+		    {
+			XMLNode *prmNd = req.childGet(i_r);
+			if( atoi(prmNd->attr("err").c_str()) ) continue;
+
+			prm = cntr.at(TSYS::pathLev(prmNd->attr("path"),2));
+
+			if( prm.at().isPrcOK ) continue;
+			prm.at().isPrcOK = true;
+			prm.at().isEVAL = false;
+
+			for( int i_a = 0; i_a < prmNd->childSize(); i_a++ )
+			{
+			    XMLNode *aNd = prmNd->childGet(i_a);
+			    if( !prm.at().vlPresent(aNd->attr("id")) ) continue;
+			    AutoHD<TVal> vl = prm.at().vlAt(aNd->attr("id"));
+
+			    if( aNd->name() == "el" )
+			    { vl.at().setS(aNd->text(),cntr.restDtTm()?atoll(aNd->attr("tm").c_str()):0,true); vl.at().setReqFlg(false); }
+			    else if( aNd->name() == "ael" && !vl.at().arch().freeStat() && aNd->childSize() )
+			    {
+				long long btm = atoll(aNd->attr("tm").c_str());
+				long long per = atoll(aNd->attr("per").c_str());
+				TValBuf buf(vl.at().arch().at().valType(),0,per,false,true);
+				for( int i_v = 0; i_v < aNd->childSize(); i_v++ )
+				    buf.setS(aNd->childGet(i_v)->text(),btm+per*i_v);
+				vl.at().arch().at().setVals(buf,buf.begin(),buf.end(),"");
+			    }
+			}
+		    }
+		}
+
+		//> Mark no processed parameters to EVAL
+		for( int i_p=0; i_p < pLS.size(); i_p++)
+		{
+		    prm = cntr.at(pLS[i_p]);
+		    if( prm.at().isPrcOK || prm.at().isEVAL ) continue;
+		    vector<string> vLs;
+		    prm.at().elem().fldList(vLs);
+		    for( int i_v = 0; i_v < vLs.size(); i_v++ )
+		    {
+			if( vLs[i_v] == "SHIFR" || vLs[i_v] == "NAME" || vLs[i_v] == "DESCR" ) continue;
+			AutoHD<TVal> vl = prm.at().vlAt(vLs[i_v]);
+			if( vl.at().getS() == EVAL_STR ) continue;
+			vl.at().setS( EVAL_STR, vl.at().arch().freeStat() ? 0 :
+			    vmin(TSYS::curTime(),vmax(vl.at().arch().at().end(""),TSYS::curTime()-(long long)(3.6e9*cntr.restDtTm()))), true );
+		    }
+		    prm.at().vlAt("err").at().setS(_("10:Data not allow."),0,true);
+		    prm.at().isEVAL = true;
 		}
 	    }
+	    //res.release( );
+	}catch(TError err)	{ mess_err(err.cat.c_str(),err.mess.c_str()); }
 
-	    //> Calc acquisition process time
-	    cntr.enRes.resRelease( );
-	    cntr.tmGath = 1e-3*(TSYS::curTime()-t_cnt);
+	//> Calc acquisition process time
+	cntr.tmGath = 1e-3*(TSYS::curTime()-t_cnt);
+
+	TSYS::taskSleep((long long)(1e9*cntr.period()));
+    }
 
-	    TSYS::taskSleep((long long)cntr.period()*1000000000);
-	}
-    }catch(TError err)	{ mess_err(err.cat.c_str(),err.mess.c_str()); }
 
     cntr.prcSt = false;
 
     return NULL;
 }
 
-int TMdContr::cntrIfCmd( XMLNode &node, bool strongSt )
+int TMdContr::cntrIfCmd( XMLNode &node, bool lockErr )
 {
-    int off = 0, rez;
-    string reqStat = TSYS::pathLev(node.attr("path"),0,true,&off);
-    string srcPath = node.attr("path").substr(off);
+    string reqStat = TSYS::pathLev(node.attr("path"),0);
 
-    map<string,float>::iterator sti = mStatWork.find(reqStat);
-    if( sti != mStatWork.end() && sti->second <= 0 )
-	try{ rez = SYS->transport().at().cntrIfCmd(node,"DAQGate"); sti->second-=1; return rez; }
-	catch(...){ sti->second = mRestTm; }
-
-    for( sti = mStatWork.begin(); !strongSt && sti != mStatWork.end(); sti++ )
+    try
     {
-	if( sti->second > 0 || sti->first == reqStat ) continue;
-	node.setAttr("path","/"+sti->first+srcPath);
-	try { rez = SYS->transport().at().cntrIfCmd(node,"DAQGate"); sti->second-=1; return rez; }
-	catch( TError err ) { sti->second = mRestTm; }
-    }
-    throw TError(nodePath().c_str(),_("Not one accessable stations present."));
+	for( int i_st = 0; i_st < mStatWork.size(); i_st++ )
+	    if( mStatWork[i_st].first == reqStat )
+	    {
+		if( mStatWork[i_st].second > 0 ) break;
+		try
+		{
+		    int rez = SYS->transport().at().cntrIfCmd(node,MOD_ID+id());
+		    mStatWork[i_st].second -= 1;
+		    return rez;
+		}
+		catch(...){ if( lockErr ) mStatWork[i_st].second = mRestTm; throw; }
+	    }
+    }catch(TError err) { node.setAttr("mcat",err.cat)->setAttr("err","10")->setText(err.mess); }
+
+    return atoi(node.attr("err").c_str());
 }
 
 void TMdContr::cntrCmdProc( XMLNode *opt )
@@ -392,7 +460,7 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
 
     //- Process command to page -
     string a_path = opt->attr("path");
-    if( a_path == "/cntr/cfg/host_lnk" && ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )
+    if( a_path == "/cntr/cfg/host_lnk" && ctrChkNode(opt,"get",0660,"root","root",SEC_RD) )
     {
 	SYS->transport().at().setSysHost(true);
 	opt->setText("/Transport");
@@ -403,7 +471,7 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
 //******************************************************
 //* TMdPrm                                             *
 //******************************************************
-TMdPrm::TMdPrm( string name, TTipParam *tp_prm ) : TParamContr(name,tp_prm), p_el("w_attr")
+TMdPrm::TMdPrm( string name, TTipParam *tp_prm ) : TParamContr(name,tp_prm), p_el("w_attr"), isPrcOK(false), isEVAL(true)
 {
     setToEnable(true);
 }
@@ -427,16 +495,12 @@ void TMdPrm::enable()
     if( enableStat() )	return;
 
     TParamContr::enable();
-
-    owner().prmEn( id(), true );
 }
 
 void TMdPrm::disable()
 {
     if( !enableStat() )  return;
 
-    owner().prmEn( id(), false );
-
     TParamContr::disable();
 
     //> Set EVAL to parameter attributes
@@ -448,6 +512,8 @@ void TMdPrm::disable()
 
 void TMdPrm::setCntrAdr( const string &vl )
 {
+    if( vl.empty() ) { mCntrAdr = ""; return; }
+
     string scntr;
     for( int off = 0; (scntr=TSYS::strSepParse(mCntrAdr,0,';',&off)).size(); )
 	if( scntr == vl ) return;
@@ -460,7 +526,6 @@ void TMdPrm::load_( )
     XMLNode req("CntrReqs");
     //> Request and update attributes list
     for( int c_off = 0; (scntr=TSYS::strSepParse(cntrAdr(),0,';',&c_off)).size(); )
-    {
 	try
 	{
 	    req.clear()->setAttr("path",scntr+id());
@@ -485,14 +550,13 @@ void TMdPrm::load_( )
 		    case TFld::Real:	dvl = TSYS::real2str(EVAL_REAL);break;
 		}
 		p_el.fldAdd( new TFld( ael->attr("id").c_str(),ael->attr("nm").c_str(),tp,
-		    atoi(ael->attr("flg").c_str())&(TFld::Selected|TFld::NoWrite|TFld::HexDec|TFld::OctDec|TFld::FullText)|TVal::DirWrite,
+		    atoi(ael->attr("flg").c_str())&(TFld::Selected|TFld::NoWrite|TFld::HexDec|TFld::OctDec|TFld::FullText)|TVal::DirWrite|TVal::DirRead,
 		    "",dvl.c_str(),ael->attr("vals").c_str(),ael->attr("names").c_str()) );
 	    }
 	    //>> Remove attributes
 	    //????
 	    return;
 	}catch(TError err) { continue; }
-    }
 }
 
 void TMdPrm::save_( )
@@ -505,54 +569,21 @@ void TMdPrm::save_( )
 	    vlAt(a_ls[i_a]).at().arch().at().save();
 }
 
-void TMdPrm::update( )
+void TMdPrm::vlGet( TVal &val )
 {
-    string scntr;
-    XMLNode req("get");
-    //> Request and update attributes list
-    for( int c_off = 0; (scntr=TSYS::strSepParse(cntrAdr(),0,';',&c_off)).size(); )
-	try
-	{
-	    //>> Attributes values request
-	    if( mRedntTmLast ) mRedntTmLast = vmax(mRedntTmLast,TSYS::curTime()-(long long)(3.6e9*owner().restDtTm()));
-	    req.clear()->setAttr("path",scntr+id()+"/%2fserv%2fattr")->setAttr("tm",TSYS::ll2str(mRedntTmLast));
-	    if( owner().cntrIfCmd(req) ) throw TError(req.attr("mcat").c_str(),req.text().c_str());
-	    mRedntTmLast = atoll(req.attr("tm").c_str());
-	    for( int i_a = 0; req.childSize(); i_a++ )
-	    {
-		XMLNode *aNd = req.childGet(i_a);
-		if( !vlPresent(aNd->attr("id")) ) continue;
-		AutoHD<TVal> vl = vlAt(aNd->attr("id"));
-		if( aNd->attr("tm").empty() ) vl.at().setS(aNd->text(),mRedntTmLast,true);
-		else
-		{
-		    long long btm = atoll(aNd->attr("tm").c_str());
-		    long long per = atoll(aNd->attr("per").c_str());
-		    if( !vl.at().arch().freeStat() )
-			for( int i_v = 0; i_v < aNd->childSize(); i_v++ )
-			{
-			    TValBuf buf(vl.at().arch().at().valType(),0,per,false,true);
-			    for( int i_v = 0; i_v < aNd->childSize(); i_v++ )
-			    buf.setS(aNd->childGet(i_v)->text(),btm+per*i_v);
-			    vl.at().arch().at().setVals(buf,buf.begin(),buf.end(),"");
-			}
-		    vl.at().setS(aNd->childGet(aNd->childSize()-1)->text(),btm+per*(aNd->childSize()-1),true);
-		}
-	    }
-	    return;
-	}catch(TError err) { continue; }
+    if( val.name() == "err" && (!enableStat() || !owner().startStat()) ) TParamContr::vlGet(val);
 }
 
 void TMdPrm::vlSet( TVal &valo, const TVariant &pvl )
 {
     if( !enableStat() || !owner().startStat() )	valo.setI(EVAL_INT,0,true);
+    if( valo.getS() == EVAL_STR || valo.getS() == pvl.getS() ) return;
 
     XMLNode req("set");
 
     //> Send to active reserve station
     if( owner().redntUse( ) )
     {
-	if( valo.getS() == pvl.getS() ) return;
 	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",valo.name())->setText(valo.getS());
 	SYS->daq().at().rdStRequest(owner().workId(),req);
 	return;
@@ -564,7 +595,7 @@ void TMdPrm::vlSet( TVal &valo, const TVariant &pvl )
 	{
 	    req.clear()->setAttr("path",scntr+id()+"/%2fserv%2fattr")->
 		childAdd("el")->setAttr("id",valo.name())->setText(valo.getS());
-	    if( owner().cntrIfCmd(req,true) )   throw TError(req.attr("mcat").c_str(),req.text().c_str());
+	    if( owner().cntrIfCmd(req) )   throw TError(req.attr("mcat").c_str(),req.text().c_str());
 	}catch(TError err) { continue; }
 }
 
@@ -572,7 +603,7 @@ void TMdPrm::vlArchMake( TVal &val )
 {
     if( val.arch().freeStat() ) return;
     val.arch().at().setSrcMode(TVArchive::PassiveAttr,val.arch().at().srcData());
-    val.arch().at().setPeriod((long long)(owner().period()*1000000));
+    val.arch().at().setPeriod((long long)(1e6*owner().period()));
     val.arch().at().setHardGrid( true );
     val.arch().at().setHighResTm( true );
 }
@@ -627,8 +658,8 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     if( a_path == "/prm/st/type" && ctrChkNode(opt) )   opt->setText(type().descr);
     else if( a_path == "/prm/st/en" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )   opt->setText(enableStat()?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )   opt->setText(enableStat()?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    if( !owner().enableStat() ) throw TError(nodePath().c_str(),"Controller is not started!");
 	    else atoi(opt->text().c_str())?enable():disable();
@@ -644,7 +675,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	    try
 	    {
 		opt->setAttr("path",scntr+id()+"/"+TSYS::strEncode(a_path,TSYS::PathEl));
-		if( owner().cntrIfCmd(*opt,true) ) throw TError(opt->attr("mcat").c_str(),opt->text().c_str());
+		if( owner().cntrIfCmd(*opt) ) throw TError(opt->attr("mcat").c_str(),opt->text().c_str());
 	    }catch(TError err) { continue; }
 	opt->setAttr("path",a_path);
     }
@@ -658,11 +689,11 @@ TMdPrm &TMdVl::owner( )	{ return *(dynamic_cast<TMdPrm*>(nodePrev())); }
 
 void TMdVl::cntrCmdProc( XMLNode *opt )
 {
-    if( !arch( ).freeStat() ) TVal::cntrCmdProc(opt);
+    if( !arch( ).freeStat() ) { TVal::cntrCmdProc(opt); return; }
 
     string a_path = opt->attr("path");
     //> Service commands process
-    if( a_path == "/serv/val"  )	//Values access
+    if( a_path == "/serv/val" && owner().owner().restDtTm() )	//Values access
     {
 	//>> Request to remote station
 	string scntr;
