@@ -39,8 +39,8 @@
 #define MOD_NAME	"DCON client"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"0.2.0"
-#define AUTORS		"Almaz Karimov"
+#define VERSION		"0.3.2"
+#define AUTORS		"Roman Savochenko, Almaz Karimov"
 #define DESCRIPTION	"Allow realisation of DCON client service. Supported I-7000 DCON protocol."
 #define LICENSE		"GPL"
 //*************************************************
@@ -87,56 +87,28 @@ TTpContr::~TTpContr()
 
 }
 
-string TTpContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TTpContr::postEnable( int flag )
 {
     TTipDAQ::postEnable( flag );
 
-    //- Controler's bd structure -
+    //> Controler's bd structure
     fldAdd( new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30","") );
     fldAdd( new TFld("PERIOD",_("Gather data period (s)"),TFld::Real,TFld::NoFlag,"6.2","1","0.01;100") );
     fldAdd( new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
     fldAdd( new TFld("ADDR",_("Serial transport"),TFld::String,TFld::NoFlag,"30","") );
+    fldAdd( new TFld("REQ_TRY",_("Request tries"),TFld::Integer,TFld::NoFlag,"1","3","1;10") );
 
-    //- Parameter type bd structure -
+    //> Parameter type bd structure
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
-    tpPrmAt(t_prm).fldAdd( new TFld("MOD_TP",_("I-7000 module type"),TFld::Integer,TFld::Selected|TCfg::NoVal,"1","0","0;1;2;3;4",_("I-7051;I-7045;I-7063;I-7017;I-7024")) );
+    tpPrmAt(t_prm).fldAdd( new TFld("MOD_TP",_("I-7000 module type"),TFld::Integer,TFld::Selected|TCfg::NoVal,"1","0","0;1;2;3;4;5;6",_("I-7051;I-7045;I-7063;I-7017;I-7024;NL-8AI;HostOK")) );
     tpPrmAt(t_prm).fldAdd( new TFld("MOD_ADDR",_("I-7000 module address"),TFld::Integer,TFld::NoFlag|TCfg::NoVal,"20","1","0;255") );
     tpPrmAt(t_prm).fldAdd( new TFld("CRC_CTRL",_("CRC control"),TFld::Boolean,TFld::NoFlag|TCfg::NoVal,"1","1") );
 }
 
 void TTpContr::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt = "h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind = opterr = 0;
-    do
-    {
-	next_opt = getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch( next_opt )
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while( next_opt != -1 );
 }
 
 void TTpContr::save_()
@@ -156,7 +128,7 @@ TController *TTpContr::ContrAttach( const string &name, const string &daq_db )
 TMdContr::TMdContr( string name_c, const string &daq_db, TElem *cfgelem ) :
 	TController( name_c, daq_db, cfgelem ), prc_st(false), endrun_req(false), tm_gath(0),
 	m_per(cfg("PERIOD").getRd()), m_prior(cfg("PRIOR").getId()),
-	m_addr(cfg("ADDR").getSd())
+	m_addr(cfg("ADDR").getSd()), connTry(cfg("REQ_TRY").getId())
 {
     cfg("PRM_BD").setS("DCONPrm_"+name_c);
 }
@@ -193,33 +165,14 @@ void TMdContr::start_( )
         SYS->transport().at().at("Serial").at().outAt(m_addr).at().start();
 
 	//> Start the gathering data task
-	pthread_attr_t pthr_attr;
-	pthread_attr_init( &pthr_attr );
-	struct sched_param prior;
-	if( m_prior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy( &pthr_attr, SCHED_RR );
-	else pthread_attr_setschedpolicy( &pthr_attr, SCHED_OTHER );
-	prior.__sched_priority = m_prior;
-	pthread_attr_setschedparam( &pthr_attr, &prior );
-
-	pthread_create( &procPthr, &pthr_attr, TMdContr::Task, this );
-	pthread_attr_destroy( &pthr_attr );
-	if( TSYS::eventWait( prc_st, true, nodePath()+"start", 5 ) )
-	    throw TError( nodePath().c_str(), _("Gathering task is not started!") );
+	SYS->taskCreate( nodePath('.',true), m_prior, TMdContr::Task, this, &prc_st );
     }
 }
 
 void TMdContr::stop_( )
 {
-    if( prc_st )
-    {
-	//- Stop the request and calc data task -
-	endrun_req = true;
-	pthread_kill( procPthr, SIGALRM );
-	if( TSYS::eventWait( prc_st, false, nodePath()+"stop", 5 ) )
-	    throw TError( nodePath().c_str(), _("Gathering task is not stopped!") );
-	pthread_join( procPthr, NULL );
-    }
+    //> Stop the request and calc data task
+    if( prc_st ) SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
 }
 
 bool TMdContr::cfgChange( TCfg &icfg )
@@ -253,27 +206,45 @@ string TMdContr::DCONReq( string &pdu )
 {
     ResAlloc res( req_res, true );
     char buf[1000];
-    string rez = "";
-
-    AutoHD<TTransportOut> tr;
-    try{ tr = SYS->transport().at().at("Serial").at().outAt(m_addr); }
-    catch(...) { tr.at().stop(); throw; }
-    if( !tr.at().startStat() )      tr.at().start();
+    string rez, err;
 
     try
     {
-	int resp_len = tr.at().messIO((pdu+"\r").data(),pdu.size()+1,buf,sizeof(buf));
-	rez.assign(buf,resp_len);
-    }catch( TError err ) { return _("14:Request error: ")+err.mess; }
-    pdu = rez;
+	AutoHD<TTransportOut> tr = SYS->transport().at().at("Serial").at().outAt(m_addr);
+	if( !tr.at().startStat() ) tr.at().start();
+	pdu += "\r";
 
-    return "";
+	ResAlloc resN( tr.at().nodeRes(), true );
+
+	for( int i_tr = 0; i_tr < vmax(1,vmin(10,connTry)); i_tr++ )
+	{
+	    int resp_len = tr.at().messIO( pdu.data(), pdu.size(), buf, sizeof(buf), 0, true );
+	    rez.assign(buf,resp_len);
+
+	    //> Wait tail
+	    while( rez.size() < 2 || rez[rez.size()-1] != '\r' )
+	    {
+		try{ resp_len = tr.at().messIO( NULL, 0, buf, sizeof(buf), 0, true ); } catch(TError er){ break; }
+		rez.append( buf, resp_len );
+	    }
+	    if( rez.size() < 2 || rez[rez.size()-1] != '\r' )	{ err = _("13:Error respond: Not full."); continue; }
+	    pdu = rez;//.substr(0,rez.size()-1);
+	    err = "";
+	    break;
+	}
+    }
+    catch(TError er) { err = _("10:Transport error: ")+er.mess; }
+
+    if( !err.empty() ) pdu = "";
+
+    return err;
 }
 
 void *TMdContr::Task( void *icntr )
 {
     const string HexSymbol[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
     string pdu;
+    int acq_len;
     long long work_tm, last_tm = 0;
     struct timespec get_tm;
     TMdContr &cntr = *(TMdContr *)icntr;
@@ -307,17 +278,20 @@ void *TMdContr::Task( void *icntr )
 				//> Request with I-7051 module
 				pdu = "@" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr];
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=7; else acq_len=5;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
+				cntr.p_hd[i_p].at().acq_err=_("No errors");
 				cntr.p_hd[i_p].at().module_err=false;
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(5,2))!=(cntr.DCONCRC(pdu.substr(0,5)))) cntr.p_hd[i_p].at().module_err=true;
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>6)) if ((pdu.substr(5,2))!=(cntr.DCONCRC(pdu.substr(0,5)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 
 				//Set DAQ atributes
-				if (!cntr.p_hd[i_p].at().module_err)
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>4))
 				{
 				    unsigned int DI = strtoul(pdu.substr(1,4).c_str(),NULL,16);
 //				    cout << DI << "\n";
@@ -350,14 +324,17 @@ void *TMdContr::Task( void *icntr )
 				pdu+=HexSymbol[8*(1&cntr.p_hd[i_p].at().DO[7])+4*(1&cntr.p_hd[i_p].at().DO[6])+2*(1&cntr.p_hd[i_p].at().DO[5])+(1&cntr.p_hd[i_p].at().DO[4])];
 				pdu+=HexSymbol[8*(1&cntr.p_hd[i_p].at().DO[3])+4*(1&cntr.p_hd[i_p].at().DO[2])+2*(1&cntr.p_hd[i_p].at().DO[1])+(1&cntr.p_hd[i_p].at().DO[0])];
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=3; else acq_len=1;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
+				cntr.p_hd[i_p].at().acq_err=_("No errors");
 				cntr.p_hd[i_p].at().module_err=false;
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) cntr.p_hd[i_p].at().module_err=true;
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>2)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 				break;
 			    }
 			    //> Read-Write DAQ parameter's for I-7063 module
@@ -366,17 +343,20 @@ void *TMdContr::Task( void *icntr )
 				//> Request with I-7063 module
 				pdu = "@" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr];
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=7; else acq_len=5;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
+				cntr.p_hd[i_p].at().acq_err=_("No errors");
 				cntr.p_hd[i_p].at().module_err=false;
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(5,2))!=(cntr.DCONCRC(pdu.substr(0,5)))) cntr.p_hd[i_p].at().module_err=true;
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>6)) if ((pdu.substr(5,2))!=(cntr.DCONCRC(pdu.substr(0,5)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 
 				//Set DAQ atributes
-				if (!cntr.p_hd[i_p].at().module_err)
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>4))
 				{
 				    unsigned int DI = strtoul(pdu.substr(3,2).c_str(),NULL,16);
 //				    cout << DI << "\n";
@@ -394,13 +374,16 @@ void *TMdContr::Task( void *icntr )
 				pdu = "@" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr];
 				pdu+=HexSymbol[4*(1&cntr.p_hd[i_p].at().DO[2])+2*(1&cntr.p_hd[i_p].at().DO[1])+(1&cntr.p_hd[i_p].at().DO[0])];
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=3; else acq_len=1;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) cntr.p_hd[i_p].at().module_err=true;
+				if (cntr.p_hd[i_p].at().acq_err=="") cntr.p_hd[i_p].at().acq_err=_("No errors");
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>2)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 				break;
 			    }
 			    //> Read DAQ parameter's for I-7017 module
@@ -409,17 +392,20 @@ void *TMdContr::Task( void *icntr )
 				//> Request with I-7017 module
 				pdu = "#" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr];
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=59; else acq_len=57;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
+				cntr.p_hd[i_p].at().acq_err=_("No errors");
 				cntr.p_hd[i_p].at().module_err=false;
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(57,2))!=(cntr.DCONCRC(pdu.substr(0,57)))) cntr.p_hd[i_p].at().module_err=true;
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>58)) if ((pdu.substr(57,2))!=(cntr.DCONCRC(pdu.substr(0,57)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 
 				//Set DAQ atributes
-				if (!cntr.p_hd[i_p].at().module_err)
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>56))
 				{
 				    cntr.p_hd[i_p].at().AI[0]=atof(pdu.substr(1,7).c_str());
 				    cntr.p_hd[i_p].at().AI[1]=atof(pdu.substr(8,7).c_str());
@@ -444,14 +430,17 @@ void *TMdContr::Task( void *icntr )
 				str="+"+str.substr(0,2)+"."+str.substr(2,3);
 				pdu = "#" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr]+"0"+str;
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=3; else acq_len=1;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
+				cntr.p_hd[i_p].at().acq_err=_("No errors");
 				cntr.p_hd[i_p].at().module_err=false;
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) cntr.p_hd[i_p].at().module_err=true;
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>2)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 
 				//> Request with I-7024 module
 				str=fcvt(cntr.p_hd[i_p].at().AO[1],ndig,&dec,&sign);
@@ -459,13 +448,16 @@ void *TMdContr::Task( void *icntr )
 				str="+"+str.substr(0,2)+"."+str.substr(2,3);
 				pdu = "#" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr]+"1"+str;
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=3; else acq_len=1;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) cntr.p_hd[i_p].at().module_err=true;
+				if (cntr.p_hd[i_p].at().acq_err=="") cntr.p_hd[i_p].at().acq_err=_("No errors");
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>2)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 
 				//> Request with I-7024 module
 				str=fcvt(cntr.p_hd[i_p].at().AO[2],ndig,&dec,&sign);
@@ -473,13 +465,16 @@ void *TMdContr::Task( void *icntr )
 				str="+"+str.substr(0,2)+"."+str.substr(2,3);
 				pdu = "#" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr]+"2"+str;
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=3; else acq_len=1;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) cntr.p_hd[i_p].at().module_err=true;
+				if (cntr.p_hd[i_p].at().acq_err=="") cntr.p_hd[i_p].at().acq_err=_("No errors");
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>2)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 
 				//- Request with I-7024 module
 				str=fcvt(cntr.p_hd[i_p].at().AO[3],ndig,&dec,&sign);
@@ -487,16 +482,76 @@ void *TMdContr::Task( void *icntr )
 				str="+"+str.substr(0,2)+"."+str.substr(2,3);
 				pdu = "#" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr]+"3"+str;
 				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=3; else acq_len=1;
 //				cout << pdu.substr(0,pdu.size()) << " writing\n";
 				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
 //				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
 
 				//Check errors
-				if (!cntr.p_hd[i_p].at().module_err) if (pdu.substr(0,1)!=">") cntr.p_hd[i_p].at().module_err=true;
-				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) cntr.p_hd[i_p].at().module_err=true;
+				if (cntr.p_hd[i_p].at().acq_err=="") cntr.p_hd[i_p].at().acq_err=_("No errors");
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>2)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
 
 				break;
 			    }
+			    //> Read DAQ parameter's for NL-8AI module
+			    case 5:
+			    {
+				//> Request with NL-8AI module
+				pdu = "#" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr];
+				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=59; else acq_len=57;
+//				cout << pdu.substr(0,pdu.size()) << " writing\n";
+				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
+//				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
+
+				//Check errors
+				cntr.p_hd[i_p].at().acq_err=_("No errors");
+				cntr.p_hd[i_p].at().module_err=false;
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>58)) if ((pdu.substr(57,2))!=(cntr.DCONCRC(pdu.substr(0,57)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
+
+				//Set DAQ atributes
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>56))
+				{
+				    cntr.p_hd[i_p].at().AI[0]=atof(pdu.substr(1,7).c_str());
+				    cntr.p_hd[i_p].at().AI[1]=atof(pdu.substr(8,7).c_str());
+				    cntr.p_hd[i_p].at().AI[2]=atof(pdu.substr(15,7).c_str());
+				    cntr.p_hd[i_p].at().AI[3]=atof(pdu.substr(22,7).c_str());
+				    cntr.p_hd[i_p].at().AI[4]=atof(pdu.substr(29,7).c_str());
+				    cntr.p_hd[i_p].at().AI[5]=atof(pdu.substr(36,7).c_str());
+				    cntr.p_hd[i_p].at().AI[6]=atof(pdu.substr(43,7).c_str());
+				    cntr.p_hd[i_p].at().AI[7]=atof(pdu.substr(50,7).c_str());
+				}
+				//> Request with NL-8AI module
+				pdu = "^" + HexSymbol[15 & (cntr.p_hd[i_p].at().mod_addr>>4)] + HexSymbol[15 & cntr.p_hd[i_p].at().mod_addr] + "DO";
+				pdu+=HexSymbol[1&cntr.p_hd[i_p].at().DO[2]] + HexSymbol[1&cntr.p_hd[i_p].at().DO[1]] + HexSymbol [1&cntr.p_hd[i_p].at().DO[0]];
+				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+				if (cntr.p_hd[i_p].at().crc_ctrl) acq_len=3; else acq_len=1;
+//				cout << pdu.substr(0,pdu.size()) << " writing\n";
+				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
+//				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
+
+				//Check errors
+				if (cntr.p_hd[i_p].at().acq_err=="") cntr.p_hd[i_p].at().acq_err=_("No errors");
+				if ((pdu.size()-1)!=acq_len) {cntr.p_hd[i_p].at().module_err=true; cntr.p_hd[i_p].at().acq_err=_("Module packet length error");}
+				if ((!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>0)) if (pdu.substr(0,1)!=">") {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module response");}
+				if ((cntr.p_hd[i_p].at().crc_ctrl)&&(!cntr.p_hd[i_p].at().module_err)&&(pdu.size()>2)) if ((pdu.substr(1,2))!=(cntr.DCONCRC(pdu.substr(0,1)))) {cntr.p_hd[i_p].at().module_err=true;cntr.p_hd[i_p].at().acq_err=_("Invalid module CRC");}
+				break;
+			    }
+			    //> Send HostOK for all modules
+			    case 6:
+			    {
+				pdu = "~**";
+				if (cntr.p_hd[i_p].at().crc_ctrl) pdu+=cntr.DCONCRC(pdu);
+//				cout << pdu.substr(0,pdu.size()) << " writing\n";
+				cntr.p_hd[i_p].at().acq_err=cntr.DCONReq(pdu);
+				cntr.p_hd[i_p].at().acq_err=_("No errors");
+//				cout << pdu.substr(0,pdu.size()-1) << " reading\n";
+				break;
+			    }
 			    default:	break;
 			}
 		    }
@@ -668,6 +723,27 @@ void TMdPrm::enable()
 	    p_el.fldAdd( new TFld("AO3", "AO3", TFld::Real, TVal::DirRead|TVal::DirWrite,"20","1","4;20") );
 	    break;
 	}
+	case 5:
+	{
+	    p_el.fldAdd( new TFld("module_err", "module_err", TFld::Boolean, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI0", "AI0", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI1", "AI1", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI2", "AI2", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI3", "AI3", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI4", "AI4", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI5", "AI5", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI6", "AI6", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+	    p_el.fldAdd( new TFld("AI7", "AI7", TFld::Real, TFld::NoWrite|TVal::DirRead, "", "", "" ) );
+
+	    p_el.fldAdd( new TFld("DO0", "DO0", TFld::Boolean, TVal::DirRead|TVal::DirWrite, "", "", "" ) );
+	    p_el.fldAdd( new TFld("DO1", "DO1", TFld::Boolean, TVal::DirRead|TVal::DirWrite, "", "", "" ) );
+	    p_el.fldAdd( new TFld("DO2", "DO2", TFld::Boolean, TVal::DirRead|TVal::DirWrite, "", "", "" ) );
+	    break;
+	}
+	case 6:
+	{
+	    break;
+	}
 	default:
 	{
 	    break;
@@ -709,6 +785,7 @@ void TMdPrm::vlGet( TVal &val )
 //    cout<<val.name().substr(0,2)<<"  "<<atoi(val.name().substr(2,val.name().size()-2).c_str())<<"\n";
 
     if (val.name()=="module_err") val.setB(module_err,0,true);
+    if (val.name()=="err") val.setS(acq_err,0,true);
     if (val.name().substr(0,2)=="DI") val.setB(DI[atoi(val.name().substr(2,val.name().size()-2).c_str())],0,true);
     if (val.name().substr(0,2)=="DO") val.setB(DO[atoi(val.name().substr(2,val.name().size()-2).c_str())],0,true);
     if (val.name().substr(0,2)=="AI") val.setR(AI[atoi(val.name().substr(2,val.name().size()-2).c_str())],0,true);
