@@ -1,8 +1,8 @@
 
 //OpenSCADA system module DAQ.DiamondBoards file: diamond.cpp
 /***************************************************************************
- *   Copyright (C) 2005-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2005-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -189,19 +189,19 @@ void TMdContr::load_( )
 
 void TMdContr::start_( )
 {
-    //- Check inited of Diamond API -
-    if( !mod->drvInitOk() )
-	throw TError(nodePath().c_str(),_("DSC driver is not initialized!"));
-
-    //- DSC strucures init -
-    ERRPARAMS errorParams;
-    DSCADSETTINGS dscadsettings;
-    memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));
-
-    //- Main DSC code -
-    if(!dataEmul())
+    //> Main DSC code
+    if( !dataEmul() )
     {
-	//-- Init Board --
+	//> Check inited of Diamond API
+	if( !mod->drvInitOk() )
+	    throw TError(nodePath().c_str(),_("DSC driver is not initialized!"));
+
+	//> DSC strucures init
+	ERRPARAMS errorParams;
+	DSCADSETTINGS dscadsettings;
+	memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));
+
+	//>> Init Board
 	DSCCB dsccb;
 	dsccb.base_address = m_addr;
 	dsccb.int_level = cfg("INT").getI();
@@ -211,45 +211,28 @@ void TMdContr::start_( )
 	    throw TError(nodePath().c_str(),_("dscInitBoard %d(%xh) error: %s %s"),
 	        cfg("BOARD").getI(),m_addr,dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
 	}
-	//-- Set DIO config --
+	//>> Set DIO config
 	BYTE cfg_byte = cfg("DIO_CFG").getI()|0x80;
 	if( dscDIOSetConfig(dscb, &cfg_byte) != DE_NONE )
  	{
 	    dscGetLastError(&errorParams);
 	    throw TError(nodePath().c_str(),_("dscDIOSetConfig error: %s %s"),dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
 	}
-	//-- Init AD acquisition --
+	//>> Init AD acquisition
 	dscadsettings.range = cfg("ADRANGE").getI();
 	dscadsettings.polarity = cfg("ADPOLAR").getI();
 	dscadsettings.gain = cfg("ADGAIN").getI();
 	dscadsettings.load_cal = 0;
     }
 
-    if(ad_int_mode)
-    {
-	//- Create interrupt AD DSC task -
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	pthread_create(&ad_dsc_pthr,&pthr_attr,AD_DSCTask,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(ad_dsc_st, true, nodePath()+"addsc_task_start",5) )
-	    throw TError(nodePath().c_str(),_("AD DSC driver task is not started!"));
-    }
+    //> Create interrupt AD DSC task
+    if( ad_int_mode ) SYS->taskCreate( nodePath('.',true)+".int", 0, AD_DSCTask, this, &ad_dsc_st );
 }
 
 void TMdContr::stop_( )
 {
-    //- Close AI DAQ task -
-    if( ad_dsc_st )
-    {
-	endrun_req_ad_dsc = true;
-	pthread_kill( ad_dsc_pthr, SIGALRM );
-	if( TSYS::eventWait(ad_dsc_st,false,nodePath()+"addsc_task_stop",5) )
-	    throw TError(nodePath().c_str(),_("AD DSC task is not stopped!"));
-	pthread_join( ad_dsc_pthr, NULL );
-    }
+    //> Close AI DAQ task
+    if( ad_dsc_st ) SYS->taskDestroy( nodePath('.',true)+".int", &ad_dsc_st, &endrun_req_ad_dsc );
 
     if( !dataEmul() )	dscFreeBoard(dscb);
 }
@@ -493,7 +476,7 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path.substr(0,11) == "/board/dio/" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    string port_n = TSYS::pathLev(a_path,2);
 	    int	cfg_b = cfg("DIO_CFG").getI();
@@ -503,7 +486,7 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
 	    else if( port_n == "c2" )	cfg_b&=0x08;
 	    opt->setText(cfg_b?"1":"0");
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    string port_n = TSYS::pathLev(a_path,2);
 	    int cfg_b = cfg("DIO_CFG").getI();
@@ -618,9 +601,9 @@ void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
     //> Send to active reserve station
     if( owner().redntUse( ) )
     {
-	if( val.getS() == pvl.getS() ) return;
+	if( val.getS(0,true) == pvl.getS() ) return;
 	XMLNode req("set");
-	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS(0,true));
 	SYS->daq().at().rdStRequest(owner().workId(),req);
 	return;
     }
