@@ -1,8 +1,8 @@
 
 //OpenSCADA system module DAQ.DiamondBoards file: diamond.h
 /***************************************************************************
- *   Copyright (C) 2005-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2005-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -124,7 +124,6 @@ class TMdContr: public TController
 	DSCB	dscb;
 	DSCADSETTINGS dscadsettings;
 
-	pthread_t ad_dsc_pthr;
 	bool	ad_dsc_st, endrun_req_ad_dsc;
 
 	Res	ai_res, ao_res, dio_res;
