@@ -38,7 +38,7 @@
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
 #define SUB_TYPE	"LIB"
-#define VERSION		"1.4.0"
+#define VERSION		"1.7.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow java-like based calculator and function's libraries engine. User can create and modify function and libraries."
 #define LICENSE		"GPL"
@@ -95,9 +95,9 @@ void TipContr::postEnable( int flag )
     //> Controller db structure
     fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"60","system") );
     fldAdd( new TFld("FUNC",_("Controller's function"),TFld::String,TFld::NoFlag,"40") );
-    fldAdd( new TFld("PERIOD",_("Calc period (ms)"),TFld::Integer,TFld::NoFlag,"7","1000","0;1000000") );
-    fldAdd( new TFld("PRIOR",_("Calc task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
-    fldAdd( new TFld("ITER",_("Iteration number in the calc period"),TFld::Integer,TFld::NoFlag,"2","1","0;99") );
+    fldAdd( new TFld("SCHEDULE",_("Calc schedule"),TFld::String,TFld::NoFlag,"100","1") );
+    fldAdd( new TFld("PRIOR",_("Calc task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
+    fldAdd( new TFld("ITER",_("Iteration number in single calc"),TFld::Integer,TFld::NoFlag,"2","1","0;99") );
 
     //> Controller value db structure
     val_el.fldAdd( new TFld("ID",_("IO ID"),TFld::String,TCfg::Key,"20") );
@@ -105,7 +105,7 @@ void TipContr::postEnable( int flag )
 
     //> Add parameter types
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
-    tpPrmAt(t_prm).fldAdd( new TFld("FLD",_("Data fields(Sep - ';')"),TFld::String,TCfg::NoVal,"50") );
+    tpPrmAt(t_prm).fldAdd( new TFld("FLD",_("Data fields"),TFld::String,TFld::FullText|TCfg::NoVal,"300") );
 
     //> Lib's db structure
     lb_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
@@ -170,25 +170,35 @@ TController *TipContr::ContrAttach( const string &name, const string &daq_db )
     return new Contr(name,daq_db,this);
 }
 
-void TipContr::compileFuncLangs( vector<string> &ls )
+bool TipContr::compileFuncLangs( vector<string> *ls )
 {
-    ls.clear();
-    ls.push_back("JavaScript");
+    if( ls )
+    {
+	ls->clear();
+	ls->push_back("JavaScript");
+    }
+    return true;
 }
 
 string TipContr::compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text, const string &usings )
 {
     if( lang != "JavaScript" )	throw TError(nodePath().c_str(),_("Compilation with the help of the program language %s is not supported."),lang.c_str());
-    if( !lbPresent("sys_compile") )	lbReg( new Lib("sys_compile","","") );
-    if( !lbAt("sys_compile").at().present(fnc_cfg.id()) )
-	lbAt("sys_compile").at().add(fnc_cfg.id().c_str(),"");
+    if( !lbPresent("sys_compile") ) lbReg( new Lib("sys_compile","","") );
+    if( !lbAt("sys_compile").at().present(fnc_cfg.id()) ) lbAt("sys_compile").at().add(fnc_cfg.id().c_str(),"");
 
     AutoHD<Func> func = lbAt("sys_compile").at().at(fnc_cfg.id());
-    ((TFunction&)func.at()).operator=(fnc_cfg);
+    
+    bool isStart = func.at().startStat();
+    try 
+    {
+	((TFunction&)func.at()).operator=(fnc_cfg);
+	if( func.at().startStat() && prog_text == func.at().prog() ) return func.at().nodePath();
+    }
+    catch(TError err) { if( isStart ) func.at().setStart(true); throw; }
     func.at().setProg(prog_text.c_str());
     try
     {
-	if(func.at().startStat()) func.at().setStart(false);
+	if( func.at().startStat() ) func.at().setStart(false);
 	func.at().setUsings(usings);
 	func.at().setStart(true);
     }
@@ -204,39 +214,9 @@ string TipContr::compileFunc( const string &lang, TFunction &fnc_cfg, const stri
     return func.at().nodePath();
 }
 
-string TipContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TipContr::load_( )
 {
     //> Load parameters from command line
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
-
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 
     //> Load parameters
 
@@ -313,15 +293,15 @@ void TipContr::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/br/lib_" || a_path == "/libs/lb" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    vector<string> lst;
 	    lbList(lst);
 	    for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",lst[i_a])->setText(lbAt(lst[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	lbReg(new Lib(TSYS::strEncode(opt->attr("id"),TSYS::ID).c_str(),opt->text().c_str(),"*.*"));
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	lbUnreg(opt->attr("id"),1);
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )	lbReg(new Lib(TSYS::strEncode(opt->attr("id"),TSYS::oscdID).c_str(),opt->text().c_str(),"*.*"));
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )	lbUnreg(opt->attr("id"),1);
     }
     else TTipDAQ::cntrCmdProc(opt);
 }
@@ -344,9 +324,8 @@ BFunc *TipContr::bFuncGet( const char *nm )
 //* Contr: Controller object                      *
 //*************************************************
 Contr::Contr( string name_c, const string &daq_db, ::TElem *cfgelem) :
-    ::TController(name_c, daq_db, cfgelem), TValFunc(name_c.c_str(),NULL,false), prc_st(false),
-    endrun_req(false),
-    mPer(cfg("PERIOD").getId()), mPrior(cfg("PRIOR").getId()),
+    ::TController(name_c, daq_db, cfgelem), TValFunc(name_c.c_str(),NULL,false), prc_st(false), endrun_req(false),
+    mSched(cfg("SCHEDULE").getSd()), mPrior(cfg("PRIOR").getId()),
     mIter(cfg("ITER").getId()), mFnc(cfg("FUNC").getSd())
 {
     cfg("PRM_BD").setS("JavaLikePrm_"+name_c);
@@ -379,19 +358,23 @@ string Contr::getStatus( )
 {
     string val = TController::getStatus( );
 
-    if( startStat( ) && !redntUse( ) ) val += TSYS::strMess(_("Calc time %.6g mks. "),calcTm());
+    if( startStat( ) && !redntUse( ) )
+    {
+	if( period() ) val += TSYS::strMess(_("Call by period %g s. "),(1e-9*period()));
+	else val += TSYS::strMess(_("Call by cron '%s'. "),cron().c_str());
+	val += TSYS::strMess(_("Calc time %.6g mks. "),calcTm());
+    }
 
     return val;
 }
 
-
 void Contr::enable_( )
 {
     if( !mod->lbPresent(TSYS::strSepParse(mFnc,0,'.')) )
-	throw TError(nodePath().c_str(),_("Functions library <%s> is not present. Please, create functions library!"),TSYS::strSepParse(mFnc,0,'.').c_str());
+	throw TError(nodePath().c_str(),_("Functions library '%s' is not present. Please, create functions library!"),TSYS::strSepParse(mFnc,0,'.').c_str());
     if( !mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().present(TSYS::strSepParse(mFnc,1,'.')) )
     {
-	mess_info(nodePath().c_str(),_("Create new function <%s>."),mFnc.c_str());
+	mess_info(nodePath().c_str(),_("Create new function '%s'."),mFnc.c_str());
 	mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().add(TSYS::strSepParse(mFnc,1,'.').c_str());
     }
     setFunc( &mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().at(TSYS::strSepParse(mFnc,1,'.')).at() );
@@ -415,11 +398,11 @@ void Contr::load_( )
     loadFunc( );
 }
 
-void Contr::loadFunc( )
+void Contr::loadFunc( bool onlyVl )
 {
     if( func() != NULL )
     {
-	((Func *)func())->load();
+	if( !onlyVl ) ((Func *)func())->load();
 
 	//> Creating special IO
 	if( func()->ioId("f_frq") < 0 ) func()->ioIns( new IO("f_frq",_("Function calculate frequency (Hz)"),IO::Real,Func::SysAttr,"1000",false),0);
@@ -440,6 +423,13 @@ void Contr::loadFunc( )
     }
 }
 
+void Contr::postIOCfgChange( )
+{
+    TValFunc::postIOCfgChange( );
+
+    loadFunc( true );
+}
+
 void Contr::save_( )
 {
     TController::save_();
@@ -475,36 +465,17 @@ void Contr::start_( )
 {
     ((Func *)func())->setStart( true );
 
-    //- Start the request data task -
-    if( !prc_st )
-    {
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	if( mPrior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority=mPrior;
-	pthread_attr_setschedparam(&pthr_attr,&prior);
+    //> Schedule process
+    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(long long)(1e9*atof(mSched.c_str()))) : 0;
 
-	pthread_create(&procPthr,&pthr_attr,Contr::Task,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
-    }
+    //> Start the request data task
+    if( !prc_st ) SYS->taskCreate( nodePath('.',true), mPrior, Contr::Task, this, &prc_st );
 }
 
 void Contr::stop_( )
 {
-    //- Stop the request and calc data task -
-    if( prc_st )
-    {
-	endrun_req = true;
-	pthread_kill( procPthr, SIGALRM );
-	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
-	pthread_join( procPthr, NULL );
-    }
+    //> Stop the request and calc data task
+    if( prc_st ) SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
 }
 
 void *Contr::Task( void *icntr )
@@ -526,7 +497,7 @@ void *Contr::Task( void *icntr )
 	if( !cntr.redntUse( ) )
 	{
 	    //> Setting special IO
-	    int ioI = cntr.ioId("f_frq");	if( ioI >= 0 ) cntr.setR(ioI,(float)cntr.iterate()*1000/(float)cntr.period());
+	    int ioI = cntr.ioId("f_frq");	if( ioI >= 0 ) cntr.setR(ioI,cntr.period()?(float)cntr.iterate()*1e9/(float)cntr.period():0);
 	    ioI = cntr.ioId("f_start");		if( ioI >= 0 ) cntr.setB(ioI,is_start);
 	    ioI = cntr.ioId("f_stop");		if( ioI >= 0 ) cntr.setB(ioI,is_stop);
 
@@ -540,7 +511,7 @@ void *Contr::Task( void *icntr )
 	}
 
 	if( is_stop ) break;
-	TSYS::taskSleep((long long)cntr.period()*1000000);
+	TSYS::taskSleep(cntr.period(),cntr.period()?0:TSYS::cron(cntr.cron()));
 	if( cntr.endrun_req ) is_stop = true;
 	is_start = false;
 	cntr.modif();
@@ -551,16 +522,15 @@ void *Contr::Task( void *icntr )
     return NULL;
 }
 
-void Contr::redntDataUpdate( bool firstArchiveSync )
+void Contr::redntDataUpdate( )
 {
-    TController::redntDataUpdate(firstArchiveSync);
+    TController::redntDataUpdate( );
 
     //> Request for template's attributes values
     XMLNode req("get"); req.setAttr("path",nodePath(0,true)+"/%2fserv%2ffncAttr");
 
     //> Send request to first active station for this controller
-    try{ owner().owner().rdStRequest(workId(),req); }
-    catch(TError err) { return; }
+    if( owner().owner().rdStRequest(workId(),req).empty() ) return;
 
     //> Redirect respond to local controller
     req.setName("set")->setAttr("path","/%2fserv%2ffncAttr");
@@ -582,10 +552,10 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	if( a_path == "/serv/fncAttr" )
 	{
 	    if( !startStat( ) || !func( ) ) throw TError(nodePath().c_str(),_("No started or no present function."));
-	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )
 		for( int i_a = 0; i_a < ioSize(); i_a++ )
 		    opt->childAdd("a")->setAttr("id",func()->io(i_a)->id())->setText(getS(i_a));
-	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )
 		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
 		{
 		    int io_id = -1;
@@ -601,22 +571,32 @@ void Contr::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TController::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/cntr/cfg/FUNC",cfg("FUNC").fld().descr(),0660,"root","root",3,"tp","str","dest","sel_ed","select","/cntr/flst");
+	ctrMkNode("fld",opt,-1,"/cntr/cfg/FUNC",cfg("FUNC").fld().descr(),0664,"root","DAQ",3,"tp","str","dest","sel_ed","select","/cntr/flst");
+	ctrMkNode("fld",opt,-1,"/cntr/cfg/SCHEDULE",cfg("SCHEDULE").fld().descr(),0664,"root","DAQ",4,"tp","str","dest","sel_ed",
+	    "sel_list",_("1;1e-3;* * * * *;10 * * * *;10-20 2 */2 * *"),
+	    "help",_("Schedule is writed in seconds periodic form or in standard Cron form.\n"
+		     "Seconds form is one real number (1.5, 1e-3).\n"
+		     "Cron it is standard form '* * * * *'. Where:\n"
+		     "  - minutes (0-59);\n"
+		     "  - hours (0-23);\n"
+		     "  - days (1-31);\n"
+		     "  - month (1-12);\n"
+		     "  - week day (0[sunday]-6)."));
 	if( enableStat() && ctrMkNode("area",opt,-1,"/fnc",_("Calcing")) )
 	{
-	    if(ctrMkNode("table",opt,-1,"/fnc/io",_("Data"),0664,"root","root",2,"s_com","add,del,ins,move","rows","15"))
+	    if(ctrMkNode("table",opt,-1,"/fnc/io",_("Data"),0664,"root","DAQ",2,"s_com","add,del,ins,move","rows","15"))
 	    {
-		ctrMkNode("list",opt,-1,"/fnc/io/0",_("Id"),0664,"root","root",1,"tp","str");
-		ctrMkNode("list",opt,-1,"/fnc/io/1",_("Name"),0664,"root","root",1,"tp","str");
-		ctrMkNode("list",opt,-1,"/fnc/io/2",_("Type"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
+		ctrMkNode("list",opt,-1,"/fnc/io/0",_("Id"),0664,"root","DAQ",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/fnc/io/1",_("Name"),0664,"root","DAQ",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/fnc/io/2",_("Type"),0664,"root","DAQ",5,"tp","dec","idm","1","dest","select",
 		    "sel_id",(TSYS::int2str(IO::Real)+";"+TSYS::int2str(IO::Integer)+";"+TSYS::int2str(IO::Boolean)+";"+TSYS::int2str(IO::String)).c_str(),
 		    "sel_list",_("Real;Integer;Boolean;String"));
-		ctrMkNode("list",opt,-1,"/fnc/io/3",_("Mode"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
+		ctrMkNode("list",opt,-1,"/fnc/io/3",_("Mode"),0664,"root","DAQ",5,"tp","dec","idm","1","dest","select",
 		    "sel_id",(TSYS::int2str(IO::Default)+";"+TSYS::int2str(IO::Output)+";"+TSYS::int2str(IO::Return)).c_str(),
 		    "sel_list",_("Input;Output;Return"));
-		ctrMkNode("list",opt,-1,"/fnc/io/4",_("Value"),0664,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/fnc/io/4",_("Value"),0664,"root","DAQ",1,"tp","str");
 	    }
-	    ctrMkNode("fld",opt,-1,"/fnc/prog",_("Programm"),0664,"root","root",3,"tp","str","cols","90","rows","10");
+	    ctrMkNode("fld",opt,-1,"/fnc/prog",_("Programm"),0664,"root","DAQ",2,"tp","str","rows","10");
 	}
 	return;
     }
@@ -647,7 +627,7 @@ void Contr::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/fnc/io" && enableStat() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","DAQ",SEC_RD) )
 	{
 	    XMLNode *n_id	= ctrMkNode("list",opt,-1,"/fnc/io/0","",0664);
 	    XMLNode *n_nm	= ctrMkNode("list",opt,-1,"/fnc/io/1","",0664);
@@ -664,11 +644,11 @@ void Contr::cntrCmdProc( XMLNode *opt )
 		if(n_val)	n_val->childAdd("el")->setText(getS(id));
 	    }
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","DAQ",SEC_WR) )
 	{ ((Func *)func())->ioAdd( new IO("new","New IO",IO::Real,IO::Default) ); modif(); }
-	if( ctrChkNode(opt,"ins",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"ins",0664,"root","DAQ",SEC_WR) )
 	{ ((Func *)func())->ioIns( new IO("new","New IO",IO::Real,IO::Default), atoi(opt->attr("row").c_str()) ); modif(); }
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"del",0664,"root","DAQ",SEC_WR) )
 	{
 	    int row = atoi(opt->attr("row").c_str());
 	    if( func()->io(row)->flg()&Func::SysAttr )
@@ -676,9 +656,9 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	    ((Func *)func())->ioDel( row ); 
 	    modif();
 	}
-	if( ctrChkNode(opt,"move",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"move",0664,"root","DAQ",SEC_WR) )
 	{ ((Func *)func())->ioMove( atoi(opt->attr("row").c_str()), atoi(opt->attr("to").c_str()) ); modif(); }
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","DAQ",SEC_WR) )
 	{
 	    int row = atoi(opt->attr("row").c_str());
 	    int col = atoi(opt->attr("col").c_str());
@@ -700,8 +680,8 @@ void Contr::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/fnc/prog" && enableStat() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(((Func *)func())->prog());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"get",0664,"root","DAQ",SEC_RD) )	opt->setText(((Func *)func())->prog());
+	if( ctrChkNode(opt,"set",0664,"root","DAQ",SEC_WR) )
 	{
 	    ((Func *)func())->setProg(opt->text().c_str());
 	    ((Func *)func())->progCompile();
@@ -735,41 +715,66 @@ void Prm::enable()
 {
     if( enableStat() )  return;
 
-    //- Init elements -
-    string dfld;
-    for( int fld_off = 0; (dfld=TSYS::strSepParse(cfg("FLD").getS(),0,';',&fld_off)).size(); )
-    {
-	unsigned	flg = TVal::DirWrite|TVal::DirRead;
-	TFld::Type	tp  = TFld::String;
-	int		io_id = ((Contr &)owner()).ioId(dfld);
-	if(io_id >= 0)
-	{
-	    //if( ((Contr &)owner()).ioMode(io_id) != IO::Input )
-	    //	flg |= Fld::NoWrite;
-	    switch( ((Contr &)owner()).ioType(io_id) )
-	    {
-		case IO::String:	tp = TFld::String;	break;
-		case IO::Integer:	tp = TFld::Integer;	break;
-		case IO::Real:		tp = TFld::Real;	break;
-		case IO::Boolean:	tp = TFld::Boolean;	break;
-	    }
-	    if( !v_el.fldPresent(dfld) ||
-		v_el.fldAt(v_el.fldId(dfld)).type() != tp ||
-		v_el.fldAt(v_el.fldId(dfld)).flg() != flg )
-	    {
-		if(v_el.fldPresent(dfld)) v_el.fldDel(v_el.fldId(dfld));
-		v_el.fldAdd( new TFld(dfld.c_str(),((Contr &)owner()).func()->io(io_id)->name().c_str(),tp,flg) );
-	    }
-	}
-    }
-
-    //- Check and delete no used fields -
+    //> Check and delete no used fields
     for(int i_fld = 0; i_fld < v_el.fldSize(); i_fld++)
     {
 	string fel;
-	for( int fld_off = 0; (fel = TSYS::strSepParse(cfg("FLD").getS(),0,';',&fld_off)).size(); )
-	    if( fel == v_el.fldAt(i_fld).name() ) break;
-	if( fel.empty() )	{ v_el.fldDel(i_fld); i_fld--; }
+	for( int io_off = 0; (fel=TSYS::strSepParse(cfg("FLD").getS(),0,'\n',&io_off)).size(); )
+	    if( TSYS::strSepParse(fel,0,':') == v_el.fldAt(i_fld).reserve() ) break;
+	if( fel.empty() )
+	{
+	    try{ v_el.fldDel(i_fld); i_fld--; }
+	    catch(TError err)
+	    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+	}
+    }
+
+    //> Init elements
+    vector<string> pls;
+    int io;
+    string mio, ionm, aid, anm, def;
+    for( int io_off = 0; (mio=TSYS::strSepParse(cfg("FLD").getS(),0,'\n',&io_off)).size(); )
+    {
+	ionm   = TSYS::strSepParse(mio,0,':');
+	aid    = TSYS::strSepParse(mio,1,':');
+	anm    = TSYS::strSepParse(mio,2,':');
+	if( aid.empty() ) aid = ionm;
+
+	int	io_id = ((Contr &)owner()).ioId(ionm);
+	if( io_id < 0 )	continue;
+
+	unsigned	flg = TVal::DirWrite|TVal::DirRead;
+	if( !(((Contr &)owner()).ioFlg(io_id) & (IO::Output|IO::Return)) ) flg |= TFld::NoWrite;
+	TFld::Type	tp  = TFld::String;
+	switch( ((Contr &)owner()).ioType(io_id) )
+	{
+	    case IO::String:	tp = TFld::String; def = EVAL_STR;			break;
+	    case IO::Integer:	tp = TFld::Integer; def = TSYS::int2str(EVAL_INT);	break;
+	    case IO::Real:	tp = TFld::Real; def = TSYS::real2str(EVAL_REAL);	break;
+	    case IO::Boolean:	tp = TFld::Boolean; def = TSYS::int2str(EVAL_BOOL);	break;
+	}
+	if( !v_el.fldPresent(aid) || v_el.fldAt(v_el.fldId(aid)).type() != tp || v_el.fldAt(v_el.fldId(aid)).flg() != flg )
+	{
+	    if( v_el.fldPresent(aid) ) v_el.fldDel(v_el.fldId(aid));
+	    v_el.fldAdd( new TFld(aid.c_str(),"",tp,flg,"",def.c_str()) );
+	}
+
+	int el_id = v_el.fldId(aid);
+	v_el.fldAt(el_id).setDescr( anm.empty() ? ((Contr &)owner()).func()->io(io_id)->name() : anm );
+	v_el.fldAt(el_id).setReserve( ionm );
+
+	pls.push_back(aid);
+    }
+
+    //> Check and delete no used attrs
+    for( int i_fld = 0; i_fld < v_el.fldSize(); i_fld++ )
+    {
+	int i_p;
+	for( i_p = 0; i_p < pls.size(); i_p++ )
+	    if( pls[i_p] == v_el.fldAt(i_fld).name() )  break;
+	if( i_p < pls.size() )  continue;
+	try{ v_el.fldDel(i_fld); i_fld--; }
+	catch( TError err ) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
     }
 
     TParamContr::enable();
@@ -791,9 +796,9 @@ void Prm::vlSet( TVal &val, const TVariant &pvl )
     //> Send to active reserve station
     if( owner().redntUse( ) )
     {
-	if( val.getS() == pvl.getS() ) return;
+	if( val.getS(0,true) == pvl.getS() ) return;
 	XMLNode req("set");
-	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS(0,true));
 	SYS->daq().at().rdStRequest(owner().workId(),req);
 	return;
     }
@@ -801,7 +806,7 @@ void Prm::vlSet( TVal &val, const TVariant &pvl )
     //> Direct write
     try
     {
-	int io_id = ((Contr &)owner()).ioId(val.name());
+	int io_id = ((Contr &)owner()).ioId(val.fld().reserve());
 	if( io_id < 0 ) disable();
 	else
 	{
@@ -836,7 +841,7 @@ void Prm::vlGet( TVal &val )
     if( owner().redntUse( ) ) return;
     try
     {
-	int io_id = ((Contr &)owner()).ioId(val.name());
+	int io_id = ((Contr &)owner()).ioId(val.fld().reserve());
 	if( io_id < 0 ) disable();
 	else
 	{
@@ -859,7 +864,26 @@ void Prm::vlArchMake( TVal &val )
 {
     if( val.arch().freeStat() ) return;
     val.arch().at().setSrcMode(TVArchive::ActiveAttr,val.arch().at().srcData());
-    val.arch().at().setPeriod(((long long)owner().period())*1000);
+    val.arch().at().setPeriod( owner().period() ? owner().period()/1e3 : 1000000 );
     val.arch().at().setHardGrid( true );
     val.arch().at().setHighResTm( true );
 }
+
+void Prm::cntrCmdProc( XMLNode *opt )
+{
+    //Get page info
+    if( opt->name() == "info" )
+    {
+	TParamContr::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/prm/cfg/FLD",cfg("FLD").fld().descr(),0664,"root","root",1,
+	    "help",_("Attributes configuration list. List must be written by lines in format: [<io>:<aid>:<anm>]\n"
+	    "Where:\n"
+	    "  io - calc function's IO;\n"
+	    "  aid - created attribute identifier;\n"
+	    "  anm - created attribute name.\n"
+	    "If 'aid' or 'anm' are not set they will be generated from selected function's IO."));
+	return;
+    }
+    //Process command to page
+    TParamContr::cntrCmdProc(opt);
+}
