@@ -40,9 +40,9 @@
 #define MOD_NAME	"Logic level"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"1.0.0"
+#define VERSION		"1.1.1"
 #define AUTORS		"Roman Savochenko"
-#define DESCRIPTION	"Allow logic level paramers."
+#define DESCRIPTION	"Allow logic level parameters."
 #define LICENSE		"GPL"
 //*************************************************
 
@@ -88,56 +88,27 @@ TTpContr::~TTpContr()
 
 }
 
-string TTpContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TTpContr::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 }
 
 void TTpContr::postEnable( int flag )
 {
     TTipDAQ::postEnable( flag );
 
-    //- Controler's bd structure -
+    //> Controler's bd structure
     fldAdd( new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30","") );
     fldAdd( new TFld("PERIOD",_("Request data period (ms)"),TFld::Integer,TFld::NoFlag,"5","1000","0;10000") );
-    fldAdd( new TFld("PRIOR",_("Request task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
+    fldAdd( new TFld("PRIOR",_("Request task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
 
-    //- Parameter type bd structure -
+    //> Parameter type bd structure
     int t_prm = tpParmAdd("std","PRM_BD",_("Logical"));
     tpPrmAt(t_prm).fldAdd( new TFld("MODE",_("Mode"),TFld::Integer,TCfg::NoVal,"1","0") );
     tpPrmAt(t_prm).fldAdd( new TFld("PRM","",TFld::String,TCfg::NoVal,"100","") );
 
-    //- Logical level parameter IO BD structure -
+    //> Logical level parameter IO BD structure
     el_prm_io.fldAdd( new TFld("PRM_ID",_("Parameter ID"),TFld::String,TCfg::Key,"20") );
     el_prm_io.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
     el_prm_io.fldAdd( new TFld("VALUE",_("Value"),TFld::String,TCfg::TransltText,"200") );
@@ -170,7 +141,7 @@ void TMdContr::postDisable(int flag)
     {
 	if( flag )
 	{
-	    //- Delete parameter's io table -
+	    //> Delete parameter's io table
 	    string tbl = DB()+"."+cfg("PRM_BD").getS()+"_io";
 	    SYS->db().at().open(tbl);
 	    SYS->db().at().close(tbl,true);
@@ -201,35 +172,13 @@ void TMdContr::start_( )
 	    prmEn(list_p[i_prm],true);
 
     //> Start the request data task
-    if( !prc_st )
-    {
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	if( m_prior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority=m_prior;
-	pthread_attr_setschedparam(&pthr_attr,&prior);
-
-	pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
-    }
+    if( !prc_st ) SYS->taskCreate( nodePath('.',true), m_prior, TMdContr::Task, this, &prc_st );
 }
 
 void TMdContr::stop_( )
 {
     //> Stop the request and calc data task
-    if( prc_st )
-    {
-	endrun_req = true;
-	pthread_kill( procPthr, SIGALRM );
-	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
-	pthread_join( procPthr, NULL );
-    }
+    if( prc_st ) SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
 
     //> Clear process parameters list
     p_hd.clear();
@@ -263,7 +212,7 @@ void *TMdContr::Task( void *icntr )
     bool is_start = true;
     bool is_stop  = false;
 
-    while(true)
+    while( true )
     {
 	//> Update controller's data
 	if( !cntr.redntUse( ) )
@@ -271,7 +220,7 @@ void *TMdContr::Task( void *icntr )
 	    long long t_cnt = TSYS::curTime();
 	    cntr.en_res.resRequestR( );
 	    for( unsigned i_p=0; i_p < cntr.p_hd.size(); i_p++ )
-		try{ cntr.p_hd[i_p].at().calc(is_start,is_stop); }
+		try { cntr.p_hd[i_p].at().calc(is_start,is_stop); }
 		catch(TError err)
 		{ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 	    cntr.en_res.resRelease( );
@@ -279,9 +228,9 @@ void *TMdContr::Task( void *icntr )
 	}
 
 	if( is_stop ) break;
-	TSYS::taskSleep((long long)cntr.period()*1000000);
+	TSYS::taskSleep( (long long)cntr.period()*1000000 );
 	if( cntr.endrun_req ) is_stop = true;
-	is_start = false;
+	if( !cntr.redntUse() ) is_start = false;
     }
 
     cntr.prc_st = false;
@@ -289,9 +238,9 @@ void *TMdContr::Task( void *icntr )
     return NULL;
 }
 
-void TMdContr::redntDataUpdate( bool firstArchiveSync )
+void TMdContr::redntDataUpdate( )
 {
-    TController::redntDataUpdate(firstArchiveSync);
+    TController::redntDataUpdate( );
 
     vector<string> pls; list(pls);
 
@@ -304,8 +253,7 @@ void TMdContr::redntDataUpdate( bool firstArchiveSync )
     }
 
     //> Send request to first active station for this controller
-    try{ owner().owner().rdStRequest(workId(),req); }
-    catch(TError err) { return; }
+    if( owner().owner().rdStRequest(workId(),req).empty() ) return;
 
     //> Redirect respond to local parameters
     req.setAttr("path","/");
@@ -320,9 +268,9 @@ void TMdContr::redntDataUpdate( bool firstArchiveSync )
 //*************************************************
 //* TMdPrm                                        *
 //*************************************************
-TMdPrm::TMdPrm( string name, TTipParam *tp_prm ) : 
+TMdPrm::TMdPrm( string name, TTipParam *tp_prm ) :
     TParamContr(name,tp_prm), p_el("w_attr"), prm_refl(NULL), m_wmode(TMdPrm::Free), chk_lnk_need(false),
-    id_freq(-1), id_start(-1), id_stop(-1), m_mode(cfg("MODE").getId()), m_prm(cfg("PRM").getSd())
+    id_freq(-1), id_start(-1), id_stop(-1), id_sh(-1), id_nm(-1), id_dscr(-1), m_mode(cfg("MODE").getId()), m_prm(cfg("PRM").getSd())
 {
 
 }
@@ -332,6 +280,24 @@ TMdPrm::~TMdPrm( )
     nodeDelAll();
 }
 
+TCntrNode &TMdPrm::operator=( TCntrNode &node )
+{
+    TParamContr::operator=( node );
+
+    TMdPrm *src_n = dynamic_cast<TMdPrm*>(&node);
+    if( !src_n || !src_n->enableStat() || !enableStat() || src_n->mode()!=TMdPrm::Template || mode()!=TMdPrm::Template )
+	return *this;
+
+    //> IO values copy
+    for( int i_io = 0; i_io < src_n->tmpl->val.func()->ioSize(); i_io++ )
+	if( src_n->tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink )
+	    lnk(lnkId(i_io)).prm_attr = src_n->lnk(src_n->lnkId(i_io)).prm_attr;
+	else tmpl->val.setS(i_io,src_n->tmpl->val.getS(i_io));
+
+    return *this;
+}
+
+
 void TMdPrm::postEnable( int flag )
 {
     TParamContr::postEnable( flag );
@@ -367,15 +333,18 @@ void TMdPrm::enable()
     {
 	mode((TMdPrm::Mode)m_mode,m_prm);
 	loadIO();
-	if(mode() == TMdPrm::Template)
+	//> Init system attributes identifiers
+	if( mode() == TMdPrm::Template )
 	{
-	    //> Init system attributes identifiers
 	    id_freq	= tmpl->val.func()->ioId("f_frq");
 	    id_start	= tmpl->val.func()->ioId("f_start");
 	    id_stop	= tmpl->val.func()->ioId("f_stop");
 	    id_err	= tmpl->val.func()->ioId("f_err");
+	    id_sh	= tmpl->val.func()->ioId("SHIFR");
+	    id_nm	= tmpl->val.func()->ioId("NAME");
+	    id_dscr	= tmpl->val.func()->ioId("DESCR");
 	}
-	if(owner().startStat())	owner().prmEn(id(),true);
+	if( owner().startStat() ) owner().prmEn(id(),true);
     }
     catch(...){ disable(); throw; }
 }
@@ -512,7 +481,7 @@ void TMdPrm::mode( TMdPrm::Mode md, const string &prm )
     else if( md == TMdPrm::Template )
     {
 	bool to_make = false;
-	if( !tmpl )	tmpl = new STmpl;
+	if( !tmpl ) tmpl = new STmpl;
 	try
 	{
 	    if( !tmpl->val.func() )
@@ -545,7 +514,7 @@ void TMdPrm::mode( TMdPrm::Mode md, const string &prm )
 		    if( tmpl->val.func()->io(i_io)->flg()&TPrmTempl::AttrRead )	flg|=TFld::NoWrite;
 		    p_el.fldAdd( new TFld(tmpl->val.func()->io(i_io)->id().c_str(),tmpl->val.func()->io(i_io)->name().c_str(),tp,flg) );
 		}
-		if( to_make && (tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink) )	tmpl->val.setS(i_io,"0");
+		if( to_make && (tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink) )	tmpl->val.setS(i_io,EVAL_STR);
 	    }
 	    //>> Init links
 	    initTmplLnks();
@@ -561,20 +530,29 @@ void TMdPrm::mode( TMdPrm::Mode md, const string &prm )
     m_wprm = prm;
 }
 
-void TMdPrm::initTmplLnks()
+void TMdPrm::initTmplLnks( bool checkNoLink )
 {
     if( mode() != TMdPrm::Template )    return;
-    //- Init links -
+    //> Init links
     chk_lnk_need = false;
+    int off;
+    string nmod, ncntr, nprm, nattr;
+
     for( int i_l = 0; i_l < lnkSize(); i_l++ )
     {
+	if( checkNoLink && !lnk(i_l).aprm.freeStat() ) continue;
 	try
 	{
 	    lnk(i_l).aprm.free();
-	    lnk(i_l).aprm = SYS->daq().at().at(TSYS::strSepParse(lnk(i_l).prm_attr,0,'.')).at().
-					    at(TSYS::strSepParse(lnk(i_l).prm_attr,1,'.')).at().
-					    at(TSYS::strSepParse(lnk(i_l).prm_attr,2,'.')).at().
-					    vlAt(TSYS::strSepParse(lnk(i_l).prm_attr,3,'.'));
+
+	    off = 0;
+	    nmod = TSYS::strParse(lnk(i_l).prm_attr,0,".",&off);
+	    ncntr = TSYS::strParse(lnk(i_l).prm_attr,0,".",&off);
+	    nprm = TSYS::strParse(lnk(i_l).prm_attr,0,".",&off);
+	    nattr = TSYS::strParse(lnk(i_l).prm_attr,0,".",&off);
+	    if( nmod.empty() || ncntr.empty() || nprm.empty() || nattr.empty() ) continue;
+
+	    lnk(i_l).aprm = SYS->daq().at().at(nmod).at().at(ncntr).at().at(nprm).at().vlAt(nattr);
 	    tmpl->val.setS(lnk(i_l).io_id,lnk(i_l).aprm.at().getS());
 	}catch(TError err){ chk_lnk_need = true; }
     }
@@ -586,10 +564,8 @@ void TMdPrm::vlGet( TVal &val )
     {
 	if( val.name() == "err" )
 	{
-	    if(!enableStat())
-		val.setS(_("1:Parameter is disabled."),0,true);
-	    else if(!owner().startStat())
-		val.setS(_("2:Controller is stoped."),0,true);
+	    if( !enableStat() ) val.setS(_("1:Parameter is disabled."),0,true);
+	    else if( !owner().startStat( ) ) val.setS(_("2:Controller is stoped."),0,true);
 	}
 	return;
     }
@@ -653,9 +629,9 @@ void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
     //> Send to active reserve station
     if( owner().redntUse( ) )
     {
-	if( val.getS() == pvl.getS() ) return;
+	if( val.getS(0,true) == pvl.getS() ) return;
 	XMLNode req("set");
-	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS(0,true));
 	SYS->daq().at().rdStRequest(owner().workId(),req);
 	return;
     }
@@ -684,6 +660,7 @@ void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
 	{
 	    int id_lnk = lnkId(val.name());
 	    if( id_lnk >= 0 && lnk(id_lnk).aprm.freeStat() ) id_lnk=-1;
+	    ResAlloc cres(calcRes,true);
 	    switch(val.fld().type())
 	    {
 		case TFld::String:
@@ -758,17 +735,23 @@ void TMdPrm::calc( bool first, bool last )
     try
     {
 	ResAlloc res(moderes,false);
-	if(chk_lnk_need) initTmplLnks();
+	ResAlloc cres(calcRes,true);
+	if( chk_lnk_need ) initTmplLnks(true);
 
-	//- Set fixed system attributes -
-	if(id_freq>=0) 	tmpl->val.setR(id_freq,1000./owner().period());
-	if(id_start>=0)	tmpl->val.setB(id_start,first);
-	if(id_stop>=0)	tmpl->val.setB(id_stop,last);
+	tmpl->val.setMdfChk(true);
 
-	//- Get input links -
+	//> Set fixed system attributes
+	if( id_freq >= 0 )	tmpl->val.setR(id_freq,1000./owner().period());
+	if( id_start >= 0 )	tmpl->val.setB(id_start,first);
+	if( id_stop >= 0 )	tmpl->val.setB(id_stop,last);
+	if( id_sh >= 0 )	tmpl->val.setS(id_sh,id());
+	if( id_nm >= 0 )	tmpl->val.setS(id_nm,name());
+	if( id_dscr >= 0 )	tmpl->val.setS(id_dscr,descr());
+
+	//> Get input links
 	for( int i_l = 0; i_l < lnkSize(); i_l++ )
 	    if( lnk(i_l).aprm.freeStat() )	tmpl->val.setS(lnk(i_l).io_id,EVAL_STR);
-	    else switch(tmpl->val.ioType(lnk(i_l).io_id))
+	    else switch( tmpl->val.ioType(lnk(i_l).io_id) )
 	    {
 		case IO::String:
 		    tmpl->val.setS(lnk(i_l).io_id,lnk(i_l).aprm.at().getS());
@@ -784,30 +767,46 @@ void TMdPrm::calc( bool first, bool last )
 		    break;
 	    }
 
-	//- Calc template -
+	//> Calc template
 	tmpl->val.calc();
 	modif();
 
-	//- Put output links -
+	//> Put output links
 	for( int i_l = 0; i_l < lnkSize(); i_l++ )
-	    if( !lnk(i_l).aprm.freeStat() &&
+	    if( !lnk(i_l).aprm.freeStat() && tmpl->val.ioMdf(lnk(i_l).io_id) &&
 		    tmpl->val.ioFlg(lnk(i_l).io_id)&(IO::Output|IO::Return) &&
 		    !(lnk(i_l).aprm.at().fld().flg()&TFld::NoWrite) )
-		switch(tmpl->val.ioType(lnk(i_l).io_id))
+		switch( tmpl->val.ioType(lnk(i_l).io_id) )
 		{
 		    case IO::String:
-			lnk(i_l).aprm.at().setS(tmpl->val.getS(lnk(i_l).io_id));
+		    {
+			string vl = tmpl->val.getS(lnk(i_l).io_id);
+			if( vl != EVAL_STR )	lnk(i_l).aprm.at().setS(vl);
 			break;
+		    }
 		    case IO::Integer:
-			lnk(i_l).aprm.at().setI(tmpl->val.getI(lnk(i_l).io_id));
+		    {
+			int vl = tmpl->val.getI(lnk(i_l).io_id);
+			if( vl != EVAL_INT )	lnk(i_l).aprm.at().setI(vl);
 			break;
+		    }
 		    case IO::Real:
-			lnk(i_l).aprm.at().setR(tmpl->val.getR(lnk(i_l).io_id));
+		    {
+			double vl = tmpl->val.getR(lnk(i_l).io_id);
+			if( vl != EVAL_REAL )	lnk(i_l).aprm.at().setR(vl);
 			break;
+		    }
 		    case IO::Boolean:
-			lnk(i_l).aprm.at().setB(tmpl->val.getB(lnk(i_l).io_id));
+		    {
+			char vl = tmpl->val.getB(lnk(i_l).io_id);
+			if( vl != EVAL_BOOL )	lnk(i_l).aprm.at().setB(vl);
 			break;
+		    }
 		}
+
+	//> Put fixed system attributes
+	if( id_nm >= 0 )	setName(tmpl->val.getS(id_nm));
+	if( id_dscr >= 0 )	setDescr(tmpl->val.getS(id_dscr));
     }catch(TError err)
     {
 	mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
@@ -825,10 +824,10 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	{
 	    ResAlloc res(moderes,false);
 	    if( mode( ) != TMdPrm::Template || !tmpl->val.func( ) ) throw TError(nodePath().c_str(),_("No template parameter or error."));
-	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )
 		for( int i_a = 0; i_a < tmpl->val.ioSize(); i_a++ )
 		    opt->childAdd("ta")->setAttr("id",tmpl->val.func()->io(i_a)->id())->setText(tmpl->val.getS(i_a));
-	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )
 		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
 		{
 		    int io_id = -1;
@@ -841,7 +840,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     }
 
     vector<string> list;
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TParamContr::cntrCmdProc(opt);
@@ -857,13 +856,13 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 		{
 		    if( !(tmpl->val.func()->io(i_io)->flg()&(TPrmTempl::CfgLink|TPrmTempl::CfgPublConst)) )
 			continue;
-		    //-- Check select param --
+		    //>> Check select param
 		    bool is_lnk = tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink;
 		    if( is_lnk && tmpl->val.func()->io(i_io)->def().size() && 
 			!atoi(TBDS::genDBGet(mod->nodePath()+"onlAttr","0",opt->attr("user")).c_str()) )
 		    {
 			string nprm = TSYS::strSepParse(tmpl->val.func()->io(i_io)->def(),0,'|');
-			//-- Check already to present parameters --
+			//>> Check already to present parameters
 			bool f_ok = false;
 			for( int i_l = 0; i_l < list.size(); i_l++ )
 			    if( list[i_l] == nprm ) { f_ok = true; break; }
@@ -891,14 +890,14 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	}
     }
 
-    //- Process command to page -
-    if( a_path == "/prm/cfg/MODE" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )
+    //> Process command to page
+    if( a_path == "/prm/cfg/MODE" && ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )
 	try
 	{
 	    m_mode = atoi(opt->text().c_str());
 	    mode( (TMdPrm::Mode)m_mode, m_prm = "" );
 	} catch(...) { disable(); throw; }
-    else if( a_path == "/prm/cfg/PRM" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )
+    else if( a_path == "/prm/cfg/PRM" && ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )
 	try
 	{
 	    m_prm = opt->text();
@@ -943,12 +942,12 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/cfg/attr_only" && mode() == TMdPrm::Template )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TBDS::genDBGet(mod->nodePath()+"onlAttr","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	TBDS::genDBSet(mod->nodePath()+"onlAttr",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(TBDS::genDBGet(mod->nodePath()+"onlAttr","0",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	TBDS::genDBSet(mod->nodePath()+"onlAttr",opt->text(),opt->attr("user"));
     }
     else if( a_path.substr(0,12) == "/cfg/prm/pr_" && mode() == TMdPrm::Template )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    string lnk_val = lnk(lnkId(atoi(a_path.substr(12).c_str()))).prm_attr;
 	    int c_lvl = 0;
@@ -956,7 +955,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	    if( c_lvl==4 ) opt->setText(lnk_val.substr(0,lnk_val.rfind(".")));
 	    else opt->setText(lnk_val);	
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    bool noonly_no_set = true;
 	    string no_set;
@@ -1040,7 +1039,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path.substr(0,12) == "/cfg/prm/el_" && mode() == TMdPrm::Template )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    int i_io = atoi(a_path.substr(12).c_str());
 	    if( tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink )
@@ -1048,7 +1047,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	    else if( tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgPublConst )
 		opt->setText(tmpl->val.getS(i_io));
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    int i_io = atoi(a_path.substr(12).c_str());
 	    if( tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink )
