@@ -54,6 +54,8 @@ class TMdPrm : public TParamContr
 	TMdPrm( string name, TTipParam *tp_prm );
 	~TMdPrm( );
 
+	TCntrNode &operator=( TCntrNode &node );
+
 	Mode mode( )	{ return m_wmode; }
 	void mode( Mode md, const string &prm = "" );
 
@@ -108,9 +110,9 @@ class TMdPrm : public TParamContr
 	int lnkId( const string &id );
 	SLnk &lnk( int num );
 
-	void loadIO();
-	void saveIO();
-	void initTmplLnks();
+	void loadIO( );
+	void saveIO( );
+	void initTmplLnks( bool checkNoLink = false );
 
 	//Attributes
 	string	&m_prm, m_wprm;
@@ -120,8 +122,8 @@ class TMdPrm : public TParamContr
 	TElem	p_el;				//Work atribute elements
 
 	bool	chk_lnk_need;			//Check lnk need flag
-	Res	moderes;			//Resource
-	int	id_freq, id_start, id_stop, id_err;	//Fixed system attributes identifiers
+	Res	moderes, calcRes;		//Resource
+	int	id_freq, id_start, id_stop, id_err, id_sh, id_nm, id_dscr;	//Fixed system attributes identifiers
 };
 
 //*************************************************
@@ -140,7 +142,7 @@ class TMdContr: public TController
 
 	AutoHD<TMdPrm> at( const string &nm )	{ return TController::at(nm); }
 
-	void redntDataUpdate( bool firstArchiveSync = false );
+	void redntDataUpdate( );
 
     protected:
 	//Methods
@@ -164,8 +166,6 @@ class TMdContr: public TController
 		endrun_req;			// Request to stop of the Process task
 	vector< AutoHD<TMdPrm> >  p_hd;
 
-	pthread_t procPthr;			// Process task thread
-
 	double	tm_calc;			// Template functions calc time
 };
 
@@ -191,7 +191,6 @@ class TTpContr: public TTipDAQ
     private:
 	//Methods
 	TController *ContrAttach( const string &name, const string &daq_db );
-	string optDescr( );
 
 	//Attributes
 	TElem	el_prm_io;
