@@ -28,6 +28,7 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <string.h>
+#include <stdint.h>
 
 #include <ttiparam.h>
 
@@ -59,60 +60,31 @@ TTpContr::~TTpContr()
 
 }
 
-string TTpContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	DAQ_TYPE,DAQ_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TTpContr::postEnable( int flag )
 {
     TTipDAQ::postEnable( flag );
 
     //> Controler's bd structure
     fldAdd( new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30","") );
-    fldAdd( new TFld("PERIOD",_("Gather data period (s)"),TFld::Real,TFld::NoFlag,"6.2","1","0.01;100") );
-    fldAdd( new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
+    fldAdd( new TFld("SCHEDULE",_("Acquisition schedule"),TFld::String,TFld::NoFlag,"100","1") );
+    fldAdd( new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
     fldAdd( new TFld("PROT",_("Modbus protocol"),TFld::String,TFld::Selected,"5","TCP","TCP;RTU;ASCII",_("TCP/IP;RTU;ASCII")) );
     fldAdd( new TFld("ADDR",_("Transport address"),TFld::String,TFld::NoFlag,"30","") );
     fldAdd( new TFld("NODE",_("Destination node"),TFld::Integer,TFld::NoFlag,"20","1","0;255") );
     fldAdd( new TFld("FRAG_MERGE",_("Data fragments merge"),TFld::Boolean,TFld::NoFlag,"1","0") );
     fldAdd( new TFld("TM_REQ",_("Connection timeout (ms)"),TFld::Integer,TFld::NoFlag,"5","0") );
     fldAdd( new TFld("TM_REST",_("Restore timeout (s)"),TFld::Integer,TFld::NoFlag,"3","30") );
-    fldAdd( new TFld("REQ_TRY",_("Request tries"),TFld::Integer,TFld::NoFlag,"1","3") );
+    fldAdd( new TFld("REQ_TRY",_("Request tries"),TFld::Integer,TFld::NoFlag,"1","3","1;10") );
 
     //> Parameter type bd structure
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
-    tpPrmAt(t_prm).fldAdd( new TFld("ATTR_LS",_("Attributes list"),TFld::String,TFld::FullText|TCfg::NoVal|TCfg::TransltText,"100","") );
+    tpPrmAt(t_prm).fldAdd( new TFld("ATTR_LS",_("Attributes list"),TFld::String,TFld::FullText|TCfg::NoVal|TCfg::TransltText,"1000","") );
 }
 
 void TTpContr::load_( )
 {
     //> Load parameters from command line
-    int next_opt;
-    const char *short_opt = "h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
 
-    optind = opterr = 0;
-    do
-    {
-	next_opt = getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch( next_opt )
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while( next_opt != -1 );
 }
 
 void TTpContr::save_()
@@ -131,7 +103,7 @@ TController *TTpContr::ContrAttach( const string &name, const string &daq_db )
 TMdContr::TMdContr( string name_c, const string &daq_db, TElem *cfgelem ) :
 	TController( name_c, daq_db, cfgelem ), prc_st(false), endrun_req(false), tmGath(0), tmDelay(0),
 	numRReg(0), numRRegIn(0), numRCoil(0), numRCoilIn(0), numWReg(0), numWCoil(0), numErrCon(0), numErrResp(0),
-	mPer(cfg("PERIOD").getRd()), mPrior(cfg("PRIOR").getId()), mPrt(cfg("PROT").getSd()),
+	mSched(cfg("SCHEDULE").getSd()), mPrior(cfg("PRIOR").getId()), mPrt(cfg("PROT").getSd()),
 	mAddr(cfg("ADDR").getSd()), mNode(cfg("NODE").getId()), mMerge(cfg("FRAG_MERGE").getBd()),
 	reqTm(cfg("TM_REQ").getId()), restTm(cfg("TM_REST").getId()), connTry(cfg("REQ_TRY").getId())
 {
@@ -149,9 +121,18 @@ string TMdContr::getStatus( )
 
     if( startStat( ) && !redntUse( ) )
     {
-	if( tmDelay ) val += TSYS::strMess(_("Connection error. Restoring in %.6g s."),tmDelay);
-	else val += TSYS::strMess(_("Gather data time %.6g ms. Read %g(%g) registers, %g(%g) coils. Write %g registers, %g coils. Errors of connection %g, of respond %g."),
+	if( tmDelay > -1 )
+	{
+	    val += TSYS::strMess(_("Connection error. Restoring in %.6g s."),tmDelay);
+	    val.replace(0,1,"10");
+	}
+	else
+	{
+	    if( period() ) val += TSYS::strMess(_("Call by period %g s. "),(1e-9*period()));
+	    else val += TSYS::strMess(_("Call by cron '%s'. "),cron().c_str());
+	    val += TSYS::strMess(_("Gather data time %.6g ms. Read %g(%g) registers, %g(%g) coils. Write %g registers, %g coils. Errors of connection %g, of respond %g."),
 				    tmGath,numRReg,numRRegIn,numRCoil,numRCoilIn,numWReg,numWCoil,numErrCon,numErrResp);
+	}
     }
 
     return val;
@@ -173,45 +154,30 @@ void TMdContr::disable_( )
 
 void TMdContr::start_( )
 {
-    if( !prc_st )
-    {
-	//> Establish connection
-	SYS->transport().at().at(TSYS::strSepParse(mAddr,0,'.')).at().outAt(TSYS::strSepParse(mAddr,1,'.')).at().start();
+    if( prc_st ) return;
 
-	//> Clear statistic
-	numRReg = numRRegIn = numRCoil = numRCoilIn = numWReg = numWCoil = numErrCon = numErrResp = tmDelay = 0;
+    //> Establish connection
+    AutoHD<TTransportOut> tr = SYS->transport().at().at(TSYS::strSepParse(mAddr,0,'.')).at().outAt(TSYS::strSepParse(mAddr,1,'.'));
+    try { tr.at().start(); }
+    catch( TError err ){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 
-	//> Start the gathering data task
-	pthread_attr_t pthr_attr;
-	pthread_attr_init( &pthr_attr );
-	struct sched_param prior;
-	if( mPrior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy( &pthr_attr, SCHED_RR );
-	else pthread_attr_setschedpolicy( &pthr_attr, SCHED_OTHER );
-	prior.__sched_priority = mPrior;
-	pthread_attr_setschedparam( &pthr_attr, &prior );
+    //> Schedule process
+    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(long long)(1e9*atof(mSched.c_str()))) : 0;
 
-	pthread_create( &procPthr, &pthr_attr, TMdContr::Task, this );
-	pthread_attr_destroy( &pthr_attr );
-	if( TSYS::eventWait( prc_st, true, nodePath()+"start", 5 ) )
-	    throw TError( nodePath().c_str(), _("Gathering task is not started!") );
-    }
+    //> Clear statistic
+    numRReg = numRRegIn = numRCoil = numRCoilIn = numWReg = numWCoil = numErrCon = numErrResp = tmDelay = 0;
+
+    //> Start the gathering data task
+    SYS->taskCreate( nodePath('.',true), mPrior, TMdContr::Task, this, &prc_st );
 }
 
 void TMdContr::stop_( )
 {
-    if( prc_st )
-    {
-	//> Stop the request and calc data task
-	endrun_req = true;
-	pthread_kill( procPthr, SIGALRM );
-	if( TSYS::eventWait( prc_st, false, nodePath()+"stop", 5 ) )
-	    throw TError( nodePath().c_str(), _("Gathering task is not stopped!") );
-	pthread_join( procPthr, NULL );
+    //> Stop the request and calc data task
+    SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
 
-	//> Clear statistic
-	numRReg = numRRegIn = numRCoil = numRCoilIn = numWReg = numWCoil = numErrCon = numErrResp = 0;
-    }
+    //> Clear statistic
+    numRReg = numRRegIn = numRCoil = numRCoilIn = numWReg = numWCoil = numErrCon = numErrResp = 0;
 }
 
 bool TMdContr::cfgChange( TCfg &icfg )
@@ -220,9 +186,8 @@ bool TMdContr::cfgChange( TCfg &icfg )
 
     if( icfg.fld().name() == "PROT" )
     {
-	if( icfg.getS() == "TCP" )	cfg("REQ_TRY").setView(false);
-	else				cfg("REQ_TRY").setView(true);
-	if(startStat())	stop();
+	cfg("REQ_TRY").setView(icfg.getS()!="TCP");
+	if( startStat() ) stop();
     }
     else if( icfg.fld().name() == "FRAG_MERGE" && enableStat( ) ) disable( );
 
@@ -307,37 +272,39 @@ void TMdContr::regVal( int reg, const string &dt )
     }
 }
 
-int TMdContr::getValR( int addr, string &err, bool in )
+int TMdContr::getValR( int addr, ResString &err, bool in )
 {
     int rez = EVAL_INT;
+    ResAlloc res( req_res, false );
     vector< SDataRec >	&workCnt = in ? acqBlksIn : acqBlks;
     for( int i_b = 0; i_b < workCnt.size(); i_b++ )
 	if( (addr*2) >= workCnt[i_b].off && (addr*2+2) <= (workCnt[i_b].off+workCnt[i_b].val.size()) )
 	{
-	    err = workCnt[i_b].err;
-	    if( err.empty() )
+	    err.setVal( workCnt[i_b].err.getVal() );
+	    if( err.getVal().empty() )
 		rez = (unsigned short)(workCnt[i_b].val[addr*2-workCnt[i_b].off]<<8)|(unsigned char)workCnt[i_b].val[addr*2-workCnt[i_b].off+1];
 	    break;
 	}
     return rez;
 }
 
-char TMdContr::getValC( int addr, string &err, bool in )
+char TMdContr::getValC( int addr, ResString &err, bool in )
 {
     char rez = EVAL_BOOL;
+    ResAlloc res( req_res, false );
     vector< SDataRec >	&workCnt = in ? acqBlksCoilIn : acqBlksCoil;
     for( int i_b = 0; i_b < workCnt.size(); i_b++ )
 	if( addr >= workCnt[i_b].off && (addr+1) <= (workCnt[i_b].off+workCnt[i_b].val.size()) )
 	{
-	    err = workCnt[i_b].err;
-	    if( err.empty() )
+	    err.setVal( workCnt[i_b].err.getVal() );
+	    if( err.getVal().empty() )
 		rez = workCnt[i_b].val[addr-workCnt[i_b].off];
 	    break;
 	}
     return rez;
 }
 
-void TMdContr::setValR( int val, int addr, string &err )
+void TMdContr::setValR( int val, int addr, ResString &err )
 {
     //> Encode request PDU (Protocol Data Units)
     string pdu;
@@ -347,9 +314,10 @@ void TMdContr::setValR( int val, int addr, string &err )
     pdu += (char)(val>>8);	//Data MSB
     pdu += (char)val;		//Data LSB
     //> Request to remote server
-    err = modBusReq( pdu );
-    if( err.empty() ) numWReg++;
+    err.setVal( modBusReq(pdu) );
+    if( err.getVal().empty() ) numWReg++;
     //> Set to acquisition block
+    ResAlloc res( req_res, false );
     for( int i_b = 0; i_b < acqBlks.size(); i_b++ )
 	if( (addr*2) >= acqBlks[i_b].off && (addr*2+2) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	{
@@ -359,7 +327,7 @@ void TMdContr::setValR( int val, int addr, string &err )
 	}
 }
 
-void TMdContr::setValC( char val, int addr, string &err )
+void TMdContr::setValC( char val, int addr, ResString &err )
 {
     //> Encode request PDU (Protocol Data Units)
     string pdu;
@@ -369,9 +337,10 @@ void TMdContr::setValC( char val, int addr, string &err )
     pdu += (char)val?0xFF:0x00;	//Data MSB
     pdu += (char)0x00;		//Data LSB
     //> Request to remote server
-    err = modBusReq( pdu );
-    if( err.empty() ) numWCoil++;
+    err.setVal( modBusReq(pdu) );
+    if( err.getVal().empty() ) numWCoil++;
     //> Set to acquisition block
+    ResAlloc res( req_res, false );
     for( int i_b = 0; i_b < acqBlksCoil.size(); i_b++ )
 	if( addr >= acqBlksCoil[i_b].off && (addr+1) <= (acqBlksCoil[i_b].off+acqBlksCoil[i_b].val.size()) )
 	{
@@ -422,122 +391,139 @@ void *TMdContr::Task( void *icntr )
 	{
 	    long long t_cnt = TSYS::curTime();
 
-	    if( cntr.tmDelay > 0 )	cntr.tmDelay-=cntr.period();
-	    else
-	    {
+	    if( cntr.tmDelay > 0 )	{ sleep(1); cntr.tmDelay = vmax(0,cntr.tmDelay-1); continue; }
 #if OSC_DEBUG >= 3
-		mess_debug(cntr.nodePath().c_str(),_("Fetch coils' and registers' blocks."));
+	    mess_debug(cntr.nodePath().c_str(),_("Fetch coils' and registers' blocks."));
 #endif
-		ResAlloc res( cntr.req_res, false );
+	    ResAlloc res( cntr.req_res, false );
 
-		//> Get coils
-		for( int i_b = 0; i_b < cntr.acqBlksCoil.size(); i_b++ )
+	    //> Get coils
+	    for( int i_b = 0; i_b < cntr.acqBlksCoil.size(); i_b++ )
+	    {
+		if( cntr.endrun_req ) break;
+		if( cntr.redntUse( ) ) { cntr.acqBlksCoil[i_b].err.setVal(_("4:Server failure.")); continue; }
+		//>> Encode request PDU (Protocol Data Units)
+		pdu = (char)0x1;					//Function, read multiple coils
+		pdu += (char)(cntr.acqBlksCoil[i_b].off>>8);		//Address MSB
+		pdu += (char)cntr.acqBlksCoil[i_b].off;			//Address LSB
+		pdu += (char)(cntr.acqBlksCoil[i_b].val.size()>>8);	//Number of coils MSB
+		pdu += (char)cntr.acqBlksCoil[i_b].val.size();		//Number of coils LSB
+		//>> Request to remote server
+		cntr.acqBlksCoil[i_b].err.setVal( cntr.modBusReq(pdu) );
+		if( cntr.acqBlksCoil[i_b].err.getVal().empty() )
 		{
-		    if( cntr.endrun_req ) break;
-		    if( cntr.redntUse( ) ) { cntr.acqBlksCoil[i_b].err = _("4:Server failure."); continue; }
-		    //>> Encode request PDU (Protocol Data Units)
-		    pdu = (char)0x1;					//Function, read multiple coils
-		    pdu += (char)(cntr.acqBlksCoil[i_b].off>>8);	//Address MSB
-		    pdu += (char)cntr.acqBlksCoil[i_b].off;		//Address LSB
-		    pdu += (char)(cntr.acqBlksCoil[i_b].val.size()>>8);	//Number of coils MSB
-		    pdu += (char)cntr.acqBlksCoil[i_b].val.size();	//Number of coils LSB
-		    //>> Request to remote server
-		    cntr.acqBlksCoil[i_b].err = cntr.modBusReq( pdu );
-		    if( cntr.acqBlksCoil[i_b].err.empty() )
+		    if( (cntr.acqBlksCoil[i_b].val.size()/8+((cntr.acqBlksCoil[i_b].val.size()%8)?1:0)) != (pdu.size()-2) )
+			cntr.acqBlksCoil[i_b].err.setVal(_("15:Response PDU size error."));
+		    else
 		    {
 			for( int i_c = 0; i_c < cntr.acqBlksCoil[i_b].val.size(); i_c++ )
 			    cntr.acqBlksCoil[i_b].val[i_c] = (bool)((pdu[2+i_c/8]>>(i_c%8))&0x01);
 			cntr.numRCoil += cntr.acqBlksCoil[i_b].val.size();
 		    }
-		    else if( atoi(cntr.acqBlksCoil[i_b].err.c_str()) == 14 )
-		    {
-			cntr.setCntrDelay(cntr.acqBlksCoil[i_b].err);
-			break;
-		    }
 		}
-		if( cntr.tmDelay > 0 )	continue;
-		//> Get input's coils
-		for( int i_b = 0; i_b < cntr.acqBlksCoilIn.size(); i_b++ )
+		else if( atoi(cntr.acqBlksCoil[i_b].err.getVal().c_str()) == 14 )
 		{
-		    if( cntr.endrun_req ) break;
-		    if( cntr.redntUse( ) ) { cntr.acqBlksCoilIn[i_b].err = _("4:Server failure."); continue; }
-		    //>> Encode request PDU (Protocol Data Units)
-		    pdu = (char)0x2;					//Function, read multiple input's coils
-		    pdu += (char)(cntr.acqBlksCoilIn[i_b].off>>8);	//Address MSB
-		    pdu += (char)cntr.acqBlksCoilIn[i_b].off;		//Address LSB
-		    pdu += (char)(cntr.acqBlksCoilIn[i_b].val.size()>>8);	//Number of coils MSB
-		    pdu += (char)cntr.acqBlksCoilIn[i_b].val.size();	//Number of coils LSB
-		    //>> Request to remote server
-		    cntr.acqBlksCoilIn[i_b].err = cntr.modBusReq( pdu );
-		    if( cntr.acqBlksCoilIn[i_b].err.empty() )
+		    cntr.setCntrDelay(cntr.acqBlksCoil[i_b].err.getVal());
+		    break;
+		}
+	    }
+	    if( cntr.tmDelay > 0 )	continue;
+	    //> Get input's coils
+	    for( int i_b = 0; i_b < cntr.acqBlksCoilIn.size(); i_b++ )
+	    {
+		if( cntr.endrun_req ) break;
+		if( cntr.redntUse( ) ) { cntr.acqBlksCoilIn[i_b].err.setVal(_("4:Server failure.")); continue; }
+		//>> Encode request PDU (Protocol Data Units)
+		pdu = (char)0x2;					//Function, read multiple input's coils
+		pdu += (char)(cntr.acqBlksCoilIn[i_b].off>>8);	//Address MSB
+		pdu += (char)cntr.acqBlksCoilIn[i_b].off;		//Address LSB
+		pdu += (char)(cntr.acqBlksCoilIn[i_b].val.size()>>8);	//Number of coils MSB
+		pdu += (char)cntr.acqBlksCoilIn[i_b].val.size();	//Number of coils LSB
+		//>> Request to remote server
+		cntr.acqBlksCoilIn[i_b].err.setVal( cntr.modBusReq(pdu) );
+		if( cntr.acqBlksCoilIn[i_b].err.getVal().empty() )
+		{
+		    if( (cntr.acqBlksCoilIn[i_b].val.size()/8+((cntr.acqBlksCoilIn[i_b].val.size()%8)?1:0)) != (pdu.size()-2) )
+			cntr.acqBlksCoilIn[i_b].err.setVal(_("15:Response PDU size error."));
+		    else
 		    {
 			for( int i_c = 0; i_c < cntr.acqBlksCoilIn[i_b].val.size(); i_c++ )
 			    cntr.acqBlksCoilIn[i_b].val[i_c] = (bool)((pdu[2+i_c/8]>>(i_c%8))&0x01);
 			cntr.numRCoilIn += cntr.acqBlksCoilIn[i_b].val.size();
 		    }
-		    else if( atoi(cntr.acqBlksCoilIn[i_b].err.c_str()) == 14 )
-		    {
-			cntr.setCntrDelay(cntr.acqBlksCoilIn[i_b].err);
-			break;
-		    }
 		}
-		if( cntr.tmDelay > 0 )	continue;
-		//> Get registers
-		for( int i_b = 0; i_b < cntr.acqBlks.size(); i_b++ )
+		else if( atoi(cntr.acqBlksCoilIn[i_b].err.getVal().c_str()) == 14 )
 		{
-		    if( cntr.endrun_req ) break;
-		    if( cntr.redntUse( ) ) { cntr.acqBlks[i_b].err = _("4:Server failure."); continue; }
-		    //>> Encode request PDU (Protocol Data Units)
-		    pdu = (char)0x3;				//Function, read multiple registers
-		    pdu += (char)((cntr.acqBlks[i_b].off/2)>>8);	//Address MSB
-		    pdu += (char)(cntr.acqBlks[i_b].off/2);		//Address LSB
-		    pdu += (char)((cntr.acqBlks[i_b].val.size()/2)>>8);	//Number of registers MSB
-		    pdu += (char)(cntr.acqBlks[i_b].val.size()/2);	//Number of registers LSB
-		    //>> Request to remote server
-		    cntr.acqBlks[i_b].err = cntr.modBusReq( pdu );
-		    if( cntr.acqBlks[i_b].err.empty() )
+		    cntr.setCntrDelay(cntr.acqBlksCoilIn[i_b].err.getVal());
+		    break;
+		}
+	    }
+	    if( cntr.tmDelay > 0 )	continue;
+	    //> Get registers
+	    for( int i_b = 0; i_b < cntr.acqBlks.size(); i_b++ )
+	    {
+		if( cntr.endrun_req ) break;
+		if( cntr.redntUse( ) ) { cntr.acqBlks[i_b].err.setVal(_("4:Server failure.")); continue; }
+		//>> Encode request PDU (Protocol Data Units)
+		pdu = (char)0x3;				//Function, read multiple registers
+		pdu += (char)((cntr.acqBlks[i_b].off/2)>>8);	//Address MSB
+		pdu += (char)(cntr.acqBlks[i_b].off/2);		//Address LSB
+		pdu += (char)((cntr.acqBlks[i_b].val.size()/2)>>8);	//Number of registers MSB
+		pdu += (char)(cntr.acqBlks[i_b].val.size()/2);	//Number of registers LSB
+		//>> Request to remote server
+		cntr.acqBlks[i_b].err.setVal( cntr.modBusReq(pdu) );
+		if( cntr.acqBlks[i_b].err.getVal().empty() )
+		{
+		    if( cntr.acqBlks[i_b].val.size() != (pdu.size()-2) ) cntr.acqBlks[i_b].err.setVal(_("15:Response PDU size error."));
+		    else
 		    {
-			cntr.acqBlks[i_b].val.replace(0,cntr.acqBlks[i_b].val.size(),pdu.substr(2).c_str(),cntr.acqBlks[i_b].val.size());
+			cntr.acqBlks[i_b].val.replace(0,cntr.acqBlks[i_b].val.size(),pdu.data()+2,cntr.acqBlks[i_b].val.size());
 			cntr.numRReg += cntr.acqBlks[i_b].val.size()/2;
 		    }
-		    else if( atoi(cntr.acqBlks[i_b].err.c_str()) == 14 )
-		    {
-			cntr.setCntrDelay(cntr.acqBlks[i_b].err);
-			break;
-		    }
 		}
-		if( cntr.tmDelay > 0 )	continue;
-		//> Get input registers
-		for( int i_b = 0; i_b < cntr.acqBlksIn.size(); i_b++ )
+		else if( atoi(cntr.acqBlks[i_b].err.getVal().c_str()) == 14 )
 		{
-		    if( cntr.endrun_req ) break;
-		    if( cntr.redntUse( ) ) { cntr.acqBlksIn[i_b].err = _("4:Server failure."); continue; }
-		    //>> Encode request PDU (Protocol Data Units)
-		    pdu = (char)0x4;				//Function, read multiple input registers
-		    pdu += (char)((cntr.acqBlksIn[i_b].off/2)>>8);	//Address MSB
-		    pdu += (char)(cntr.acqBlksIn[i_b].off/2);		//Address LSB
-		    pdu += (char)((cntr.acqBlksIn[i_b].val.size()/2)>>8);	//Number of registers MSB
-		    pdu += (char)(cntr.acqBlksIn[i_b].val.size()/2);	//Number of registers LSB
-		    //>> Request to remote server
-		    cntr.acqBlksIn[i_b].err = cntr.modBusReq( pdu );
-		    if( cntr.acqBlksIn[i_b].err.empty() )
+		    cntr.setCntrDelay(cntr.acqBlks[i_b].err.getVal());
+		    break;
+		}
+	    }
+	    if( cntr.tmDelay > 0 )	continue;
+	    //> Get input registers
+	    for( int i_b = 0; i_b < cntr.acqBlksIn.size(); i_b++ )
+	    {
+		if( cntr.endrun_req ) break;
+		if( cntr.redntUse( ) ) { cntr.acqBlksIn[i_b].err.setVal(_("4:Server failure.")); continue; }
+		//>> Encode request PDU (Protocol Data Units)
+		pdu = (char)0x4;				//Function, read multiple input registers
+		pdu += (char)((cntr.acqBlksIn[i_b].off/2)>>8);	//Address MSB
+		pdu += (char)(cntr.acqBlksIn[i_b].off/2);		//Address LSB
+		pdu += (char)((cntr.acqBlksIn[i_b].val.size()/2)>>8);	//Number of registers MSB
+		pdu += (char)(cntr.acqBlksIn[i_b].val.size()/2);	//Number of registers LSB
+		//>> Request to remote server
+		cntr.acqBlksIn[i_b].err.setVal( cntr.modBusReq(pdu) );
+		if( cntr.acqBlksIn[i_b].err.getVal().empty() )
+		{
+		    if( cntr.acqBlksIn[i_b].val.size() != (pdu.size()-2) ) cntr.acqBlksIn[i_b].err.setVal(_("15:Response PDU size error."));
+		    else
 		    {
-			cntr.acqBlksIn[i_b].val.replace(0,cntr.acqBlksIn[i_b].val.size(),pdu.substr(2).c_str(),cntr.acqBlksIn[i_b].val.size());
+			cntr.acqBlksIn[i_b].val.replace(0,cntr.acqBlksIn[i_b].val.size(),pdu.data()+2,cntr.acqBlksIn[i_b].val.size());
 			cntr.numRRegIn += cntr.acqBlksIn[i_b].val.size()/2;
 		    }
-		    else if( atoi(cntr.acqBlksIn[i_b].err.c_str()) == 14 )
-		    {
-			cntr.setCntrDelay(cntr.acqBlksIn[i_b].err);
-			break;
-		    }
 		}
-		res.release();
+		else if( atoi(cntr.acqBlksIn[i_b].err.getVal().c_str()) == 14 )
+		{
+		    cntr.setCntrDelay(cntr.acqBlksIn[i_b].err.getVal());
+		    break;
+		}
 	    }
+	    res.release();
+
+	    if( cntr.tmDelay <= 0 ) cntr.tmDelay--;
 
 	    //> Calc acquisition process time
 	    cntr.tmGath = 1e-3*(TSYS::curTime()-t_cnt);
 
-	    TSYS::taskSleep((long long)(1e9*cntr.period()));
+	    TSYS::taskSleep(cntr.period(),cntr.period()?0:TSYS::cron(cntr.cron()));
 	}
     }
     catch( TError err )	{ mess_err( err.cat.c_str(), err.mess.c_str() ); }
@@ -551,20 +537,32 @@ void TMdContr::setCntrDelay( const string &err )
 {
     tmDelay = restTm;
     ResAlloc res( req_res, false );
-    for( int i_b = 0; i_b < acqBlksCoil.size(); i_b++ )	acqBlksCoil[i_b].err = err;
-    for( int i_b = 0; i_b < acqBlks.size(); i_b++ )	acqBlks[i_b].err = err;
+    for( int i_b = 0; i_b < acqBlksCoil.size(); i_b++ )	acqBlksCoil[i_b].err.setVal( err );
+    for( int i_b = 0; i_b < acqBlksCoilIn.size(); i_b++ )	acqBlksCoilIn[i_b].err.setVal( err );
+    for( int i_b = 0; i_b < acqBlks.size(); i_b++ )	acqBlks[i_b].err.setVal( err );
+    for( int i_b = 0; i_b < acqBlksIn.size(); i_b++ )	acqBlksIn[i_b].err.setVal( err );
 }
 
 void TMdContr::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TController::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/cntr/cfg/ADDR",cfg("ADDR").fld().descr(),0664,"root","root",3,"tp","str","dest","select","select","/cntr/cfg/trLst");
+	ctrMkNode("fld",opt,-1,"/cntr/cfg/SCHEDULE",cfg("SCHEDULE").fld().descr(),0664,"root","DAQ",4,"tp","str","dest","sel_ed",
+	    "sel_list",_("1;1e-3;* * * * *;10 * * * *;10-20 2 */2 * *"),
+	    "help",_("Schedule is writed in seconds periodic form or in standard Cron form.\n"
+		     "Seconds form is one real number (1.5, 1e-3).\n"
+		     "Cron it is standard form '* * * * *'. Where:\n"
+		     "  - minutes (0-59);\n"
+		     "  - hours (0-23);\n"
+		     "  - days (1-31);\n"
+		     "  - month (1-12);\n"
+		     "  - week day (0[sunday]-6)."));
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/cntr/cfg/trLst" && ctrChkNode(opt) )
     {
@@ -604,41 +602,58 @@ void TMdPrm::enable()
 
     TParamContr::enable();
 
-    //- Parse ModBus attributes and convert to strong list -
+    //> Parse ModBus attributes and convert to string list
     vector<string> als;
-    int  ai;
-    string sel, atp, aid, anm, awr;
+    string ai, sel, atp, atp_m, atp_sub, aid, anm, awr;
     for( int ioff = 0; (sel=TSYS::strSepParse(m_attrLs,0,'\n',&ioff)).size(); )
     {
 	atp = TSYS::strSepParse(sel,0,':');
 	if( atp.empty() ) atp = "R";
-	ai  = strtol(TSYS::strSepParse(sel,1,':').c_str(),NULL,0);
+	atp_m = TSYS::strSepParse(atp,0,'_');
+	atp_sub = TSYS::strSepParse(atp,1,'_');
+	ai  = TSYS::strSepParse(sel,1,':');
 	awr = TSYS::strSepParse(sel,2,':');
 	aid = TSYS::strSepParse(sel,3,':');
-	if( aid.empty() ) aid = TSYS::int2str(ai);
+	if( aid.empty() ) aid = ai;
 	anm = TSYS::strSepParse(sel,4,':');
-	if( anm.empty() ) anm = TSYS::int2str(ai);
+	if( anm.empty() ) anm = ai;
 
 	if( vlPresent(aid) && !p_el.fldPresent(aid) )	continue;
-	TFld::Type	tp  = (atp[0]=='C') ? TFld::Boolean : TFld::Integer;
+
+	TFld::Type tp = TFld::Integer;
+	if( atp[0]=='C' || (atp_sub.size() && atp_sub[0] == 'b') ) tp = TFld::Boolean;
+	else if( atp_sub == "f" ) tp = TFld::Real;
+
 	if( !p_el.fldPresent(aid) || p_el.fldAt(p_el.fldId(aid)).type() != tp )
 	{
 	    if( p_el.fldPresent(aid)) p_el.fldDel(p_el.fldId(aid));
-	    p_el.fldAdd( new TFld(aid.c_str(),"",tp,TFld::NoFlag,"",TSYS::int2str((atp[0]=='C')?EVAL_BOOL:EVAL_INT).c_str()) );
+	    p_el.fldAdd( new TFld(aid.c_str(),"",tp,TFld::NoFlag) );
 	}
 	int el_id = p_el.fldId(aid);
+
 	unsigned flg = (awr=="rw") ? TVal::DirWrite|TVal::DirRead :
 				     ((awr=="w") ? TVal::DirWrite : TFld::NoWrite|TVal::DirRead);
 	if( atp.size() >= 2 && atp[1] == 'I' )	flg = (flg & (~TVal::DirWrite)) | TFld::NoWrite;
 	p_el.fldAt(el_id).setFlg( flg );
 	p_el.fldAt(el_id).setDescr( anm );
-	p_el.fldAt(el_id).setReserve( atp+":"+TSYS::int2str(ai) );
-	if( flg&TVal::DirRead ) owner().regVal(ai,atp);
+
+	if( flg&TVal::DirRead )
+	{
+	    int reg = strtol(ai.c_str(),NULL,0);
+	    owner().regVal(reg,atp_m);
+	    if( atp[0] == 'R' && (atp_sub == "i4" || atp_sub == "f") )
+	    {
+		int reg2 = TSYS::strSepParse(ai,1,',').empty() ? (reg+1) : strtol(TSYS::strSepParse(ai,1,',').c_str(),NULL,0);
+		owner().regVal( reg2, atp_m );
+		ai = TSYS::int2str(reg)+","+TSYS::int2str(reg2);
+	    }
+	}
+	p_el.fldAt(el_id).setReserve( atp+":"+ai );
 
 	als.push_back(aid);
     }
 
-    //- Check for delete DAQ parameter's attributes -
+    //> Check for delete DAQ parameter's attributes
     for( int i_p = 0; i_p < p_el.fldSize(); i_p++ )
     {
 	int i_l;
@@ -657,7 +672,7 @@ void TMdPrm::disable()
 
     TParamContr::disable();
 
-    //- Set EVAL to parameter attributes -
+    //> Set EVAL to parameter attributes
     vector<string> ls;
     elem().fldList(ls);
     for(int i_el = 0; i_el < ls.size(); i_el++)
@@ -681,17 +696,39 @@ void TMdPrm::vlGet( TVal &val )
 
     int off = 0;
     string tp = TSYS::strSepParse(val.fld().reserve(),0,':',&off);
-    int aid = atoi(TSYS::strSepParse(val.fld().reserve(),0,':',&off).c_str());
+    string atp_sub = TSYS::strSepParse(tp,1,'_');
+    bool isInputs = (tp.size()>=2 && tp[1]=='I');
+    string aids = TSYS::strSepParse(val.fld().reserve(),0,':',&off);
+    int aid = strtol(aids.c_str(),NULL,0);
     if( !tp.empty() )
-	switch(val.fld().type())
+    {
+	if( tp[0] == 'C' ) val.setB(owner().getValC(aid,acq_err,isInputs),0,true);
+	if( tp[0] == 'R' )
 	{
-	    case TFld::Boolean:	val.setB(owner().getValC(aid,acq_err,tp=="CI"),0,true);	break;
-	    case TFld::Integer:	val.setI(owner().getValR(aid,acq_err,tp=="RI"),0,true);	break;
+	    int vl = owner().getValR(aid,acq_err,isInputs);
+	    if( !atp_sub.empty() && atp_sub[0] == 'b' ) val.setB((vl>>atoi(atp_sub.c_str()+1))&1,0,true);
+	    else if( !atp_sub.empty() && atp_sub == "f" )
+	    {
+		int vl2 = owner().getValR( strtol(TSYS::strSepParse(aids,1,',').c_str(),NULL,0), acq_err, isInputs );
+		if( vl == EVAL_INT || vl2 == EVAL_INT ) val.setR(EVAL_REAL,0,true);
+		union { uint32_t i; float f; } wl;
+		wl.i = ((vl2&0xffff)<<16) | (vl&0xffff);
+		val.setR(wl.f,0,true);
+	    }
+	    else if( !atp_sub.empty() && atp_sub == "i2" )	val.setI((int16_t)vl,0,true);
+	    else if( !atp_sub.empty() && atp_sub == "i4" )
+	    {
+		int vl2 = owner().getValR( strtol(TSYS::strSepParse(aids,1,',').c_str(),NULL,0), acq_err, isInputs );
+		if( vl == EVAL_INT || vl2 == EVAL_INT ) val.setI(EVAL_INT,0,true);
+		val.setI((int)(((vl2&0xffff)<<16)|(vl&0xffff)),0,true);
+	    }
+	    else val.setI(vl,0,true);
 	}
+    }
     else if( val.name() == "err" )
     {
-	if( acq_err.empty() )	val.setS("0",0,true);
-	else			val.setS(acq_err,0,true);
+	if( acq_err.getVal().empty() )	val.setS("0",0,true);
+	else				val.setS(acq_err.getVal(),0,true);
     }
 }
 
@@ -702,23 +739,49 @@ void TMdPrm::vlSet( TVal &valo, const TVariant &pvl )
     //> Send to active reserve station
     if( owner().redntUse( ) )
     {
-	if( valo.getS() == pvl.getS() ) return;
+	if( valo.getS(NULL,true) == pvl.getS() ) return;
 	XMLNode req("set");
-	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",valo.name())->setText(valo.getS());
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",valo.name())->setText(valo.getS(NULL,true));
 	SYS->daq().at().rdStRequest(owner().workId(),req);
 	return;
     }
 
+    string vl = valo.getS(NULL,true);
+    if( vl == EVAL_STR || vl == pvl.getS() ) return;
+
     //> Direct write
-    int aid = atoi(TSYS::strSepParse(valo.fld().reserve(),1,':').c_str());
-    switch(valo.fld().type())
+    int off = 0;
+    string tp = TSYS::strSepParse(valo.fld().reserve(),0,':',&off);
+    string atp_sub = TSYS::strSepParse(tp,1,'_');
+    string aids = TSYS::strSepParse(valo.fld().reserve(),0,':',&off);
+    int aid = strtol(aids.c_str(),NULL,0);
+
+    if( !tp.empty() )
     {
-	case TFld::Boolean:
-	    owner().setValC(valo.getB(NULL,true),aid,acq_err);
-	    break;
-	case TFld::Integer:
-	    owner().setValR(valo.getI(NULL,true),aid,acq_err);
-	    break;
+	if( tp[0] == 'C' )	owner().setValC(valo.getB(NULL,true),aid,acq_err);
+	if( tp[0] == 'R' )
+	{
+	    if( !atp_sub.empty() && atp_sub[0] == 'b' )
+	    {
+		int vl = owner().getValR(aid,acq_err);
+		if( vl != EVAL_INT )
+		    owner().setValR( valo.getB(NULL,true) ? (vl|(1<<atoi(atp_sub.c_str()+1))) : (vl & ~(1<<atoi(atp_sub.c_str()+1))), aid, acq_err);
+	    }
+	    else if( !atp_sub.empty() && atp_sub == "f" )
+	    {
+		union { uint32_t i; float f; } wl;
+		wl.f = valo.getR(NULL,true);
+		owner().setValR( wl.i&0xFFFF, aid, acq_err );
+		owner().setValR( (wl.i>>16)&0xFFFF, strtol(TSYS::strSepParse(aids,1,',').c_str(),NULL,0), acq_err );
+	    }
+	    else if( !atp_sub.empty() && atp_sub == "i4" )
+	    {
+		int vl = valo.getI(NULL,true);
+		owner().setValR( vl&0xFFFF, aid, acq_err );
+		owner().setValR( (vl>>16)&0xFFFF, strtol(TSYS::strSepParse(aids,1,',').c_str(),NULL,0), acq_err );
+	    }
+	    else owner().setValR(valo.getI(NULL,true),aid,acq_err);
+	}
     }
 }
 
@@ -726,30 +789,34 @@ void TMdPrm::vlArchMake( TVal &val )
 {
     if( val.arch().freeStat() ) return;
     val.arch().at().setSrcMode( TVArchive::ActiveAttr, val.arch().at().srcData() );
-    val.arch().at().setPeriod( (long long)(owner().period()*1000000) );
+    val.arch().at().setPeriod( owner().period() ? owner().period()/1e3 : 1000000 );
     val.arch().at().setHardGrid( true );
     val.arch().at().setHighResTm( true );
 }
 
 void TMdPrm::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TParamContr::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/prm/cfg/ATTR_LS",cfg("ATTR_LS").fld().descr(),0664,"root","root",1,
+	ctrMkNode("fld",opt,-1,"/prm/cfg/ATTR_LS",cfg("ATTR_LS").fld().descr(),RWRWR_,"root","DAQ",1,
 	    "help",_("Attributes configuration list. List must be written by lines in format: [dt:numb:rw:id:name]\n"
 		    "Where:\n"
-		    "  dt - Modbus data type (R-register,C-coil,RI-input register,CI-input coil);\n"
+		    "  dt - Modbus data type (R-register,C-coil,RI-input register,CI-input coil).\n"
+		    "       R and RI can expanded by suffixes: i2-Int16, i4-Int32, f-Float, b5-Bit5;\n"
 		    "  numb - ModBus device's data address (dec, hex or octal);\n"
 		    "  rw - read-write mode (r-read; w-write; rw-readwrite);\n"
 		    "  id - created attribute identifier;\n"
 		    "  name - created attribute name.\n"
 		    "Example:\n"
 		    "  'R:0x300:rw:var:Variable' - register access;\n"
-		    "  'C:100:r:var1:Variable 1' - coin access."));
+		    "  'C:100:r:var1:Variable 1' - coin access;\n"
+		    "  'R_f:200:r:float:Float' - get float from registers 200 and 201;\n"
+		    "  'R_i4:300,400:r:int32:Int32' - get int32 from registers 300 and 400;\n"
+		    "  'R_b10:25:r:rBit:Reg bit' - get bit 10 from register 25."));
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     TParamContr::cntrCmdProc(opt);
 }
