@@ -92,38 +92,9 @@ void TProt::nAdd( const string &iid, const string &db )
     chldAdd( mNode, new Node(iid,db,&nodeEl()) );
 }
 
-string TProt::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	PRT_TYPE,PRT_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TProt::load_( )
 {
     //> Load parameters from command line
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL, 0 }
-    };
-
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 
     //> Load DB
     //>> Search and create new nodes
@@ -183,7 +154,7 @@ TProtocolIn *TProt::in_open( const string &name )
     return new TProtIn(name);
 }
 
-ui8 TProt::CRCHi[] =
+uint8_t TProt::CRCHi[] =
 {
     0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
     0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
@@ -203,7 +174,7 @@ ui8 TProt::CRCHi[] =
     0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40
 };
 
-ui8 TProt::CRCLo[] =
+uint8_t TProt::CRCLo[] =
 {
     0x00, 0xC0, 0xC1, 0x01, 0xC3, 0x03, 0x02, 0xC2, 0xC6, 0x06, 0x07, 0xC7, 0x05, 0xC5, 0xC4, 0x04,
     0xCC, 0x0C, 0x0D, 0xCD, 0x0F, 0xCF, 0xCE, 0x0E, 0x0A, 0xCA, 0xCB, 0x0B, 0xC9, 0x09, 0x08, 0xC8,
@@ -223,32 +194,32 @@ ui8 TProt::CRCLo[] =
     0x44, 0x84, 0x85, 0x45, 0x87, 0x47, 0x46, 0x86, 0x82, 0x42, 0x43, 0x83, 0x41, 0x81, 0x80, 0x40
 };
 
-ui16 TProt::CRC16( const string &mbap )
+uint16_t TProt::CRC16( const string &mbap )
 {
-    ui8 hi = 0xFF;
-    ui8 lo = 0xFF;
-    ui16 index;
+    uint8_t hi = 0xFF;
+    uint8_t lo = 0xFF;
+    uint16_t index;
     for( int i_b = 0; i_b < mbap.size(); i_b++ )
     {
-	index = lo^(ui8)mbap[i_b];
+	index = lo^(uint8_t)mbap[i_b];
 	lo = hi^CRCHi[index];
 	hi = CRCLo[index];
     }
     return hi|(lo<<8);
 }
 
-ui8 TProt::LRC( const string &mbap )
+uint8_t TProt::LRC( const string &mbap )
 {
-    ui8 ch = 0;
+    uint8_t ch = 0;
     for( int i_b = 0; i_b < mbap.size(); i_b++ )
-	ch += (ui8)mbap[i_b];
+	ch += (uint8_t)mbap[i_b];
 
-    return ch;
+    return -ch;
 }
 
 string TProt::DataToASCII( const string &in )
 {
-    ui8 ch;
+    uint8_t ch;
     string rez;
 
     for( int i = 0; i < in.size(); i++ )
@@ -264,7 +235,7 @@ string TProt::DataToASCII( const string &in )
 
 string TProt::ASCIIToData( const string &in )
 {
-    ui8 ch1, ch2;
+    uint8_t ch1, ch2;
     string rez;
 
     for( int i=0; i < (in.size()&(~0x01)); i+=2 )
@@ -290,6 +261,8 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
     string mbap, err, rez;
     char buf[1000];
 
+    ResAlloc resN( tro.nodeRes(), true );
+
     string prt   = io.name();
     string sid   = io.attr("id");
     int    reqTm = atoi(io.attr("reqTm").c_str());
@@ -303,7 +276,8 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 	if( prt == "TCP" )		// Modbus/TCP protocol process
 	{
 	    //> Encode MBAP (Modbus Application Protocol)
-	    mbap  = (char)0x15;			//Transaction ID MSB
+	    mbap.reserve( pdu.size()+7 );
+	    mbap += (char)0x15;			//Transaction ID MSB
 	    mbap += (char)0x01;			//Transaction ID LSB
 	    mbap += (char)0x00;			//Protocol ID MSB
 	    mbap += (char)0x00;			//Protocol ID LSB
@@ -313,7 +287,7 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 	    mbap += pdu;
 
 	    //> Send request
-	    int resp_len = tro.messIO( mbap.data(), mbap.size(), buf, sizeof(buf), reqTm );
+	    int resp_len = tro.messIO( mbap.data(), mbap.size(), buf, sizeof(buf), reqTm, true );
 	    rez.assign(buf,resp_len);
 	    if( rez.size() < 7 )	err = _("13:Error server respond");
 	    else
@@ -323,7 +297,7 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 		//> Wait tail
 		while( rez.size() < (resp_sz+6) )
 		{
-		    resp_len = tro.messIO( NULL, 0, buf, sizeof(buf), reqTm );
+		    resp_len = tro.messIO( NULL, 0, buf, sizeof(buf), reqTm, true );
 		    rez.append( buf, resp_len );
 		}
 		pdu = rez.substr(7);
@@ -331,20 +305,27 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 	}
 	else if( prt == "RTU" )		// Modbus/RTU protocol process
 	{
-	    mbap = (ui8)node;			//Unit identifier
+	    mbap.reserve( pdu.size()+3 );
+	    mbap += (uint8_t)node;		//Unit identifier
 	    mbap += pdu;
-	    ui16 crc = CRC16( mbap );
-	    mbap += crc >> 8;
+	    uint16_t crc = CRC16( mbap );
+	    mbap += (crc>>8);
 	    mbap += crc;
 
 	    //> Send request
 	    for( int i_tr = 0; i_tr < reqTry; i_tr++ )
 	    {
-		int resp_len = tro.messIO( mbap.data(), mbap.size(), buf, sizeof(buf), reqTm );
-		rez.assign(buf,resp_len);
+		int resp_len = tro.messIO( mbap.data(), mbap.size(), buf, sizeof(buf), reqTm, true );
+		rez.assign( buf, resp_len );
+		//> Wait tail
+		while( true )
+		{
+		    try{ resp_len = tro.messIO( NULL, 0, buf, sizeof(buf), 0, true ); } catch(TError err){ break; }
+		    rez.append( buf, resp_len );
+		}
 
 		if( rez.size() < 2 )	{ err = _("13:Error respond: Too short."); continue; }
-		if( CRC16(rez.substr(0,rez.size()-2)) != (ui16)((rez[rez.size()-2]<<8)+(ui8)rez[rez.size()-1]) )
+		if( CRC16(rez.substr(0,rez.size()-2)) != (uint16_t)((rez[rez.size()-2]<<8)+(uint8_t)rez[rez.size()-1]) )
 		{ err = _("13:Error respond: CRC check error."); continue; }
 		pdu = rez.substr( 1, rez.size()-3 );
 		err = "";
@@ -353,7 +334,8 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 	}
 	else if( prt == "ASCII" )	// Modbus/ASCII protocol process
 	{
-	    mbap = (ui8)node;			//Unit identifier
+	    mbap.reserve( pdu.size()+2 );
+	    mbap += (uint8_t)node;		//Unit identifier
 	    mbap += pdu;
 	    mbap += LRC(mbap);
 	    mbap = ":"+DataToASCII(mbap)+"\r\n";
@@ -361,13 +343,19 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 	    //> Send request
 	    for( int i_tr = 0; i_tr < reqTry; i_tr++ )
 	    {
-		int resp_len = tro.messIO( mbap.data(), mbap.size(), buf, sizeof(buf), reqTm );
+		int resp_len = tro.messIO( mbap.data(), mbap.size(), buf, sizeof(buf), reqTm, true );
 		rez.assign(buf,resp_len);
+		//> Wait tail
+		while( rez.size() < 3 || rez.substr(rez.size()-2,2) != "\r\n" )
+		{
+		    try{ resp_len = tro.messIO( NULL, 0, buf, sizeof(buf), 0, true ); } catch(TError err){ break; }
+		    rez.append( buf, resp_len );
+		}
 
 		if( rez.size() < 3 || rez[0] != ':' || rez.substr(rez.size()-2,2) != "\r\n" )
 		{ err = _("13:Error respond: Error format."); continue; }
 		string rezEnc = ASCIIToData(rez.substr(1,rez.size()-3));
-		if( LRC(rezEnc.substr(0,rezEnc.size()-1)) != rezEnc[rezEnc.size()-1] )
+		if( LRC(rezEnc.substr(0,rezEnc.size()-1)) != (uint8_t)rezEnc[rezEnc.size()-1] )
 		{ err = _("13:Error respond: LRC check error."); continue; }
 		pdu = rezEnc.substr(1,rezEnc.size()-2);
 		err = "";
@@ -447,7 +435,6 @@ void TProt::cntrCmdProc( XMLNode *opt )
 	    if( prtLen() )
 		ctrMkNode("fld",opt,-1,"/rep/rep",_("Report"),0444,"root","DAQ",3,"tp","str","cols","90","rows","20");
 	}
-	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
 	return;
     }
 
@@ -455,24 +442,24 @@ void TProt::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/br/n_" || a_path == "/node/node" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    vector<string> lst;
 	    nList(lst);
 	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
 		opt->childAdd("el")->setAttr("id",lst[i_f])->setText(nAt(lst[i_f]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    nAdd(vid); nAt(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	chldDel(mNode,opt->attr("id"),-1,1);
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )	chldDel(mNode,opt->attr("id"),-1,1);
     }
     else if( a_path == "/rep/repLen" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","DAQ",SEQ_RD) )	opt->setText( TSYS::int2str(prtLen()) );
-	if( ctrChkNode(opt,"set",0664,"root","DAQ",SEQ_WR) )	setPrtLen( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","DAQ",SEC_RD) )	opt->setText( TSYS::int2str(prtLen()) );
+	if( ctrChkNode(opt,"set",0664,"root","DAQ",SEC_WR) )	setPrtLen( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/rep/rep" && ctrChkNode(opt) )
     {
@@ -480,7 +467,6 @@ void TProt::cntrCmdProc( XMLNode *opt )
 	for( int i_p = 0; i_p < mPrt.size(); i_p++ )
 	    opt->setText(opt->text()+mPrt[i_p]+"\n");
     }
-    else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )	opt->setText(optDescr());
     else TProtocol::cntrCmdProc(opt);
 }
 
@@ -514,13 +500,13 @@ retry:
     {
 	prt = "ASCII";
 	string req = modPrt->ASCIIToData(reqst.substr(1,reqst.size()-3));
-	if( modPrt->LRC(req.substr(0,req.size()-1)) != req[req.size()-1] ) return false;
+	if( modPrt->LRC(req.substr(0,req.size()-1)) != (uint8_t)req[req.size()-1] ) return false;
 	node = req[0];
 	pdu = req.substr( 1, req.size()-2 );
     }
     //>> RTU check
     else if( reqst.size() > 3 && reqst.size() <= 256 &&
-	modPrt->CRC16(reqst.substr(0,reqst.size()-2)) == (ui16)((reqst[reqst.size()-2]<<8)+(ui8)reqst[reqst.size()-1]) )
+	modPrt->CRC16(reqst.substr(0,reqst.size()-2)) == (uint16_t)((reqst[reqst.size()-2]<<8)+(uint8_t)reqst[reqst.size()-1]) )
     {
 	prt = "RTU";
 	node = reqst[0];
@@ -555,10 +541,12 @@ retry:
 	if( modPrt->nAt(nls[i_n]).at().req(srcTr(),prt,node,pdu) ) break;
     if( i_n >= nls.size() ) return false;
 
+    answer = "";
     if( prt == "TCP" )
     {
 	//> Encode MBAP (Modbus Application Protocol)
-	answer = reqst[0];			//Transaction ID MSB
+	answer.reserve(pdu.size()+7);
+	answer += reqst[0];			//Transaction ID MSB
 	answer += reqst[1];			//Transaction ID LSB
 	answer += reqst[2];			//Protocol ID MSB
 	answer += reqst[3];			//Protocol ID LSB
@@ -569,15 +557,17 @@ retry:
     }
     else if( prt == "RTU" )
     {
-	answer = (ui8)node;			//Unit identifier
+	answer.reserve(pdu.size()+3);
+	answer += (uint8_t)node;		//Unit identifier
 	answer += pdu;
-	ui16 crc = modPrt->CRC16( answer );
+	uint16_t crc = modPrt->CRC16( answer );
 	answer += crc>>8;
 	answer += crc;
     }
     else if( prt == "ASCII" )
     {
-	answer = (ui8)node;			//Unit identifier
+	answer.reserve(pdu.size()+2);
+	answer += (uint8_t)node;		//Unit identifier
 	answer += pdu;
 	answer += modPrt->LRC(answer);
 	answer = ":"+modPrt->DataToASCII(answer)+"\r\n";
@@ -740,7 +730,7 @@ void Node::load_( )
 	u_pos[pos] = sid;
 
 	int iid = ioId(sid);
-	
+
 	if( iid < 0 )
 	    iid = ioIns( new IO(sid.c_str(),cfg.cfg("NAME").getS().c_str(),(IO::Type)cfg.cfg("TYPE").getI(),cfg.cfg("FLAGS").getI(),"",false),pos );
 	else
@@ -752,13 +742,16 @@ void Node::load_( )
 	if( io(iid)->flg()&Node::IsLink ) io(iid)->setRez(cfg.cfg("VALUE").getS());
 	else io(iid)->setDef(cfg.cfg("VALUE").getS());
     }
+
+    //> Remove holes
+    for( int i_p = 0; i_p < u_pos.size(); i_p++ )
+	if( u_pos[i_p].empty() ) { u_pos.erase(u_pos.begin()+i_p); i_p--; }
+
     //> Position fixing
     for( int i_p = 0; i_p < u_pos.size(); i_p++ )
     {
-	if( u_pos[i_p].empty() ) continue;
 	int iid = ioId(u_pos[i_p]);
-	if( iid != i_p )
-	    try{ ioMove(iid,i_p); } catch(...){ }
+	if( iid != i_p ) try{ ioMove(iid,i_p); } catch(...){ }
     }
 }
 
@@ -854,26 +847,13 @@ void Node::setEnable( bool vl )
 	}
 
 	//>> Start task
-	pthread_attr_t pthrAttr;
-	pthread_attr_init( &pthrAttr );
-	pthread_attr_setschedpolicy( &pthrAttr, SCHED_OTHER );
-	pthread_create( &pthrTsk, &pthrAttr, Task, this );
-	pthread_attr_destroy( &pthrAttr );
-	if( TSYS::eventWait(prcSt,true,nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Not started!"));
+	SYS->taskCreate( nodePath('.',true), 0, Task, this, &prcSt );
     }
     //> Disable node
     if( !vl )
     {
 	//> Stop the calc data task
-	if( prcSt )
-	{
-	    endrunRun = true;
-	    pthread_kill( pthrTsk, SIGALRM );
-	    if( TSYS::eventWait(prcSt,false,nodePath()+"stop",5) )
-		throw TError(nodePath().c_str(),_("Data process task is not stopped!"));
-	    pthread_join( pthrTsk, NULL );
-	}
+	if( prcSt ) SYS->taskDestroy( nodePath('.',true), &prcSt, &endrunRun );
 
 	//> Data structure delete
 	if( data ) { delete data; data = NULL; }
@@ -1177,13 +1157,13 @@ void Node::cntrCmdProc( XMLNode *opt )
     if( a_path == "/nd/st/status" && ctrChkNode(opt) )	opt->setText(getStatus());
     else if( a_path == "/nd/st/en_st" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEQ_RD) )	opt->setText(enableStat()?"1":"0");
-	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEQ_WR) )	setEnable(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEC_RD) )	opt->setText(enableStat()?"1":"0");
+	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEC_WR) )	setEnable(atoi(opt->text().c_str()));
     }
     else if( a_path == "/nd/st/db" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEQ_RD) )	opt->setText(DB());
-	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEQ_WR) )	setDB(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEC_RD) )	opt->setText(DB());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEC_WR) )	setDB(opt->text());
     }
     else if( a_path == "/nd/cfg/ls_itr" && ctrChkNode(opt) )
     {
@@ -1203,7 +1183,7 @@ void Node::cntrCmdProc( XMLNode *opt )
     else if( a_path.substr(0,7) == "/nd/cfg" ) TConfig::cntrCmdProc(opt,TSYS::pathLev(a_path,2),"root","root",RWRWR_);
     else if( a_path == "/dt/io" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEC_RD) )
 	{
 	    XMLNode *nId   = ctrMkNode("list",opt,-1,"/dt/io/id","");
 	    XMLNode *nNm   = ctrMkNode("list",opt,-1,"/dt/io/nm","");
@@ -1220,17 +1200,17 @@ void Node::cntrCmdProc( XMLNode *opt )
 		if( nVal )	nVal->childAdd("el")->setText( (data && data->val.func()) ? data->val.getS(id) : io(id)->def() );
 	    }
 	}
-	if( ctrChkNode(opt,"add",RWRWR_,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"add",RWRWR_,"root","root",SEC_WR) )
 	{
 	    if( enableStat( ) ) throw TError(nodePath().c_str(),_("Disable node for this operation"));
 	    ioAdd( new IO("new",_("New IO"),IO::Integer,IO::Default) ); modif();
 	}
-	if( ctrChkNode(opt,"ins",RWRWR_,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"ins",RWRWR_,"root","root",SEC_WR) )
 	{
 	    if( enableStat( ) ) throw TError(nodePath().c_str(),_("Disable node for this operation"));
 	    ioIns( new IO("new",_("New IO"),IO::Integer,IO::Default), atoi(opt->attr("row").c_str()) ); modif();
 	}
-	if( ctrChkNode(opt,"del",RWRWR_,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"del",RWRWR_,"root","root",SEC_WR) )
 	{
 	    if( enableStat( ) ) throw TError(nodePath().c_str(),_("Disable node for this operation"));
 	    int row = atoi(opt->attr("row").c_str());
@@ -1239,12 +1219,12 @@ void Node::cntrCmdProc( XMLNode *opt )
 	    ioDel( row );
 	    modif();
 	}
-	if( ctrChkNode(opt,"move",RWRWR_,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"move",RWRWR_,"root","root",SEC_WR) )
 	{
 	    if( enableStat( ) ) throw TError(nodePath().c_str(),_("Disable node for this operation"));
 	    ioMove( atoi(opt->attr("row").c_str()), atoi(opt->attr("to").c_str()) ); modif();
 	}
-	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEC_WR) )
 	{
 	    int row = atoi(opt->attr("row").c_str());
 	    string col = opt->attr("col");
@@ -1261,13 +1241,13 @@ void Node::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/dt/progLang" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEQ_RD) )	opt->setText(progLang());
-	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEQ_WR) )	setProgLang(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEC_RD) )	opt->setText(progLang());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEC_WR) )	setProgLang(opt->text());
     }
     else if( a_path == "/dt/prog" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEQ_RD) )	opt->setText(prog());
-	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEQ_WR) )	setProg(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEC_RD) )	opt->setText(prog());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEC_WR) )	setProg(opt->text());
     }
     else if( a_path == "/dt/plang_ls" && ctrChkNode(opt) )
     {
@@ -1284,10 +1264,15 @@ void Node::cntrCmdProc( XMLNode *opt )
 	vector<string>  ls;
 	switch(c_lv)
 	{
-	    case 0:	SYS->daq().at().modList(ls);	break;
+	    case 0:
+		SYS->daq().at().modList(ls);
+		for( int i_l = 0; i_l < ls.size(); i_l++ )
+		    if( !SYS->daq().at().at(ls[i_l]).at().compileFuncLangs() )
+		    { ls.erase(ls.begin()+i_l); i_l--; }
+		break;
 	    case 1:
 		if( SYS->daq().at().modPresent(TSYS::strSepParse(tplng,0,'.')) )
-		    SYS->daq().at().at(TSYS::strSepParse(tplng,0,'.')).at().compileFuncLangs(ls);
+		    SYS->daq().at().at(TSYS::strSepParse(tplng,0,'.')).at().compileFuncLangs(&ls);
 		break;
 	}
 	for(int i_l = 0; i_l < ls.size(); i_l++)
@@ -1331,9 +1316,9 @@ void Node::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path.substr(0,8) == "/lnk/el_" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	    opt->setText(io(atoi(a_path.substr(8).c_str()))->rez());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{ io(atoi(a_path.substr(8).c_str()))->setRez(opt->text()); modif(); }
     }
     else TCntrNode::cntrCmdProc(opt);
