@@ -122,8 +122,6 @@ class TMdContr: public TController
 		endrun_req;	// Request to stop of the Process task
 	vector< AutoHD<TMdPrm> >  p_hd;
 
-	pthread_t procPthr;	// Process task thread
-
 	double	tm_gath;	// Gathering time
 };
 
@@ -147,7 +145,6 @@ class TTpContr: public TTipDAQ
     private:
 	//Methods
 	TController *ContrAttach( const string &name, const string &daq_db );
-	string optDescr( );
 };
 
 extern TTpContr *mod;
