@@ -43,7 +43,7 @@
 #define MOD_NAME	"Siemens DAQ"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"1.2.0"
+#define VERSION		"1.2.2"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow data source Siemens PLC by CP of Hilscher CIF cards use MPI protocol and library Libnodave for other."
 #define LICENSE		"GPL"
@@ -93,18 +93,6 @@ TTpContr::~TTpContr( )
     if( drvCIFOK( ) )	DevCloseDriver( );
 }
 
-string TTpContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TTpContr::postEnable( int flag )
 {
     TTipDAQ::postEnable(flag);
@@ -112,7 +100,7 @@ void TTpContr::postEnable( int flag )
     //- Controler's DB structure -
     fldAdd( new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30","") );
     fldAdd( new TFld("PERIOD",_("Request data period (ms)"),TFld::Integer,TFld::NoFlag,"5","1000","0;10000") );
-    fldAdd( new TFld("PRIOR",_("Request task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
+    fldAdd( new TFld("PRIOR",_("Request task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
     fldAdd( new TFld("ASINC_WR",_("Asynchronous write mode"),TFld::Boolean,TFld::NoFlag,"1","0") );
     fldAdd( new TFld("TYPE",_("Connection type"),TFld::Integer,TFld::Selected,"1","0",
 	(TSYS::int2str(TMdContr::CIF_PB)+";"+TSYS::int2str(TMdContr::ISO_TCP)).c_str(),"CIF_PB;ISO_TCP") );
@@ -147,27 +135,9 @@ void TTpContr::postEnable( int flag )
 
 void TTpContr::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
-
-    //-- Load CIF devices configuration --
+    //> Load CIF devices configuration
     TConfig cfg(&CIFDevE());
     string bd_tbl = modId()+"_CIFdevs";
     for( int i_b = 0; i_b < MAX_DEV_BOARDS; i_b++ )
@@ -453,7 +423,7 @@ void TTpContr::cntrCmdProc( XMLNode *opt )
 		if(n_pbspd)	n_pbspd->childAdd("el")->setText(TSYS::int2str(cif_devs[i_b].pbspeed));
 	    }
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    int brd = atoi(opt->attr("row").c_str());
 	    string col  = opt->attr("col");
@@ -482,9 +452,9 @@ void TTpContr::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/PB/dev" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	    opt->setText(TBDS::genDBGet(mod->nodePath()+"lifeLsDev","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	    TBDS::genDBSet(mod->nodePath()+"lifeLsDev",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/PB/lifels" && ctrChkNode(opt) )
@@ -589,35 +559,14 @@ void TMdContr::start_( )
 	    prmEn( list_p[i_prm], true );
 
     //> Start the request data task
-    if( !prc_st )
-    {
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	if( m_prior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority=m_prior;
-	pthread_attr_setschedparam(&pthr_attr,&prior);
-
-	pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
-    }
+    if( !prc_st ) SYS->taskCreate( nodePath('.',true), m_prior, TMdContr::Task, this, &prc_st );
 }
 
 void TMdContr::stop_( )
 {
     //> Stop the request and calc data task
-    if( prc_st )
-    {
-	endrun_req = true;
-	pthread_kill( procPthr, SIGALRM );
-	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
-	pthread_join( procPthr, NULL );
-    }
+    if( prc_st ) SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
+
     //> Clear proccess parameters list
     pHd.clear();
 
@@ -1060,23 +1009,23 @@ void TMdContr::putDB( unsigned n_db, long offset, const string &buffer )
     }
 }
 
-char TMdContr::getValB( SValData ival, string &err )
+char TMdContr::getValB( SValData ival, ResString &err )
 {
     for(int i_b = 0; i_b < acqBlks.size(); i_b++)
 	if(acqBlks[i_b].db == ival.db && ival.off >= acqBlks[i_b].off && 
 	    (ival.off+1) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	{
-	    if(!acqBlks[i_b].err.size())
+	    if( !acqBlks[i_b].err.size() )
 	        return (bool)(acqBlks[i_b].val[ival.off-acqBlks[i_b].off]&(0x01<<ival.sz));
-	    else err = acqBlks[i_b].err;
+	    else err.setVal(acqBlks[i_b].err);
 	    break;
 	}
-    err = err.size()?err:_("11:Value is not gathered.");
+    if( err.getVal().empty() ) err.setVal( _("11:Value is not gathered.") );
 
     return EVAL_BOOL;
 }
 
-int TMdContr::getValI( SValData ival, string &err )
+int TMdContr::getValI( SValData ival, ResString &err )
 {
     int iv_sz = valSize( IO::Integer, ival.sz );
     for(int i_b = 0; i_b < acqBlks.size(); i_b++)
@@ -1087,18 +1036,18 @@ int TMdContr::getValI( SValData ival, string &err )
 		switch(iv_sz)
 		{
 		    case 1:	return acqBlks[i_b].val[ival.off-acqBlks[i_b].off];
-		    case 2:	return *(si16*)revers(acqBlks[i_b].val.substr(ival.off-acqBlks[i_b].off,iv_sz)).c_str();
-		    case 4:	return *(si32*)revers(acqBlks[i_b].val.substr(ival.off-acqBlks[i_b].off,iv_sz)).c_str();
+		    case 2:	return *(int16_t*)revers(acqBlks[i_b].val.substr(ival.off-acqBlks[i_b].off,iv_sz)).c_str();
+		    case 4:	return *(int32_t*)revers(acqBlks[i_b].val.substr(ival.off-acqBlks[i_b].off,iv_sz)).c_str();
 		}
-	    else err = acqBlks[i_b].err;
+	    else err.setVal( acqBlks[i_b].err );
 	    break;
 	}
-    err = err.size()?err:_("11:Value is not gathered.");
+    if( err.getVal().empty() ) err.setVal( _("11:Value is not gathered.") );
 
     return EVAL_INT;
 }
 
-double TMdContr::getValR( SValData ival, string &err )
+double TMdContr::getValR( SValData ival, ResString &err )
 {
     int iv_sz = valSize( IO::Real, ival.sz );
     for(int i_b = 0; i_b < acqBlks.size(); i_b++)
@@ -1111,15 +1060,15 @@ double TMdContr::getValR( SValData ival, string &err )
 		    case 4:	return *(float*)revers(acqBlks[i_b].val.substr(ival.off-acqBlks[i_b].off,iv_sz)).c_str();
 		    case 8:	return *(double*)revers(acqBlks[i_b].val.substr(ival.off-acqBlks[i_b].off,iv_sz)).c_str();
 		}
-	    else err = acqBlks[i_b].err;
+	    else err.setVal( acqBlks[i_b].err );
 	    break;
 	}
-    err = err.size()?err:_("11:Value is not gathered.");
+    if( err.getVal().empty() ) err.setVal( _("11:Value is not gathered.") );
 
     return EVAL_REAL;
 }
 
-string TMdContr::getValS( SValData ival, string &err )
+string TMdContr::getValS( SValData ival, ResString &err )
 {
     int iv_sz = valSize( IO::String, ival.sz );
     char buf[iv_sz];
@@ -1128,15 +1077,15 @@ string TMdContr::getValS( SValData ival, string &err )
 	    (ival.off+iv_sz) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	{
 	    if( !acqBlks[i_b].err.size() )	return acqBlks[i_b].val.substr(ival.off-acqBlks[i_b].off,iv_sz);
-	    else err = acqBlks[i_b].err;
+	    else err.setVal( acqBlks[i_b].err );
 	    break;
 	}
-    err = err.size()?err:_("11:Value is not gathered.");
+    if( err.getVal().empty() ) err.setVal( _("11:Value is not gathered.") );
 
     return EVAL_STR;
 }
 
-void TMdContr::setValB( bool ivl, SValData ival, string &err )
+void TMdContr::setValB( bool ivl, SValData ival, ResString &err )
 {
     int val = getValI(SValData(ival.db,ival.off,1),err);
     if(val==EVAL_INT || (bool)(val&(0x01<<ival.sz)) == ivl) return;
@@ -1160,13 +1109,13 @@ void TMdContr::setValB( bool ivl, SValData ival, string &err )
 		    (ival.off+1) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	    { acqBlks[i_b].val[ival.off-acqBlks[i_b].off] = val; break; }
     }
-    catch(TError cerr){ err = err.size()?err:cerr.mess; }
+    catch(TError cerr){ if( err.getVal().empty() ) err.setVal(cerr.mess); }
 }
 
-void TMdContr::setValI( int ivl, SValData ival, string &err )
+void TMdContr::setValI( int ivl, SValData ival, ResString &err )
 {
     int val = getValI(ival,err);
-    if(val==EVAL_INT || val == ivl) return;
+    if( val==EVAL_INT || val == ivl ) return;
     //> Write data to controller or write data block
     val = ivl;
     int iv_sz = valSize( IO::Integer, ival.sz );
@@ -1188,10 +1137,10 @@ void TMdContr::setValI( int ivl, SValData ival, string &err )
 		    (ival.off+iv_sz) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	    { acqBlks[i_b].val.replace(ival.off-acqBlks[i_b].off,iv_sz,revers(string((char *)&val,iv_sz))); break; }
     }
-    catch(TError cerr){ err = err.size()?err:cerr.mess; }
+    catch(TError cerr){ if( err.getVal().empty() ) err.setVal(cerr.mess); }
 }
 
-void TMdContr::setValR( double ivl, SValData ival, string &err )
+void TMdContr::setValR( double ivl, SValData ival, ResString &err )
 {
     double val = getValR(ival,err);
     float  val_4 = val;
@@ -1220,10 +1169,10 @@ void TMdContr::setValR( double ivl, SValData ival, string &err )
 		break;
 	    }
     }
-    catch(TError cerr){ err = err.size()?err:cerr.mess; }
+    catch(TError cerr){ if( err.getVal().empty() ) err.setVal(cerr.mess); }
 }
 
-void TMdContr::setValS( const string &ivl, SValData ival, string &err )
+void TMdContr::setValS( const string &ivl, SValData ival, ResString &err )
 {
     string val = getValS(ival,err);
     int iv_sz = valSize( IO::String, ival.sz );
@@ -1249,7 +1198,7 @@ void TMdContr::setValS( const string &ivl, SValData ival, string &err )
 		    (ival.off+iv_sz) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	    { acqBlks[i_b].val.replace(ival.off-acqBlks[i_b].off,iv_sz,vali.c_str()); break; }
     }
-    catch(TError cerr){ err = err.size()?err:cerr.mess; }
+    catch(TError cerr){ if( err.getVal().empty() ) err.setVal(cerr.mess); }
 }
 
 int TMdContr::valSize( IO::Type itp, int iv_sz )
@@ -1291,7 +1240,6 @@ void *TMdContr::Task( void *icntr )
 		{
 		    if( cntr.redntUse( ) ) { cntr.writeBlks[i_b].err = _("-1:No data"); continue; }
 		    if( atoi(cntr.writeBlks[i_b].err.c_str()) == -1 ) continue;
-		    //printf("TEST 00: Put %d: (%d:%d)\n",
 		    //	cntr.writeBlks[i_b].db, cntr.writeBlks[i_b].off, cntr.writeBlks[i_b].val.size() );
 		    cntr.putDB( cntr.writeBlks[i_b].db,cntr.writeBlks[i_b].off,cntr.writeBlks[i_b].val );
 		    cntr.writeBlks[i_b].err="";
@@ -1302,7 +1250,6 @@ void *TMdContr::Task( void *icntr )
 	    try
 	    {
 		if( cntr.redntUse( ) ) { cntr.acqBlks[i_b].err = _("-1:No data"); continue; }
-		//printf("TEST 00: Get %d: (%d:%d)\n",
 		//    cntr.acqBlks[i_b].db, cntr.acqBlks[i_b].off, cntr.acqBlks[i_b].val.size() );
 		cntr.getDB(cntr.acqBlks[i_b].db, cntr.acqBlks[i_b].off, cntr.acqBlks[i_b].val);
 		cntr.acqBlks[i_b].err="";
@@ -1330,9 +1277,9 @@ void *TMdContr::Task( void *icntr )
     return NULL;
 }
 
-void TMdContr::redntDataUpdate( bool firstArchiveSync )
+void TMdContr::redntDataUpdate( )
 {
-    TController::redntDataUpdate(firstArchiveSync);
+    TController::redntDataUpdate( );
 
     vector<string> pls; list(pls);
 
@@ -1345,8 +1292,7 @@ void TMdContr::redntDataUpdate( bool firstArchiveSync )
     }
 
     //> Send request to first active station for this controller
-    try{ owner().owner().rdStRequest(workId(),req); }
-    catch(TError err) { return; }
+    if( owner().owner().rdStRequest(workId(),req).empty() ) return;
 
     //> Redirect respond to local parameters
     req.setAttr("path","/");
@@ -1572,7 +1518,7 @@ void TMdPrm::vlGet( TVal &val )
     {
 	if( val.name() == "err" )
 	{
-	    if(acq_err.size())	val.setS(acq_err,0,true);
+	    if( acq_err.getVal().size()) val.setS(acq_err.getVal(),0,true);
 	    else if(id_err>=0)	val.setS(getS(id_err),0,true);
 	    else val.setS("0",0,true);
 	}
@@ -1584,9 +1530,9 @@ void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
     //> Send to active reserve station
     if( owner().redntUse( ) )
     {
-	if( val.getS() == pvl.getS() ) return;
+	if( val.getS(0,true) == pvl.getS() ) return;
 	XMLNode req("set");
-	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS(0,true));
 	SYS->daq().at().rdStRequest(owner().workId(),req);
 	return;
     }
@@ -1595,26 +1541,41 @@ void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
     try
     {
 	int id_lnk = lnkId(val.name());
-	if( id_lnk >= 0 && lnk(id_lnk).val.db < 0 )
-	    id_lnk=-1;
-	switch(val.fld().type())
+	if( id_lnk >= 0 && lnk(id_lnk).val.db < 0 ) id_lnk = -1;
+	switch( val.fld().type() )
 	{
 	    case TFld::String:
-		if( id_lnk < 0 ) setS(ioId(val.name()),val.getS(0,true));
-		else owner().setValS(val.getS(0,true),lnk(id_lnk).val,acq_err);
+	    {
+		string vl = val.getS(0,true);
+		if( vl == EVAL_STR || vl == pvl.getS() ) break;
+		if( id_lnk < 0 ) setS( ioId(val.name()), vl );
+		else owner().setValS( vl, lnk(id_lnk).val, acq_err );
 		break;
+	    }
 	    case TFld::Integer:
-		if( id_lnk < 0 ) setI(ioId(val.name()),val.getI(0,true));
-		else owner().setValI(val.getI(0,true),lnk(id_lnk).val,acq_err);
+	    {
+		int vl = val.getI(0,true);
+		if( vl == EVAL_INT || vl == pvl.getI() ) break;
+		if( id_lnk < 0 ) setI( ioId(val.name()), vl );
+		else owner().setValI( vl, lnk(id_lnk).val, acq_err );
 		break;
+	    }
 	    case TFld::Real:
-		if( id_lnk < 0 ) setR(ioId(val.name()),val.getR(0,true));
-		else owner().setValR(val.getR(0,true),lnk(id_lnk).val,acq_err);
+	    {
+		double vl = val.getR(0,true);
+		if( vl == EVAL_REAL || vl == pvl.getR() ) break;
+		if( id_lnk < 0 ) setR( ioId(val.name()), vl );
+		else owner().setValR( vl, lnk(id_lnk).val, acq_err );
 		break;
+	    }
 	    case TFld::Boolean:
-		if( id_lnk < 0 ) setB(ioId(val.name()),val.getB(0,true));
-		else owner().setValB(val.getB(0,true),lnk(id_lnk).val,acq_err);
+	    {
+		char vl = val.getB(0,true);
+		if( vl == EVAL_BOOL || vl == pvl.getB() ) break;
+		if( id_lnk < 0 ) setB( ioId(val.name()), vl );
+		else owner().setValB( vl, lnk(id_lnk).val, acq_err );
 		break;
+	    }
 	}
     }catch(TError err) {  }
 }
@@ -1683,20 +1644,20 @@ void TMdPrm::calc( bool first, bool last )
 {
     try
     {
-	//- Proccess error hold -
-	if( acq_err.size() )
+	//> Proccess error hold
+	if( !acq_err.getVal().empty() )
 	{
 	    time_t tm = time(NULL);
 	    if( !acq_err_tm )	acq_err_tm = tm+5;
-	    if( tm>acq_err_tm )	{ acq_err = ""; acq_err_tm=0; }
+	    if( tm>acq_err_tm )	{ acq_err.setVal(""); acq_err_tm=0; }
 	}
 
-	//- Set fixed system attributes -
+	//> Set fixed system attributes
 	if( id_freq>=0 )	setR(id_freq,1000./owner().period());
 	if( id_start>=0 )	setB(id_start,first);
 	if( id_stop>=0 )	setB(id_stop,last);
 
-	//- Get input links -
+	//> Get input links
 	for( int i_l = 0; i_l < lnkSize(); i_l++ )
 	    if( lnk(i_l).val.db >= 0 )
 		switch(ioType(lnk(i_l).io_id))
@@ -1752,10 +1713,10 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	if( a_path == "/serv/tmplAttr" )
 	{
 	    if( !enableStat() || !func() ) throw TError(nodePath().c_str(),_("Parameter disabled or error."));
-	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )
 		for( int i_a = 0; i_a < ioSize(); i_a++ )
 		    opt->childAdd("a")->setAttr("id",func()->io(i_a)->id())->setText(getS(i_a));
-	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )
 		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
 		{
 		    int io_id = -1;
@@ -1814,7 +1775,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	return;
     }
     //- Process command to page -
-    if( a_path == "/prm/cfg/TMPL" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )
+    if( a_path == "/prm/cfg/TMPL" && ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )
     {
 	m_tmpl = opt->text();
 	disable();
@@ -1846,14 +1807,14 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/cfg/only_off" && enableStat() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	    opt->setText(TBDS::genDBGet(mod->nodePath()+"onlOff","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	    TBDS::genDBSet(mod->nodePath()+"onlOff",opt->text(),opt->attr("user"));
     }
     else if( a_path.substr(0,12) == "/cfg/prm/pr_" && enableStat() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    int lnk_id = lnkId(atoi(a_path.substr(12).c_str()));
 	    string sdb = TSYS::strSepParse(lnk(lnk_id).db_addr,0,'.');
@@ -1862,7 +1823,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	    if((off-t_off)>0)	sdb=sdb+"."+TSYS::int2str(off-t_off);
 	    opt->setText(sdb);
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    string p_nm, cp_nm;
 	    int off, coff, cbit, csz;
@@ -1890,7 +1851,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path.substr(0,12) == "/cfg/prm/el_" && enableStat() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    int i_io = atoi(a_path.substr(12).c_str());
 	    if( func()->io(i_io)->flg()&TPrmTempl::CfgLink )
@@ -1898,7 +1859,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	    else if( func()->io(i_io)->flg()&TPrmTempl::CfgPublConst )
 		opt->setText(getS(i_io));
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    int i_io = atoi(a_path.substr(12).c_str());
 	    if( func()->io(i_io)->flg()&TPrmTempl::CfgLink )
