@@ -104,8 +104,7 @@ void Sensors::getSensors( TMdPrm *prm, bool onlyCreate )
 		if( !feature ) continue;
 		s_id = string(name->prefix)+"_"+main_feature->name;
 		if( !prm->vlPresent(s_id) )
-		    fldAdd( new TFld(s_id.c_str(),(string(name->prefix)+" "+main_feature->name).c_str(),
-			TFld::Real,TFld::NoWrite,"",TSYS::real2str(EVAL_REAL).c_str()) );
+		    fldAdd( new TFld(s_id.c_str(),(string(name->prefix)+" "+main_feature->name).c_str(),TFld::Real,TFld::NoWrite) );
 		if( !onlyCreate && sensors_get_value( name, feature->number, &val) == 0 )
 		    prm->vlAt(s_id).at().setR(val,0,true);
 	    }
@@ -120,8 +119,7 @@ void Sensors::getSensors( TMdPrm *prm, bool onlyCreate )
 		{
 		    s_id = string(name->prefix)+"_"+feature->name;
 		    if( !prm->vlPresent(s_id) )
-			fldAdd( new TFld(s_id.c_str(),(string(name->prefix)+" "+feature->name).c_str(),
-				TFld::Real,TFld::NoWrite,"",TSYS::real2str(EVAL_REAL).c_str()) );
+			fldAdd( new TFld(s_id.c_str(),(string(name->prefix)+" "+feature->name).c_str(), TFld::Real,TFld::NoWrite) );
 		    if( !onlyCreate )
 		    {
 			sensors_get_feature( *name, feature->number, &val);
@@ -142,8 +140,7 @@ void Sensors::getSensors( TMdPrm *prm, bool onlyCreate )
 	while( fgets(buf,sizeof(buf),fp) )
 	{
 	    if( sscanf(buf, "%31s : %f", name, &val) != 2 ) continue;
-	    if( !prm->vlPresent(name) )
-		fldAdd( new TFld(name,name,TFld::Real,TFld::NoWrite,"",TSYS::real2str(EVAL_REAL).c_str()) );
+	    if( !prm->vlPresent(name) )	fldAdd( new TFld(name,name,TFld::Real,TFld::NoWrite) );
 	    if( !onlyCreate ) prm->vlAt(name).at().setR(val,0,true);
 	}
 	pclose(fp);
