@@ -118,11 +118,9 @@ void HddSmart::getVal( TMdPrm *prm )
     FILE *fp = popen(cmd.c_str(),"r");
     if( fp )
     {
-	printf("TEST 00\n");
 	while( fgets(buf,sizeof(buf),fp) != NULL )
 	{
 	    if( sscanf(buf,"%30s : %50s",name,info) != 2 ) continue;
-	    printf("TEST 01: <%s>\n",name);
 	    if(!prm->vlPresent(name))
                 fldAdd( new TFld(name,name,TFld::String,Fld::NoWrite) );
 	    prm->vlAt(name).at().setS(info,NULL,true);
@@ -139,8 +137,7 @@ void HddSmart::getVal( TMdPrm *prm )
 	{
 	    if( sscanf(buf,"%d %30s %*x %*d %*d %*d %*s %*s %*s %lu\n",&id,name,&val) != 3 ) continue;
 	    string s_id = TSYS::int2str(id);
-	    if(!prm->vlPresent(s_id))
-		fldAdd( new TFld(s_id.c_str(),name,TFld::Integer,TFld::NoWrite,"",TSYS::int2str(EVAL_INT).c_str()) );
+	    if( !prm->vlPresent(s_id) )	fldAdd( new TFld(s_id.c_str(),name,TFld::Integer,TFld::NoWrite) );
 	    prm->vlAt(s_id).at().setI(val,0,true);
 	}
 	fclose(fp);
