@@ -49,7 +49,7 @@
 #define MOD_NAME	"System DA"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"1.7.0"
+#define VERSION		"1.7.1"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow operation system data acquisition. Support OS Linux data sources: HDDTemp, Sensors, Uptime, Memory, CPU and other."
 #define LICENSE		"GPL"
@@ -99,46 +99,17 @@ TTpContr::~TTpContr()
     m_da.clear();
 }
 
-string TTpContr::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return(buf);
-}
-
 void TTpContr::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 }
 
 void TTpContr::postEnable( int flag )
 {
     TTipDAQ::postEnable(flag);
 
-    //- Init DA sources -
+    //> Init DA sources
     daReg( new CPU() );
     daReg( new Mem() );
     daReg( new Sensors() );
@@ -148,14 +119,14 @@ void TTpContr::postEnable( int flag )
     daReg( new HddStat() );
     daReg( new NetStat() );
 
-    //- Controler's bd structure -
+    //> Controler's bd structure
     fldAdd( new TFld("AUTO_FILL",_("Auto create active DA"),TFld::Boolean,TFld::NoFlag,"1","0") );
     fldAdd( new TFld("PRM_BD",_("System parameteres table"),TFld::String,TFld::NoFlag,"30","system") );
     fldAdd( new TFld("PERIOD",_("Request data period (ms)"),TFld::Integer,TFld::NoFlag,"5","1000","0;10000") );
-    fldAdd( new TFld("PRIOR",_("Request task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
+    fldAdd( new TFld("PRIOR",_("Request task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
 
-    //- Parameter type bd structure -
-    //-- Make enumerated --
+    //> Parameter type bd structure
+    //>> Make enumerated
     string el_id,el_name,el_def;
     vector<string> list;
     daList(list);
@@ -235,37 +206,16 @@ void TMdContr::enable_(  )
 
 void TMdContr::start_( )
 {
-    //- Start the request data task -
-    if( !prc_st )
-    {
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	if( m_prior && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority=m_prior;
-	pthread_attr_setschedparam(&pthr_attr,&prior);
-
-	pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no started!"));
-    }
+    //> Start the request data task
+    if( !prc_st ) SYS->taskCreate( nodePath('.',true), m_prior, TMdContr::Task, this, &prc_st );
 }
 
 void TMdContr::stop_( )
 {
-    //- Stop the request and calc data task -
-    if( prc_st )
-    {
-	endrun_req = true;
-	pthread_kill( procPthr, SIGALRM );
-	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no stoped!"));
-	pthread_join( procPthr, NULL );
-    }
-    //- Set Eval for parameters -
+    //> Stop the request and calc data task
+    if( prc_st ) SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
+
+    //> Set Eval for parameters
     ResAlloc res(en_res,true);
     for( int i_prm = 0; i_prm < p_hd.size(); i_prm++ )
         p_hd[i_prm].at().setEval();
@@ -421,7 +371,7 @@ void TMdPrm::setType( const string &da_id )
 {
     if( m_da && da_id == m_da->id() )	return;
 
-    //- Free previous type -
+    //> Free previous type
     if( m_da )
     {
 	m_da->deInit(this);
@@ -429,7 +379,7 @@ void TMdPrm::setType( const string &da_id )
 	m_da = NULL;
     }
 
-    //- Create new type -
+    //> Create new type
     try
     {
 	if(da_id.size())
@@ -447,7 +397,7 @@ void TMdPrm::setType( const string &da_id )
 
 bool TMdPrm::cfgChange( TCfg &i_cfg )
 {
-    //- Change TYPE parameter -
+    //> Change TYPE parameter
     if( i_cfg.name() == "TYPE" )
     {
 	setType(i_cfg.getS());
