@@ -59,9 +59,9 @@ class TMdPrm : public TParamContr
 	void autoC( bool val )	{ m_auto = val; }
 	bool autoC( )		{ return m_auto; }
 
-	//- Set perameter type -
+	//> Set perameter type
 	void setType( const string &da_id );
-	//- Get new value -
+	//> Get new value
 	void getVal( );
 	void setEval( );
 
@@ -125,7 +125,6 @@ class TMdContr: public TController
 	vector< AutoHD<TMdPrm> >  p_hd;
 
 	double	tm_calc;	// Scheme's calc time
-	pthread_t procPthr;	// Process task thread
 };
 
 //*************************************************
@@ -152,7 +151,6 @@ class TTpContr: public TTipDAQ
     private:
 	//Methods
 	TController *ContrAttach( const string &name, const string &daq_db );
-	string optDescr( );
 
 	//Attributes
 	vector<DA *> m_da;
