@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Protocol.HTTP file: http.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -112,7 +112,7 @@ void TProt::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -189,8 +189,8 @@ void TProt::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/prm/lf_tm" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText( TSYS::int2str(authTime()) );
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	setAuthTime( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0660,"root","root",SEC_RD) )	opt->setText( TSYS::int2str(authTime()) );
+	if( ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )	setAuthTime( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )	opt->setText(optDescr());
     else TProtocol::cntrCmdProc(opt);
@@ -216,9 +216,7 @@ string TProtIn::httpHead( const string &rcode, int cln, const string &addattr )
 	    "Server: "+PACKAGE_STRING+"\r\n"
 	    "Accept-Ranges: bytes\r\n"
 	    "Content-Length: "+TSYS::int2str(cln)+"\r\n"
-	    "Keep-Alive: timeout=5, max=100\r\n"
-	    "Connection: Keep-Alive\r\n"
-	    "Content-type: text/html\r\n"
+	    "Content-Type: text/html\r\n"
 	    "Charset="+Mess->charset()+"\r\n"+addattr+"\r\n";
 }
 
@@ -243,7 +241,7 @@ bool TProtIn::mess( const string &reqst, string &answer, const string &sender )
     {
 	int	pos = 0;
 #if OSC_DEBUG >= 3
-	mess_debug(nodePath().c_str(),_("Content:\n%s"),request.c_str());
+	mess_debug(nodePath().c_str(),_("Content: %d:\n%s"),request.size(),request.c_str());
 #endif
 
 	//> Parse first record
