@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Protocol.SelfSystem file: self.h
 /***************************************************************************
- *   Copyright (C) 2007-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2007-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -94,12 +94,10 @@ class TProt: public TProtocol
     private:
 	//Methods
 	void cntrCmdProc( XMLNode *opt );
-	string optDescr( );
 	TProtocolIn *in_open( const string &name );
 
 	//Attributes
 	Res		ses_res;
-	Res		ores;
 	vector<SAuth>	auth_lst;
 	int		m_t_auth, mComprLev, mComprBrd;
 };
