@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Special.FLibComplex1 file: libcompl1.h
 /***************************************************************************
- *   Copyright (C) 2005-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2005-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -476,7 +476,7 @@ class Increm : public TFunction
 	string descr( )
 	{
 	    return _("Incrementator per formula:\n"
-		"out = if( in1 > in2 )then in2 + in3*(in1-in2); else in2 - in4*(in2-in1);");
+		"out = if( in > prev ) then prev + (k+)*(in-prev); else prev - (k-)*(prev-in);");
 	}
 
 	void calc( TValFunc *v )
@@ -536,29 +536,32 @@ class Divider : public TFunction
 //*   1:sp    - Setpoint                                              *
 //*   2:max   - Maximum scale                                         *
 //*   3:min   - Minimum scale                                         *
-//*   4:out   - Output                                                *
-//*   5:auto  - Automatic mode                                        *
-//*   6:casc  - Cascade mode                                          *
-//*   7:Kp    - Gain                                                  *
-//*   8:Ti    - Integral time (ms)                                    *
-//*   9:Td    - Differencial time (ms)                                *
-//*   10:Tf   - Filter (lag) time (ms)                                *
-//*   11:Hup  - Up output limit (%)                                   *
-//*   12:Hdwn - Down output limit (%)                                 *
-//*   13:Zi   - Insensibility zone (%)                                *
-//*   14:K1   - Scale input 1                                         *
-//*   15:in1  - Add input 1                                           *
-//*   16:K2   - Scale input 2                                         *
-//*   17:in2  - Add input 2                                           *
-//*   18:K3   - Scale input 3                                         *
-//*   19:in3  - Add input 3                                           *
-//*   20:K4   - Scale input 4                                         *
-//*   21:in4  - Add input 4                                           *
-//*   22:cycle- Calc cycle (ms)                                       *
+//*   4:manIn - Manual input                                          *
+//*   5:out   - Output                                                *
+//*   6:auto  - Automatic mode                                        *
+//*   7:casc  - Cascade mode                                          *
+//*   8:Kp    - Gain                                                  *
+//*   9:Ti    - Integral time (ms)                                    *
+//*   10:Kd   - Kd                                                    *
+//*   11:Td   - Differencial time (ms)                                *
+//*   12:Tzd  - Filter (lag) time (ms)                                *
+//*   13:Hup  - Up output limit (%)                                   *
+//*   14:Hdwn - Down output limit (%)                                 *
+//*   15:Zi   - Insensibility zone (%)                                *
+//*   16:followSp - Follow to sp from variable for manual             *
+//*   17:K1   - Scale input 1                                         *
+//*   18:in1  - Add input 1                                           *
+//*   19:K2   - Scale input 2                                         *
+//*   20:in2  - Add input 2                                           *
+//*   21:K3   - Scale input 3                                         *
+//*   22:in3  - Add input 3                                           *
+//*   23:K4   - Scale input 4                                         *
+//*   24:in4  - Add input 4                                           *
+//*   25:cycle- Calc cycle (ms)                                       *
 //* Internal data:                                                    *
-//*   23:#int - Curent integral value                                 *
-//*   24:#dif - Curent differencial value                             *
-//*   25:#lag - Curent lag value                                      *
+//*   26:#int - Curent integral value                                 *
+//*   27:#dif - Curent differencial value                             *
+//*   28:#lag - Curent lag value                                      *
 //*********************************************************************
 class PID : public TFunction
 {
@@ -567,19 +570,22 @@ class PID : public TFunction
 	{
 	    //Inputs
 	    ioAdd( new IO("var",_("Variable"),IO::Real,IO::Default,"0") );
-	    ioAdd( new IO("sp",_("Setpoint"),IO::Real,IO::Default,"0") );
+	    ioAdd( new IO("sp",_("Setpoint"),IO::Real,IO::Output,"0") );
 	    ioAdd( new IO("max",_("Max scale"),IO::Real,IO::Default,"100") );
 	    ioAdd( new IO("min",_("Min scale"),IO::Real,IO::Default,"0") );
+	    ioAdd( new IO("manIn",_("Manual input (%)"),IO::Real,IO::Default,"0") );
 	    ioAdd( new IO("out",_("Output (%)"),IO::Real,IO::Return,"0") );
 	    ioAdd( new IO("auto",_("Auto mode"),IO::Boolean,IO::Default,"0") );
 	    ioAdd( new IO("casc",_("Cascade mode"),IO::Boolean,IO::Default,"0") );
 	    ioAdd( new IO("Kp",_("Kp"),IO::Real,IO::Default,"1") );
 	    ioAdd( new IO("Ti",_("Ti (ms)"),IO::Integer,IO::Default,"1000") );
+	    ioAdd( new IO("Kd",_("Kd"),IO::Real,IO::Default,"1") );
 	    ioAdd( new IO("Td",_("Td (ms)"),IO::Integer,IO::Default,"0") );
-	    ioAdd( new IO("Tf",_("Tf-lag (ms)"),IO::Integer,IO::Default,"0") );
+	    ioAdd( new IO("Tzd",_("Td lag (ms)"),IO::Integer,IO::Default,"0") );
 	    ioAdd( new IO("Hup",_("Out up limit (%)"),IO::Real,IO::Default,"100") );
 	    ioAdd( new IO("Hdwn",_("Out down limit (%)"),IO::Real,IO::Default,"0") );
 	    ioAdd( new IO("Zi",_("Insensibility (%)"),IO::Real,IO::Default,"1") );
+	    ioAdd( new IO("followSp",_("Follow sp from var on manual"),IO::Boolean,IO::Default,"1") );
 
 	    ioAdd( new IO("K1",_("K input 1"),IO::Real,IO::Default,"0") );
 	    ioAdd( new IO("in1",_("Input 1"),IO::Real,IO::Default,"0") );
@@ -590,12 +596,12 @@ class PID : public TFunction
 	    ioAdd( new IO("K4",_("K input 4"),IO::Real,IO::Default,"0") );
 	    ioAdd( new IO("in4",_("Input 4"),IO::Real,IO::Default,"0") );
 
-	    ioAdd( new IO("cycle",_("Calc cycle (ms)"),IO::Integer,IO::Default,"1000") );
+	    ioAdd( new IO("f_frq",_("Calc frequency (Hz)"),IO::Real,IO::Default,"1") );
 
 	    //Internal data:
 	    ioAdd( new IO("int",_("Integral value"),IO::Real,IO::Output,"0",true) );
 	    ioAdd( new IO("dif",_("Differencial value"),IO::Real,IO::Output,"0",true) );
-	    ioAdd( new IO("lag",_("Lag value"),IO::Real,IO::Output,"0",true) );
+	    ioAdd( new IO("lag",_("Differencial lag"),IO::Real,IO::Output,"0",true) );
 	}
 
 	string name( )	{ return _("PID regulator"); }
@@ -607,27 +613,30 @@ class PID : public TFunction
 			sp	= v->getR(1),
 			max	= v->getR(2),
 			min	= v->getR(3),
-			out	= v->getR(4),
-			kp	= v->getR(7),
-			h_up	= v->getR(11),
-			h_dwn	= v->getR(12),
-			zi	= v->getR(13),
-			k1	= v->getR(14),
-			in1	= v->getR(15),
-			k2	= v->getR(16),
-			in2	= v->getR(17),
-			k3	= v->getR(18),
-			in3	= v->getR(19),
-			k4	= v->getR(20),
-			in4	= v->getR(21),
-			cycle	= v->getI(22),
-			integ	= v->getR(23),
-			difer	= v->getR(24),
-			lag	= v->getR(25);
+			manIn	= v->getR(4),
+			out	= v->getR(5),
+			kp	= v->getR(8),
+			kpd	= v->getR(10),
+			h_up	= v->getR(13),
+			h_dwn	= v->getR(14),
+			zi	= v->getR(15),
+			followSp = v->getB(16),
+			k1	= v->getR(17),
+			in1	= v->getR(18),
+			k2	= v->getR(19),
+			in2	= v->getR(20),
+			k3	= v->getR(21),
+			in3	= v->getR(22),
+			k4	= v->getR(23),
+			in4	= v->getR(24),
+			frq	= v->getR(25),
+			integ	= v->getR(26),
+			difer	= v->getR(27),
+			dlag	= v->getR(28);
 
-	    double	Kf	= (v->getI(10)>cycle)?cycle/v->getI(10):1.;
-	    double	Kint	= (v->getI(8)>cycle)?cycle/v->getI(8):1.;
-	    double	Kdif	= (v->getI(9)>cycle)?cycle/v->getI(9):1.;
+	    double	Tzd	= vmin(1e3/(frq*v->getI(12)),1);
+	    double	Kint	= vmin(1e3/(frq*v->getI(9)),1);
+	    double	Kdif	= vmin(1e3/(frq*v->getI(11)),1);
 
 	    //> Scale error
 	    if( max <= min )	return;
@@ -636,7 +645,7 @@ class PID : public TFunction
 	    sp = 100.*(sp+min)/(max-min);
 	    val = 100.*(val+min)/(max-min);
 	    val += k1*in1 + k2*in2;
-	    val=vmin(100.,vmax(-100.,val));
+	    val = vmin(100.,vmax(-100.,val));
 
 	    //> Error
 	    double err = sp - val;
@@ -645,29 +654,34 @@ class PID : public TFunction
 	    err = (fabs(err)<zi)?0:((err>0)?err-zi:err+zi);
 
 	    //> Gain
-	    err*=kp;
-	    err=vmin(100.,vmax(-100.,err));
+	    err *= kp;
+	    err = vmin(100.,vmax(-100.,err));
 
-	    //> Input filter lag
-	    lag+=Kf*(err-lag);
-	    integ+=Kint*lag;		//Integral
-	    difer-=Kdif*(difer-lag);	//Differecial lag
+	    integ += Kint*err;		//Integral
+	    difer -= Kdif*(difer-val);	//Differecial wait
+	    dlag  += Tzd*((difer-val)-dlag);	//Differecial lag
 
 	    //> Automatic mode enabled
-	    if( v->getB(5) ) out = (2.*lag + integ - difer) + k3*in3 + k4*in4;
-	    else { v->setB(6,false); v->setR(1,v->getR(0)); }
+	    if( v->getB(6) ) out = err + integ + kpd*dlag + k3*in3 + k4*in4;
+	    else
+	    {
+		v->setB(7,false);
+		if( followSp ) v->setR(1,v->getR(0));
+		out = manIn;
+	    }
 
 	    //> Check output limits
-	    out=vmin(h_up,vmax(h_dwn,out));
+	    out = manIn = vmin(h_up,vmax(h_dwn,out));
 
 	    //> Fix integral for manual and limits
-	    integ = out - 2.*lag + difer - k3*in3 + k4*in4;
+	    integ = out - err - kpd*dlag - k3*in3 - k4*in4;
 
 	    //> Write outputs
-	    v->setR(4,out);
-	    v->setR(23,integ);
-	    v->setR(24,difer);
-	    v->setR(25,lag);
+	    v->setR(4,manIn);
+	    v->setR(5,out);
+	    v->setR(26,integ);
+	    v->setR(27,difer);
+	    v->setR(28,dlag);
 	}
 };
 
@@ -739,8 +753,8 @@ class Flow : public TFunction
 //* Sum+mult                                                          *
 //* Formula: out = in1_1*in1_2*(in1_3*in1_4+in1_5) +                  *
 //*                in2_1*in2_2*(in2_3*in2_4+in2_5) +                  *
-//*                in4_1*in4_2*(in4_3*in4_4+in4_5) +                  *
-//*                in5_1*in5_2*(in5_3*in5_4+in5_5);                   *
+//*                in3_1*in3_2*(in3_3*in3_4+in3_5) +                  *
+//*                in4_1*in4_2*(in4_3*in4_4+in4_5);                   *
 //*********************************************************************
 class SumMult : public TFunction
 {
@@ -764,7 +778,7 @@ class SumMult : public TFunction
 	{
 	    return _("Sum and mult per formule:\n"
 		"out = in1_1*in1_2*(in1_3*in1_4+in1_5) + in2_1*in2_2*(in2_3*in2_4+in2_5) +\n"
-		"      in4_1*in4_2*(in4_3*in4_4+in4_5) + in5_1*in5_2*(in5_3*in5_4+in5_5);");
+		"      in3_1*in3_2*(in3_3*in3_4+in3_5) + in4_1*in4_2*(in4_3*in4_4+in4_5);");
 	}
 
 	void calc( TValFunc *v )
@@ -782,8 +796,8 @@ class SumMult : public TFunction
 //* Sum+div                                                           *
 //* Formula: out = in1_1*in1_2*(in1_3+in1_4/in1_5) +                  *
 //*                in2_1*in2_2*(in2_3+in2_4/in2_5) +                  *
-//*                in4_1*in4_2*(in4_3+in4_4/in4_5) +                  *
-//*                in5_1*in5_2*(in5_3+in5_4/in5_5);                   *
+//*                in3_1*in3_2*(in3_3+in3_4/in3_5) +                  *
+//*                in4_1*in4_2*(in4_3+in4_4/in4_5);                   *
 //*********************************************************************
 class SumDiv : public TFunction
 {
@@ -807,7 +821,7 @@ class SumDiv : public TFunction
 	{
 	    return _("Sum and divide per formule:\n"
 		"out = in1_1*in1_2*(in1_3+in1_4/in1_5) + in2_1*in2_2*(in2_3+in2_4/in2_5) +\n"
-		"      in4_1*in4_2*(in4_3+in4_4/in4_5) + in5_1*in5_2*(in5_3+in5_4/in5_5);");
+		"      in3_1*in3_2*(in3_3+in3_4/in3_5) + in4_1*in4_2*(in4_3+in4_4/in4_5);");
 	}
 
 	void calc( TValFunc *v )
