@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Special.FLibSYS file: statfunc.h
 /***************************************************************************
- *   Copyright (C) 2005-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2005-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -26,6 +26,7 @@
 #include <vector>
 
 #include <tspecials.h>
+#include <tarchval.h>
 
 #undef _
 #define _(mess) mod->I18N(mess)
@@ -55,15 +56,6 @@ class Lib : public TSpecial
 	void reg( TFunction *fnc )		{ chldAdd(m_fnc,fnc); }
 	void unreg( const char *id )		{ chldDel(m_fnc,id); }
 
-	//- Value archives and buffers access methods -
-	int varchOpen( const string &inm );
-	int varchBufOpen( TFld::Type vtp, int isz, int ipr, bool ihgrd, bool ihres );
-	void varchClose( int id );
-	bool isArch( int id );
-	AutoHD<TVArchive> varch( int id );
-	TValBuf *vbuf( int id );
-	void varchFree( );
-
     private:
 	//Methods
 	void postEnable( int flag );
@@ -71,20 +63,6 @@ class Lib : public TSpecial
 
 	//Attributes
 	int	m_fnc;
-
-	//- Value archive resources -
-	Res	varch_res;
-	struct SVarch
-	{
-	    bool isArch;
-	    union
-	    {
-		AutoHD<TVArchive> *arch;
-		TValBuf	*buf;
-	    };
-	};
-
-	vector<SVarch>	varch_lst;
 };
 
 extern Lib *mod;
