@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Special.SystemTests file: test_kernel.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -39,7 +39,7 @@
 #define MOD_TYPE	"Special"
 #define VER_TYPE	VER_SPC
 #define SUB_TYPE	"TEST"
-#define VERSION		"1.3.6"
+#define VERSION		"1.3.7"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow the group tests for OpenSCADA system."
 #define LICENSE		"GPL"
@@ -165,7 +165,7 @@ void TTest::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -175,25 +175,12 @@ void TTest::load_( )
 
 void TTest::modStart(  )
 {
-    if( run_st ) return;
-    pthread_attr_t pthr_attr;
-
-    pthread_attr_init(&pthr_attr);
-    pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-    pthread_create(&pthr_tsk,&pthr_attr,Task,this);
-    pthread_attr_destroy(&pthr_attr);
-    if( TSYS::eventWait( run_st, true, string(MOD_ID)+": Is starting....",5) )
-	throw TError(nodePath().c_str(),_("Not started!"));
+    if( !run_st ) SYS->taskCreate( nodePath('.',true), 0, Task, this, &run_st );
 }
 
 void TTest::modStop(  )
 {
-    if( !run_st ) return;
-
-    endrun = true;
-    if( TSYS::eventWait( run_st, false, string(MOD_ID)+": Is stopping....",5) )
-	throw TError(nodePath().c_str(),_("Not stopped!"));
-    pthread_join( pthr_tsk, NULL );
+    if( run_st ) SYS->taskDestroy( nodePath('.',true), &run_st, &endrun );
 }
 
 void *TTest::Task( void *CfgM )
