@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Special.SystemTests file: test_kernel.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -56,8 +56,6 @@ class TTest: public TSpecial
 	void Test( const string &id, XMLNode *t_n );
 
 	//Attributes
-	pthread_t pthr_tsk;
-
 	bool	endrun;		// Stop pthread command
 };
 
