@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Transport.SSL file: modssl.cpp
 /***************************************************************************
- *   Copyright (C) 2008 by Roman Savochenko                                *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2008-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,8 +19,6 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-//#include <sys/socket.h>
-
 #include <getopt.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -41,7 +39,7 @@
 #define MOD_NAME	"SSL"
 #define MOD_TYPE	"Transport"
 #define VER_TYPE	VER_TR
-#define VERSION		"0.8.1"
+#define VERSION		"0.9.1"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow security socket layer based transport. Used OpenSSL and supported SSLv2, SSLv3 and TLSv1."
 #define LICENSE		"GPL"
@@ -83,7 +81,7 @@ TTransSock::TTransSock( string name )
 
     mod		= this;
 
-    //- CRYPTO reentrant init -
+    //> CRYPTO reentrant init
     mutex_buf = (pthread_mutex_t *)malloc( CRYPTO_num_locks( ) * sizeof(pthread_mutex_t) );
     for( int i = 0; i < CRYPTO_num_locks( ); i++ )
 	pthread_mutex_init(&mutex_buf[i],NULL);
@@ -93,7 +91,7 @@ TTransSock::TTransSock( string name )
     CRYPTO_set_dynlock_lock_callback(dyn_lock_function);
     CRYPTO_set_dynlock_destroy_callback(dyn_destroy_function);
 
-    //- SSL init -
+    //> SSL init
     SSL_library_init();
     SSL_load_error_strings();
     RAND_load_file("/dev/urandom",1024);
@@ -117,15 +115,8 @@ void TTransSock::postEnable( int flag )
 
     if( flag&TCntrNode::NodeConnect )
     {
-	//> Add self DB-fields to input transport
-	owner().inEl().fldAdd( new TFld("BufLen",_("Input socket buffer length (kB)"),TFld::Integer,0,"4","5") );
-	owner().inEl().fldAdd( new TFld("MaxClients",_("Maximum clients process"),TFld::Integer,0,"3","10") );
-	owner().inEl().fldAdd( new TFld("SSLCertKey",_("SSL PEM certificat and private key"),TFld::String,TFld::FullText,"10000") );
-	owner().inEl().fldAdd( new TFld("SSLKeyPass",_("SSL key password"),TFld::String,0,"20") );
-
-	//> Add self DB-fields to input transport
-	owner().outEl().fldAdd( new TFld("SSLCertKey",_("SSL PEM certificat and private key"),TFld::String,TFld::FullText,"10000") );
-	owner().outEl().fldAdd( new TFld("SSLKeyPass",_("SSL key password"),TFld::String,0,"20") );
+	owner().inEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
+	owner().outEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
     }
 }
 
@@ -161,38 +152,10 @@ void TTransSock::dyn_destroy_function( struct CRYPTO_dynlock_value *l, const cha
     free(l);
 }
 
-string TTransSock::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TTransSock::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 }
 
 TTransportIn *TTransSock::In( const string &name, const string &idb )
@@ -205,29 +168,13 @@ TTransportOut *TTransSock::Out( const string &name, const string &idb )
     return new TSocketOut(name,idb,&owner().outEl());
 }
 
-void TTransSock::cntrCmdProc( XMLNode *opt )
-{
-    //> Get page info
-    if( opt->name() == "info" )
-    {
-	TTipTransport::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
-	return;
-    }
-
-    //> Process command to page
-    string a_path = opt->attr("path");
-    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )   opt->setText(optDescr());
-    else TTipTransport::cntrCmdProc(opt);
-}
-
 //************************************************
 //* TSocketIn                                    *
 //************************************************
 TSocketIn::TSocketIn( string name, const string &idb, TElem *el ) :
     TTransportIn(name,idb,el), ctx(NULL), cl_free(true),
-    mBufLen(cfg("BufLen").getId()), mMaxFork(cfg("MaxClients").getId()),
-    mCertKey(cfg("SSLCertKey").getSd()), mKeyPass(cfg("SSLKeyPass").getSd())
+    mMaxFork(10), mBufLen(5), mKeepAliveCon(100), mKeepAliveTm(5), mTaskPrior(0),
+    mAPrms(cfg("A_PRMS").getSd())
 {
     setAddr("localhost:10042");
 }
@@ -252,40 +199,67 @@ string TSocketIn::getStatus( )
     return rez;
 }
 
+void TSocketIn::load_( )
+{
+    TTransportIn::load_();
+
+    try
+    {
+	XMLNode prmNd;
+	string  vl;
+	prmNd.load(mAPrms);
+	vl = prmNd.attr("MaxClients");	if( !vl.empty() ) mMaxFork = atoi(vl.c_str());
+	vl = prmNd.attr("BufLen");	if( !vl.empty() ) mBufLen = atoi(vl.c_str());
+	vl = prmNd.attr("KeepAliveCnt");if( !vl.empty() ) mKeepAliveCon = atoi(vl.c_str());
+	vl = prmNd.attr("KeepAliveTm");	if( !vl.empty() ) mKeepAliveTm = atoi(vl.c_str());
+	vl = prmNd.attr("TaskPrior");	if( !vl.empty() ) mTaskPrior = atoi(vl.c_str());
+	if( prmNd.childGet("CertKey",0,true) ) mCertKey = prmNd.childGet("CertKey")->text();
+	mKeyPass = prmNd.attr("PKeyPass");
+    } catch(...){ }
+}
+
+void TSocketIn::save_( )
+{
+    XMLNode prmNd("prms");
+    prmNd.setAttr("MaxClients",TSYS::int2str(mMaxFork));
+    prmNd.setAttr("BufLen",TSYS::int2str(mBufLen));
+    prmNd.setAttr("KeepAliveCnt",TSYS::int2str(mKeepAliveCon));
+    prmNd.setAttr("KeepAliveTm",TSYS::int2str(mKeepAliveTm));
+    prmNd.setAttr("TaskPrior",TSYS::int2str(mTaskPrior));
+    if( prmNd.childGet("CertKey",0,true) ) prmNd.childGet("CertKey")->setText(mCertKey);
+    else prmNd.childAdd("CertKey")->setText(mCertKey);
+    prmNd.setAttr("PKeyPass",mKeyPass);
+    mAPrms = prmNd.save(XMLNode::BrAllPast);
+
+    TTransportIn::save_();
+}
+
 void TSocketIn::start()
 {
     pthread_attr_t pthr_attr;
 
     if( run_st ) return;
 
-    //- Status clear -
+    //> Status clear
     stErr = "";
     trIn = trOut = 0;
     connNumb = clsConnByLim = 0;
 
-    //- Wait connection main task start -
-    pthread_attr_init(&pthr_attr);
-    pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-    pthread_create(&pthr_tsk,&pthr_attr,Task,this);
-    pthread_attr_destroy(&pthr_attr);
-    if( TSYS::eventWait( run_st, true,nodePath()+"open",5) )
-	throw TError(nodePath().c_str(),_("Not opened!"));
+    //> Wait connection main task start
+    SYS->taskCreate( nodePath('.',true), taskPrior(), Task, this, &run_st );
 }
 
 void TSocketIn::stop()
 {
     if( !run_st )	return;
 
-    //- Status clear -
+    //> Status clear
     stErr = "";
     trIn = trOut = 0;
     connNumb = clsConnByLim = 0;
 
-    //- Wait connection main task stop -
-    endrun = true;
-    if( TSYS::eventWait( run_st, false, nodePath()+"close",5) )
-	throw TError(nodePath().c_str(),_("Not closed!"));
-    pthread_join( pthr_tsk, NULL );
+    //> Wait connection main task stop
+    SYS->taskDestroy( nodePath('.',true), &run_st, &endrun );
 }
 
 void *TSocketIn::Task( void *sock_in )
@@ -301,20 +275,28 @@ void *TSocketIn::Task( void *sock_in )
     mess_debug(s.nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
 #endif
 
-    //- Client's sockets attrs init
+    //> Client's sockets attrs init
     pthread_t th;
     pthread_attr_t pthr_attr;
     pthread_attr_init(&pthr_attr);
-    pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
+    struct sched_param prior;
+    pthread_attr_setschedpolicy(&pthr_attr,(s.taskPrior()&&SYS->user()=="root")?SCHED_RR:SCHED_OTHER);
+    prior.__sched_priority = s.taskPrior();
+    pthread_attr_setschedparam(&pthr_attr,&prior);
     pthread_attr_setdetachstate(&pthr_attr, PTHREAD_CREATE_DETACHED);
 
-    //- SSL context init -
+    //> SSL context init
     string ssl_host = TSYS::strSepParse(s.addr(),0,':');
     string ssl_port = TSYS::strSepParse(s.addr(),1,':');
     string ssl_method = TSYS::strSepParse(s.addr(),2,':');
 
-    //-- Set SSL method --
-    SSL_METHOD *meth = SSLv23_server_method();
+    //>> Set SSL method
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L
+    const SSL_METHOD *meth;
+#else
+    SSL_METHOD *meth;
+#endif
+    meth = SSLv23_server_method();
     if( ssl_method == "SSLv2" )		meth = SSLv2_server_method();
     else if( ssl_method == "SSLv3" )	meth = SSLv3_server_method();
     else if( ssl_method == "TLSv1" )	meth = TLSv1_server_method();
@@ -328,32 +310,32 @@ void *TSocketIn::Task( void *sock_in )
 	    throw TError(s.nodePath().c_str(),"SSL_CTX_new: %s",err);
 	}
 
-	//- Write certificate and private key to temorary file -
+	//> Write certificate and private key to temorary file
 	cfile = tmpnam(err);
 	int icfile = open(cfile.c_str(),O_EXCL|O_CREAT|O_WRONLY,0644);
 	if( icfile < 0 ) throw TError(s.nodePath().c_str(),_("Open temporaty file '%s' error: '%s'"),cfile.c_str(),strerror(errno));
 	write(icfile,s.certKey().data(),s.certKey().size());
 	close(icfile);
 
-	//-- Set private key password --
+	//>> Set private key password
 	SSL_CTX_set_default_passwd_cb_userdata(s.ctx,(char*)s.pKeyPass().c_str());
-	//-- Load certificate --
+	//>> Load certificate
 	if( SSL_CTX_use_certificate_chain_file(s.ctx,cfile.c_str()) != 1 )
 	{
 	    ERR_error_string_n(ERR_peek_last_error(),err,sizeof(err));
 	    throw TError(s.nodePath().c_str(),"SSL_CTX_use_certificate_chain_file: %s",err);
 	}
-	//-- Load private key --
+	//>> Load private key
 	if( SSL_CTX_use_PrivateKey_file(s.ctx,cfile.c_str(),SSL_FILETYPE_PEM) != 1 )
 	{
 	    ERR_error_string_n(ERR_peek_last_error(),err,sizeof(err));
 	    throw TError(s.nodePath().c_str(),"SSL_CTX_use_PrivateKey_file: %s",err);
 	}
 
-	//- Remove temporary certificate file -
+	//> Remove temporary certificate file
 	remove(cfile.c_str()); cfile = "";
 
-	//- Create BIO object -
+	//> Create BIO object
 	if( (bio=BIO_new_ssl(s.ctx,0)) == NULL )
 	{
 	    ERR_error_string_n(ERR_peek_last_error(),err,sizeof(err));
@@ -369,7 +351,7 @@ void *TSocketIn::Task( void *sock_in )
 	BIO_set_accept_bios(abio, bio);
 	BIO_set_bind_mode(abio,BIO_BIND_REUSEADDR);
 
-	//- Sets up accept BIO -
+	//> Sets up accept BIO
 	if( BIO_do_accept(abio) <= 0 )
 	{
 	    ERR_error_string_n(ERR_peek_last_error(),err,sizeof(err));
@@ -380,7 +362,7 @@ void *TSocketIn::Task( void *sock_in )
 	s.endrun	= false;
 	s.endrun_cl	= false;
 
-	//- Select mode -
+	//> Select mode
 	struct  timeval tv;
 	fd_set  rd_fd;
 	while( !s.endrun )
@@ -401,7 +383,7 @@ void *TSocketIn::Task( void *sock_in )
 	    BIO *cbio = BIO_pop(abio);
 
 	    if( s.maxFork() <= s.opConnCnt() )	{ s.clsConnByLim++; /*BIO_reset(cbio);*/ close(BIO_get_fd(cbio,NULL)); BIO_free(cbio); }
-	    //- Make client's socket thread -
+	    //> Make client's socket thread
 	    else if( pthread_create( &th, &pthr_attr, ClTask, new SSockIn(&s,cbio) ) < 0 )
 	    {
 		mess_err(s.nodePath().c_str(),_("Error creation of the thread!"));
@@ -412,16 +394,18 @@ void *TSocketIn::Task( void *sock_in )
 	}
     }catch(TError err)	{ s.stErr = err.mess; mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 
-    //- Client tasks stop command -
+    //> Client tasks stop command
     s.endrun_cl = true;
     TSYS::eventWait( s.cl_free, true, string(MOD_ID)+": "+s.id()+" client tasks is stopping....");
 
-    //- Free context -
+    //> Free context
     if( abio )	BIO_reset(abio);
     if( bio )	BIO_free_all(bio);
     if( s.ctx )	{ SSL_CTX_free(s.ctx); s.ctx = NULL; }
     if( !cfile.empty() ) remove(cfile.c_str());
 
+    pthread_attr_destroy(&pthr_attr);
+
     s.run_st = false;
 
     return NULL;
@@ -465,17 +449,24 @@ void *TSocketIn::ClTask( void *s_inf )
     }
 
     int sock_fd = BIO_get_fd(s.bio,NULL);
+    BIO_get_ssl(s.bio,&ssl);
 
     //> Select mode
     struct  timeval tv;
     fd_set  rd_fd;
-    while( !s.s->endrun_cl )
-    {
-	tv.tv_sec  = 0; tv.tv_usec = STD_WAIT_DELAY*1000;
-	FD_ZERO(&rd_fd); FD_SET(sock_fd,&rd_fd);
+    int cnt = 0;		//> Requests counter
+    int tm = time(NULL);	//> Last connection time
 
-	int kz = select(sock_fd+1,&rd_fd,NULL,NULL,&tv);
-	if( kz == 0 || (kz == -1 && errno == EINTR) || kz < 0 || !FD_ISSET(sock_fd,&rd_fd) ) continue;
+    do
+    {
+	if( !SSL_pending(ssl) )
+	{
+	    tv.tv_sec  = 0; tv.tv_usec = STD_WAIT_DELAY*1000;
+	    FD_ZERO(&rd_fd); FD_SET(sock_fd,&rd_fd);
+
+	    int kz = select(sock_fd+1,&rd_fd,NULL,NULL,&tv);
+	    if( kz == 0 || (kz == -1 && errno == EINTR) || kz < 0 || !FD_ISSET(sock_fd,&rd_fd) ) continue;
+	}
 
 	rez = BIO_read(s.bio,buf,sizeof(buf));
 	if( rez == 0 )	break;		//Connection closed by client
@@ -491,10 +482,16 @@ void *TSocketIn::ClTask( void *s_inf )
 #if OSC_DEBUG >= 4
             mess_debug(s.s->nodePath().c_str(),_("The message is replied with the size <%d>."),answ.size());
 #endif
-	    rez = BIO_write(s.bio,answ.data(),answ.size()); s.s->trOut += (float)rez/1024;
+	    do { rez=BIO_write(s.bio,answ.data(),answ.size()); } while( rez < 0 && SSL_get_error(ssl,rez) == SSL_ERROR_WANT_WRITE );
+	    s.s->trOut += (float)answ.size()/1024;
 	    answ = "";
+	    cnt++;
+	    tm = time(NULL);
 	}
     }
+    while( !s.s->endrun_cl &&
+		(!s.s->keepAliveCon() || cnt < s.s->keepAliveCon()) &&
+		(!s.s->keepAliveTm() || (time(NULL)-tm) < s.s->keepAliveTm()) );
 
     BIO_flush(s.bio);
     close(sock_fd);
@@ -589,44 +586,64 @@ void TSocketIn::clientUnreg( pthread_t thrid )
 
 void TSocketIn::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TTransportIn::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","root",2,"tp","str","help",
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),RWRWR_,"root","Transport",2,"tp","str","help",
 	    _("SSL input transport has address format:\n"
 	    "  [addr]:[port]:[mode] - where:\n"
 	    "    addr - address for SSL to be opened, '*' address opens for all interfaces;\n"
 	    "    port - network port (/etc/services);\n"
 	    "    mode - SSL mode and version (SSLv2, SSLv3, SSLv23 and TLSv1)."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/certKey",_("Certificates and private key"),0664,"root","root",4,"tp","str","cols","90","rows","7","help",_("SSL PAM certificates chain and private key."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/pkey_pass",_("Private key password"),0664,"root","root",1,"tp","str");
-	ctrMkNode("fld",opt,-1,"/prm/cfg/cl_n",_("Clients maximum"),0664,"root","root",1,"tp","dec");
-	ctrMkNode("fld",opt,-1,"/prm/cfg/bf_ln",_("Input buffer (kbyte)"),0664,"root","root",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/certKey",_("Certificates and private key"),RWRWR_,"root","Transport",4,
+	    "tp","str","cols","90","rows","7","help",_("SSL PAM certificates chain and private key."));
+	ctrMkNode("fld",opt,-1,"/prm/cfg/pkey_pass",_("Private key password"),RWRWR_,"root","Transport",1,"tp","str");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/cl_n",_("Clients maximum"),RWRWR_,"root","Transport",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/bf_ln",_("Input buffer (kbyte)"),RWRWR_,"root","Transport",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/keepAliveCon",_("Keep alive connections"),RWRWR_,"root","Transport",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/keepAliveTm",_("Keep alive timeout (s)"),RWRWR_,"root","Transport",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/taskPrior",_("Priority"),RWRWR_,"root","Transport",1,"tp","dec");
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/prm/cfg/certKey" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(certKey());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setCertKey(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(certKey());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setCertKey(opt->text());
     }
     else if( a_path == "/prm/cfg/pkey_pass" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(string(pKeyPass().size(),'*'));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setPKeyPass(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(string(pKeyPass().size(),'*'));
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setPKeyPass(opt->text());
     }
     else if( a_path == "/prm/cfg/cl_n" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( TSYS::int2str(maxFork()) );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setMaxFork( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(maxFork()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setMaxFork( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/prm/cfg/bf_ln" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( TSYS::int2str(bufLen()) );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setBufLen( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(bufLen()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setBufLen( atoi(opt->text().c_str()) );
     }
+    else if( a_path == "/prm/cfg/keepAliveCon" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(keepAliveCon()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setKeepAliveCon( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/prm/cfg/keepAliveTm" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(keepAliveTm()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setKeepAliveTm( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/prm/cfg/taskPrior" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(taskPrior()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setTaskPrior( atoi(opt->text().c_str()) );
+    }
+
     else TTransportIn::cntrCmdProc(opt);
 }
 
@@ -634,9 +651,10 @@ void TSocketIn::cntrCmdProc( XMLNode *opt )
 //* TSocketOut                                   *
 //************************************************
 TSocketOut::TSocketOut(string name, const string &idb, TElem *el) :
-    TTransportOut(name,idb,el), mCertKey(cfg("SSLCertKey").getSd()), mKeyPass(cfg("SSLKeyPass").getSd())
+    TTransportOut(name,idb,el), mAPrms(cfg("A_PRMS").getSd())
 {
     setAddr("localhost:10042");
+    setTimings("5:1");
 }
 
 TSocketOut::~TSocketOut()
@@ -644,6 +662,14 @@ TSocketOut::~TSocketOut()
     if( startStat() )	stop();
 }
 
+void TSocketOut::setTimings( const string &vl )
+{
+    mTimings = vl;
+    mTmCon = vmin(60000,(int)(atof(TSYS::strParse(timings(),0,":").c_str())*1e3));
+    mTmNext = vmin(60000,(int)(atof(TSYS::strParse(timings(),1,":").c_str())*1e3));
+    modif();
+}
+
 string TSocketOut::getStatus( )
 {
     string rez = TTransportOut::getStatus( );
@@ -653,6 +679,33 @@ string TSocketOut::getStatus( )
     return rez;
 }
 
+void TSocketOut::load_( )
+{
+    TTransportOut::load_();
+
+    try
+    {
+	XMLNode prmNd;
+	string  vl;
+	prmNd.load(mAPrms);
+	if( prmNd.childGet("CertKey",0,true) ) setCertKey( prmNd.childGet("CertKey")->text() );
+	vl = prmNd.attr("PKeyPass");	if( !vl.empty() ) setPKeyPass(vl);
+	vl = prmNd.attr("TMS");		if( !vl.empty() ) setTimings(vl);
+    } catch(...){ }
+}
+
+void TSocketOut::save_( )
+{
+    XMLNode prmNd("prms");
+    if( prmNd.childGet("CertKey",0,true) ) prmNd.childGet("CertKey")->setText(certKey());
+    else prmNd.childAdd("CertKey")->setText(certKey());
+    prmNd.setAttr("PKeyPass",pKeyPass());
+    prmNd.setAttr("TMS",timings());
+    mAPrms = prmNd.save(XMLNode::BrAllPast);
+
+    TTransportOut::save_();
+}
+
 void TSocketOut::start()
 {
     string	cfile;
@@ -670,7 +723,12 @@ void TSocketOut::start()
     string ssl_method = TSYS::strSepParse(addr(),2,':');
 
     //> Set SSL method
-    SSL_METHOD *meth = SSLv23_client_method();
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L
+    const SSL_METHOD *meth;
+#else
+    SSL_METHOD *meth;
+#endif
+    meth = SSLv23_client_method();
     if( ssl_method == "SSLv2" )		meth = SSLv2_client_method();
     else if( ssl_method == "SSLv3" )	meth = SSLv3_client_method();
     else if( ssl_method == "TLSv1" )	meth = TLSv1_client_method();
@@ -754,7 +812,7 @@ void TSocketOut::stop()
 
     if( !run_st ) return;
 
-    //- Status clear -
+    //> Status clear
     trIn = trOut = 0;
 
     //SSL deinit
@@ -766,29 +824,32 @@ void TSocketOut::stop()
     run_st = false;
 }
 
-int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, int time )
+int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, int time, bool noRes )
 {
-    int		ret = 0;
+    int		ret = 0, reqTry = 0;;
     char	err[255];
+    bool	writeReq = false;
 
-    if( !time ) time = 5000;
+    if( !noRes ) ResAlloc resN( nodeRes(), true );
     ResAlloc res( wres, true );
 
     if( !run_st ) throw TError(nodePath().c_str(),_("Transport is not started!"));
 
+repeate:
+    if( reqTry ) usleep(500000);
+    if( reqTry++ >= 3 )	throw TError(nodePath().c_str(),_("Connection error"));
     //> Write request
     if( obuf != NULL && len_ob > 0 )
     {
-	if( (ret=BIO_write(conn,obuf,len_ob)) != len_ob )
-	{
-	    if( ret == 0 )	{ res.release(); stop(); return 0; }
-	    else
-	    {
-		ERR_error_string_n(ERR_peek_last_error(),err,sizeof(err));
-		throw TError(nodePath().c_str(),"BIO_write: %s",err);
-	    }
-	}
+	do { ret=BIO_write(conn,obuf,len_ob); } while( ret < 0 && SSL_get_error(ssl,ret) == SSL_ERROR_WANT_WRITE );
+	if( ret <= 0 ) { res.release(); stop(); start(); res.request(true); goto repeate; }
+
+	if( !time ) time = mTmCon;
+	writeReq = true;
     }
+    else time = mTmNext;
+    if( !time ) time = 5000;
+
     trOut += (float)ret/1024;
 #if OSC_DEBUG >= 4
     if( ret > 0 ) mess_debug(nodePath().c_str(),_("The message is sent with the size <%d>."),ret);
@@ -797,10 +858,10 @@ int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, in
     //> Read reply
     if( ibuf != NULL && len_ib > 0 )
     {
-	ret=BIO_read(conn,ibuf,len_ib);
+	ret = BIO_read(conn,ibuf,len_ib);
 	if( ret > 0 ) trIn += (float)ret/1024;
-	else if( ret == 0 ) { res.release(); stop(); return 0; }
-	else if( ret < 0 && SSL_get_error(ssl,ret) != SSL_ERROR_WANT_READ )
+	else if( ret == 0 ) { res.release(); stop(); start(); res.request(true); goto repeate; }
+	else if( ret < 0 && SSL_get_error(ssl,ret) != SSL_ERROR_WANT_READ && SSL_get_error(ssl,ret) != SSL_ERROR_WANT_WRITE )
 	{
 	    ERR_error_string_n(ERR_peek_last_error(),err,sizeof(err));
 	    throw TError(nodePath().c_str(),"BIO_read: %s",err);
@@ -812,7 +873,6 @@ int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, in
 	    fd_set rd_fd;
 	    struct timeval tv;
 	    int sock_fd = BIO_get_fd(conn,NULL);
-
 	    do
 	    {
 		tv.tv_sec  = time/1000; tv.tv_usec = 1000*(time%1000);
@@ -820,12 +880,12 @@ int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, in
 		kz = select(sock_fd+1,&rd_fd,NULL,NULL,&tv);
 	    }
 	    while( kz == -1 && errno == EINTR );
-	    if( kz == 0 )	{ res.release(); stop(); throw TError(nodePath().c_str(),_("Timeouted!")); }
-	    else if( kz < 0)	{ res.release(); stop(); throw TError(nodePath().c_str(),_("Socket error!")); }
+	    if( kz == 0 ) { res.release(); if(writeReq) stop(); throw TError(nodePath().c_str(),_("Timeouted!")); }
+	    else if( kz < 0) { res.release(); stop(); throw TError(nodePath().c_str(),_("Socket error!")); }
 	    else if( FD_ISSET(sock_fd, &rd_fd) )
 	    {
 		ret = BIO_read(conn,ibuf,len_ib);
-		if( ret < 0 )	{ res.release(); stop(); throw TError(nodePath().c_str(),_("Read reply error: %s"),strerror(errno)); }
+		if( ret < 0 ) { res.release(); stop(); start(); res.request(true); goto repeate; }
 		trIn += (float)ret/1024;
 	    }
 	}
@@ -840,31 +900,41 @@ int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, in
 
 void TSocketOut::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TTransportOut::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","root",2,"tp","str","help",
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),RWRWR_,"root","Transport",2,"tp","str","help",
 	    _("SSL output transport has address format:\n"
 	    "  [addr]:[port]:[mode] - where:\n"
 	    "    addr - remote SSL host address;\n"
 	    "    port - network port (/etc/services);\n"
 	    "    mode - SSL mode and version (SSLv2, SSLv3, SSLv23 and TLSv1)."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/certKey",_("Certificates and private key"),0660,"root","root",4,"tp","str","cols","90","rows","7","help",_("SSL PAM certificates chain and private key."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/pkey_pass",_("Private key password"),0660,"root","root",1,"tp","str");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/certKey",_("Certificates and private key"),RWRW__,"root","Transport",4,"tp","str","cols","90","rows","7",
+	    "help",_("SSL PAM certificates chain and private key."));
+	ctrMkNode("fld",opt,-1,"/prm/cfg/pkey_pass",_("Private key password"),RWRW__,"root","Transport",1,"tp","str");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/TMS",_("Timings"),RWRWR_,"root","Transport",2,"tp","str","help",
+	    _("Connection timings in format: \"[conn]:[next]\". Where:\n"
+	    "    conn - maximum time for connection respond wait, in seconds;\n"
+	    "    next - maximum time for continue respond wait, in seconds."));
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/prm/cfg/certKey" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText(certKey());
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	setCertKey(opt->text());
+	if( ctrChkNode(opt,"get",RWRW__,"root","Transport",SEC_RD) )	opt->setText(certKey());
+	if( ctrChkNode(opt,"set",RWRW__,"root","Transport",SEC_WR) )	setCertKey(opt->text());
     }
     else if( a_path == "/prm/cfg/pkey_pass" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText(string(pKeyPass().size(),'*'));
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	setPKeyPass(opt->text());
+	if( ctrChkNode(opt,"get",RWRW__,"root","Transport",SEC_RD) )	opt->setText(string(pKeyPass().size(),'*'));
+	if( ctrChkNode(opt,"set",RWRW__,"root","Transport",SEC_WR) )	setPKeyPass(opt->text());
+    }
+    if( a_path == "/prm/cfg/TMS" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(timings());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setTimings(opt->text());
     }
     else TTransportOut::cntrCmdProc(opt);
 }
