@@ -1,8 +1,8 @@
 
 //OpenSCADA system module Transport.Sockets file: socket.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2010 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -46,7 +46,7 @@
 #define MOD_NAME	"Sockets"
 #define MOD_TYPE	"Transport"
 #define VER_TYPE	VER_TR
-#define VERSION		"1.4.0"
+#define VERSION		"1.4.2"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow sockets based transport. Support inet and unix sockets. Inet socket use TCP and UDP protocols."
 #define LICENSE		"GPL"
@@ -100,48 +100,15 @@ void TTransSock::postEnable( int flag )
 
     if( flag&TCntrNode::NodeConnect )
     {
-	//> Add self DB-fields to input transport
-	owner().inEl().fldAdd( new TFld("BufLen",_("Input buffer length (kB)"),TFld::Integer,0,"4","5") );
-	owner().inEl().fldAdd( new TFld("MaxClients",_("Maximum clients process"),TFld::Integer,0,"3","10") );
-	owner().inEl().fldAdd( new TFld("SocketsMaxQueue",_("Maximum queue of input socket"),TFld::Integer,0,"2","10") );
-
-	//> Add self DB-fields to output transport
-	owner().outEl().fldAdd( new TFld("TMS",_("Timeout (ms)"),TFld::String,0,"30") );
+	owner().inEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
+	owner().outEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
     }
 }
 
-string TTransSock::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TTransSock::load_( )
 {
     //> Load parameters from command line
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
 }
 
 TTransportIn *TTransSock::In( const string &name, const string &idb )
@@ -154,28 +121,13 @@ TTransportOut *TTransSock::Out( const string &name, const string &idb )
     return new TSocketOut(name,idb,&owner().outEl());
 }
 
-void TTransSock::cntrCmdProc( XMLNode *opt )
-{
-    //- Get page info -
-    if( opt->name() == "info" )
-    {
-	TTipTransport::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
-	return;
-    }
-
-    //- Process command to page -
-    string a_path = opt->attr("path");
-    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )   opt->setText(optDescr());
-    else TTipTransport::cntrCmdProc(opt);
-}
-
 //************************************************
 //* TSocketIn                                    *
 //************************************************
 TSocketIn::TSocketIn( string name, const string &idb, TElem *el ) :
-    TTransportIn(name,idb,el), cl_free(true), max_queue(cfg("SocketsMaxQueue").getId()),
-    mMaxFork(cfg("MaxClients").getId()), mBufLen(cfg("BufLen").getId())
+    TTransportIn(name,idb,el), cl_free(true),
+    mMaxQueue(10), mMaxFork(10), mBufLen(5), mKeepAliveCon(100), mKeepAliveTm(5), mTaskPrior(0),
+    mAPrms(cfg("A_PRMS").getSd())
 {
     setAddr("TCP:localhost:10002:0");
 }
@@ -196,17 +148,49 @@ string TSocketIn::getStatus( )
     return rez;
 }
 
+void TSocketIn::load_( )
+{
+    TTransportIn::load_();
+
+    try
+    {
+	XMLNode prmNd;
+	string  vl;
+	prmNd.load(mAPrms);
+	vl = prmNd.attr("MaxQueue");	if( !vl.empty() ) mMaxQueue = atoi(vl.c_str());
+	vl = prmNd.attr("MaxClients");	if( !vl.empty() ) mMaxFork = atoi(vl.c_str());
+	vl = prmNd.attr("BufLen");	if( !vl.empty() ) mBufLen = atoi(vl.c_str());
+	vl = prmNd.attr("KeepAliveCnt");if( !vl.empty() ) mKeepAliveCon = atoi(vl.c_str());
+	vl = prmNd.attr("KeepAliveTm");	if( !vl.empty() ) mKeepAliveTm = atoi(vl.c_str());
+	vl = prmNd.attr("TaskPrior");	if( !vl.empty() ) mTaskPrior = atoi(vl.c_str());
+    } catch(...){ }
+}
+
+void TSocketIn::save_( )
+{
+    XMLNode prmNd("prms");
+    prmNd.setAttr("MaxQueue",TSYS::int2str(mMaxQueue));
+    prmNd.setAttr("MaxClients",TSYS::int2str(mMaxFork));
+    prmNd.setAttr("BufLen",TSYS::int2str(mBufLen));
+    prmNd.setAttr("KeepAliveCnt",TSYS::int2str(mKeepAliveCon));
+    prmNd.setAttr("KeepAliveTm",TSYS::int2str(mKeepAliveTm));
+    prmNd.setAttr("TaskPrior",TSYS::int2str(mTaskPrior));
+    mAPrms = prmNd.save(XMLNode::BrAllPast);
+
+    TTransportIn::save_();
+}
+
 void TSocketIn::start()
 {
     pthread_attr_t pthr_attr;
 
     if( run_st ) return;
 
-    //- Status clear -
+    //> Status clear
     trIn = trOut = 0;
     connNumb = clsConnByLim = 0;
 
-    //- Socket init -
+    //> Socket init
     string s_type = TSYS::strSepParse(addr(),0,':');
 
     if( s_type == S_NM_TCP )
@@ -263,7 +247,7 @@ void TSocketIn::start()
 		close( sock_fd );
 		throw TError(nodePath().c_str(),_("TCP socket doesn't bind to <%s>!"),addr().c_str());
 	    }
-	    listen(sock_fd,max_queue);
+	    listen(sock_fd,mMaxQueue);
 	}
 	else if(type == SOCK_UDP )
 	{
@@ -273,11 +257,11 @@ void TSocketIn::start()
 	    else if( htons(atol(port.c_str())) > 0 ) name_in.sin_port = htons( atol(port.c_str()) );
 	    else name_in.sin_port = 10001;
 
-	    if( bind(sock_fd,(sockaddr *)&name_in,sizeof(name_in) ) == -1) 
+	    if( bind(sock_fd,(sockaddr *)&name_in,sizeof(name_in) ) == -1)
 	    {
 		shutdown( sock_fd,SHUT_RDWR );
 		close( sock_fd );
-                throw TError(nodePath().c_str(),_("UDP socket doesn't bind to <%s>!"),addr().c_str());
+		throw TError(nodePath().c_str(),_("UDP socket doesn't bind to <%s>!"),addr().c_str());
 	    }
 	}
     }
@@ -294,34 +278,26 @@ void TSocketIn::start()
 	if( bind(sock_fd,(sockaddr *)&name_un,sizeof(name_un) ) == -1)
 	{
 	    close( sock_fd );
-            throw TError(nodePath().c_str(),_("UNIX socket doesn't bind to <%s>!"),addr().c_str());
+	    throw TError(nodePath().c_str(),_("UNIX socket doesn't bind to <%s>!"),addr().c_str());
 	}
-	listen(sock_fd,max_queue);
+	listen(sock_fd,mMaxQueue);
     }
 
-    pthread_attr_init(&pthr_attr);
-    pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-    pthread_create(&pthr_tsk,&pthr_attr,Task,this);
-    pthread_attr_destroy(&pthr_attr);
-    if( TSYS::eventWait( run_st, true,nodePath()+"open",5) )
-	throw TError(nodePath().c_str(),_("Not opened!"));
+    SYS->taskCreate( nodePath('.',true), taskPrior(), Task, this, &run_st );
 }
 
 void TSocketIn::stop()
 {
     if( !run_st ) return;
 
-    //- Status clear -
+    //> Status clear
     trIn = trOut = 0;
     connNumb = clsConnByLim = 0;
 
-    endrun = true;
-    if( TSYS::eventWait( run_st, false, nodePath()+"close",5) )
-	throw TError(nodePath().c_str(),_("Not closed!"));
-    pthread_join( pthr_tsk, NULL );
+    SYS->taskDestroy( nodePath('.',true), &run_st, &endrun );
 
     shutdown(sock_fd,SHUT_RDWR);
-    close(sock_fd); 
+    close(sock_fd);
     if( type == SOCK_UNIX ) remove( path.c_str() );
 }
 
@@ -340,7 +316,10 @@ void *TSocketIn::Task(void *sock_in)
     pthread_t      th;
     pthread_attr_t pthr_attr;
     pthread_attr_init(&pthr_attr);
-    pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
+    struct sched_param prior;
+    pthread_attr_setschedpolicy(&pthr_attr,(sock->taskPrior()&&SYS->user()=="root")?SCHED_RR:SCHED_OTHER);
+    prior.__sched_priority = sock->taskPrior();
+    pthread_attr_setschedparam(&pthr_attr,&prior);
     pthread_attr_setdetachstate(&pthr_attr, PTHREAD_CREATE_DETACHED);
 
     sock->run_st    = true;
@@ -418,10 +397,10 @@ void *TSocketIn::Task(void *sock_in)
     pthread_attr_destroy(&pthr_attr);
 
     if( sock->type == SOCK_UDP ) delete []buf;
-    //- Client tasks stop command -
+    //> Client tasks stop command
     sock->endrun_cl = true;
     ResAlloc res(sock->sock_res,false);
-    //- Find already registry -
+    //> Find already registry
     for( int i_id = 0; i_id < sock->cl_id.size(); i_id++)
         pthread_kill(sock->cl_id[i_id].cl_id,SIGALRM);
     res.release();
@@ -435,6 +414,8 @@ void *TSocketIn::Task(void *sock_in)
 void *TSocketIn::ClTask( void *s_inf )
 {
     SSockIn &s = *(SSockIn*)s_inf;
+    int cnt = 0;		//> Requests counter
+    int tm = time(NULL);	//> Last connection time
 
 #if OSC_DEBUG >= 2
     mess_debug(s.s->nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
@@ -462,7 +443,7 @@ void *TSocketIn::ClTask( void *s_inf )
 	int kz = select(s.cSock+1,&rd_fd,NULL,NULL,&tv);
 	if( kz == 0 || (kz == -1 && errno == EINTR) || kz < 0 || !FD_ISSET(s.cSock, &rd_fd) ) continue;
 	r_len = read(s.cSock,buf,s.s->bufLen()*1000);
-	if(r_len <= 0) break;
+	if( r_len <= 0 ) break;
 	s.s->trIn += (float)r_len/1024;
 
 #if OSC_DEBUG >= 5
@@ -476,11 +457,20 @@ void *TSocketIn::ClTask( void *s_inf )
 #if OSC_DEBUG >= 5
 	    mess_debug(s.s->nodePath().c_str(),_("Socket replied message <%d> to <%s>."), answ.size(), s.sender.c_str() );
 #endif
-	    r_len = write(s.cSock,answ.c_str(),answ.size()); s.s->trOut += (float)r_len/1024;
+	    for( int wOff = 0, wL = 1; wOff != answ.size() && wL > 0; wOff += wL )
+	    {
+		wL = write(s.cSock,answ.data()+wOff,answ.size()-wOff);
+		s.s->trOut += (float)wL/1024;
+	    }
 	    answ = "";
+	    cnt++;
+	    tm = time(NULL);
 	}
 	sessOk = true;
-    }while( !s.s->endrun_cl && (!sessOk || s.s->mode || !prot_in.freeStat()) );
+    }while( !s.s->endrun_cl && (!sessOk ||
+	    ((s.s->mode || !prot_in.freeStat()) &&
+	     (!s.s->keepAliveCon() || cnt < s.s->keepAliveCon()) &&
+	     (!s.s->keepAliveTm() || (time(NULL)-tm) < s.s->keepAliveTm())) ) );
 
     //> Close protocol on broken connection
     if( !prot_in.freeStat() )
@@ -508,6 +498,7 @@ void TSocketIn::messPut( int sock, string &request, string &answer, string sende
     string n_pr = id()+TSYS::int2str(sock);
     try
     {
+	if( !SYS->protocol().at().modPresent(protocol()) ) throw TError(nodePath().c_str(),_("The protocol '%s' is not present."),protocol().c_str());
 	proto = SYS->protocol().at().modAt(protocol());
 	if( prot_in.freeStat() )
 	{
@@ -530,7 +521,7 @@ void TSocketIn::messPut( int sock, string &request, string &answer, string sende
 void TSocketIn::clientReg( pthread_t thrid, int i_sock )
 {
     ResAlloc res(sock_res,true);
-    //- Find already registry -
+    //> Find already registry
     for( unsigned i_id = 0; i_id < cl_id.size(); i_id++)
 	if( cl_id[i_id].cl_id == thrid ) return;
     SSockCl scl = { thrid, i_sock };
@@ -558,7 +549,7 @@ void TSocketIn::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TTransportIn::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","root",2,"tp","str","help",
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),RWRWR_,"root","Transport",2,"tp","str","help",
 	    _("Socket's input transport has address format:\n"
 	    "  TCP:[addr]:[port]:[mode] - TCP socket:\n"
 	    "    addr - address for socket to be opened, empty address opens socket for all interfaces;\n"
@@ -570,27 +561,45 @@ void TSocketIn::cntrCmdProc( XMLNode *opt )
 	    "  UNIX:[name]:[mode] - UNIX socket:\n"
 	    "    name - UNIX-socket's file name;\n"
 	    "    mode - work mode (0 - break connection; 1 - keep alive)."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/q_ln",_("Queue length"),0664,"root","root",2,"tp","dec","help",_("Used for TCP and UNIX sockets."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/cl_n",_("Clients maximum"),0664,"root","root",2,"tp","dec","help",_("Used for TCP and UNIX sockets."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/bf_ln",_("Input buffer (kbyte)"),0664,"root","root",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/qLn",_("Queue length"),RWRWR_,"root","Transport",2,"tp","dec","help",_("Used for TCP and UNIX sockets."));
+	ctrMkNode("fld",opt,-1,"/prm/cfg/clMax",_("Clients maximum"),RWRWR_,"root","Transport",2,"tp","dec","help",_("Used for TCP and UNIX sockets."));
+	ctrMkNode("fld",opt,-1,"/prm/cfg/bfLn",_("Input buffer (kbyte)"),RWRWR_,"root","Transport",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/keepAliveCon",_("Keep alive connections"),RWRWR_,"root","Transport",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/keepAliveTm",_("Keep alive timeout (s)"),RWRWR_,"root","Transport",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/taskPrior",_("Priority"),RWRWR_,"root","Transport",1,"tp","dec");
 	return;
     }
     //- Process command to page -
     string a_path = opt->attr("path");
-    if( a_path == "/prm/cfg/q_ln" )
+    if( a_path == "/prm/cfg/qLn" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( TSYS::int2str(maxQueue()) );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setMaxQueue( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(maxQueue()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setMaxQueue( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/prm/cfg/cl_n" )
+    else if( a_path == "/prm/cfg/clMax" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( TSYS::int2str(maxFork()) );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setMaxFork( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(maxFork()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setMaxFork( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/prm/cfg/bf_ln" )
+    else if( a_path == "/prm/cfg/bfLn" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( TSYS::int2str(bufLen()) );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setBufLen( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(bufLen()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setBufLen( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/prm/cfg/keepAliveCon" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(keepAliveCon()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setKeepAliveCon( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/prm/cfg/keepAliveTm" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(keepAliveTm()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setKeepAliveTm( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/prm/cfg/taskPrior" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(taskPrior()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setTaskPrior( atoi(opt->text().c_str()) );
     }
     else TTransportIn::cntrCmdProc(opt);
 }
@@ -598,10 +607,11 @@ void TSocketIn::cntrCmdProc( XMLNode *opt )
 //************************************************
 //* TSocketOut                                   *
 //************************************************
-TSocketOut::TSocketOut(string name, const string &idb, TElem *el) : TTransportOut(name,idb,el), sock_fd(-1)
+TSocketOut::TSocketOut(string name, const string &idb, TElem *el) : TTransportOut(name,idb,el), sock_fd(-1),
+    mAPrms(cfg("A_PRMS").getSd())
 {
     setAddr("TCP:localhost:10002");
-    setTimeout(1000);
+    setTimings("5:1");
 }
 
 TSocketOut::~TSocketOut()
@@ -609,6 +619,14 @@ TSocketOut::~TSocketOut()
     if( startStat() )	stop();
 }
 
+void TSocketOut::setTimings( const string &vl )
+{
+    mTimings = vl;
+    mTmCon = vmin(60000,(int)(atof(TSYS::strParse(timings(),0,":").c_str())*1e3));
+    mTmNext = vmin(60000,(int)(atof(TSYS::strParse(timings(),1,":").c_str())*1e3));
+    modif();
+}
+
 string TSocketOut::getStatus( )
 {
     string rez = TTransportOut::getStatus( );
@@ -618,9 +636,27 @@ string TSocketOut::getStatus( )
     return rez;
 }
 
-int TSocketOut::timeout( )	{ return vmax(1,vmin(100000,cfg("TMS").getI())); }
+void TSocketOut::load_( )
+{
+    TTransportOut::load_();
 
-void TSocketOut::setTimeout( int vl )	{ cfg("TMS").setI(vl); modif(); }
+    try
+    {
+	XMLNode prmNd;
+	string  vl;
+	prmNd.load(mAPrms);
+	vl = prmNd.attr("tms");	if( !vl.empty() ) setTimings(vl);
+    } catch(...){ }
+}
+
+void TSocketOut::save_( )
+{
+    XMLNode prmNd("prms");
+    prmNd.setAttr("tms",timings());
+    mAPrms = prmNd.save(XMLNode::BrAllPast);
+
+    TTransportOut::save_();
+}
 
 void TSocketOut::start()
 {
@@ -682,7 +718,7 @@ void TSocketOut::start()
 	    struct timeval tv;
 	    socklen_t slen;
 	    fd_set fdset;
-	    tv.tv_sec = timeout()/1000; tv.tv_usec = 1000*(timeout()%1000);
+	    tv.tv_sec = tmCon()/1000; tv.tv_usec = 1000*(tmCon()%1000);
 	    FD_ZERO( &fdset ); FD_SET( sock_fd, &fdset );
 	    if( (res=select( sock_fd+1, NULL, &fdset, NULL, &tv )) > 0 && !getsockopt(sock_fd,SOL_SOCKET,SO_ERROR,&res,&slen) && !res ) res = 0;
 	    else res = -1;
@@ -693,21 +729,6 @@ void TSocketOut::start()
 	    sock_fd = -1;
 	    throw TError(nodePath().c_str(),_("Connect to Internet socket error: %s!"),strerror(errno));
 	}
-
-	/*time_t wTm = time(NULL);
-	int res = -1;
-	do
-	{
-	    res = ::connect(sock_fd, (sockaddr *)&name_in, sizeof(name_in));
-	    if( res == 0 || (res == -1 && !(errno == EINPROGRESS || errno == EALREADY)) ) break;
-	    usleep(10000);
-	}while( (time(NULL)-wTm) < timeout()/1000 );
-	if( res == -1 )
-	{
-	    close(sock_fd);
-	    sock_fd = -1;
-	    throw TError(nodePath().c_str(),_("Connect to Internet socket error: %s!"),strerror(errno));
-	}*/
     }
     else if( type == SOCK_UNIX )
     {
@@ -717,7 +738,7 @@ void TSocketOut::start()
 	name_un.sun_family = AF_UNIX;
 	strncpy( name_un.sun_path,path.c_str(),sizeof(name_un.sun_path) );
 	
-	//- Create socket -
+	//> Create socket
 	if( (sock_fd = socket(PF_UNIX,SOCK_STREAM,0) )== -1)
 	    throw TError(nodePath().c_str(),_("Error creation UNIX socket: %s!"),strerror(errno));
 	if( ::connect(sock_fd, (sockaddr *)&name_un, sizeof(name_un)) == -1 )
@@ -748,26 +769,42 @@ void TSocketOut::stop()
     run_st = false;
 }
 
-int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, int time )
+int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, int time, bool noRes )
 {
     int kz = 0, reqTry = 0;
+    bool writeReq = false;
 
+    if( !noRes ) ResAlloc resN( nodeRes(), true );
     ResAlloc res( wres, true );
-    int prevTmOut = timeout( );
-    if( time ) setTimeout(time);
+
+    int prevTmOut = 0;
+    if( time ) { prevTmOut = tmCon(); setTmCon(time); }
 
     if( !run_st ) throw TError(nodePath().c_str(),_("Transport is not started!"));
 
 repeate:
     if( reqTry++ >= 2 ) throw TError(nodePath().c_str(),_("Read reply error: %s"),strerror(errno));
     //> Write request
-    if( obuf != NULL && len_ob > 0 && (kz=write(sock_fd,obuf,len_ob)) <= 0 )
+    if( obuf != NULL && len_ob > 0 )
     {
-	res.release();
-	stop(); start();
-	res.request(true);
-	goto repeate;
+	for( int wOff = 0; wOff != len_ob; wOff += kz )
+	{
+	    kz = write(sock_fd,obuf+wOff,len_ob-wOff);
+	    if( kz <= 0 )
+	    {
+		res.release();
+		stop(); start();
+		res.request(true);
+		goto repeate;
+	    }
+	}
+
+	if( !time ) time = mTmCon;
+	writeReq = true;
     }
+    else time = mTmNext;
+    if( !time ) time = 5000;
+
     trOut += (float)kz/1024;
 
     //> Read reply
@@ -779,22 +816,22 @@ repeate:
 
 	do
 	{
-	    tv.tv_sec  = timeout()/1000; tv.tv_usec = 1000*(timeout()%1000);
+	    tv.tv_sec  = time/1000; tv.tv_usec = 1000*(time%1000);
 	    FD_ZERO(&rd_fd); FD_SET(sock_fd,&rd_fd);
 	    kz = select(sock_fd+1,&rd_fd,NULL,NULL,&tv);
 	}
 	while( kz == -1 && errno == EINTR );
-	if( kz == 0 )	{ res.release(); stop( ); throw TError(nodePath().c_str(),_("Timeouted!")); }
+	if( kz == 0 )	{ res.release(); if(writeReq) stop( ); throw TError(nodePath().c_str(),_("Timeouted!")); }
 	else if( kz < 0){ res.release(); stop( ); throw TError(nodePath().c_str(),_("Socket error!")); }
 	else if( FD_ISSET(sock_fd, &rd_fd) )
 	{
 	    i_b = read(sock_fd,ibuf,len_ib);
-	    if(i_b <= 0) { res.release(); stop(); start(); res.request(true); goto repeate; }
+	    if( i_b <= 0 ) { res.release(); stop(); start(); res.request(true); goto repeate; }
 	    trIn += (float)i_b/1024;
 	}
     }
 
-    if( time ) setTimeout(prevTmOut);
+    if( prevTmOut ) setTmCon(prevTmOut);
 
     return i_b;
 }
@@ -805,7 +842,7 @@ void TSocketOut::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TTransportOut::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","root",2,"tp","str","help",
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),RWRWR_,"root","Transport",2,"tp","str","help",
 	    _("Socket's output transport has address format:\n"
 	    "  TCP:[addr]:[port] - TCP socket:\n"
 	    "    addr - address for remote socket to be opened;\n"
@@ -815,16 +852,19 @@ void TSocketOut::cntrCmdProc( XMLNode *opt )
 	    "    port - network port (/etc/services).\n"
 	    "  UNIX:[name] - UNIX socket:\n"
 	    "    name - UNIX-socket's file name."));
-	ctrMkNode("fld",opt,-1,"/prm/cfg/tmOut",_("Timeout (ms)"),0664,"root","root",1,"tp","dec");
+	ctrMkNode("fld",opt,-1,"/prm/cfg/TMS",_("Timings"),RWRWR_,"root","Transport",2,"tp","str","help",
+	    _("Connection timings in format: \"[conn]:[next]\". Where:\n"
+	    "    conn - maximum time for connection respond wait, in seconds;\n"
+	    "    next - maximum time for continue respond wait, in seconds."));
 	return;
     }
 
     //> Process command to page
     string a_path = opt->attr("path");
-    if( a_path == "/prm/cfg/tmOut" )
+    if( a_path == "/prm/cfg/TMS" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TSYS::int2str(timeout()));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setTimeout(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(timings());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setTimings(opt->text());
     }
     else TTransportOut::cntrCmdProc(opt);
 }
