@@ -19,6 +19,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include <QApplication>
 #include <QPainter>
 #include <QPaintEvent>
 #include <QVBoxLayout>
@@ -32,6 +33,9 @@
 #include <QItemEditorFactory>
 #include <QMetaProperty>
 #include <QSpinBox>
+#include <QCalendarWidget>
+#include <QToolTip>
+#include <QStatusBar>
 
 #include <tsys.h>
 
@@ -100,41 +104,42 @@ void ImgView::paintEvent( QPaintEvent * )
 //* QTimeEdit, QDateEdit and QDateTimeEdit.                                                   *
 //*********************************************************************************************
 LineEdit::LineEdit( QWidget *parent, LType tp, bool prev_dis ) :
-    QWidget( parent ), m_tp((LineEdit::LType)-1), bt_fld(NULL), ed_fld(NULL)
+    QWidget( parent ), m_tp((LineEdit::LType)-1), bt_fld(NULL), ed_fld(NULL), mPrev(!prev_dis)
 {
     QHBoxLayout *box = new QHBoxLayout(this);
     box->setMargin( 0 );
     box->setSpacing( 0 );
 
-    if( !prev_dis )
+    setType( tp );
+}
+
+void LineEdit::viewApplyBt( bool view )
+{
+    if( view == (bool)bt_fld ) return;
+
+    if( view && !bt_fld )
     {
 	bt_fld = new QPushButton(this);
 	bt_fld->setIcon( QIcon(":/images/ok.png") );
 	bt_fld->setIconSize( QSize(12,12) );
 	bt_fld->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed) );
 	//bt_fld->setMaximumWidth( 15 );
-	bt_fld->setEnabled( false );
-	bt_fld->setVisible( false );
-	connect( bt_fld, SIGNAL( pressed() ), this, SLOT( applySlot() ) );
-	box->addWidget( bt_fld );
+	connect( bt_fld, SIGNAL( clicked() ), this, SLOT( applySlot() ) );
+	layout()->addWidget( bt_fld );
     }
-    setType( tp );
+    if( !view && bt_fld ) { bt_fld->deleteLater(); bt_fld = NULL; }
 }
 
-bool LineEdit::isEdited( )
-{
-    if( bt_fld && bt_fld->isVisible() )	return true;
-    return false;
-}
+bool LineEdit::isEdited( )	{ return bt_fld; }
 
 void LineEdit::setType( LType tp )
 {
     if( tp == m_tp ) return;
 
-    //- Delete previous -
+    //> Delete previous
     if( tp >= 0 && ed_fld ) delete ed_fld;
 
-    //- Create new widget -
+    //> Create new widget
     switch( tp )
     {
 	case Text:
@@ -155,10 +160,16 @@ void LineEdit::setType( LType tp )
 	    break;
 	case Date:
 	    ed_fld = new QDateEdit(this);
+	    ((QDateEdit*)ed_fld)->setCalendarPopup(true);
+	    ((QDateEdit*)ed_fld)->calendarWidget()->setGridVisible(true);
+	    ((QDateEdit*)ed_fld)->calendarWidget()->setFirstDayOfWeek(Qt::Monday);
 	    connect( (QDateEdit*)ed_fld, SIGNAL( dateChanged(const QDate&) ), SLOT( changed() ) );
 	    break;
 	case DateTime:
 	    ed_fld = new QDateTimeEdit(this);
+	    ((QDateTimeEdit*)ed_fld)->setCalendarPopup(true);
+	    ((QDateTimeEdit*)ed_fld)->calendarWidget()->setGridVisible(true);
+	    ((QDateTimeEdit*)ed_fld)->calendarWidget()->setFirstDayOfWeek(Qt::Monday);
 	    connect( (QDateTimeEdit*)ed_fld, SIGNAL( dateTimeChanged(const QDateTime&) ), SLOT( changed() ) );
 	    break;
 	case Combo:
@@ -176,13 +187,8 @@ void LineEdit::setType( LType tp )
 
 void LineEdit::changed( )
 {
-    //- Enable apply
-    if( bt_fld && !bt_fld->isEnabled() )
-    {
-	bt_fld->setEnabled(true);
-	bt_fld->setVisible(true);
-	//QWidget::setTabOrder( mod->getFocusedWdg(ed_fld), mod->getFocusedWdg(bt_fld) );
-    }
+    //> Enable apply
+    if( mPrev && !bt_fld )	viewApplyBt(true);
 
     emit valChanged(value());
 }
@@ -217,11 +223,7 @@ void LineEdit::setValue(const QString &txt)
 
     m_val = txt;
 
-    if( bt_fld && bt_fld->isEnabled() )
-    {
-        bt_fld->setEnabled(false);
-        bt_fld->setVisible(false);
-    }
+    if( bt_fld ) viewApplyBt(false);
 }
 
 void LineEdit::setCfg(const QString &cfg)
@@ -281,11 +283,7 @@ void LineEdit::setCfg(const QString &cfg)
 	    break;
 	}
     }
-    if( bt_fld && bt_fld->isEnabled() )
-    {
-	bt_fld->setEnabled(false);
-	bt_fld->setVisible(false);
-    }
+    if( bt_fld ) viewApplyBt(false);
 }
 
 QString LineEdit::value()
@@ -305,19 +303,18 @@ QString LineEdit::value()
 
 void LineEdit::applySlot( )
 {
-    emit valChanged(value());
-
-    bt_fld->setEnabled(false);
-    bt_fld->setVisible(false);
+    viewApplyBt(false);
 
+    if( m_val == value() ) return;
     m_val = value();
 
+    emit valChanged(value());
     emit apply();
 }
 
 bool LineEdit::event( QEvent * e )
 {
-    if( e->type() == QEvent::KeyRelease && bt_fld && bt_fld->isEnabled() )
+    if( e->type() == QEvent::KeyRelease && bt_fld )
     {
 	QKeyEvent *keyEvent = (QKeyEvent *)e;
 	if( keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return )
@@ -347,7 +344,10 @@ TextEdit::TextEdit( QWidget *parent, const char *name, bool prev_dis ) :
     box->setSpacing(0);
 
     ed_fld = new QTextEdit(this);
+    ed_fld->setTabStopWidth(20);
+    ed_fld->setAcceptRichText(false);
     connect( ed_fld, SIGNAL( textChanged() ), this, SLOT( changed() ) );
+    connect( ed_fld, SIGNAL( cursorPositionChanged() ), this, SLOT( curPosChange() ) );
     box->addWidget(ed_fld);
 
     if( !prev_dis )
@@ -413,6 +413,30 @@ void TextEdit::btCancel( )
     emit cancel();
 }
 
+void TextEdit::curPosChange( )
+{
+    ((QMainWindow*)window())->statusBar()->showMessage(QString(_("Cursor = (%1:%2)")).arg(ed_fld->textCursor().blockNumber()+1).arg(ed_fld->textCursor().columnNumber()+1),10000);
+}
+
+bool TextEdit::event( QEvent *e )
+{
+    if( but_box && e->type() == QEvent::KeyRelease )
+    {
+	QKeyEvent *keyEvent = (QKeyEvent *)e;
+	if( (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return) && QApplication::keyboardModifiers()&Qt::ControlModifier )
+	{
+	    but_box->button(QDialogButtonBox::Apply)->animateClick( );
+	    return true;
+	}
+	else if( keyEvent->key() == Qt::Key_Escape )
+	{
+	    but_box->button(QDialogButtonBox::Cancel)->animateClick( );
+	    return true;
+	}
+    }
+    return QWidget::event(e);
+}
+
 //*************************************************
 //* InputDlg: Id and name input dialog.           *
 //*************************************************
@@ -738,12 +762,12 @@ QWidget *TableDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem
     return w_del;
 }
 
-void TableDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
+void TableDelegate::setEditorData( QWidget *editor, const QModelIndex &index ) const
 {
     QVariant value = index.data(Qt::DisplayRole);
     QVariant val_user = index.data(Qt::UserRole);
 
-    if(dynamic_cast<QComboBox*>(editor))
+    if( dynamic_cast<QComboBox*>(editor) )
     {
 	QComboBox *comb = dynamic_cast<QComboBox*>(editor);
 	if(value.type() == QVariant::Bool)	comb->setCurrentIndex(value.toBool());
@@ -754,7 +778,7 @@ void TableDelegate::setEditorData(QWidget *editor, const QModelIndex &index) con
 	}
 	return;
     }
-    if(dynamic_cast<QLineEdit*>(editor))
+    if( dynamic_cast<QLineEdit*>(editor) )
     {
 	QLineEdit *led = dynamic_cast<QLineEdit*>(editor);
 	led->setText(value.toString());
@@ -762,7 +786,7 @@ void TableDelegate::setEditorData(QWidget *editor, const QModelIndex &index) con
     }
 }
 
-void TableDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
+void TableDelegate::setModelData( QWidget *editor, QAbstractItemModel *model, const QModelIndex &index ) const
 {
     if(dynamic_cast<QComboBox*>(editor))
     {
@@ -781,18 +805,18 @@ void TableDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, con
     }
 }
 
-void TableDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &) const
+void TableDelegate::updateEditorGeometry( QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex & ) const
 {
     editor->setGeometry(option.rect);
 }
 
-bool TableDelegate::eventFilter(QObject *object, QEvent *event)
+bool TableDelegate::eventFilter( QObject *object, QEvent *event )
 {
-    if(dynamic_cast<QComboBox*>(object))
+    if( dynamic_cast<QComboBox*>(object) )
     {
 	QComboBox *comb = dynamic_cast<QComboBox*>(object);
-	if(event->type() == QEvent::KeyRelease)
-	    switch (static_cast<QKeyEvent *>(event)->key()) 
+	if( event->type() == QEvent::KeyRelease )
+	    switch( static_cast<QKeyEvent *>(event)->key() )
 	    {
 		case Qt::Key_Enter:
 		case Qt::Key_Return:
