@@ -71,6 +71,7 @@ class LineEdit : public QWidget
 	void setCfg( const QString& );
 
 	QWidget *workWdg( )	{ return ed_fld; }
+	void viewApplyBt( bool view );
 
     signals:
 	void apply( );
@@ -87,6 +88,7 @@ class LineEdit : public QWidget
     private:
 	LType		m_tp;
 	QString		m_val;
+	bool		mPrev;
 	QWidget		*ed_fld;
 	QPushButton	*bt_fld;
 };
@@ -116,11 +118,15 @@ class TextEdit : public QWidget
 	void cancel( );
 	void textChanged( const QString& );
 
+    protected:
+	bool event( QEvent * e );
+
     private slots:
 	//Private slots
 	void changed( );
 	void btApply( );
 	void btCancel( );
+	void curPosChange( );
 
     private:
 	//Private attributes
@@ -272,7 +278,7 @@ class TableDelegate : public QItemDelegate
     Q_OBJECT
 
     public:
-	//- Public attributes -
+	//> Public attributes
 	TableDelegate( QObject *parent = 0 );
 
 	void paint( QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index ) const;
@@ -285,7 +291,7 @@ class TableDelegate : public QItemDelegate
 	void updateEditorGeometry( QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index ) const;
 
     private:
-	//- Private attributes -
+	//> Private attributes
 	bool eventFilter( QObject *object, QEvent *event );
 };
 
