@@ -34,6 +34,7 @@
 #include <QTextCodec>
 #include <QTimer>
 #include <QSplashScreen>
+#include <QLocale>
 
 #include <tsys.h>
 #include <tmess.h>
@@ -45,7 +46,7 @@
 #define MOD_NAME	"QT GUI starter"
 #define MOD_TYPE	"UI"
 #define VER_TYPE	VER_UI
-#define VERSION		"1.5.3"
+#define VERSION		"1.5.4"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow QT GUI starter. It is single for all QT GUI modules!"
 #define LICENSE		"GPL"
@@ -105,7 +106,8 @@ TUIMod::TUIMod( string name ) : end_run(false), demon_mode(false), start_com(fal
 	"Duplex Printing","None","Long side","Short side","Options","&Options >>","&Options <<","&Print","Print to File (PDF)","Print to File (Postscript)",
 	"Local file","Write %1 file","Paper","Page size:","Width:","Height:","Paper source:","Orientation","Portrait","Landscape","Reverse landscape",
 	"Reverse portrait","Margins","top margin","left margin","right margin","bottom margin","Points (pt)","Inches (in)",
-	"Millimeters (mm)","Centimeters (cm)","Page","Advanced"
+	"Millimeters (mm)","Centimeters (cm)","Page","Advanced",
+	"Mon"
     };
 #endif
 }
@@ -121,10 +123,10 @@ void TUIMod::postEnable( int flag )
 
     if( flag&TCntrNode::NodeConnect )
     {
-	//- Set QT environments -
+	//> Set QT environments
 	QTextCodec::setCodecForCStrings( QTextCodec::codecForLocale () ); //codepage for QT across QString recode!
 
-	//- Check command line for options no help and no daemon -
+	//> Check command line for options no help and no daemon
 	bool isHelp = false;
 	int next_opt;
 	const char *short_opt="h";
@@ -147,31 +149,19 @@ void TUIMod::postEnable( int flag )
 	    }
 	} while(next_opt != -1);
 
-	//- Start main QT thread if no help and no daemon -
+	//> Start main QT thread if no help and no daemon
 	if( !(run_st || demon_mode || isHelp) )
 	{
 	    end_run = false;
 
-	    pthread_attr_t pthr_attr;
-	    pthread_attr_init(&pthr_attr);
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	    pthread_create(&pthr_tsk,&pthr_attr,Task,this);
-	    pthread_attr_destroy(&pthr_attr);
-	    if( TSYS::eventWait( run_st, true, nodePath()+"start",5) )
-		throw TError(nodePath().c_str(),_("QT main thread is not started!"));
+	    SYS->taskCreate( nodePath('.',true), 0, Task, this, &run_st );
 	}
     }
 }
 
 void TUIMod::postDisable( int flag )
 {
-    if( run_st )
-    {
-	end_run = true;
-	if( TSYS::eventWait( run_st, false, nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("QT main thread is not stopped!"));
-	pthread_join(pthr_tsk,NULL);
-    }
+    if( run_st ) SYS->taskDestroy( nodePath('.',true), &run_st, &end_run );
 }
 
 void TUIMod::load_( )
@@ -196,7 +186,7 @@ void TUIMod::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case 'd': demon_mode = true; break;
 	    case -1 : break;
 	}
@@ -258,6 +248,9 @@ void *TUIMod::Task( void * )
     mess_debug(mod->nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
 #endif
 
+    //> Init locale setLocale
+    QLocale::setDefault(QLocale(Mess->lang().c_str()));
+
     //> QT application object init
     QApplication *QtApp = new QApplication( (int&)SYS->argc,(char **)SYS->argv );
     QtApp->setQuitOnLastWindowClosed(false);
@@ -338,22 +331,31 @@ void *TUIMod::Task( void * )
 
 void TUIMod::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TUI::cntrCmdProc(opt);
 	if(ctrMkNode("area",opt,1,"/prm/cfg",_("Module options")))
-	    ctrMkNode("fld",opt,-1,"/prm/cfg/st_mod",_("Start QT modules (sep - ';')"),0660,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/prm/cfg/st_mod",_("Start QT modules (sep - ';')"),0660,"root","root",3,"tp","str","dest","sel_ed","select","/prm/cfg/lsQTmod");
 	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
 	return;
     }
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/prm/cfg/st_mod" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText( startMod() );
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	setStartMod( opt->text() );
+	if( ctrChkNode(opt,"get",0660,"root","root",SEC_RD) )	opt->setText( startMod() );
+	if( ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )	setStartMod( opt->text() );
+    }
+    else if( a_path == "/prm/cfg/lsQTmod" && ctrChkNode(opt) )
+    {
+	vector<string> list;
+	mod->owner().modList(list);
+	for( unsigned i_l = 0; i_l < list.size(); i_l++ )
+	    if( mod->owner().modAt(list[i_l]).at().modInfo("SubType") == "QT" &&
+		    mod->owner().modAt(list[i_l]).at().modFuncPresent("QMainWindow *openWindow();") )
+		opt->childAdd("el")->setText(list[i_l]);
     }
     else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )	opt->setText(optDescr());
     else TUI::cntrCmdProc(opt);
@@ -436,7 +438,7 @@ bool WinControl::callQTModule( const string &nm )
 	act_1->setObjectName(list[i_l].c_str());
 	//act_1->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_1);
 	act_1->setToolTip(qt_mod.at().modName().c_str());
-	act_1->setWhatsThis(qt_mod.at().modInfo("Descript").c_str());
+	act_1->setWhatsThis(qt_mod.at().modInfo("Description").c_str());
 	QObject::connect(act_1, SIGNAL(activated()), this, SLOT(callQTModule()));
 
 	if( toolBar ) toolBar->addAction(act_1);
