@@ -53,7 +53,7 @@ TCntrNode &WidgetLib::operator=( TCntrNode &node )
     WidgetLib *src_n = dynamic_cast<WidgetLib*>(&node);
     if( !src_n ) return *this;
 
-    //- Copy generic configuration -
+    //> Copy generic configuration
     string tid = mId;
     *(TConfig*)this = *(TConfig*)src_n;
     mId  = tid;
@@ -64,7 +64,7 @@ TCntrNode &WidgetLib::operator=( TCntrNode &node )
     if( !enable() ) setEnable(true);
 
     vector<string> pls;
-    //- Mime data copy -
+    //> Mime data copy
     src_n->mimeDataList(pls);
     string mimeType, mimeData;
     for( int i_m = 0; i_m < pls.size(); i_m++ )
@@ -73,7 +73,7 @@ TCntrNode &WidgetLib::operator=( TCntrNode &node )
 	mimeDataSet( pls[i_m], mimeType, mimeData );
     }
 
-    //- Copy include pages -
+    // Copy include pages
     src_n->list(pls);
     for( int i_p = 0; i_p < pls.size(); i_p++ )
     {
@@ -158,7 +158,7 @@ void WidgetLib::save_( )
 
     SYS->db().at().dataSet(DB()+"."+mod->wlbTable(),mod->nodePath()+"LIB/",*this);
 
-    //- Check for need copy mime data to other DB and same copy -
+    //> Check for need copy mime data to other DB and same copy
     if( !mOldDB.empty() && mOldDB != TBDS::realDBName(DB()) )
     {
 	vector<string> pls;
@@ -209,7 +209,7 @@ bool WidgetLib::mimeDataGet( const string &iid, string &mimeType, string *mimeDa
 
     if( !is_file )
     {
-	//- Get resource file from DB -
+	//> Get resource file from DB
 	string dbid = is_res ? iid.substr(4) : iid;
 	string wtbl = tbl()+"_mime";
 	string wdb  = idb.empty() ? DB() : idb;
@@ -225,7 +225,7 @@ bool WidgetLib::mimeDataGet( const string &iid, string &mimeType, string *mimeDa
     }
     if( !is_res )
     {
-	//- Get resource file from file system -
+	//> Get resource file from file system
 	string filepath = is_file ? iid.substr(5) : iid;
 	int len;
 	char buf[STR_BUF_LEN];
@@ -285,7 +285,7 @@ AutoHD<LWidget> WidgetLib::at( const string &id )
 
 void WidgetLib::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TCntrNode::cntrCmdProc(opt);
@@ -321,57 +321,57 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
 	return;
     }
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/obj/st/en" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( TSYS::int2str(enable()) );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setEnable( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( TSYS::int2str(enable()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setEnable( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/obj/st/db" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( fullDB() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setFullDB( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( fullDB() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setFullDB( opt->text() );
     }
     else if( a_path == "/obj/cfg/ico" || a_path == "/ico" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( ico() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setIco( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( ico() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setIco( opt->text() );
     }
     else if( a_path == "/obj/cfg/id" && ctrChkNode(opt,"get",R_R_R_,"root","UI") ) opt->setText(id());
     else if( a_path == "/obj/cfg/name" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( name() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setName( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( name() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setName( opt->text() );
     }
     else if( a_path == "/obj/cfg/descr" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( descr() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setDescr( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( descr() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setDescr( opt->text() );
     }
     else if( a_path == "/br/wdg_" || a_path == "/wdg/wdg" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	{
 	    vector<string> lst;
 	    list(lst);
 	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
 		opt->childAdd("el")->setAttr("id",lst[i_f])->setText(at(lst[i_f]).at().name());
 	}
-	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    add(vid,opt->text().c_str()); at(vid).at().setOwner(opt->attr("user"));
 	}
-	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEQ_WR) ) del(opt->attr("id"),true);
+	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEC_WR) ) del(opt->attr("id"),true);
     }
     else if( a_path == "/mime/mime" )
     {
-	//-- Request data --
+	//>> Request data
 	string idmime = opt->attr("key_id");
 	string idcol  = opt->attr("col");
 
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	{
 	    if( !idmime.empty() && idcol == "dt" && atoi(opt->attr("data").c_str()) )
 	    {
@@ -396,15 +396,15 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
 		    }
 	    }
 	}
-	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEQ_WR) )	mimeDataSet("newMime","image/new;0","");
-	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEQ_WR) )	mimeDataDel(opt->attr("key_id"));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEC_WR) )	mimeDataSet("newMime","image/new;0","");
+	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEC_WR) )	mimeDataDel(opt->attr("key_id"));
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )
 	{
-	    //-- Request data --
+	    //>> Request data
 	    if( idcol == "id" )
 	    {
 		string mimeType, mimeData;
-		//--- Copy mime data to new record ---
+		//>>> Copy mime data to new record
 		if( mimeDataGet( "res:"+idmime, mimeType, &mimeData ) )
 		{
 		    mimeDataSet( opt->text(), mimeType, mimeData );
@@ -414,7 +414,7 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
 	    else if( idcol == "tp" )
 	    {
 		string mimeType;
-		//--- Copy mime data to new record ---
+		//>>> Copy mime data to new record
 		if( mimeDataGet( "res:"+idmime, mimeType ) )
 		    mimeDataSet( idmime, opt->text()+";"+TSYS::strSepParse(mimeType,1,';'), "");
 	    }
@@ -562,7 +562,7 @@ int LWidget::calcPer( )
 
 void LWidget::setCalcLang( const string &ilng )
 {
-    m_proc = ilng+"\n"+calcProg();
+    m_proc = ilng.empty() ? "" : ilng+"\n"+calcProg();
     modif();
 }
 
@@ -578,6 +578,13 @@ void LWidget::setCalcPer( int vl )
     modif();
 }
 
+void LWidget::setParentNm( const string &isw )
+{
+    if( enable() && mParent != isw ) setEnable(false);
+    mParent = isw;
+    modif();
+}
+
 void LWidget::load_( )
 {
     if( !SYS->chkSelDB(ownerLib().DB()) ) return;
@@ -594,7 +601,7 @@ void LWidget::load_( )
     {
 	if( !attrPresent(als[i_a]) ) continue;
 	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( attr.at().modif() && m_attrs.find(als[i_a]) == string::npos )
+	if( attr.at().modif() && m_attrs.find(als[i_a]+";") == string::npos )
 	{
 	    attr.at().setModif(0);
 	    inheritAttr(als[i_a]);
@@ -602,24 +609,9 @@ void LWidget::load_( )
     }
 
     //> Load generic attributes
-    tbl = tbl+"_io";
-    TConfig c_el(&mod->elWdgIO());
-    c_el.cfg("IDW").setS(id());
-    string tstr;
-    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
-    {
-	if( !attrPresent(tstr) )    continue;
-	AutoHD<Attr> attr = attrAt(tstr);
-	if( !(attr.at().flgGlob()&Attr::Generic) ) continue;
-	c_el.cfg("ID").setS(tstr);
-	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
-	attr.at().setS(c_el.cfg("IO_VAL").getS(),true);
-	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
-	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
-	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-    }
+    mod->attrsLoad( *this, db+"."+tbl, cfg("DBV").getI(), id(), "", m_attrs, true );
 
-    //> Other attributes load
+    //> Load all other attributes
     loadIO();
 }
 
@@ -628,52 +620,13 @@ void LWidget::loadIO( )
     if( !enable() ) return;
 
     //> Load widget's work attributes
-    string db  = ownerLib().DB();
-    string tbl = ownerLib().tbl()+"_io";
-
-    //> Same attributes load
-    TConfig c_el(&mod->elWdgIO());
-    c_el.cfg("IDW").setS(id());
-    string tstr;
-    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
-    {
-	if( !attrPresent(tstr) )    continue;
-	AutoHD<Attr> attr = attrAt(tstr);
-	if( attr.at().flgGlob()&Attr::Generic || (!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
-	c_el.cfg("ID").setS(tstr);
-	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
-	attr.at().setS(c_el.cfg("IO_VAL").getS());
-	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
-	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
-	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-    }
-
-    //> Load widget's user attributes
-    tbl = ownerLib().tbl()+"_uio";
-    c_el.setElem(&mod->elWdgUIO());
-    c_el.cfg("IDW").setS(id(),true);
-    for( int fld_cnt = 0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
-    {
-	string sid = c_el.cfg("ID").getS();
-	unsigned flg = c_el.cfg("IO_TYPE").getI();
-
-	if( !TSYS::pathLev(sid,1).empty() ) continue;
-	if( !attrPresent(sid) )
-	    attrAdd( new TFld(sid.c_str(),c_el.cfg("NAME").getS().c_str(),(TFld::Type)(flg&0x0f),flg>>4) );
-	AutoHD<Attr> attr = attrAt(sid);
-	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
-	attr.at().setS(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),0,'|'));
-	attr.at().fld().setValues(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),1,'|'));
-	attr.at().fld().setSelNames(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),2,'|'));
-	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
-	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
-	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-    }
+    mod->attrsLoad( *this, ownerLib().DB()+"."+ownerLib().tbl(), cfg("DBV").getI(), id(), "", m_attrs );
 
     //> Load cotainer widgets
-    if( !enable() || !isContainer() ) return;
-    c_el.setElem(&mod->elInclWdg());
-    tbl=ownerLib().tbl()+"_incl";
+    if( !isContainer() ) return;
+    TConfig c_el(&mod->elInclWdg());
+    string db  = ownerLib().DB();
+    string tbl = ownerLib().tbl()+"_incl";
     c_el.cfg("IDW").setS(id(),true);
     for( int fld_cnt=0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
     {
@@ -683,7 +636,9 @@ void LWidget::loadIO( )
 	    if( wdgPresent(sid) )	wdgDel(sid);
 	    continue;
 	}
-	if( !wdgPresent(sid) ) wdgAdd(sid,"","");
+	if( !wdgPresent(sid) )
+	    try{ wdgAdd(sid,"",""); }
+	    catch(TError err){ mess_err(err.cat.c_str(),err.mess.c_str()); }
 
 	wdgAt(sid).at().load();
     }
@@ -693,30 +648,13 @@ void LWidget::save_( )
 {
     string db  = ownerLib().DB();
     string tbl = ownerLib().tbl();
-    string tbl_io = tbl+"_io";
+
+    cfg("DBV").setI(2);
+
+    //> Save generic attributes
+    m_attrs = mod->attrsSave( *this, db+"."+tbl, cfg("DBV").getI(), id(), "", true );
 
     //> Save generic widget's data
-    m_attrs="";
-    vector<string> als;
-    attrList( als );
-    TConfig c_el(&mod->elWdgIO());
-    c_el.cfg("IDW").setS(id());
-    for( int i_a = 0; i_a < als.size(); i_a++ )
-    {
-	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( !attr.at().modif() ) continue;
-	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) m_attrs+=als[i_a]+";";
-	if( attr.at().flgGlob()&Attr::Generic )
-	{
-	    c_el.cfg("ID").setS(als[i_a]);
-	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
-	    c_el.cfg("IO_VAL").setS(attr.at().getS());
-	    c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
-	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
-	    c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
-	    SYS->db().at().dataSet(db+"."+tbl_io,mod->nodePath()+tbl_io,c_el);
-	}
-    }
     SYS->db().at().dataSet( db+"."+tbl, mod->nodePath()+tbl, *this );
 
     //> Save widget's attributes
@@ -725,63 +663,10 @@ void LWidget::save_( )
 
 void LWidget::saveIO( )
 {
-    vector<string> als;
-
     if( !enable() ) return;
 
     //> Save widget's attributes
-    string db  = ownerLib().DB();
-    string tbl = ownerLib().tbl()+"_io";
-    string utbl = ownerLib().tbl()+"_uio";
-
-    attrList( als );
-    TConfig c_el(&mod->elWdgIO());
-    c_el.cfg("IDW").setS(id());
-    TConfig c_elu(&mod->elWdgUIO());
-    c_elu.cfg("IDW").setS(id(),true);
-    for( int i_a = 0; i_a < als.size(); i_a++ )
-    {
-	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( !attr.at().modif() || attr.at().flgGlob()&Attr::Generic )	continue;
-	if( !(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser )
-	{
-	    //>> User attribute store
-	    c_elu.cfg("ID").setS(als[i_a]);
-	    c_elu.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
-	    c_elu.cfg("IO_VAL").setS(attr.at().getS()+"|"+
-				    attr.at().fld().values()+"|"+
-				    attr.at().fld().selNames());
-	    c_elu.cfg("NAME").setS(attr.at().name());
-	    c_elu.cfg("IO_TYPE").setI(attr.at().fld().type()+(attr.at().fld().flg()<<4));
-	    c_elu.cfg("SELF_FLG").setI(attr.at().flgSelf());
-	    c_elu.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
-	    c_elu.cfg("CFG_VAL").setS(attr.at().cfgVal());
-	    SYS->db().at().dataSet(db+"."+utbl,mod->nodePath()+utbl,c_elu);
-	}
-	else
-	{
-	    //>> Work attribute store
-	    c_el.cfg("ID").setS(als[i_a]);
-	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
-	    c_el.cfg("IO_VAL").setS(attr.at().getS());
-	    c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
-	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
-	    c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
-	    SYS->db().at().dataSet(db+"."+tbl,mod->nodePath()+tbl,c_el);
-	}
-    }
-
-    //> Clear no present IO for user io table
-    c_elu.cfgViewAll(false);
-    for( int fld_cnt=0; SYS->db().at().dataSeek(db+"."+utbl,mod->nodePath()+utbl,fld_cnt++,c_elu); )
-    {
-	string sid = c_elu.cfg("ID").getS();
-	if( TSYS::pathLev(sid,1).empty() && !attrPresent(TSYS::pathLev(sid,0)) )
-	{
-	    SYS->db().at().dataDel(db+"."+utbl,mod->nodePath()+utbl,c_elu,true);
-	    fld_cnt--;
-	}
-    }
+    mod->attrsSave( *this, ownerLib().DB()+"."+ownerLib().tbl(), cfg("DBV").getI(), id(), "" );
 }
 
 void LWidget::wClear( )
@@ -807,7 +692,7 @@ void LWidget::wdgAdd( const string &wid, const string &name, const string &path,
 	if( SYS->db().at().dataGet( db+"."+tbl, mod->nodePath()+tbl, c_el ) && c_el.cfg("PARENT").getS() == "<deleted>" )
 	{
 	    if( !parent().at().wdgPresent(wid) )	SYS->db().at().dataDel( db+"."+tbl, mod->nodePath()+tbl, c_el, true );
-	    return;
+	    else throw TError(nodePath().c_str(),_("You try to create widget with name '%s' of the widget that was the early inherited and deleted from base container!"),wid.c_str());
 	}
     }
 
@@ -930,6 +815,13 @@ void CWidget::setOwner( const string &iown )
     modif();
 }
 
+void CWidget::setParentNm( const string &isw )
+{
+    if( enable() && mParent != isw ) setEnable(false);
+    mParent = isw;
+    modif();
+}
+
 string CWidget::grp( )
 {
     return SYS->security().at().grpPresent(m_grp) ? m_grp : Widget::grp( );
@@ -941,13 +833,12 @@ void CWidget::setEnable( bool val )
 
     Widget::setEnable(val);
 
-    //- Disable heritors widgets -
+    //> Enable heritors widgets
     if( val )
 	for( int i_h = 0; i_h < ownerLWdg().herit().size(); i_h++ )
-	    if( ownerLWdg().herit()[i_h].at().wdgPresent(id()) && !ownerLWdg().herit()[i_h].at().wdgAt(id()).at().enable( ) )
-	    try { ownerLWdg().herit()[i_h].at().wdgAt(id()).at().setEnable(true); }
-	    catch(...)
-            { mess_err(nodePath().c_str(),_("Inheriting widget <%s> enable error"),id().c_str()); }
+	    if( !ownerLWdg().herit()[i_h].at().wdgAt(id()).at().enable( ) && ownerLWdg().herit()[i_h].at().wdgPresent(id()) )
+		try { ownerLWdg().herit()[i_h].at().wdgAt(id()).at().setEnable(true); }
+		catch(...) { mess_err(nodePath().c_str(),_("Inheriting widget <%s> enable error"),id().c_str()); }
 }
 
 string CWidget::calcId( )
@@ -990,7 +881,7 @@ void CWidget::load_( )
     {
 	if( !attrPresent(als[i_a]) ) continue;
 	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( attr.at().modif() && m_attrs.find(als[i_a]) == string::npos )
+	if( attr.at().modif() && m_attrs.find(als[i_a]+";") == string::npos )
 	{
 	    attr.at().setModif(0);
 	    inheritAttr(als[i_a]);
@@ -998,24 +889,9 @@ void CWidget::load_( )
     }
 
     //> Load generic attributes
-    tbl = ownerLWdg().ownerLib().tbl()+"_io";
-    TConfig c_el(&mod->elWdgIO());
-    c_el.cfg("IDW").setS(ownerLWdg().id());
-    string tstr;
-    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
-    {
-	if( !attrPresent(tstr) ) continue;
-	AutoHD<Attr> attr = attrAt(tstr);
-	if( !(attr.at().flgGlob()&Attr::Generic) ) continue;
-	c_el.cfg("ID").setS(id()+"/"+tstr);
-	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
-	attr.at().setS(c_el.cfg("IO_VAL").getS(),true);
-	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
-	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
-	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-    }
+    mod->attrsLoad( *this, db+"."+ownerLWdg().ownerLib().tbl(), cfg("DBV").getI(), ownerLWdg().id(), id(), m_attrs, true );
 
-    //> Load widget's attributes
+    //> Load all other attributes
     loadIO();
 }
 
@@ -1024,55 +900,13 @@ void CWidget::loadIO( )
     if( !enable() ) return;
 
     //> Load widget's work attributes
-    string db  = ownerLWdg().ownerLib().DB();
-    string tbl = ownerLWdg().ownerLib().tbl()+"_io";
-
-    //> Same load
-    TConfig c_el(&mod->elWdgIO());
-    c_el.cfg("IDW").setS(ownerLWdg().id());
-    string tstr;
-    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
-    {
-	if( !attrPresent(tstr) )    continue;
-	AutoHD<Attr> attr = attrAt(tstr);
-	if( attr.at().flgGlob()&Attr::Generic || (!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
-	c_el.cfg("ID").setS(id()+"/"+tstr);
-	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
-	attr.at().setS(c_el.cfg("IO_VAL").getS());
-	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
-	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
-	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-    }
-    //> Load widget's user attributes
-    tbl = ownerLWdg().ownerLib().tbl()+"_uio";
-    c_el.setElem(&mod->elWdgUIO());
-    c_el.cfg("IDW").setS(ownerLWdg().id(),true);
-    for( int fld_cnt = 0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
-    {
-	string sid = c_el.cfg("ID").getS();
-	unsigned flg = c_el.cfg("IO_TYPE").getI();
-
-	if( TSYS::pathLev(sid,0) == id() && !TSYS::pathLev(sid,1).empty() ) sid = TSYS::pathLev(sid,1);
-	else continue;
-
-	if( !attrPresent(sid) )
-	    attrAdd( new TFld(sid.c_str(),c_el.cfg("NAME").getS().c_str(),(TFld::Type)(flg&0x0f),flg>>4) );
-	AutoHD<Attr> attr = attrAt(sid);
-	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
-	attr.at().setS(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),0,'|'));
-	attr.at().fld().setValues(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),1,'|'));
-	attr.at().fld().setSelNames(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),2,'|'));
-	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
-	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
-	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-    }
+    mod->attrsLoad( *this, ownerLWdg().ownerLib().DB()+"."+ownerLWdg().ownerLib().tbl(), cfg("DBV").getI(), ownerLWdg().id(), id(), m_attrs );
 }
 
 void CWidget::save_( )
 {
     string db  = ownerLWdg().ownerLib().DB();
     string tbl = ownerLWdg().ownerLib().tbl();
-    string tbl_io = tbl+"_io";
 
     //> Delete from DB
     if( nodeMode() == TCntrNode::Disable )
@@ -1086,44 +920,39 @@ void CWidget::save_( )
 	else SYS->db().at().dataDel( db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this, true );
 
 	//>> Remove widget's work and users IO from library IO table
-	TConfig c_el( &mod->elWdgIO() );
-	c_el.cfg("IDW").setS( ownerLWdg().id(), true );
-	for( int i_a = 0; i_a < m_attrs.size(); i_a++ )
+	if( cfg("DBV").getI() == 1 )
 	{
-	    c_el.cfg("ID").setS(id()+"/"+m_attrs[i_a],true);
+	    TConfig c_el( &mod->elWdgIO() );
+	    c_el.cfg("IDW").setS( ownerLWdg().id(), true );
+	    for( int i_a = 0; i_a < m_attrs.size(); i_a++ )
+	    {
+		c_el.cfg("ID").setS(id()+"/"+m_attrs[i_a],true);
+		SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el );
+	    }
+	    c_el.setElem(&mod->elWdgUIO());
+	    c_el.cfg("IDW").setS( ownerLWdg().id(), true );
+	    for( int io_cnt = 0; SYS->db().at().dataSeek( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", io_cnt++, c_el ); )
+		if( c_el.cfg("ID").getS().find(id()+"/") == 0 )
+		{ SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el, true ); io_cnt--; }
+	}
+	if( cfg("DBV").getI() == 2 )
+	{
+	    TConfig c_el( &mod->elWdgIO() );
+	    c_el.cfg("IDW").setS( ownerLWdg().id(), true ); c_el.cfg("IDC").setS( id(), true );
 	    SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el );
+	    c_el.setElem(&mod->elWdgUIO());
+	    c_el.cfg("IDW").setS( ownerLWdg().id(), true ); c_el.cfg("IDC").setS( id(), true );
+	    SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el );
 	}
-	c_el.setElem(&mod->elWdgUIO());
-	c_el.cfg("IDW").setS( ownerLWdg().id(), true );
-	for( int io_cnt = 0; SYS->db().at().dataSeek( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", io_cnt++, c_el ); )
-	    if( c_el.cfg("ID").getS().find(id()+"/") == 0 )
-	    { SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el, true ); io_cnt--; }
     }
     //> Save widget's data
     else
     {
+	cfg("DBV").setI(2);
+	//> Save generic attributes
+	m_attrs = mod->attrsSave( *this, db+"."+tbl, cfg("DBV").getI(), ownerLWdg().id(), id(), true );
+
 	//> Save generic widget's data
-	m_attrs="";
-	vector<string> als;
-	attrList( als );
-	TConfig c_el(&mod->elWdgIO());
-	c_el.cfg("IDW").setS(ownerLWdg().id());
-	for( int i_a = 0; i_a < als.size(); i_a++ )
-	{
-	    AutoHD<Attr> attr = attrAt(als[i_a]);
-	    if( !attr.at().modif() ) continue;
-	    if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) m_attrs+=als[i_a]+";";
-	    if( attr.at().flgGlob()&Attr::Generic )
-	    {
-		c_el.cfg("ID").setS( id()+"/"+als[i_a] );
-		c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
-		c_el.cfg("IO_VAL").setS(attr.at().getS());
-		c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
-		c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
-	        c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
-		SYS->db().at().dataSet(db+"."+tbl_io,mod->nodePath()+tbl_io,c_el);
-	    }
-	}
 	SYS->db().at().dataSet(db+"."+tbl+"_incl",mod->nodePath()+tbl+"_incl",*this);
 
 	//>> Save widget's attributes
@@ -1133,60 +962,9 @@ void CWidget::save_( )
 
 void CWidget::saveIO( )
 {
-    vector<string> als;
+    if( !enable() ) return;
 
-    //- Save widget's attributes -
-    string db  = ownerLWdg().ownerLib().DB();
-    string tbl = ownerLWdg().ownerLib().tbl()+"_io";
-    string utbl = ownerLWdg().ownerLib().tbl()+"_uio";
-
-    attrList( als );
-    TConfig c_el(&mod->elWdgIO());
-    c_el.cfg("IDW").setS(ownerLWdg().id());
-    TConfig c_elu(&mod->elWdgUIO());
-    c_elu.cfg("IDW").setS(ownerLWdg().id(),true);
-    for( int i_a = 0; i_a < als.size(); i_a++ )
-    {
-	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( !attr.at().modif() || attr.at().flgGlob()&Attr::Generic )	continue;
-	if( !(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser )
-	{
-	    //-- User attribute store --
-	    c_elu.cfg("ID").setS( id()+"/"+als[i_a] );
-	    c_elu.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
-	    c_elu.cfg("IO_VAL").setS(attr.at().getS()+"|"+
-				    attr.at().fld().values()+"|"+
-				    attr.at().fld().selNames());
-	    c_elu.cfg("NAME").setS(attr.at().name());
-	    c_elu.cfg("IO_TYPE").setI(attr.at().fld().type()+(attr.at().fld().flg()<<4));
-	    c_elu.cfg("SELF_FLG").setI(attr.at().flgSelf());
-	    c_elu.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
-	    c_elu.cfg("CFG_VAL").setS(attr.at().cfgVal());
-	    SYS->db().at().dataSet(db+"."+utbl,mod->nodePath()+utbl,c_elu);
-	}
-	else
-	{
-	    //-- Work attribute store --
-	    c_el.cfg("ID").setS( id()+"/"+als[i_a] );
-	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
-	    c_el.cfg("IO_VAL").setS(attr.at().getS());
-	    c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
-	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
-	    c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
-	    SYS->db().at().dataSet(db+"."+tbl,mod->nodePath()+tbl,c_el);
-	}
-    }
-    //> Clear no present IO for user io table
-    c_elu.cfgViewAll(false);
-    for( int fld_cnt=0; SYS->db().at().dataSeek(db+"."+utbl,mod->nodePath()+utbl,fld_cnt++,c_elu); )
-    {
-	string sid = c_elu.cfg("ID").getS();
-	if( TSYS::pathLev(sid,0) == id() && TSYS::pathLev(sid,1).size() && !attrPresent(TSYS::pathLev(sid,1)) )
-	{
-	    SYS->db().at().dataDel(db+"."+utbl,mod->nodePath()+utbl,c_elu,true);
-	    fld_cnt--;
-	}
-    }
+    mod->attrsSave( *this, ownerLWdg().ownerLib().DB()+"."+ownerLWdg().ownerLib().tbl(), cfg("DBV").getI(), ownerLWdg().id(), id() );
 }
 
 void CWidget::wClear( )
@@ -1217,7 +995,7 @@ void CWidget::inheritAttr( const string &attr )
 void CWidget::cntrCmdProc( XMLNode *opt )
 {
     if( cntrCmdServ(opt) ) return;
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	cntrCmdGeneric(opt);
