@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.VCAEngine file: origwidg.cpp
 /***************************************************************************
- *   Copyright (C) 2006-2008 by Roman Savochenko                           *
+ *   Copyright (C) 2006-2009 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -57,7 +57,7 @@ void PrWidget::setEnable( bool val )
 
     LWidget::setEnable( val );
 
-    //- Init active attributes -
+    //> Init active attributes
     if( val )
     {
 	vector<string> ls;
@@ -73,7 +73,7 @@ void PrWidget::setEnable( bool val )
 
 bool PrWidget::cntrCmdGeneric( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	Widget::cntrCmdGeneric(opt);
@@ -83,9 +83,9 @@ bool PrWidget::cntrCmdGeneric( XMLNode *opt )
 	return true;
     }
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
-    if( a_path == "/wdg/st/parent" && ctrChkNode(opt,"get",R_R_R_,owner().c_str(),grp().c_str(),SEQ_RD) )
+    if( a_path == "/wdg/st/parent" && ctrChkNode(opt,"get",R_R_R_,owner().c_str(),grp().c_str(),SEC_RD) )
 	opt->setText(parentNm());
     else return Widget::cntrCmdGeneric(opt);
 
@@ -96,7 +96,7 @@ void PrWidget::cntrCmdProc( XMLNode *opt )
 {
     if( cntrCmdServ(opt) ) return;
 
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	cntrCmdGeneric(opt);
@@ -104,7 +104,7 @@ void PrWidget::cntrCmdProc( XMLNode *opt )
 	return;
     }
 
-    //- Process command to page -
+    //> Process command to page
     if( !(cntrCmdGeneric(opt) || cntrCmdAttributes(opt)) )
 	TCntrNode::cntrCmdProc(opt);
 }
@@ -158,7 +158,7 @@ bool OrigElFigure::attrChange( Attr &cfg, TVariant prev )
 	string sel, sel1;
 	string ls_prev = prev.getS();
 	map<int,char> pntls, pntls_prev, wls, wls_prev, clrls, clrls_prev, imgls, imgls_prev, lstls, lstls_prev;
-	//- Parse last attributes list and make point list -
+	//> Parse last attributes list and make point list
 	for( int i_p = 0; i_p < 2; i_p++ )
 	{
 	    string ls_w = (i_p==0)?ls_prev:cfg.getS();
@@ -191,8 +191,8 @@ bool OrigElFigure::attrChange( Attr &cfg, TVariant prev )
 	    }
 	}
 
-	//- Add new dynamic items -
-	//-- Add no present dynamic points --
+	//> Add new dynamic items
+	//>> Add no present dynamic points
 	for( map<int,char>::iterator it = pntls.begin(); it != pntls.end(); ++it )
 	    if( it->first && pntls_prev.find(it->first) == pntls_prev.end() && !cfg.owner()->attrPresent("p"+TSYS::int2str(it->first)+"x") )
 	    {
@@ -201,48 +201,48 @@ bool OrigElFigure::attrChange( Attr &cfg, TVariant prev )
 		cfg.owner()->attrAdd( new TFld(("p"+TSYS::int2str(it->first)+"y").c_str(),(_("Point ")+TSYS::int2str(it->first)+":y").c_str(),
 		    TFld::Real,Attr::Mutable,"","0","","",TSYS::int2str(30+it->first*6+1).c_str()) );
 	    }
-	//-- Add no present dynamic widths --
+	//>> Add no present dynamic widths
 	for( map<int,char>::iterator it = wls.begin(); it != wls.end(); ++it )
 	    if( it->first && wls_prev.find(it->first) == wls_prev.end() && !cfg.owner()->attrPresent("w"+TSYS::int2str(it->first)) )
 		cfg.owner()->attrAdd( new TFld(("w"+TSYS::int2str(it->first)).c_str(),(_("Width ")+TSYS::int2str(it->first)).c_str(),
 		    TFld::Real,Attr::Mutable,"","1","","",TSYS::int2str(30+it->first*6+2).c_str()) );
-	//-- Add no present dynamic colors --
+	//>> Add no present dynamic colors
 	for( map<int,char>::iterator it = clrls.begin(); it != clrls.end(); ++it )
 	    if( it->first && clrls_prev.find(it->first) == clrls_prev.end() && !cfg.owner()->attrPresent("c"+TSYS::int2str(it->first)) )
 		cfg.owner()->attrAdd( new TFld(("c"+TSYS::int2str(it->first)).c_str(),(_("Color ")+TSYS::int2str(it->first)).c_str(),
 		    TFld::String,Attr::Mutable|Attr::Color,"","","","",TSYS::int2str(30+it->first*6+3).c_str()) );
-	//-- Add no present dynamic images --
+	//>> Add no present dynamic images
 	for( map<int,char>::iterator it = imgls.begin(); it != imgls.end(); ++it )
 	    if( it->first && imgls_prev.find(it->first) == imgls_prev.end() && !cfg.owner()->attrPresent("i"+TSYS::int2str(it->first)) )
 		cfg.owner()->attrAdd( new TFld(("i"+TSYS::int2str(it->first)).c_str(),(_("Image ")+TSYS::int2str(it->first)).c_str(),
 		    TFld::String,Attr::Mutable|Attr::Image,"","","","",TSYS::int2str(30+it->first*6+4).c_str()) );
-	//-- Add no present line styles --
+	//>> Add no present line styles
 	for( map<int,char>::iterator it = lstls.begin(); it != lstls.end(); ++it )
 	    if( it->first && lstls_prev.find(it->first) == lstls_prev.end() && !cfg.owner()->attrPresent("s"+TSYS::int2str(it->first)) )
 		cfg.owner()->attrAdd( new TFld(("s"+TSYS::int2str(it->first)).c_str(),(_("Style ")+TSYS::int2str(it->first)).c_str(),
 		    TFld::Integer,Attr::Mutable|TFld::Selected,"","0","0;1;2",_("Solid;Dashed;Dotted"),TSYS::int2str(30+it->first*6+5).c_str()) );
 
-	//- Delete no dynamic items -
-	//-- Delete dynamic points --
+	//> Delete no dynamic items
+	//>> Delete dynamic points
 	for( map<int,char>::iterator it = pntls_prev.begin(); it != pntls_prev.end(); ++it )
 	    if( it->first && pntls.find(it->first) == pntls.end() )
 	    {
 		cfg.owner()->attrDel("p"+TSYS::int2str(it->first)+"x");
 		cfg.owner()->attrDel("p"+TSYS::int2str(it->first)+"y");
 	    }
-	//-- Delete dynamic widths --
+	//>> Delete dynamic widths
 	for( map<int,char>::iterator it = wls_prev.begin(); it != wls_prev.end(); ++it )
 	    if( it->first && wls.find(it->first) == wls.end() )
 		cfg.owner()->attrDel("w"+TSYS::int2str(it->first));
-	//-- Delete dynamic colors --
+	//>> Delete dynamic colors
 	for( map<int,char>::iterator it = clrls_prev.begin(); it != clrls_prev.end(); ++it )
 	    if( it->first && clrls.find(it->first) == clrls.end() )
 		cfg.owner()->attrDel("c"+TSYS::int2str(it->first));
-	//-- Delete dynamic images --
+	//>> Delete dynamic images
 	for( map<int,char>::iterator it = imgls_prev.begin(); it != imgls_prev.end(); ++it )
 	    if( it->first && imgls.find(it->first) == imgls.end() )
 		cfg.owner()->attrDel("i"+TSYS::int2str(it->first));
-	//-- Delete dynamic line styles --
+	//>> Delete dynamic line styles
 	for( map<int,char>::iterator it = lstls_prev.begin(); it != lstls_prev.end(); ++it )
 	    if( it->first && lstls.find(it->first) == lstls.end() )
 		cfg.owner()->attrDel("s"+TSYS::int2str(it->first));
@@ -281,52 +281,53 @@ bool OrigFormEl::attrChange( Attr &cfg, TVariant prev )
 {
     if( cfg.flgGlob()&Attr::Active && cfg.id() == "elType" )
     {
-	//- Delete specific attributes -
-	switch( prev.getI() )
-	{
-	    case 0:	//Line edit
-		cfg.owner()->attrDel("value");
-		cfg.owner()->attrDel("view");
-		cfg.owner()->attrDel("cfg");
-		cfg.owner()->attrDel("font");
-		break;
-	    case 1:	//Text edit
-		cfg.owner()->attrDel("value");
-		cfg.owner()->attrDel("wordWrap");
-		cfg.owner()->attrDel("font");
-		break;
-	    case 2:	//Check box
-		cfg.owner()->attrDel("value");
-		cfg.owner()->attrDel("font");
-		cfg.owner()->attrAt("name").at().fld().setReserve("");
-		break;
-	    case 3:	//Button
-		cfg.owner()->attrDel("value");
-		cfg.owner()->attrDel("img");
-		cfg.owner()->attrDel("color");
-		cfg.owner()->attrDel("colorText");
-		cfg.owner()->attrDel("checkable");
-		cfg.owner()->attrDel("font");
-		cfg.owner()->attrAt("name").at().fld().setReserve("");
-		break;
-	    case 4: case 5:	//Combo box and list
-		cfg.owner()->attrDel("value");
-		cfg.owner()->attrDel("items");
-		cfg.owner()->attrDel("font");
-		break;
-	    case 6: case 7:	//Slider and scroll bar
-		cfg.owner()->attrDel("value");
-		cfg.owner()->attrDel("cfg");
-		break;
-	}
+	//> Delete specific attributes
+	if( cfg.getI() != prev.getI() )
+	    switch( prev.getI() )
+	    {
+		case 0:	//Line edit
+		    cfg.owner()->attrDel("value");
+		    cfg.owner()->attrDel("view");
+		    cfg.owner()->attrDel("cfg");
+		    cfg.owner()->attrDel("font");
+		    break;
+		case 1:	//Text edit
+		    cfg.owner()->attrDel("value");
+		    cfg.owner()->attrDel("wordWrap");
+		    cfg.owner()->attrDel("font");
+		    break;
+		case 2:	//Check box
+		    cfg.owner()->attrDel("value");
+		    cfg.owner()->attrDel("font");
+		    cfg.owner()->attrAt("name").at().fld().setReserve("");
+		    break;
+		case 3:	//Button
+		    cfg.owner()->attrDel("value");
+		    cfg.owner()->attrDel("img");
+		    cfg.owner()->attrDel("color");
+		    cfg.owner()->attrDel("colorText");
+		    cfg.owner()->attrDel("checkable");
+		    cfg.owner()->attrDel("font");
+		    cfg.owner()->attrAt("name").at().fld().setReserve("");
+		    break;
+		case 4: case 5:	//Combo box and list
+		    cfg.owner()->attrDel("value");
+		    cfg.owner()->attrDel("items");
+		    cfg.owner()->attrDel("font");
+		    break;
+		case 6: case 7:	//Slider and scroll bar
+		    cfg.owner()->attrDel("value");
+		    cfg.owner()->attrDel("cfg");
+		    break;
+	    }
 
-	//- Create specific attributes -
+	//> Create specific attributes
 	switch(cfg.getI())
 	{
 	    case 0:	//Line edit
 		cfg.owner()->attrAdd( new TFld("value",_("Value"),TFld::String,Attr::Mutable,"200","","","","21") );
 		cfg.owner()->attrAdd( new TFld("view",_("View"),TFld::Integer,TFld::Selected|Attr::Mutable|Attr::Active,
-		    "1","0","0;1;2;3;4;5;6",_("Text;Combo;Integer;Real;Time;Data;Data and time"),"22") );
+		    "1","0","0;1;2;3;4;5;6",_("Text;Combo;Integer;Real;Time;Date;Date and time"),"22") );
 		cfg.owner()->attrAdd( new TFld("cfg",_("Config"),TFld::String,TFld::FullText|Attr::Mutable,"","","","","23") );
 		cfg.owner()->attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"50","Arial 11","","","25") );
 		break;
@@ -372,7 +373,7 @@ bool OrigFormEl::attrChange( Attr &cfg, TVariant prev )
 	{
 	    case 2: case 4:	ntp = TFld::Integer;	break;
 	    case 3:		ntp = TFld::Real;	break;
-	    case 5: case 6:	ntp = TFld::Integer; flg|=Attr::DataTime;	break;
+	    case 5: case 6:	ntp = TFld::Integer; flg|=Attr::DateTime;	break;
 	}
 	int apos = cfg.owner()->attrPos("value");
 	cfg.owner()->attrDel("value");
@@ -437,7 +438,7 @@ bool OrigText::attrChange( Attr &cfg, TVariant prev )
 	if( cfg.id() == "numbArg" )
 	{
 	    string fid("arg"), fnm(_("Argument ")), fidp, fnmp;
-	    //- Delete specific unnecessary attributes of parameters -
+	    //> Delete specific unnecessary attributes of parameters
 	    for( int i_p = 0; true; i_p++ )
 	    {
 		fidp = fid+TSYS::int2str(i_p);
@@ -449,7 +450,7 @@ bool OrigText::attrChange( Attr &cfg, TVariant prev )
 		    cfg.owner()->attrDel(fidp+"cfg");
 		}
 	    }
-	    //- Create ullage attributes of parameters -
+	    //> Create ullage attributes of parameters
 	    for( int i_p = 0; i_p < cfg.getI(); i_p++ )
 	    {
 		fidp = fid+TSYS::int2str(i_p);
@@ -467,7 +468,7 @@ bool OrigText::attrChange( Attr &cfg, TVariant prev )
 	{
 	    int narg = (aid/10)-5;
 	    string fid = "arg"+TSYS::int2str(narg)+"val";
-	    string fnm = _("Argument ")+TSYS::int2str(narg)+":value";
+	    string fnm = _("Argument ")+TSYS::int2str(narg)+_(":value");
 	    int apos = cfg.owner()->attrPos(fid);
 	    VCA::Attr::SelfAttrFlgs sflg =  cfg.owner()->attrAt(fid).at().flgSelf();
 	    cfg.owner()->attrDel(fid);
@@ -585,7 +586,7 @@ void OrigDiagram::postEnable( int flag )
 {
     LWidget::postEnable(flag);
 
-    if( flag&TCntrNode::NodeConnect ) 
+    if( flag&TCntrNode::NodeConnect )
     {
 	attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","black","","","20") );
 	attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
@@ -612,7 +613,7 @@ bool OrigDiagram::attrChange( Attr &cfg, TVariant prev )
 	}
 	else
 	{
-	    cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","30") );
+	    cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DateTime|Attr::Mutable,"","","","","30") );
 	    cfg.owner()->attrAdd( new TFld("curUSek",_("Cursor:usek"),TFld::Integer,Attr::Mutable,"","","","","31") );
 	    cfg.owner()->attrAdd( new TFld("curColor",_("Cursor:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","32") );
 	}
@@ -644,12 +645,12 @@ bool OrigDiagram::attrChange( Attr &cfg, TVariant prev )
 	switch( cfg.getI() )
 	{
 	    case 0: case 1:
-		cfg.owner()->attrAdd( new TFld("tSek",_("Time:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","27") );
+		cfg.owner()->attrAdd( new TFld("tSek",_("Time:sek"),TFld::Integer,Attr::DateTime|Attr::Mutable,"","","","","27") );
 		cfg.owner()->attrAdd( new TFld("tUSek",_("Time:usek"),TFld::Integer,Attr::Mutable,"","","","","28") );
-		cfg.owner()->attrAdd( new TFld("tSize",_("Size, sek"),TFld::Real,Attr::Mutable,"","60","0;3000000","","29") );
+		cfg.owner()->attrAdd( new TFld("tSize",_("Size, sek"),TFld::Real,Attr::Mutable,"","60","0;3e6","","29") );
 		if( cfg.owner()->attrAt("active").at().getB() )
 		{
-		    cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","30") );
+		    cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DateTime|Attr::Mutable,"","","","","30") );
 		    cfg.owner()->attrAdd( new TFld("curUSek",_("Cursor:usek"),TFld::Integer,Attr::Mutable,"","","","","31") );
 		    cfg.owner()->attrAdd( new TFld("curColor",_("Cursor:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","32") );
 		}
@@ -658,10 +659,12 @@ bool OrigDiagram::attrChange( Attr &cfg, TVariant prev )
 		    "1","0","0;1;2;3",_("No draw;Grid;Markers;Grid and markers"),"34") );
 		cfg.owner()->attrAdd( new TFld("sclVer",_("Scale:vertical"),TFld::Integer,Attr::Mutable|TFld::Selected,
 		    "1","0","0;1;2;3;5;6;7",_("No draw;Grid;Markers;Grid and markers;Grid (log);Marker (log);Grid and markers (log)"),"35") );
+		cfg.owner()->attrAdd( new TFld("sclVerScl",_("Scale:vertical scale (%)"),TFld::Real,Attr::Mutable,"","100","10;1000","","40") );
+		cfg.owner()->attrAdd( new TFld("sclVerSclOff",_("Scale:vertical scale offset (%)"),TFld::Real,Attr::Mutable,"","0","-100;100","","41") );
 		cfg.owner()->attrAdd( new TFld("sclMarkColor",_("Scale:Markers:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","36") );
 		cfg.owner()->attrAdd( new TFld("sclMarkFont",_("Scale:Markers:font"),TFld::String,Attr::Font|Attr::Mutable,"","Arial 10","","","37") );
 		cfg.owner()->attrAdd( new TFld("valArch",_("Value archivator"),TFld::String,Attr::Mutable,"","","","","38") );
-		cfg.owner()->attrAdd( new TFld("parNum",_("Parameters number"),TFld::Integer,Attr::Mutable|Attr::Active,"1","1","0;10","","39") );
+		cfg.owner()->attrAdd( new TFld("parNum",_("Parameters number"),TFld::Integer,Attr::Mutable|Attr::Active,"","1","0;10","","39") );
 		break;
 	}
     }
@@ -734,16 +737,17 @@ void OrigProtocol::postEnable( int flag )
 	attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","","","","20") );
 	attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
 	attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"","Arial 11","","","22") );
-	attrAdd( new TFld("time",_("Time, sek"),TFld::Integer,Attr::DataTime,"","","","","24") );
+	attrAdd( new TFld("headVis",_("Header visible"),TFld::Boolean,TFld::NoFlag,"","1","","","23") );
+	attrAdd( new TFld("time",_("Time, sek"),TFld::Integer,Attr::DateTime,"","","","","24") );
 	attrAdd( new TFld("tSize",_("Size, sek"),TFld::Integer,TFld::NoFlag,"","60","","","25") );
 	attrAdd( new TFld("trcPer",_("Tracing period (s)"),TFld::Integer,TFld::NoFlag,"","0","0;360","","26") );
 	attrAdd( new TFld("arch",_("Archivator"),TFld::String,TFld::NoFlag,"","","","","27") );
 	attrAdd( new TFld("tmpl",_("Template"),TFld::String,TFld::NoFlag,"","","","","28") );
-	attrAdd( new TFld("lev",_("Level"),TFld::Integer,TFld::NoFlag,"","0","","","29") );
-//	attrAdd( new TFld("viewOrd",_("View order"),TFld::Integer,TFld::Selected,"","0",
-//	    "0;1;2",_("On time;On level;On level and trigered"),"30") );
+	attrAdd( new TFld("lev",_("Level"),TFld::Integer,TFld::NoFlag,"","0","-7;7","","29") );
+	attrAdd( new TFld("viewOrd",_("View order"),TFld::Integer,TFld::Selected,"","0",
+	    "0;1;2;3;4;5;6;7",_("On time;On level;On category;On messages;On time (reverse);On level (reverse);On category (reverse);On messages (reverse)"),"30") );
 	attrAdd( new TFld("col",_("View columns"),TFld::String,TFld::NoFlag,"","pos;tm;utm;lev;cat;mess","","","31") );
-//	attrAdd( new TFld("itProp",_("Items properties"),TFld::Integer,Attr::Active,"","0","0;10","","32") );
+	attrAdd( new TFld("itProp",_("Item's properties"),TFld::Integer,Attr::Active,"","","","","32") );
     }
 }
 
@@ -754,7 +758,7 @@ bool OrigProtocol::attrChange( Attr &cfg, TVariant prev )
 	if( cfg.id() == "itProp" )
 	{
 	    string fid("it"), fnm(_("Item ")), fidp, fnmp;
-	    //- Delete specific unnecessary items -
+	    //> Delete specific unnecessary items
 	    for( int i_p = 0; true; i_p++ )
 	    {
 		fidp = fid+TSYS::int2str(i_p);
@@ -764,27 +768,23 @@ bool OrigProtocol::attrChange( Attr &cfg, TVariant prev )
 		    cfg.owner()->attrDel(fidp+"lev");
 		    cfg.owner()->attrDel(fidp+"tmpl");
 		    cfg.owner()->attrDel(fidp+"fnt");
-		    cfg.owner()->attrDel(fidp+"img");
 		    cfg.owner()->attrDel(fidp+"color");
-		    cfg.owner()->attrDel(fidp+"blink");
 		}
 	    }
-	    //- Create ullage items -
+	    //> Create ullage items
 	    for( int i_p = 0; i_p < cfg.getI(); i_p++ )
 	    {
 		fidp = fid+TSYS::int2str(i_p);
 		fnmp = fnm+TSYS::int2str(i_p);
 		if( cfg.owner()->attrPresent( fidp+"lev" ) ) continue;
-		cfg.owner()->attrAdd( new TFld((fidp+"lev").c_str(),(fnmp+_(":levels")).c_str(),
-					       TFld::String,Attr::Mutable,"","","","",TSYS::int2str(50+10*i_p).c_str()) );
+		cfg.owner()->attrAdd( new TFld((fidp+"lev").c_str(),(fnmp+_(":level")).c_str(),
+					       TFld::Integer,Attr::Mutable,"","0","0;7","",TSYS::int2str(40+5*i_p).c_str()) );
 		cfg.owner()->attrAdd( new TFld((fidp+"tmpl").c_str(),(fnmp+_(":template")).c_str(),
-					       TFld::String,Attr::Mutable,"","","","",TSYS::int2str(51+10*i_p).c_str()) );
+					       TFld::String,Attr::Mutable,"","","","",TSYS::int2str(41+5*i_p).c_str()) );
 		cfg.owner()->attrAdd( new TFld((fidp+"fnt").c_str(),(fnmp+_(":font")).c_str(),
-					       TFld::String,Attr::Font|Attr::Mutable,"","","","",TSYS::int2str(52+10*i_p).c_str()) );
+					       TFld::String,Attr::Font|Attr::Mutable,"","","","",TSYS::int2str(42+5*i_p).c_str()) );
 		cfg.owner()->attrAdd( new TFld((fidp+"color").c_str(),(fnmp+_(":color")).c_str(),
-					       TFld::String,Attr::Color|Attr::Mutable,"","","","",TSYS::int2str(53+10*i_p).c_str()) );
-		cfg.owner()->attrAdd( new TFld((fidp+"blink").c_str(),(fnmp+_(":blink")).c_str(),
-					       TFld::Boolean,Attr::Mutable,"","0","","",TSYS::int2str(54+10*i_p).c_str()) );
+					       TFld::String,Attr::Color|Attr::Mutable,"","","","",TSYS::int2str(43+5*i_p).c_str()) );
 	    }
 	}
     }
@@ -824,8 +824,8 @@ void OrigDocument::postEnable( int flag )
 	attrAdd( new TFld("tmpl",_("Template"),TFld::String,TFld::FullText,"","","","","21") );
 	attrAdd( new TFld("doc",_("Document"),TFld::String,TFld::FullText,"","","","","22") );
 	attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"","Arial 11","","","26") );
-	attrAdd( new TFld("bTime",_("Time:begin"),TFld::Integer,Attr::DataTime,"","0","","","24") );
-	attrAdd( new TFld("time",_("Time:current"),TFld::Integer,Attr::DataTime|Attr::Active,"","0","","","23") );
+	attrAdd( new TFld("bTime",_("Time:begin"),TFld::Integer,Attr::DateTime,"","0","","","24") );
+	attrAdd( new TFld("time",_("Time:current"),TFld::Integer,Attr::DateTime|Attr::Active,"","0","","","23") );
 	attrAdd( new TFld("n",_("Archive size"),TFld::Integer,Attr::Active,"","0","0;99","","25") );
     }
 }
@@ -874,9 +874,15 @@ bool OrigDocument::attrChange( Attr &cfg, TVariant prev )
 	else
 	{
 	    if( !cfg.owner()->attrPresent("vCur") )
+	    {
 		cfg.owner()->attrAdd( new TFld("vCur",_("Cursor:view"),TFld::Integer,Attr::Mutable|Attr::Active,"","0","-2;99") );
+		cfg.owner()->inheritAttr("vCur");
+	    }
 	    if( !cfg.owner()->attrPresent("aCur") )
+	    {
 		cfg.owner()->attrAdd( new TFld("aCur",_("Cursor:archive"),TFld::Integer,Attr::Mutable|Attr::Active,"","0","-1;99") );
+		cfg.owner()->inheritAttr("aCur");
+	    }
 	}
 
 	string fidp;
@@ -884,7 +890,7 @@ bool OrigDocument::attrChange( Attr &cfg, TVariant prev )
 	for( int i_p = 0; true; i_p++ )
 	{
 	    fidp = "doc"+TSYS::int2str(i_p);
-	    if( !cfg.owner()->attrPresent(fidp) )      break;
+	    if( !cfg.owner()->attrPresent(fidp) )	break;
 	    else if( i_p >= cfg.getI() )	cfg.owner()->attrDel(fidp);
 	}
 
@@ -892,7 +898,7 @@ bool OrigDocument::attrChange( Attr &cfg, TVariant prev )
 	for( int i_p = 0; i_p < cfg.getI(); i_p++ )
 	{
 	    fidp = "doc"+TSYS::int2str(i_p);
-	    if( cfg.owner()->attrPresent(fidp) ) continue;
+	    if( cfg.owner()->attrPresent(fidp) )	continue;
 	    cfg.owner()->attrAdd( new TFld(fidp.c_str(),(_("Document ")+TSYS::int2str(i_p)).c_str(),TFld::String,TFld::FullText|Attr::Mutable|Attr::Active) );
 	}
     }
@@ -901,30 +907,7 @@ bool OrigDocument::attrChange( Attr &cfg, TVariant prev )
     if( !sw )	return Widget::attrChange(cfg,prev);
 
     //> Make document after time set
-    if( cfg.id() == "time" && cfg.getI() != prev.getI() )
-    {
-	cfg.setFlgSelf((Attr::SelfAttrFlgs)(cfg.flgSelf()|0x100));
-	/*string mkDk;
-	int n = cfg.owner()->attrAt("n").at().getI();
-	if( !n )
-	{
-	    mkDk = cfg.owner()->attrAt("doc").at().getS();
-	    if( mkDk.empty() )	mkDk = cfg.owner()->attrAt("tmpl").at().getS();
-	    mkDk = makeDoc(mkDk,cfg.owner());
-	    cfg.owner()->attrAt("doc").at().setS(mkDk);
-	}
-	else
-	{
-	    int aCur = cfg.owner()->attrAt("aCur").at().getI();
-	    mkDk = cfg.owner()->attrAt("doc"+TSYS::int2str(aCur)).at().getS();
-	    if( mkDk.empty() )	mkDk = cfg.owner()->attrAt("tmpl").at().getS();
-
-	    mkDk = makeDoc(mkDk,cfg.owner());
-	    cfg.owner()->attrAt("doc"+TSYS::int2str(aCur)).at().setS(mkDk);
-	    if( aCur == cfg.owner()->attrAt("vCur").at().getI() )
-		cfg.owner()->attrAt("doc").at().setS(mkDk);
-	}*/
-    }
+    if( cfg.id() == "time" && cfg.getI() != prev.getI() ) cfg.setFlgSelf((Attr::SelfAttrFlgs)(cfg.flgSelf()|0x100));
     //> Load document's from project's DB
     else if( cfg.id() == "n" && cfg.getI() != prev.getI() )
     {
@@ -970,7 +953,7 @@ bool OrigDocument::attrChange( Attr &cfg, TVariant prev )
 		cfg.owner()->attrAt("vCur").at().setI(cfg.getI());
 
 	    //>> Save cursor to document to project's DB
-	    if( prev.getI() < n && prev.getI() >=0 )
+	    if( prev.getI() < n && prev.getI() >= 0 )
 	    {
 		string db  = sw->ownerSess()->parent().at().DB();
 		string tbl = sw->ownerSess()->parent().at().tbl()+"_ses";
@@ -1191,9 +1174,12 @@ void OrigDocument::nodeProcess( Widget *wdg, XMLNode *xcur, TValFunc &funcV, TFu
 	    XMLNode *reptN = xcur->childGet(i_c);
 	    bool docRevers = atoi(xcur->childGet(i_c)->attr("docRevers").c_str());
 
+	    int off = 0;
+	    int dACat = atoi(TSYS::strSepParse(dAMess,0,':',&off).c_str());
+	    string dATmpl = dAMess.substr(off);
+
 	    vector<TMess::SRec> mess;
-	    SYS->archive().at().messGet( funcV.getI(3), funcV.getI(1), mess,
-		TSYS::strSepParse(dAMess,1,':'), (TMess::Type)atoi(TSYS::strSepParse(dAMess,0,':').c_str()) );
+	    SYS->archive().at().messGet( funcV.getI(3), funcV.getI(1), mess, dATmpl, (TMess::Type)dACat );
 
 	    for( int i_r = 0; i_r < mess.size(); i_r++ )
 	    {
