@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.VCAEngine file: session.cpp
 /***************************************************************************
- *   Copyright (C) 2007-2008 by Roman Savochenko                           *
+ *   Copyright (C) 2007-2009 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -33,10 +33,10 @@ using namespace VCA;
 //* Session: Project's session			 *
 //************************************************
 Session::Session( const string &iid, const string &iproj ) :
-    mEnable(false), mStart(false), endrun_req(false), tm_calc(0.0),
+    mEnable(false), mStart(false), endrun_req(false), tm_calc(0.0), mStyleIdW(-1),
     mUser("root"), mPrjnm(iproj), mPer(100), mCalcClk(1),
     mOwner("root"), mGrp("UI"), mPermit(RWRWR_),
-    mBackgrnd(false), mConnects(0)
+    mBackgrnd(false), mConnects(0), mAlrmSndPlay(-1)
 {
     mId = iid;
     mPage = grpAdd("pg_");
@@ -83,6 +83,14 @@ void Session::setEnable( bool val )
 	    mPermit	= parent().at().permit( );
 	    setPeriod( parent().at().period( ) );
 
+	    //> Load previous style
+	    TConfig c_el(&mod->elPrjSes());
+	    c_el.cfg("IDW").setS("<Style>");
+	    c_el.cfg("ID").setS(user());
+	    if( SYS->db().at().dataGet(parent().at().DB()+"."+parent().at().tbl()+"_ses",mod->nodePath()+parent().at().tbl()+"_ses",c_el) )
+		stlCurentSet(c_el.cfg("IO_VAL").getI());
+	    else stlCurentSet( parent().at().stlCurent( ) );
+
 	    //> Create root pages
 	    parent().at().list(pg_ls);
 	    for( int i_p = 0; i_p < pg_ls.size(); i_p++ )
@@ -94,7 +102,7 @@ void Session::setEnable( bool val )
 	    for( int i_ls = 0; i_ls < pg_ls.size(); i_ls++ )
 		try{ at(pg_ls[i_ls]).at().setEnable(true); }
 		catch( TError err )	{ mess_err( err.cat.c_str(), "%s", err.mess.c_str() ); }
-	
+
 	    modifGClr();
 	}
 	catch(...){ mParent.free(); }
@@ -128,38 +136,27 @@ void Session::setStart( bool val )
 	//> Enable session if it disabled
 	if( !enable() )	setEnable(true);
 
+	//> Load Styles from project
+	mStProp.clear();
+	if( stlCurent() >= 0 )
+	{
+	    parent().at().stlPropList(pg_ls);
+	    for( int i_sp = 0; i_sp < pg_ls.size(); i_sp++ )
+		mStProp[pg_ls[i_sp]] = parent().at().stlPropGet( pg_ls[i_sp], "", stlCurent() );
+	}
+
 	//> Process all pages is on
 	list(pg_ls);
 	for( int i_ls = 0; i_ls < pg_ls.size(); i_ls++ )
 	    at(pg_ls[i_ls]).at().setProcess(true);
 
 	//> Start process task
-	if( !mStart )
-	{
-	    pthread_attr_t pthr_attr;
-	    pthread_attr_init(&pthr_attr);
-	    struct sched_param prior;
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	    prior.__sched_priority=2;
-	    pthread_attr_setschedparam(&pthr_attr,&prior);
-
-	    pthread_create(&calcPthr,&pthr_attr,Session::Task,this);
-	    pthread_attr_destroy(&pthr_attr);
-	    if( TSYS::eventWait(mStart, true, nodePath()+"start",5) )
-		throw TError(nodePath().c_str(),_("Session processing task is not started!"));
-	}
+	if( !mStart ) SYS->taskCreate( nodePath('.',true), 0, Session::Task, this, &mStart );
     }
     else
     {
 	//> Stop process task
-	if( mStart )
-	{
-	    endrun_req = true;
-	    pthread_kill( calcPthr, SIGALRM );
-	    if( TSYS::eventWait(mStart,false,nodePath()+"stop",5) )
-		throw TError(nodePath().c_str(),_("Sesion processing task is not stopped!"));
-	    pthread_join( calcPthr, NULL );
-	}
+	if( mStart ) SYS->taskDestroy( nodePath('.',true), &mStart, &endrun_req );
 
 	//> Process all pages is off
 	list(pg_ls);
@@ -168,6 +165,11 @@ void Session::setStart( bool val )
     }
 }
 
+bool Session::modifChk( unsigned int tm, unsigned int iMdfClc )
+{
+    return (mCalcClk>=tm) ? (iMdfClc >= tm && iMdfClc <= mCalcClk) : (iMdfClc >= tm || iMdfClc <= mCalcClk);
+}
+
 string Session::ico( )
 {
     if( !parent().freeStat() )	return parent().at().ico();
@@ -228,7 +230,7 @@ void Session::uiComm( const string &com, const string &prm, SessWdg *src )
 	string cur_pt_el;
 	int i_el = 0;
 	AutoHD<SessPage> cpg;
-	while((cur_pt_el=TSYS::pathLev(prm,i_el++)).size())
+	while( (cur_pt_el=TSYS::pathLev(prm,i_el++)).size() )
 	{
 	    string op_pg;
 	    if( cur_pt_el.substr(0,3) == "pg_" ) op_pg = cur_pt_el.substr(3);
@@ -272,10 +274,12 @@ void Session::uiComm( const string &com, const string &prm, SessWdg *src )
 	    //>> Go to next page
 	    cpg = cpg.freeStat() ? at(op_pg) : cpg.at().pageAt(op_pg);
 	}
+
 	//> Open found page
 	if( !cpg.freeStat() )
 	{
-	    if( !oppg.empty() )	((AutoHD<SessPage>)mod->nodeAt(oppg)).at().attrAt("pgOpenSrc").at().setS("");
+	    if( !oppg.empty() && ((AutoHD<SessPage>)mod->nodeAt(oppg)).at().path() != cpg.at().path() )
+		((AutoHD<SessPage>)mod->nodeAt(oppg)).at().attrAt("pgOpenSrc").at().setS("");
 	    cpg.at().attrAt("pgOpenSrc").at().setS(src->path());
 	}
     }catch(...){ }
@@ -285,7 +289,7 @@ void Session::alarmSet( const string &wpath, const string &alrm )
 {
     if( wpath.empty() ) return;
 
-    //- Alarms queue process -
+    //> Alarms queue process
     ResAlloc res( mAlrmRes, true );
 
     Alarm aobj( wpath, alrm, calcClk( ) );
@@ -297,20 +301,29 @@ void Session::alarmSet( const string &wpath, const string &alrm )
 	if( i_q < mAlrm.size() ) mAlrm.erase( mAlrm.begin()+i_q );
 	return;
     }
-    if( i_q < mAlrm.size() && aobj.lev == mAlrm[i_q].lev )	mAlrm[i_q] = aobj;
+    if( i_q < mAlrm.size() && aobj.lev == mAlrm[i_q].lev ) mAlrm[i_q] = aobj;
     else
     {
-	if( i_q < mAlrm.size() )	mAlrm.erase( mAlrm.begin()+i_q );
+	if( i_q < mAlrm.size() )
+	{
+	    mAlrm.erase( mAlrm.begin()+i_q );
+	    if( i_q == mAlrmSndPlay ) mAlrmSndPlay = -1;
+	    if( i_q < mAlrmSndPlay && mAlrmSndPlay >= 0 ) mAlrmSndPlay--;
+	}
 	int i_q1 = 0;
 	while( i_q1 < mAlrm.size() && aobj.lev >= mAlrm[i_q].lev ) i_q1++;
-	if( i_q1 < mAlrm.size() )	mAlrm.insert(mAlrm.begin()+i_q1,aobj);
+	if( i_q1 < mAlrm.size() )
+	{
+	    mAlrm.insert(mAlrm.begin()+i_q1,aobj);
+	    if( i_q <= mAlrmSndPlay && mAlrmSndPlay >= 0 ) mAlrmSndPlay++;
+	}
 	else mAlrm.push_back(aobj);
     }
 }
 
 int Session::alarmStat( )
 {
-    ui8 alev = 0, atp = 0, aqtp = 0;
+    uint8_t alev = 0, atp = 0, aqtp = 0;
     vector<string> ls;
     list( ls );
     for( int i_p = 0; i_p < ls.size(); i_p++ )
@@ -324,7 +337,7 @@ int Session::alarmStat( )
     return (aqtp<<16)|(atp<<8)|alev;
 }
 
-void Session::alarmQuittance( const string &wpath, ui8 quit_tmpl )
+void Session::alarmQuittance( const string &wpath, uint8_t quit_tmpl )
 {
     if( !wpath.empty() ) ((AutoHD<SessWdg>)mod->nodeAt(wpath)).at().alarmQuittance( quit_tmpl, true );
     else
@@ -363,7 +376,7 @@ void *Session::Task( void *icontr )
 
 	//> Calc session pages and all other items at recursion
 	for( int i_l = 0; i_l < pls.size(); i_l++ )
-	    try{ ses.at(pls[i_l]).at().calc(false,false); }
+	    try { ses.at(pls[i_l]).at().calc(false,false); }
 	    catch( TError err )
 	    {
 		mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -388,13 +401,86 @@ void *Session::Task( void *icontr )
     return NULL;
 }
 
+void Session::stlCurentSet( int sid )
+{
+    if( mStyleIdW == sid ) return;
+
+    mStyleIdW = sid;
+
+    if( start() )
+    {
+	ResAlloc res( mStRes, true );
+	//> Load Styles from project
+	mStProp.clear();
+
+	if( sid >= 0 && sid < parent().at().stlSize() )
+	{
+	    vector<string> pg_ls;
+	    parent().at().stlPropList(pg_ls);
+	    for( int i_sp = 0; i_sp < pg_ls.size(); i_sp++ )
+		mStProp[pg_ls[i_sp]] = parent().at().stlPropGet( pg_ls[i_sp], "", sid );
+	}
+	else mStyleIdW = -1;
+    }
+
+    //> Write to DB
+    if( enable() )
+    {
+	TConfig c_el(&mod->elPrjSes());
+	c_el.cfg("IDW").setS("<Style>");
+	c_el.cfg("ID").setS(user());
+	c_el.cfg("IO_VAL").setI(mStyleIdW);
+	SYS->db().at().dataSet(parent().at().DB()+"."+parent().at().tbl()+"_ses",mod->nodePath()+parent().at().tbl()+"_ses",c_el);
+    }
+}
+
+string Session::stlPropGet( const string &pid, const string &def )
+{
+    ResAlloc res( mStRes, false );
+
+    if( stlCurent() < 0 || pid.empty() || pid == "<Styles>" ) return def;
+
+    map<string,string>::iterator iStPrp = mStProp.find(pid);
+    if( iStPrp != mStProp.end() ) return iStPrp->second;
+
+    return def;
+}
+
+bool Session::stlPropSet( const string &pid, const string &vl )
+{
+    ResAlloc res( mStRes, true );
+    if( stlCurent() < 0 || pid.empty() || pid == "<Styles>" ) return false;
+    map<string,string>::iterator iStPrp = mStProp.find(pid);
+    if( iStPrp == mStProp.end() ) return false;
+    iStPrp->second = vl;
+
+    return true;
+}
+
+TVariant Session::objFuncCall( const string &iid, vector<TVariant> &prms, const string &cuser )
+{
+    if( iid == "user" )	return user();
+    else if( iid == "alrmSndPlay" )
+    {
+	ResAlloc res( mAlrmRes, false );
+	if( mAlrmSndPlay < 0 || mAlrmSndPlay >= mAlrm.size() ) return string("");
+	return mAlrm[mAlrmSndPlay].path;
+    }
+    else if( iid == "alrmQuittance" && prms.size() >= 1 )
+    {
+	alarmQuittance( (prms.size()>=2) ? prms[1].getS() : "", ~prms[0].getI() );
+	return 0;
+    }
+    return TCntrNode::objFuncCall(iid,prms,cuser);
+}
+
 void Session::cntrCmdProc( XMLNode *opt )
 {
     string a_path = opt->attr("path");
     //> Service commands process
     if( a_path == "/serv/pg" )	//Pages operations
     {
-	if( ctrChkNode(opt,"openlist",permit(),owner().c_str(),grp().c_str(),SEQ_RD) )	//Open pages list
+	if( ctrChkNode(opt,"openlist",permit(),owner().c_str(),grp().c_str(),SEC_RD) )	//Open pages list
 	{
 	    unsigned tm = strtoul(opt->attr("tm").c_str(),NULL,10);
 	    vector<string> &lst = openList();
@@ -407,7 +493,7 @@ void Session::cntrCmdProc( XMLNode *opt )
 		{
 		    //> Permission to view page check
 		    AutoHD<SessWdg> swdg = nodeAt(lst[i_f],1);
-		    if( !SYS->security().at().access(opt->attr("user"),SEQ_RD,swdg.at().owner(),swdg.at().grp(),swdg.at().permit()) )
+		    if( !SYS->security().at().access(opt->attr("user"),SEC_RD,swdg.at().owner(),swdg.at().grp(),swdg.at().permit()) )
 			continue;
 		    //> Changed widgets list add
 		    vector<string> updEl;
@@ -417,18 +503,19 @@ void Session::cntrCmdProc( XMLNode *opt )
 	    }
 	    opt->setAttr("tm",TSYS::uint2str(calcClk()));
 	}
-	else if( ctrChkNode(opt,"open",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )		//Open pages
+	else if( ctrChkNode(opt,"open",permit(),owner().c_str(),grp().c_str(),SEC_WR) )		//Open pages
 	    ((AutoHD<SessWdg>)nodeAt(opt->attr("pg"),1)).at().attrAt("pgOpen").at().setB(true);
-	else if( ctrChkNode(opt,"close",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )	//Close open pages
+	else if( ctrChkNode(opt,"close",RWRWRW,owner().c_str(),grp().c_str(),SEC_WR) )		//Close open pages
 	    ((AutoHD<SessWdg>)nodeAt(opt->attr("pg"),1)).at().attrAt("pgOpen").at().setB(false);
 	return;
     }
     else if( a_path == "/serv/alarm" )	//Alarm operations
     {
-	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEC_RD) )
 	{
 	    //> Get alarm status
-	    opt->setAttr("alarmSt",TSYS::int2str(alarmStat()));
+	    int aSt = alarmStat();
+	    opt->setAttr("alarmSt",TSYS::int2str(aSt));
 
 	    //> Get alarm from sound queue
 	    if( opt->attr("mode") == "sound" )
@@ -443,7 +530,7 @@ void Session::cntrCmdProc( XMLNode *opt )
 		for( i_q = mAlrm.size()-1; i_q >= 0; i_q-- )
 		{
 		    if( !(mAlrm[i_q].qtp & Engine::Sound) ) continue;
-		    if( wdg.empty() || mAlrm[i_q].clc >= a_tm || i_next > 0 )	break;	//First, new and next alarms break
+		    if( wdg.empty() || modifChk(a_tm,mAlrm[i_q].clc) || i_next > 0 )	break;	//First, new and next alarms break
 		    if( i_first < 0 ) i_first = i_q;
 		    if( wdg == mAlrm[i_q].path ) i_next = i_q;
 		}
@@ -454,10 +541,11 @@ void Session::cntrCmdProc( XMLNode *opt )
 		    if( !mAlrm[i_q].tpArg.empty() )
 			opt->setText( ((AutoHD<SessWdg>)mod->nodeAt(mAlrm[i_q].path)).at().resourceGet(mAlrm[i_q].tpArg) );
 		    else opt->setText( mod->callSynth(mAlrm[i_q].mess) );
-		}
-	    }
+		    mAlrmSndPlay = i_q;
+		}else mAlrmSndPlay = -1;
+	    } else if( !((aSt>>16) & Engine::Sound) ) mAlrmSndPlay = -1;
 	}
-	else if( ctrChkNode(opt,"quittance",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )
+	else if( ctrChkNode(opt,"quittance",permit(),owner().c_str(),grp().c_str(),SEC_WR) )
 	    alarmQuittance(opt->attr("wdg"),~atoi(opt->attr("tmpl").c_str()));
 	return;
     }
@@ -485,7 +573,7 @@ void Session::cntrCmdProc( XMLNode *opt )
 		    "sel_id","0;4;6","sel_list",_("No access;View;View and control"));
 		ctrMkNode("fld",opt,-1,"/obj/st/o_a","",R_R_R_,"root","UI",4,"tp","dec","dest","select",
 		    "sel_id","0;4;6","sel_list",_("No access;View;View and control"));
-		ctrMkNode("fld",opt,-1,"/obj/st/prj",_("Project"),permit(),owner().c_str(),grp().c_str(),3,"tp","str","dest","sel_ed","select","/obj/prj_ls");
+		ctrMkNode("fld",opt,-1,"/obj/st/prj",_("Project"),permit(),owner().c_str(),grp().c_str(),4,"tp","str","idm","1","dest","select","select","/obj/prj_ls");
 		ctrMkNode("fld",opt,-1,"/obj/st/backgrnd",_("Background execution mode"),R_R_R_,"root","UI",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/obj/st/connect",_("Connections counter"),R_R_R_,"root","UI",1,"tp","int");
 		ctrMkNode("fld",opt,-1,"/obj/st/calc_tm",_("Calc session time (ms)"),R_R_R_,"root","UI",1,"tp","real");
@@ -493,6 +581,7 @@ void Session::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/obj/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/obj/cfg/per",_("Period (ms)"),permit(),owner().c_str(),grp().c_str(),1,"tp","dec");
+		ctrMkNode("fld",opt,-1,"/obj/cfg/style",_("Style"),permit(),owner().c_str(),grp().c_str(),3,"tp","dec","dest","select","select","/obj/cfg/stLst");
 		ctrMkNode("list",opt,-1,"/obj/cfg/openPg",_("Opened pages"),R_R_R_,"root","UI",1,"tp","str");
 	    }
 	}
@@ -506,6 +595,7 @@ void Session::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/alarm/alarm/cat",_("Category"),R_R_R_,"root","UI",1,"tp","str");
 		ctrMkNode("list",opt,-1,"/alarm/alarm/mess",_("Messages"),R_R_R_,"root","UI",1,"tp","str");
 		ctrMkNode("list",opt,-1,"/alarm/alarm/tp",_("Type"),R_R_R_,"root","UI",1,"tp","hex");
+		ctrMkNode("list",opt,-1,"/alarm/alarm/qtp",_("Quitance"),R_R_R_,"root","UI",1,"tp","hex");
 		ctrMkNode("list",opt,-1,"/alarm/alarm/tpArg",_("Type argument"),R_R_R_,"root","UI",1,"tp","str");
 	    }
 	return;
@@ -515,13 +605,13 @@ void Session::cntrCmdProc( XMLNode *opt )
     if( a_path == "/ico" && ctrChkNode(opt) )   opt->setText(ico());
     else if( a_path == "/obj/st/en" )
     {
-	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEQ_RD) )	opt->setText(TSYS::int2str(enable()));
-	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )	setEnable(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEC_RD) )	opt->setText(TSYS::int2str(enable()));
+	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEC_WR) )	setEnable(atoi(opt->text().c_str()));
     }
     else if( a_path == "/obj/st/start" )
     {
-	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEQ_RD) )	opt->setText(TSYS::int2str(start()));
-	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )	setStart(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEC_RD) )	opt->setText(TSYS::int2str(start()));
+	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEC_WR) )	setStart(atoi(opt->text().c_str()));
     }
     else if( a_path == "/obj/st/user" && ctrChkNode(opt) )	opt->setText(user());
     else if( a_path == "/obj/st/owner" && ctrChkNode(opt) )	opt->setText(owner());
@@ -534,8 +624,8 @@ void Session::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/obj/st/prj" )
     {
-	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEQ_RD) )	opt->setText(projNm());
-	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )	setProjNm(opt->text());
+	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEC_RD) )	opt->setText(projNm());
+	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEC_WR) )	setProjNm(opt->text());
     }
     else if( a_path == "/obj/st/backgrnd" && ctrChkNode(opt) )	opt->setText(TSYS::int2str(backgrnd()));
     else if( a_path == "/obj/st/connect" && ctrChkNode(opt) )	opt->setText(TSYS::int2str(connects()));
@@ -549,8 +639,21 @@ void Session::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/obj/cfg/per" )
     {
-	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEQ_RD) )	opt->setText(TSYS::int2str(period()));
-	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )	setPeriod(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEC_RD) )	opt->setText(TSYS::int2str(period()));
+	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEC_WR) )	setPeriod(atoi(opt->text().c_str()));
+    }
+    else if( a_path == "/obj/cfg/style" )
+    {
+	if( ctrChkNode(opt,"get",permit(),owner().c_str(),grp().c_str(),SEC_RD) )	opt->setText( TSYS::int2str(stlCurent()) );
+	if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEC_WR) )	stlCurentSet( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/obj/cfg/stLst" && ctrChkNode(opt) )
+    {
+	opt->childAdd("el")->setAttr("id","-1")->setText(_("No style"));
+	if( enable() )
+	    for( int iSt = 0; iSt < parent().at().stlSize(); iSt++ )
+		opt->childAdd("el")->setAttr("id",TSYS::int2str(iSt))->setText(TSYS::strSepParse(parent().at().stlGet(iSt),0,';'));
+
     }
     else if( a_path == "/obj/cfg/openPg" && ctrChkNode(opt) )
     {
@@ -573,6 +676,7 @@ void Session::cntrCmdProc( XMLNode *opt )
 	XMLNode *n_cat	= ctrMkNode("list",opt,-1,"/alarm/alarm/cat","",0444);
 	XMLNode *n_mess	= ctrMkNode("list",opt,-1,"/alarm/alarm/mess","",0444);
 	XMLNode *n_tp	= ctrMkNode("list",opt,-1,"/alarm/alarm/tp","",0444);
+	XMLNode *n_qtp	= ctrMkNode("list",opt,-1,"/alarm/alarm/qtp","",0444);
 	XMLNode *n_tpArg= ctrMkNode("list",opt,-1,"/alarm/alarm/tpArg","",0444);
 
 	ResAlloc res( mAlrmRes, false );
@@ -583,6 +687,7 @@ void Session::cntrCmdProc( XMLNode *opt )
 	    if( n_cat )	n_cat->childAdd("el")->setText( mAlrm[i_q].cat );
 	    if( n_mess )n_mess->childAdd("el")->setText( mAlrm[i_q].mess );
 	    if( n_tp )	n_tp->childAdd("el")->setText( TSYS::int2str(mAlrm[i_q].tp) );
+	    if( n_qtp )	n_qtp->childAdd("el")->setText( TSYS::int2str(mAlrm[i_q].qtp) );
 	    if( n_tpArg ) n_tpArg->childAdd("el")->setText( mAlrm[i_q].tpArg );
 	}
     }
@@ -604,7 +709,8 @@ Session::Alarm::Alarm( const string &ipath, const string &alrm, unsigned iclc )
 //************************************************
 //* SessPage: Page of Project's session          *
 //************************************************
-SessPage::SessPage( const string &iid, const string &ipage, Session *sess ) : SessWdg(iid,ipage,sess)
+SessPage::SessPage( const string &iid, const string &ipage, Session *sess ) : 
+    SessWdg(iid,ipage,sess), mClosePgCom(false)
 {
     mPage = grpAdd("pg_");
 }
@@ -707,11 +813,13 @@ void SessPage::calc( bool first, bool last )
     //> Process self data
     if( process() ) SessWdg::calc(first,last);
 
+    if( mClosePgCom ) { mClosePgCom = false; setProcess(false); return; }
+
     //> Put calculate to include pages
     vector<string> ls;
     pageList(ls);
     for(int i_l = 0; i_l < ls.size(); i_l++ )
-        pageAt(ls[i_l]).at().calc(first,last);
+	pageAt(ls[i_l]).at().calc(first,last);
 }
 
 bool SessPage::attrChange( Attr &cfg, TVariant prev )
@@ -729,7 +837,7 @@ bool SessPage::attrChange( Attr &cfg, TVariant prev )
 	    else
 	    {
 		ownerSess()->openUnreg(path());
-		if( process() && !attrAt("pgNoOpenProc").at().getB() )	setProcess(false);
+		if( process() && !attrAt("pgNoOpenProc").at().getB() )	mClosePgCom = true;
 		if( !attrAt("pgOpenSrc").at().getS().empty() ) attrAt("pgOpenSrc").at().setS("");
 	    }
 	}
@@ -767,6 +875,21 @@ bool SessPage::attrChange( Attr &cfg, TVariant prev )
 			    emptyPresnt = true;
 			}
 		    }
+		    //> Find links into source if no link find
+		    if( prm_lnk.empty() )
+		    {
+			vector<string> sAtrLs;
+			src.at().attrList(sAtrLs);
+			for( int i_al = 0; i_al < sAtrLs.size(); i_al++ )
+			{
+			    AutoHD<Attr> attr = src.at().attrAt(sAtrLs[i_al]);
+			    if( attr.at().flgSelf()&(Attr::CfgLnkIn|Attr::CfgLnkOut) && attr.at().cfgVal().size() > 4 && attr.at().cfgVal().substr(0,4) == "prm:" )
+			    {
+				prm_lnk = attr.at().cfgVal().substr(4);
+				break;
+			    }
+			}
+		    }
 		    //> Fill parameter's links for other attributes
 		    if( emptyPresnt && !prm_lnk.empty() )
 		    {
@@ -800,8 +923,7 @@ void SessPage::alarmSet( bool isSet )
     string aCur = attrAt("alarm").at().getS( );
     int alev = atoi(TSYS::strSepParse(aCur,0,'|').c_str()) & 0xFF;
     int atp  = atoi(TSYS::strSepParse(aCur,3,'|').c_str()) & 0xFF;
-    int aqtp = (aStCur>>16) & 0xFF & atp;
-    if( isSet ) aqtp |= atp;
+    int aqtp = isSet ? atp : (aStCur>>16) & 0xFF & atp;
 
     vector<string> lst;
     //> Included pages process
@@ -813,6 +935,7 @@ void SessPage::alarmSet( bool isSet )
 	atp |= (iacur>>8) & 0xFF;
 	aqtp |= (iacur>>16) & 0xFF;
     }
+
     //> Included widgets process
     wdgList( lst );
     for( int i_w = 0; i_w < lst.size(); i_w++ )
@@ -829,7 +952,7 @@ void SessPage::alarmSet( bool isSet )
     if( isSet )	ownerSess( )->alarmSet( path(), aCur );
 }
 
-void SessPage::alarmQuittance( ui8 quit_tmpl, bool isSet )
+void SessPage::alarmQuittance( uint8_t quit_tmpl, bool isSet )
 {
     int alarmSt = attrAt("alarmSt").at().getI();
     if( !((((alarmSt>>16)&0xFF)^quit_tmpl)&((alarmSt>>16)&0xFF)) ) return;
@@ -850,6 +973,16 @@ void SessPage::alarmQuittance( ui8 quit_tmpl, bool isSet )
     if( isSet && ownerSessWdg(true) )	ownerSessWdg(true)->alarmSet();
 }
 
+TVariant SessPage::stlReq( Attr &a, const TVariant &vl, bool wr )
+{
+    if( stlLock() ) return vl;
+    string pid = TSYS::strNoSpace(a.cfgTempl());
+    if( pid.empty() ) pid = a.id();
+    if( !wr ) return ownerSess()->stlPropGet( pid, vl.getS() );
+    if( ownerSess()->stlPropSet(pid,vl.getS()) ) return TVariant();
+    return vl;
+}
+
 bool SessPage::cntrCmdGeneric( XMLNode *opt )
 {
     //> Get page info
@@ -873,9 +1006,9 @@ bool SessPage::cntrCmdGeneric( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/wdg/st/open" && enable() && !(parent().at().prjFlags( )&Page::Empty) )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,owner().c_str(),grp().c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,owner().c_str(),grp().c_str(),SEC_RD) )
 	    opt->setText(TSYS::int2str(attrAt("pgOpen").at().getB()));
-	if( ctrChkNode(opt,"set",RWRWR_,owner().c_str(),grp().c_str(),SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,owner().c_str(),grp().c_str(),SEC_WR) )
 	    attrAt("pgOpen").at().setB(atoi(opt->text().c_str()));
     }
     else if( (a_path == "/br/pg_" || a_path == "/page/page") && ctrChkNode(opt) )
@@ -890,13 +1023,12 @@ bool SessPage::cntrCmdGeneric( XMLNode *opt )
     return true;
 }
 
-
 //************************************************
 //* SessWdg: Session page's widget               *
 //************************************************
 SessWdg::SessWdg( const string &iid, const string &iparent, Session *isess ) :
     Widget(iid,iparent), mProc(false), TValFunc(iid+"_wdg",NULL), mMdfClc(0),
-    mSess(isess), inLnkGet(true)
+    mSess(isess), inLnkGet(true), mCalcClk(isess->calcClk())
 {
 
 }
@@ -906,6 +1038,13 @@ SessWdg::~SessWdg( )
 
 }
 
+void SessWdg::preDisable( int flag )
+{
+    if( process() ) setProcess( false );
+
+    Widget::preDisable( flag );
+}
+
 void SessWdg::postEnable( int flag )
 {
     Widget::postEnable(flag);
@@ -977,6 +1116,7 @@ void SessWdg::setProcess( bool val )
 	fio.ioIns( new IO("f_frq",_("Function calculate frequency (Hz)"),IO::Real,IO::Default,"1000",false),0);
 	fio.ioIns( new IO("f_start",_("Function start flag"),IO::Boolean,IO::Default,"0",false),1);
 	fio.ioIns( new IO("f_stop",_("Function stop flag"),IO::Boolean,IO::Default,"0",false),2);
+	fio.ioIns( new IO("this",_("This widget's object for access to user's API"),IO::Object,IO::Default),3);
 	//>>> Add calc widget's attributes
 	vector<string> iwls, als;
 	//>>> Self attributes check
@@ -998,6 +1138,7 @@ void SessWdg::setProcess( bool val )
 		fio.ioAdd( new IO(als[i_a].c_str(),cattr.at().name().c_str(),tp,IO::Output,"",false,("./"+als[i_a]).c_str()) );
 	    }
 	}
+
 	//>>> Include attributes check
 	wdgList(iwls);
 	for( int i_w = 0; i_w < iwls.size(); i_w++ )
@@ -1027,20 +1168,38 @@ void SessWdg::setProcess( bool val )
 	fio.ioAdd( new IO("alarm",_("Alarm"),IO::String,IO::Output,"",false,"./alarm") );
 
 	//>> Compile function
-	try
+	mWorkProg = "";
+	try 
 	{
 	    mWorkProg = SYS->daq().at().at(TSYS::strSepParse(calcLang(),0,'.')).at().
 		compileFunc(TSYS::strSepParse(calcLang(),1,'.'),fio,calcProg(),mod->nodePath('.',true)+";");
-	    //>> Connect to compiled function
+	}
+	catch( TError err )
+	{
+	    //>> Second compile try
+	    try
+	    {
+		fio.setId(TSYS::path2sepstr(path(),'_'));
+		mWorkProg = SYS->daq().at().at(TSYS::strSepParse(calcLang(),0,'.')).at().
+		    compileFunc(TSYS::strSepParse(calcLang(),1,'.'),fio,calcProg(),mod->nodePath('.',true)+";");
+	    }
+	    catch( TError err )
+	    { mess_err(nodePath().c_str(),_("Compile function '%s' by language '%s' for widget error: %s"),fio.id().c_str(),calcLang().c_str(),err.mess.c_str()); }
+	}
+
+	//>> Connect to compiled function
+	if( mWorkProg.size() )
+	{
 	    TValFunc::setFunc(&((AutoHD<TFunction>)SYS->nodeAt(mWorkProg,1)).at());
 	    TValFunc::setUser(ownerSess()->user());
-	}catch( TError err )
-	{ mess_err(nodePath().c_str(),_("Compile function '%s' by language '%s' for widget error: %s"),fio.id().c_str(),calcLang().c_str(),err.mess.c_str()); }
+	    setO( 3, new TCntrNodeObj(AutoHD<TCntrNode>(this),ownerSess()->user()) );
+	}
     }
     if( !val )
     {
 	//>> Free function link
 	mProc = false;
+	ResAlloc res(mCalcRes,true);
 	TValFunc::setFunc(NULL);
     }
 
@@ -1064,24 +1223,24 @@ string SessWdg::ico( )
 
 string SessWdg::owner( )
 {
-    SessWdg *sWdg = ownerSessWdg(false);
-    if( sWdg )	return sWdg->owner();
+    //SessWdg *sWdg = ownerSessWdg(false);
+    //if( sWdg )	return sWdg->owner();
     if( !parent().freeStat() )  return parent().at().owner();
     return Widget::owner();
 }
 
 string SessWdg::grp( )
 {
-    SessWdg *sWdg = ownerSessWdg(false);
-    if( sWdg )	return sWdg->grp();
+    //SessWdg *sWdg = ownerSessWdg(false);
+    //if( sWdg )	return sWdg->grp();
     if( !parent().freeStat() )  return parent().at().grp();
     return Widget::grp();
 }
 
 short SessWdg::permit( )
 {
-    SessWdg *sWdg = ownerSessWdg(false);
-    if( sWdg )	return sWdg->permit();
+    //SessWdg *sWdg = ownerSessWdg(false);
+    //if( sWdg )	return sWdg->permit();
     if( !parent().freeStat() )	return parent().at().permit();
     return Widget::permit();
 }
@@ -1167,8 +1326,7 @@ void SessWdg::alarmSet( bool isSet )
     string aCur = attrAt("alarm").at().getS( );
     int alev = atoi(TSYS::strSepParse(aCur,0,'|').c_str()) & 0xFF;
     int atp  = atoi(TSYS::strSepParse(aCur,3,'|').c_str()) & 0xFF;
-    int aqtp = (aStCur>>16) & 0xFF;
-    if( isSet )	aqtp |= atp;
+    int aqtp = isSet ? atp : (aStCur>>16) & 0xFF;
 
     vector<string> lst;
     //> Included widgets process
@@ -1187,7 +1345,7 @@ void SessWdg::alarmSet( bool isSet )
     if( isSet )	ownerSess( )->alarmSet( path(), aCur );
 }
 
-void SessWdg::alarmQuittance( ui8 quit_tmpl, bool isSet )
+void SessWdg::alarmQuittance( uint8_t quit_tmpl, bool isSet )
 {
     int alarmSt = attrAt("alarmSt").at().getI();
     if( !((((alarmSt>>16)&0xFF)^quit_tmpl)&((alarmSt>>16)&0xFF)) ) return;
@@ -1227,7 +1385,7 @@ void SessWdg::prcElListUpdate( )
 void SessWdg::getUpdtWdg( const string &path, unsigned int tm, vector<string> &els )
 {
     string wpath = path+"/"+id();
-    if( modifChk(tm) ) els.push_back(wpath);
+    if( modifChk(tm,mMdfClc) ) els.push_back(wpath);
     for( int i_ch = 0; i_ch < mWdgChldAct.size(); i_ch++ )
 	if( wdgPresent(mWdgChldAct[i_ch]) )
 	    wdgAt(mWdgChldAct[i_ch]).at().getUpdtWdg(wpath,tm,els);
@@ -1235,15 +1393,13 @@ void SessWdg::getUpdtWdg( const string &path, unsigned int tm, vector<string> &e
 
 unsigned int SessWdg::modifVal( Attr &cfg )
 {
-    int m_clc = ownerSess()->calcClk( );
-    if( atoi(cfg.fld().reserve().c_str()) ) mMdfClc = m_clc;
-    return m_clc;
+    if( atoi(cfg.fld().reserve().c_str()) ) mMdfClc = mCalcClk;
+    return mCalcClk;
 }
 
-bool SessWdg::modifChk( unsigned int tm )
+bool SessWdg::modifChk( unsigned int tm, unsigned int iMdfClc )
 {
-    int m_clc = ownerSess()->calcClk( );
-    return (m_clc>tm) ? (mMdfClc >= tm && mMdfClc <= m_clc) : (mMdfClc >= tm || mMdfClc <= m_clc);
+    return (mCalcClk>=tm) ? (iMdfClc >= tm && iMdfClc <= mCalcClk) : (iMdfClc >= tm || iMdfClc <= mCalcClk);
 }
 
 void SessWdg::calc( bool first, bool last )
@@ -1263,6 +1419,8 @@ void SessWdg::calc( bool first, bool last )
 
     try
     {
+	int pgOpenPrc = -1;
+
 	//> Load events to process
 	if( !((ownerSess()->calcClk())%(vmax(calcPer()/ownerSess()->period(),1))) || first || last )
 	{
@@ -1289,7 +1447,7 @@ void SessWdg::calc( bool first, bool last )
 			else switch( attr.at().type() )
 			{
 			    case TFld::Boolean:	attr.at().setB(vl.at().getB());	break;
-			    case TFld::Integer: attr.at().setI(vl.at().getI());	break;
+			    case TFld::Integer:	attr.at().setI(vl.at().getI());	break;
 			    case TFld::Real:	attr.at().setR(vl.at().getR());	break;
 			    case TFld::String:	attr.at().setS(vl.at().getS());	break;
 			}
@@ -1322,7 +1480,7 @@ void SessWdg::calc( bool first, bool last )
 		setR(0,1000.0/(ownerSess()->period()*vmax(calcPer()/ownerSess()->period(),1)));
 		setB(1,first);
 		setB(2,last);
-		for( int i_io = 3; i_io < ioSize( ); i_io++ )
+		for( int i_io = 4; i_io < ioSize( ); i_io++ )
 		{
 		    if( func()->io(i_io)->rez().empty() ) continue;
 		    sw_attr = TSYS::pathLev(func()->io(i_io)->rez(),0);
@@ -1332,19 +1490,23 @@ void SessWdg::calc( bool first, bool last )
 		    {
 			case IO::String:	setS(i_io,attr.at().getS());	break;
 			case IO::Integer:	setI(i_io,attr.at().getI());	break;
-			case IO::Real:	setR(i_io,attr.at().getR());	break;
+			case IO::Real:		setR(i_io,attr.at().getR());	break;
 			case IO::Boolean:	setB(i_io,attr.at().getB());	break;
 		    }
 		}
+
 		//>> Calc
+		setMdfChk( true );
 		TValFunc::calc();
+
 		//>> Load data from calc area
-		for( int i_io = 3; i_io < ioSize( ); i_io++ )
+		for( int i_io = 4; i_io < ioSize( ); i_io++ )
 		{
-		    if( func()->io(i_io)->rez().empty() ) continue;
+		    if( func()->io(i_io)->rez().empty() || !ioMdf(i_io) ) continue;
 		    sw_attr = TSYS::pathLev(func()->io(i_io)->rez(),0);
 		    s_attr  = TSYS::pathLev(func()->io(i_io)->rez(),1);
 		    attr = (sw_attr==".")?attrAt(s_attr):wdgAt(sw_attr).at().attrAt(s_attr);
+		    if( s_attr == "pgOpen" && attr.at().getB() != getB(i_io) ) { pgOpenPrc = i_io; continue; }
 		    switch(ioType(i_io))
 		    {
 			case IO::String:	attr.at().setS(getS(i_io));	break;
@@ -1357,6 +1519,11 @@ void SessWdg::calc( bool first, bool last )
 		if( evId >= 0 ) wevent = getS(evId);
 	    }
 
+	    res.release();
+
+	    //> Close page process by pgOpen changing
+	    if( pgOpenPrc >= 0 ) attrAt("pgOpen").at().setB(getB(pgOpenPrc));
+
 	    //>> Process widget's events
 	    if( !wevent.empty() )
 	    {
@@ -1461,21 +1628,67 @@ bool SessWdg::attrChange( Attr &cfg, TVariant prev )
 		}
 	}catch(...)	{ }
     }
+
     return true;
 }
 
+TVariant SessWdg::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    if( iid == "ownerSess" )	return new TCntrNodeObj(ownerSess(),user);
+    if( iid == "ownerPage" )
+    {
+	SessPage *opg = ownerPage();
+	if( !opg ) return 0;
+	return new TCntrNodeObj(opg,user);
+    }
+    if( iid == "ownerWdg" )
+    {
+	SessWdg *wdg = ownerSessWdg( prms.size() ? prms[0].getB() : 0 );
+	if( !wdg ) return 0;
+	return new TCntrNodeObj(wdg,user);
+    }
+    if( iid == "attrPresent" && prms.size() )	return attrPresent( prms[0].getS() );
+    if( iid == "attr" && prms.size() )
+    {
+	if( !attrPresent( prms[0].getS() ) ) return string("");
+	AutoHD<Attr> attr = attrAt(prms[0].getS());
+	switch( attr.at().type() )
+	{
+	    case TFld::String:	return attr.at().getS();
+	    case TFld::Integer:	return attr.at().getI();
+	    case TFld::Real:	return attr.at().getR();
+	    case TFld::Boolean:	return attr.at().getB();
+	}
+	return string("");
+    }
+    if( iid == "attrSet" && prms.size() >= 2 )
+    {
+	if( !attrPresent( prms[0].getS() ) ) return -1;
+	AutoHD<Attr> attr = attrAt(prms[0].getS());
+	switch( attr.at().type() )
+	{
+	    case TFld::String:	attr.at().setS(prms[1].getS());	break;
+	    case TFld::Integer:	attr.at().setI(prms[1].getI());	break;
+	    case TFld::Real:	attr.at().setR(prms[1].getR());	break;
+	    case TFld::Boolean:	attr.at().setB(prms[1].getB());	break;
+	}
+	return 0;
+    }
+    return TCntrNode::objFuncCall(iid,prms,user);
+}
+
 bool SessWdg::cntrCmdServ( XMLNode *opt )
 {
     string a_path = opt->attr("path");
     if( a_path == "/serv/attr" )		//Attribute's value operations
     {
-	if( ctrChkNode(opt,"get",R_R_R_,"root","UI",SEQ_RD) )	//Get values
+	if( ctrChkNode(opt,"get",R_R_R_,"root","UI",SEC_RD) )	//Get values
 	{
 	    unsigned  tm = strtoul(opt->attr("tm").c_str(),0,10);
 	    if( !tm )
 		opt->childAdd("el")->setAttr("id","perm")->setAttr("p","-3")->
-		    setText(TSYS::int2str(ownerSess()->sec.at().access(opt->attr("user"),SEQ_RD|SEQ_WR,owner(),grp(),permit())) );
-	    if( !tm || modifChk(tm) )
+		    setText(TSYS::int2str(ownerSess()->sec.at().access(opt->attr("user"),SEC_RD|SEC_WR,owner(),grp(),permit())) );
+	    if( !tm || modifChk(tm,mMdfClc) )
 	    {
 		AutoHD<Attr> attr;
 		vector<string> als;
@@ -1483,28 +1696,31 @@ bool SessWdg::cntrCmdServ( XMLNode *opt )
 		for( int i_l = 0; i_l < als.size(); i_l++ )
 		{
 		    attr = attrAt(als[i_l]);
-		    if( !(attr.at().flgGlob()&Attr::IsUser) && attr.at().modif() >= tm && atoi(attr.at().fld().reserve().c_str()) )
+		    if( !(attr.at().flgGlob()&Attr::IsUser) && modifChk(tm,attr.at().modif()) && atoi(attr.at().fld().reserve().c_str()) )
 			opt->childAdd("el")->setAttr("id",als[i_l].c_str())->
 					     setAttr("p",attr.at().fld().reserve())->
 					     setText(attr.at().getS());
 		}
 	    }
 	}
-	else if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEQ_WR) )	//Set values
+	else if( ctrChkNode(opt,"set",permit(),owner().c_str(),grp().c_str(),SEC_WR) )	//Set values
+	{
+	    if( ownerSess()->user() != opt->attr("user") ) ownerSess()->setUser(opt->attr("user"));
 	    for( int i_ch = 0; i_ch < opt->childSize(); i_ch++ )
 	    {
 	        string aid = opt->childGet(i_ch)->attr("id");
 	        if( aid == "event" ) eventAdd(opt->childGet(i_ch)->text()+"\n");
 	        else attrAt(aid).at().setS(opt->childGet(i_ch)->text());
 	    }
+	}
     }
-    else if( a_path == "/serv/attrBr" && ctrChkNode(opt,"get",R_R_R_,"root","UI",SEQ_RD) )	//Get attributes all updated elements' of the branch
+    else if( a_path == "/serv/attrBr" && ctrChkNode(opt,"get",R_R_R_,"root","UI",SEC_RD) )	//Get attributes all updated elements' of the branch
     {
 	unsigned tm = strtoul(opt->attr("tm").c_str(),NULL,10);
-	int perm = ownerSess()->sec.at().access(opt->attr("user"),(tm?SEQ_RD:SEQ_RD|SEQ_WR),owner(),grp(),permit());
+	int perm = ownerSess()->sec.at().access(opt->attr("user"),(tm?SEC_RD:SEC_RD|SEC_WR),owner(),grp(),permit());
 
 	//>> Self attributes put
-	if( !tm || modifChk(tm) )
+	if( !tm || modifChk(tm,mMdfClc) )
 	{
 	    if( !tm )
 		opt->childAdd("el")->setAttr("id","perm")->setAttr("p","-3")->setText(TSYS::int2str(perm));
@@ -1514,7 +1730,7 @@ bool SessWdg::cntrCmdServ( XMLNode *opt )
 	    for( int i_l = 0; i_l < als.size(); i_l++ )
 	    {
 		attr = attrAt(als[i_l]);
-		if( !(attr.at().flgGlob()&Attr::IsUser) && attr.at().modif() >= tm && atoi(attr.at().fld().reserve().c_str()) )
+		if( !(attr.at().flgGlob()&Attr::IsUser) && modifChk(tm,attr.at().modif()) && atoi(attr.at().fld().reserve().c_str()) )
 		    opt->childAdd("el")->setAttr("id",als[i_l].c_str())->
 				     setAttr("p",attr.at().fld().reserve())->
 				     setText(attr.at().getS());
@@ -1522,7 +1738,7 @@ bool SessWdg::cntrCmdServ( XMLNode *opt )
 	}
 
 	//>> Child widgets process
-	if( enable() && perm&SEQ_RD )
+	if( enable() && perm&SEC_RD )
 	{
 	    vector<string>	lst;
 	    wdgList(lst);
@@ -1561,8 +1777,8 @@ bool SessWdg::cntrCmdGeneric( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/wdg/st/proc" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,owner().c_str(),grp().c_str(),SEQ_RD) ) opt->setText(TSYS::int2str(process()));
-	if( ctrChkNode(opt,"set",RWRWR_,owner().c_str(),grp().c_str(),SEQ_WR) ) setProcess(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,owner().c_str(),grp().c_str(),SEC_RD) ) opt->setText(TSYS::int2str(process()));
+	if( ctrChkNode(opt,"set",RWRWR_,owner().c_str(),grp().c_str(),SEC_WR) ) setProcess(atoi(opt->text().c_str()));
     }
     else return Widget::cntrCmdGeneric(opt);
 
@@ -1594,12 +1810,12 @@ bool SessWdg::cntrCmdAttributes( XMLNode *opt )
     if( a_path.substr(0,6) == "/attr/" )
     {
 	AutoHD<Attr> attr = attrAt(TSYS::pathLev(a_path,1));
-	if( ctrChkNode(opt,"get",((attr.at().fld().flg()&TFld::NoWrite)?(permit()&~0222):permit())|R_R_R_,owner().c_str(),grp().c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",((attr.at().fld().flg()&TFld::NoWrite)?(permit()&~0222):permit())|R_R_R_,owner().c_str(),grp().c_str(),SEC_RD) )
 	{
 	    if( attr.at().fld().flg()&TFld::Selected )  opt->setText(attr.at().getSEL());
 	    else                                        opt->setText(attr.at().getS());
 	}
-	if( ctrChkNode(opt,"set",((attr.at().fld().flg()&TFld::NoWrite)?(permit()&~0222):permit())|R_R_R_,owner().c_str(),grp().c_str(),SEQ_WR) )
+	if( ctrChkNode(opt,"set",((attr.at().fld().flg()&TFld::NoWrite)?(permit()&~0222):permit())|R_R_R_,owner().c_str(),grp().c_str(),SEC_WR) )
 	{
 	    if( attr.at().id() == "event" )	eventAdd(opt->text()+"\n");
 	    else if( attr.at().fld().flg()&TFld::Selected )
