@@ -22,6 +22,8 @@
 #ifndef SESSION_H
 #define SESSION_H
 
+#include <stdint.h>
+
 #include <tcntrnode.h>
 #include <tconfig.h>
 
@@ -56,8 +58,9 @@ class Session : public TCntrNode
 	bool   start( )		{ return mStart; }		//Start stat
 	bool   backgrnd( )	{ return mBackgrnd; }		//Background session execution
 	int    connects( )	{ return mConnects; }		//Connections counter
-	unsigned calcClk( )	{ return mCalcClk; }		//Calc clock
+	unsigned &calcClk( )	{ return mCalcClk; }		//Calc clock
 	AutoHD<Project> parent( );
+	int stlCurent( )	{ return mStyleIdW; }
 
 	void setProjNm( const string &it )	{ mPrjnm = it; }
 	void setUser( const string &it );
@@ -67,8 +70,11 @@ class Session : public TCntrNode
 	void setBackgrnd( bool val )		{ mBackgrnd = val; }
 	void connect( )				{ mConnects++; }
 	void disconnect( )			{ if(mConnects>0) mConnects--; }
+	void stlCurentSet( int sid );
 
-	//- Pages -
+	bool modifChk( unsigned int tm, unsigned int iMdfClc );
+
+	//> Pages
 	void list( vector<string> &ls ) 	{ chldList(mPage,ls); }
 	bool present( const string &id )	{ return chldPresent(mPage,id); }
 	AutoHD<SessPage> at( const string &id );
@@ -83,10 +89,14 @@ class Session : public TCntrNode
 
 	void uiComm( const string &com, const string &prm, SessWdg *src = NULL );
 
-	//- Alarms process -
+	//> Alarms process
 	void alarmSet( const string &wpath, const string &alrm );	//Alarm set
 	int  alarmStat( );						//Alarm status
-	void alarmQuittance( const string &wpath, ui8 quit_tmpl );	//Alarm quittance send
+	void alarmQuittance( const string &wpath, uint8_t quit_tmpl );	//Alarm quittance send
+
+	//> Style
+	string stlPropGet( const string &pid, const string &def = "" );
+	bool stlPropSet( const string &pid, const string &vl );
 
 	//Attributes
 	AutoHD<TSecurity> sec;
@@ -96,6 +106,8 @@ class Session : public TCntrNode
 	string nodeName( )	{ return mId; }
 	void cntrCmdProc( XMLNode *opt );				//Control interface command process
 
+	TVariant objFuncCall( const string &id, vector<TVariant> &prms, const string &user );
+
 	void postEnable( int flag );
 	void preDisable( int flag );
 
@@ -105,13 +117,13 @@ class Session : public TCntrNode
 	{
 	    public:
 		//Methods
-		Alarm( const string &ipath, ui8 ilev, ui8 itp, const string &icat, const string &imess, const string &itpArg = "", unsigned iclc = 0 ) :
+		Alarm( const string &ipath, uint8_t ilev, uint8_t itp, const string &icat, const string &imess, const string &itpArg = "", unsigned iclc = 0 ) :
 		    path(ipath), lev(ilev), tp(itp), cat(icat), mess(imess), tpArg(itpArg), clc(iclc)	{ }
 		Alarm( const string &path, const string &alrm, unsigned clc );
 		Alarm( ) : lev(0), tp(0), qtp(0)	{ }
 
 		//Attributes
-		ui8	    lev,	//Level
+		uint8_t	    lev,	//Level
 			    tp,		//Type
 			    qtp;	//Quitance type
 		string	    path,	//Widget path
@@ -132,7 +144,6 @@ class Session : public TCntrNode
 	bool	mBackgrnd;			//Backgrounded execution of a session
 	int	mConnects;			//Connections counter
 
-	pthread_t	calcPthr;		//Calc pthread
 	unsigned	mCalcClk;		//Calc clock
 	float		tm_calc;		//Scheme's calc time
 	float		rez_calc;
@@ -144,6 +155,12 @@ class Session : public TCntrNode
 
 	Res		mAlrmRes;		//Alarms resource
 	vector<Alarm>	mAlrm;			//Alarms queue
+	int		mAlrmSndPlay;		//Now played sound alarm
+
+	//> Styles
+	Res		mStRes;
+	int		mStyleIdW;
+	map<string,string>	mStProp;	//Styles' properties
 };
 
 //************************************************
@@ -186,7 +203,7 @@ class SessWdg : public Widget, public TValFunc
 
 	//> Alarms process
 	virtual void alarmSet( bool isSet = false );
-	virtual void alarmQuittance( ui8 quit_tmpl, bool isSet = false );
+	virtual void alarmQuittance( uint8_t quit_tmpl, bool isSet = false );
 
 	//> Access to mime resource
 	string resourceGet( const string &id, string *mime = NULL );
@@ -200,6 +217,7 @@ class SessWdg : public Widget, public TValFunc
     protected:
 	//Methods
 	void postEnable( int flag );
+	void preDisable( int flag );
 
 	bool cntrCmdServ( XMLNode *opt );
 	bool cntrCmdGeneric( XMLNode *opt );
@@ -207,16 +225,19 @@ class SessWdg : public Widget, public TValFunc
 	void cntrCmdProc( XMLNode *opt );			//Control interface command process
 	bool attrChange( Attr &cfg, TVariant prev );
 
+	TVariant objFuncCall( const string &id, vector<TVariant> &prms, const string &user );
+
 	unsigned int modifVal( Attr &cfg );
-	bool modifChk( unsigned int tm );
+	bool modifChk( unsigned int tm, unsigned int iMdfClc );
 
     private:
 	//Attributes
-	char		mProc	: 1;
-	char		inLnkGet: 1;
-	
+	char		mProc		: 1;
+	char		inLnkGet	: 1;
+
 	string		mWorkProg;
 	unsigned int	mMdfClc;
+	unsigned int	&mCalcClk;
 	Res		mCalcRes;
 
 	vector<string>	mWdgChldAct,	//Active childs widget's list
@@ -245,26 +266,28 @@ class SessPage : public SessWdg
 
 	AutoHD<Page> parent( );
 
-	//- Pages -
+	//> Pages
 	void pageList( vector<string> &ls )			{ chldList(mPage,ls); }
 	bool pagePresent( const string &id )			{ return chldPresent(mPage,id); }
 	AutoHD<SessPage> pageAt( const string &id );
 	void pageAdd( const string &id, const string &parent = "" );
 	void pageDel( const string &id, bool full = false )	{ chldDel(mPage,id,-1,full); }
 
-	//- Alarms process -
+	//> Alarms process
 	void alarmSet( bool isSet = false );
-	void alarmQuittance( ui8 quit_tmpl, bool isSet = false );
+	void alarmQuittance( uint8_t quit_tmpl, bool isSet = false );
 
     protected:
 	//Methods
 	bool cntrCmdGeneric( XMLNode *opt );
 
 	bool attrChange( Attr &cfg, TVariant prev );
+	TVariant stlReq( Attr &a, const TVariant &vl, bool wr );
 
     private:
 	//Attributes
-	int	mPage;		//Pages container identifier
+	unsigned mPage		: 4;		//Pages container identifier
+	unsigned mClosePgCom	: 1;
 };
 
 }
