@@ -37,7 +37,7 @@
 #define MOD_TYPE	"UI"
 #define MOD_SUBTYPE	"VCAEngine"
 #define VER_TYPE	VER_UI
-#define VERSION		"0.8.10"
+#define VERSION		"0.9.6"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Generic visual control area engine."
 #define LICENSE		"GPL"
@@ -66,7 +66,7 @@ using namespace VCA;
 //************************************************
 //* Engine                                       *
 //************************************************
-Engine::Engine( string name ) : mSynthCom("echo \"%t\" | ru_tts | sox -t raw -s -b -r 10000 -c 1 -v 0.8 - -t ogg -")
+Engine::Engine( string name ) : mSynthCom("echo \"%t\" | ru_tts | sox -t raw -s -b -r 10000 -c 1 -v 0.8 - -t ogg -"), mFrcClr(false)
 {
     mId		= MOD_ID;
     mName	= MOD_NAME;
@@ -108,7 +108,7 @@ string Engine::optDescr( )
 
     snprintf(buf,sizeof(buf),_(
 	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
+	"    --VCADBClearForce            Force clear VCA DB from data of API 1.\n\n"),
 	    MOD_TYPE,MOD_ID,nodePath().c_str());
 
     return buf;
@@ -142,33 +142,37 @@ void Engine::postEnable( int flag )
     wdg_el.fldAdd( new TFld("GRP",_("Group"),TFld::String,TFld::NoFlag,"20","UI") );
     wdg_el.fldAdd( new TFld("PERMIT",_("Permision"),TFld::Integer,TFld::OctDec,"3","436") );
     wdg_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
+    wdg_el.fldAdd( new TFld("DBV",_("DB version"),TFld::Integer,TFld::NoFlag,"1","1") );
 
     //> Make include widgets' DB structure
     inclwdg_el.fldAdd( new TFld("IDW",_("IDW"),TFld::String,TCfg::Key,"100") );
     inclwdg_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
     inclwdg_el.fldAdd( new TFld("PARENT",_("Parent widget"),TFld::String,TFld::NoFlag,"200") );
-    inclwdg_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
     inclwdg_el.fldAdd( new TFld("USER",_("User"),TFld::String,TFld::NoFlag,"20","root") );
     inclwdg_el.fldAdd( new TFld("GRP",_("Group"),TFld::String,TFld::NoFlag,"20","UI") );
     inclwdg_el.fldAdd( new TFld("PERMIT",_("Permision"),TFld::Integer,TFld::OctDec,"3","436") );
+    inclwdg_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
+    inclwdg_el.fldAdd( new TFld("DBV",_("DB version"),TFld::Integer,TFld::NoFlag,"1","1") );
 
     //> Make widget's IO DB structure
     wdgio_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"100") );
+    wdgio_el.fldAdd( new TFld("IDC",_("Child ID"),TFld::String,TCfg::Key,"30") );
     wdgio_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"61") );
     wdgio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TCfg::TransltText,"100000") );
     wdgio_el.fldAdd( new TFld("SELF_FLG",_("Attribute self flags"),TFld::Integer,TFld::NoFlag,"5") );
     wdgio_el.fldAdd( new TFld("CFG_TMPL",_("Configuration template"),TFld::String,TCfg::TransltText,"30") );
-    wdgio_el.fldAdd( new TFld("CFG_VAL",_("Configuration value"),TFld::String,TFld::NoFlag,"1000") );
+    wdgio_el.fldAdd( new TFld("CFG_VAL",_("Configuration value"),TFld::String,TCfg::TransltText,"1000") );
 
     //> Make widget's user IO DB structure
     wdguio_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"100") );
+    wdguio_el.fldAdd( new TFld("IDC",_("Child ID"),TFld::String,TCfg::Key,"30") );
     wdguio_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"61") );
-    wdguio_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    wdguio_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"100") );
     wdguio_el.fldAdd( new TFld("IO_TYPE",_("Attribute generic flags and type"),TFld::Integer,TFld::NoFlag,"10") );
     wdguio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TCfg::TransltText,"100000") );
     wdguio_el.fldAdd( new TFld("SELF_FLG",_("Attribute self flags"),TFld::Integer,TFld::NoFlag,"5") );
     wdguio_el.fldAdd( new TFld("CFG_TMPL",_("Configuration template"),TFld::String,TCfg::TransltText,"30") );
-    wdguio_el.fldAdd( new TFld("CFG_VAL",_("Configuration value"),TFld::String,TFld::NoFlag,"1000") );
+    wdguio_el.fldAdd( new TFld("CFG_VAL",_("Configuration value"),TFld::String,TCfg::TransltText,"1000") );
 
     //> Make project's DB structure
     prj_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
@@ -180,6 +184,7 @@ void Engine::postEnable( int flag )
     prj_el.fldAdd( new TFld("GRP",_("Group"),TFld::String,TFld::NoFlag,"20","UI") );
     prj_el.fldAdd( new TFld("PERMIT",_("Permision"),TFld::Integer,TFld::OctDec,"3","436") );
     prj_el.fldAdd( new TFld("PER",_("Calc period (ms)"),TFld::Integer,TFld::NoFlag,"4","100") );
+    prj_el.fldAdd( new TFld("STYLE",_("Work style"),TFld::Integer,TFld::NoFlag,"2","-1") );
     prj_el.fldAdd( new TFld("FLGS",_("Flags"),TFld::Integer,TFld::NoFlag,"4") );
 
     //> Make pages DB structure
@@ -194,12 +199,26 @@ void Engine::postEnable( int flag )
     page_el.fldAdd( new TFld("PERMIT",_("Permision"),TFld::Integer,TFld::OctDec,"3","436") );
     page_el.fldAdd( new TFld("FLGS",_("Flags"),TFld::Integer,TFld::NoFlag,"1","0") );
     page_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
+    page_el.fldAdd( new TFld("DBV",_("DB version"),TFld::Integer,TFld::NoFlag,"1","1") );
 
     //> Make sessions' IO values of projects DB structure
     prj_ses_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"200") );
     prj_ses_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
     prj_ses_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TFld::NoFlag,"100000") );
 
+    //> Make styles' IO DB structure
+    prjStl_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
+    prjStl_el.fldAdd( new TFld("V_0",_("Value 0"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_1",_("Value 1"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_2",_("Value 2"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_3",_("Value 3"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_4",_("Value 4"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_5",_("Value 5"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_6",_("Value 6"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_7",_("Value 7"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_8",_("Value 8"),TFld::String,TFld::NoFlag,"100") );
+    prjStl_el.fldAdd( new TFld("V_9",_("Value 9"),TFld::String,TFld::NoFlag,"100") );
+
     //> Init original widgets library
     wlbAdd("originals",_("Original widget's library"));
     //>> Set default library icon
@@ -222,6 +241,7 @@ void Engine::postEnable( int flag )
     chldAdd( idFnc, new attrList() );
     chldAdd( idFnc, new attrGet() );
     chldAdd( idFnc, new attrSet() );
+    chldAdd( idFnc, new sesUser() );
 }
 
 void Engine::preDisable( int flag )
@@ -241,6 +261,7 @@ void Engine::load_( )
     struct option long_opt[] =
     {
 	{"help"    ,0,NULL,'h'},
+	{"VCADBClearForce" ,0,NULL,'f'},
 	{NULL      ,0,NULL,0  }
     };
 
@@ -250,7 +271,8 @@ void Engine::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
+	    case 'f': mFrcClr = true; break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -259,6 +281,10 @@ void Engine::load_( )
     setSynthCom( TBDS::genDBGet(nodePath()+"SynthCom",synthCom()) );
     setSynthCode( TBDS::genDBGet(nodePath()+"SynthCode",synthCode()) );
 
+#if OSC_DEBUG >= 3
+    long long w_tm = TSYS::curTime();
+#endif
+
     //>> Load widget's libraries
     try
     {
@@ -284,10 +310,20 @@ void Engine::load_( )
 		if( !wlbPresent(l_id) )	wlbAdd(l_id,"","*.*");
 	    }
 
+#if OSC_DEBUG >= 3
+	w_tm = TSYS::curTime();
+#endif
+
 	//>>>> Load present libraries
 	wlbList(db_ls);
 	for( int l_id = 0; l_id < db_ls.size(); l_id++ )
+	{
 	    wlbAt(db_ls[l_id]).at().load();
+#if OSC_DEBUG >= 3
+	    mess_debug(nodePath().c_str(),_("Load library '%s' time: %f ms."),db_ls[l_id].c_str(),1e-3*(TSYS::curTime()-w_tm));
+	    w_tm = TSYS::curTime();
+#endif
+	}
     }catch( TError err )
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -319,10 +355,20 @@ void Engine::load_( )
 		if( !prjPresent(prj_id) )	prjAdd(prj_id,"","*.*");
 	    }
 
+#if OSC_DEBUG >= 3
+	w_tm = TSYS::curTime();
+#endif
+
 	//>>>> Load present projects
 	prjList(db_ls);
 	for( int el_id = 0; el_id < db_ls.size(); el_id++ )
+	{
 	    prjAt(db_ls[el_id]).at().load();
+#if OSC_DEBUG >= 3
+	    mess_debug(nodePath().c_str(),_("Load project '%s' time: %f ms."),db_ls[el_id].c_str(),1e-3*(TSYS::curTime()-w_tm));
+	    w_tm = TSYS::curTime();
+#endif
+	}
     }catch( TError err )
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -333,12 +379,24 @@ void Engine::load_( )
     vector<string> ls;
     wlbList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
+    {
 	wlbAt(ls[l_id]).at().setEnable(true);
+#if OSC_DEBUG >= 3
+	mess_debug(nodePath().c_str(),_("Enable library '%s' time: %f ms."),ls[l_id].c_str(),1e-3*(TSYS::curTime()-w_tm));
+	w_tm = TSYS::curTime();
+#endif
+    }
 
     //> Projects enable
     prjList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
+    {
 	prjAt(ls[l_id]).at().setEnable(true);
+#if OSC_DEBUG >= 3
+	mess_debug(nodePath().c_str(),_("Enable project '%s' time: %f ms."),ls[l_id].c_str(),1e-3*(TSYS::curTime()-w_tm));
+	w_tm = TSYS::curTime();
+#endif
+    }
 
     modifGClr( );
 }
@@ -347,7 +405,7 @@ void Engine::save_( )
 {
     mess_info(nodePath().c_str(),_("Save module."));
 
-    //- Save parameters to DB -
+    //> Save parameters to DB
     TBDS::genDBSet( nodePath()+"SynthCom", synthCom() );
     TBDS::genDBSet( nodePath()+"SynthCode", synthCode() );
 }
@@ -356,13 +414,13 @@ void Engine::modStart()
 {
     mess_info(nodePath().c_str(),_("Start module."));
 
-    //- Libraries enable -
+    //> Libraries enable
     vector<string> ls;
     wlbList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
 	wlbAt(ls[l_id]).at().setEnable(true);
 
-    //- Projects enable -
+    //> Projects enable
     prjList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
 	prjAt(ls[l_id]).at().setEnable(true);
@@ -444,13 +502,13 @@ string Engine::callSynth( const string &itxt )
 
     string com = synthCom( );
 
-    //- Put text to command -
+    //> Put text to command
     bool textToPipe = false;
     if( (comPos=com.find("%t")) != string::npos )
 	com.replace( comPos, 2, txt );
     else textToPipe = true;
 
-    //- Put result file name to command -
+    //> Put result file name to command
     bool rezFromPipe = false;
     if( (comPos=com.find("%f")) != string::npos )
 	com.replace( comPos, 2, synthRez );
@@ -458,17 +516,17 @@ string Engine::callSynth( const string &itxt )
 
     if( rezFromPipe && textToPipe )	return "";
 
-    //- Open pipe -
+    //> Open pipe
     FILE *fp = popen( com.c_str(), textToPipe?"w":"r" );
     if( !fp )	return "";
-    //- Write text to pipe -
+    //> Write text to pipe
     if( textToPipe )	fwrite( txt.c_str(), txt.size(), 1, fp );
-    //- Read result from pipe -
+    //> Read result from pipe
     if( rezFromPipe )
 	while( comPos = fread( buf, 1, sizeof(buf), fp ) )
 	    rez.append(buf,comPos);
     pclose(fp);
-    //- Read result from result file -
+    //> Read result from result file
     if( !rezFromPipe )
     {
 	FILE *fp = fopen( synthRez, "r" );
@@ -482,13 +540,170 @@ string Engine::callSynth( const string &itxt )
     return TSYS::strEncode( rez, TSYS::base64 );
 }
 
+void Engine::attrsLoad( Widget &w, const string &fullDB, int vDB, const string &idw, const string &idc, const string &attrs, bool ldGen )
+{
+    string wdb = fullDB+"_io";
+    string tbl = TSYS::strSepParse(wdb,2,';');
+
+    if( vDB < 1 || vDB > 2 ) return;
+
+    TConfig c_el(&elWdgIO());
+    c_el.cfg("IDW").setS(idw);
+    if( vDB == 2 ) c_el.cfg("IDC").setS(idc);
+    string tstr;
+
+    for( int off = 0; !(tstr = TSYS::strSepParse(attrs,0,';',&off)).empty(); )
+    {
+	if( !w.attrPresent(tstr) ) continue;
+	AutoHD<Attr> attr = w.attrAt(tstr);
+
+	if( (ldGen && !(attr.at().flgGlob()&Attr::Generic)) ||
+		(!ldGen && (attr.at().flgGlob()&Attr::Generic || (!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser))) )
+	    continue;
+
+	if( vDB == 1 ) c_el.cfg("ID").setS( idc.empty() ? tstr : idc+"/"+tstr );
+	else if( vDB == 2 ) c_el.cfg("ID").setS(tstr);
+
+	if( !SYS->db().at().dataGet(wdb,nodePath()+tbl,c_el) ) continue;
+	attr.at().setS(c_el.cfg("IO_VAL").getS(),true);
+	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
+	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
+	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
+    }
+
+    if( ldGen )	return;
+
+    //> Load widget's user attributes
+    wdb = fullDB+"_uio";
+    tbl = TSYS::strSepParse(wdb,2,';');
+    c_el.setElem(&elWdgUIO());
+    c_el.cfg("IDW").setS(idw,true);
+    if( vDB == 2 ) c_el.cfg("IDC").setS(idc,true);
+
+    for( int fld_cnt = 0; SYS->db().at().dataSeek(wdb,nodePath()+tbl,fld_cnt++,c_el); )
+    {
+	string sid = c_el.cfg("ID").getS();
+	unsigned flg = c_el.cfg("IO_TYPE").getI();
+
+	if( vDB == 1 )
+	{
+	    if( idc.empty() && !TSYS::pathLev(sid,1).empty() ) continue;
+	    if( !idc.empty() )
+	    {
+		if( TSYS::pathLev(sid,0) == idc && !TSYS::pathLev(sid,1).empty() ) sid = TSYS::pathLev(sid,1);
+		else continue;
+	    }
+	}
+	else if( vDB == 2 && !TSYS::pathLev(sid,1).empty() ) continue;
+
+	if( !w.attrPresent(sid) )
+	    w.attrAdd( new TFld(sid.c_str(),c_el.cfg("NAME").getS().c_str(),(TFld::Type)(flg&0x0f),flg>>4) );
+	AutoHD<Attr> attr = w.attrAt(sid);
+	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
+	attr.at().setS(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),0,'|'));
+	attr.at().fld().setValues(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),1,'|'));
+	attr.at().fld().setSelNames(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),2,'|'));
+	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
+	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
+	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
+    }
+}
+
+string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string &idw, const string &idc, bool ldGen )
+{
+    string tbl = TSYS::strSepParse(fullDB,2,';');
+    string m_attrs = "";
+    vector<string> als;
+
+    if( vDB < 1 || vDB > 2 ) return m_attrs;
+
+    w.attrList( als );
+    TConfig c_el(&mod->elWdgIO()); c_el.cfg("IDW").setS(idw,true);
+    if( vDB == 2 ) c_el.cfg("IDC").setS(idc,true);
+    TConfig c_elu(&mod->elWdgUIO()); c_elu.cfg("IDW").setS(idw,true);
+    if( vDB == 2 ) c_elu.cfg("IDC").setS(idc,true);
+    for( int i_a = 0; i_a < als.size(); i_a++ )
+    {
+	AutoHD<Attr> attr = w.attrAt(als[i_a]);
+	if( !attr.at().modif() ) continue;
+	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) m_attrs += als[i_a]+";";
+	if( ldGen != (bool)(attr.at().flgGlob()&Attr::Generic) ) continue;
+
+	//> Main attributes store
+	if( attr.at().flgSelf()&Attr::IsInher || !(attr.at().flgGlob()&Attr::IsUser) )
+	{
+	    if( vDB == 1 ) c_el.cfg("ID").setS( idc.empty() ? als[i_a] : idc+"/"+als[i_a] );
+	    if( vDB == 2 ) c_el.cfg("ID").setS( als[i_a] );
+	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
+		    !(attr.at().flgGlob()&(Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address))) );
+	    c_el.cfg("IO_VAL").setS(attr.at().getS());
+	    c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
+	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
+	    c_el.cfg("CFG_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
+		    !(attr.at().flgGlob()&(Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address)) &&
+		    (attr.at().flgSelf()&Attr::CfgConst || (attr.at().flgSelf()&Attr::CfgLnkIn && attr.at().cfgVal().substr(0,4) == "val:"))) );
+	    c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
+	    SYS->db().at().dataSet(fullDB+"_io",nodePath()+tbl+"_io",c_el);
+	}
+	//> User attributes store
+	else if( !ldGen )
+	{
+	    if( vDB == 1 ) c_elu.cfg("ID").setS( idc.empty() ? als[i_a] : idc+"/"+als[i_a] );
+	    if( vDB == 2 ) c_elu.cfg("ID").setS( als[i_a] );
+	    c_elu.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
+		    !(attr.at().flgGlob()&(Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address))) );
+	    c_elu.cfg("IO_VAL").setS(attr.at().getS()+"|"+
+		    attr.at().fld().values()+"|"+
+		    attr.at().fld().selNames());
+	    c_elu.cfg("NAME").setS(attr.at().name());
+	    c_elu.cfg("IO_TYPE").setI(attr.at().fld().type()+(attr.at().fld().flg()<<4));
+	    c_elu.cfg("SELF_FLG").setI(attr.at().flgSelf());
+	    c_elu.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
+	    c_elu.cfg("CFG_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
+		    !(attr.at().flgGlob()&(Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address)) &&
+		    (attr.at().flgSelf()&Attr::CfgConst || (attr.at().flgSelf()&Attr::CfgLnkIn && attr.at().cfgVal().substr(0,4) == "val:"))) );
+	    c_elu.cfg("CFG_VAL").setS(attr.at().cfgVal());
+	    SYS->db().at().dataSet(fullDB+"_uio",nodePath()+tbl+"_uio",c_elu);
+	}
+    }
+
+    if( !ldGen )
+    {
+	//> Clear no present IO for main io table
+	c_el.cfgViewAll(false);
+	for( int fld_cnt=0; vDB == 2 && SYS->db().at().dataSeek(fullDB+"_io",nodePath()+tbl+"_io",fld_cnt++,c_el); )
+	{
+	    string sid = c_el.cfg("ID").getS();
+	    if( w.attrPresent(sid) || (idc.empty() && !TSYS::pathLev(sid,1).empty() && !forceDBClear()) ) continue;
+
+	    SYS->db().at().dataDel(fullDB+"_io",nodePath()+tbl+"_io",c_el,true);
+	    fld_cnt--;
+	}
+
+	//> Clear no present IO for user io table
+	c_elu.cfgViewAll(false);
+	for( int fld_cnt=0; SYS->db().at().dataSeek(fullDB+"_uio",nodePath()+tbl+"_uio",fld_cnt++,c_elu); )
+	{
+	    string sid = c_elu.cfg("ID").getS();
+	    if( vDB == 1 && idc.empty() && (!TSYS::pathLev(sid,1).empty() || w.attrPresent(sid)) ) continue;
+	    if( vDB == 1 && !idc.empty() && (TSYS::pathLev(sid,0) != idc || w.attrPresent(TSYS::pathLev(sid,1))) ) continue;
+	    if( vDB == 2 && (w.attrPresent(sid) || (idc.empty() && !TSYS::pathLev(sid,1).empty() && !forceDBClear())) ) continue;
+
+	    SYS->db().at().dataDel(fullDB+"_uio",nodePath()+tbl+"_uio",c_elu,true);
+	    fld_cnt--;
+	}
+    }
+
+    return m_attrs;
+}
+
 void Engine::cntrCmdProc( XMLNode *opt )
 {
     string a_path = opt->attr("path");
     //> Service commands process
     if( a_path == "/serv/sess" )	//Session operation
     {
-	if( ctrChkNode(opt,"list",RWRWRW,"root","root",SEQ_RD) ) //List session for some project
+	if( ctrChkNode(opt,"list",RWRWRW,"root","root",SEC_RD) ) //List session for some project
 	{
 	    string prj = opt->attr("prj");
 	    vector<string> ls;
@@ -497,18 +712,18 @@ void Engine::cntrCmdProc( XMLNode *opt )
 		if( prj.empty() || sesAt(ls[i_l]).at().projNm() == prj )
 		{
 		    AutoHD<Project> prj = sesAt(ls[i_l]).at().parent();
-		    if( SYS->security().at().access(opt->attr("user"),SEQ_RD,prj.at().owner(),prj.at().grp(),prj.at().permit()) )
+		    if( SYS->security().at().access(opt->attr("user"),SEC_RD,prj.at().owner(),prj.at().grp(),prj.at().permit()) )
 			opt->childAdd("el")->setText(ls[i_l]);
 		}
 	}
-	else if( ctrChkNode(opt,"connect",RWRWRW,"root","root",SEQ_WR) )
+	else if( ctrChkNode(opt,"connect",RWRWRW,"root","root",SEC_WR) )
 	{
 	    string sess = opt->attr("sess");
 	    string prj  = opt->attr("prj");
 
 	    //>> User permission check
 	    AutoHD<Project> wprj = (!sess.empty()) ? sesAt(sess).at().parent() : prjAt(prj);
-	    if( !SYS->security().at().access(opt->attr("user"),SEQ_RD,wprj.at().owner(),wprj.at().grp(),wprj.at().permit()) )
+	    if( !SYS->security().at().access(opt->attr("user"),SEC_RD,wprj.at().owner(),wprj.at().grp(),wprj.at().permit()) )
 		throw TError(nodePath().c_str(),_("Connection to session is not permited for '%s'."),opt->attr("user").c_str());
 	    //>> Connect to present session
 	    if( !sess.empty() )	sesAt(sess).at().connect();
@@ -526,7 +741,7 @@ void Engine::cntrCmdProc( XMLNode *opt )
 		opt->setAttr("sess",sess);
 	    }else throw TError(nodePath().c_str(),_("Connect/create session arguments error."));
 	}
-	else if( ctrChkNode(opt,"disconnect",RWRWRW,"root","root",SEQ_WR) )
+	else if( ctrChkNode(opt,"disconnect",RWRWRW,"root","root",SEC_WR) )
 	{
 	    string sess = opt->attr("sess");
 	    sesAt(sess).at().disconnect();
@@ -581,7 +796,7 @@ void Engine::cntrCmdProc( XMLNode *opt )
 	return;
     }
 
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TUI::cntrCmdProc(opt);
@@ -610,10 +825,10 @@ void Engine::cntrCmdProc( XMLNode *opt )
 	return;
     }
 
-    //- Process command for page -
+    //> Process command for page
     if( a_path == "/br/prj_" || a_path == "/prm/cfg/prj" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","UI",SEC_RD) )
 	{
 	    vector<string> lst;
 	    prjList(lst);
@@ -623,36 +838,36 @@ void Engine::cntrCmdProc( XMLNode *opt )
 		if( chkUserPerm )
 		{
 		    AutoHD<Project> prj = prjAt(lst[i_a]);
-		    if( !SYS->security().at().access(opt->attr("user"),SEQ_RD,prj.at().owner(),prj.at().grp(),prj.at().permit()) )
+		    if( !SYS->security().at().access(opt->attr("user"),SEC_RD,prj.at().owner(),prj.at().grp(),prj.at().permit()) )
 			continue;
 		}
 		opt->childAdd("el")->setAttr("id",lst[i_a])->setText(prjAt(lst[i_a]).at().name());
 	    }
 	}
-	if( ctrChkNode(opt,"add",0664,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","UI",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    prjAdd(vid,opt->text()); prjAt(vid).at().setOwner(opt->attr("user"));
 	}
-	if( ctrChkNode(opt,"del",0664,"root","UI",SEQ_WR) )   prjDel(opt->attr("id"),true);
+	if( ctrChkNode(opt,"del",0664,"root","UI",SEC_WR) )   prjDel(opt->attr("id"),true);
     }
     else if( a_path == "/br/wlb_" || a_path == "/prm/cfg/wlb" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","UI",SEC_RD) )
 	{
 	    vector<string> lst;
 	    wlbList(lst);
 	    for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",lst[i_a])->setText(wlbAt(lst[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","UI",SEQ_WR) )	wlbAdd(TSYS::strEncode(opt->attr("id"),TSYS::ID),opt->text());
-	if( ctrChkNode(opt,"del",0664,"root","UI",SEQ_WR) )	wlbDel(opt->attr("id"),true);
+	if( ctrChkNode(opt,"add",0664,"root","UI",SEC_WR) )	wlbAdd(TSYS::strEncode(opt->attr("id"),TSYS::oscdID),opt->text());
+	if( ctrChkNode(opt,"del",0664,"root","UI",SEC_WR) )	wlbDel(opt->attr("id"),true);
     }
-    else if( a_path == "/prm/cfg/cp/cp" && ctrChkNode(opt,"set",0660,"root","UI",SEQ_WR) )
+    else if( a_path == "/prm/cfg/cp/cp" && ctrChkNode(opt,"set",0660,"root","UI",SEC_WR) )
 	nodeCopy( nodePath(0,true)+opt->attr("src"), nodePath(0,true)+opt->attr("dst"), opt->attr("user") );
     else if( a_path == "/br/ses_" || a_path == "/ses/ses" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","UI",SEC_RD) )
 	{
 	    vector<string> lst;
 	    sesList(lst);
@@ -662,18 +877,18 @@ void Engine::cntrCmdProc( XMLNode *opt )
 		if( chkUserPerm )
 		{
 		    AutoHD<Project> prj = sesAt(lst[i_a]).at().parent();
-		    if( !SYS->security().at().access(opt->attr("user"),SEQ_RD,prj.at().owner(),prj.at().grp(),prj.at().permit()) )
+		    if( !SYS->security().at().access(opt->attr("user"),SEC_RD,prj.at().owner(),prj.at().grp(),prj.at().permit()) )
 			continue;
 		}
 		opt->childAdd("el")->setAttr("user",sesAt(lst[i_a]).at().user())->setText(lst[i_a]);
 	    }
 	}
-	if( ctrChkNode(opt,"add",0664,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","UI",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->text(),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->text(),TSYS::oscdID);
 	    sesAdd(vid); sesAt(vid).at().setUser(opt->attr("user")); sesAt(vid).at().setBackgrnd(true);
 	}
-	if( ctrChkNode(opt,"del",0664,"root","UI",SEQ_WR) )   sesDel(opt->text(),true);
+	if( ctrChkNode(opt,"del",0664,"root","UI",SEC_WR) )   sesDel(opt->text(),true);
     }
     else if( a_path == "/br/vca" && ctrChkNode(opt) )
     {
@@ -684,17 +899,15 @@ void Engine::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/tts/code" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","UI",SEQ_RD) )	opt->setText(synthCode());
-	if( ctrChkNode(opt,"set",0664,"root","UI",SEQ_WR) )	setSynthCode(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","UI",SEC_RD) )	opt->setText(synthCode());
+	if( ctrChkNode(opt,"set",0664,"root","UI",SEC_WR) )	setSynthCode(opt->text());
     }
     else if( a_path == "/tts/comm" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","UI",SEQ_RD) )	opt->setText(synthCom());
-	if( ctrChkNode(opt,"set",0664,"root","UI",SEQ_WR) )	setSynthCom(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","UI",SEC_RD) )	opt->setText(synthCom());
+	if( ctrChkNode(opt,"set",0664,"root","UI",SEC_WR) )	setSynthCom(opt->text());
     }
     else if( a_path == "/tts/comm_ls" && ctrChkNode(opt) )
-    {
-	opt->childAdd("el")->setText("echo \"%t\" | ru_tts | sox -t raw -s -b -r 10000 -c 1 -v 0.8 - -t ogg -");
-    }
+	opt->childAdd("el")->setText("echo \"%t\" | ru_tts | sox -t raw -s -b 8 -r 10000 -c 1 -v 0.8 - -t ogg -");
     else TUI::cntrCmdProc(opt);
 }
