@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.VCAEngine file: widget.cpp
 /***************************************************************************
- *   Copyright (C) 2006-2008 by Roman Savochenko                           *
+ *   Copyright (C) 2006-2009 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -27,11 +27,12 @@
 
 using namespace VCA;
 
+
 //************************************************
 //* Widget                                       *
 //************************************************
 Widget::Widget( const string &id, const string &isrcwdg ) :
-    mId(id), mEnable(false), m_lnk(false), mParentNm(isrcwdg)
+    mId(id), mEnable(false), m_lnk(false), mStlLock(false), mParentNm(isrcwdg)
 {
     inclWdg = grpAdd("wdg_");
     attrId  = grpAdd("a_",true);
@@ -74,6 +75,7 @@ TCntrNode &Widget::operator=( TCntrNode &node )
     AutoHD<Attr> attr, pattr;
     for( int i_a = 0; i_a < els.size(); i_a++ )
     {
+	pattr = src_n->attrAt(els[i_a]);
 	if( !attrPresent(els[i_a]) )
 	{
 	    bool isInher = pattr.at().flgSelf()&Attr::IsInher;
@@ -82,7 +84,6 @@ TCntrNode &Widget::operator=( TCntrNode &node )
 	}
 	attr  = attrAt(els[i_a]);
 	if( attr.at().flgGlob( )&Attr::DirRead ) continue;
-	pattr = src_n->attrAt(els[i_a]);
 	attr.at().setFlgSelf(pattr.at().flgSelf());
 	switch( attr.at().type() )
 	{
@@ -210,14 +211,18 @@ void Widget::setEnable( bool val )
 	{
 	    try
 	    {
-		if( TSYS::strNoSpace(parentNm()).empty() ) throw TError(nodePath().c_str(),"%s",_("Empty parent!"));
+		if( TSYS::strNoSpace(parentNm()).empty() || parentNm() == path() )
+		    throw TError(nodePath().c_str(),"%s",_("Empty parent or parent identical self!"));
 		if( parentNm() == ".." ) mParent = AutoHD<TCntrNode>(nodePrev());
 		else mParent = mod->nodeAt(parentNm());
+
 		//> Check for enable parent widget and enable if not
 		if( !parent().at().enable() )	parent().at().setEnable(true);
+
 		//> Inherit
 		inheritAttr( );
 		inheritIncl( );
+
 		//> Register of heritater
 		parent().at().heritReg(this);
 	    }catch(TError err)
@@ -228,10 +233,11 @@ void Widget::setEnable( bool val )
 	    }
 	}
 	mEnable = true;
-	//- Load self values from DB -
+
+	//> Load self values from DB
 	loadIO( );
     }
-    if(!val)
+    if( !val )
     {
 	//> Free no base attributes and restore base
 	vector<string>  ls;
@@ -239,8 +245,8 @@ void Widget::setEnable( bool val )
 	for( int i_l = 0; i_l < ls.size(); i_l++ )
 	{
 	    if( !(attrAt(ls[i_l]).at().flgGlob()&Attr::Generic) ) attrDel( ls[i_l], true );
-	    else if( attrAt(ls[i_l]).at().flgSelf( )&Attr::IsInher )
-		attrAt(ls[i_l]).at().setFld( new TFld(attrAt(ls[i_l]).at().fld()), false );
+	    //else if( attrAt(ls[i_l]).at().flgSelf( )&Attr::IsInher )
+	    //	attrAt(ls[i_l]).at().setFld( new TFld(attrAt(ls[i_l]).at().fld()), false );
 	}
 
 	//> Disable heritors widgets
@@ -277,6 +283,12 @@ void Widget::setEnable( bool val )
     mEnable = val;
 }
 
+void Widget::setParentNm( const string &isw )
+{
+    if( enable() && mParentNm != isw ) setEnable(false);
+    mParentNm = isw;
+}
+
 AutoHD<Widget> Widget::parent( )
 {
     return mParent;
@@ -290,7 +302,7 @@ AutoHD<Widget> Widget::parentNoLink( )
 
 void Widget::heritReg( Widget *wdg )
 {
-    //- Search already herited widget -
+    //> Search already herited widget
     for( int i_h = 0; i_h < m_herit.size(); i_h++ )
 	if( &m_herit[i_h].at() == wdg )	return;
     m_herit.push_back(AutoHD<Widget>(wdg));
@@ -298,7 +310,7 @@ void Widget::heritReg( Widget *wdg )
 
 void Widget::heritUnreg( Widget *wdg )
 {
-    //- Search herited widget -
+    //> Search herited widget
     for( int i_h = 0; i_h < m_herit.size(); i_h++ )
 	if( &m_herit[i_h].at() == wdg )	
 	{
@@ -321,6 +333,11 @@ void Widget::inheritAttr( const string &iattr )
 	else return;
     }
 
+    setStlLock(true);
+    if( !loadDef ) parent().at().setStlLock(true);
+
+
+
     //> Configuration inherit
     AutoHD<Attr> attr, pattr;
     for( int i_l = 0; i_l < ls.size(); i_l++ )
@@ -342,9 +359,9 @@ void Widget::inheritAttr( const string &iattr )
 	    continue;
 	}
 	pattr = parent().at().attrAt(ls[i_l]);
-	if( !(attr.at().flgSelf()&Attr::IsInher) ) attr.at().setFld(&pattr.at().fld(),true);
+	if( !(attr.at().flgSelf()&Attr::IsInher) /*&& !(attr.at().flgGlob()&Attr::Mutable)*/ ) attr.at().setFld(&pattr.at().fld(),true);
 	if( attr.at().modif() && !(attr.at().flgSelf()&Attr::SessAttrInh) )	continue;
-	attr.at().setFlgSelf((Attr::SelfAttrFlgs)pattr.at().flgSelf());
+	attr.at().setFlgSelf( (Attr::SelfAttrFlgs)pattr.at().flgSelf() );
 	if( !(attr.at().flgGlob( )&Attr::DirRead) )
 	{
 	    bool active = attr.at().flgGlob()&Attr::Active;
@@ -363,6 +380,11 @@ void Widget::inheritAttr( const string &iattr )
 	attr.at().setCfgVal(pattr.at().cfgVal());
 	attr.at().setModif(0);
     }
+
+    setStlLock(false);
+    if( !loadDef ) parent().at().setStlLock(false);
+
+
 }
 
 void Widget::inheritIncl( const string &iwdg )
@@ -379,7 +401,8 @@ void Widget::inheritIncl( const string &iwdg )
     else parw.at().wdgList(ls);
     for( int i_w = 0; i_w < ls.size(); i_w++ )
 	if( !wdgPresent(ls[i_w]) )
-	    wdgAdd(ls[i_w],"",parw.at().wdgAt(ls[i_w]).at().path());
+	    try{ wdgAdd(ls[i_w],"",parw.at().wdgAt(ls[i_w]).at().path()); }
+	    catch(TError err){ mess_err(err.cat.c_str(),err.mess.c_str()); }
 }
 
 void Widget::wClear( )
@@ -418,10 +441,12 @@ void Widget::wClear( )
 	    parw.at().wdgList(ls);
 	    for( int i_w = 0; i_w < ls.size(); i_w++ )
 		if( !wdgPresent(ls[i_w]) )
-		    wdgAdd(ls[i_w],"",parw.at().wdgAt(ls[i_w]).at().path(),true);
+		    try{ wdgAdd(ls[i_w],"",parw.at().wdgAt(ls[i_w]).at().path(),true); }
+		    catch(TError err){ mess_err(err.cat.c_str(),err.mess.c_str()); }
 		else wdgAt(ls[i_w]).at().wClear();
 	}
     }
+    modif();
 }
 
 void Widget::attrAdd( TFld *attr, int pos, bool inher )
@@ -512,13 +537,20 @@ TVariant Widget::vlGet( Attr &a )
     return TVariant();
 }
 
+TVariant Widget::stlReq( Attr &a, const TVariant &vl, bool wr )
+{
+    if( !stlLock() && dynamic_cast<Widget*>(nodePrev()) )
+	return ((Widget*)nodePrev())->stlReq( a, vl, wr );
+    return vl;
+}
+
 bool Widget::cntrCmdServ( XMLNode *opt )
 {
     string a_path = opt->attr("path");
     //> Service commands process -
     if( a_path == "/serv/attr" )	//Attribute's access
     {
-	if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEQ_RD) )		//Get values
+	if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEC_RD) )		//Get values
 	{
 	    vector<string> ls;
 	    attrList(ls);
@@ -532,11 +564,11 @@ bool Widget::cntrCmdServ( XMLNode *opt )
 				     setText(attr.at().getS());
 	    }
 	}
-	else if( ctrChkNode(opt,"set",RWRWRW,"root","root",SEQ_WR) )	//Set values
+	else if( ctrChkNode(opt,"set",RWRWRW,"root","root",SEC_WR) )	//Set values
 	    for( int i_ch = 0; i_ch < opt->childSize(); i_ch++ )
 	        attrAt(opt->childGet(i_ch)->attr("id")).at().setS(opt->childGet(i_ch)->text());
     }
-    else if( a_path == "/serv/attrBr" && ctrChkNode(opt,"get",R_R_R_,"root","root",SEQ_RD) )	//Get attributes all updated elements' of the branch
+    else if( a_path == "/serv/attrBr" && ctrChkNode(opt,"get",R_R_R_,"root","root",SEC_RD) )	//Get attributes all updated elements' of the branch
     {
 	//>> Self attributes put
 	vector<string> ls;
@@ -578,7 +610,7 @@ bool Widget::cntrCmdServ( XMLNode *opt )
 
 bool Widget::cntrCmdGeneric( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TCntrNode::cntrCmdProc(opt);
@@ -620,7 +652,7 @@ bool Widget::cntrCmdGeneric( XMLNode *opt )
 	return true;
     }
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/wdg/res" && ctrChkNode(opt) )	//Service command for resources request
     {
@@ -630,36 +662,36 @@ bool Widget::cntrCmdGeneric( XMLNode *opt )
     }
     else if( a_path == "/wdg/st/en" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( TSYS::int2str(enable()) );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setEnable( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( TSYS::int2str(enable()) );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setEnable( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/wdg/st/parent" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( parentNm() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setParentNm( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( parentNm() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setParentNm( opt->text() );
     }
     else if( a_path == "/wdg/st/goparent" &&
-	    ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) && !parent().freeStat() )
+	    ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) && !parent().freeStat() )
 	opt->setText(parent().at().nodePath(0,true));
     else if( a_path == "/wdg/cfg/owner" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( owner() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setOwner( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( owner() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setOwner( opt->text() );
     }
     else if( a_path == "/wdg/cfg/grp" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( grp() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setGrp( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( grp() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setGrp( opt->text() );
     }
     else if( a_path == "/wdg/cfg/u_a" || a_path == "/wdg/cfg/g_a" || a_path == "/wdg/cfg/o_a" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	{
 	    if( a_path == "/wdg/cfg/u_a" )	opt->setText(TSYS::int2str((permit()>>6)&0x7));
 	    if( a_path == "/wdg/cfg/g_a" )	opt->setText(TSYS::int2str((permit()>>3)&0x7));
 	    if( a_path == "/wdg/cfg/o_a" )	opt->setText(TSYS::int2str(permit()&0x7));
 	}
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )
 	{
 	    if( a_path == "/wdg/cfg/u_a" )	setPermit( (permit()&(~(0x07<<6)))|(atoi(opt->text().c_str())<<6) );
 	    if( a_path == "/wdg/cfg/g_a" )	setPermit( (permit()&(~(0x07<<3)))|(atoi(opt->text().c_str())<<3) );
@@ -668,8 +700,8 @@ bool Widget::cntrCmdGeneric( XMLNode *opt )
     }
     else if( a_path == "/wdg/cfg/ico" || a_path == "/ico" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( ico() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setIco( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( ico() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setIco( opt->text() );
     }
     else if( a_path == "/wdg/cfg/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/wdg/cfg/type" && ctrChkNode(opt) )	opt->setText(type());
@@ -678,15 +710,24 @@ bool Widget::cntrCmdGeneric( XMLNode *opt )
 	opt->setText((isLink()&&atoi(opt->attr("resLink").c_str())) ? parentNoLink().at().path() : path());
     else if( a_path == "/wdg/cfg/name" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( name() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setName( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( name() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setName( opt->text() );
     }
     else if( a_path == "/wdg/cfg/descr" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText( descr() );
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setDescr( opt->text() );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText( descr() );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setDescr( opt->text() );
+    }
+    else if( a_path == "/wdg/cfg/clear" && ctrChkNode(opt,"set",RWRWR_,"root","UI") )
+    {
+	if( opt->attr("attr").empty() ) wClear();
+	else if( attrAt(opt->attr("attr")).at().modif() )
+	{
+	    attrAt(opt->attr("attr")).at().setModif(0);
+	    inheritAttr(opt->attr("attr"));
+	    modif();
+	}
     }
-    else if( a_path == "/wdg/cfg/clear" && ctrChkNode(opt,"set",RWRWR_,"root","UI") )	wClear();
     else if( a_path == "/wdg/u_lst" && ctrChkNode(opt) )
     {
 	vector<string> ls;
@@ -714,41 +755,45 @@ bool Widget::cntrCmdGeneric( XMLNode *opt )
 	    opt->childAdd("el")->setText(c_path);
 	}
 	vector<string>  ls;
-	switch(c_lv)
+	try
 	{
-	    case 0: mod->nodeList(ls);   break;
-	    case 1:
-		if( TSYS::pathLev(lnk,0) != ".." )
-		    mod->nodeAt(TSYS::pathLev(lnk,0)).at().nodeList(ls);
-		break;
-	    case 2:
-		if( TSYS::pathLev(lnk,0) != ".." )
-		    mod->nodeAt(TSYS::pathLev(lnk,0)).at().nodeAt(TSYS::pathLev(lnk,1)).at().nodeList(ls,"wdg_");
-		break;
-	}
-	for(int i_l = 0; i_l < ls.size(); i_l++)
-	    opt->childAdd("el")->setText(c_path+"/"+ls[i_l]);
+	    switch(c_lv)
+	    {
+		case 0: mod->nodeList(ls);   break;
+		case 1:
+		    if( TSYS::pathLev(lnk,0) != ".." )
+			mod->nodeAt(TSYS::pathLev(lnk,0)).at().nodeList(ls);
+		    break;
+		case 2:
+		    if( TSYS::pathLev(lnk,0) != ".." )
+			mod->nodeAt(TSYS::pathLev(lnk,0)).at().nodeAt(TSYS::pathLev(lnk,1)).at().nodeList(ls,"wdg_");
+		    break;
+	    }
+	    for(int i_l = 0; i_l < ls.size(); i_l++)
+		opt->childAdd("el")->setText(c_path+"/"+ls[i_l]);
+	}catch(TError err) { }
     }
     else if( a_path == "/br/wdg_" || a_path == "/inclwdg/wdg" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	{
 	    bool chkUserPerm = atoi(opt->attr("chkUserPerm").c_str());
-	    if( !chkUserPerm || SYS->security().at().access(opt->attr("user"),SEQ_RD,owner(),grp(),permit()) )
+	    if( !chkUserPerm || SYS->security().at().access(opt->attr("user"),SEC_RD,owner(),grp(),permit()) )
 	    {
 		vector<string>  lst;
 		wdgList(lst);
 		for( unsigned i_f=0; i_f < lst.size(); i_f++ )
 		{
 		    AutoHD<Widget> iwdg = wdgAt(lst[i_f]);
-		    if( chkUserPerm && !SYS->security().at().access(opt->attr("user"),SEQ_RD,iwdg.at().owner(),iwdg.at().grp(),iwdg.at().permit()) )
+		    if( chkUserPerm && !SYS->security().at().access(opt->attr("user"),SEC_RD,iwdg.at().owner(),iwdg.at().grp(),iwdg.at().permit()) )
 			continue;
 		    opt->childAdd("el")->setAttr("id",lst[i_f])->setText(iwdg.at().name());
 		}
 	    }
 	}
-	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEQ_WR) )	wdgAdd(TSYS::strEncode(opt->attr("id"),TSYS::ID).c_str(),opt->text(),"");
-	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEQ_WR) )	wdgDel(opt->attr("id").c_str(),true);
+	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEC_WR) )
+	    wdgAdd(TSYS::strEncode(opt->attr("id"),TSYS::oscdID).c_str(),opt->text(),"");
+	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEC_WR) )	wdgDel(opt->attr("id").c_str(),true);
     }
     else return false;
 
@@ -780,15 +825,15 @@ bool Widget::cntrCmdAttributes( XMLNode *opt )
     if( a_path.substr(0,6) == "/attr/" )
     {
 	AutoHD<Attr> attr = attrAt(TSYS::pathLev(a_path,1));
-	if( ctrChkNode(opt,"get",(attr.at().fld().flg()&TFld::NoWrite)?R_R_R_:RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",(attr.at().fld().flg()&TFld::NoWrite)?R_R_R_:RWRWR_,"root","UI",SEC_RD) )
 	{
 	    if( attr.at().fld().flg()&TFld::Selected )	opt->setText(attr.at().getSEL());
 	    else opt->setText(attr.at().getS());
 	}
-	if( ctrChkNode(opt,"set",(attr.at().fld().flg()&TFld::NoWrite)?R_R_R_:RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"set",(attr.at().fld().flg()&TFld::NoWrite)?R_R_R_:RWRWR_,"root","UI",SEC_WR) )
 	{
-	    if( attr.at().fld().flg()&TFld::Selected )	attr.at().setSEL(opt->text());
-	    else attr.at().setS(opt->text());
+	    if( attr.at().fld().flg()&TFld::Selected )	{ attr.at().setSEL(opt->text()); opt->setText(attr.at().getSEL()); }
+	    else { attr.at().setS(opt->text()); opt->setText(attr.at().getS()); }
 	}
     }
     else return false;
@@ -798,7 +843,7 @@ bool Widget::cntrCmdAttributes( XMLNode *opt )
 
 bool Widget::cntrCmdLinks( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	if(ctrMkNode("area",opt,-1,"/links",_("Links")))
@@ -829,17 +874,17 @@ bool Widget::cntrCmdLinks( XMLNode *opt )
 
 			if( !(wdg.at().attrAt(alist[i_a]).at().flgSelf()&(Attr::CfgLnkIn|Attr::CfgLnkOut|Attr::CfgConst)) ||
 			    (!shwAttr && wdg.at().attrAt(alist[i_a]).at().flgSelf()&Attr::CfgConst) ) continue;
-			//-- Get attributes --
+			//>> Get attributes
 			bool shwTmpl = wdg.at().attrAt(alist[i_a]).at().cfgTempl().size();
 			if( shwTmpl )	grpprm = TSYS::strSepParse(wdg.at().attrAt(alist[i_a]).at().cfgTempl(),0,'|');
 
-			//-- Check select param --
+			//>> Check select param
 			if( shwTmpl && !shwAttr )
 			{
 			    nprm = grpprm;
 			    if( i_w >= 0 ) nprm.insert(0,wdg.at().id()+".");
 
-			    //-- Check already to present parameters --
+			    //>> Check already to present parameters
 			    bool f_ok = false;
 			    for( int i_l = 0; i_l < list.size(); i_l++ )
 				if( list[i_l] == nprm ) { f_ok = true; break; }
@@ -867,13 +912,13 @@ bool Widget::cntrCmdLinks( XMLNode *opt )
 	return true;
     }
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/links/showAttr" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	    opt->setText(TBDS::genDBGet(mod->nodePath()+"showAttr","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )
 	    TBDS::genDBSet(mod->nodePath()+"showAttr",opt->text(),opt->attr("user"));
     }
     else if( a_path.substr(0,14) == "/links/lnk/pr_" )
@@ -894,7 +939,7 @@ bool Widget::cntrCmdLinks( XMLNode *opt )
 		{ nattr = a_ls[i_a]; break; }
 	}
 
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	{
 	    string cfg_val = srcwdg.at().attrAt(nattr).at().cfgVal();
 	    string obj_tp  = TSYS::strSepParse(cfg_val,0,':')+":";
@@ -924,7 +969,7 @@ bool Widget::cntrCmdLinks( XMLNode *opt )
 
 	    opt->setText(rez);
 	}
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )
 	{
 	    bool noonly_no_set = true;
 	    string no_set;
@@ -993,7 +1038,7 @@ bool Widget::cntrCmdLinks( XMLNode *opt )
 
 	string m_prm = srcwdg.at().attrAt(nattr).at().cfgVal();
 
-	//-- Link interface process --
+	//>> Link interface process
 	int c_lv = 0;
 	string obj_tp = TSYS::strSepParse(m_prm,0,':')+":";
 	if( obj_tp.empty() || !(obj_tp == "val:" || obj_tp == "prm:" || obj_tp == "wdg:") )
@@ -1002,7 +1047,7 @@ bool Widget::cntrCmdLinks( XMLNode *opt )
 	    opt->childAdd("el")->setText("prm:");
 	    opt->childAdd("el")->setText("wdg:");
 	}
-	//-- Link elements process --
+	//>> Link elements process
 	else
 	{
 	    int c_off = obj_tp.size();
@@ -1066,9 +1111,9 @@ bool Widget::cntrCmdLinks( XMLNode *opt )
 	if( nattr.size() )	srcwdg = wdgAt(nwdg);
 	else nattr = nwdg;
 	
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	    opt->setText(srcwdg.at().attrAt(nattr).at().cfgVal());
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )
 	{
 	    srcwdg.at().attrAt(nattr).at().setCfgVal(opt->text());
 	    if( srcwdg.at().attrAt(nattr).at().flgSelf()&Attr::CfgConst )
@@ -1086,7 +1131,7 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 
     if( isLink() ) return false;
 
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	if(ctrMkNode("area",opt,-1,"/proc",_("Process")))
@@ -1114,17 +1159,17 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 	return true;
     }
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/proc/wdg" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	{
 	    wattr = TBDS::genDBGet(mod->nodePath()+"wdgAttr",".",opt->attr("user"));
 	    if( wattr != "." && !wdgPresent(wattr) )	wattr = ".";
 	    opt->setText(wattr);
 	}
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )
 	    TBDS::genDBSet(mod->nodePath()+"wdgAttr",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/proc/w_lst" && ctrChkNode(opt) )
@@ -1142,7 +1187,7 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 	    wattr = TBDS::genDBGet(mod->nodePath()+"wdgAttr",".",opt->attr("user"));
 	if( wattr != "." && !wdgPresent(wattr) )	wattr = ".";
 	
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )
 	{
 	    XMLNode *n_id	= ctrMkNode("list",opt,-1,"/proc/attr/id","");
 	    XMLNode *n_name	= ctrMkNode("list",opt,-1,"/proc/attr/name","");
@@ -1164,11 +1209,11 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 		if( n_wa )	n_wa->childAdd("el")->setText( wdg.at().attrAt(lst[i_el]).at().fld().values()+"|"+
 							    wdg.at().attrAt(lst[i_el]).at().fld().selNames());
 		if( n_proc )	n_proc->childAdd("el")->setText(TSYS::int2str(wdg.at().attrAt(lst[i_el]).at().flgSelf()&Attr::ProcAttr));
-		if( n_cfg )	n_cfg->childAdd("el")->setText(TSYS::int2str(wdg.at().attrAt(lst[i_el]).at().flgSelf()&(Attr::CfgLnkIn|Attr::CfgLnkOut|Attr::CfgConst)));
+		if( n_cfg )	n_cfg->childAdd("el")->setText(TSYS::int2str(wdg.at().attrAt(lst[i_el]).at().flgSelf()&(Attr::CfgLnkIn|Attr::CfgLnkOut|Attr::CfgConst|Attr::FromStyle)));
 		if( n_cfgtmpl )	n_cfgtmpl->childAdd("el")->setText(wdg.at().attrAt(lst[i_el]).at().cfgTempl());
 	    }
 	}
-	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEC_WR) )
 	{
 	    AutoHD<Widget> wdg = (wattr==".")?AutoHD<Widget>(this):wdgAt(wattr);
 	    wdg.at().attrAdd( new TFld("newAttr",_("New attribute"),TFld::String,Attr::IsUser) );
@@ -1176,20 +1221,20 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 	    wdg.at().attrAt("newAttr").at().setModif(1);
 	    wdg.at().modif();
 	}
-	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEC_WR) )
 	{
 	    AutoHD<Widget> wdg = (wattr==".")?AutoHD<Widget>(this):wdgAt(wattr);
 	    if( !wdg.at().attrPresent(opt->attr("key_id")) )
-                throw TError(nodePath().c_str(),_("Deleting the enclosed widget's elements error."));
+		throw TError(nodePath().c_str(),_("Deleting the enclosed widget's elements error."));
 	    if( !(wdg.at().attrAt(opt->attr("key_id")).at().flgSelf()&Attr::IsInher) &&
 		    wdg.at().attrAt(opt->attr("key_id")).at().fld().flg()&Attr::IsUser )
 		wdg.at().attrDel(opt->attr("key_id"));
 	    else throw TError(nodePath().c_str(),_("Deletint the not user element error."));
 	    wdg.at().modif();
 	}
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )
 	{
-	    //-- Request data --
+	    //>> Request data
 	    string idattr = opt->attr("key_id");
 	    string idcol  = opt->attr("col");
 	    AutoHD<Widget> wdg = (wattr==".")?AutoHD<Widget>(this):wdgAt(wattr);
@@ -1211,7 +1256,11 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 		string tmpl	= wdg.at().attrAt(idattr).at().cfgTempl();
 		string cfgval	= wdg.at().attrAt(idattr).at().cfgVal();
 
-		if( idcol == "id" )		tid = opt->text();
+		if( idcol == "id" )
+		{
+		    tid = opt->text();
+		    if( wdg.at().attrPresent(tid) ) throw TError(nodePath().c_str(),_("New attribute's ID '%s' already present."),tid.c_str());
+		}
 		else if( idcol == "name" )	tnm = opt->text();
 		else if( idcol == "type" )
 		{
@@ -1243,8 +1292,8 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 		{
 		    Attr::SelfAttrFlgs sflg =  wdg.at().attrAt(idattr).at().flgSelf();
 		    Attr::SelfAttrFlgs stflg = (Attr::SelfAttrFlgs)atoi(opt->text().c_str());
-		    if( (sflg^stflg)&(Attr::CfgLnkIn|Attr::CfgLnkOut|Attr::CfgConst) )
-			wdg.at().attrAt(idattr).at().setFlgSelf( (Attr::SelfAttrFlgs)(sflg^((sflg^stflg)&(Attr::CfgLnkIn|Attr::CfgLnkOut|Attr::CfgConst))) );
+		    if( (sflg^stflg)&(Attr::CfgLnkIn|Attr::CfgLnkOut|Attr::CfgConst|Attr::FromStyle) )
+			wdg.at().attrAt(idattr).at().setFlgSelf( (Attr::SelfAttrFlgs)(sflg^((sflg^stflg)&(Attr::CfgLnkIn|Attr::CfgLnkOut|Attr::CfgConst|Attr::FromStyle))) );
 		}
 		else if( idcol == "cfgtmpl" )	wdg.at().attrAt(idattr).at().setCfgTempl(opt->text());
 	    }
@@ -1253,18 +1302,18 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
     }
     else if( a_path == "/proc/calc/progLng" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText(calcLang());
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setCalcLang(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText(calcLang());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setCalcLang(opt->text());
     }
     else if( a_path == "/proc/calc/per" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText(TSYS::int2str(calcPer()));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setCalcPer(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText(TSYS::int2str(calcPer()));
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setCalcPer(atoi(opt->text().c_str()));
     }
     else if( a_path == "/proc/calc/prog" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEQ_RD) )	opt->setText(calcProg());
-	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEQ_WR) )	setCalcProg(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","UI",SEC_RD) )	opt->setText(calcProg());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","UI",SEC_WR) )	setCalcProg(opt->text());
     }
     else if( a_path == "/proc/calc/plangLs" && ctrChkNode(opt) )
     {
@@ -1281,10 +1330,15 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 	vector<string>  ls;
 	switch(c_lv)
 	{
-	    case 0:	SYS->daq().at().modList(ls);    break;
+	    case 0:
+		SYS->daq().at().modList(ls);
+		for( int i_l = 0; i_l < ls.size(); i_l++ )
+		    if( !SYS->daq().at().at(ls[i_l]).at().compileFuncLangs() )
+		    { ls.erase(ls.begin()+i_l); i_l--; }
+		break;
 	    case 1:
 		if(SYS->daq().at().modPresent(TSYS::strSepParse(tplng,0,'.')))
-		    SYS->daq().at().at(TSYS::strSepParse(tplng,0,'.')).at().compileFuncLangs(ls);
+		    SYS->daq().at().at(TSYS::strSepParse(tplng,0,'.')).at().compileFuncLangs(&ls);
 		break;
 	}
 	for(int i_l = 0; i_l < ls.size(); i_l++)
@@ -1302,7 +1356,7 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(TFld::String+(Attr::Color<<4)))->setText(_("Color"));
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(TFld::String+(Attr::Image<<4)))->setText(_("Image"));
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(TFld::String+(Attr::Font<<4)))->setText(_("Font"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(TFld::Integer+(Attr::DataTime<<4)))->setText(_("DataTime"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(TFld::Integer+(Attr::DateTime<<4)))->setText(_("DateTime"));
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(TFld::String+(Attr::Address<<4)))->setText(_("Address"));
     }
     else if( a_path == "/proc/lnk_ls" && ctrChkNode(opt) )
@@ -1312,6 +1366,7 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(Attr::CfgLnkIn))->setText(_("Input link"));
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(Attr::CfgLnkOut))->setText(_("Output link"));
 	opt->childAdd("el")->setAttr("id",TSYS::int2str(Attr::CfgLnkIn|Attr::CfgLnkOut))->setText(_("Full link"));
+	opt->childAdd("el")->setAttr("id",TSYS::int2str(Attr::FromStyle))->setText(_("From style"));
     }
     else return false;
 
@@ -1324,58 +1379,70 @@ bool Widget::cntrCmdProcess( XMLNode *opt )
 Attr::Attr( TFld *ifld, bool inher ) : mFld(NULL), m_modif(0), self_flg((SelfAttrFlgs)0)
 {
     setFld( ifld, inher );
-    switch( fld().type() )
-    {
-	case TFld::String:
-	    m_val.s_val = NULL;
-	    if( fld().flg()&Attr::DirRead ) break;
-	    m_val.s_val    = new ResString();
-	    m_val.s_val->setVal(fld().def());
-	    break;
-	case TFld::Integer:	m_val.i_val = atoi(mFld->def().c_str());	break;
-	case TFld::Real:	m_val.r_val = atof(mFld->def().c_str());	break;
-	case TFld::Boolean:	m_val.b_val = atoi(mFld->def().c_str());	break;
-    }
 }
 
 Attr::~Attr(  )
 {
-    if( fld().type() == TFld::String && m_val.s_val )	delete m_val.s_val;
-    setFld(NULL,false);
+    setFld( NULL, false );
 }
 
 void Attr::setFld( TFld *fld, bool inher )
 {
-    if( mFld && !(self_flg&Attr::IsInher) ) delete mFld;
+    if( mFld && (!fld || fld->type() != TFld::String) && mFld->type() == TFld::String && m_val.s_val ) delete m_val.s_val;
+
+    if( fld && (!mFld || fld->type() != mFld->type()) )
+	switch( fld->type() )
+	{
+	    case TFld::String:
+		m_val.s_val = NULL;
+		if( fld->flg()&Attr::DirRead ) break;
+		m_val.s_val    = new ResString();
+		m_val.s_val->setVal(fld->def());
+		break;
+	    case TFld::Integer:	m_val.i_val = atoi(fld->def().c_str());	break;
+	    case TFld::Real:	m_val.r_val = atof(fld->def().c_str());	break;
+	    case TFld::Boolean:	m_val.b_val = atoi(fld->def().c_str());	break;
+	}
+
+    if( mFld )
+    {
+	mFld->setLen(mFld->len()-1);
+	if( !mFld->len() ) delete mFld;
+	mFld = NULL;
+    }
     mFld = fld;
+    if( mFld && !inher )	mFld->setLen(1);
+    else if( mFld && inher )	mFld->setLen(mFld->len()+1);
     self_flg = inher ? self_flg|Attr::IsInher : self_flg & ~Attr::IsInher;
 }
 
-string Attr::id()	{ return fld().name(); }
+string Attr::id( )	{ return fld().name(); }
 
-string Attr::name()	{ return fld().descr(); }
+string Attr::name( )	{ return fld().descr(); }
 
-TFld::Type Attr::type()	{ return fld().type(); }
+TFld::Type Attr::type( ){ return fld().type(); }
 
-Widget *Attr::owner()	{ return (Widget *)nodePrev(); }
+Widget *Attr::owner( )	{ return (Widget *)nodePrev(); }
 
-int Attr::flgGlob()	{ return fld().flg(); }
+int Attr::flgGlob( )	{ return fld().flg(); }
 
-string Attr::getSEL( )
+string Attr::getSEL( bool sys )
 {
     if( !(fld().flg()&TFld::Selected) ) throw TError("Cfg",_("Element type is not selected!"));
     switch( fld().type() )
     {
-	case TFld::String:	return fld().selVl2Nm(getS());
-	case TFld::Integer:	return fld().selVl2Nm(getI());
-	case TFld::Real:	return fld().selVl2Nm(getR());
-	case TFld::Boolean:	return fld().selVl2Nm(getB());
+	case TFld::String:	return fld().selVl2Nm(getS(sys));
+	case TFld::Integer:	return fld().selVl2Nm(getI(sys));
+	case TFld::Real:	return fld().selVl2Nm(getR(sys));
+	case TFld::Boolean:	return fld().selVl2Nm(getB(sys));
     }
 }
 
-string Attr::getS( )
+string Attr::getS( bool sys )
 {
     if( flgGlob()&Attr::DirRead )	return owner()->vlGet(*this).getS();
+    if( flgSelf()&Attr::FromStyle && !sys )	return owner()->stlReq(*this,getS(true),false).getS();
+
     switch( fld().type() )
     {
 	case TFld::Integer:	return (m_val.i_val!=EVAL_INT) ? TSYS::int2str(m_val.i_val) : EVAL_STR;
@@ -1385,9 +1452,10 @@ string Attr::getS( )
     }
 }
 
-int Attr::getI( )
+int Attr::getI( bool sys )
 {
     if( flgGlob()&Attr::DirRead )	return owner()->vlGet(*this).getI();
+    if( flgSelf()&Attr::FromStyle && !sys )	return owner()->stlReq(*this,getI(true),false).getI();
     switch( fld().type() )
     {
 	case TFld::String:	return (m_val.s_val->getVal()!=EVAL_STR) ? atoi(m_val.s_val->getVal().c_str()) : EVAL_INT;
@@ -1397,9 +1465,10 @@ int Attr::getI( )
     }
 }
 
-double Attr::getR( )
+double Attr::getR( bool sys )
 {
     if( flgGlob()&Attr::DirRead )	return owner()->vlGet(*this).getR();
+    if( flgSelf()&Attr::FromStyle && !sys )	return owner()->stlReq(*this,getR(true),false).getR();
     switch( fld().type() )
     {
 	case TFld::String:	return (m_val.s_val->getVal()!=EVAL_STR) ? atof(m_val.s_val->getVal().c_str()) : EVAL_REAL;
@@ -1409,9 +1478,10 @@ double Attr::getR( )
     }
 }
 
-char Attr::getB( )
+char Attr::getB( bool sys )
 {
     if( flgGlob()&Attr::DirRead )	return owner()->vlGet(*this).getB();
+    if( flgSelf()&Attr::FromStyle && !sys )	return owner()->stlReq(*this,getB(true),false).getB();
     switch( fld().type() )
     {
 	case TFld::String:	return (m_val.s_val->getVal()!=EVAL_STR) ? (bool)atoi(m_val.s_val->getVal().c_str()) : EVAL_BOOL;
@@ -1445,7 +1515,8 @@ void Attr::setS( const string &val, bool strongPrev, bool sys )
 	case TFld::Boolean:	setB( (val!=EVAL_STR) ? (bool)atoi(val.c_str()) : EVAL_BOOL, strongPrev, sys );	break;
 	case TFld::String:
 	{
-	    if( !strongPrev && m_val.s_val->getVal() == val )	break;
+	    if( (!strongPrev && m_val.s_val->getVal() == val) ||
+		(flgSelf()&Attr::FromStyle && !sys && owner()->stlReq(*this,val,true).isNull()) )	break;
 	    string t_str = m_val.s_val->getVal();
 	    m_val.s_val->setVal(val);
 	    if( !sys && !owner()->attrChange(*this,TVariant(t_str)) )	m_val.s_val->setVal(t_str);
@@ -1471,7 +1542,8 @@ void Attr::setI( int val, bool strongPrev, bool sys )
 	{
 	    if( !(fld().flg()&TFld::Selected) && fld().selValI()[0] < fld().selValI()[1] )
 		val = vmin(fld().selValI()[1],vmax(fld().selValI()[0],val));
-	    if( !strongPrev && m_val.i_val == val )	break;
+	    if( (!strongPrev && m_val.i_val == val) ||
+		(flgSelf()&Attr::FromStyle && !sys && owner()->stlReq(*this,val,true).isNull()) )	break;
 	    int t_val = m_val.i_val;
 	    m_val.i_val = val;
 	    if( !sys && !owner()->attrChange(*this,TVariant(t_val)) )	m_val.i_val = t_val;
@@ -1497,10 +1569,10 @@ void Attr::setR( double val, bool strongPrev, bool sys )
 	{
 	    if( !(fld().flg()&TFld::Selected) && fld().selValR()[0] < fld().selValR()[1] )
 		val = vmin(fld().selValR()[1],vmax(fld().selValR()[0],val));
-	    if( !strongPrev && m_val.r_val == val )	break;
+	    if( (!strongPrev && m_val.r_val == val) ||
+		(flgSelf()&Attr::FromStyle && !sys && owner()->stlReq(*this,val,true).isNull()) )	break;
 	    double t_val = m_val.r_val;
 	    m_val.r_val = val;
-
 	    if( !sys && !owner()->attrChange(*this,TVariant(t_val)) )	m_val.r_val = t_val;
 	    else
 	    {
@@ -1522,7 +1594,8 @@ void Attr::setB( char val, bool strongPrev, bool sys )
 	case TFld::Real:	setR( (val!=EVAL_BOOL) ? (bool)val : EVAL_REAL, strongPrev, sys );			break;
 	case TFld::Boolean:
 	{
-	    if( !strongPrev && m_val.b_val == val )	break;
+	    if( (!strongPrev && m_val.b_val == val) ||
+		(flgSelf()&Attr::FromStyle && !sys && owner()->stlReq(*this,val,true).isNull()) )	break;
 	    bool t_val = m_val.b_val;
 	    m_val.b_val = val;
 	    if( !sys && !owner()->attrChange(*this,TVariant(t_val)) )
@@ -1532,8 +1605,8 @@ void Attr::setB( char val, bool strongPrev, bool sys )
 		unsigned imdf = owner()->modifVal(*this);
 		m_modif = imdf ? imdf : m_modif+1;
 	    }
+	    break;
 	}
-	break;
     }
 }
 
