@@ -34,6 +34,7 @@ namespace VCA
 //************************************************
 class Widget;
 
+#pragma pack(push,1)
 class Attr : public TCntrNode
 {
     public:
@@ -44,7 +45,7 @@ class Attr : public TCntrNode
 	    Active	= 0x00100,	//Active attribute for primitives process
 
 	    Image	= 0x00200,	//Store image link to DB or to file
-	    DataTime	= 0x00200,	//Store data and time into integer attribute
+	    DateTime	= 0x00200,	//Store data and time into integer attribute
 	    Color	= 0x00400,	//Store color
 	    Font	= 0x00800,	//Store font
 	    Address	= 0x01000,	//Store address to a parameter or an attribute
@@ -62,6 +63,7 @@ class Attr : public TCntrNode
 	    CfgLnkIn	= 0x02,		//Input link
 	    CfgLnkOut	= 0x04,		//Output link
 	    ProcAttr	= 0x08,		//Process attribute
+	    FromStyle	= 0x40,		//Get value from style
 
 	    SessAttrInh	= 0x10,		//Inherit attribute into running session
 	    IsInher	= 0x20		//Inherit attribute
@@ -87,11 +89,11 @@ class Attr : public TCntrNode
 	void setCfgVal( const string &vl );
 
 	//> Get value
-	string getSEL( );
-	string getS( );
-	double getR( );
-	int    getI( );
-	char   getB( );
+	string getSEL( bool sys = false );
+	string getS( bool sys = false );
+	double getR( bool sys = false );
+	int    getI( bool sys = false );
+	char   getB( bool sys = false );
 
 	//> Set value
 	void setSEL( const string &val, bool strongPrev = false, bool sys = false );
@@ -126,6 +128,7 @@ class Attr : public TCntrNode
 
 	string	cfg;			//Config template and value
 };
+#pragma pack(pop)
 
 //************************************************
 //* Widget                                       *
@@ -156,6 +159,7 @@ class Widget : public TCntrNode
 	virtual int    calcPer( )	{ return -1; }		//Calc widget period. 0 value talk for calc on session period.
 	virtual bool   isContainer( );				//Is container (Is define of the terminator)
 	virtual bool   isLink( )	{ return m_lnk; }	//Widget as link
+	bool stlLock( )			{ return mStlLock; }	//Style lock
 
 	virtual void setName( const string &inm );
 	virtual void setDescr( const string &idscr );
@@ -166,6 +170,7 @@ class Widget : public TCntrNode
 	virtual void setCalcLang( const string &ilng )	{ };
 	virtual void setCalcProg( const string &iprg )	{ };
 	virtual void setCalcPer( int vl )		{ };
+	void setStlLock( bool vl )	{ mStlLock = vl; }
 
 	//> Storing
 	virtual void loadIO( )		{ }			//Load widget's IO
@@ -184,7 +189,7 @@ class Widget : public TCntrNode
 	void heritReg( Widget *wdg );				//Register heritator
 	void heritUnreg( Widget *wdg );				//Unregister heritator
 	vector< AutoHD<Widget> > &herit( )		{ return m_herit; }
-	virtual void setParentNm( const string &isw )	{ mParentNm = isw; }
+	virtual void setParentNm( const string &isw );
 	virtual void inheritAttr( const string &attr = "" );	//Inherit parent attributes
 	void inheritIncl( const string &wdg = "" );		//Inherit parent include widgets
 
@@ -221,23 +226,26 @@ class Widget : public TCntrNode
 
 	virtual bool attrChange( Attr &cfg, TVariant prev );   //Process attribute change local and into terminator
 	virtual unsigned int modifVal( Attr &cfg )	{ return 0; }
-	virtual void calc( Widget *base );
+	virtual TVariant vlGet( Attr &a );
+	virtual TVariant stlReq( Attr &a, const TVariant &vl, bool wr );
 
-	TVariant vlGet( Attr &a );
+	virtual void calc( Widget *base );
 
 	//Attributes
 	//> Generic data
 	string	mId;			//Widget identifier
 
-	char	mEnable	:1;		//Enable status
-	char	m_lnk	:1;		//Widget as link
-	char	attrId	:3;		//The widget's container id
-	char	inclWdg	:3;
+	unsigned char	mEnable	:1;		//Enable status
+	unsigned char	m_lnk	:1;		//Widget as link
+	unsigned char	attrId	:3;		//The widget's container id
+	unsigned char	inclWdg	:3;
+	unsigned char	mStlLock:1;		//Style lock
 	string	mParentNm;		//Parent widget name
 	AutoHD<Widget>	mParent;	//Parent widget
 	vector< AutoHD<Widget> > m_herit;	//Heritators
 };
 
+
 }
 
 #endif //WIDGET_H
