@@ -53,9 +53,7 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
 
     setProperty("QTStarterToolDis",true);
 
-#if QT_VERSION >= 0x040301
     setDockOptions(dockOptions() | QMainWindow::VerticalTabs);
-#endif
     mod->regWin( this );
 
     setWindowTitle(_("Vision developing"));
@@ -123,7 +121,7 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     actDBSave->setToolTip(_("Save item data to DB"));
     actDBSave->setWhatsThis(_("The button for saving item data to DB"));
     actDBSave->setStatusTip(_("Press to save item data to DB."));
-    actDBSave->setShortcut(QKeySequence("Ctrl+S"));    
+    actDBSave->setShortcut(QKeySequence("Ctrl+S"));
     actDBSave->setEnabled(false);
     connect(actDBSave, SIGNAL(activated()), this, SLOT(itDBSave()));
     //>>> Run project execution
@@ -519,9 +517,11 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     w_scale->setToolTip(_("Field for display widgets' scaling mode."));
     w_scale->setStatusTip(_("Click to change widgets' scaling mode."));
     statusBar()->insertPermanentWidget(0,w_scale);
-    mStModify = new QLabel(" ",this);
+    mStModify = new WMdfStBar( this );
+    connect( mStModify, SIGNAL(press()), this, SLOT(itDBSave()) );
     mStModify->setWhatsThis(_("This label displays modifying."));
     mStModify->setToolTip(_("Field for display modifying."));
+    mStModify->setStatusTip(_("Click to save all modifyings."));
     statusBar()->insertPermanentWidget(0,mStModify);
     mWVisScale = new QLabel( "100%", this );
     mWVisScale->setWhatsThis(_("This label displays current widget's scale."));
@@ -546,7 +546,7 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
 
     attrInsp = new InspAttrDock(this);
     connect(attrInsp, SIGNAL(modified(const string &)), this, SIGNAL(modifiedItem(const string &)));
-    connect(attrInsp, SIGNAL(modified(const string &)), this, SLOT(modifyToolUpdate(const string &)));
+    //connect(attrInsp, SIGNAL(modified(const string &)), this, SLOT(modifyToolUpdate(const string &)));
     attrInsp->setWhatsThis(_("Dock window for widget's attributes inspection."));
     lnkInsp  = new InspLnkDock(this);
     lnkInsp->setWhatsThis(_("Dock window for widget's links inspection."));
@@ -576,8 +576,15 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     prjTree->updateTree();
 
     //Restore main window state
-    string st = TSYS::strDecode(TBDS::genDBGet(mod->nodePath()+"devWinState","",user()),TSYS::base64);
+    string st = TSYS::strDecode(mod->uiPropGet("devWinState",user()),TSYS::base64);
     restoreState(QByteArray(st.data(),st.size()));
+    //Restore ToolBars icons size
+    for( int i_ch = 0; i_ch < children().size(); i_ch++ )
+    {
+	if( !qobject_cast<QToolBar*>(children()[i_ch]) ) continue;
+	int icSz = atoi(mod->uiPropGet((children()[i_ch]->objectName()+"_icSz").toAscii().data(),user()).c_str());
+	if( icSz ) ((QToolBar*)children()[i_ch])->setIconSize(QSize(icSz,icSz));
+    }
 
     //Hide specific tools
     wdgToolView->setVisible(false);
@@ -591,15 +598,15 @@ VisDevelop::~VisDevelop()
 {
     winClose = true;
 
-    //- Save main window state -
+    //> Save main window state
     QByteArray st = saveState();
-    TBDS::genDBSet(mod->nodePath()+"devWinState",TSYS::strEncode(string(st.data(),st.size()),TSYS::base64),user());
+    mod->uiPropSet("devWinState",TSYS::strEncode(string(st.data(),st.size()),TSYS::base64),user());
 
-    //- Timers stop -
+    //> Timers stop
     endRunTimer->stop();
     work_wdgTimer->stop();
 
-    //- Other data clean -
+    //> Other data clean
     if( prjLibPropDlg )	delete prjLibPropDlg;
     if( visItPropDlg )	delete visItPropDlg;
 
@@ -650,17 +657,72 @@ void VisDevelop::closeEvent( QCloseEvent* ce )
 {
     winClose = true;
 
-    work_space->closeAllWindows();
-
     if( !SYS->stopSignal() && !exitModifChk( ) )
     {
 	ce->ignore();
+	winClose = false;
 	return;
     }
 
+    work_space->closeAllWindows();
     ce->accept();
 }
 
+QMenu *VisDevelop::createPopupMenu( )
+{
+    QMenu *mn = QMainWindow::createPopupMenu( );
+    if( !mn ) return mn;
+
+    //> Check for widget under cursor
+    QWidget *ucw = childAt(mapFromGlobal(QCursor::pos()));
+    if( qobject_cast<QToolBar*>(ucw) && !mn->children().isEmpty() )
+    {
+	QAction *first = mn->actions().isEmpty() ? NULL : mn->actions()[0];
+	QMenu *iSz = new QMenu(_("Icons size"));
+	mn->insertMenu(first,iSz);
+	mn->insertSeparator(first);
+
+	QAction *act = new QAction(_("Small (16x16)"),iSz);
+	connect(act, SIGNAL(activated()), this, SLOT(setToolIconSize()));
+	act->setObjectName("16"); 
+	act->setProperty("toolAddr",TSYS::addr2str(ucw).c_str());
+	iSz->addAction( act );
+
+	act = new QAction(_("Medium (22x22)"),iSz);
+	connect(act, SIGNAL(activated()), this, SLOT(setToolIconSize()));
+	act->setObjectName("22"); 
+	act->setProperty("toolAddr",TSYS::addr2str(ucw).c_str());
+	iSz->addAction( act );
+
+	act = new QAction(_("Big (32x32)"),iSz);
+	connect(act, SIGNAL(activated()), this, SLOT(setToolIconSize()));
+	act->setObjectName("32"); 
+	act->setProperty("toolAddr",TSYS::addr2str(ucw).c_str());
+	iSz->addAction( act );
+
+	act = new QAction(_("Huge (48x48)"),iSz);
+	connect(act, SIGNAL(activated()), this, SLOT(setToolIconSize()));
+	act->setObjectName("32"); 
+	act->setProperty("toolAddr",TSYS::addr2str(ucw).c_str());
+	iSz->addAction( act );
+    }
+
+    return mn;
+}
+
+void VisDevelop::setToolIconSize( )
+{
+    if( !sender() ) return;
+
+    QToolBar *tb = qobject_cast<QToolBar*>((QToolBar*)TSYS::str2addr(sender()->property("toolAddr").toString().toAscii().data()));
+    int icSz = atoi(sender()->objectName().toAscii().data());
+    if( tb )
+    {
+	tb->setIconSize(QSize(icSz,icSz));
+	mod->uiPropSet((tb->objectName()+"_icSz").toAscii().data(),TSYS::int2str(icSz),user());
+    }
+}
+
 bool VisDevelop::exitModifChk( )
 {
     XMLNode req("modify");
@@ -715,7 +777,7 @@ void VisDevelop::about()
 {
     QMessageBox::about(this,windowTitle(),
 	    QString(_("%1 v%2.\n%3\nAutor: %4\nDevelopers: %5\nLicense: %6\n\n%7 v%8.\n%9\nLicense: %10\nAuthor: %11\nWeb site: %12")).
-		arg(mod->modInfo("Name").c_str()).arg(mod->modInfo("Version").c_str()).arg(mod->modInfo("Descript").c_str()).
+		arg(mod->modInfo("Name").c_str()).arg(mod->modInfo("Version").c_str()).arg(mod->modInfo("Description").c_str()).
 		arg(mod->modInfo("Author").c_str()).arg(mod->modInfo(_("Developers")).c_str()).arg(mod->modInfo("License").c_str()).
 		arg(PACKAGE_NAME).arg(VERSION).arg(_(PACKAGE_DESCR)).arg(PACKAGE_LICENSE).arg(_(PACKAGE_AUTHOR)).arg(PACKAGE_SITE));
 }
@@ -783,7 +845,7 @@ void VisDevelop::modifyToolUpdate( const string &wdgs )
 {
     actDBLoad->setEnabled(false); actDBSave->setEnabled(false);
 
-    //- Request modify flag for select widgets -
+    //> Request modify flag for select widgets
     string cur_wdg;
     XMLNode req("modify");
     for( int i_off = 0; (cur_wdg=TSYS::strSepParse(wdgs,0,';',&i_off)).size(); )
@@ -796,7 +858,7 @@ void VisDevelop::modifyToolUpdate( const string &wdgs )
 	}
     }
 
-    //- Request global VCA modify -
+    //> Request global VCA modify
     mStModify->setText(" ");
     req.setAttr("path","/%2fobj");
     if( !cntrIfCmd(req) && atoi(req.text().c_str()) )
@@ -842,8 +904,9 @@ void VisDevelop::updateMenuWindow( )
 void VisDevelop::itDBLoad( )
 {
     string own_wdg = work_wdg;
+    if( own_wdg.empty() ) return;
 
-    //- Request to confirm -
+    //> Request to confirm
     InputDlg dlg(this,actDBLoad->icon(),
 	    QString(_("Are you sure of loading visual items '%1' from DB?")).arg(own_wdg.c_str()),
 	    _("Load visual item's data from DB"),false,false);
@@ -852,7 +915,7 @@ void VisDevelop::itDBLoad( )
 	string cur_wdg;
 	for( int i_off = 0; (cur_wdg=TSYS::strSepParse(own_wdg,0,';',&i_off)).size(); )
 	{
-	    //-- Send load request --
+	    //>> Send load request
 	    XMLNode req("load");
 	    req.setAttr("path",cur_wdg+"/%2fobj");
 	    if( cntrIfCmd(req) )
@@ -866,22 +929,32 @@ void VisDevelop::itDBSave( )
 {
     string own_wdg = work_wdg;
 
-    //- Request to confirm -
-    InputDlg dlg(this,actDBSave->icon(),
-	    QString(_("Are you sure of saving visual items '%1' to DB?")).arg(own_wdg.c_str()),
-	    _("Save visual item's data to DB"),false,false);
-    if( dlg.exec() == QDialog::Accepted )
+    if( sender() == mStModify )
     {
-	string cur_wdg;
-	for( int i_off = 0; (cur_wdg=TSYS::strSepParse(own_wdg,0,';',&i_off)).size(); )
+	if( mStModify->text() != "*" ) return;
+	own_wdg = "/";
+    }
+    if( !own_wdg.empty() )
+    {
+	//> Request to confirm
+	InputDlg dlg(this,actDBSave->icon(),
+		(own_wdg == "/" ? QString(_("Are you sure of saving all modifings to DB?")) :
+		QString(_("Are you sure of saving visual items '%1' to DB?")).arg(own_wdg.c_str())),
+		_("Save visual item's data to DB"),false,false);
+	if( dlg.exec() == QDialog::Accepted )
 	{
-	    //-- Send load request --
-	    XMLNode req("save");
-	    req.setAttr("path",cur_wdg+"/%2fobj");
-	    if( cntrIfCmd(req) )
-		mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
+	    string cur_wdg;
+	    for( int i_off = 0; (cur_wdg=TSYS::strSepParse(own_wdg,0,';',&i_off)).size(); )
+	    {
+		//>> Send load request
+		XMLNode req("save");
+		req.setAttr("path",cur_wdg+"/%2fobj");
+		if( cntrIfCmd(req) )
+		    mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
+	    }
 	}
     }
+    modifyToolUpdate(own_wdg);
 }
 
 void VisDevelop::prjRun( )
@@ -1185,27 +1258,43 @@ void VisDevelop::visualItEdit( )
 	connect(this, SIGNAL(modifiedItem(const string&)), vw, SLOT(load(const string &)));
 
 	scrl->setWidget( vw );
-	scrl->resize(vmax(300,vmin(650,vw->size().width()+10)),vmax(200,vmin(550,vw->size().height()+10)));
+	scrl->resize(vmax(300,vmin(950,vw->size().width()+10)),vmax(200,vmin(650,vw->size().height()+10)));
 	work_space->addWindow(scrl);
 	scrl->show();
     }
 }
 
-void VisDevelop::visualItClear( )
+void VisDevelop::visualItClear( const string &el_wa )
 {
     string clrW;
-    string work_wdg_loc = work_wdg;
+    string work_wdg_loc, work_attr;
 
-    InputDlg dlg(this,actVisItClear->icon(),
+    if( el_wa.empty() )
+    {
+	work_wdg_loc = work_wdg;
+
+	InputDlg dlg(this,actVisItClear->icon(),
 		QString(_("Are you sure of clear all changes for visual items: '%1'? "
 			  "All changes will be lost and variables are returning to default values or will be inherited!")).arg(work_wdg_loc.c_str()),
 		_("Visual items' changes clear"),false,false);
-    if( dlg.exec() != QDialog::Accepted )	return;
+	if( dlg.exec() != QDialog::Accepted )	return;
+    }
+    else
+    {
+	string work_tmp;
+	for( int off = 0; (work_tmp=TSYS::pathLev(el_wa,0,true,&off)).size(); )
+	{
+	    if( work_attr.size() ) work_wdg_loc += "/"+work_attr;
+	    work_attr = work_tmp;
+	}
+	if( work_attr.size() > 2 && work_attr.substr(0,2) == "a_" ) work_attr = work_attr.substr(2);
+	else { work_wdg_loc += "/"+work_attr; work_attr = ""; }
+    }
 
     for( int w_off = 0; (clrW=TSYS::strSepParse(work_wdg_loc,0,';',&w_off)).size(); )
     {
 	XMLNode req("set");
-	req.setAttr("path",clrW+"/%2fwdg%2fcfg%2fclear");
+	req.setAttr("path",clrW+"/%2fwdg%2fcfg%2fclear")->setAttr("attr",work_attr);;
 	if( cntrIfCmd(req) )
 	    mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	else emit modifiedItem(clrW);
@@ -1239,18 +1328,18 @@ void VisDevelop::visualItPaste( )
     for( int w_off = 0; (copy_buf_el=TSYS::strSepParse(copy_buf.substr(1),0,';',&w_off)).size(); )
     {
 	string s_elp, d_elp, s_el, d_el, t_el, t1_el;
-	//- Destination elements calc -
+	//> Destination elements calc
 	int n_del = 0;
 	for( int off = 0; !(t_el=TSYS::pathLev(work_wdg_work,0,true,&off)).empty(); n_del++ )
 	{ if( n_del ) d_elp += ("/"+d_el); d_el = t_el; }
-	//- Src elements calc -
+	//> Src elements calc
 	int n_sel = 0;
 	for( int off = 0; !(t_el=TSYS::pathLev(copy_buf_el,0,true,&off)).empty(); n_sel++ )
 	{ if( n_sel ) s_elp += ("/"+s_el); s_el = t_el; }
 
-	//- Copy visual item -
+	//> Copy visual item
 	XMLNode req("get");
-	//-- Project copy --
+	//>> Project copy
 	if( s_el.substr(0,4)=="prj_" )
 	{
 	    t_el = (QString((copy_buf[0] == '1') ? _("Project '%1' move.\n") : _("Project '%1' copy.\n"))+
@@ -1259,7 +1348,7 @@ void VisDevelop::visualItPaste( )
 	    d_el = "prj_";
 	    t1_el = s_el.substr(4);
 	}
-	//-- Widget's library copy --
+	//>> Widget's library copy
 	else if( s_el.substr(0,4)=="wlb_" )
 	{
 	    t_el = (QString((copy_buf[0] == '1') ? _("Widget's library '%1' move.\n") : _("Widget's library '%1' copy.\n"))+
@@ -1268,7 +1357,7 @@ void VisDevelop::visualItPaste( )
 	    d_el = "wlb_";
 	    t1_el = s_el.substr(4);
 	}
-	//-- Page copy --
+	//>> Page copy
 	else if( s_el.substr(0,3)=="pg_" && (d_el.substr(0,4)=="prj_" || d_el.substr(0,3)=="pg_" || d_el.substr(0,4)=="wlb_") )
 	{
 	    t_el = (QString((copy_buf[0] == '1') ? _("Move page '%1' to '%2'.\n") : _("Copy page '%1' to '%2'.\n"))+
@@ -1279,7 +1368,7 @@ void VisDevelop::visualItPaste( )
 	    d_el = (d_el.substr(0,4)=="wlb_") ? "wdg_" : "pg_";
 	    t1_el = s_el.substr(3);
 	}
-	//-- Widget copy --
+	//>> Widget copy
 	else if( s_el.substr(0,4)=="wdg_" && (d_el.substr(0,3)=="pg_" || d_el.substr(0,4)=="wlb_" || (TSYS::pathLev(d_elp,0).substr(0,4)=="wlb_" && n_del==2)) )
 	{
 	    t_el = (QString((copy_buf[0] == '1') ? _("Move widget '%1' to '%2'.\n") : _("Copy widget '%1' to '%2'.\n"))+
@@ -1290,15 +1379,15 @@ void VisDevelop::visualItPaste( )
 	    d_el = "wdg_";
 	    t1_el = s_el.substr(4);
 	}
-	//-- Copy scheme error --
+	//>> Copy scheme error
 	else
 	{
 	    mod->postMess(mod->nodePath().c_str(),QString(_("Copy/move scheme from '%1' to '%2' is not supported.")).
 		arg(copy_buf_el.c_str()).arg(work_wdg_work.c_str()),TVision::Error,this);
 	    return;
 	}
-	//-- Prepare new widget identifier --
-	//--- Remove digits from end of new identifier ---
+	//>> Prepare new widget identifier
+	//>>> Remove digits from end of new identifier
 	int i_w = 0;
 	if( cntrIfCmd(req) ) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	else
@@ -1311,7 +1400,7 @@ void VisDevelop::visualItPaste( )
 		int no_numb = t1_el.size()-1;
 		while( no_numb >= 0 && t1_el[no_numb]>='0' && t1_el[no_numb]<='9' ) no_numb--;
 		if( no_numb >= 0 ) t1_el = t1_el.substr(0,no_numb+1);
-		//--- New identifier generator ---
+		//>>> New identifier generator
 		int i_c = 1, i_w = 0;
 		while( i_w < req.childSize() )
 		    if( req.childGet(i_w)->attr("id") == t1_el+TSYS::int2str(i_c) ) { i_w = 0; i_c++; }
@@ -1319,10 +1408,10 @@ void VisDevelop::visualItPaste( )
 		t1_el += TSYS::int2str(i_c);
 	    }
 	}
-	//-- Make request dialog --
+	//>> Make request dialog
 	dlg.setMess(t_el.c_str());
 	dlg.setId(t1_el.c_str());
-	//-- Add Link flag for copy operation --
+	//>> Add Link flag for copy operation
 	if( copy_buf[0] != '1' && d_el.substr(0,4) != "prj_" && d_el.substr(0,4) != "wlb_" )
 	{
 	    dlg.edLay()->addWidget( new QLabel(_("Inherit:"),&dlg), 2, 0 );
