@@ -60,7 +60,7 @@ using namespace VISION;
 
 //****************************************
 //* Inspector of attributes model        *
-//**************************************** 
+//****************************************
 ModInspAttr::ModInspAttr( const string &iwdg, VisDevelop *mainWind ) : main_win(mainWind)
 {
     rootItem = new Item("wgrp",Item::WdgGrp);
@@ -82,14 +82,16 @@ string ModInspAttr::user( )
 
 void ModInspAttr::setWdg( const string &iwdg )
 {
-    cur_wdg = iwdg;
     bool full_reset = false;
     string sval;
     vector<string> wdg_ls;
 
-    //- Get widgets list -
-    for( int v_off = 0; (sval=TSYS::strSepParse(iwdg,0,';',&v_off)).size(); )
-        wdg_ls.push_back(sval);
+    bool isChange = (cur_wdg != iwdg);
+    cur_wdg = iwdg;
+
+    //> Get widgets list
+    for( int v_off = 0; (sval=TSYS::strSepParse(iwdg,0,';',&v_off)).size(); ) wdg_ls.push_back(sval);
+
     if( wdg_ls.size() == 0 )
     {
 	delete rootItem;
@@ -98,19 +100,19 @@ void ModInspAttr::setWdg( const string &iwdg )
     }
     else if( wdg_ls.size() == 1 )
     {
-	//- Set one widget. Check for change root item -
+	//> Set one widget. Check for change root item
 	if( rootItem->type() != Item::Wdg || rootItem->id() != wdg_ls[0] )
 	{
 	    delete rootItem;
 	    rootItem = new Item(wdg_ls[0],Item::Wdg);
 	    full_reset = true;
 	}
-	//-- Update attributes --
-	wdgAttrUpdate( QModelIndex() );// rootItem );
+	//>> Update attributes
+	wdgAttrUpdate( QModelIndex() );
     }
     else if( wdg_ls.size() > 1 )
     {
-	//- Set group widget -
+	//> Set group widget
 	if( rootItem->type() != Item::WdgGrp )
 	{
 	    beginRemoveRows(QModelIndex(),0,rootItem->childCount());
@@ -118,8 +120,10 @@ void ModInspAttr::setWdg( const string &iwdg )
 	    rootItem = new Item("wgrp",Item::WdgGrp);
 	    endRemoveRows();
 	}
-	//- Check for delete widgets from group -
-	for( int i_it = 0; i_it < rootItem->childCount(); i_it++ )
+
+	//> Check for delete widgets from group
+	bool masterWdg = !isChange && (rootItem->childCount() && rootItem->child(0)->id() == "<*>");
+	for( int i_it = (masterWdg?1:0); i_it < rootItem->childCount(); i_it++ )
 	{
 	    int i_w;
 	    for( i_w = 0; i_w < wdg_ls.size(); i_w++ )
@@ -133,17 +137,27 @@ void ModInspAttr::setWdg( const string &iwdg )
 		i_it--;
 	    }
 	}
-	//- Add new items and update attributes -
+
+	//> Check for add master widget
+	if( !masterWdg )
+	{
+	    beginInsertRows(QModelIndex(),0,0);
+	    rootItem->childInsert("<*>",0,Item::Wdg);
+	    rootItem->child(0)->setName(_("<Group>"));
+	    endInsertRows();
+	}
+
+	//> Add new items and update attributes
 	for( int i_w = 0; i_w < wdg_ls.size(); i_w++ )
 	{
 	    int row = rootItem->childGet(wdg_ls[i_w]);
 	    if( row < 0 )
 	    {
-		beginInsertRows(QModelIndex(),i_w,i_w);
-		row = rootItem->childInsert(wdg_ls[i_w],i_w,Item::Wdg);
+		beginInsertRows(QModelIndex(),i_w+1,i_w+1);
+		row = rootItem->childInsert(wdg_ls[i_w],i_w+1,Item::Wdg);
 		endInsertRows();
 	    }
-	    wdgAttrUpdate( index(i_w,0,QModelIndex()) );// rootItem->child(i_w));
+	    wdgAttrUpdate( index(i_w+1,0,QModelIndex()), index(0,0,QModelIndex()) );
 	}
     }
 
@@ -151,30 +165,35 @@ void ModInspAttr::setWdg( const string &iwdg )
     else emit layoutChanged();
 }
 
-void ModInspAttr::wdgAttrUpdate( const QModelIndex &mod_it ) // Item *it)
+void ModInspAttr::wdgAttrUpdate( const QModelIndex &mod_it, const QModelIndex &grp_it )
 {
+    vector<int> idst;
+    bool grpW = false;
     Item *it = mod_it.isValid() ? static_cast<Item*>(mod_it.internalPointer()) : rootItem;
     if( it->type() != Item::Wdg )   return;
+    QModelIndex curmod = mod_it;
+    string itId = it->id();
 
     try
     {
 	XMLNode info_req("info");
 	XMLNode req("get");
 
-        //> Set/update widget name
-	req.setAttr("path",it->id()+"/"+TSYS::strEncode("/wdg/cfg/name",TSYS::PathEl));
+	//> Set/update widget name
+	req.setAttr("path",itId+"/"+TSYS::strEncode("/wdg/cfg/name",TSYS::PathEl));
 	if( !mainWin()->cntrIfCmd(req) )	it->setName(req.text().c_str());
 
-	info_req.setAttr("path",it->id()+"/%2fattr" );
+	info_req.setAttr("path",itId+"/%2fattr" );
 	mainWin()->cntrIfCmd(info_req);
 	XMLNode *root = info_req.childGet(0);
 
+	repIt:
 	//> Delete items of a no present attributes
-	vector<int> idst;
+	idst.clear();
 	idst.push_back(0);
 	int it_lev = 0;
-	QModelIndex curmod = mod_it;
 	Item *curit = it;
+
 	//>> Get next item
 	while( idst[it_lev] < curit->childCount() )
 	{
@@ -188,7 +207,7 @@ void ModInspAttr::wdgAttrUpdate( const QModelIndex &mod_it ) // Item *it)
 		    if( root->childGet(i_a)->attr("id") == it_id )
 			break;
 		//>>> Remove no present item
-		if( i_a >= root->childSize() )
+		if( i_a >= root->childSize() && (!grpW || !curit->child(idst[it_lev])->setWdgs(itId,true)) )
 		{
 		    beginRemoveRows(curmod,idst[it_lev],idst[it_lev]);
 		    curit->childDel(idst[it_lev]);
@@ -235,12 +254,14 @@ void ModInspAttr::wdgAttrUpdate( const QModelIndex &mod_it ) // Item *it)
 	for( int i_a = 0; i_a < root->childSize(); i_a++ )
 	{
 	    XMLNode *gnd = root->childGet(i_a);
+	    if( grpW && !(atoi(gnd->attr("acs").c_str())&SEC_WR) ) continue;
+
 	    string a_id = gnd->attr("id");
 	    string a_nm = gnd->attr("dscr");
 	    Item *cur_it = it;
 	    //>> Parse attributes group
 	    if( TSYS::strSepParse(a_nm,1,':').size() )
-		for(int i_l = 0; true; i_l++)
+		for( int i_l = 0; true; i_l++ )
 		{
 		    string c_sel = TSYS::strSepParse(a_nm,i_l,':');
 		    if( TSYS::strSepParse(a_nm,i_l+1,':').size() )
@@ -258,14 +279,16 @@ void ModInspAttr::wdgAttrUpdate( const QModelIndex &mod_it ) // Item *it)
 	    }
 	    //>> Check attribute item
 	    int ga_id = cur_it->childGet(a_id);
-	    if( ga_id < 0 ) ga_id = cur_it->childInsert(a_id,-1,Item::Attr);
-	    cur_it->child(ga_id)->setName(a_nm);
-	    cur_it->child(ga_id)->setEdited(atoi(gnd->attr("acs").c_str())&SEQ_WR);
+	    if( grpW && ga_id >= 0 ) { cur_it->child(ga_id)->setWdgs(itId); continue; }
+	    if( ga_id < 0 ) ga_id = cur_it->childInsert( a_id, -1, Item::Attr );
+	    cur_it->child(ga_id)->setName( a_nm );
+	    cur_it->child(ga_id)->setEdited( atoi(gnd->attr("acs").c_str())&SEC_WR );
 	    cur_it->child(ga_id)->setFlag( atoi(gnd->attr("wdgFlg").c_str()) );
-	    cur_it->child(ga_id)->setModify( atoi(gnd->attr("modif").c_str()) );
+	    cur_it->child(ga_id)->setModify( grpW ? false : atoi(gnd->attr("modif").c_str()) );
+	    if( grpW ) cur_it->child(ga_id)->setWdgs(itId);
 	    //>> Get Value
 	    string sval;
-	    req.clear()->setAttr("path",it->id()+"/%2fattr%2f"+a_id);
+	    req.clear()->setAttr( "path", itId+"/%2fattr%2f"+a_id );
 	    if( !mainWin()->cntrIfCmd(req) )	sval = req.text();
 	    string stp = gnd->attr("tp");
 	    if( stp == "bool" )		cur_it->child(ga_id)->setData((bool)atoi(sval.c_str()));
@@ -277,6 +300,14 @@ void ModInspAttr::wdgAttrUpdate( const QModelIndex &mod_it ) // Item *it)
 	    if( gnd->attr("dest") == "select" )
 		cur_it->child(ga_id)->setDataEdit( QString(gnd->attr("sel_list").c_str()).split(";") );
 	}
+
+	if( grp_it.isValid() && !grpW )
+	{
+	    curmod = grp_it;
+	    it = static_cast<Item*>(grp_it.internalPointer());
+	    grpW = true;
+	    goto repIt;
+	}
     } catch(...){ }
 }
 
@@ -364,7 +395,7 @@ QVariant ModInspAttr::data( const QModelIndex &index, int role ) const
 	    {
 		case Qt::DisplayRole:
 		    val = it->data();
-		    if( val.type() == QVariant::Int && it->flag()&ModInspAttr::Item::DataTime )
+		    if( val.type() == QVariant::Int && it->flag()&ModInspAttr::Item::DateTime )
 			val = QDateTime::fromTime_t(val.toInt()?val.toInt():time(NULL)).toString("dd.MM.yyyy hh:mm:ss");
 		    break;
 		case Qt::EditRole:		val = it->dataEdit();	break;
@@ -441,37 +472,43 @@ bool ModInspAttr::setData( const QModelIndex &index, const QVariant &value, int
 {
     if( !index.isValid() )  return false;
 
-    //Attribute
+    //> Attribute
     Item *it = static_cast<Item*>(index.internalPointer());
+    if( it->data() == value ) return true;
     string nattr = it->id();
-    //Attribute widget
-    string nwdg;
-    Item *cit = it;
-    while(cit)
-	if(cit->type() == Item::Wdg)
-	{
-	    nwdg = cit->id();
-	    break;
-	}
-	else cit = cit->parent();
+
+    //> Attribute widget(s)
+    string nwdg = it->wdgs( ), swdg;
+    if( nwdg.empty() )
+    {    
+	Item *cit = it;
+	while(cit)
+	    if(cit->type() == Item::Wdg)
+	    {
+		nwdg = cit->id();
+		break;
+	    }
+	    else cit = cit->parent();
+    }
 
     try
     {
+	string val = (value.type()==QVariant::Bool) ? (value.toBool()?"1":"0") : value.toString().toAscii().data();
 	XMLNode req("set");
-	req.setAttr("path",nwdg+"/%2fattr%2f"+nattr)->
-	    setText((value.type()==QVariant::Bool) ?
-			(value.toBool()?"1":"0") : value.toString().toAscii().data());
-	if( !mainWin()->cntrIfCmd(req) )
+	for( int off = 0; (swdg=TSYS::strSepParse(nwdg,0,';',&off)).size(); )
 	{
-	    it->setData( (it->data().type()==QVariant::Bool) ? value.toBool() : value );
-	    it->setModify(true);
-	    emit modified(nwdg);
-	    emit dataChanged(index,index);
-	    if( it->flag()&Item::Active ) setWdg(cur_wdg);
+	    req.setAttr("path",swdg+"/%2fattr%2f"+nattr)->setText(val);
+	    if( !mainWin()->cntrIfCmd(req) && req.text() == val )
+	    {
+		it->setData( (it->data().type()==QVariant::Bool) ? value.toBool() : value );
+		it->setModify(true);
+		emit modified(swdg);
+		emit dataChanged(index,index);
+		if( it->flag()&Item::Active ) setWdg(cur_wdg);
+	    }
 	}
     }catch(...){ return false; }
 
-
     return true;
 }
 
@@ -551,6 +588,15 @@ QVariant ModInspAttr::Item::dataEdit( )
     return dataEditItem.isValid()?dataEditItem:dataItem;
 }
 
+bool ModInspAttr::Item::setWdgs( const string &w, bool del )
+{
+    int pos = 0;
+    if( !del && wdgsItem.find(w+";") == string::npos ) wdgsItem += w+";";
+    if( del && (pos=wdgsItem.find(w+";")) != string::npos ) wdgsItem.replace(pos,w.size()+1,"");
+
+    return !wdgsItem.empty();
+}
+
 //****************************************
 //* Inspector of attributes widget       *
 //****************************************
@@ -561,6 +607,7 @@ InspAttr::InspAttr( QWidget * parent, VisDevelop *mainWind ) : QTreeView(parent)
     setModel(&modelData);
     setItemDelegate(new ItemDelegate);
     connect(&modelData, SIGNAL(modified(const string &)), this, SIGNAL(modified(const string &)));
+    setContextMenuPolicy(Qt::DefaultContextMenu);
 }
 
 InspAttr::~InspAttr( )
@@ -594,6 +641,54 @@ bool InspAttr::event( QEvent *event )
     return QTreeView::event( event );
 }
 
+void InspAttr::contextMenuEvent( QContextMenuEvent *event )
+{
+    string nattr, nwdg;
+    QAction *actClr = NULL;
+    ModInspAttr::Item *it = NULL;
+
+    //Attribute
+    if( selectedIndexes().size() && selectedIndexes()[0].isValid() )
+    {
+	it = static_cast<ModInspAttr::Item*>(selectedIndexes()[0].internalPointer());
+	nattr = it->id();
+	//Attribute widget
+	ModInspAttr::Item *cit = it;
+	while(cit)
+	    if( cit->type() == ModInspAttr::Item::Wdg )
+	    {
+		nwdg = cit->id();
+		break;
+	    }
+	    else cit = cit->parent();
+    }
+
+    QMenu popup;
+
+    //Add actions
+    //> Changes clear action
+    if( it && it->modify() )
+    {
+	QImage ico_t;
+	if(!ico_t.load(TUIS::icoPath("reload").c_str())) ico_t.load(":/images/reload.png");
+	actClr = new QAction(QPixmap::fromImage(ico_t),_("Clear changes"),this);
+	actClr->setStatusTip(_("Press to clear attribute's changes."));
+	popup.addAction(actClr);
+    }
+
+    if( !popup.isEmpty() )
+    {
+	QAction *rez = popup.exec(QCursor::pos());
+	if( actClr && rez == actClr )
+	{
+	    modelData.mainWin()->visualItClear(nwdg+"/a_"+nattr);
+	    modelData.setWdg(modelData.curWdg());
+	}
+
+	popup.clear();
+    }
+}
+
 //* Attributes item delegate    *
 //*******************************
 InspAttr::ItemDelegate::ItemDelegate( QObject *parent ) : QItemDelegate(parent)
@@ -613,6 +708,7 @@ QWidget *InspAttr::ItemDelegate::createEditor(QWidget *parent, const QStyleOptio
     else if( value.type() == QVariant::String && flag&ModInspAttr::Item::FullText )
     {
 	w_del = new QTextEdit(parent);
+	((QTextEdit*)w_del)->setTabStopWidth(40);
 	((QTextEdit*)w_del)->setLineWrapMode(QTextEdit::NoWrap);
 	((QTextEdit*)w_del)->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 	((QTextEdit*)w_del)->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -622,9 +718,10 @@ QWidget *InspAttr::ItemDelegate::createEditor(QWidget *parent, const QStyleOptio
 	w_del = new LineEditProp(parent,LineEditProp::Font);
     else if( value.type() == QVariant::String && flag&ModInspAttr::Item::Color )
 	w_del = new LineEditProp(parent,LineEditProp::Color);
-    else if( value.type() == QVariant::Int && flag&ModInspAttr::Item::DataTime )
+    else if( value.type() == QVariant::Int && flag&ModInspAttr::Item::DateTime )
     {
 	w_del = new QDateTimeEdit(parent);
+	((QDateTimeEdit*)w_del)->setCalendarPopup(true);
 	((QDateTimeEdit*)w_del)->setDisplayFormat("dd.MM.yyyy hh:mm:ss");
     }
     else if( value.type() == QVariant::Int )
@@ -665,7 +762,7 @@ void InspAttr::ItemDelegate::setEditorData(QWidget *editor, const QModelIndex &i
 	((QTextEdit*)editor)->setPlainText(value.toString());
     else if( value.type() == QVariant::String && (flag&ModInspAttr::Item::Font || flag&ModInspAttr::Item::Color) && dynamic_cast<LineEditProp*>(editor) )
 	((LineEditProp*)editor)->setValue(value.toString());
-    else if( value.type() == QVariant::Int && flag&ModInspAttr::Item::DataTime && dynamic_cast<QDateTimeEdit*>(editor) )
+    else if( value.type() == QVariant::Int && flag&ModInspAttr::Item::DateTime && dynamic_cast<QDateTimeEdit*>(editor) )
 	((QDateTimeEdit*)editor)->setDateTime(QDateTime::fromTime_t(value.toInt()?value.toInt():time(NULL)));
     else QItemDelegate::setEditorData(editor, index);
 }
@@ -681,7 +778,7 @@ void InspAttr::ItemDelegate::setModelData(QWidget *editor, QAbstractItemModel *m
 	model->setData(index,((QTextEdit*)editor)->toPlainText(),Qt::EditRole);
     else if( value.type() == QVariant::String && (flag&ModInspAttr::Item::Font || flag&ModInspAttr::Item::Color) && dynamic_cast<LineEditProp*>(editor) )
 	model->setData(index,((LineEditProp*)editor)->value());
-    else if( value.type() == QVariant::Int && flag&ModInspAttr::Item::DataTime && dynamic_cast<QDateTimeEdit*>(editor) )
+    else if( value.type() == QVariant::Int && flag&ModInspAttr::Item::DateTime && dynamic_cast<QDateTimeEdit*>(editor) )
     {
 	int tm = ((QDateTimeEdit*)editor)->dateTime().toTime_t();
 	model->setData(index,(tm>(time(NULL)+3600))?0:tm,Qt::EditRole);
@@ -828,12 +925,12 @@ void InspLnk::setWdg( const string &iwdg )
     //Update tree
     XMLNode req("get");
 
-    //- Get links info -
+    //> Get links info
     XMLNode info_req("info");
     info_req.setAttr("path",it_wdg+"/%2flinks%2flnk")->setAttr("showAttr","1")->setAttr("inclValue","1");
     if( mainWin()->cntrIfCmd(info_req) ) return;
     XMLNode *rootel = info_req.childGet(0);
-    //- Create widget's root items -
+    //> Create widget's root items
     for( int i_l = 0; i_l < rootel->childSize(); i_l++ )
     {
 	lnid  = rootel->childGet(i_l)->attr("id");
@@ -842,7 +939,7 @@ void InspLnk::setWdg( const string &iwdg )
 	lnatr = TSYS::strSepParse(lnid.substr(3),1,'.');
 	if( lnatr.empty() )	{ lnatr = lnwdg; lnwdg = "."; }
 
-	//- Search widget item -
+	//> Search widget item
 	QTreeWidgetItem *wdg_it;
 	int i_it;
 	for( i_it = 0; i_it < topLevelItemCount(); i_it++ )
@@ -857,7 +954,7 @@ void InspLnk::setWdg( const string &iwdg )
 
 	if( !lngrp.empty() )
 	{
-	    //-- Search group --
+	    //>> Search group
 	    for( i_it = 0; i_it < wdg_it->childCount(); i_it++ )
 		if( lngrp == wdg_it->child(i_it)->text(0).toAscii().data() )
 		    break;
@@ -869,7 +966,7 @@ void InspLnk::setWdg( const string &iwdg )
 		wdg_it->setText(0,lngrp.c_str());
 		wdg_it->setData(0,Qt::UserRole,QString(lnid.substr(3).c_str()));
 	    }
-	    //--- Get group value ---
+	    //>>> Get group value
 	    grpcd = TSYS::addr2str(wdg_it)+lngrp;
 	    if( ugrps.find(grpcd) == ugrps.end() )
 	    {
@@ -878,7 +975,7 @@ void InspLnk::setWdg( const string &iwdg )
 		ugrps.insert(std::pair<string,bool>(grpcd,true));
 	    }
 	}
-	//-- Search parameter --
+	//>> Search parameter
 	QTreeWidgetItem *prm_it;
 	for( i_it = 0; i_it < wdg_it->childCount(); i_it++ )
 	    if( lnatr == wdg_it->child(i_it)->text(0).toAscii().data() )
@@ -891,14 +988,14 @@ void InspLnk::setWdg( const string &iwdg )
 	    prm_it->setText(0,lnatr.c_str());
 	    prm_it->setData(0,Qt::UserRole,QString(lnid.substr(3).c_str()));
 	}
-	//--- Get parameter's value ---
+	//>>> Get parameter's value
 	prm_it->setText(1,rootel->childGet(i_l)->text().c_str());
 	//req.clear()->setAttr("path",it_wdg+"/%2flinks%2flnk%2f"+lnid);
 	//if( !mainWin()->cntrIfCmd(req) )
 	//    prm_it->setText(1,req.text().c_str());
     }
 
-    //- Check for deleted links -
+    //> Check for deleted links
     for( int i_it = 0; i_it < topLevelItemCount(); i_it++ )
 	for( int i_g = 0, i_a = 0; i_g < topLevelItem(i_it)->childCount(); )
 	{
@@ -935,7 +1032,7 @@ void InspLnk::setWdg( const string &iwdg )
 	    if( wdg_g != wdg_it && i_a >= wdg_g->childCount() )	{ i_a = 0; i_g++; }
 	}
 
-    //- Set widget's path -
+    //> Set widget's path
     if( topLevelItemCount() )	topLevelItem(0)->setData(0,Qt::UserRole,QString(it_wdg.c_str()));
 
     show_init = false;
@@ -1437,7 +1534,7 @@ void WdgTree::updateTree( const string &vca_it )
 		img.loadFromData((const uchar*)simg.c_str(),simg.size());
 		if( !img.isNull() ) nit_cw->setIcon(0,QPixmap::fromImage(img));
 		nit_cw->setText(0,cwdgN->text().c_str());
-		nit_cw->setText(1,_("Container widget"));
+		nit_cw->setText(1,_("Container's widget"));
 		nit_cw->setText(2,cwdgId.c_str());
 	    }
 	}
@@ -1825,6 +1922,19 @@ void WScaleStBar::mousePressEvent( QMouseEvent * event )
     setScale(!scale());
 }
 
+//*********************************************
+//* Status bar modify indicator               *
+//*********************************************
+WMdfStBar::WMdfStBar( QWidget *parent ) : QLabel(parent)
+{
+
+}
+
+void WMdfStBar::mousePressEvent( QMouseEvent * event )
+{
+    emit press();
+}
+
 //****************************************
 //* Shape widget view development mode   *
 //****************************************
@@ -1848,17 +1958,17 @@ DevelWdgView::DevelWdgView( const string &iwid, int ilevel, VisDevelop *mainWind
     }
     //> Select only created widgets by user
     else if( wLevel() == 1 && ((WdgView*)parentWidget())->isReload )
-    { setSelect(true); z_coord = 100000; }
+    { setSelect(true,PrcChilds); z_coord = 100000; }
 }
 
 DevelWdgView::~DevelWdgView( )
 {
     if( select() && !mod->endRun( ) )
     {
-	setSelect(false,false);
+	setSelect(false);
 	for( int i_c = 0; i_c < children().size(); i_c++ )
 	    if( qobject_cast<DevelWdgView*>(children().at(i_c)) )
-		((DevelWdgView*)children().at(i_c))->setSelect(false);
+		((DevelWdgView*)children().at(i_c))->setSelect(false,PrcChilds);
     }
 }
 
@@ -1903,21 +2013,21 @@ void DevelWdgView::saveGeom( const string& item )
 	    if( qobject_cast<DevelWdgView*>(children().at(i_c)) )
 		((DevelWdgView*)children().at(i_c))->saveGeom(item);
 
-    //- For top items (like inspector) data update -
-    if( wLevel() == 0  )  setSelect(true);
+    //> For top items (like inspector) data update
+    if( wLevel() == 0  )  setSelect(true,PrcChilds);
 }
 
-void DevelWdgView::setSelect( bool vl, bool childs, bool onlyFlag )
+void DevelWdgView::setSelect( bool vl, char flgs )// bool childs, bool onlyFlag, bool noUpdate )
 {
     int chld_cnt = 0;
 
     fWdgSelect = vl;
-    if( !vl && edit() && !onlyFlag ) setEdit(false);
+    if( !vl && edit() && !(flgs&OnlyFlag) ) setEdit(false);
 
-    //- Level 0 process -
+    //> Level 0 process
     if( wLevel() != 0 && !edit() ) return;
 
-    if( vl && !onlyFlag )
+    if( vl && !(flgs&OnlyFlag) )
     {
 	string sel_chlds = selectChilds(&chld_cnt);
 	if( sel_chlds.size() )	emit selected(sel_chlds);
@@ -1925,16 +2035,16 @@ void DevelWdgView::setSelect( bool vl, bool childs, bool onlyFlag )
     }
     if( !vl )
     {
-	if( childs )
+	if( flgs&PrcChilds )
 	    for( int i_c = 0; i_c < children().size(); i_c++ )
 		if( qobject_cast<DevelWdgView*>(children().at(i_c)) )
-		    qobject_cast<DevelWdgView*>(children().at(i_c))->setSelect(false,true,onlyFlag);
-	if( !onlyFlag )	emit selected("");
+		    qobject_cast<DevelWdgView*>(children().at(i_c))->setSelect(false,flgs|OnlyFlag);
+	if( !(flgs&OnlyFlag) )	emit selected("");
     }
 
-    //- Update actions access -
-    //-- Enable view toolbar --
-    if( !onlyFlag )
+    //> Update actions access
+    //>> Enable view toolbar
+    if( !(flgs&OnlyFlag) )
     {
 	if( !edit() )
 	{
@@ -1942,12 +2052,12 @@ void DevelWdgView::setSelect( bool vl, bool childs, bool onlyFlag )
 	    disconnect( mainWin()->wdgToolView, SIGNAL(actionTriggered(QAction*)), this, SLOT(wdgViewTool(QAction*)) );
 	    if( vl ) connect( mainWin()->wdgToolView, SIGNAL(actionTriggered(QAction*)), this, SLOT(wdgViewTool(QAction*)) );
 
-	    //-- Update widget view tools --
+	    //>> Update widget view tools
 	    for( int i_a = 0; i_a < mainWin()->wdgToolView->actions().size(); i_a++ )
 		mainWin()->wdgToolView->actions().at(i_a)->setEnabled(chld_cnt>0);
 	}
 
-	update();
+	if( !(flgs&NoUpdate) ) update();
     }
 }
 
@@ -2316,7 +2426,7 @@ void DevelWdgView::editExit( )
 {
     for( int i_c = 0; i_c < children().size(); i_c++ )
 	if( qobject_cast<DevelWdgView*>(children().at(i_c)) )
-	    ((DevelWdgView*)children().at(i_c))->setSelect(false);
+	    ((DevelWdgView*)children().at(i_c))->setSelect(false,PrcChilds);
     setEdit(false);
     update();
 }
@@ -2510,20 +2620,20 @@ void DevelWdgView::incDecVisScale( )
 
 bool DevelWdgView::event( QEvent *event )
 {
-    //- Paint event process -
+    //> Paint event process
     if( event->type() == QEvent::Paint )
     {
 	QPainter pnt( this );
 	pnt.setWindow( rect() );
 
-	//- Draw background for root widget -
+	//> Draw background for root widget
 	if( wLevel() == 0 )
 	{
 	    pnt.setPen("black");
 	    pnt.setBrush(QBrush(QColor("white")));
 	    pnt.drawRect(rect().adjusted(0,0,-1,-1));
 	}
-	//- Draw widget border geometry -
+	//> Draw widget border geometry
 	else if( levelWidget(1)->select() && levelWidget(0)->fMoveHoldMove )
 	{
 	    if( !fHideChilds )
@@ -2548,14 +2658,14 @@ bool DevelWdgView::event( QEvent *event )
 		if( qobject_cast<QWidget*>(children().at(i_c)) )
 		    ((QWidget*)children().at(i_c))->setEnabled(true);//show();
 	}
-	//- Check widget -
+	//> Check widget
 	if( !shape )
 	{
 	    pnt.drawImage(rect(),QImage(":/images/attention.png"));
 	    setToolTip(QString(_("Widget is not enabled or shape is not supported!")));
 	}
 
-	//- Update select widget data -
+	//> Update select widget data
 	if( wLevel() == 0 && !fHoldSelRect )
 	{
 	    QRectF rsel;
@@ -2571,7 +2681,7 @@ bool DevelWdgView::event( QEvent *event )
 	return QWidget::event(event);
     }
 
-    //- Other events process -
+    //> Other events process
     if( wLevel() == 0 )
 	switch( event->type() )
 	{
@@ -2588,11 +2698,11 @@ bool DevelWdgView::event( QEvent *event )
 	    }
 	    case QEvent::Drop:
 	    {
-	        QDropEvent *ev = static_cast<QDropEvent*>(event);
-	        if( ev->mimeData()->hasFormat("application/OpenSCADA-libwdg") )
-	        {
-	            QByteArray itemData = ev->mimeData()->data("application/OpenSCADA-libwdg");
-	            QDataStream dataStream(&itemData, QIODevice::ReadOnly);
+		QDropEvent *ev = static_cast<QDropEvent*>(event);
+		if( ev->mimeData()->hasFormat("application/OpenSCADA-libwdg") )
+		{
+		    QByteArray itemData = ev->mimeData()->data("application/OpenSCADA-libwdg");
+		    QDataStream dataStream(&itemData, QIODevice::ReadOnly);
 
 		    QString lwdg;
 		    dataStream >> lwdg;
@@ -2613,7 +2723,7 @@ bool DevelWdgView::event( QEvent *event )
 
 		QPoint curp = mapFromGlobal(cursor().pos());
 
-		//- New widget inserting -
+		//> New widget inserting
 		QAction *act = mainWin()->actGrpWdgAdd->checkedAction();
 		if( act && act->isChecked() && (static_cast<QMouseEvent*>(event))->buttons()&Qt::LeftButton )
 		{
@@ -2623,14 +2733,14 @@ bool DevelWdgView::event( QEvent *event )
 		    return true;
 		}
 
-		//- Select widget -
+		//> Select widget
 		if( (static_cast<QMouseEvent*>(event))->buttons()&Qt::LeftButton )
 		{
 		    dragStartPos = ((QMouseEvent*)event)->pos();
 		    bool sh_hold = QApplication::keyboardModifiers()&Qt::ShiftModifier;
 		    if( cursor().shape() == Qt::ArrowCursor || sh_hold )
 		    {
-			//-- Scan childs --
+			//>> Scan childs
 			bool sel_modif = false;
 			bool chld_sel = false;
 			DevelWdgView *cwdg = NULL;
@@ -2640,30 +2750,30 @@ bool DevelWdgView::event( QEvent *event )
 			    if( !cwdg ) continue;
 			    if( cwdg->geometryF().contains(curp) )
 			    {
-				if( !cwdg->select() )	{ cwdg->setSelect(true,true,true);  sel_modif = true; }
-				else if( sh_hold )	{ cwdg->setSelect(false,true,true); sel_modif = true; }
+				if( !cwdg->select() )	{ cwdg->setSelect(true,PrcChilds|OnlyFlag);  sel_modif = true; }
+				else if( sh_hold )	{ cwdg->setSelect(false,PrcChilds|OnlyFlag); sel_modif = true; }
 				if( cwdg->select() )	chld_sel = true;
 				break;
 			    }
 			}
-			//-- Select clean for childs --
+			//>> Select clean for childs
 			if( !sh_hold )
 			    for( int i_c = 0; i_c < children().size(); i_c++ )
 			    {
 				DevelWdgView *curw = qobject_cast<DevelWdgView*>(children().at(i_c));
 				if( !curw || (chld_sel && (curw == cwdg)) )	continue;
-				if( curw->select() )	{ curw->setSelect(false,true,true); sel_modif = true; }
+				if( curw->select() )	{ curw->setSelect(false,PrcChilds|OnlyFlag); sel_modif = true; }
 			    }
-			if( sel_modif || !select() )	{ setSelect(true,true,true); fSelChange = true; }
+			if( sel_modif || !select() )	{ setSelect(true,PrcChilds|OnlyFlag); fSelChange = true; }
 			event->accept();
 
 			upMouseCursors(mapFromGlobal(cursor().pos()));
 
-			//-- Update status bar --
+			//>> Update status bar
 			mainWin()->statusBar()->showMessage(QString(_("Select elements: '%1'")).
 							arg(selectChilds().c_str()), 10000 );
 
-			//-- Hold select rect paint --
+			//>> Hold select rect paint
 			if( !chld_sel )
 			{
 			    fHoldSelRect = true;
@@ -2675,27 +2785,53 @@ bool DevelWdgView::event( QEvent *event )
 			fMoveHold = true;
 			holdPnt = curp;
 		    }
+		    update();	// ???? For QT's included widget's update bug hack (Document,Protocol and other)
 		    return true;
 		}
 		break;
 	    }
 	    case QEvent::MouseButtonRelease:
+	    {
 		if( edit() ) break;
 
+		QPoint curp = mapFromGlobal(cursor().pos());
+
 		if( fHoldSelRect )
 		{
-		    QPoint curp = mapFromGlobal(cursor().pos());
 		    for( int i_c = children().size()-1; i_c >= 0; i_c-- )
 		    {
 			DevelWdgView *cwdg = qobject_cast<DevelWdgView*>(children().at(i_c));
 			if( !cwdg || !QRect(holdPnt,curp).contains(cwdg->geometryF().toRect()) ) continue;
-			cwdg->setSelect(true,true,true);
+			cwdg->setSelect(true,PrcChilds|OnlyFlag);
 		    }
-		    setSelect(true);
+		    setSelect(true,PrcChilds);
 		    fHoldSelRect = false;
 		    //pntView->setSelArea(QRectF());
 		}
-		if( fSelChange )	{ setSelect(true); fSelChange = false; }
+
+		//> Check for select next underly widget
+		if( fMoveHold && cursor().shape() != Qt::ArrowCursor && !fSelChange && !fMoveHoldMove )
+		{
+		    DevelWdgView *fsel = NULL, *nsel = NULL;
+		    int i_c;
+		    for( i_c = children().size()-1; i_c >= 0; i_c-- )
+		    {
+			DevelWdgView *curw = qobject_cast<DevelWdgView*>(children().at(i_c));
+			if( !curw ) continue;
+			if( !fsel && curw->select() ) fsel = curw;
+			else if( fsel && curw->geometryF().contains(curp) ) { nsel = curw; break; }
+		    }
+		    if( fsel ) fsel->setSelect(false,PrcChilds|OnlyFlag);
+		    if( nsel ) nsel->setSelect(true,PrcChilds|OnlyFlag);
+		    else setCursor(Qt::ArrowCursor);
+		    setSelect(true,PrcChilds);
+		}
+
+		if( fSelChange )
+		{
+		    setSelect(true,PrcChilds|NoUpdate);	// ???? For QT's included widget's update bug hack (Document,Protocol and other)
+		    fSelChange = false;
+		}
 
 		if( fMoveHold )
 		{
@@ -2721,7 +2857,9 @@ bool DevelWdgView::event( QEvent *event )
 		    fMoveHold = fMoveHoldMove = false;
 		    return true;
 		}
+
 		break;
+	    }
 	    case QEvent::MouseButtonDblClick:
 	    {
 		if( edit() )	break;
@@ -2738,9 +2876,9 @@ bool DevelWdgView::event( QEvent *event )
 		        if( !cwdg->shape || !cwdg->shape->isEditable( ) )	continue;
 		        edwdg = cwdg;
 		    }
-		    else if( cwdg->select() ) cwdg->setSelect(false);
+		    else if( cwdg->select() ) cwdg->setSelect(false,PrcChilds);
 		}
-		if( edwdg && !edwdg->select() )	edwdg->setSelect(true);
+		if( edwdg && !edwdg->select() )	edwdg->setSelect(true,PrcChilds);
 		editEnter( );
 		return true;
 	    }
@@ -2752,20 +2890,20 @@ bool DevelWdgView::event( QEvent *event )
 		if( edit() )    break;
 		//-- Unselect and store child widgets --
 		//if(select())
-		setSelect(true);
+		setSelect(true,PrcChilds);
 		mainWin()->setWdgScale(false);
 		return true;
 	    case QEvent::FocusOut:
 		if( cursor().shape() != Qt::ArrowCursor )	setCursor(Qt::ArrowCursor);
 		if( QApplication::focusWidget() != this && !mainWin()->attrInsp->hasFocus() && !mainWin()->lnkInsp->hasFocus() )
 		{
-		    if( editWdg )	editWdg->setSelect(false);
+		    if( editWdg )	editWdg->setSelect(false,PrcChilds);
 		    //emit selected("");
-		    setSelect(false,false);
+		    setSelect(false);
 		    //-- Unselect and store child widgets --
 		    /*for( int i_c = 0; i_c < children().size(); i_c++ )
 			if( qobject_cast<DevelWdgView*>(children().at(i_c)) && ((DevelWdgView*)children().at(i_c))->select() )
-			    ((DevelWdgView*)children().at(i_c))->setSelect(false);*/
+			    ((DevelWdgView*)children().at(i_c))->setSelect(false,PrcChilds);*/
 		}
 		return true;
 	    case QEvent::MouseMove:
@@ -2821,8 +2959,8 @@ bool DevelWdgView::event( QEvent *event )
 			    if( !(QApplication::keyboardModifiers()&Qt::ControlModifier) )	break;
 			    for( int i_c = children().size()-1; i_c >= 0; i_c-- )
 				if( qobject_cast<DevelWdgView*>(children().at(i_c)) )
-				    ((DevelWdgView*)children().at(i_c))->setSelect(true);
-			    setSelect(true);
+				    ((DevelWdgView*)children().at(i_c))->setSelect(true,PrcChilds);
+			    setSelect(true,PrcChilds);
 			    break;
 			case Qt::Key_Left: case Qt::Key_Right: case Qt::Key_Up: case Qt::Key_Down:
 			{
@@ -2883,6 +3021,7 @@ SizePntWdg::SizePntWdg( QWidget* parent ) : QWidget(parent), view(SizeDots)
 void SizePntWdg::setSelArea( const QRectF &geom, WView iview )
 {
     if( view == iview && mWPos == geom.topLeft() && mWSize == geom.size() ) return;
+
     view   = iview;
     mWPos  = geom.topLeft();
     mWSize = geom.size();
@@ -2967,9 +3106,9 @@ bool SizePntWdg::event( QEvent *ev )
 		return true;
 	    }
 	    break;
-	case QEvent::MouseButtonPress:
-	case QEvent::MouseButtonRelease:
-	    return QApplication::sendEvent(parent(),ev);
+	//case QEvent::MouseButtonPress:
+	//case QEvent::MouseButtonRelease:
+	//    return QApplication::sendEvent(parent(),ev);
     }
 
     return QWidget::event(ev);
