@@ -65,13 +65,13 @@ class ModInspAttr: public QAbstractTableModel
 		enum Type { WdgGrp, Wdg, AttrGrp, Attr };
 		enum Flag
 		{
-		    Select	= 0x01,
-		    FullText	= 0x08,
+		    Select	= 0x0001,
+		    FullText	= 0x0008,
 		    Active	= 0x0100,
 		    Image	= 0x0200,
 		    Color	= 0x0400,
 		    Font	= 0x0800,
-		    DataTime	= 0x0200
+		    DateTime	= 0x0200
 		};
 
 		//Public attributes
@@ -86,6 +86,7 @@ class ModInspAttr: public QAbstractTableModel
 		bool	modify( )	{ return mModify; }
 		QVariant data( );
 		QVariant dataEdit( );
+		string	wdgs( )		{ return wdgsItem; }
 
 		void setName( const string &nit )	{ nameItem = nit; }
 		void setEdited( bool ied )		{ edit_access = ied; }
@@ -93,6 +94,7 @@ class ModInspAttr: public QAbstractTableModel
 		void setData( const QVariant &idt )	{ dataItem = idt; }
 		void setDataEdit( const QVariant &idt )	{ dataEditItem = idt; }
 		void setModify( bool vl )		{ mModify = vl; }
+		bool setWdgs( const string &w, bool del = false );
 
 		void clean( );
 
@@ -105,7 +107,7 @@ class ModInspAttr: public QAbstractTableModel
 		Item *parent()		{ return parentItem; }
 
 	    private:
-		string	idItem, nameItem;
+		string	idItem, nameItem, wdgsItem;
 		Type	typeItem;
 		QVariant	dataItem, dataEditItem;
 		bool	edit_access;
@@ -122,6 +124,7 @@ class ModInspAttr: public QAbstractTableModel
 
 	string user( );
 
+	string curWdg( )	{ return cur_wdg; }
 	void setWdg( const string &iwdg );
 
 	Qt::ItemFlags flags( const QModelIndex &index ) const;
@@ -141,7 +144,7 @@ class ModInspAttr: public QAbstractTableModel
 
     private:
 	//Private methods
-	void wdgAttrUpdate( const QModelIndex &mod_it );// Item *it);
+	void wdgAttrUpdate( const QModelIndex &mod_it, const QModelIndex &grp_it = QModelIndex() );
 
 	//Private attributes
 	string cur_wdg;
@@ -190,6 +193,7 @@ class InspAttr: public QTreeView
 
 	//Private methods
 	bool event( QEvent *event );
+	void contextMenuEvent( QContextMenuEvent *event );
 
 	//Private attributes
 	ModInspAttr modelData;
@@ -421,6 +425,23 @@ class WScaleStBar : public QLabel
 	bool isScale;
 };
 
+//*********************************************
+//* Status bar modify indicator               *
+//*********************************************
+class WMdfStBar : public QLabel
+{
+    Q_OBJECT
+
+    public:
+	WMdfStBar( QWidget *parent = 0 );
+
+    signals:
+	void press( );
+
+    protected:
+	void mousePressEvent( QMouseEvent * event );
+};
+
 //*************************************************
 //* Size points widget                            *
 //*************************************************
@@ -459,6 +480,9 @@ class DevelWdgView: public WdgView
     Q_OBJECT
 
     public:
+	//Data
+	enum SelFlgs { PrcChilds = 0x01, OnlyFlag = 0x02, NoUpdate = 0x04 };
+
 	//Public methods
 	DevelWdgView( const string &iwid, int ilevel, VisDevelop *mainWind, QWidget* parent = 0 );
 	~DevelWdgView( );
@@ -473,7 +497,7 @@ class DevelWdgView: public WdgView
 	string selectChilds( int *cnt = NULL, vector<DevelWdgView*> *wdgs = NULL );	//Get selected include widgets list
 	bool edit( )		{ return fWdgEdit; }		//Edit mode state
 
-	void setSelect( bool vl, bool childs = true, bool onlyFlag = false );
+	void setSelect( bool vl, char flgs = 0 );// bool childs = true, bool onlyFlag = false, bool noUpdate = false );
 	void setEdit( bool vl );
 	void wdgsMoveResize( const QPointF &dP );
 	void setVisScale( float val );
