@@ -183,11 +183,11 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
     actAlrmLev->setStatusTip(_("Press for all alarms quitance."));
     //>>> Alarm by Light
     if(!ico_t.load(TUIS::icoPath("alarmLight").c_str())) ico_t.load(":/images/alarmLight.png");
-    actAlrmLight = new QAction( QPixmap::fromImage(ico_t), _("Light alarm"), this );
+    actAlrmLight = new QAction( QPixmap::fromImage(ico_t), _("Blink alarm"), this );
     actAlrmLight->setObjectName("alarmLight");
-    actAlrmLight->setToolTip(_("Light alarm"));
-    actAlrmLight->setWhatsThis(_("The button for all light alarms quitance"));
-    actAlrmLight->setStatusTip(_("Press for all light alarms quitance."));
+    actAlrmLight->setToolTip(_("Blink alarm"));
+    actAlrmLight->setWhatsThis(_("The button for all blink alarms quitance"));
+    actAlrmLight->setStatusTip(_("Press for all blink alarms quitance."));
     actAlrmLight->setVisible(false);
     //>>> Alarm by mono sound (PC speaker)
     if(!ico_t.load(TUIS::icoPath("alarmAlarm").c_str())) ico_t.load(":/images/alarmAlarm.png");
@@ -242,6 +242,7 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
     //> Init status bar
     mWTime = new QLabel( this );
     mWTime->setVisible(false);
+    mWTime->setAlignment(Qt::AlignCenter);
     mWTime->setWhatsThis(_("This label displays curent system's time."));
     statusBar()->insertPermanentWidget(0,mWTime);
     mWUser = new UserStBar( open_user.c_str(), user_pass.c_str(), VCAstat.c_str(), this );
@@ -253,7 +254,14 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
     mWStat = new QLabel( VCAStation().c_str(), this );
     mWStat->setWhatsThis(_("This label displays used VCA engine station."));
     mWStat->setToolTip(_("Field for display of the used VCA engine station."));
+    mWStat->setVisible( VCAStation() != "." );
     statusBar()->insertPermanentWidget(0,mWStat);
+    mStlBar = new StylesStBar( -1, this );
+    mStlBar->setWhatsThis(_("This label displays used interface style."));
+    mStlBar->setToolTip(_("Field for display the used interface style."));
+    mStlBar->setStatusTip(_("Double click for style change."));
+    connect( mStlBar, SIGNAL(styleChanged()), this, SLOT(styleChanged()) );
+    statusBar()->insertPermanentWidget(0,mStlBar);
     statusBar()->insertPermanentWidget(0,toolBarStatus);
 
     //> Init scroller
@@ -274,7 +282,7 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
 
     alrmPlay = new SndPlay(this);
 
-    menuBar()->setVisible(SYS->security().at().access(user(),SEQ_WR,"root","root",RWRWR_));
+    menuBar()->setVisible(SYS->security().at().access(user(),SEC_WR,"root","root",RWRWR_));
 
     resize( 600, 400 );
 
@@ -309,20 +317,15 @@ VisRun::~VisRun()
     pgCacheClear();
 }
 
-string VisRun::user()
-{
-    return mWUser->user().toAscii().data();
-}
+string VisRun::user( )		{ return mWUser->user().toAscii().data(); }
 
-string VisRun::password( )
-{
-    return mWUser->pass().toAscii().data();
-}
+string VisRun::password( )	{ return mWUser->pass().toAscii().data(); }
 
-string VisRun::VCAStation( )
-{
-    return mWUser->VCAStation().toAscii().data();
-}
+string VisRun::VCAStation( )	{ return mWUser->VCAStation().toAscii().data(); }
+
+int VisRun::style( )		{ return mStlBar->style(); }
+
+void VisRun::setStyle( int istl )	{ mStlBar->setStyle(istl); }
 
 int VisRun::cntrIfCmd( XMLNode &node, bool glob )
 {
@@ -503,8 +506,8 @@ void VisRun::printDiag( const string &idg )
 	for( int i_e = 0; i_e < sD->prms.size(); i_e++ )
 	{
 	    QPoint pnt((i_e/elLine)*(pagl.width()/2),im.height()+fntSize*(2+i_e%elLine));
-	    if( TSYS::strNoSpace(sD->prms[i_e].addr()).empty() ) continue;
-	    painter.fillRect(QRect(pnt.x()+2,pnt.y()+2,fntSize-5,fntSize-5),QBrush(QColor(sD->prms[i_e].color().c_str())));
+	    if( sD->prms[i_e].val().empty() || !sD->prms[i_e].color().isValid() ) continue;
+	    painter.fillRect(QRect(pnt.x()+2,pnt.y()+2,fntSize-5,fntSize-5),QBrush(sD->prms[i_e].color()));
 	    painter.drawRect(QRect(pnt.x()+2,pnt.y()+2,fntSize-5,fntSize-5));
 	    painter.drawText(QRect(pnt.x()+fntSize,pnt.y(),pagl.width()/2,fntSize),Qt::AlignLeft,
 		QString("%1 [%2...%3]").arg(sD->prms[i_e].addr().c_str()).arg(sD->prms[i_e].bordL()).arg(sD->prms[i_e].bordU()));
@@ -637,7 +640,7 @@ void VisRun::exportDiag( const string &idg )
 
     QPixmap img = QPixmap::grabWidget(rwdg);
 
-    //> Call save file dialog -
+    //> Call save file dialog
     QString fileName = QFileDialog::getSaveFileName(this,_("Save diagram's image"),
 	(TSYS::path2sepstr(rwdg->name())+".png").c_str(), _("Images (*.png *.xpm *.jpg)"));
     if( !fileName.isEmpty() && !img.save(fileName) )
@@ -708,7 +711,7 @@ void VisRun::about()
 {
     QMessageBox::about(this,windowTitle(),
 	QString(_("%1 v%2.\n%3\nAutor: %4\nDevelopers: %5\nLicense: %6\n\n%7 v%8.\n%9\nLicense: %10\nAuthor: %11\nWeb site: %12")).
-	arg(mod->modInfo("Name").c_str()).arg(mod->modInfo("Version").c_str()).arg(mod->modInfo("Descript").c_str()).
+	arg(mod->modInfo("Name").c_str()).arg(mod->modInfo("Version").c_str()).arg(mod->modInfo("Description").c_str()).
 	arg(mod->modInfo("Author").c_str()).arg(mod->modInfo(_("Developers")).c_str()).arg(mod->modInfo("License").c_str()).
 	arg(PACKAGE_NAME).arg(VERSION).arg(_(PACKAGE_DESCR)).arg(PACKAGE_LICENSE).arg(_(PACKAGE_AUTHOR)).arg(PACKAGE_SITE));
 }
@@ -731,7 +734,7 @@ void VisRun::userChanged( const QString &oldUser, const QString &oldPass )
     //> Update pages after user change
     pgCacheClear();
     bool oldMenuVis = menuBar()->isVisible();
-    menuBar()->setVisible(SYS->security().at().access(user(),SEQ_WR,"root","root",RWRWR_));
+    menuBar()->setVisible(SYS->security().at().access(user(),SEC_WR,"root","root",RWRWR_));
     QApplication::processEvents();
     if( master_pg )
     {
@@ -744,6 +747,19 @@ void VisRun::userChanged( const QString &oldUser, const QString &oldPass )
     }
 }
 
+void VisRun::styleChanged( )
+{
+    //> Get current style
+    XMLNode req("set");
+    req.setAttr("path","/ses_"+work_sess+"/%2fobj%2fcfg%2fstyle")->setText(TSYS::int2str(style()));
+    if( cntrIfCmd(req) )
+    {
+	mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
+	return;
+    }
+    fullUpdatePgs();
+}
+
 void VisRun::aboutQt()
 {
     QMessageBox::aboutQt( this, mod->modInfo("Name").c_str() );
@@ -781,6 +797,9 @@ void VisRun::alarmAct( QAction *alrm )
 	setAttr("tmpl",TSYS::uint2str(quitance))->
 	setAttr("wdg",qwdg);
     cntrIfCmd(req);
+
+    //> Send event to master page
+    if( master_pg ) master_pg->attrSet("event",("ws_"+alrm->objectName()).toAscii().data());
 }
 
 void VisRun::initSess( const string &prj_it, bool crSessForce )
@@ -892,6 +911,19 @@ void VisRun::initSess( const string &prj_it, bool crSessForce )
     req.clear()->setAttr("path","/ses_"+work_sess+"/%2fobj%2fcfg%2fper");
     if( !cntrIfCmd(req) ) mPeriod = atoi(req.text().c_str());
 
+    //> Get current style
+    req.clear()->setAttr("path","/ses_"+work_sess+"/%2fobj%2fcfg%2fstyle");
+    if( !cntrIfCmd(req) )
+    {
+	setStyle( atoi(req.text().c_str()) );
+	//> Check for styles present
+	if( style() < 0 )
+	{
+	    req.clear()->setAttr("path","/ses_"+work_sess+"/%2fobj%2fcfg%2fstLst");
+	    if( !cntrIfCmd(req) && req.childSize() <= 1 ) mStlBar->setVisible(false);
+	}
+    }
+
     //> Get project's flags
     req.clear()->setName("get")->setAttr("path","/prj_"+src_prj+"/%2fobj%2fcfg%2frunWin");
     if( !cntrIfCmd(req) )
@@ -930,7 +962,7 @@ void VisRun::initSess( const string &prj_it, bool crSessForce )
 
     QCoreApplication::processEvents();
 
-    //- Start timer -
+    //> Start timer
     updateTimer->start(period());
 }
 
@@ -1231,7 +1263,7 @@ void VisRun::updatePage( )
     //> Old pages from cache for close checking
     RunWdgView *pg = NULL;
     for( int i_pg = 0; i_pg < cache_pg.size(); i_pg++ )
-	if( (period()*(reqTm()-cache_pg[i_pg]->reqTm())/1000) > 60*60 )
+	if( mod->cachePgLife() > 0.01 && (period()*(reqTm()-cache_pg[i_pg]->reqTm())/1000) > (int)(mod->cachePgLife()*60*60) )
 	{
 	    delete cache_pg[i_pg];
 	    cache_pg.erase(cache_pg.begin()+i_pg);
@@ -1247,10 +1279,25 @@ void VisRun::updatePage( )
     }
 #endif
 
+    //> Time update
     if( mWTime->isVisible() && !(wPrcCnt%vmax(1000/period(),1)) )
     {
-	mWTime->setText(QTime::currentTime().toString("hh:mm:ss"));
-	mWTime->setToolTip(QDate::currentDate().toString("dddd, dd-MMM-yyyy"));
+	QDateTime dtm = QDateTime::currentDateTime();
+	mWTime->setText( locale().toString(dtm,"hh:mm:ss\nddd, d MMM") );
+	mWTime->setToolTip( locale().toString(dtm,"dddd, dd-MMM-yyyy") );
+    }
+
+    //> Scale for full screen check
+    if( (windowState() == Qt::WindowMaximized || windowState() == Qt::WindowFullScreen) && !(wPrcCnt%vmax(1000/period(),1)) )
+    {
+	float xSc = (float)((QScrollArea*)centralWidget())->maximumViewportSize().width()/(float)master_pg->size().width();
+	float ySc = (float)((QScrollArea*)centralWidget())->maximumViewportSize().height()/(float)master_pg->size().height();
+	if( xSc < 1 || ySc < 1 )
+	{
+	    x_scale *= xSc;
+	    y_scale *= ySc;
+	    fullUpdatePgs();
+	}
     }
 
     wPrcCnt++;
