@@ -24,6 +24,7 @@
 #include <QKeyEvent>
 #include <QApplication>
 #include <QPainter>
+#include <QComboBox>
 
 #include <tsys.h>
 
@@ -205,8 +206,8 @@ bool RunWdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
 	    if( (bool)atoi(val.c_str()) ) setFocus(Qt::OtherFocusReason);
 	    return true;
 	case -3:	//perm
-	    setPermCntr( atoi(val.c_str())&SEQ_WR );
-	    setPermView( atoi(val.c_str())&SEQ_RD );
+	    setPermCntr( atoi(val.c_str())&SEC_WR );
+	    setPermView( atoi(val.c_str())&SEC_RD );
 	    return true;
 	case 4:		//pgGrp
 	    setProperty("pgGrp",val.c_str());
@@ -255,6 +256,12 @@ bool RunWdgView::event( QEvent *event )
 		pnt.setPen(QColor("red"));
 		to.setAlignment(Qt::AlignCenter);
 		to.setWrapMode(QTextOption::WordWrap);
+		if( rect().width() > 500 && rect().height() > 100 )
+		{
+		    QFont cfnt = pnt.font();
+		    cfnt.setPointSize( 16 );
+		    pnt.setFont( cfnt );
+		}
 		pnt.drawText(rect(),QString(_("Page: '%1'.\nView access is not permited.")).arg(id().c_str()),to);
 	    }
 	    return true;
@@ -286,13 +293,13 @@ bool RunWdgView::event( QEvent *event )
 
     //> Key events process for send to model
     string mod_ev;
+    if( property("active").toBool() && permCntr() )
     switch( event->type() )
     {
 	case QEvent::Paint:	return true;
 	case QEvent::KeyPress:
 	    mod_ev = "key_pres";
 	case QEvent::KeyRelease:
-	    //printf("TEST 00: %s: %d\n",id().c_str(),event->type());
 	    if( ((QKeyEvent*)event)->key() == Qt::Key_Tab ) { mod_ev = ""; break; }
 	    if( mod_ev.empty() ) mod_ev = "key_rels";
 	    if( QApplication::keyboardModifiers()&Qt::ControlModifier )	mod_ev+="Ctrl";
@@ -427,6 +434,22 @@ bool RunWdgView::event( QEvent *event )
 	case QEvent::FocusIn:	attrSet("focus","1");	attrSet("event","ws_FocusIn");	return true;
 	case QEvent::FocusOut:	attrSet("focus","0");	attrSet("event","ws_FocusOut");	return true;
     }
+
+    //> Try put mouse event to next level widget into same container
+    if( !qobject_cast<RunPageView*>(this) && 
+	(event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease || event->type() == QEvent::MouseButtonDblClick) )
+    {
+	bool isOk = false;
+	QPoint curp = parentWidget()->mapFromGlobal(cursor().pos());
+	for( int i_c = parentWidget()->children().size()-1; i_c >= 0; i_c-- )
+	{
+	    RunWdgView *wdg = qobject_cast<RunWdgView*>(parentWidget()->children().at(i_c));
+	    if( !wdg ) continue;
+	    if( wdg == this ) isOk = true;
+	    else if( isOk && wdg->geometry().contains(curp) )	return QApplication::sendEvent(wdg,event);
+	}
+    }
+
     return QWidget::event(event);
 }
 
@@ -524,15 +547,24 @@ bool RunPageView::callPage( const string &pg_it, const string &pgGrp, const stri
 		((RunPageView *)children().at(i_ch))->callPage(pg_it,pgGrp,pgSrc))
 	    return true;
     //> Check for open child page or for unknown and empty source pages open as master page child windows
-    if( pgSrc == id().c_str() || this == mainWin()->master_pg )
+    if( (pgGrp.empty() && pgSrc == id()) || this == mainWin()->master_pg )
     {
 	RunPageView *pg = new RunPageView(pg_it,mainWin(),this);
-	pg->setAttribute(Qt::WA_DeleteOnClose);
-	pg->setWindowFlags(Qt::Sheet);
-	pg->setWindowTitle(mainWin()->windowTitle());
+	pg->setAttribute( Qt::WA_DeleteOnClose );
+	pg->setWindowFlags( Qt::Tool );
 	pg->load("");
 	pg->moveF(QPointF(mapToGlobal(pos()).x()+sizeF().width()/2-pg->sizeF().width()/2,
 			  mapToGlobal(pos()).y()+sizeF().height()/2-pg->sizeF().height()/2));
+	pg->setMinimumSize(pg->frameGeometry().size());
+	pg->setMaximumSize(pg->frameGeometry().size());
+	pg->setWindowState( pg->windowState() | Qt::WindowActive );
+
+	//>> Get page name
+	XMLNode req("get");
+	req.setAttr("path",pg->id()+"/%2fwdg%2fcfg%2fname");
+	if( !mainWin()->cntrIfCmd(req) ) pg->setWindowTitle(req.text().c_str());
+	else pg->setWindowTitle(mainWin()->windowTitle());
+
 	return true;
     }
 
@@ -564,10 +596,7 @@ SndPlay::SndPlay( QObject *parent ) : QThread(parent)
 
 }
 
-VisRun *SndPlay::mainWin( )
-{
-    return (VisRun *)parent();
-}
+VisRun *SndPlay::mainWin( )	{ return (VisRun *)parent(); }
 
 void SndPlay::run( )
 {
@@ -577,13 +606,13 @@ void SndPlay::run( )
     string com = mod->playCom( );
     string srcFile = "/var/tmp/oscadaPlayTmp_"+mainWin()->workSess( );
 
-    //- Put source file name to command -
+    //> Put source file name to command
     bool srcToPipe = false;
     if( (comPos=com.find("%f")) != string::npos )
 	com.replace( comPos, 2, srcFile.c_str() );
     else srcToPipe = true;
 
-    //- Write play data to file -
+    //> Write play data to file
     if( !srcToPipe )
     {
 	FILE *fp = fopen( srcFile.c_str(), "w" );
@@ -592,13 +621,77 @@ void SndPlay::run( )
 	fclose(fp);
     }
 
-    //- Call play command -
+    //> Call play command
     FILE *fp = popen( com.c_str(), "w" );
     if( !fp )		{ mPlayData.clear(); return; }
-    //- Write data to pipe -
+    //> Write data to pipe
     if( srcToPipe )	fwrite( mPlayData.data(), mPlayData.size(), 1, fp );
     pclose(fp);
     if( !srcToPipe )	remove( srcFile.c_str() );
 
     mPlayData.clear();
 };
+
+//*********************************************
+//* Status bar styles                         *
+//*********************************************
+StylesStBar::StylesStBar( int styleId, QWidget *parent ) : QLabel(parent), mStyle(-1)
+{
+    setStyle(styleId);
+}
+
+VisRun *StylesStBar::mainWin( )	{ return (VisRun *)window(); }
+
+void StylesStBar::setStyle( int istl, const string &nm )
+{
+    mStyle = istl;
+    if( mStyle < 0 ) setText( _("No style") );
+    else if( !nm.empty() ) setText( nm.c_str() );
+    else
+    {
+	XMLNode req("get");
+	req.setAttr("path","/ses_"+mainWin()->workSess()+"/%2fobj%2fcfg%2fstLst");
+	mainWin()->cntrIfCmd(req);
+	for( int i_s = 0; i_s < req.childSize(); i_s++ )
+	    if( atoi(req.childGet(i_s)->attr("id").c_str()) == istl )
+		setText(req.childGet(i_s)->text().c_str());
+    }
+}
+
+bool StylesStBar::styleSel( )
+{
+    //> Get syles list
+    XMLNode req("get");
+    req.setAttr("path","/ses_"+mainWin()->workSess()+"/%2fobj%2fcfg%2fstLst");
+    mainWin()->cntrIfCmd(req);
+
+    if( req.childSize() <= 1 ) return false;
+
+    InputDlg dlg( this, mainWin()->windowIcon(),_("Select your style from list."),_("Style select"),false,false);
+    QLabel *lab = new QLabel(_("Style:"),&dlg);
+    lab->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred) );
+    dlg.edLay()->addWidget( lab, 0, 0 );
+    QComboBox *stls = new QComboBox(&dlg);
+    dlg.edLay()->addWidget( stls, 0, 1 );
+    for( int i_s = 0; i_s < req.childSize(); i_s++ )
+    {
+	stls->addItem(req.childGet(i_s)->text().c_str(),atoi(req.childGet(i_s)->attr("id").c_str()));
+	if( atoi(req.childGet(i_s)->attr("id").c_str()) == style() )
+	    stls->setCurrentIndex(i_s);
+    }
+    dlg.resize(300,120);
+    if( dlg.exec() == QDialog::Accepted && stls->currentIndex() >= 0 )
+    {
+	setStyle( stls->itemData(stls->currentIndex()).toInt(), stls->itemText(stls->currentIndex()).toAscii().data() );
+	emit styleChanged( );
+	return true;
+    }
+
+    return false;
+}
+
+bool StylesStBar::event( QEvent *event )
+{
+    if( event->type() == QEvent::MouseButtonDblClick )	styleSel();
+    return QLabel::event( event );
+}
