@@ -1,8 +1,8 @@
 
 //OpenSCADA system module UI.VISION file: vis_shape_elfig.h
 /***************************************************************************
- *   Copyright (C) 2007-2008 by Lysenko Maxim (mlisenko@ukr.net)
- *   			     by Yashina Kseniya (sobacurka@ukr.net)
+ *   Copyright (C) 2007-2008 by Lysenko Maxim (mlisenko@oscada.org.ua)
+ *   			     by Yashina Kseniya (ksu@oscada.org.ua)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 
 #include <math.h>
 #include <map>
+
+#include <QPainterPath>
 #include <QPen>
 
 #include "vis_shapes.h"
@@ -43,26 +45,26 @@ namespace VISION
 //************************************************
 //* ShapeItem                                    *
 //************************************************
-class ShapeItem 
+class ShapeItem
 {
     public:
 	ShapeItem( )	{ }
-        ShapeItem( const QPainterPath &ipath, const QPainterPath &path_simple, short num_1, short num_2, short num_3, short num_4, short num_5,
+	ShapeItem( const QPainterPath &ipath, const QPainterPath &path_simple, short num_1, short num_2, short num_3, short num_4, short num_5,
 		    const QPointF &ctrlpos_4, const short &ilineColor, const short &iborderColor, const short &istyle , short iwidth, short bwidth, short itype, double iangle_temp ) : 
-    	    ctrlPos4(ctrlpos_4), n1(num_1), n2(num_2), n3(num_3), n4(num_4), n5(num_5), lineColor(ilineColor),
+	    ctrlPos4(ctrlpos_4), n1(num_1), n2(num_2), n3(num_3), n4(num_4), n5(num_5), lineColor(ilineColor),
 	    borderColor(iborderColor), style(istyle), width(iwidth), border_width(bwidth), type(itype), path(ipath), pathSimple(path_simple), angle_temp(iangle_temp)
-	{ };								
+	{ };
 
-	QPainterPath 	path, 
-		        pathSimple;
+	QPainterPath	path,
+			pathSimple;
 	QPointF		ctrlPos4;
-        short 		n1, n2, n3, n4, n5;
+        short		n1, n2, n3, n4, n5;
         short           lineColor, borderColor;
         short           style;
-	short 		width;
+	short		width;
         short           border_width;
-	short 		type :3;
-        double 		angle_temp;
+	short		type :3;
+        double		angle_temp;
 };
 
 //************************************************
@@ -118,15 +120,15 @@ class ElFigDt : public QObject
         short   geomMargin  :8;
         double  orient;
         string  elLst;
-        QVector<ShapeItem> shapeItems;
-        QVector<inundationItem> inundationItems;
-        PntMap  shapePnts;
-        WidthMap shapeWidths;
-        ColorMap shapeColors;
-        ImageMap shapeImages;
-        StyleMap shapeStyles;
+        QVector<ShapeItem> shapeItems, shapeItems_temp;
+        QVector<inundationItem> inundationItems, inundationItems_temp;
+        PntMap  shapePnts, shapePnts_temp;
+        WidthMap shapeWidths, shapeWidths_temp;
+        ColorMap shapeColors, shapeColors_temp; 
+        ImageMap shapeImages, shapeImages_temp;
+        StyleMap shapeStyles, shapeStyles_temp;
         WdgView *w;
-        QImage  pictObj;
+        QPixmap pictObj;
     private slots:
         void dynamic();
 };
@@ -161,7 +163,7 @@ class ShapeElFigure : public WdgShape
 	//Public attributes
 	QVector<RectItem> rectItems;		//RectItem's container
 	QPainterPath 	newPath,		//Null path
-			rectPath,		//Path fo RectItem creation
+			rectPath,		//Path for RectItem creation
         		ellipse_startPath, ellipse_endPath,
         		ellipse_draw_startPath, ellipse_draw_endPath,
                         inundationPath,
@@ -170,7 +172,6 @@ class ShapeElFigure : public WdgShape
     
 	bool status; 				//Check fo any primitive paint key pressed
 	int shapeType; 				//Selected figure type
-        int kr;    
     
     private slots:
         void toolAct( QAction* );
@@ -181,19 +182,19 @@ class ShapeElFigure : public WdgShape
         int itemAt( const QPointF &pos, const QVector<ShapeItem> &shapeItems, WdgView *w );			//Check for figure type under cursor
         void moveItemTo( const QPointF &pos, QVector<ShapeItem> &shapeItems, PntMap *pnts, WdgView *w );	//Move figure procedure
     
-	QPointF rotate( const QPointF &pnt, double alpha );
-	QPointF unRotate( const QPointF &pnt, double alpha, double a, double b );
-	QPointF arc( double t, double a, double b);	
-	double angle( const QLineF &l, const QLineF &l1 );
-	double length( const QPointF &pt1, const QPointF &pt2 );				
-        bool holds( const QVector<ShapeItem> &shapeItems, PntMap *pnts );
-        void moveUpDown( QVector<ShapeItem> &shapeItems, PntMap *pnts, QVector<inundationItem> &inundationItems, WdgView *w );
-        int  realRectNum( int rect_num_old, const QVector<ShapeItem> &shapeItems );
+	QPointF rotate( const QPointF &pnt, double alpha );                                                     //Rotate the point around the centre of the widget
+        QPointF unRotate( const QPointF &pnt, double alpha, double a, double b );                               //Unrotate the point around the centre of the widget
+	QPointF arc( double t, double a, double b);	                                                        //Compute the point of the arc due to the given parameter "t"
+        double angle( const QLineF &l, const QLineF &l1 );                                                      //Compute the angle between two lines
+        double length( const QPointF &pt1, const QPointF &pt2 );				                //Compute the lenght between two points
+        bool holds( const QVector<ShapeItem> &shapeItems, PntMap *pnts );                                       //Compute the number of connected figures with the given one
+        void moveUpDown( QVector<ShapeItem> &shapeItems, PntMap *pnts, QVector<inundationItem> &inundationItems, WdgView *w ); //Moving the figure(s) with the help of keyboard 
+        int  realRectNum( int rect_num_old, const QVector<ShapeItem> &shapeItems );                             //Compute the real rect number of the figure when several figures are selected
         void rectNum0_1( const QVector<ShapeItem> &shapeItems, int rect_num_temp, PntMap *pnts, WdgView *w );
         void rectNum3_4( const QVector<ShapeItem> &shapeItems);
         void moveAll( const QPointF &pos, QVector<ShapeItem> &shapeItems, PntMap *pnts, QVector<inundationItem> &inundationItems, WdgView *w );
         void checkPoint_checkInundation( QVector<ShapeItem> &shapeItems, PntMap *pnts, QVector<inundationItem> &inundationItems );
-        void paintImage( WdgView *view );
+        void paintImage( WdgView *view );                                                                       //Building the image to be drawn in the Paint event
         QPainterPath painterPath( float el_width, float el_border_width, int el_type, double el_ang,
 		QPointF el_p1 = QPointF(0,0), QPointF el_p2 = QPointF(0,0), QPointF el_p3 = QPointF(0,0), 
 		QPointF el_p4 = QPointF(0,0), QPointF el_p5 = QPointF(0,0), QPointF el_p6 = QPointF(0,0) );
@@ -250,15 +251,9 @@ class ShapeElFigure : public WdgShape
         QPointF Prev_pos_1, Prev_pos_2;
         QVector<int> num_vector;
         double angle_temp;
-        bool flag_angle_temp;
+        bool flag_angle_temp, flag_geom;
         int rect_dyn;
-        QImage rect_img;
-        PntMap  shapePnts_temp;
-        WidthMap shapeWidths_temp, shapeWidths_unScale;
-        ColorMap shapeColors_temp;
-        ImageMap shapeImages_temp;
-        StyleMap shapeStyles_temp;
-
+        QPixmap rect_img;
     };
 }
 
