@@ -40,6 +40,7 @@
 #include <QScrollArea>
 #include <QTextBrowser>
 
+#include <tmess.h>
 #include <xml.h>
 
 using std::string;
@@ -82,6 +83,10 @@ class WdgShape : public QObject
 	virtual bool event( WdgView *view, QEvent *event );
 	virtual bool eventFilter( WdgView *view, QObject *object, QEvent *event )	{ }
 
+	//Static methods
+	static QFont getFont( const string &vl, float fsc = 1, bool pixSize = true );
+	static QColor getColor( const string &vl );
+
     protected:
 	//Methods
 	void borderDraw( QPainter &pnt, QRect dA, QPen bpen, int bordStyle );
@@ -112,11 +117,11 @@ class ShapeFormEl : public WdgShape
 
     public slots:
 	//Public slots
-	//-- Edit line events --
+	//>> Edit line events
 	void lineAccept( );
-	//-- Edit text events --
+	//>> Edit text events
 	void textAccept( );
-	//-- Check box events --
+	//>> Check box events
 	void checkChange(int);
 	//-- Combo box and list events --
 	void comboChange(const QString&);
@@ -298,7 +303,7 @@ class ShapeDiagram : public WdgShape
 		string	addr( )		{ return m_addr; }
 		double	bordL( )	{ return m_bord_low; }
 		double	bordU( )	{ return m_bord_up; }
-		string	color( )	{ return m_color; }
+		QColor	color( )	{ return m_color; }
 		double	curVal( )	{ return m_curvl; }
 		int	valTp( )	{ return val_tp; }
 		long long valBeg( );
@@ -309,7 +314,7 @@ class ShapeDiagram : public WdgShape
 		void setAddr( const string &vl );
 		void setBordL( double vl )	{ m_bord_low = vl; }
 		void setBordU( double vl )	{ m_bord_up  = vl; }
-		void setColor( const string &vl ){ m_color = vl; }
+		void setColor( const QColor &vl ){ m_color = vl; }
 		void setCurVal( double vl )	{ m_curvl = vl; }
 
 		void loadData( bool full = false );
@@ -326,7 +331,7 @@ class ShapeDiagram : public WdgShape
 		string		m_addr;		//A parameter or an archive item address
 		double m_bord_low, m_bord_up;	//Borders
 		double		m_curvl;	//Curent value
-		string		m_color;	//Values line color
+		QColor		m_color;	//Values line color
 		//> Archive
 		int		arh_per;	//Archive period
 		long long	arh_beg;	//Archive begin time
@@ -342,8 +347,8 @@ class ShapeDiagram : public WdgShape
 	{
 	    public:
 		//Methods
-		ShpDt( ) : en(true), active(true), geomMargin(0), tTime(0), tPict(0), curTime(0),
-		    tTimeCurent(false), trcPer(0), holdCur(true)	{ }
+		ShpDt( ) : en(true), active(true), geomMargin(0), tTime(0), tPict(0), curTime(0), 
+		    sclVerScl(100), sclVerSclOff(0), tTimeCurent(false), trcPer(0), holdCur(true)	{ }
 		//Attributes
 		short	en		:1;
 		short	active		:1;
@@ -358,7 +363,7 @@ class ShapeDiagram : public WdgShape
 		QColor	curColor, sclColor, sclMarkColor;
 		QBrush	backGrnd;
 		QPen		border;
-		QPicture	pictObj;
+		QImage		pictObj;
 		QRect		pictRect;
 		QFont		sclMarkFont;
 		QTimer 		*trcTimer;
@@ -366,6 +371,7 @@ class ShapeDiagram : public WdgShape
 		long long	tTime, tPict, curTime;
 		float		tSize;
 		float		fftBeg, fftEnd;
+		float		sclVerScl, sclVerSclOff;
 		string		valArch;
 	};
 
@@ -414,10 +420,21 @@ class ShapeProtocol : public WdgShape
 
     private:
 	//Data
-	//- Shape node date -
+	//> Shape node date
 	class ShpDt
 	{
 	    public:
+		//Data
+		class ItProp
+		{
+		    public:
+			ItProp( ) : lev(0)	{ }
+
+			int lev;
+			string tmpl;
+			QColor clr;
+			QFont font;
+		};
 		//Methods
 		ShpDt( ) : active(true), time(0), tSize(60), timeCurent(false), arhBeg(0), arhEnd(0)	{ }
 		//Attributes
@@ -425,18 +442,18 @@ class ShapeProtocol : public WdgShape
 		short	timeCurent	:1;
 		short	trcPer		:10;
 		short	lev		:4;
-		short	viewOrd		:3;
-		short	itProp		:3;
+		short	viewOrd		:4;
 		QTimer 		*trcTimer;
 		QTableWidget	*addrWdg;
-		QMap<QString,int> indMap;
 		unsigned int	time, tSize;
 		unsigned int	arhBeg, arhEnd;
 		string		arch, tmpl, col;
+		vector<ItProp>	itProps;
+		deque<TMess::SRec>	messList;
 	};
 	//Private methods
 	void loadData( WdgView *view, bool full = false );
-	//- Recursively widgets process for disable focusable and events filter set -
+	//> Recursively widgets process for disable focusable and events filter set
 	void eventFilterSet( WdgView *view, QWidget *wdg, bool en );
 	void setFocus(WdgView *view, QWidget *wdg, bool en = false, bool devel = false );
 };
