@@ -36,6 +36,11 @@
 #include <QFontComboBox>
 #include <QGroupBox>
 #include <QCheckBox>
+#include <QTimer>
+#include <QCalendarWidget>
+#include <QApplication>
+#include <QStatusBar>
+#include <QToolTip>
 
 #include <tsys.h>
 
@@ -49,9 +54,8 @@ using namespace VISION;
 //*************************************************
 //* Id and name input dialog                      *
 //*************************************************
-InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
-	const QString &ndlg, bool with_id, bool with_nm ) :
-		QDialog(parent), m_id(NULL), m_name(NULL)
+InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess, const QString &ndlg, bool with_id, bool with_nm ) :
+	QDialog(parent), m_id(NULL), m_name(NULL)
 {
     setWindowTitle(ndlg);
     setMinimumSize( QSize( 120, 150 ) );
@@ -158,7 +162,7 @@ void InputDlg::setIdLen( int len )
 //*************************************************
 //* User select dialog                            *
 //*************************************************
-DlgUser::DlgUser( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent ) : 
+DlgUser::DlgUser( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent ) :
     QDialog(parent), VCAstat(iVCAstat)
 {
     setWindowTitle(_("Select user"));
@@ -348,7 +352,8 @@ void FontDlg::cfgChange()
 //*********************************************
 //* Status bar user widget                    *
 //*********************************************
-UserStBar::UserStBar( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent ) : QLabel(parent)
+UserStBar::UserStBar( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent ) : 
+    QLabel(parent)
 {
     setUser(iuser);
     setPass(ipass);
@@ -390,44 +395,52 @@ bool UserStBar::userSel()
 //* Universal edit line widget. Contain support of: QLineEdit, QSpinBox, QDoubleSpinBox,      *
 //* QTimeEdit, QDateEdit and QDateTimeEdit.                                                   *
 //*********************************************************************************************
-LineEdit::LineEdit( QWidget *parent, LType tp, bool prev_dis ) :
-    QWidget( parent ), m_tp((LineEdit::LType)-1), bt_fld(NULL), ed_fld(NULL)
+LineEdit::LineEdit( QWidget *parent, LType tp, bool prev_dis, bool resApply ) :
+    QWidget( parent ), m_tp((LineEdit::LType)-1), bt_fld(NULL), bt_tm(NULL), ed_fld(NULL),
+    mPrev(!prev_dis), applyReserve(resApply)
 {
     QHBoxLayout *box = new QHBoxLayout(this);
     box->setMargin(0);
     box->setSpacing(0);
 
-    if( !prev_dis )
-    {
-	bt_fld = new QPushButton(this);
-	bt_fld->setIcon(QIcon(":/images/ok.png"));
-	bt_fld->setIconSize(QSize(12,12));
-	bt_fld->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed) );
-	bt_fld->setMaximumWidth(15);
-	//bt_fld->setMinimumSize(15,15);
-	//bt_fld->setFlat( true );
-	bt_fld->setEnabled( false );
-	bt_fld->setVisible( false );
-	connect( bt_fld, SIGNAL( pressed() ), this, SLOT( applySlot() ) );
-	box->addWidget(bt_fld);
-    }
     setType(tp);
 }
 
-bool LineEdit::isEdited( )
+void LineEdit::viewApplyBt( bool view )
 {
-    if( bt_fld && bt_fld->isVisible() )	return true;
-    return false;
+    if( view == (bool)bt_fld ) return;
+
+    if( view && !bt_fld )
+    {
+	bt_fld = new QPushButton(this);
+	bt_fld->setIcon( QIcon(":/images/ok.png") );
+	bt_fld->setIconSize( QSize(12,12) );
+	bt_fld->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed) );
+	bt_fld->setMaximumWidth( 15 );
+	connect( bt_fld, SIGNAL( clicked() ), this, SLOT( applySlot() ) );
+	layout()->addWidget( bt_fld );
+
+	bt_tm = new QTimer(this);
+	connect( bt_tm, SIGNAL( timeout() ), this, SLOT( cancelSlot() ) );
+    }
+    if( !view && bt_fld )
+    {
+	bt_tm->stop(); bt_tm->deleteLater(); bt_tm = NULL;
+	bt_fld->deleteLater(); bt_fld = NULL;
+    }
 }
 
+bool LineEdit::isEdited( )	{ return bt_fld; }
+
 void LineEdit::setType( LType tp )
 {
+    needReserver = false;
     if( tp == m_tp ) return;
 
-    //- Delete previous -
+    //> Delete previous
     if( tp >= 0 && ed_fld ) delete ed_fld;
 
-    //- Create new widget -
+    //> Create new widget
     switch( tp )
     {
 	case Text:
@@ -437,30 +450,47 @@ void LineEdit::setType( LType tp )
 	case Integer:
 	    ed_fld = new QSpinBox(this);
 	    connect( (QSpinBox*)ed_fld, SIGNAL( valueChanged(int) ), SLOT( changed() ) );
+	    if( mPrev ) needReserver = true;
 	    break;
 	case Real:
 	    ed_fld = new QDoubleSpinBox(this);
 	    connect( (QDoubleSpinBox*)ed_fld, SIGNAL( valueChanged(double) ), SLOT( changed() ) );
+	    if( mPrev ) needReserver = true;
 	    break;
 	case Time:
 	    ed_fld = new QTimeEdit(this);
 	    connect( (QTimeEdit*)ed_fld, SIGNAL( timeChanged(const QTime&) ), SLOT( changed() ) );
+	    if( mPrev ) needReserver = true;
 	    break;
 	case Date:
 	    ed_fld = new QDateEdit(this);
+	    ((QDateEdit*)ed_fld)->setCalendarPopup(true);
+	    ((QDateEdit*)ed_fld)->calendarWidget()->setGridVisible(true);
+	    ((QDateEdit*)ed_fld)->calendarWidget()->setFirstDayOfWeek(Qt::Monday);
 	    connect( (QDateEdit*)ed_fld, SIGNAL( dateChanged(const QDate&) ), SLOT( changed() ) );
 	    break;
 	case DateTime:
+	{
 	    ed_fld = new QDateTimeEdit(this);
+	    ((QDateEdit*)ed_fld)->setCalendarPopup(true);
+	    ((QDateEdit*)ed_fld)->calendarWidget()->setGridVisible(true);
+	    ((QDateEdit*)ed_fld)->calendarWidget()->setFirstDayOfWeek(Qt::Monday);
 	    connect( (QDateTimeEdit*)ed_fld, SIGNAL( dateTimeChanged(const QDateTime&) ), SLOT( changed() ) );
 	    break;
+	}
 	case Combo:
 	    ed_fld = new QComboBox(this);
 	    ((QComboBox*)ed_fld)->setEditable(true);
 	    connect( (QComboBox*)ed_fld, SIGNAL( editTextChanged(const QString&) ), SLOT( changed() ) );
+	    connect( (QComboBox*)ed_fld, SIGNAL( activated(int) ), this, SLOT( applySlot() ) );
 	    break;
     }
     ((QBoxLayout*)layout())->insertWidget(0,ed_fld);
+    if( applyReserve && needReserver )
+    {
+	ed_fld->setMaximumWidth(width()-12); ed_fld->setMinimumWidth(width()-12);
+	((QBoxLayout*)layout())->setAlignment(ed_fld,Qt::AlignLeft);
+    }
     setFocusProxy( ed_fld );
 
     m_tp = tp;
@@ -468,13 +498,9 @@ void LineEdit::setType( LType tp )
 
 void LineEdit::changed( )
 {
-    //- Enable apply
-    if( bt_fld && !bt_fld->isEnabled() )
-    {
-	bt_fld->setEnabled(true);
-	bt_fld->setVisible(true);
-	//QWidget::setTabOrder( mod->getFocusedWdg(ed_fld), mod->getFocusedWdg(bt_fld) );
-    }
+    //> Enable apply
+    if( mPrev && !bt_fld )	viewApplyBt(true);
+    if( bt_tm ) bt_tm->start(5000);
 
     emit valChanged(value());
 }
@@ -498,6 +524,7 @@ void LineEdit::setValue( const QString &txt )
 	    ((QTimeEdit*)ed_fld)->setTime(QTime().addSecs(txt.toInt()));
 	    break;
 	case Date: case DateTime:
+	    if( ((QDateTimeEdit*)ed_fld)->calendarWidget()->isVisible() ) break;
 	    ((QDateTimeEdit*)ed_fld)->setDateTime(QDateTime::fromTime_t(txt.toInt()));
 	    break;
 	case Combo:
@@ -509,11 +536,7 @@ void LineEdit::setValue( const QString &txt )
 
     m_val = txt;
 
-    if( bt_fld && bt_fld->isEnabled() )
-    {
-        bt_fld->setEnabled(false);
-        bt_fld->setVisible(false);
-    }
+    if( bt_fld ) viewApplyBt(false);
 }
 
 void LineEdit::setCfg(const QString &cfg)
@@ -576,11 +599,7 @@ void LineEdit::setCfg(const QString &cfg)
 	    break;
 	}
     }
-    if( bt_fld && bt_fld->isEnabled() )
-    {
-	bt_fld->setEnabled(false);
-	bt_fld->setVisible(false);
-    }
+    if( bt_fld ) viewApplyBt(false);
 }
 
 QString LineEdit::value()
@@ -600,30 +619,36 @@ QString LineEdit::value()
 
 void LineEdit::applySlot( )
 {
-    bt_fld->setEnabled(false);
-    bt_fld->setVisible(false);
+    viewApplyBt(false);
 
+    if( m_val == value() ) return;
     m_val = value();
 
     emit apply();
 }
 
+void LineEdit::cancelSlot( )
+{
+    setValue(m_val);
+    emit cancel();
+}
+
 bool LineEdit::event( QEvent * e )
 {
-    if(e->type() == QEvent::KeyRelease && bt_fld && bt_fld->isEnabled())
+    if( e->type() == QEvent::KeyRelease && bt_fld )
     {
-        QKeyEvent *keyEvent = (QKeyEvent *)e;
-        if(keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return)
-        {
+	QKeyEvent *keyEvent = (QKeyEvent *)e;
+	if( keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return )
+	{
 	    bt_fld->animateClick( );
 	    return true;
-        }
-        else if(keyEvent->key() == Qt::Key_Escape )
-        {
-	    emit cancel();
-	    setValue(m_val);
-	    return true;
-        }
+	}
+	else if(keyEvent->key() == Qt::Key_Escape )	{ cancelSlot(); return true; }
+    }
+    else if( e->type() == QEvent::Resize && applyReserve && needReserver )
+    {
+	ed_fld->setMaximumWidth(width()-12);
+	ed_fld->setMinimumWidth(width()-12);
     }
     return QWidget::event(e);
 }
@@ -639,9 +664,12 @@ TextEdit::TextEdit( QWidget *parent, bool prev_dis ) :
     box->setSpacing(0);
 
     ed_fld = new QTextEdit(this);
+    ed_fld->setTabStopWidth(20);
+    ed_fld->setAcceptRichText(false);
     ed_fld->setLineWrapMode(QTextEdit::NoWrap);
     setFocusProxy( ed_fld );
     connect( ed_fld, SIGNAL( textChanged() ), this, SLOT( changed() ) );
+    connect( ed_fld, SIGNAL( cursorPositionChanged() ), this, SLOT( curPosChange() ) );
     box->addWidget(ed_fld);
 
     if( !prev_dis )
@@ -714,6 +742,39 @@ void TextEdit::cancelSlot( )
     emit cancel();
 }
 
+void TextEdit::curPosChange( )
+{
+    QMainWindow *w = dynamic_cast<QMainWindow *>(window());
+    if( !w || !w->statusBar() ) w = dynamic_cast<QMainWindow *>(window()->parentWidget());
+    if( !w || !w->statusBar() ) return;
+    w->statusBar()->showMessage(QString(_("Cursor = (%1:%2)")).arg(ed_fld->textCursor().blockNumber()+1).arg(ed_fld->textCursor().columnNumber()+1),10000);
+}
+
+bool TextEdit::event( QEvent * e )
+{
+    if( but_box && e->type() == QEvent::KeyRelease )
+    {
+	QKeyEvent *keyEvent = (QKeyEvent *)e;
+	if( (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return) && QApplication::keyboardModifiers()&Qt::ControlModifier )
+	{
+	    but_box->button(QDialogButtonBox::Apply)->animateClick( );
+	    return true;
+	}
+	else if( keyEvent->key() == Qt::Key_Escape )
+	{
+	    but_box->button(QDialogButtonBox::Cancel)->animateClick( );
+	    return true;
+	}
+    }
+    if( e->type() == QEvent::ToolTip && hasFocus() && toolTip().isEmpty() )
+    {
+	QToolTip::showText( ((QHelpEvent *)e)->globalPos(), QString(_("Cursor = (%1:%2)")).arg(ed_fld->textCursor().blockNumber()+1).arg(ed_fld->textCursor().columnNumber()+1) );
+	return true;
+    }
+
+    return QWidget::event(e);
+}
+
 //****************************************
 //* Shape widget view                    *
 //****************************************
