@@ -170,7 +170,7 @@ namespace VISION
 	    enum LType { Text, Integer, Real, Time, Date, DateTime, Combo };
 
 	    //Methods
-	    LineEdit( QWidget *parent, LType tp = Text, bool prev_dis = false );
+	    LineEdit( QWidget *parent, LType tp = Text, bool prev_dis = false, bool resApply = true );
 
 	    LType type( )	{ return m_tp; }
 	    QString value( );
@@ -181,6 +181,7 @@ namespace VISION
 	    void setCfg( const QString& );
 
 	    QWidget *workWdg( )	{ return ed_fld; }
+	    void viewApplyBt( bool view );
 
 	signals:
 	    void apply( );
@@ -193,12 +194,15 @@ namespace VISION
 	private slots:
 	    void changed( );
 	    void applySlot( );
+	    void cancelSlot( );
 
 	private:
 	    LType	m_tp;
 	    QString	m_val;
+	    bool	mPrev, applyReserve, needReserver;
 	    QWidget	*ed_fld;
 	    QPushButton	*bt_fld;
+	    QTimer	*bt_tm;
     };
 
     //*********************************************
@@ -221,10 +225,14 @@ namespace VISION
 	    void cancel( );
 	    void textChanged( const QString& );
 
+	protected:
+	    bool event( QEvent * e );
+
 	private slots:
 	    void changed( );
 	    void applySlot( );
 	    void cancelSlot( );
+	    void curPosChange( );
 
 	private:
 	    bool		isInit;
