@@ -123,40 +123,11 @@ void TWEB::modInfo( vector<string> &list )
     list.push_back("Auth");
 }
 
-string TWEB::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TWEB::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
-
-    //- Load parameters from config file -
+    //> Load parameters from config file
 }
 
 void TWEB::modStart()
@@ -175,7 +146,7 @@ string TWEB::httpHead( const string &rcode, int cln, const string &cnt_tp, const
 	    "Server: "+PACKAGE_STRING+"\r\n"
 	    "Accept-Ranges: bytes\r\n"
 	    "Content-Length: "+TSYS::int2str(cln)+"\r\n"
-	    "Content-type: "+cnt_tp+"\r\n"
+	    "Content-Type: "+cnt_tp+"\r\n"
 	    "Charset="+Mess->charset()+"\r\n"+addattr+"\r\n";
 }
 
@@ -450,7 +421,7 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
 {
     string	path = string("/")+MOD_ID+ses.url;
 
-    bool wr = atoi(node.attr("acs").c_str())&SEQ_WR;
+    bool wr = atoi(node.attr("acs").c_str())&SEC_WR;
     if( node.name() == "fld" )
     {
 	XMLNode dt_req("get");
@@ -649,7 +620,7 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
     {
 	if( wr ) ses.page = ses.page+"<form action='"+path+"' method='post' enctype='multipart/form-data'>\n";
 
-	XMLNode req("get"); 
+	XMLNode req("get");
 	req.setAttr("path",ses.url+"/"+TSYS::strEncode( a_path, TSYS::PathEl ))->
 	    setAttr("user",ses.user);
 	if(cntrIfCmd(req)) ses.mess.push_back(req.text().c_str());
@@ -663,7 +634,7 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
 	{
 	    XMLNode *t_lsel = req.childGet(i_cl);
 	    XMLNode *t_linf = node.childGet("id",t_lsel->attr("id"),true);
-	    if(!t_linf) continue;
+	    if( !t_linf ) { t_linf = node.childIns(i_cl); *t_linf = *t_lsel; }
 
 	    ses.page = ses.page + "<td>"+TSYS::strEncode(t_linf->attr("dscr"),TSYS::Html)+"</td>";
 	    //-- Calc column sizes --
@@ -679,7 +650,7 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
 	if( wr ) ses.page = ses.page + "<td bgcolor='Green'>*</td>";
 	ses.page = ses.page + "</tr>\n";
 
-	for( int i_rw=0; i_rw < req.childGet(0)->childSize(); i_rw++)
+	for( int i_rw=0; req.childSize() && i_rw < req.childGet(0)->childSize(); i_rw++)
 	{
 	    ses.page = ses.page + "<tr bgcolor='#cccccc'>";
 	    for( int i_cl=0; i_cl < req.childSize(); i_cl++)
@@ -687,7 +658,7 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
 		XMLNode *t_lsel = req.childGet(i_cl);
 		XMLNode *t_linf = node.childGet("id",t_lsel->attr("id"),true);
 		if(!t_linf) continue;
-		bool c_wr = wr && (atoi(t_linf->attr("acs").c_str())&SEQ_WR);
+		bool c_wr = wr && (atoi(t_linf->attr("acs").c_str())&SEC_WR);
 
 		XMLNode *x_el = t_lsel->childGet(i_rw);
 		if( t_linf->attr("tp") == "time" )
@@ -874,7 +845,7 @@ int  TWEB::postVal( SSess &ses, XMLNode &node, string prs_path)
     for( unsigned i_cf = 0; i_cf < node.childSize(); i_cf++)
     {
 	XMLNode *t_c = node.childGet(i_cf);
-	if( (t_c->name() == "fld") && (atoi(t_c->attr("acs").c_str())&SEQ_WR) &&
+	if( (t_c->name() == "fld") && (atoi(t_c->attr("acs").c_str())&SEC_WR) &&
 		t_c->attr("tp") != "bool" && !valPrepare( ses, *t_c,prs_path, true ) )
 	    ses.cnt.erase(t_c->attr("id"));
     }
@@ -883,7 +854,7 @@ int  TWEB::postVal( SSess &ses, XMLNode &node, string prs_path)
     for( unsigned i_cf = 0; i_cf < node.childSize(); i_cf++)
     {
 	XMLNode *t_c = node.childGet(i_cf);
-	if( t_c->name() == "fld" && (atoi(t_c->attr("acs").c_str())&SEQ_WR) )
+	if( t_c->name() == "fld" && (atoi(t_c->attr("acs").c_str())&SEC_WR) )
 	{
 	    if( !valPrepare( ses, *t_c,prs_path, true ) )
 		continue;
@@ -1085,7 +1056,7 @@ int TWEB::postTable( SSess &ses, XMLNode &node, string prs_path )
     string	f_path, l_path, l_com;
     int		op_cnt = 0;
 
-    bool wr = atoi(node.attr("acs").c_str())&SEQ_WR;
+    bool wr = atoi(node.attr("acs").c_str())&SEC_WR;
     if( !wr )	return 0x01;
 
     //- Get command name and path -
@@ -1115,7 +1086,7 @@ int TWEB::postTable( SSess &ses, XMLNode &node, string prs_path )
 	    XMLNode *t_lsel = dt_tbl.childGet(i_cl);
 	    XMLNode *t_linf = node.childGet("id",t_lsel->attr("id"),true);
 	    if(!t_linf) continue;
-	    if(wr && (atoi(t_linf->attr("acs").c_str())&SEQ_WR) )
+	    if(wr && (atoi(t_linf->attr("acs").c_str())&SEC_WR) )
 		for( int i_rw=0; i_rw < t_lsel->childSize(); i_rw++)
 		{
 		    XMLNode *w_el = t_lsel->childGet(i_rw);
