@@ -1,8 +1,8 @@
 
 //OpenSCADA system module UI.WebCfgD file: VCA.js
 /***************************************************************************
- *   Copyright (C) 2008 by Roman Savochenko                                *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2008-2010 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -31,9 +31,9 @@ ndNext = new Array();	//Next nodes array
 pgInfo = document.createElement('info');	//Curent page XML tree.
 root = document.createElement('oscada_cntr');	//Root page's node.
 selPath = '';		//Select item path
-SEQ_XT = 0x01;		//Extended
-SEQ_WR = 0x02;		//Write access
-SEQ_RD = 0x04;		//Read access
+SEC_XT = 0x01;		//Extended
+SEC_WR = 0x02;		//Write access
+SEC_RD = 0x04;		//Read access
 copyBuf = '0';		//Copy node address buffer
 //> Browser type detect
 var isNN = navigator.appName.indexOf('Netscape') != -1;
@@ -45,7 +45,7 @@ var isKonq = navigator.appName.indexOf('Konqueror') != -1;
  ***************************************************/
 function strEncode( vl, tp )
 {
-  if( !tp || tp == "html" ) return vl.replace('&','&amp;').replace('>','&gt;').replace('<','&lt;').replace('"','&quot;');
+  if( !tp || tp == "html" ) return vl.replace(/&/g,'&amp;').replace(/>/g,'&gt;').replace(/</g,'&lt;').replace(/"/g,'&quot;');
   return vl;
 }
 /***************************************************
@@ -237,7 +237,7 @@ function expand( el, val, upTree )
 	var treeIco = '/'+MOD_ID+'/img_tree'+(isUsable?(liN.isExpand?'Minus':'Plus'):'')+'Up'+((i_g!=(el.grps.length-1))?'Down':'');
 	var liCont = isUsable?"<a class='pm' onclick='expand(this.parentNode,!this.parentNode.isExpand); return false;'>":"";
 	liCont += "<img src='"+treeIco+"'/></a>";
-	liCont += "<span style='font-style: italic;'>"+el.grps[i_g].getAttribute('dscr')+":</span>";
+	liCont += "<span style='font-style: italic;'>"+strEncode(el.grps[i_g].getAttribute('dscr'))+":</span>";
 	//>> Next node for update
 	if( upTree && liN.isExpand )
 	{
@@ -297,7 +297,7 @@ function expand( el, val, upTree )
 	  if( parseInt(hostN.childNodes[i_e].getAttribute('icoSize')) )
 	    liCont += "<span><img height='16px' src='/"+MOD_ID+liN.getAttribute('id')+"?com=ico'/></span>";
 	  liCont += "<span><a onclick='selectPage(this.parentNode.parentNode.getAttribute(\"id\")); return false;' "+
-	    "onmouseover='setStatus(this.parentNode.parentNode.getAttribute(\"id\"),10000);' href='#'>"+nodeText(hostN.childNodes[i_e])+"</a></span>";
+	    "onmouseover='setStatus(this.parentNode.parentNode.getAttribute(\"id\"),10000);' href='#'>"+strEncode(nodeText(hostN.childNodes[i_e]))+"</a></span>";
 	  //>> Next node for update
 	  if( upTree && liN.isExpand )
 	  {
@@ -395,11 +395,11 @@ function pageDisplay( path )
     if( root.childNodes[i_ch].getAttribute('id') == 'br' )
     {
       for( var i_g = 0; i_g < root.childNodes[i_ch].childNodes.length; i_g++ )
-        if( parseInt(root.childNodes[i_ch].childNodes[i_g].getAttribute('acs'))&SEQ_WR )
+        if( parseInt(root.childNodes[i_ch].childNodes[i_g].getAttribute('acs'))&SEC_WR )
         { actEnable('actAddIt',true); break; }
       break;
     }
-  actEnable('actDelIt',parseInt(root.getAttribute('acs'))&SEQ_WR)
+  actEnable('actDelIt',parseInt(root.getAttribute('acs'))&SEC_WR)
 
   //> Load and Save allow check
   var reqModif = servGet(selPath,'com=modify');
@@ -414,7 +414,7 @@ function pageDisplay( path )
  ***************************************************/
 function editToolUpdate( )
 {
-  actEnable('actCut',(selPath.length&&parseInt(root.getAttribute('acs'))&SEQ_WR));
+  actEnable('actCut',(selPath.length&&parseInt(root.getAttribute('acs'))&SEC_WR));
   actEnable('actCopy',selPath.length);
   actEnable('actPaste',false);
 
@@ -429,11 +429,11 @@ function editToolUpdate( )
     if( root.childNodes[i_ch].getAttribute('id') == 'br' )
     {
       for( var i_g = 0; i_g < root.childNodes[i_ch].childNodes.length; i_g++ )
-        if( parseInt(root.childNodes[i_ch].childNodes[i_g].getAttribute('acs'))&SEQ_WR )
+        if( parseInt(root.childNodes[i_ch].childNodes[i_g].getAttribute('acs'))&SEC_WR )
         { actEnable('actPaste',true); break; }
       break;
     }
-  if( parseInt(root.getAttribute('acs'))&SEQ_WR ) actEnable('actPaste',true);
+  if( parseInt(root.getAttribute('acs'))&SEC_WR ) actEnable('actPaste',true);
 }
 /***************************************************
  * selectChildRecArea - Make page content          *
@@ -445,7 +445,12 @@ function selectChildRecArea( node, aPath, cBlk )
   if( aPath == "/" )
   {
     //>> Set node icon
-    document.getElementById('pgIco').src = (nodeTextByTagId(node,'img','ico') != null) ? ('/'+MOD_ID+selPath+'?com=ico') : '';
+    if( nodeTextByTagId(node,'img','ico') != null )
+    {
+	document.getElementById('pgIco').src = '/'+MOD_ID+selPath+'?com=ico';
+	document.getElementById('pgIco').style.visibility = 'visible';
+    }
+    else document.getElementById('pgIco').style.visibility = 'hidden';
     //>> Set title
     setNodeText(document.getElementById('pgTitle'),node.getAttribute('dscr'));
     //>> Delete tabs of deleted areas
@@ -506,7 +511,7 @@ function selectChildRecArea( node, aPath, cBlk )
     var t_s = node.childNodes[i_cf];
 
     //>> Check access to node
-    var wr = parseInt(t_s.getAttribute('acs'))&SEQ_WR;
+    var wr = parseInt(t_s.getAttribute('acs'))&SEC_WR;
 
     //>> View areas
     if( t_s.nodeName.toLowerCase() == 'area' )
@@ -546,7 +551,7 @@ function selectChildRecArea( node, aPath, cBlk )
 	  var optEl = '';
 	  if( this.srcNode.getAttribute('tp') == 'br' && this.selectedIndex >= 0 )
 	    optEl += "<option posId='go'>###Go###</option><option disabled='true'>------------</option>";
-	  if( (parseInt(this.srcNode.getAttribute('acs'))&SEQ_WR) && this.srcNode.getAttribute('s_com') )
+	  if( (parseInt(this.srcNode.getAttribute('acs'))&SEC_WR) && this.srcNode.getAttribute('s_com') )
 	  {
 	    if( this.srcNode.getAttribute('s_com').search('add') != -1 ) optEl += "<option posId='add'>###Add###</option>";
 	    if( this.srcNode.getAttribute('s_com').search('ins') != -1 && this.selectedIndex >= 0 ) optEl += "<option posId='ins'>###Insert###</option>";
@@ -789,17 +794,25 @@ function selectChildRecArea( node, aPath, cBlk )
 
 	//>>>> Copy values to info tree
 	for( var i_cl = 0; i_cl < dataReq.childNodes.length; i_cl++ )
-	  for( var i_cli = 0; i_cli < t_s.childNodes.length; i_cli++ )
-	    if( dataReq.childNodes[i_cl].getAttribute('id') == t_s.childNodes[i_cli].getAttribute('id') )
-	    {
-	      while( t_s.childNodes[i_cli].lastChild ) t_s.childNodes[i_cli].removeChild(t_s.childNodes[i_cli].lastChild);
-	      for( var i_rw = 0; i_rw < dataReq.childNodes[i_cl].childNodes.length; i_rw++ )
-	      {
-		var el = t_s.ownerDocument.createElement(dataReq.childNodes[i_cl].childNodes[i_rw].nodeName);
-		setNodeText(el,nodeText(dataReq.childNodes[i_cl].childNodes[i_rw]))
-		t_s.childNodes[i_cli].appendChild(el);
-	      }
-	    }
+	{
+	  var i_cli;
+	  for( i_cli = 0; i_cli < t_s.childNodes.length; i_cli++ )
+	    if( dataReq.childNodes[i_cl].getAttribute('id') == t_s.childNodes[i_cli].getAttribute('id') ) break;
+	  if( i_cli == t_s.childNodes.length )
+	  {
+	    var el = t_s.ownerDocument.createElement(dataReq.childNodes[i_cl].nodeName);
+	    for( var i_a = 0; i_a < dataReq.childNodes[i_cl].attributes.length; i_a++ )
+		el.setAttribute(dataReq.childNodes[i_cl].attributes[i_a].name,dataReq.childNodes[i_cl].attributes[i_a].value);
+	    t_s.appendChild(el);
+	  }
+	  while( t_s.childNodes[i_cli].lastChild ) t_s.childNodes[i_cli].removeChild(t_s.childNodes[i_cli].lastChild);
+	  for( var i_rw = 0; i_rw < dataReq.childNodes[i_cl].childNodes.length; i_rw++ )
+	  {
+	    var el = t_s.ownerDocument.createElement(dataReq.childNodes[i_cl].childNodes[i_rw].nodeName);
+	    setNodeText(el,nodeText(dataReq.childNodes[i_cl].childNodes[i_rw]))
+	    t_s.childNodes[i_cli].appendChild(el);
+	  }
+	}
 	//>>>> Calc rows and columns
 	var n_col = t_s.childNodes.length;
 	var n_row = n_col ? t_s.childNodes[0].childNodes.length : 0;
@@ -880,7 +893,7 @@ function selectChildRecArea( node, aPath, cBlk )
 		}
 	      }
 	    }
-	    else if( i_rw && wr && parseInt(t_s.childNodes[table.childNodes[i_rw].childNodes.length-1].getAttribute('acs'))&SEQ_WR )
+	    else if( i_rw && wr && parseInt(t_s.childNodes[table.childNodes[i_rw].childNodes.length-1].getAttribute('acs'))&SEC_WR )
 	      cCell.ondblclick = function( )
 	      {
 		this.isEnter = true;
@@ -1358,7 +1371,7 @@ function basicFields( t_s, aPath, cBlk, wr, comm )
       if( val_r )
       {
 	val_r.title = t_s.getAttribute('help');
-	setNodeText(val_r,parseInt(nodeText(dataReq))?'On':'Off');
+	setNodeText(val_r,nodeText(dataReq)=='<EVAL>'?'<EVAL>':(parseInt(nodeText(dataReq))?'On':'Off'));
       }
     }
     //> View edit fields
@@ -1387,8 +1400,7 @@ function basicFields( t_s, aPath, cBlk, wr, comm )
 	    btApply.onclick = function()
 	    {
 	      var wEl = this.parentNode.parentNode;
-	      wEl.childNodes[2].value;
-	      var rez = servSet(wEl.childNodes[2].itPath,'com=com','<set>'+wEl.childNodes[2].value+'</set>',true);
+	      var rez = servSet(wEl.childNodes[2].itPath,'com=com','<set>'+strEncode(wEl.childNodes[2].value,'html')+'</set>',true);
 	      if( rez && parseInt(rez.getAttribute('rez')) != 0 ) alert(nodeText(rez));
 	      setTimeout('pageRefresh()',500);
 	      wEl.removeChild(this.parentNode);
@@ -1955,7 +1967,7 @@ function itAdd( )
   //> Load branches list
   var typeCfg = '';
   for( var i_b = 0; i_b < branchS.childNodes.length; i_b++ )
-    if( parseInt(branchS.childNodes[i_b].getAttribute('acs'))&SEQ_WR )
+    if( parseInt(branchS.childNodes[i_b].getAttribute('acs'))&SEC_WR )
       typeCfg+="<option idSz='"+branchS.childNodes[i_b].getAttribute('idSz')+
 		     "' gid='"+branchS.childNodes[i_b].getAttribute('id')+
 		     "' idm='"+(parseInt(branchS.childNodes[i_b].getAttribute('idm'))?"1":"0")+"'>"+
@@ -2023,7 +2035,7 @@ function itDel( iit )
     for( var i_b = 0; i_b < branch.childNodes.length; i_b++ )
     {
       var b_id = branch.childNodes[i_b].getAttribute('id');
-      if( b_id == sel_el.substr(0,b_id.length) && parseInt(branch.childNodes[i_b].getAttribute('acs'))&SEQ_WR )
+      if( b_id == sel_el.substr(0,b_id.length) && parseInt(branch.childNodes[i_b].getAttribute('acs'))&SEC_WR )
       {
 	var idm = parseInt(branch.childNodes[i_b].getAttribute('idm'));
 	var com = idm ? ("<del id='"+sel_el.substr(b_id.length)+"'/>") :
@@ -2069,13 +2081,13 @@ function itPaste( )
 
   if( pathLev(copyBuf.substr(1),0) != pathLev(selPath,0) ) { alert('###Copy is imposible.###'); return; }
 
-  if( parseInt(root.getAttribute('acs'))&SEQ_WR ) { typeCfg+="<option idSz='-1' gid=''>###Selected###</option>"; itCnt++; }
+  if( parseInt(root.getAttribute('acs'))&SEC_WR ) { typeCfg+="<option idSz='-1' gid=''>###Selected###</option>"; itCnt++; }
   for( var i_ch = 0; i_ch < root.childNodes.length; i_ch++ )
     if( root.childNodes[i_ch].nodeName.toLowerCase() == 'branches' && root.childNodes[i_ch].getAttribute('id') == 'br' )
       branchS =  root.childNodes[i_ch];
   if( branchS )
     for( var i_b = 0; i_b < branchS.childNodes.length; i_b++, itCnt++ )
-      if( parseInt(branchS.childNodes[i_b].getAttribute('acs'))&SEQ_WR )
+      if( parseInt(branchS.childNodes[i_b].getAttribute('acs'))&SEC_WR )
       {
 	var gbrId = branchS.childNodes[i_b].getAttribute('id');
 	typeCfg+="<option idSz='"+branchS.childNodes[i_b].getAttribute('idSz')+"' gid='"+gbrId+"'>"+branchS.childNodes[i_b].getAttribute('dscr')+"</option>";
