@@ -1,8 +1,8 @@
 
 //OpenSCADA system module UI.WebCfgD file: web_cfg.cpp
 /***************************************************************************
- *   Copyright (C) 2008 by Roman Savochenko                                *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2008-2010 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -39,7 +39,7 @@
 #define MOD_TYPE	"UI"
 #define VER_TYPE	VER_UI
 #define SUB_TYPE	"WWW"
-#define MOD_VERSION	"0.6.1"
+#define MOD_VERSION	"0.6.4"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow the dynamic WEB based OpenSCADA system configurator. Use XHTML, CSS and JavaScript technologies."
 #define LICENSE		"GPL"
@@ -84,7 +84,7 @@ TWEB::TWEB( string name )
 
     mod		= this;
 
-    //- Reg export functions -
+    //> Reg export functions
     modFuncReg( new ExpFunc("void HttpGet(const string&,string&,const string&,vector<string>&,const string&);",
 	"Process Get comand from http protocol's!",(void(TModule::*)( )) &TWEB::HttpGet) );
     modFuncReg( new ExpFunc("void HttpPost(const string&,string&,const string&,vector<string>&,const string&);",
@@ -131,40 +131,11 @@ void TWEB::modInfo( vector<string> &list )
     list.push_back("Auth");
 }
 
-string TWEB::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
-	MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void TWEB::load_( )
 {
-    //- Load parameters from command line -
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{NULL      ,0,NULL,0  }
-    };
+    //> Load parameters from command line
 
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
-
-    //- Load parameters from config file -
+    //> Load parameters from config file
 }
 
 void TWEB::modStart()
@@ -231,6 +202,7 @@ void TWEB::HttpGet( const string &urli, string &page, const string &sender, vect
     try
     {
 	string zero_lev = TSYS::pathLev(ses.url,0);
+
 	//> Get about module page
 	if( zero_lev == "about" )	getAbout(ses);
 	//> Get module icon and global image
@@ -297,7 +269,8 @@ void TWEB::HttpGet( const string &urli, string &page, const string &sender, vect
 	    {
 		string itp = "png";
 		XMLNode req("get"); req.setAttr("path",ses.url+"/%2fico");
-		if( !mod->cntrIfCmd(req,ses.user) ) ses.page = TSYS::strDecode(req.text(),TSYS::base64);
+		if( !mod->cntrIfCmd(req,ses.user) )
+		    ses.page = TSYS::strDecode(req.text(),TSYS::base64);
 		else ses.page = TUIS::icoGet("disconnect",&itp);
 		page = httpHead("200 OK",ses.page.size(),string("image/")+itp)+ses.page;
 		return;
@@ -458,21 +431,21 @@ string TWEB::trMessReplace( const string &tsrc )
 
     int txtBeg = 0, i_s;
     for( i_s = 0; i_s < tsrc.size(); i_s++ )
-      if( tsrc[i_s] == '#' && tsrc.substr(i_s,3) == "###" && (i_s+3)<tsrc.size() && tsrc[i_s+3] != '#' )
-      {
-        int i_r;
-        for( i_r = i_s+3; i_r < tsrc.size(); i_r++ )
-          if( (tsrc[i_r] == '#' && tsrc.substr(i_r,3) == "###" && ((i_r+3)>=tsrc.size() || tsrc[i_r+3] != '#')) || tsrc[i_r] == '\n' )
-            break;
-        if( i_r < tsrc.size() && tsrc[i_r] != '\n' )
-        {
-          trez.append(tsrc.substr(txtBeg,i_s-txtBeg));
-          trez.append(_(tsrc.substr(i_s+3,i_r-i_s-3).c_str()));
-          i_s = i_r+2;
-          txtBeg = i_r+3;
-          continue;
-        }
-      }
+	if( tsrc[i_s] == '#' && tsrc.substr(i_s,3) == "###" && (i_s+3)<tsrc.size() && tsrc[i_s+3] != '#' )
+	{
+	    int i_r;
+	    for( i_r = i_s+3; i_r < tsrc.size(); i_r++ )
+		if( (tsrc[i_r] == '#' && tsrc.substr(i_r,3) == "###" && ((i_r+3)>=tsrc.size() || tsrc[i_r+3] != '#')) || tsrc[i_r] == '\n' )
+		    break;
+	    if( i_r < tsrc.size() && tsrc[i_r] != '\n' )
+	    {
+		trez.append(tsrc.substr(txtBeg,i_s-txtBeg));
+		trez.append(_(tsrc.substr(i_s+3,i_r-i_s-3).c_str()));
+		i_s = i_r+2;
+		txtBeg = i_r+3;
+		continue;
+	    }
+	}
     if( txtBeg < i_s ) trez.append(tsrc.substr(txtBeg,i_s-txtBeg));
 
     return trez;
@@ -517,18 +490,16 @@ void TWEB::cntrCmdProc( XMLNode *opt )
 	TUI::cntrCmdProc(opt);
 	if(ctrMkNode("area",opt,1,"/prm/cfg",_("Module options")))
 	    ctrMkNode("comm",opt,-1,"/prm/cfg/host_lnk",_("Go to remote stations list configuration"),0660,"root","root",1,"tp","lnk");
-	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
         return;
     }
 
     //> Process command to page
     string a_path = opt->attr("path");
-    if( a_path == "/prm/cfg/host_lnk" && ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )
+    if( a_path == "/prm/cfg/host_lnk" && ctrChkNode(opt,"get",0660,"root","root",SEC_RD) )
     {
 	SYS->transport().at().setSysHost(false);
 	opt->setText("/Transport");
     }
-    else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )	opt->setText(optDescr());
     else TUI::cntrCmdProc(opt);
 }
 
@@ -576,7 +547,7 @@ SSess::SSess( const string &iurl, const string &isender, const string &iuser,
     {
 	pos = content.find(boundary,pos);
 	if( pos == string::npos || content.substr(pos+boundary.size(),2) == "--" ) break;
-	    pos += boundary.size()+strlen(c_term);
+	pos += boundary.size()+strlen(c_term);
 	string c_head = content.substr(pos, content.find(c_term,pos)-pos);
 	if( c_head.find(c_fd,0) == string::npos ) continue;
 
@@ -609,4 +580,3 @@ SSess::SSess( const string &iurl, const string &isender, const string &iuser,
 	}
     }
 }
-
