@@ -36,7 +36,7 @@
 #define MOD_TYPE	"UI"
 #define VER_TYPE	VER_UI
 #define SUB_TYPE	"WWW"
-#define MOD_VERSION	"0.6.10"
+#define MOD_VERSION	"0.8.0"
 #define AUTORS		"Roman Savochenko"
 #define DEVELOPERS	"Roman Savochenko, Lysenko Maxim, Yashina Kseniya"
 #define DESCRIPTION	"Web operation user interface for visual control area (VCA) projects playing."
@@ -325,7 +325,7 @@ void TWEB::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -392,7 +392,7 @@ string TWEB::httpHead( const string &rcode, int cln, const string &cnt_tp, const
 	"Accept-Ranges: bytes\r\n"
 	"Content-Length: "+TSYS::int2str(cln)+"\r\n"
 	"Connection: close\r\n"
-	"Content-type: "+cnt_tp+"\r\n"
+	"Content-Type: "+cnt_tp+"\r\n"
 	"Charset="+Mess->charset()+"\r\n"+addattr+"\r\n";
 }
 
@@ -481,7 +481,7 @@ void TWEB::HttpGet( const string &url, string &page, const string &sender, vecto
 			"<table border='0' cellspacing='3px' width='100%'>\n";
 		    for( int i_ch = 0; i_ch < req.childSize(); i_ch++ )
 		    {
-			if( !SYS->security().at().access(user,SEQ_WR,"root","root",RWRWR_) && self_sess.find(req.childGet(i_ch)->text()+";",0) != string::npos )
+			if( !SYS->security().at().access(user,SEC_WR,"root","root",RWRWR_) && self_sess.find(req.childGet(i_ch)->text()+";",0) != string::npos )
 			    continue;
 			ses.page += "<tr><td style='text-align: center;'><a href='/"MOD_ID"/prj_"+req.childGet(i_ch)->attr("id")+"/'>"+
 			    req.childGet(i_ch)->text()+"</a></td></tr>";
@@ -661,8 +661,8 @@ void TWEB::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/prm/cfg/lf_tm" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )   opt->setText( TSYS::int2str(sessTime()) );
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )   setSessTime( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0660,"root","root",SEC_RD) )   opt->setText( TSYS::int2str(sessTime()) );
+	if( ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )   setSessTime( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )   opt->setText(optDescr());
     else TUI::cntrCmdProc(opt);
