@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: resalloc.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,6 +19,8 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include "errno.h"
+
 #include "tsys.h"
 #include "resalloc.h"
 
@@ -37,30 +39,52 @@ Res::~Res( )
     pthread_rwlock_destroy(&rwc);
 }
 
-void Res::resRequestW( long tm )
+void Res::resRequestW( unsigned short tm )
 {
-    if( !tm ) pthread_rwlock_wrlock(&rwc);
+    int rez = 0;
+    if( !tm ) rez = pthread_rwlock_wrlock(&rwc);
     else
     {
 	timespec wtm;
 	clock_gettime(CLOCK_REALTIME,&wtm);
-	wtm.tv_sec += tm;
-	if( pthread_rwlock_timedwrlock(CLOCK_REALTIME,&wtm) )
-	    throw TError("ResAlloc",_("Timeouted!"));
+	wtm.tv_nsec += 1000000*(tm%1000);
+	wtm.tv_sec += tm/1000 + wtm.tv_nsec/1000000000; wtm.tv_nsec = wtm.tv_nsec%1000000000;
+	rez = pthread_rwlock_timedwrlock(&rwc,&wtm);
     }
+    if( rez == EDEADLK ) throw TError(10,"ResAlloc",_("Resource is try deadlock a thread!"));
+    else if( tm && rez == ETIMEDOUT ) throw TError("ResAlloc",_("Resource is timeouted!"));
 }
 
-void Res::resRequestR( long tm )
+bool Res::resTryW( )
 {
-    if( !tm ) pthread_rwlock_rdlock(&rwc);
+    int rez = pthread_rwlock_trywrlock(&rwc);
+    if( rez == EBUSY ) return false;
+    else if( rez == EDEADLK ) throw TError(10,"ResAlloc",_("Resource is try deadlock a thread!"));
+    return true;
+}
+
+void Res::resRequestR( unsigned short tm )
+{
+    int rez = 0;
+    if( !tm ) rez = pthread_rwlock_rdlock(&rwc);
     else
     {
 	timespec wtm;
 	clock_gettime(CLOCK_REALTIME,&wtm);
-	wtm.tv_sec += tm;
-	if( pthread_rwlock_timedrdlock(CLOCK_REALTIME,&wtm) )
-	    throw TError("ResAlloc",_("Timeouted!"));
+	wtm.tv_nsec += 1000000*(tm%1000);
+	wtm.tv_sec += tm/1000 + wtm.tv_nsec/1000000000; wtm.tv_nsec = wtm.tv_nsec%1000000000;
+	rez = pthread_rwlock_timedrdlock(&rwc,&wtm);
     }
+    if( rez == EDEADLK ) throw TError(10,"ResAlloc",_("Resource is try deadlock a thread!"));
+    else if( tm && rez == ETIMEDOUT ) throw TError("ResAlloc",_("Resource is timeouted!"));
+}
+
+bool Res::resTryR( )
+{
+    int rez = pthread_rwlock_tryrdlock(&rwc);
+    if( rez == EBUSY ) return false;
+    else if( rez == EDEADLK ) throw TError(10,"ResAlloc",_("Resource is try deadlock a thread!"));
+    return true;
 }
 
 void Res::resRelease( )
@@ -76,7 +100,7 @@ ResAlloc::ResAlloc( Res &rid ) : mId(rid), mAlloc(false)
 
 }
 
-ResAlloc::ResAlloc( Res &rid, bool write, long tm ) : mId(rid), mAlloc(false)
+ResAlloc::ResAlloc( Res &rid, bool write, unsigned short tm ) : mId(rid), mAlloc(false)
 {
     request( write, tm );
 }
@@ -86,13 +110,16 @@ ResAlloc::~ResAlloc( )
     if( mAlloc ) release();
 }
 
-void ResAlloc::request( bool write, long tm )
+void ResAlloc::request( bool write, unsigned short tm )
 {
     if( mAlloc ) release();
     mAlloc = false;
-    if( write ) mId.resRequestW(tm);
-    else mId.resRequestR(tm);
-    mAlloc = true;
+    try
+    {
+	if( write ) mId.resRequestW(tm);
+	else mId.resRequestR(tm);
+	mAlloc = true;
+    }catch(TError err) { if(err.cod!=10) throw; }
 }
 
 void ResAlloc::release()
@@ -101,3 +128,28 @@ void ResAlloc::release()
     mId.resRelease( );
     mAlloc = false;
 }
+
+//********************************************
+//* String+resource for                      *
+//********************************************
+ResString::ResString( const string &vl )
+{
+    setVal(vl);
+}
+
+ResString::~ResString( )
+{
+    ResAlloc wres( mRes, true );
+}
+
+void ResString::setVal( const string &vl )
+{
+    ResAlloc wres( mRes, true );
+    str = vl;
+}
+
+string ResString::getVal( )
+{
+    ResAlloc wres( mRes, false );
+    return str;
+}
