@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: resalloc.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -37,9 +37,11 @@ class Res
 	Res( );
 	~Res( );
 
-	void resRequestW( long tm = 0 );	// Write request
-	void resRequestR( long tm = 0 );	// Read request
-	void resRelease( );			// Release
+	void resRequestW( unsigned short tm = 0 );	// Write request, tm in milliseconds
+	bool resTryW( );
+	void resRequestR( unsigned short tm = 0 );	// Read request, tm in milliseconds
+	bool resTryR( );
+	void resRelease( );				// Release
 
     private:
 	pthread_rwlock_t rwc;
@@ -53,10 +55,10 @@ class ResAlloc
     public:
 	//Methods
 	ResAlloc( Res &rid );
-	ResAlloc( Res &rid, bool write, long tm = 0 );
+	ResAlloc( Res &rid, bool write, unsigned short tm = 0 );
 	~ResAlloc( );
 
-	void request( bool write = false, long tm = 0 );
+	void request( bool write = false, unsigned short tm = 0 );
 	void release( );
 
     private:
@@ -72,13 +74,14 @@ class ResString
 {
     public:
 	//Methods
-	ResString( const string &vl = "" ) : str(vl)	{ }
+	explicit ResString( const string &vl = "" );
+	~ResString( );
 
-	void setVal( const string &vl )	{ ResAlloc wres(res,true); str = vl; }
-	string getVal( )		{ ResAlloc wres(res,false); return str; }
+	void setVal( const string &vl );
+	string getVal( );
 
 	//Attributes
-	Res	res;
+	Res	mRes;
 	string	str;
 };
 
