@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tarchives.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -98,13 +98,7 @@ TArchiveS::~TArchiveS(  )
     timer_delete(tmIdMess);
 
     //> Stop values archiving task
-    if( prcStVal )
-    {
-	endrunReqVal = true;
-	pthread_kill( mValPthr, SIGALRM );
-	TSYS::eventWait(prcStVal,false,nodePath()+"val_task_stop",5);
-	pthread_join( mValPthr, NULL );
-    }
+    if( prcStVal ) SYS->taskDestroy( nodePath('.',true)+".vals", &prcStVal, &endrunReqVal );
 
     //> Free other resources
     nodeDelAll();
@@ -112,6 +106,8 @@ TArchiveS::~TArchiveS(  )
 
 int TArchiveS::valPeriod( )		{ return vmax(1,mValPer); }
 
+void TArchiveS::setValPrior( int ivl )	{ mValPrior = vmax(-1,vmin(99,ivl)); modif(); }
+
 void TArchiveS::load_( )
 {
     //> Load parameters from command line
@@ -129,7 +125,7 @@ void TArchiveS::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -327,22 +323,7 @@ void TArchiveS::subStart( )
     timer_settime(tmIdMess, 0, &itval, NULL);
 
     //> Start values acquisition task
-    if( !prcStVal )
-    {
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	struct sched_param prior;
-	if( valPrior() && SYS->user() == "root" )
-	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority = valPrior();
-	pthread_attr_setschedparam(&pthr_attr,&prior);
-
-	pthread_create(&mValPthr,&pthr_attr,TArchiveS::ArhValTask,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prcStVal, true, nodePath()+"val_task_start",5) )
-	    throw TError(nodePath().c_str(),_("Values acquisition task is not started!"));
-    }
+    if( !prcStVal ) SYS->taskCreate( nodePath('.',true)+".vals", valPrior(), TArchiveS::ArhValTask, this, &prcStVal );
 
     TSubSYS::subStart( );
 }
@@ -364,14 +345,7 @@ void TArchiveS::subStop( )
 	throw TError(nodePath().c_str(),_("Archive messages thread is not stopped!"));
 
     //> Values acquisition task stop
-    if( prcStVal )
-    {
-	endrunReqVal = true;
-	pthread_kill( mValPthr, SIGALRM );
-	if( TSYS::eventWait(prcStVal,false,nodePath()+"val_task_stop",10) )
-	    throw TError(nodePath().c_str(),_("Archive values task is not stopped!"));
-	pthread_join( mValPthr, NULL );
-    }
+    if( prcStVal ) SYS->taskDestroy( nodePath('.',true)+".vals", &prcStVal, &endrunReqVal );
 
     //> Last messages archivation call
     sigval obj; obj.sival_ptr = this;
@@ -425,14 +399,14 @@ void TArchiveS::subStop( )
     }
 }
 
-void TArchiveS::messPut( time_t tm, int utm, const string &categ, TMess::Type level, const string &mess )
+void TArchiveS::messPut( time_t tm, int utm, const string &categ, char level, const string &mess )
 {
     //> Put message to buffer
     ResAlloc res(mRes,true);
     mBuf[headBuf].time  = tm;
-    mBuf[headBuf].utime  = utm;
+    mBuf[headBuf].utime = utm;
     mBuf[headBuf].categ = categ;
-    mBuf[headBuf].level = level;
+    mBuf[headBuf].level = (TMess::Type)abs(level);
     mBuf[headBuf].mess  = mess;
     if( ++headBuf >= mBuf.size() ) headBuf = 0;
     //> Check to no archivated messages
@@ -459,6 +433,11 @@ void TArchiveS::messPut( time_t tm, int utm, const string &categ, TMess::Type le
 	}
     }
     else bufErr = 0;
+
+    //> Alarms processing. For level less 0 alarm is set
+    map<string,TMess::SRec>::iterator p;
+    if( level < 0 ) mAlarms[categ] = TMess::SRec(tm,utm,categ,(TMess::Type)abs(level),mess);
+    else if( (p=mAlarms.find(categ)) != mAlarms.end() ) mAlarms.erase(p);
 }
 
 void TArchiveS::messPut( const vector<TMess::SRec> &recs )
@@ -467,14 +446,15 @@ void TArchiveS::messPut( const vector<TMess::SRec> &recs )
 	messPut(recs[i_r].time,recs[i_r].utime,recs[i_r].categ,recs[i_r].level,recs[i_r].mess);
 }
 
-void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category, TMess::Type level, const string &arch )
+void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category, char level, const string &arch )
 {
     recs.clear();
 
     ResAlloc res(mRes,false);
+
     //> Get records from buffer
     int i_buf = headBuf;
-    while( !arch.size() || arch==BUF_ARCH_NM )
+    while( level >= 0 && (!arch.size() || arch==BUF_ARCH_NM) )
     {
 	if( mBuf[i_buf].time >= b_tm && mBuf[i_buf].time != 0 && mBuf[i_buf].time <= e_tm &&
 		mBuf[i_buf].level >= level && TMess::chkPattern( mBuf[i_buf].categ, category ) )
@@ -486,7 +466,7 @@ void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, c
     //> Get records from archives
     vector<string> t_lst, o_lst;
     modList(t_lst);
-    for( int i_t = 0; i_t < t_lst.size(); i_t++ )
+    for( int i_t = 0; level >= 0 && i_t < t_lst.size(); i_t++ )
     {
 	at(t_lst[i_t]).at().messList(o_lst);
 	for( int i_o = 0; i_o < o_lst.size(); i_o++ )
@@ -496,6 +476,18 @@ void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, c
 		archtor.at().get(b_tm,e_tm,recs,category,level);
 	}
     }
+
+    //> Alarms request processing
+    if( level < 0 )
+    {
+	vector< pair<long long,TMess::SRec* > > mb;
+	for( map<string,TMess::SRec>::iterator p = mAlarms.begin(); p != mAlarms.end(); p++ )
+	    if( (p->second.time >= b_tm || b_tm == e_tm) && p->second.time <= e_tm &&
+		    p->second.level >= abs(level) && Mess->chkPattern(p->second.categ,category) )
+		mb.push_back(pair<long long,TMess::SRec* >(FTM(p->second),&p->second));
+	sort(mb.begin(),mb.end());
+	for( int i_b = 0; i_b < mb.size(); i_b++ ) recs.push_back(*mb[i_b].second);
+    }
 }
 
 time_t TArchiveS::messBeg( const string &arch )
@@ -548,7 +540,7 @@ time_t TArchiveS::messEnd( const string &arch )
 	if( !arch.empty() ) return rez;
     }
 
-    //- Get records from archives -
+    //> Get records from archives
     vector<string> t_lst, o_lst;
     modList(t_lst);
     AutoHD<TMArchivator> archtor;
@@ -691,19 +683,43 @@ void *TArchiveS::ArhValTask( void *param )
     return NULL;
 }
 
+TVariant TArchiveS::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    if( iid == "messGet" && prms.size() >= 2 )
+    {
+	vector<TMess::SRec> recs;
+	messGet( prms[0].getI(), prms[1].getI(), recs, ((prms.size()>=3) ? prms[2].getS() : ""), 
+	    ((prms.size()>=4) ? prms[3].getI() : 0), ((prms.size()>=5) ? prms[4].getS() : 0) );
+	TArrayObj *rez = new TArrayObj();
+	for( int i_m = 0; i_m < recs.size(); i_m++ )
+	{
+	    TVarObj *am = new TVarObj();
+	    am->propSet("tm",(int)recs[i_m].time);
+	    am->propSet("utm",recs[i_m].utime);
+	    am->propSet("categ",recs[i_m].categ);
+	    am->propSet("level",recs[i_m].level);
+	    am->propSet("mess",recs[i_m].mess);
+	    rez->propSet(TSYS::int2str(i_m),am);
+	}
+	return rez;
+    }
+
+    return TCntrNode::objFuncCall(iid,prms,user);
+}
+
 void TArchiveS::cntrCmdProc( XMLNode *opt )
 {
     string a_path = opt->attr("path");
-    //- Service commands process -
+    //> Service commands process
     if( a_path == "/serv/mess" )           //Messages access
     {
-	if( ctrChkNode(opt,"info",RWRWRW,"root","root",SEQ_RD) )        //Messages information
+	if( ctrChkNode(opt,"info",RWRWRW,"root","root",SEC_RD) )        //Messages information
 	{
 	    string arch = opt->attr("arch");
 	    opt->setAttr("end",TSYS::uint2str(messEnd(arch)));
 	    opt->setAttr("beg",TSYS::uint2str(messBeg(arch)));
 	}
-	else if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEQ_RD) )    //Value's data request
+	else if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEC_RD) )    //Value's data request
 	{
 	    time_t tm      = strtoul(opt->attr("tm").c_str(),0,10);
 	    time_t tm_grnd = strtoul(opt->attr("tm_grnd").c_str(),0,10);
@@ -740,7 +756,7 @@ void TArchiveS::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/m_arch/view/size",_("Size (s)"),0664,"root",my_gr.c_str(),1,"tp","dec");
 		ctrMkNode("fld",opt,-1,"/m_arch/view/cat",_("Category pattern"),0664,"root",my_gr.c_str(),2,"tp","str",
 		    "help",_("Messages category. Use template symbols for group selection:\n  '*' - any substring;\n  '?' - any symbol."));
-		ctrMkNode("fld",opt,-1,"/m_arch/view/lvl",_("Level"),0664,"root",my_gr.c_str(),4,"tp","dec","min","0","max","7",
+		ctrMkNode("fld",opt,-1,"/m_arch/view/lvl",_("Level"),0664,"root",my_gr.c_str(),4,"tp","dec","min","-7","max","7",
 		    "help",_("Get messages for level more and equaly it."));
 		ctrMkNode("fld",opt,-1,"/m_arch/view/archtor",_("Archivator"),0664,"root",my_gr.c_str(),4,"tp","str","dest","select","select","/m_arch/lstAMess",
 		    "help",_("Messages archivator.\nNo set archivator for process by buffer and all archivators.\nSet '<buffer>' for process by buffer."));
@@ -758,57 +774,58 @@ void TArchiveS::cntrCmdProc( XMLNode *opt )
 	{
 	    ctrMkNode("fld",opt,-1,"/v_arch/per",_("Get data period (ms)"),0664,"root",my_gr.c_str(),1,"tp","dec");
 	    ctrMkNode("fld",opt,-1,"/v_arch/prior",_("Get data task priority level"),0664,"root",my_gr.c_str(),1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/v_arch/nmb",_("Number"),0444,"root",my_gr.c_str(),1,"tp","str");
 	    ctrMkNode("list",opt,-1,"/v_arch/archs",_("Value archives"),0664,"root",my_gr.c_str(),5,"tp","br","idm","1","s_com","add,del","br_pref","va_","idSz","20");
 	}
 	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root",my_gr.c_str(),3,"tp","str","cols","90","rows","10");
 	return;
     }
 
-    //- Process command to page -
+    //> Process command to page
     if( a_path == "/m_arch/max_am_req" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText(TSYS::int2str(max_req_mess));
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	{ max_req_mess = atoi(opt->text().c_str()); modif(); }
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText(TSYS::int2str(max_req_mess));
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	{ max_req_mess = atoi(opt->text().c_str()); modif(); }
     }
     else if( a_path == "/m_arch/per" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText(TSYS::int2str(messPeriod()));
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	setMessPeriod(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText(TSYS::int2str(messPeriod()));
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	setMessPeriod(atoi(opt->text().c_str()));
     }
     else if( a_path == "/m_arch/size" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText(TSYS::int2str(messBufLen()));
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	setMessBufLen(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText(TSYS::int2str(messBufLen()));
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	setMessBufLen(atoi(opt->text().c_str()));
     }
     else if( a_path == "/m_arch/view/tm" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )
 	{
 	    opt->setText(TBDS::genDBGet(nodePath()+"messTm","0",opt->attr("user")));
 	    if( !atoi(opt->text().c_str()) )    opt->setText(TSYS::int2str(time(NULL)));
 	}
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )
 	    TBDS::genDBSet(nodePath()+"messTm",(atoi(opt->text().c_str())>=time(NULL))?"0":opt->text(),opt->attr("user"));
     }
     else if( a_path == "/m_arch/view/size" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messSize","60",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	TBDS::genDBSet(nodePath()+"messSize",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messSize","60",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	TBDS::genDBSet(nodePath()+"messSize",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/m_arch/view/cat" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messCat","",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	TBDS::genDBSet(nodePath()+"messCat",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messCat","",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	TBDS::genDBSet(nodePath()+"messCat",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/m_arch/view/lvl" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messLev","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	TBDS::genDBSet(nodePath()+"messLev",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messLev","0",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	TBDS::genDBSet(nodePath()+"messLev",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/m_arch/view/archtor" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messArch","",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	TBDS::genDBSet(nodePath()+"messArch",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messArch","",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	TBDS::genDBSet(nodePath()+"messArch",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/m_arch/lstAMess"  && ctrChkNode(opt,"get",0440) )
     {
@@ -831,7 +848,7 @@ void TArchiveS::cntrCmdProc( XMLNode *opt )
 	int    gsz = atoi(TBDS::genDBGet(nodePath()+"messSize","60",opt->attr("user")).c_str());
 	messGet( gtm-gsz, gtm, rec,
 		 TBDS::genDBGet(nodePath()+"messCat","",opt->attr("user")),
-		 (TMess::Type)atoi(TBDS::genDBGet(nodePath()+"messLev","0",opt->attr("user")).c_str()),
+		 atoi(TBDS::genDBGet(nodePath()+"messLev","0",opt->attr("user")).c_str()),
 		 TBDS::genDBGet(nodePath()+"messArch","",opt->attr("user")) );
 
 	XMLNode *n_tm	= ctrMkNode("list",opt,-1,"/m_arch/view/mess/0","",0440);
@@ -850,29 +867,38 @@ void TArchiveS::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/v_arch/per" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText( TSYS::int2str(valPeriod()) );
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	setValPeriod( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText( TSYS::int2str(valPeriod()) );
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	setValPeriod( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/v_arch/prior" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )	opt->setText( TSYS::int2str(valPrior()) );
-	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEQ_WR) )	setValPrior( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )	opt->setText( TSYS::int2str(valPrior()) );
+	if( ctrChkNode(opt,"set",0664,"root",my_gr.c_str(),SEC_WR) )	setValPrior( atoi(opt->text().c_str()) );
+    }
+    else if( a_path == "/v_arch/nmb" && ctrChkNode(opt) )
+    {
+	vector<string> list;
+	valList(list);
+	int e_c = 0;
+	for( int i_a = 0; i_a < list.size(); i_a++ )
+	    if( valAt(list[i_a]).at().startStat() )	e_c++;
+	opt->setText(TSYS::strMess(_("All: %d; Enabled: %d"),list.size(),e_c));
     }
     else if( a_path == "/br/va_" || a_path == "/v_arch/archs" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root",my_gr.c_str(),SEC_RD) )
 	{
 	    vector<string> list;
 	    valList(list);
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",list[i_a])->setText(valAt(list[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root",my_gr.c_str(),SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root",my_gr.c_str(),SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    valAdd(vid); valAt(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root",my_gr.c_str(),SEQ_WR) )	chldDel(mAval,opt->attr("id"),-1,1);
+	if( ctrChkNode(opt,"del",0664,"root",my_gr.c_str(),SEC_WR) )	chldDel(mAval,opt->attr("id"),-1,1);
     }
     else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )	opt->setText(optDescr());
     else TSubSYS::cntrCmdProc(opt);
@@ -911,7 +937,7 @@ void TTipArchivator::valAdd( const string &iid, const string &idb )
 
 void TTipArchivator::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TModule::cntrCmdProc(opt);
@@ -922,39 +948,39 @@ void TTipArchivator::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("list",opt,-1,"/arch/val",_("Value archivators"),0664,"root","Archive",5,"tp","br","idm","1","s_com","add,del","br_pref","val_","idSz","20");
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/br/mess_" || a_path == "/arch/mess" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )
 	{
 	    vector<string> list;
 	    messList(list);
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",list[i_a])->setText(messAt(list[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","Archive",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","Archive",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    messAdd(vid); messAt(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root","Archive",SEQ_WR) )	messDel(opt->attr("id"),true);
+	if( ctrChkNode(opt,"del",0664,"root","Archive",SEC_WR) )	messDel(opt->attr("id"),true);
     }
     else if( a_path == "/br/val_" || a_path == "/arch/val" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )
 	{
 	    vector<string> list;
 	    valList(list);
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",list[i_a])->setText(valAt(list[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","Archive",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","Archive",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    valAdd(vid); valAt(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root","Archive",SEQ_WR) )	valDel(opt->attr("id"),true);
+	if( ctrChkNode(opt,"del",0664,"root","Archive",SEC_WR) )	valDel(opt->attr("id"),true);
     }
     else TModule::cntrCmdProc(opt);
 }
@@ -979,7 +1005,7 @@ TCntrNode &TMArchivator::operator=( TCntrNode &node )
     TMArchivator *src_n = dynamic_cast<TMArchivator*>(&node);
     if( !src_n ) return *this;
 
-    //- Configuration copy -
+    //> Configuration copy
     string tid = id();
     *(TConfig*)this = *(TConfig*)src_n;
     cfg("MODUL").setS(owner().modId());
@@ -1054,7 +1080,7 @@ bool TMArchivator::chkMessOK( const string &icateg, TMess::Type ilvl )
 
 void TMArchivator::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TCntrNode::cntrCmdProc(opt);
@@ -1106,71 +1132,71 @@ void TMArchivator::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/prm/st/st" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText( startStat() ? "1" : "0" );
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	atoi(opt->text().c_str()) ? start() : stop();
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText( startStat() ? "1" : "0" );
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	atoi(opt->text().c_str()) ? start() : stop();
     }
     else if( a_path == "/prm/st/db" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText( DB() );
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setDB( opt->text() );
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText( DB() );
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	setDB( opt->text() );
     }
     else if( a_path == "/prm/st/end" && ctrChkNode(opt) )		opt->setText( TSYS::int2str(end()) );
     else if( a_path == "/prm/st/beg" && ctrChkNode(opt) )		opt->setText( TSYS::int2str(begin()) );
     else if( a_path == "/prm/cfg/id" && ctrChkNode(opt) )		opt->setText( id() );
     else if( a_path == "/prm/cfg/nm" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText( name() );
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setName( opt->text() );
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText( name() );
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	setName( opt->text() );
     }
     else if( a_path == "/prm/cfg/dscr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText( dscr() );
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setDscr( opt->text() );
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText( dscr() );
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	setDscr( opt->text() );
     }
     else if( a_path == "/prm/cfg/addr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText( addr() );
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setAddr( opt->text() );
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText( addr() );
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	setAddr( opt->text() );
     }
     else if( a_path == "/prm/cfg/lvl" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str(level()));
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setLevel( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText(TSYS::int2str(level()));
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	setLevel( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/prm/cfg/start" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText( toStart() ? "1" : "0" );
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setToStart( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText( toStart() ? "1" : "0" );
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	setToStart( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/prm/cfg/cats" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(m_cat_o);
-	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	{ m_cat_o = opt->text(); modif(); }
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEC_RD) )	opt->setText(m_cat_o);
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEC_WR) )	{ m_cat_o = opt->text(); modif(); }
     }
     else if( a_path == "/mess/tm" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","Archive",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0660,"root","Archive",SEC_RD) )
 	{
 	    opt->setText(TBDS::genDBGet(nodePath()+"messTm","0",opt->attr("user")));
 	    if( !atoi(opt->text().c_str()) )    opt->setText(TSYS::int2str(time(NULL)));
 	}
-	if( ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0660,"root","Archive",SEC_WR) )
 	    TBDS::genDBSet(nodePath()+"messTm",(atoi(opt->text().c_str())>=time(NULL))?"0":opt->text(),opt->attr("user"));
     }
     else if( a_path == "/mess/size" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","Archive",SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messSize","10",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )	TBDS::genDBSet(nodePath()+"messSize",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0660,"root","Archive",SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messSize","10",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0660,"root","Archive",SEC_WR) )	TBDS::genDBSet(nodePath()+"messSize",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/mess/cat" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","Archive",SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messCat","",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )	TBDS::genDBSet(nodePath()+"messCat",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0660,"root","Archive",SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messCat","",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0660,"root","Archive",SEC_WR) )	TBDS::genDBSet(nodePath()+"messCat",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/mess/lvl" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","Archive",SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messLev","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )	TBDS::genDBSet(nodePath()+"messLev",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0660,"root","Archive",SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"messLev","0",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0660,"root","Archive",SEC_WR) )	TBDS::genDBSet(nodePath()+"messLev",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/mess/mess" && run_st && ctrChkNode(opt,"get",0440,"root","Archive") )
     {
