@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tarchives.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -123,6 +123,7 @@ class TMArchivator : public TCntrNode, public TConfig
 //* TTipArchivator                               *
 //************************************************
 class TArchiveS;
+class TVArchivator;
 
 class TTipArchivator: public TModule
 {
@@ -164,6 +165,8 @@ class TTipArchivator: public TModule
 //************************************************
 //* TArchiveS                                    *
 //************************************************
+class TVArchive;
+
 class TArchiveS : public TSubSYS
 {
     public:
@@ -179,7 +182,7 @@ class TArchiveS : public TSubSYS
 
 	void setMessPeriod( int ivl );
 	void setValPeriod( int ivl )	{ mValPer = ivl; modif(); }
-	void setValPrior( int ivl )	{ mValPrior = ivl; modif(); }
+	void setValPrior( int ivl );
 
 	void subStart( );
 	void subStop( );
@@ -197,10 +200,10 @@ class TArchiveS : public TSubSYS
 	AutoHD<TTipArchivator> at( const string &name )		{ return modAt(name); }
 
 	//> Message archive function
-	void messPut( time_t tm, int utm, const string &categ, TMess::Type level, const string &mess );
+	void messPut( time_t tm, int utm, const string &categ, char level, const string &mess );
 	void messPut( const vector<TMess::SRec> &recs );
 	void messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category = "",
-		TMess::Type level = TMess::Debug, const string &arch = "" );
+	    char level = TMess::Debug, const string &arch = "" );
 	time_t messBeg( const string &arch = "" );
 	time_t messEnd( const string &arch = "" );
 
@@ -224,6 +227,7 @@ class TArchiveS : public TSubSYS
 	static void *ArhValTask( void *param );
 
 	void cntrCmdProc( XMLNode *opt );       //Control interface command process
+	TVariant objFuncCall( const string &id, vector<TVariant> &prms, const string &user );
 
 	int messBufLen( )	{ return mBuf.size(); }
 	void setMessBufLen( int len );
@@ -243,12 +247,12 @@ class TArchiveS : public TSubSYS
 	unsigned headBuf,		//Head of messages buffer
 		headLstread;		//Last read and archived head of messages buffer
 	vector<TMess::SRec> mBuf;	//Messages buffer
+	map<string,TMess::SRec> mAlarms;//Alarms buffer
 
 	//> Value archiving
 	Res	vRes;			//Value access resource
 	int	mValPer;		//Value arhiving period
 	int	mValPrior;		//Value archive task priority
-	pthread_t mValPthr;		//Value get realtime pthread
 	bool	prcStVal;		//Process value flag
 	bool	endrunReqVal;		//Endrun value request
 	int	mAval;
