@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tbds.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -201,7 +201,15 @@ bool TBDS::dataSeek( const string &bdn, const string &path, int lev, TConfig &cf
 		    if( i_el == cf_el.size() && lev <= c_lev++ )
 		    {
 			for( int i_el = 0; i_el < cf_el.size(); i_el++ )
-			    cfg.cfg(cf_el[i_el]).setS(el->attr(cf_el[i_el]));
+			{
+			    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
+			    u_cfg.setS(el->attr(cf_el[i_el]));
+			    if( !cfg.noTransl() && u_cfg.fld().flg()&TCfg::TransltText && (Mess->lang2CodeBase().empty() || Mess->lang2Code() != Mess->lang2CodeBase()) )
+			    {
+				string vl = el->attr(cf_el[i_el]+"_"+Mess->lang2Code());
+				if( !vl.empty() ) u_cfg.setS(vl);
+			    }
+			}
 			return true;
 		    }
 		}
@@ -267,10 +275,9 @@ bool TBDS::dataGet( const string &bdn, const string &path, TConfig &cfg )
 		{
 		    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 		    u_cfg.setS(el->attr(cf_el[i_el]));
-		    if( u_cfg.fld().flg()&TCfg::TransltText && !Mess->lang2CodeBase().empty() &&
-			((!cfg.lang2Code().empty() && cfg.lang2Code() != Mess->lang2CodeBase()) || (cfg.lang2Code().empty() && Mess->lang2Code() != Mess->lang2CodeBase())) )
+		    if( !cfg.noTransl() && u_cfg.fld().flg()&TCfg::TransltText && (Mess->lang2CodeBase().empty() || Mess->lang2Code() != Mess->lang2CodeBase()) )
 		    {
-			string vl = el->attr(cf_el[i_el]+"_"+(cfg.lang2Code().empty()?Mess->lang2Code():cfg.lang2Code()));
+			string vl = el->attr(cf_el[i_el]+"_"+Mess->lang2Code());
 			if( !vl.empty() ) u_cfg.setS(vl);
 		    }
 		}
@@ -344,7 +351,7 @@ void TBDS::genDBSet(const string &path, const string &val, const string &user, c
 	if( !tbl.freeStat() )
 	{
 	    TConfig db_el(&dbs.at());
-	    if( !(rFlg&TBDS::UseTranslate) ) db_el.setLang2Code(Mess->lang2CodeBase());
+	    db_el.setNoTransl( !(rFlg&TBDS::UseTranslate) );
 	    db_el.cfg("user").setS(user);
 	    db_el.cfg("id").setS(path);
 	    db_el.cfg("val").setS(val);
@@ -387,7 +394,7 @@ string TBDS::genDBGet(const string &path, const string &oval, const string &user
 	if( !tbl.freeStat() )
 	{
 	    TConfig db_el(&dbs.at());
-	    if( !(rFlg&TBDS::UseTranslate) ) db_el.setLang2Code(Mess->lang2CodeBase());
+	    db_el.setNoTransl( !(rFlg&TBDS::UseTranslate) );
 	    db_el.cfg("user").setS(user);
 	    db_el.cfg("id").setS(path);
 	    try
@@ -400,7 +407,7 @@ string TBDS::genDBGet(const string &path, const string &oval, const string &user
 	}
     }
 
-    if(!bd_ok)
+    if( !bd_ok )
     {
 	//> Get from cache
 	if( SYS->present("BD") )
@@ -417,7 +424,7 @@ string TBDS::genDBGet(const string &path, const string &oval, const string &user
 	ResAlloc res(SYS->nodeRes(),false);
 	XMLNode *tgtN = TCntrNode::ctrId(&SYS->cfgRoot(),path,true);
 	if( tgtN ) rez = tgtN->text();
-	if( rFlg&TBDS::UseTranslate && !Mess->lang2CodeBase().empty() && Mess->lang2Code() != Mess->lang2CodeBase() )
+	if( rFlg&TBDS::UseTranslate )
 	{
 	    tgtN = TCntrNode::ctrId(&SYS->cfgRoot(),path+"_"+Mess->lang2Code(),true);
 	    if( tgtN ) rez = tgtN->text();
@@ -431,8 +438,7 @@ string TBDS::optDescr(  )
 {
     char buf[STR_BUF_LEN];
     snprintf(buf,sizeof(buf),_(
-	"========================= Subsystem \"DB\" options ========================\n"
-	"------------ Parameters of section <%s> in config file -----------\n\n"
+	"========================= Subsystem \"DB\" options ========================\n\n"
 	),nodePath().c_str());
 
     return(buf);
@@ -455,7 +461,7 @@ void TBDS::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -507,7 +513,7 @@ void TBDS::cntrCmdProc( XMLNode *opt )
     }
     //Process command to page
     string a_path = opt->attr("path");
-    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root",subId().c_str(),SEQ_RD) )	opt->setText(optDescr());
+    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root",subId().c_str(),SEC_RD) )	opt->setText(optDescr());
     else TSubSYS::cntrCmdProc(opt);
 }
 
@@ -552,24 +558,24 @@ void TTipBD::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/db/ful_db_del"  )
     {
-	if( ctrChkNode(opt,"get",0660,"root","BD",SEQ_RD) )	opt->setText(full_db_del?"1":"0");
-	if( ctrChkNode(opt,"set",0660,"root","BD",SEQ_WR) )	full_db_del = atoi(opt->text().c_str());
+	if( ctrChkNode(opt,"get",0660,"root","BD",SEC_RD) )	opt->setText(full_db_del?"1":"0");
+	if( ctrChkNode(opt,"set",0660,"root","BD",SEC_WR) )	full_db_del = atoi(opt->text().c_str());
     }
     else if( a_path == "/br/db_" || a_path == "/db/odb" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )
 	{
 	    vector<string> lst;
 	    list(lst);
 	    for( int i_l=0; i_l < lst.size(); i_l++)
 		opt->childAdd("el")->setAttr("id",lst[i_l])->setText(at(lst[i_l]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","BD",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","BD",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    open(vid); at(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root","BD",SEQ_WR) )	close(opt->attr("id"),true);
+	if( ctrChkNode(opt,"del",0664,"root","BD",SEC_WR) )	close(opt->attr("id"),true);
     }
     else TModule::cntrCmdProc(opt);
 }
@@ -604,12 +610,12 @@ TCntrNode &TBD::operator=( TCntrNode &node )
 	src_n->allowList(tbl_ls);
 	for( int i_l = 0; i_l < tbl_ls.size(); i_l++ )
 	{
-	    //-- Open source and destination tables --
+	    //> Open source and destination tables
 	    src_n->open(tbl_ls[i_l], false);
 	    open(tbl_ls[i_l], true);
-	    //-- Copy table --
+	    //> Copy table
 	    (TCntrNode&)at(tbl_ls[i_l]).at() = (TCntrNode&)src_n->at(tbl_ls[i_l]).at();
-	    //-- Close source and destination tables --
+	    //> Close source and destination tables
 	    src_n->close(tbl_ls[i_l]);
 	    close(tbl_ls[i_l]);
 	}
@@ -684,6 +690,28 @@ void TBD::save_( )
     SYS->db().at().dataSet(owner().owner().fullDB(),SYS->db().at().nodePath()+"DB/",*this);
 }
 
+TVariant TBD::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    if( iid == "SQLReq" && prms.size() >= 1 )
+    {
+	TArrayObj *rez = new TArrayObj();
+	try
+	{
+	    vector< vector<string> > rtbl;
+	    sqlReq( prms[0].getS(), &rtbl );
+	    for( int i_r = 0; i_r < rtbl.size(); i_r++ )
+	    {
+		TArrayObj *row = new TArrayObj();
+		for( int i_c = 0; i_c < rtbl[i_r].size(); i_c++ )
+		    row->propSet(TSYS::int2str(i_c),rtbl[i_r][i_c]);
+		rez->propSet(TSYS::int2str(i_r),row);
+	    }
+	}catch(...){ }
+	return rez;
+    }
+    return TCntrNode::objFuncCall(iid,prms,user);
+}
+
 void TBD::cntrCmdProc( XMLNode *opt )
 {
     //Get page info
@@ -716,6 +744,12 @@ void TBD::cntrCmdProc( XMLNode *opt )
 	if(ctrMkNode("area",opt,1,"/tbls",_("Tables"),0444))
 	    ctrMkNode("list",opt,-1,"/tbls/otbl",_("Opened tables"),0664,"root","BD",4,
 		"tp","br","s_com","add,del","br_pref","tbl_","help",_("Opened table list.\nAdding and deleting tables operations are really open and close tables operations."));
+	if( enableStat( ) && ctrMkNode("area",opt,-1,"/sql",_("SQL"),R_R___,"root","BD") )
+	{
+	    ctrMkNode("fld",opt,-1,"/sql/req",_("Request"),RWRW__,"root","BD",3,"tp","str","cols","100","rows","5");
+	    ctrMkNode("comm",opt,-1,"/sql/send",_("Send"),RWRW__,"root","BD");
+	    ctrMkNode("table",opt,-1,"/sql/tbl",_("Result"),R_R___,"root","BD");
+	}
 	return;
     }
 
@@ -723,25 +757,25 @@ void TBD::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/prm/st/st" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )	opt->setText(enableStat()?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )	atoi(opt->text().c_str())?enable():disable();
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )	opt->setText(enableStat()?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","BD",SEC_WR) )	atoi(opt->text().c_str())?enable():disable();
     }
     else if( a_path == "/prm/st/allow_tbls" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )
 	{
 	    vector<string> lst;
 	    allowList(lst);
 	    for( int i_l=0; i_l < lst.size(); i_l++)
 		opt->childAdd("el")->setText(lst[i_l]);
 	}
-	if( ctrChkNode(opt,"del",0664,"root","BD",SEQ_WR) )
+	if( ctrChkNode(opt,"del",0664,"root","BD",SEC_WR) )
 	{
 	    open( opt->text(), false );
 	    close( opt->text(), true );
 	}
     }
-    else if( a_path == "/prm/st/load" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )
+    else if( a_path == "/prm/st/load" && ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )
     {
 	SYS->setSelDB(owner().modId()+"."+id());
 	SYS->modifG();
@@ -751,41 +785,56 @@ void TBD::cntrCmdProc( XMLNode *opt )
     else if( a_path == "/prm/cfg/id" && ctrChkNode(opt) )		opt->setText(id());
     else if( a_path == "/prm/cfg/nm" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )	opt->setText(name());
-	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )	setName(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )	opt->setText(name());
+	if( ctrChkNode(opt,"set",0664,"root","BD",SEC_WR) )	setName(opt->text());
     }
     else if( a_path == "/prm/cfg/dscr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )	opt->setText(dscr());
-	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )	setDscr(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )	opt->setText(dscr());
+	if( ctrChkNode(opt,"set",0664,"root","BD",SEC_WR) )	setDscr(opt->text());
     }
     else if( a_path == "/prm/cfg/addr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )	opt->setText(addr());
-	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )	setAddr(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )	opt->setText(addr());
+	if( ctrChkNode(opt,"set",0664,"root","BD",SEC_WR) )	setAddr(opt->text());
     }
     else if( a_path == "/prm/cfg/codep" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )	opt->setText(codePage());
-	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )	setCodePage(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )	opt->setText(codePage());
+	if( ctrChkNode(opt,"set",0664,"root","BD",SEC_WR) )	setCodePage(opt->text());
     }
     else if( a_path == "/prm/cfg/toen" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )	opt->setText(toEnable()?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )	setToEnable( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )	opt->setText(toEnable()?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","BD",SEC_WR) )	setToEnable( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/br/tbl_" || a_path == "/tbls/otbl" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )
 	{
 	    vector<string> lst;
 	    list(lst);
 	    for( int i_l=0; i_l < lst.size(); i_l++)
 		opt->childAdd("el")->setText(lst[i_l]);
 	}
-	if( ctrChkNode(opt,"add",0664,"root","BD",SEQ_WR) )	open(opt->text(),true);
-	if( ctrChkNode(opt,"del",0664,"root","BD",SEQ_WR) )	close(opt->text());
+	if( ctrChkNode(opt,"add",0664,"root","BD",SEC_WR) )	open(opt->text(),true);
+	if( ctrChkNode(opt,"del",0664,"root","BD",SEC_WR) )	close(opt->text());
     }
+    else if( a_path == "/sql/req" )
+    {
+	if( ctrChkNode(opt,"get",RWRW__,"root","BD") )	opt->setText(userSQLReq);
+	if( ctrChkNode(opt,"set",RWRW__,"root","BD") )	userSQLReq = opt->text();
+    }
+    else if( a_path == "/sql/send" && enableStat( ) && ctrChkNode(opt,"set",RWRW__,"root","BD",SEC_WR) )
+	sqlReq(userSQLReq,&userSQLResTbl);
+    else if( a_path == "/sql/tbl" && ctrChkNode(opt,"get",R_R___,"root","BD",SEC_RD) )
+	for( int i_r = 0; i_r < userSQLResTbl.size(); i_r++ )
+	    for( int i_c = 0; i_c < userSQLResTbl[i_r].size(); i_c++ )
+	    {
+		if( i_r == 0 )
+		    ctrMkNode("list",opt,i_c,("/sql/tbl/"+userSQLResTbl[0][i_c]).c_str(),userSQLResTbl[0][i_c],R_R___,"root","BD",1,"tp","str");
+		else opt->childGet(i_c)->childAdd("el")->setText(userSQLResTbl[i_r][i_c]);
+	    }
     else TCntrNode::cntrCmdProc(opt);
 }
 
@@ -857,7 +906,7 @@ void TTable::cntrCmdProc( XMLNode *opt )
 	TConfig req;
 	string eid;
 	fieldStruct(req);
-	if( ctrChkNode(opt,"get",0664,"root","BD",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","BD",SEC_RD) )
 	    for( int i_r = 0; fieldSeek(i_r,req); i_r++ )
 	        for( int i_f = 0; i_f < req.elem().fldSize(); i_f++ )
 		{
@@ -866,7 +915,7 @@ void TTable::cntrCmdProc( XMLNode *opt )
 			ctrMkNode("list",opt,-1,("/prm/tbl/"+eid).c_str(),"",0664);
 		    opt->childGet(i_f)->childAdd("el")->setText(req.cfg(eid).getS());
 		}
-	if( ctrChkNode(opt,"add",0664,"root","BD",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","BD",SEC_WR) )
 	{
 	    for( int i_f = 0; i_f < req.elem().fldSize(); i_f++ )
 	    {
@@ -877,7 +926,7 @@ void TTable::cntrCmdProc( XMLNode *opt )
 	    req.cfgViewAll(false);
 	    fieldSet(req);
 	}
-	if( ctrChkNode(opt,"del",0664,"root","BD",SEQ_WR) )
+	if( ctrChkNode(opt,"del",0664,"root","BD",SEC_WR) )
 	{
 	    for( int i_f = 0; i_f < req.elem().fldSize(); i_f++ )
 		if( req.elem().fldAt(i_f).flg()&TCfg::Key )
@@ -887,7 +936,7 @@ void TTable::cntrCmdProc( XMLNode *opt )
 		}
 	    fieldDel(req);
 	}
-	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","BD",SEC_WR) )
 	{
 	    //> Get structure
 	    string  col = opt->attr("col");
