@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tcntrnode.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -31,6 +31,7 @@
 #include "xml.h"
 #include "tsys.h"
 #include "tmess.h"
+#include "tvariant.h"
 #include "tcntrnode.h"
 
 //*************************************************
@@ -240,7 +241,7 @@ AutoHD<TCntrNode> TCntrNode::nodeAt( const string &path, int lev, char sep, int
 			TSYS::pathLev(path,lev,true,&off);
     if( s_br.empty() )
     {
-	if( nodeMode() == Disable ) throw TError(nodePath().c_str(),"Node is disabled!");
+	if( nodeMode() == Disable ) throw TError(nodePath().c_str(),_("Node is disabled!"));
 	return this;
     }
     ResAlloc res(hd_res,false);
@@ -249,6 +250,7 @@ AutoHD<TCntrNode> TCntrNode::nodeAt( const string &path, int lev, char sep, int
 	    return chldAt(i_g,s_br.substr((*chGrp)[i_g].id.size())).at().nodeAt(path,0,sep,off);
     //> Go to default group
     if( chGrp )	return chldAt(0,s_br).at().nodeAt(path,0,sep,off);
+    throw TError(nodePath().c_str(),_("Node '%s' no present!"),s_br.c_str());
 }
 
 void TCntrNode::nodeDel( const string &path, char sep, int flag, bool shDel )
@@ -264,20 +266,20 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
 {
     if( src == dst )	return;
 
-    //- Attach to source node -
+    //> Attach to source node
     AutoHD<TCntrNode> src_n = SYS->nodeAt(src);
 
-    //- Parse destination node path -
+    //> Parse destination node path
     string d_elp, d_el, t_el;
     int n_del = 0;
     for( int off = 0; !(t_el=TSYS::pathLev(dst,0,true,&off)).empty(); n_del++ )
     { if( n_del ) d_elp += ("/"+d_el); d_el = t_el; }
     if( !n_del ) throw TError(SYS->nodePath().c_str(),_("Copy from '%s' to '%s' is impossible"),src.c_str(),dst.c_str());
 
-    //- Connect to destination containers node -
+    //> Connect to destination containers node
     AutoHD<TCntrNode> dst_n = SYS->nodeAt(d_elp);
 
-    //- Get allow branches' containers and find want group -
+    //> Get allow branches' containers and find want group
     XMLNode br_req("info");
     br_req.setAttr("user",user)->setAttr("path","/%2fbr");
     dst_n.at().cntrCmd(&br_req);
@@ -287,7 +289,7 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
     int i_b;
     for( i_b = 0; i_b < branch->childSize(); i_b++ )
 	if( branch->childGet(i_b)->attr("id") == d_el.substr(0,branch->childGet(i_b)->attr("id").size()) && 
-		atoi(branch->childGet(i_b)->attr("acs").c_str())&SEQ_WR )
+		atoi(branch->childGet(i_b)->attr("acs").c_str())&SEC_WR )
 	    break;
     if( i_b >= branch->childSize() )
         throw TError(SYS->nodePath().c_str(),_("Destination node doesn't have necessary branche."));
@@ -297,7 +299,7 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
     i_b = dst_n.at().grpId(n_grp);
     if( i_b < 0 ) throw TError(SYS->nodePath().c_str(),_("Destination node doesn't have necessary branche."));
 
-    //- Connect or create new destination node -
+    //> Connect or create new destination node
     if( !dst_n.at().chldPresent( i_b, d_el ) )
     {
 	br_req.clear()->setName("add")->setAttr("user",user)->setAttr("path","/%2fbr%2f"+n_grp);
@@ -307,7 +309,7 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
 	if( atoi(br_req.attr("rez").c_str()) )	throw TError(br_req.attr("mcat").c_str(),br_req.text().c_str());
     }
 
-    //- Same copy call -
+    //> Same copy call
     dst_n.at().chldAt( i_b, d_el ).at() = src_n.at();
 }
 
@@ -547,7 +549,7 @@ void TCntrNode::modifGClr( )
 
 void TCntrNode::load( )
 {
-    //- Self load -
+    //> Self load
     if( isModify(Self)&Self )
 	try
 	{
@@ -559,7 +561,7 @@ void TCntrNode::load( )
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 	    mess_err(nodePath().c_str(),_("Loading node error."));
 	}
-    //- Childs load process -
+    //> Childs load process
     if( isModify(Child)&Child )
     {
 	ResAlloc res( hd_res, false );
@@ -572,7 +574,7 @@ void TCntrNode::load( )
 
 void TCntrNode::save( )
 {
-    //- Self load -
+    //> Self load
     if( isModify(Self)&Self )
 	try{ save_(); }
 	catch(TError err)
@@ -580,7 +582,7 @@ void TCntrNode::save( )
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 	    mess_err(nodePath().c_str(),_("Saving node error."));
 	}
-    //- Childs load process -
+    //> Childs load process
     if( isModify(Child)&Child )
     {
 	ResAlloc res( hd_res, false );
@@ -619,6 +621,15 @@ void TCntrNode::AHDDisConnect()
     pthread_mutex_unlock(&connM);
 }
 
+TVariant TCntrNode::objPropGet( const string &id )			{ return TVariant(); }
+
+void TCntrNode::objPropSet( const string &id, TVariant val )		{ }
+
+TVariant TCntrNode::objFuncCall( const string &id, vector<TVariant> &prms, const string &user )
+{
+    throw TError(nodePath().c_str(),_("Function '%s' error or not enough parameters."),id.c_str());
+}
+
 XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const char *path, const string &dscr,
     int perm, const char *user, const char *grp, int n_attr, ... )
 {
@@ -626,7 +637,7 @@ XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const cha
     string req = nd->attr("path");
     string reqt, reqt1;
 
-    //- Check displaing node -
+    //> Check displaing node
     int itbr = 0;
     for( int i_off = 0, i_off1 = 0; (reqt=TSYS::pathLev(req,0,true,&i_off)).size(); woff=i_off )
 	if( reqt != (reqt1=TSYS::pathLev(path,0,true,&i_off1)) )
@@ -636,9 +647,9 @@ XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const cha
 	    break;
 	}
 
-    //- Check permission -
-    char n_acs = SYS->security().at().access(nd->attr("user"),SEQ_RD|SEQ_WR|SEQ_XT,user,grp,perm);
-    if( !(n_acs&SEQ_RD) ) return NULL;
+    //> Check permission
+    char n_acs = SYS->security().at().access(nd->attr("user"),SEC_RD|SEC_WR|SEC_XT,user,grp,perm);
+    if( !(n_acs&SEC_RD) ) return NULL;
     if( itbr )	return nd;
 
     XMLNode *obj = nd;
@@ -649,7 +660,7 @@ XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const cha
 	nd->setAttr("rez","0");
     }
 
-    //- Go to element -
+    //> Go to element
     for( ;(reqt=TSYS::pathLev(path,0,true,&woff)).size(); reqt1=reqt )
     {
 	XMLNode *obj1 = obj->childGet("id",reqt,true);
@@ -661,7 +672,7 @@ XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const cha
     }
     obj->setName(n_nd)->setAttr("id",reqt1)->setAttr("dscr",dscr)->setAttr("acs",TSYS::int2str(n_acs));
 
-    //- Get addon attributes -
+    //> Get addon attributes
     if( n_attr )
     {
 	char *atr_id, *atr_vl;
@@ -679,6 +690,23 @@ XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const cha
     return obj;
 }
 
+bool TCntrNode::ctrRemoveNode( XMLNode *nd, const char *path )
+{
+    XMLNode *obj = nd;
+    if( obj->name() == "info" )	obj = nd->childGet(0,true);
+    if( !obj )	return false;
+
+    //> Find element
+    string reqt;
+    for( int woff = 0; (reqt=TSYS::pathLev(path,0,true,&woff)).size(); )
+    {
+	XMLNode *obj1 = obj->childGet("id",reqt,true);
+	if( !obj1 ) return false;
+	obj = obj1;
+    }
+    obj->parent()->childDel(obj);
+}
+
 bool TCntrNode::ctrChkNode( XMLNode *nd, const char *cmd, int perm, const char *user, const char *grp, char mode, const char *warn )
 {
     if( nd->name() != cmd ) return false;
@@ -702,14 +730,14 @@ void TCntrNode::cntrCmdProc( XMLNode *opt )
 	//>> Get node modify flag
 	if( ctrChkNode(opt,"modify",R_R_R_) )	opt->setText(isModify(TCntrNode::All)?"1":"0");
 	//>> Do load node
-	else if( ctrChkNode(opt,"load",RWRWRW,"root","root",SEQ_WR) )	load( );
+	else if( ctrChkNode(opt,"load",RWRWRW,"root","root",SEC_WR) )	load( );
 	//>> Do save node
-	else if( ctrChkNode(opt,"save",RWRWRW,"root","root",SEQ_WR) )	save( );
+	else if( ctrChkNode(opt,"save",RWRWRW,"root","root",SEC_WR) )	save( );
 	//>> Do copy node
-	else if( ctrChkNode(opt,"copy",RWRWRW,"root","root",SEQ_WR) )
+	else if( ctrChkNode(opt,"copy",RWRWRW,"root","root",SEC_WR) )
 	    nodeCopy(opt->attr("src"),opt->attr("dst"),opt->attr("user"));
 	//>> Request node childs parameters
-	else if( ctrChkNode(opt,"chlds",R_R_R_,"root","root",SEQ_RD) )
+	else if( ctrChkNode(opt,"chlds",R_R_R_,"root","root",SEC_RD) )
 	{
 	    string chGrp = opt->attr("grp");
 	    bool icoCheck = atoi(opt->attr("icoCheck").c_str());
