@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tconfig.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -37,6 +37,7 @@ using std::map;
 //*************************************************
 class TConfig;
 
+#pragma pack(push,1)
 class TCfg
 {
     public:
@@ -107,6 +108,7 @@ class TCfg
 	TFld	*mFld;
 	TConfig	&mOwner;
 };
+#pragma pack(pop)
 
 //*************************************************
 //* TConfig                                       *
@@ -139,8 +141,8 @@ class TConfig: public TValElem
         void cntrCmdProc( XMLNode *fld, const string &elem,
 		const string &user = "root", const string &grp = "root", int perm = 0664 );
 
-	string lang2Code( )	{ return mLang2Code; }
-	void setLang2Code( const string &vl )	{ mLang2Code = vl; }
+	bool noTransl( )	{ return mNoTransl; }
+	void setNoTransl( bool vl )		{ mNoTransl = vl; }
 
     protected:
 	//Methods
@@ -155,7 +157,7 @@ class TConfig: public TValElem
 	TCfgMap		value;
 	TElem		*m_elem;
 	char		single		: 1;
-	string		mLang2Code;
+	char		mNoTransl	: 1;
 };
 
 #endif // TCONFIG_H
