@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tcontroller.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -90,7 +90,7 @@ class TController : public TCntrNode, public TConfig
 	void setRedntMode( Redundant vl );
 	string redntRun( );
 	void setRedntRun( const string &vl );
-	virtual void redntDataUpdate( bool firstArchiveSync = false );
+	virtual void redntDataUpdate( );
 
 	TTipDAQ &owner( );
 
@@ -129,10 +129,10 @@ class TController : public TCntrNode, public TConfig
 	bool	&mAStart;
 
 	string	mDB;
+	ResString mRedntSt;
 
 	unsigned mPrm		: 2;
 	unsigned mRedntUse	: 1;
-	unsigned mRedntFirst	: 1;
 };
 
 
