@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tdaqs.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -122,7 +122,7 @@ void TDAQS::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -285,16 +285,7 @@ void TDAQS::subStart(  )
     if( !SYS->archive().at().subStartStat( ) ) SYS->archive().at().subStart( );
 
     //> Redundant task start
-    if( !prcStRd )
-    {
-	pthread_attr_t pthr_attr;
-	pthread_attr_init(&pthr_attr);
-	pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
-	pthread_create(&mRdPthr,&pthr_attr,TDAQS::RdTask,this);
-	pthread_attr_destroy(&pthr_attr);
-	if( TSYS::eventWait(prcStRd,true,nodePath()+"redundant_task_start",5) )
-	    throw TError(nodePath().c_str(),_("Redundant process task is not started!"));
-    }
+    if( !prcStRd ) SYS->taskCreate( nodePath('.',true)+".redundant", 5, TDAQS::RdTask, this, &prcStRd );
 
     //> Controllers start
     TSubSYS::subStart( );
@@ -306,14 +297,7 @@ void TDAQS::subStop( )
     mess_debug(nodePath().c_str(),_("Stop subsystem."));
 #endif
 
-    if( prcStRd )
-    {
-	endrunRd = true;
-	pthread_kill( mRdPthr, SIGALRM );
-	if( TSYS::eventWait(prcStRd,false,nodePath()+"redundant_task_stop",10) )
-	    throw TError(nodePath().c_str(),_("Redundant process task is not stopped!"));
-	pthread_join( mRdPthr, NULL );
-    }
+    if( prcStRd ) SYS->taskDestroy( nodePath('.',true)+".redundant", &prcStRd, &endrunRd );
 
     vector<string> m_l;
 
@@ -440,7 +424,7 @@ void *TDAQS::RdTask( void *param )
 		req.clear()->setAttr("path","/"+sit->first+"/DAQ/%2fserv%2fredundant");
 		try
 		{
-		    if( SYS->transport().at().cntrIfCmd(req,"DAQRedndt") ) continue;
+		    if( SYS->transport().at().cntrIfCmd(req,"DAQRedundant") ) continue;
 		    sit->second.lev = atoi(req.attr("StLevel").c_str());
 		    sit->second.actCntr.clear();
 		    for( int i_c = 0; i_c < req.childSize(); i_c++ )
@@ -490,8 +474,8 @@ void *TDAQS::RdTask( void *param )
 		}
 		else if( cntr.at().redntRun( ) == "<optimal>" )
 		{
-		    vector<string> cls;
-		    daq.rdActCntrList(cls,true);
+		    vector<string> cls_lc;
+		    daq.rdActCntrList(cls_lc,true);
 		    bool remPresent = false;
 		    for( sit = daq.mSt.begin(); sit != daq.mSt.end(); sit++ )
 			if( sit->second.isLive && (cit=sit->second.actCntr.find(cntr.at().workId())) != sit->second.actCntr.end() )
@@ -500,9 +484,9 @@ void *TDAQS::RdTask( void *param )
 			    int aCntr = 0;
 			    for( map<string,bool>::iterator scit = sit->second.actCntr.begin(); scit != sit->second.actCntr.end(); scit++ )
 				if( scit->second ) aCntr++;
-			    if( (cls.size()-aCntr) > 1 && cit->second ) break;
+			    if( ((int)cls_lc.size()-aCntr) >= 0 && cit->second ) break;
 			}
-		    cntr.at().setRedntUse( sit != daq.mSt.end() || remPresent );
+		    cntr.at().setRedntUse( sit != daq.mSt.end() );
 		}
 		else
 		{
@@ -550,7 +534,7 @@ void TDAQS::cntrCmdProc( XMLNode *opt )
     //> Service commands process
     if( a_path == "/serv/redundant" )	//Redundant service requests
     {
-	if( ctrChkNode(opt,"st",RWRWR_,"root","DAQ",SEQ_RD) )
+	if( ctrChkNode(opt,"st",RWRWR_,"root","DAQ",SEC_RD) )
 	{
 	    opt->setAttr("StLevel",TSYS::int2str(rdStLevel()));
 	    map<string,bool> cntrLs;
@@ -593,8 +577,8 @@ void TDAQS::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/redund/cntr/nm",_("Name"),R_R_R_,"root","DAQ",1,"tp","str");
 		ctrMkNode("list",opt,-1,"/redund/cntr/start",_("Started"),RWRWR_,"root","DAQ",1,"tp","bool");
 		ctrMkNode("list",opt,-1,"/redund/cntr/rdndt",_("Redundant"),RWRWR_,"root","DAQ",4,"tp","dec","dest","select",
-		    "sel_id",(TSYS::int2str(TController::Off)+";"+TSYS::int2str(TController::Asymmetric)+";"+TSYS::int2str(TController::Symmetric)).c_str(),
-		    "sel_list",_("Off;Asymmetric;Symmetric"));
+		    "sel_id",(TSYS::int2str(TController::Off)+";"+TSYS::int2str(TController::Asymmetric)/*+";"+TSYS::int2str(TController::Symmetric)*/).c_str(),
+		    "sel_list",_("Off;Asymmetric"/*;Symmetric"*/));
 		ctrMkNode("list",opt,-1,"/redund/cntr/prefRun",_("Preferable run"),RWRWR_,"root","DAQ",4,"tp","str","idm","1","dest","select","select","/redund/lsMode");
 		ctrMkNode("list",opt,-1,"/redund/cntr/remoted",_("Remoted"),R_R_R_,"root","DAQ",1,"tp","bool");
 	    }
@@ -608,44 +592,44 @@ void TDAQS::cntrCmdProc( XMLNode *opt )
     //> Process command to page
     if( a_path == "/br/tmplb_" || a_path == "/tpllibs/lb" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )
 	{
 	    vector<string> lst;
 	    tmplLibList(lst);
 	    for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",lst[i_a])->setText(tmplLibAt(lst[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",RWRWR_,"root","DAQ",SEQ_WR) )
-	    tmplLibReg(new TPrmTmplLib(TSYS::strEncode(opt->attr("id"),TSYS::ID).c_str(),opt->text().c_str(),"*.*"));
-	if( ctrChkNode(opt,"del",RWRWR_,"root","DAQ",SEQ_WR) )
+	if( ctrChkNode(opt,"add",RWRWR_,"root","DAQ",SEC_WR) )
+	    tmplLibReg(new TPrmTmplLib(TSYS::strEncode(opt->attr("id"),TSYS::oscdID).c_str(),opt->text().c_str(),"*.*"));
+	if( ctrChkNode(opt,"del",RWRWR_,"root","DAQ",SEC_WR) )
 	    tmplLibUnreg(opt->attr("id"),1);
     }
     else if( a_path == "/redund/status" && ctrChkNode(opt,"get",R_R_R_,"root","DAQ") )
 	opt->setText(TSYS::strMess(_("Process time %.6g ms."),mRdPrcTm));
     else if( a_path == "/redund/statLev" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )	opt->setText(TSYS::int2str(rdStLevel()));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )	setRdStLevel(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )	opt->setText(TSYS::int2str(rdStLevel()));
+	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )	setRdStLevel(atoi(opt->text().c_str()));
     }
     else if( a_path == "/redund/tskPer" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )	opt->setText(TSYS::int2str(rdTaskPer()));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )	setRdTaskPer(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )	opt->setText(TSYS::int2str(rdTaskPer()));
+	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )	setRdTaskPer(atoi(opt->text().c_str()));
     }
     else if( a_path == "/redund/restConn" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )	opt->setText(TSYS::int2str(rdRestConnTm()));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )	setRdRestConnTm(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )	opt->setText(TSYS::int2str(rdRestConnTm()));
+	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )	setRdRestConnTm(atoi(opt->text().c_str()));
     }
     else if( a_path == "/redund/restDtTm" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )	opt->setText(TSYS::real2str(rdRestDtTm()));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )	setRdRestDtTm(atof(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )	opt->setText(TSYS::real2str(rdRestDtTm()));
+	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )	setRdRestDtTm(atof(opt->text().c_str()));
     }
     else if( a_path == "/redund/sts" )
     {
 	ResAlloc res(nodeRes(),true);
-	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )
 	{
 	    XMLNode *n_st	= ctrMkNode("list",opt,-1,"/redund/sts/st","",RWRWR_,"root","DAQ");
 	    XMLNode *n_name	= ctrMkNode("list",opt,-1,"/redund/sts/name","",R_R_R_,"root","DAQ");
@@ -665,17 +649,18 @@ void TDAQS::cntrCmdProc( XMLNode *opt )
 		{
 		    string cls;
 		    for( map<string,bool>::iterator cit = sit->second.actCntr.begin(); cit != sit->second.actCntr.end(); cit++ )
-			cls += cit->first+(cit->second?" (+) ;":" ;");
+			cls += cit->first+(cit->second?" (+); ":"; ");
 		    n_run->childAdd("el")->setText(cls);
 		}
 	    }
 	}
-	if( ctrChkNode(opt,"add",RWRWR_,"root","DAQ",SEQ_WR) )	mSt["<newStat>"] = SStat();
-	if( ctrChkNode(opt,"del",RWRWR_,"root","DAQ",SEQ_WR) )	mSt.erase(opt->attr("key_st"));
-	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) && opt->attr("col") == "st" )
+	if( ctrChkNode(opt,"add",RWRWR_,"root","DAQ",SEC_WR) )	{ mSt["<newStat>"] = SStat(); modif(); }
+	if( ctrChkNode(opt,"del",RWRWR_,"root","DAQ",SEC_WR) )	{ mSt.erase(opt->attr("key_st")); modif(); }
+	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) && opt->attr("col") == "st" )
 	{
 	    mSt.erase(opt->attr("key_st"));
 	    mSt[opt->text()] = SStat();
+	    modif();
 	}
     }
     else if( a_path == "/redund/lsSt" && ctrChkNode(opt) )
@@ -685,14 +670,14 @@ void TDAQS::cntrCmdProc( XMLNode *opt )
 	for( int i_h = 0; i_h < hls.size(); i_h++ )
 	    opt->childAdd("el")->setText(hls[i_h]);
     }
-    else if( a_path == "/redund/hostLnk" && ctrChkNode(opt,"get",0660,"root","Transport",SEQ_RD) )
+    else if( a_path == "/redund/hostLnk" && ctrChkNode(opt,"get",0660,"root","Transport",SEC_RD) )
     {
 	SYS->transport().at().setSysHost(true);
 	opt->setText("/Transport");
     }
     else if( a_path == "/redund/cntr" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+	if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEC_RD) )
 	{
 	    XMLNode *n_id	= ctrMkNode("list",opt,-1,"/redund/cntr/id","",R_R_R_,"root","DAQ");
 	    XMLNode *n_nm	= ctrMkNode("list",opt,-1,"/redund/cntr/nm","",R_R_R_,"root","DAQ");
@@ -720,7 +705,7 @@ void TDAQS::cntrCmdProc( XMLNode *opt )
 		}
 	    }
 	}
-	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+	if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEC_WR) )
 	{
 	    string col = opt->attr("col");
 	    AutoHD<TController> cntr  = at(TSYS::strSepParse(opt->attr("key_id"),0,'.')).at().
