@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tfunction.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -41,10 +41,8 @@ TFunction::~TFunction( )
 
 TFunction &TFunction::operator=( TFunction &func )
 {
-    if( mId.empty() )	mId = func.id();
     //> Copy IO
     //>> Clear no present IO
-
     for( int i_io = 0; i_io < ioSize(); )
 	if( func.ioId(io(i_io)->id()) < 0 )	ioDel(i_io);
 	else i_io++;
@@ -56,9 +54,11 @@ TFunction &TFunction::operator=( TFunction &func )
 	    dst_io = ioIns( new IO( func.io(i_io)->id().c_str(), func.io(i_io)->name().c_str(), func.io(i_io)->type(), func.io(i_io)->flg(),
 		func.io(i_io)->def().c_str(), func.io(i_io)->hide(), func.io(i_io)->rez().c_str() ), i_io );
 	else *io(dst_io) = *func.io(i_io);
-	if( dst_io != i_io ) ioMove(dst_io,i_io);
+	if( dst_io != i_io && !use() ) ioMove(dst_io,i_io);
     }
 
+    if( mId.empty() )	mId = func.id();
+
     return *this;
 }
 
@@ -74,6 +74,11 @@ void TFunction::preDisable( int flag )
     }
 }
 
+void TFunction::setId( const string &vl )
+{
+    if( !nodePrev(true) ) mId = vl;
+}
+
 int TFunction::ioSize( )
 {
     return mIO.size( );
@@ -148,7 +153,7 @@ void TFunction::preIOCfgChange()
     if( blk_lst.size() )
 	throw TError(nodePath().c_str(),_("Change is not permited while function is used: %s"),blk_lst.c_str());
 
-    for(unsigned i=0; i < used.size(); i++)
+    for( unsigned i=0; i < used.size(); i++ )
 	used[i]->preIOCfgChange();
 }
 
@@ -161,8 +166,8 @@ void TFunction::postIOCfgChange()
 void TFunction::valAtt( TValFunc *vfnc )
 {
     ResAlloc res(f_res,true);
-    for(unsigned i=0; i < used.size() ;i++)
-	if(used[i] == vfnc)
+    for( unsigned i=0; i < used.size(); i++ )
+	if( used[i] == vfnc )
 	    throw TError(nodePath().c_str(),_("Value <%s> is already attached!"),vfnc->vfName().c_str());
     used.push_back(vfnc);
 }
@@ -181,7 +186,7 @@ void TFunction::valDet( TValFunc *vfnc )
 
 void TFunction::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TCntrNode::cntrCmdProc(opt);
@@ -189,7 +194,10 @@ void TFunction::cntrCmdProc( XMLNode *opt )
 	if(ctrMkNode("area",opt,-1,"/func",_("Function")))
 	{
 	    if(ctrMkNode("area",opt,-1,"/func/st",_("State")))
+	    {
 		ctrMkNode("fld",opt,-1,"/func/st/st",_("Accessing"),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/func/st/use",_("Used"),R_R_R_,"root","root",1,"tp","dec");
+	    }
 	    if(ctrMkNode("area",opt,-1,"/func/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/func/cfg/id",_("Id"),0444,"root","root",1,"tp","str");
@@ -210,7 +218,7 @@ void TFunction::cntrCmdProc( XMLNode *opt )
 	if(ctrMkNode("area",opt,-1,"/exec",_("Execute")))
 	{
 	    ctrMkNode("fld",opt,-1,"/exec/en",_("Enable"),0660,"root","root",1,"tp","bool");
-	    //-- Add test form --
+	    //>> Add test form
 	    if( mTVal )
 	    {
 		if(ctrMkNode("area",opt,-1,"/exec/io",_("IO")))
@@ -227,12 +235,13 @@ void TFunction::cntrCmdProc( XMLNode *opt )
 				    if( io(i_io)->flg( )&IO::FullText ) nd->setAttr("cols","100")->setAttr("rows","4");
 				    break;
 				case IO::Integer:	nd->setAttr("tp","dec");	break;
-				case IO::Real:	nd->setAttr("tp","real");		break;
+				case IO::Real:		nd->setAttr("tp","real");	break;
 				case IO::Boolean:	nd->setAttr("tp","bool");	break;
+				case IO::Object:	nd->setAttr("tp","str")->setAttr("cols","100")->setAttr("rows","4");	break;
 			    }
 			}
 		    }
-		//-- Add Calc button and Calc time --
+		//>> Add Calc button and Calc time
 		ctrMkNode("fld",opt,-1,"/exec/n_clc",_("Number calcs"),0664,"root","root",1,"tp","dec");
 		ctrMkNode("fld",opt,-1,"/exec/tm",_("Calc time (mks)"),0444,"root","root",1,"tp","real");
 		ctrMkNode("comm",opt,-1,"/exec/calc",_("Calc"),0666);
@@ -241,24 +250,25 @@ void TFunction::cntrCmdProc( XMLNode *opt )
         return;
     }
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/func/st/st" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(run_st?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setStart(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(run_st?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setStart(atoi(opt->text().c_str()));
     }
+    else if( a_path == "/func/st/use" && ctrChkNode(opt) )	{ ResAlloc res(f_res,false); opt->setText(TSYS::int2str(used.size())); }
     else if( a_path == "/func/cfg/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/func/cfg/name" && ctrChkNode(opt) )	opt->setText(name());
     else if( a_path == "/func/cfg/descr" && ctrChkNode(opt) )	opt->setText(descr());
-    else if( a_path == "/io/io" && ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )
+    else if( a_path == "/io/io" && ctrChkNode(opt,"get",0440,"root","root",SEC_RD) )
     {
 	XMLNode *n_id	= ctrMkNode("list",opt,-1,"/io/io/0","");
-	XMLNode *n_nm  	= ctrMkNode("list",opt,-1,"/io/io/1","");
+	XMLNode *n_nm	= ctrMkNode("list",opt,-1,"/io/io/1","");
 	XMLNode *n_type	= ctrMkNode("list",opt,-1,"/io/io/2","");
-	XMLNode *n_mode = ctrMkNode("list",opt,-1,"/io/io/3","");
-	XMLNode *n_hide = ctrMkNode("list",opt,-1,"/io/io/4","");
-	XMLNode *n_def 	= ctrMkNode("list",opt,-1,"/io/io/5","");
+	XMLNode *n_mode	= ctrMkNode("list",opt,-1,"/io/io/3","");
+	XMLNode *n_hide	= ctrMkNode("list",opt,-1,"/io/io/4","");
+	XMLNode *n_def	= ctrMkNode("list",opt,-1,"/io/io/5","");
 	//XMLNode *n_vect	= ctrId(opt,"6");
 	for( int i_io = 0; i_io < ioSize(); i_io++ )
 	{
@@ -273,6 +283,7 @@ void TFunction::cntrCmdProc( XMLNode *opt )
 		    case IO::Integer:	tmp_str = _("Integer");	break;
 		    case IO::Real:	tmp_str = _("Real");	break;
 		    case IO::Boolean:	tmp_str = _("Bool");	break;
+		    case IO::Object:	tmp_str = _("Object");	break;
 		}
 		n_type->childAdd("el")->setText(tmp_str);
 	    }
@@ -289,8 +300,8 @@ void TFunction::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/exec/en" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText(mTVal?"1":"0");
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"get",0660,"root","root",SEC_RD) )	opt->setText(mTVal?"1":"0");
+	if( ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )
 	{
 	    bool to_en_exec = atoi(opt->text().c_str());
 	    if( to_en_exec && !mTVal )	{ mTVal = new TValFunc(id()+"_exec",this); mTVal->setDimens(true); }
@@ -299,8 +310,8 @@ void TFunction::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/exec/n_clc" && mTVal )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"ntCalc","10",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	TBDS::genDBSet(nodePath()+"ntCalc",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"ntCalc","10",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	TBDS::genDBSet(nodePath()+"ntCalc",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/exec/tm" && mTVal && ctrChkNode(opt) )opt->setText(TSYS::real2str(mTVal->calcTm(),6));
     else if( a_path.substr(0,8) == "/exec/io" && mTVal )
@@ -309,23 +320,23 @@ void TFunction::cntrCmdProc( XMLNode *opt )
 	for( int i_io = 0; i_io < mIO.size(); i_io++ )
 	    if( io_id == io(i_io)->id() )
 	    {
-		if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+		if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 		    opt->setText( (mTVal->ioType(i_io)==IO::Real) ? TSYS::real2str(mTVal->getR(i_io),6) : mTVal->getS(i_io) );
-		if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+		if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 		    mTVal->setS(i_io,opt->text());
 		break;
 	    }
     }
-    else if( a_path == "/exec/calc" && mTVal && ctrChkNode(opt,"set",0666,"root","root",SEQ_WR) )
+    else if( a_path == "/exec/calc" && mTVal && ctrChkNode(opt,"set",0666,"root","root",SEC_WR) )
     {
-        double c_rez = 0;
+	double c_rez = 0;
 	int n_tcalc = atoi(TBDS::genDBGet(nodePath()+"ntCalc","10",opt->attr("user")).c_str());
 	for(int i_c = 0; i_c < n_tcalc; i_c++ )
 	{
 	    mTVal->calc(opt->attr("user"));
 	    c_rez += mTVal->calcTm();
 	}
-        mTVal->setCalcTm(c_rez);
+	mTVal->setCalcTm(c_rez);
     }
     else TCntrNode::cntrCmdProc(opt);
 }
@@ -438,11 +449,16 @@ void TValFunc::setFunc( TFunction *ifunc, bool att_det )
 	{
 	    SVl val;
 	    val.tp = mFunc->io(i_vl)->type();
-	    if( val.tp == IO::String ) 		val.val.s = new ResString(mFunc->io(i_vl)->def());
-	    else if( val.tp == IO::Integer )	val.val.i = atoi(mFunc->io(i_vl)->def().c_str());
-	    else if( val.tp == IO::Real ) 	val.val.r = atof(mFunc->io(i_vl)->def().c_str());
-	    else if( val.tp == IO::Boolean )	val.val.b = atoi(mFunc->io(i_vl)->def().c_str());
-	    m_val.push_back(val);
+	    val.mdf = false;
+	    switch( val.tp )
+	    {
+		case IO::String:	val.val.s = new ResString(mFunc->io(i_vl)->def());	break;
+		case IO::Integer:	val.val.i = atoi(mFunc->io(i_vl)->def().c_str());	break;
+		case IO::Real:		val.val.r = atof(mFunc->io(i_vl)->def().c_str());	break;
+		case IO::Boolean:	val.val.b = atoi(mFunc->io(i_vl)->def().c_str());	break;
+		case IO::Object:	val.val.o = new TVarObj(); val.val.o->connect();	break;
+	    }
+	    mVal.push_back(val);
 	}
     }
 }
@@ -451,9 +467,15 @@ void TValFunc::funcDisConnect( bool det )
 {
     if( mFunc )
     {
-	for( int i_vl = 0; i_vl < m_val.size(); i_vl++ )
-	    if( m_val[i_vl].tp == IO::String )		delete m_val[i_vl].val.s;
-	m_val.clear();
+	for( int i_vl = 0; i_vl < mVal.size(); i_vl++ )
+	    switch( mVal[i_vl].tp )
+	    {
+		case IO::String:	delete mVal[i_vl].val.s;	break;
+		case IO::Object:
+		    if( mVal[i_vl].val.o && !mVal[i_vl].val.o->disconnect() ) delete mVal[i_vl].val.o;
+		    break;
+	    }
+	mVal.clear();
 	if(det)
 	{
 	    mFunc->valDet(this);
@@ -483,105 +505,134 @@ int TValFunc::ioSize( )
 
 string TValFunc::getS( unsigned id )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
-    switch(m_val[id].tp)
+    if( id >= mVal.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
+    switch(mVal[id].tp)
     {
-	case IO::Integer:	return (m_val[id].val.i!=EVAL_INT) ? TSYS::int2str(m_val[id].val.i) : EVAL_STR;
-	case IO::Real:		return (m_val[id].val.r!=EVAL_REAL) ? TSYS::real2str(m_val[id].val.r) : EVAL_STR;
-	case IO::Boolean:	return (m_val[id].val.b!=EVAL_BOOL) ? TSYS::int2str((bool)m_val[id].val.b) : EVAL_STR;
-	case IO::String:	return m_val[id].val.s->getVal();
+	case IO::Integer:	return (mVal[id].val.i!=EVAL_INT) ? TSYS::int2str(mVal[id].val.i) : EVAL_STR;
+	case IO::Real:		return (mVal[id].val.r!=EVAL_REAL) ? TSYS::real2str(mVal[id].val.r) : EVAL_STR;
+	case IO::Boolean:	return (mVal[id].val.b!=EVAL_BOOL) ? TSYS::int2str((bool)mVal[id].val.b) : EVAL_STR;
+	case IO::String:	return mVal[id].val.s->getVal();
+	case IO::Object:	return mVal[id].val.o->getStrXML();
     }
     return "";
 }
 
 int TValFunc::getI( unsigned id )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
-    switch(m_val[id].tp)
+    if( id >= mVal.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
+    switch( mVal[id].tp )
     {
-	case IO::String:	return (m_val[id].val.s->getVal()!=EVAL_STR) ? atoi(m_val[id].val.s->getVal().c_str()) : EVAL_INT;
-	case IO::Real:		return (m_val[id].val.r!=EVAL_REAL) ? (int)m_val[id].val.r : EVAL_INT;
-	case IO::Boolean:	return (m_val[id].val.b!=EVAL_BOOL) ? (bool)m_val[id].val.b : EVAL_INT;
-	case IO::Integer:	return m_val[id].val.i;
+	case IO::String:	return (mVal[id].val.s->getVal()!=EVAL_STR) ? atoi(mVal[id].val.s->getVal().c_str()) : EVAL_INT;
+	case IO::Real:		return (mVal[id].val.r!=EVAL_REAL) ? (int)mVal[id].val.r : EVAL_INT;
+	case IO::Boolean:	return (mVal[id].val.b!=EVAL_BOOL) ? (bool)mVal[id].val.b : EVAL_INT;
+	case IO::Integer:	return mVal[id].val.i;
     }
     return 0;
 }
 
 double TValFunc::getR( unsigned id )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
-    switch(m_val[id].tp)
+    if( id >= mVal.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
+    switch( mVal[id].tp )
     {
-	case IO::String:	return (m_val[id].val.s->getVal()!=EVAL_STR) ? atof(m_val[id].val.s->getVal().c_str()) : EVAL_REAL;
-	case IO::Integer:	return (m_val[id].val.i!=EVAL_INT) ? m_val[id].val.i : EVAL_REAL;
-	case IO::Boolean:	return (m_val[id].val.b!=EVAL_BOOL) ? (bool)m_val[id].val.b : EVAL_REAL;
-	case IO::Real:		return m_val[id].val.r;
+	case IO::String:	return (mVal[id].val.s->getVal()!=EVAL_STR) ? atof(mVal[id].val.s->getVal().c_str()) : EVAL_REAL;
+	case IO::Integer:	return (mVal[id].val.i!=EVAL_INT) ? mVal[id].val.i : EVAL_REAL;
+	case IO::Boolean:	return (mVal[id].val.b!=EVAL_BOOL) ? (bool)mVal[id].val.b : EVAL_REAL;
+	case IO::Real:		return mVal[id].val.r;
     }
     return 0.0;
 }
 
 char TValFunc::getB( unsigned id )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
-    switch(m_val[id].tp)
+    if( id >= mVal.size() )	throw TError("ValFnc",_("Id or IO %d error!"),id);
+    switch( mVal[id].tp )
     {
-	case IO::String:	return (m_val[id].val.s->getVal()!=EVAL_STR) ? (bool)atoi(m_val[id].val.s->getVal().c_str()) : EVAL_BOOL;
-	case IO::Integer:	return (m_val[id].val.i!=EVAL_INT) ? (bool)m_val[id].val.i : EVAL_BOOL;
-	case IO::Real:		return (m_val[id].val.r!=EVAL_REAL) ? (bool)m_val[id].val.r : EVAL_BOOL;
-	case IO::Boolean:	return m_val[id].val.b;
+	case IO::String:	return (mVal[id].val.s->getVal()!=EVAL_STR) ? (bool)atoi(mVal[id].val.s->getVal().c_str()) : EVAL_BOOL;
+	case IO::Integer:	return (mVal[id].val.i!=EVAL_INT) ? (bool)mVal[id].val.i : EVAL_BOOL;
+	case IO::Real:		return (mVal[id].val.r!=EVAL_REAL) ? (bool)mVal[id].val.r : EVAL_BOOL;
+	case IO::Boolean:	return mVal[id].val.b;
     }
     return false;
 }
 
+TVarObj *TValFunc::getO( unsigned id )
+{
+    if( id >= mVal.size() )	throw TError("ValFnc",_("Id or IO %d error!"),id);
+    if( mVal[id].tp != IO::Object )	throw TError("ValFnc",_("Get object from not object's IO %d error!"),id);
+    return mVal[id].val.o;
+}
+
 void TValFunc::setS( unsigned id, const string &val )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
-    switch(m_val[id].tp)
+    if( id >= mVal.size() )	throw TError("ValFnc",_("Id or IO %d error!"),id);
+    if( mdfChk() && val != getS(id) ) mVal[id].mdf = true;
+    switch( mVal[id].tp )
     {
-	case IO::Integer:	m_val[id].val.i = (val!=EVAL_STR) ? atoi(val.c_str()) : EVAL_INT;	break;
-	case IO::Real:		m_val[id].val.r = (val!=EVAL_STR) ? atof(val.c_str()) : EVAL_REAL;	break;
-	case IO::Boolean:	m_val[id].val.b = (val!=EVAL_STR) ? (bool)atoi(val.c_str()) : EVAL_BOOL;	break;
-	case IO::String:	m_val[id].val.s->setVal(val);	break;
+	case IO::Integer:	mVal[id].val.i = (val!=EVAL_STR) ? atoi(val.c_str()) : EVAL_INT;	break;
+	case IO::Real:		mVal[id].val.r = (val!=EVAL_STR) ? atof(val.c_str()) : EVAL_REAL;	break;
+	case IO::Boolean:	mVal[id].val.b = (val!=EVAL_STR) ? (bool)atoi(val.c_str()) : EVAL_BOOL;	break;
+	case IO::String:	mVal[id].val.s->setVal(val);	break;
     }
 }
 
 void TValFunc::setI( unsigned id, int val )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
-    switch(m_val[id].tp)
+    if( id >= mVal.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
+    if( mdfChk() && val != getI(id) ) mVal[id].mdf = true;
+    switch( mVal[id].tp )
     {
-	case IO::String:	m_val[id].val.s->setVal( (val!=EVAL_INT) ? TSYS::int2str(val) : EVAL_STR );	break;
-	case IO::Real:		m_val[id].val.r = (val!=EVAL_INT) ? val : EVAL_REAL;		break;
-	case IO::Boolean:	m_val[id].val.b = (val!=EVAL_INT) ? (bool)val : EVAL_BOOL;	break;
-	case IO::Integer:	m_val[id].val.i = val;	break;
+	case IO::String:	mVal[id].val.s->setVal( (val!=EVAL_INT) ? TSYS::int2str(val) : EVAL_STR );	break;
+	case IO::Real:		mVal[id].val.r = (val!=EVAL_INT) ? val : EVAL_REAL;		break;
+	case IO::Boolean:	mVal[id].val.b = (val!=EVAL_INT) ? (bool)val : EVAL_BOOL;	break;
+	case IO::Integer:	mVal[id].val.i = val;	break;
     }
 }
 
 void TValFunc::setR( unsigned id, double val )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
+    if( id >= mVal.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
     if( isnan(val) ) val = 0.;	//Check for 'Not a Number'
-    switch(m_val[id].tp)
+    if( mdfChk() && val != getR(id) ) mVal[id].mdf = true;
+    switch( mVal[id].tp )
     {
-	case IO::String:	m_val[id].val.s->setVal( (val!=EVAL_REAL) ? TSYS::real2str(val) : EVAL_STR );	break;
-	case IO::Integer:	m_val[id].val.i = (val!=EVAL_REAL) ? (int)val : EVAL_INT;	break;
-	case IO::Boolean:	m_val[id].val.b = (val!=EVAL_REAL) ? (bool)val : EVAL_BOOL;	break;
-	case IO::Real:		m_val[id].val.r = val;	break;
+	case IO::String:	mVal[id].val.s->setVal( (val!=EVAL_REAL) ? TSYS::real2str(val) : EVAL_STR );	break;
+	case IO::Integer:	mVal[id].val.i = (val!=EVAL_REAL) ? (int)val : EVAL_INT;	break;
+	case IO::Boolean:	mVal[id].val.b = (val!=EVAL_REAL) ? (bool)val : EVAL_BOOL;	break;
+	case IO::Real:		mVal[id].val.r = val;	break;
     }
 }
 
 void TValFunc::setB( unsigned id, char val )
 {
-    if( id >= m_val.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
-    switch(m_val[id].tp)
+    if( id >= mVal.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
+    if( mdfChk() && val != getB(id) ) mVal[id].mdf = true;
+    switch( mVal[id].tp )
     {
-	case IO::String:	m_val[id].val.s->setVal( (val!=EVAL_BOOL) ? TSYS::int2str((bool)val) : EVAL_STR );	break;
-	case IO::Integer:	m_val[id].val.i = (val!=EVAL_BOOL) ? (bool)val : EVAL_INT;	break;
-	case IO::Real:		m_val[id].val.r = (val!=EVAL_BOOL) ? (bool)val : EVAL_REAL;	break;
-	case IO::Boolean:	m_val[id].val.b = val;	break;
+	case IO::String:	mVal[id].val.s->setVal( (val!=EVAL_BOOL) ? TSYS::int2str((bool)val) : EVAL_STR );	break;
+	case IO::Integer:	mVal[id].val.i = (val!=EVAL_BOOL) ? (bool)val : EVAL_INT;	break;
+	case IO::Real:		mVal[id].val.r = (val!=EVAL_BOOL) ? (bool)val : EVAL_REAL;	break;
+	case IO::Boolean:	mVal[id].val.b = val;	break;
     }
 }
 
+void TValFunc::setO( unsigned id, TVarObj *val )
+{
+    if( id >= mVal.size() )    throw TError("ValFnc",_("Id or IO %d error!"),id);
+    if( mVal[id].tp != IO::Object )	throw TError("ValFnc",_("Set object to not object's IO %d error!"),id);
+    if( mVal[id].val.o && !mVal[id].val.o->disconnect() ) delete mVal[id].val.o;
+    mVal[id].val.o = val;
+    mVal[id].val.o->connect();
+}
+
+void TValFunc::setMdfChk( bool set )
+{
+    mMdfChk = set;
+    if( set )
+	for( int i_v = 0; i_v < mVal.size(); i_v++ )
+	    mVal[i_v].mdf = false;
+}
+
 void TValFunc::calc( const string &user )
 {
     if( !mFunc || !mFunc->startStat() ) return;
