@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tfunction.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -27,6 +27,7 @@
 
 #include "tmess.h"
 #include "tcntrnode.h"
+#include <tvariant.h>
 
 using std::string;
 using std::vector;
@@ -41,7 +42,7 @@ class IO
     friend class TFunction;
     public:
 	//Data
-	enum Type { String, Integer, Real, Boolean };
+	enum Type { String, Integer, Real, Boolean, Object };
 	enum IOFlgs
 	{
 	    Default = 0x00,	//Default mode (Input IO)
@@ -105,9 +106,10 @@ class TFunction : public TCntrNode
 	bool startStat( )		{ return run_st; }
 	int use( )			{ return used.size(); }
 
+	void setId( const string &vl );
 	virtual void setStart( bool val )  { run_st = val; }
 
-	//- IO -
+	//> IO
 	void ioList( vector<string> &list );
 	int ioId( const string &id );
 	int ioSize( );
@@ -158,55 +160,60 @@ class TValFunc
 
 	string user( )				{ return mUser; }
 	const string &vfName( )			{ return mName; }
+	bool blk( )				{ return mBlk; }
+	bool dimens( )				{ return mDimens; }
+	bool mdfChk( )				{ return mMdfChk; }
 
 	void setUser( const string &iuser )	{ mUser = iuser; }
 	void setVfName( const string &inm )	{ mName = inm; }
+	void setDimens( bool set )		{ mDimens = set; }
+	void setMdfChk( bool set );
 
 	void ioList( vector<string> &list );
 	int  ioId( const string &id );
 	int  ioSize( );
 	IO::Type ioType( unsigned id )
 	{
-	    if( id >= m_val.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
+	    if( id >= mVal.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
 	    return mFunc->io(id)->type();
 	}
 	unsigned ioFlg( unsigned id )
 	{
-	    if( id >= m_val.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
+	    if( id >= mVal.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
 	    return mFunc->io(id)->flg();
 	}
 	bool ioHide( unsigned id )
 	{
-	    if( id >= m_val.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
+	    if( id >= mVal.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
 	    return mFunc->io(id)->hide();
 	}
+	bool ioMdf( unsigned id )
+	{
+	    if( id >= mVal.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
+	    return mVal[id].mdf;
+	}
 
-	//- get IO value -
+	//> get IO value
 	string	getS( unsigned id );
 	int	getI( unsigned id );
 	double	getR( unsigned id );
 	char	getB( unsigned id );
+	TVarObj	*getO( unsigned id );
 
-	//- set IO value -
+	//> set IO value
 	void setS( unsigned id, const string &val );
 	void setI( unsigned id, int val );
 	void setR( unsigned id, double val );
 	void setB( unsigned id, char val );
+	void setO( unsigned id, TVarObj *val );
 
-	//- Blocked values screen -
-	bool	blk( )			{ return mBlk; }
-
-	//- Dimension controll -
-	bool	dimens( )		{ return mDimens; }
-	void	setDimens( bool set )	{ mDimens = set; }
-
-	//- Calc function -
+	//> Calc function
 	virtual void calc( const string &user = "" );
-	//- Calc time function -
+	//> Calc time function
 	double  calcTm( )		{ return tm_calc; }
 	void setCalcTm( double ivl )	{ tm_calc = ivl; }
 
-	//- Attached function -
+	//> Attached function
 	TFunction *func( )		{ return mFunc; }
 	void setFunc( TFunction *func, bool att_det = true );
 
@@ -217,28 +224,31 @@ class TValFunc
 	//Data
 	struct SVl
 	{
-	    IO::Type	tp;
+	    unsigned short	tp	: 4;
+	    unsigned short	mdf	: 1;
 	    union
 	    {
 		ResString *s;
 		double r;
 		int    i;
 		char   b;
+		TVarObj	*o;
 	    }val;
 	};
 
 	//Attributes
-	vector<SVl>	m_val;	//pointer to: string, int, double, bool
+	vector<SVl>	mVal;	//pointer to: string, int, double, bool
 
     private:
 	//Methods
 	void funcDisConnect( bool det = true );
 
 	//Attributes
-	string	mName,		//Value name
-		mUser;		//Link to user
-	bool	mBlk,		//Blocked values screen
-		mDimens;	//Make dimension of the calc time
+	string	mName,			//Value name
+		mUser;			//Link to user
+	unsigned short	mBlk	:1;	//Blocked values screen
+	unsigned short	mDimens	:1;	//Make dimension of the calc time
+	unsigned short	mMdfChk	:1;	//Modify attributes check
 
 	double tm_calc;		//Calc time in mikroseconds
 
