@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tprmtmpl.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -117,9 +117,7 @@ void TPrmTempl::setStart( bool vl )
     if( startStat() == vl ) return;
     if(vl)
     {
-	//Check old compile function	????
-
-	//- Compile new function -
+	//> Compile new function
 	if(prog().size())
 	    work_prog = SYS->daq().at().at(TSYS::strSepParse(progLang(),0,'.')).at().
 					compileFunc(TSYS::strSepParse(progLang(),1,'.'),*this,prog());
@@ -166,13 +164,16 @@ void TPrmTempl::load_( )
 	    io(iid)->setDef(cfg.cfg("VALUE").getS());
 	}
     }
-    //- Position fixing -
+
+    //> Remove holes
+    for( int i_p = 0; i_p < u_pos.size(); i_p++ )
+	if( u_pos[i_p].empty() ) { u_pos.erase(u_pos.begin()+i_p); i_p--; }
+
+    //> Position fixing
     for( int i_p = 0; i_p < u_pos.size(); i_p++ )
     {
-	if( u_pos[i_p].empty() ) continue;
 	int iid = ioId(u_pos[i_p]);
-	if( iid != i_p )
-	    try{ ioMove(iid,i_p); } catch(...){ }
+	if( iid != i_p ) try{ ioMove(iid,i_p); } catch(...){ }
     }
 }
 
@@ -218,7 +219,7 @@ void TPrmTempl::preIOCfgChange()
 
 void TPrmTempl::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TCntrNode::cntrCmdProc(opt);
@@ -251,7 +252,7 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/io/io/4",_("Attribute"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
 		    "sel_id",(TSYS::int2str(IO::Default)+";"+TSYS::int2str(TPrmTempl::AttrRead)+";"+TSYS::int2str(TPrmTempl::AttrFull)).c_str(),
 		    "sel_list",_("No attribute;Read only;Full access"));
-		ctrMkNode("list",opt,-1,"/io/io/5",_("Access"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
+		ctrMkNode("list",opt,-1,"/io/io/5",_("Configure"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
 		    "sel_id",(TSYS::int2str(IO::Default)+";"+TSYS::int2str(TPrmTempl::CfgPublConst)+";"+TSYS::int2str(TPrmTempl::CfgLink)).c_str(),
 		    "sel_list",_("Constant;Public constant;Link"));
 		ctrMkNode("list",opt,-1,"/io/io/6",_("Value"),0664,"root","root",1,"tp","str");
@@ -262,29 +263,29 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	return;
     }
 
-    //- Process command to page -
+    //> Process command to page
     vector<string> list;
     string a_path = opt->attr("path");
     if( a_path == "/tmpl/st/st" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(startStat()?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setStart(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(startStat()?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setStart(atoi(opt->text().c_str()));
     }
     else if( a_path == "/tmpl/st/use" && ctrChkNode(opt) )	opt->setText(TSYS::int2str(startStat()?func().at().use():0));
     else if( a_path == "/tmpl/cfg/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/tmpl/cfg/name" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(name());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setName(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(name());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setName(opt->text());
     }
     else if( a_path == "/tmpl/cfg/descr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(descr());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setDescr(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(descr());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setDescr(opt->text());
     }
     else if( a_path == "/io/io" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    XMLNode *n_id   = ctrMkNode("list",opt,-1,"/io/io/0","");
 	    XMLNode *n_nm   = ctrMkNode("list",opt,-1,"/io/io/1","");
@@ -305,9 +306,9 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 		if( n_val )	n_val->childAdd("el")->setText(io(id)->def());
 	    }
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	{ ioAdd( new IO("new",_("New IO"),IO::Real,IO::Default) ); modif(); }
-	if( ctrChkNode(opt,"ins",0664,"root","root",SEQ_WR) )	{ ioIns( new IO("new",_("New IO"),IO::Real,IO::Default), atoi(opt->attr("row").c_str()) ); modif(); }
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )	{ ioAdd( new IO("new",_("New IO"),IO::Real,IO::Default) ); modif(); }
+	if( ctrChkNode(opt,"ins",0664,"root","root",SEC_WR) )	{ ioIns( new IO("new",_("New IO"),IO::Real,IO::Default), atoi(opt->attr("row").c_str()) ); modif(); }
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )
 	{
 	    int row = atoi(opt->attr("row").c_str());
 	    if( io(row)->flg()&TPrmTempl::LockAttr )
@@ -315,8 +316,8 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	    ioDel( row );
 	    modif();
 	}
-	if( ctrChkNode(opt,"move",0664,"root","root",SEQ_WR) )  { ioMove( atoi(opt->attr("row").c_str()), atoi(opt->attr("to").c_str()) ); modif(); }
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"move",0664,"root","root",SEC_WR) )  { ioMove( atoi(opt->attr("row").c_str()), atoi(opt->attr("to").c_str()) ); modif(); }
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    int row = atoi(opt->attr("row").c_str());
 	    int col = atoi(opt->attr("col").c_str());
@@ -337,13 +338,13 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/io/prog_lang" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(progLang());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	 setProgLang(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(progLang());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	 setProgLang(opt->text());
     }
     else if( a_path == "/io/prog" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(prog());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setProg(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(prog());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setProg(opt->text());
     }
     else if( a_path == "/io/plang_ls" && ctrChkNode(opt) )
     {
@@ -360,10 +361,15 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	vector<string>  ls;
 	switch(c_lv)
 	{
-	    case 0:	SYS->daq().at().modList(ls);	break;
+	    case 0:
+		SYS->daq().at().modList(ls);
+		for( int i_l = 0; i_l < ls.size(); i_l++ )
+		    if( !SYS->daq().at().at(ls[i_l]).at().compileFuncLangs() )
+		    { ls.erase(ls.begin()+i_l); i_l--; }
+		break;
 	    case 1:
 		if( SYS->daq().at().modPresent(TSYS::strSepParse(tplng,0,'.')) )
-		    SYS->daq().at().at(TSYS::strSepParse(tplng,0,'.')).at().compileFuncLangs(ls);
+		    SYS->daq().at().at(TSYS::strSepParse(tplng,0,'.')).at().compileFuncLangs(&ls);
 		break;
 	}
 	for(int i_l = 0; i_l < ls.size(); i_l++)
@@ -528,36 +534,36 @@ void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/lib/st/st" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( startStat() ? "1" : "0" );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	start( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText( startStat() ? "1" : "0" );
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	start( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/lib/st/db" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText( fullDB() );
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	setFullDB( opt->text() );
+	if( ctrChkNode(opt,"get",0660,"root","root",SEC_RD) )	opt->setText( fullDB() );
+	if( ctrChkNode(opt,"set",0660,"root","root",SEC_WR) )	setFullDB( opt->text() );
     }
     else if( a_path == "/lib/cfg/id" && ctrChkNode(opt) )	opt->setText( id() );
     else if( a_path == "/lib/cfg/name" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( name() );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setName( opt->text() );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText( name() );
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setName( opt->text() );
     }
     else if( a_path == "/lib/cfg/descr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(descr());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setDescr( opt->text() );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(descr());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	setDescr( opt->text() );
     }
     else if( a_path == "/br/tmpl_" || a_path == "/tmpl/tmpl" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    vector<string> lst;
 	    list(lst);
 	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
 		opt->childAdd("el")->setAttr("id",lst[i_f])->setText(at(lst[i_f]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	add(TSYS::strEncode(opt->attr("id"),TSYS::ID).c_str(),opt->text().c_str());
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	del(opt->attr("id").c_str(),true);
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )	add(TSYS::strEncode(opt->attr("id"),TSYS::oscdID).c_str(),opt->text().c_str());
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )	del(opt->attr("id").c_str(),true);
     }
     else TCntrNode::cntrCmdProc(opt);
 }
