@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tprotocols.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -61,7 +61,7 @@ void TProtocolS::load_()
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -111,6 +111,8 @@ void TProtocol::close( const string &name )
     chldDel(m_pr,name);
 }
 
+void TProtocol::itemListIn( vector<string> &ls, const string &curIt )	{ ls.clear(); }
+
 //************************************************
 //* TProtocolIn                                  *
 //************************************************
@@ -124,4 +126,6 @@ TProtocolIn::~TProtocolIn()
 
 }
 
+
+
 TProtocol &TProtocolIn::owner( )	{ return *(TProtocol*)nodePrev(); }
