@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tsecurity.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -49,23 +49,23 @@ void TSecurity::postEnable(int flag)
 {
     if(!(flag&TCntrNode::NodeRestore))
     {
-	//- Add surely users, groups and set their parameters -
-	//-- Administrator --
+	//> Add surely users, groups and set their parameters
+	//>> Administrator
 	usrAdd("root");
 	usrAt("root").at().setLName(_("Administrator (superuser)!!!"));
 	usrAt("root").at().setSysItem(true);
 	usrAt("root").at().setPass("openscada");
-	//-- Simple user --
+	//>> Simple user
 	usrAdd("user");
 	usrAt("user").at().setLName(_("Simple user."));
 	usrAt("user").at().setSysItem(true);
 	usrAt("user").at().setPass("user");
-	//-- Administrators group --
+	//>> Administrators group
 	grpAdd("root");
 	grpAt("root").at().setLName(_("Administrators group."));
 	grpAt("root").at().setSysItem(true);
 	grpAt("root").at().userAdd("root");
-	//-- Simple users group --
+	//>> Simple users group
 	grpAdd("users");
 	grpAt("users").at().setLName(_("Users group."));
 	grpAt("users").at().setSysItem(true);
@@ -151,7 +151,7 @@ void TSecurity::load_( )
 	next_opt=getopt_long(SYS->argc,( char *const * ) SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -224,8 +224,7 @@ string TSecurity::optDescr( )
 {
     char buf[STR_BUF_LEN];
     snprintf(buf,sizeof(buf),_(
-	"======================= Subsystem \"Security\" options ====================\n"
-	"------------ Parameters of section <%s> in config file -----------\n\n"
+	"======================= Subsystem \"Security\" options ====================\n\n"
 	),nodePath().c_str());
 
     return buf;
@@ -254,27 +253,27 @@ void TSecurity::cntrCmdProc( XMLNode *opt )
     if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root",subId().c_str()) )	opt->setText(optDescr());
     else if( a_path == "/br/usr_" || a_path == "/usgr/users" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",subId().c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root",subId().c_str(),SEC_RD) )
 	{
 	    vector<string> list;
 	    usrList(list);
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 	        opt->childAdd("el")->setText(list[i_a]);
 	}
-	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEQ_WR) )	usrAdd(TSYS::strEncode(opt->text(),TSYS::ID));
-	if( ctrChkNode(opt,"del",0664,"root",subId().c_str(),SEQ_WR) )	usrDel(opt->text(),true);
+	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEC_WR) )	usrAdd(TSYS::strEncode(opt->text(),TSYS::oscdID));
+	if( ctrChkNode(opt,"del",0664,"root",subId().c_str(),SEC_WR) )	usrDel(opt->text(),true);
     }
     else if( a_path == "/br/grp_" || a_path == "/usgr/grps" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",subId().c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root",subId().c_str(),SEC_RD) )
 	{
 	    vector<string> list;
 	    grpList(list);
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 	        opt->childAdd("el")->setText(list[i_a]);
 	}
-	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEQ_WR) )	grpAdd(TSYS::strEncode(opt->text(),TSYS::ID));
-	if( ctrChkNode(opt,"del",0664,"root",subId().c_str(),SEQ_WR) )	grpDel(opt->text(),true);
+	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEC_WR) )	grpAdd(TSYS::strEncode(opt->text(),TSYS::oscdID));
+	if( ctrChkNode(opt,"del",0664,"root",subId().c_str(),SEC_WR) )	grpDel(opt->text(),true);
     }
     else TSubSYS::cntrCmdProc(opt);
 }
@@ -376,28 +375,28 @@ void TUser::cntrCmdProc( XMLNode *opt )
 	opt->setText( TSYS::int2str(auth(opt->attr("password"))) );
     else if( a_path == "/prm/db" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",SYS->db().at().subId().c_str(),SEQ_RD) )	opt->setText(DB());
-	if( ctrChkNode(opt,"set",0664,"root",SYS->db().at().subId().c_str(),SEQ_WR) )	setDB(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root",SYS->db().at().subId().c_str(),SEC_RD) )	opt->setText(DB());
+	if( ctrChkNode(opt,"set",0664,"root",SYS->db().at().subId().c_str(),SEC_WR) )	setDB(opt->text());
     }
     else if( a_path == "/prm/name" && ctrChkNode(opt) )	opt->setText(name());
     else if( a_path == "/prm/pct" || a_path == "/ico" )
     {
-	if( ctrChkNode(opt,"get",0664,name().c_str(),owner().subId().c_str(),SEQ_RD) )	opt->setText(picture());
-	if( ctrChkNode(opt,"set",0664,name().c_str(),owner().subId().c_str(),SEQ_WR) )	setPicture(opt->text());
+	if( ctrChkNode(opt,"get",0664,name().c_str(),owner().subId().c_str(),SEC_RD) )	opt->setText(picture());
+	if( ctrChkNode(opt,"set",0664,name().c_str(),owner().subId().c_str(),SEC_WR) )	setPicture(opt->text());
     }
     else if( a_path == "/prm/dscr" )
     {
-	if( ctrChkNode(opt,"get",0664,name().c_str(),owner().subId().c_str(),SEQ_RD) )	opt->setText(lName());
-	if( ctrChkNode(opt,"set",0664,name().c_str(),owner().subId().c_str(),SEQ_WR) )	setLName(opt->text());
+	if( ctrChkNode(opt,"get",0664,name().c_str(),owner().subId().c_str(),SEC_RD) )	opt->setText(lName());
+	if( ctrChkNode(opt,"set",0664,name().c_str(),owner().subId().c_str(),SEC_WR) )	setLName(opt->text());
     }
     else if( a_path == "/prm/pass" )
     {
-	if( ctrChkNode(opt,"get",0660,name().c_str(),owner().subId().c_str(),SEQ_RD) )	opt->setText("**********");
-	if( ctrChkNode(opt,"set",0660,name().c_str(),owner().subId().c_str(),SEQ_WR) )	setPass(opt->text());
+	if( ctrChkNode(opt,"get",0660,name().c_str(),owner().subId().c_str(),SEC_RD) )	opt->setText("**********");
+	if( ctrChkNode(opt,"set",0660,name().c_str(),owner().subId().c_str(),SEC_WR) )	setPass(opt->text());
     }
     else if( a_path == "/prm/grps" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","Security",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0660,"root","Security",SEC_RD) )
 	{
 	    //-- Prepare headers --
 	    XMLNode *grp = ctrMkNode("list",opt,-1,"/prm/grps/grp","",0440);
@@ -410,7 +409,7 @@ void TUser::cntrCmdProc( XMLNode *opt )
 		if(vl)	vl->childAdd("el")->setText(TSYS::int2str(owner().grpAt(ls[i_g]).at().user(name())));
 	    }
 	}
-	/*if( ctrChkNode(opt,"set",0660,"root","Security",SEQ_WR) )
+	/*if( ctrChkNode(opt,"set",0660,"root","Security",SEC_WR) )
 	{
 	    if(atoi(opt->text().c_str()))
 		owner().grpAt(opt->attr("key_grp")).at().userAdd(name());
@@ -520,26 +519,26 @@ void TGroup::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/prm/db" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",SYS->db().at().subId().c_str(),SEQ_RD) )	opt->setText(DB());
-	if( ctrChkNode(opt,"set",0664,"root",SYS->db().at().subId().c_str(),SEQ_WR) )	setDB(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root",SYS->db().at().subId().c_str(),SEC_RD) )	opt->setText(DB());
+	if( ctrChkNode(opt,"set",0664,"root",SYS->db().at().subId().c_str(),SEC_WR) )	setDB(opt->text());
     }
-    else if( a_path == "/prm/name" && ctrChkNode(opt,"get",0444,"root",owner().subId().c_str(),SEQ_RD) )
+    else if( a_path == "/prm/name" && ctrChkNode(opt,"get",0444,"root",owner().subId().c_str(),SEC_RD) )
 	opt->setText(name());
     else if( a_path == "/prm/dscr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",owner().subId().c_str(),SEQ_RD) )	opt->setText(lName());
-	if( ctrChkNode(opt,"set",0664,"root",owner().subId().c_str(),SEQ_WR) )	setLName(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root",owner().subId().c_str(),SEC_RD) )	opt->setText(lName());
+	if( ctrChkNode(opt,"set",0664,"root",owner().subId().c_str(),SEC_WR) )	setLName(opt->text());
     }
     else if( a_path == "/prm/users" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",owner().subId().c_str(),SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root",owner().subId().c_str(),SEC_RD) )
 	{
 	    string val;
 	    for( int off = 0; (val=TSYS::strSepParse(m_usrs,0,';',&off)).size(); )
 		opt->childAdd("el")->setText(val);
 	}
-	if( ctrChkNode(opt,"add",0664,"root",owner().subId().c_str(),SEQ_WR) )	userAdd(opt->text());
-	if( ctrChkNode(opt,"del",0664,"root",owner().subId().c_str(),SEQ_WR) )	userDel(opt->text());
+	if( ctrChkNode(opt,"add",0664,"root",owner().subId().c_str(),SEC_WR) )	userAdd(opt->text());
+	if( ctrChkNode(opt,"del",0664,"root",owner().subId().c_str(),SEC_WR) )	userDel(opt->text());
     }
     else TCntrNode::cntrCmdProc(opt);
 }
