@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tsecurity.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -24,9 +24,9 @@
 
 #include "tbds.h"
 
-#define SEQ_XT 0x01
-#define SEQ_WR 0x02
-#define SEQ_RD 0x04
+#define SEC_XT 0x01
+#define SEC_WR 0x02
+#define SEC_RD 0x04
 
 //*************************************************
 //* TUser					  *
