@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tsys.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -68,7 +68,7 @@ TSYS::TSYS( int argi, char ** argb, char **env ) :
     //signal(SIGCHLD,sighandler);
     signal(SIGALRM,sighandler);
     signal(SIGPIPE,sighandler);
-    signal(SIGFPE,sighandler);
+    //signal(SIGFPE,sighandler);
     //signal(SIGSEGV,sighandler);
     signal(SIGABRT,sighandler);
 }
@@ -91,6 +91,12 @@ TSYS::~TSYS(  )
     delete Mess;
 }
 
+string TSYS::host( )
+{
+    utsname ubuf; uname(&ubuf);
+    return ubuf.nodename;
+}
+
 string TSYS::workDir( )
 {
     char buf[STR_BUF_LEN];
@@ -100,7 +106,8 @@ string TSYS::workDir( )
 void TSYS::setWorkDir( const string &wdir )
 {
     if( chdir(wdir.c_str()) != 0 )
-	mess_err(nodePath().c_str(),_("Change work directory to '%s' error: %s"),wdir.c_str(),strerror(errno));
+	mess_warning(nodePath().c_str(),_("Change work directory to '%s' error: %s. Perhaps current directory already set correct to '%s'."),
+	    wdir.c_str(),strerror(errno),workDir().c_str());
     modif( );
 }
 
@@ -242,7 +249,7 @@ bool TSYS::cfgFileLoad()
 	switch(next_opt)
 	{
 	    case 'h':
-		fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str());
+		fprintf(stdout,"%s",optDescr().c_str());
 		Mess->setMessLevel(7);
 		cmd_help = true;
 		break;
@@ -331,7 +338,11 @@ void TSYS::load_()
 
 	//> Load modules
 	modSchedul().at().load();
-	modSchedul().at().loadLibS();
+	if( !modSchedul().at().loadLibS() )
+	{
+	    mess_err(nodePath().c_str(),_("No one module is loaded. Your configuration broken!"));
+	    stop();
+	}
 
 	//> First DB subsystem load
 	db().at().load();
@@ -382,7 +393,7 @@ int TSYS::start(  )
 
     mess_info(nodePath().c_str(),_("Start!"));
     for( unsigned i_a=0; i_a < lst.size(); i_a++ )
-	try{ at(lst[i_a]).at().subStart(); }
+	try { at(lst[i_a]).at().subStart(); }
 	catch(TError err)
 	{
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -390,6 +401,9 @@ int TSYS::start(  )
 	}
 
     cfgFileScan( true );
+
+    mess_info(nodePath().c_str(),_("Final started!"));
+
     unsigned int i_cnt = 1;
     while( !mStopSignal )
     {
@@ -444,11 +458,12 @@ void TSYS::sighandler( int signal )
 	    SYS->mStopSignal=signal;
 	    break;
 	case SIGTERM:
-            mess_warning(SYS->nodePath().c_str(),_("The Terminate signal is received.. Server is being stoped!"));
+	    mess_warning(SYS->nodePath().c_str(),_("The Terminate signal is received.. Server is being stoped!"));
 	    SYS->mStopSignal=signal;
 	    break;
 	case SIGFPE:
-            mess_warning(SYS->nodePath().c_str(),_("Floating point exception is catched!"));
+	    mess_warning(SYS->nodePath().c_str(),_("Floating point exception is catched!"));
+            exit(1);
 	    break;
 	case SIGCHLD:
 	{
@@ -572,6 +587,34 @@ string TSYS::strSepParse( const string &path, int level, char sep, int *off )
     return "";
 }
 
+string TSYS::strParse( const string &path, int level, const string &sep, int *off, bool mergeSepSymb )
+{
+    int an_dir = off ? *off : 0;
+    int t_lev = 0;
+    int t_dir;
+
+    if( an_dir >= path.size() || sep.empty() ) return "";
+    while(true)
+    {
+	t_dir = path.find(sep,an_dir);
+	if( t_dir == string::npos )
+	{
+	    if( off ) *off = path.size();
+	    return (t_lev == level) ? path.substr(an_dir) : "";
+	}
+	else if( t_lev == level )
+	{
+	    if( off ) *off = t_dir+1;
+	    return path.substr(an_dir,t_dir-an_dir);
+	}
+	if( mergeSepSymb && sep.size() == 1 )
+	    for( an_dir = t_dir; an_dir < path.size() && path[an_dir] == sep[0]; ) an_dir++;
+	else an_dir = t_dir+sep.size();
+	t_lev++;
+    }
+    return "";
+}
+
 string TSYS::pathLev( const string &path, int level, bool encode, int *off )
 {
     int an_dir = off ? *off : 0;
@@ -666,7 +709,7 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 		    case '<':	sout+="&lt;";	break;
 		    case '"':	sout+="&quot;";	break;
 		    case '&':	sout+="&amp;";	break;
-		    case '\'':	sout+="&#039;";	break;
+		    case '\'':	sout+="&apos;";	break;
 		    default:	sout+=in[i_sz];
 		}
 	    break;
@@ -742,7 +785,7 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 	    for( i_sz = 0; i_sz < sout.size(); i_sz++ )
 		if( sout[i_sz] == '%' ) { sout.replace(i_sz,1,"%%"); i_sz++; }
 	    break;
-	case TSYS::ID:
+	case TSYS::oscdID:
 	    sout.reserve(in.size());
 	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
 		switch( in[i_sz] )
@@ -769,6 +812,25 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 	case TSYS::Reverse:
 	    for( i_sz = in.size()-1; i_sz >= 0; i_sz-- ) sout += in[i_sz];
 	    break;
+	case TSYS::ShieldSimb:
+	    sout.reserve(in.size());
+	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
+		if( in[i_sz] == '\\' && i_sz < (in.size()-1) )
+		{
+		    switch( in[i_sz+1] )
+		    {
+			case 'a':	sout += '\a';	break;
+			case 'b':	sout += '\b';	break;
+			case 'f':	sout += '\f';	break;
+			case 'n':	sout += '\n';	break;
+			case 'r':	sout += '\r';	break;
+			case 't':	sout += '\t';	break;
+			case 'v':	sout += '\v';	break;
+			default:	sout += in[i_sz+1];
+		    }
+		    i_sz++;
+		}else sout += in[i_sz];
+	    break;
     }
     return sout;
 }
@@ -910,18 +972,339 @@ long TSYS::HZ()
     return sysconf(_SC_CLK_TCK);
 }
 
-void TSYS::taskSleep( long long per )
+bool TSYS::cntrEmpty( )
+{
+    ResAlloc res( nodeRes(), false );
+    return mCntrs.empty();
+}
+
+double TSYS::cntrGet( const string &id )
+{
+    ResAlloc res( nodeRes(), false );
+    map<string,double>::iterator icnt = mCntrs.find(id);
+    if( icnt == mCntrs.end() )	return 0;
+    return icnt->second;
+}
+
+void TSYS::cntrSet( const string &id, double vl )
+{
+    ResAlloc res( nodeRes(), true );
+    mCntrs[id] = vl;
+}
+
+void TSYS::taskCreate( const string &path, int priority, void *(*start_routine)(void *), void *arg, bool *startCntr, int wtm )
+{
+    ResAlloc res( taskRes, false );
+    if( mTasks.find(path) != mTasks.end() && (!startCntr || (startCntr && *startCntr)) )
+	throw TError(nodePath().c_str(),_("Task '%s' is already created!"),path.c_str());
+    res.release();
+
+    pthread_t procPthr;
+    pthread_attr_t pthr_attr;
+    pthread_attr_init( &pthr_attr );
+    pthread_attr_setinheritsched( &pthr_attr, PTHREAD_EXPLICIT_SCHED );
+    struct sched_param prior;
+    prior.sched_priority = 0;
+
+    int policy = SCHED_OTHER;
+    if( priority < 0 )	policy = SCHED_BATCH;
+    else if( priority > 0 /*&& SYS->user() == "root"*/ )	policy = SCHED_RR;
+    pthread_attr_setschedpolicy( &pthr_attr, policy );
+    prior.sched_priority = vmax(sched_get_priority_min(policy),vmin(sched_get_priority_max(policy),priority));
+    pthread_attr_setschedparam(&pthr_attr,&prior);
+
+    int rez = pthread_create( &procPthr, &pthr_attr, start_routine, arg );
+    if( rez == EPERM )
+    {
+	mess_warning(nodePath().c_str(),_("No permition for create realtime policy. Default thread is created!"));
+	policy = SCHED_OTHER;
+	pthread_attr_setschedpolicy( &pthr_attr, policy );
+	prior.sched_priority = 0;
+	pthread_attr_setschedparam(&pthr_attr,&prior);
+	rez = pthread_create( &procPthr, &pthr_attr, start_routine, arg );
+    }
+    pthread_attr_destroy( &pthr_attr );
+
+    if( rez ) throw TError( nodePath().c_str(), _("Task creation error %d."), rez );
+
+    if( startCntr && TSYS::eventWait( *startCntr, true, nodePath()+": "+path+": start", wtm ) )
+	throw TError( nodePath().c_str(), _("Task '%s' is not started!"), path.c_str() );
+
+    res.request(true);
+    mTasks[path] = STask( procPthr, policy, prior.sched_priority );
+    res.release();
+}
+
+void TSYS::taskDestroy( const string &path, bool *startCntr, bool *endrunCntr, int wtm )
+{
+    ResAlloc res(taskRes,false);
+    map<string,STask>::iterator it = mTasks.find(path);
+    if( it == mTasks.end() )	return;//throw TError(nodePath().c_str(),_("Task '%s' is not present!"),path.c_str());
+    pthread_t thr = it->second.thr;
+    res.release();
+
+    if( endrunCntr ) *endrunCntr = true;
+    pthread_kill( thr, SIGALRM );
+
+    if( startCntr && TSYS::eventWait( *startCntr, false, nodePath()+": "+path+": stop", wtm ) )
+	throw TError(nodePath().c_str(),_("Task '%s' is not stopped!"),path.c_str());
+
+    pthread_join( thr, NULL );
+
+    res.request(true);
+    mTasks.erase( it );
+}
+
+void TSYS::taskSleep( long long per, time_t cron )
 {
     struct timespec sp_tm;
 
-    clock_gettime(CLOCK_REALTIME,&sp_tm);
-    long long pnt_tm = ( ((long long)sp_tm.tv_sec*1000000000+sp_tm.tv_nsec)/per + 1)*per;
-    do
+    if( !cron )
     {
-	sp_tm.tv_sec = pnt_tm/1000000000; sp_tm.tv_nsec = pnt_tm%1000000000;
-	if( clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&sp_tm,NULL) )	return;
+	if( !per ) per = 1000000000;
 	clock_gettime(CLOCK_REALTIME,&sp_tm);
-    }while( ((long long)sp_tm.tv_sec*1000000000+sp_tm.tv_nsec) < pnt_tm );
+	long long pnt_tm = ( ((long long)sp_tm.tv_sec*1000000000+sp_tm.tv_nsec)/per + 1)*per;
+	do
+	{
+	    sp_tm.tv_sec = pnt_tm/1000000000; sp_tm.tv_nsec = pnt_tm%1000000000;
+	    if( clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&sp_tm,NULL) )	return;
+	    clock_gettime(CLOCK_REALTIME,&sp_tm);
+	}while( ((long long)sp_tm.tv_sec*1000000000+sp_tm.tv_nsec) < pnt_tm );
+    }
+    else
+    {
+	sp_tm.tv_sec = cron; sp_tm.tv_nsec = 0;
+	clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&sp_tm,NULL);
+    }
+}
+
+time_t TSYS::cron( const string &vl, time_t base )
+{
+    bool isReset = false;
+    string cronEl, tEl;
+    int vbeg, vend, vstep, vm;
+
+    time_t ctm = base?base:time(NULL);
+    struct tm ttm;
+    localtime_r(&ctm,&ttm);
+    ttm.tm_sec = 0;
+
+reload:
+    bool isReload = false;
+    //> Minutes check
+    cronEl = TSYS::strSepParse(vl,0,' ');
+    vm = 200;
+    for( int eoff = 0; (tEl=TSYS::strSepParse(cronEl,0,',',&eoff)).size(); )
+    {
+	vbeg = vend = -1; vstep = 0;
+	sscanf(tEl.c_str(),"%d-%d/%d",&vbeg,&vend,&vstep);
+	if( vbeg < 0 ) { sscanf(tEl.c_str(),"*/%d",&vstep); vbeg=0; vend=59; }
+	if( vend < 0 ) vm = vmin(vm,vbeg+((ttm.tm_min>=vbeg)?60:0));
+	else if( (vbeg=vmax(0,vbeg)) < (vend=vmin(59,vend)) )
+	{
+	    if( ttm.tm_min < vbeg ) vm = vmin(vm,vbeg);
+	    else if( (vstep>1 && ttm.tm_min >= (vbeg+((vend-vbeg)/vstep)*vstep)) || (vstep <= 0 && ttm.tm_min >= vend) )
+		vm = vmin(vm,vbeg+60);
+	    else if( vstep>1 ) vm = vmin( vm, vbeg + vstep*( ((ttm.tm_min+1)-vbeg)/vstep + ((((ttm.tm_min+1)-vbeg)%vstep)?1:0) ) );
+	    else vm = vmin( vm, ttm.tm_min+1 );
+	}
+	if( vm == ttm.tm_min+1 ) break;
+    }
+    ttm.tm_min = vm;
+    mktime(&ttm);
+
+    //> Hours check
+    cronEl = TSYS::strSepParse(vl,1,' ');
+    vm = 200;
+    for( int eoff = 0; (tEl=TSYS::strSepParse(cronEl,0,',',&eoff)).size(); )
+    {
+	vbeg = vend = -1; vstep = 0;
+	sscanf(tEl.c_str(),"%d-%d/%d",&vbeg,&vend,&vstep);
+	if( vbeg < 0 ) { sscanf(tEl.c_str(),"*/%d",&vstep); vbeg=0; vend=23; }
+	if( vend < 0 ) vm = vmin(vm,vbeg+((ttm.tm_hour>vbeg)?24:0));
+	else if( (vbeg=vmax(0,vbeg)) < (vend=vmin(23,vend)) )
+	{
+	    if( ttm.tm_hour < vbeg ) vm = vmin(vm,vbeg);
+	    else if( (vstep>1 && ttm.tm_hour > (vbeg+((vend-vbeg)/vstep)*vstep)) || (vstep <= 0 && ttm.tm_hour > vend) )
+		vm = vmin(vm,vbeg+24);
+	    else if( vstep>1 ) vm = vmin( vm, vbeg + vstep*( (ttm.tm_hour-vbeg)/vstep + (((ttm.tm_hour-vbeg)%vstep)?1:0) ) );
+	    else vm = vmin( vm, ttm.tm_hour );
+	}
+	if( vm == ttm.tm_hour ) break;
+    }
+    isReload = (ttm.tm_hour!=vm);
+    ttm.tm_hour = vm;
+    mktime(&ttm);
+    if( isReload )	{ ttm.tm_min = -1; goto reload; }
+
+    //> Day check
+    cronEl = TSYS::strSepParse(vl,2,' ');
+    string cronElw = TSYS::strSepParse(vl,4,' ');
+    vm = 200;
+    if( cronEl != "*" )
+	for( int eoff = 0; (tEl=TSYS::strSepParse(cronEl,0,',',&eoff)).size(); )
+	{
+	    vbeg = vend = -1; vstep = 0;
+	    sscanf(tEl.c_str(),"%d-%d/%d",&vbeg,&vend,&vstep);
+	    if( vbeg < 0 ) { sscanf(tEl.c_str(),"*/%d",&vstep); vbeg=1; vend=31; }
+	    if( vend < 0 ) vm = vmin(vm,vbeg+((ttm.tm_mday>vbeg)?31:0));
+	    else if( (vbeg=vmax(1,vbeg)) < (vend=vmin(31,vend)) )
+	    {
+		if( ttm.tm_mday < vbeg ) vm = vmin(vm,vbeg);
+		else if( (vstep>1 && ttm.tm_mday > (vbeg+((vend-vbeg)/vstep)*vstep)) || (vstep <= 0 && ttm.tm_mday > vend) )
+		    vm = vmin(vm,vbeg+31);
+		else if( vstep>1 ) vm = vmin( vm, vbeg + vstep*( (ttm.tm_mday-vbeg)/vstep + (((ttm.tm_mday-vbeg)%vstep)?1:0) ) );
+		else vm = vmin( vm, ttm.tm_mday );
+	    }
+	    if( vm == ttm.tm_mday ) break;
+        }
+    if( cronEl == "*" || (cronElw != "*" && !cronElw.empty()) )
+	for( int eoff = 0; (tEl=TSYS::strSepParse(cronElw,0,',',&eoff)).size(); )
+	{
+	    vbeg = vend = -1; vstep = 0;
+	    sscanf(tEl.c_str(),"%d-%d/%d",&vbeg,&vend,&vstep);
+	    if( vbeg < 0 ) { sscanf(tEl.c_str(),"*/%d",&vstep); vbeg=0; vend=6; }
+	    if( vend < 0 ) vm = vmin( vm, ttm.tm_mday - ttm.tm_wday + vbeg+((ttm.tm_wday>vbeg)?7:0) );
+	    else if( (vbeg=vmax(1,vbeg)) < (vend=vmin(7,vend)) )
+	    {
+		if( ttm.tm_wday < vbeg ) vm = vmin( vm, ttm.tm_mday - ttm.tm_wday + vbeg );
+		else if( (vstep>1 && ttm.tm_wday > (vbeg+((vend-vbeg)/vstep)*vstep)) || (vstep <= 0 && ttm.tm_wday > vend) )
+		    vm = vmin( vm, ttm.tm_mday - ttm.tm_wday + vbeg+7 );
+		else if( vstep>1 ) vm = vmin( vm, ttm.tm_mday - ttm.tm_wday + vbeg + vstep*( (ttm.tm_wday-vbeg)/vstep + (((ttm.tm_wday-vbeg)%vstep)?1:0) ) );
+		else vm = vmin( vm, ttm.tm_mday );
+	    }
+	    if( vm == ttm.tm_mday ) break;
+	}
+    isReload = (ttm.tm_mday!=vm);
+    if( vm <= 31 ) ttm.tm_mday = vm;
+    { ttm.tm_mday = vm-31; ttm.tm_mon++; }
+    mktime(&ttm);
+    if( isReload )	{ ttm.tm_min = -1; ttm.tm_hour = 0; goto reload; }
+
+    //> Month check
+    cronEl = TSYS::strSepParse(vl,3,' ');
+    vm = 200;
+    for( int eoff = 0; (tEl=TSYS::strSepParse(cronEl,0,',',&eoff)).size(); )
+    {
+	vbeg = vend = -1; vstep = 0;
+	sscanf(tEl.c_str(),"%d-%d/%d",&vbeg,&vend,&vstep);
+	if( vbeg < 0 ) { sscanf(tEl.c_str(),"*/%d",&vstep); vbeg=1; vend=12; }
+	if( vend < 0 ) vm = vmin(vm,vbeg+(((ttm.tm_mon+1)>vbeg)?12:0));
+	else if( (vbeg=vmax(1,vbeg)) < (vend=vmin(12,vend)) )
+	{
+	    if( (ttm.tm_mon+1) < vbeg ) vm = vmin(vm,vbeg);
+	    else if( (vstep>1 && (ttm.tm_mon+1) > (vbeg+((vend-vbeg)/vstep)*vstep)) || (vstep <= 0 && (ttm.tm_mon+1) > vend) )
+		vm = vmin(vm,vbeg+12);
+	    else if( vstep>1 ) vm = vmin( vm, vbeg + vstep*( ((ttm.tm_mon+1)-vbeg)/vstep + ((((ttm.tm_mon+1)-vbeg)%vstep)?1:0) ) );
+	    else vm = vmin( vm, ttm.tm_mon+1 );
+	}
+	if( vm == (ttm.tm_mon+1) ) break;
+    }
+    isReload = (ttm.tm_mon!=(vm-1));
+    ttm.tm_mon = vm-1;
+    mktime(&ttm);
+    if( isReload )	{ ttm.tm_min = -1; ttm.tm_hour = 0; ttm.tm_mday = 1; goto reload; }
+
+    return mktime(&ttm);
+}
+
+TVariant TSYS::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    if( iid == "message" && prms.size() >= 3 )	{ message( prms[0].getS().c_str(), (TMess::Type)prms[1].getI(), "%s", prms[2].getS().c_str() ); return 0; }
+    if( iid == "messDebug" && prms.size() >= 2 ){ mess_debug( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "messInfo" && prms.size() >= 2 )	{ mess_info( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "messNote" && prms.size() >= 2 )	{ mess_note( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "messWarning" && prms.size() >= 2 )	{ mess_warning( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "messErr" && prms.size() >= 2 )	{ mess_err( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "messCrit" && prms.size() >= 2 )	{ mess_crit( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "messAlert" && prms.size() >= 2 ){ mess_alert( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "messEmerg" && prms.size() >= 2 ){ mess_emerg( prms[0].getS().c_str(), "%s", prms[1].getS().c_str() ); return 0; }
+    if( iid == "system" && prms.size() >= 1 )
+    {
+	if( prms.size() >= 2 && prms[1].getB() ) return system( prms[0].getS().c_str() );
+	FILE *fp = popen(prms[0].getS().c_str(),"r");
+	if( !fp ) return string("");
+
+	char buf[STR_BUF_LEN];
+	string rez;
+	for( int r_cnt = 0; (r_cnt=fread(buf,1,sizeof(buf),fp)); )
+	    rez.append(buf,r_cnt);
+
+	pclose(fp);
+	return rez;
+    }
+    if( iid == "XMLNode" ) return new XMLNodeObj( (prms.size()>=1) ? prms[0].getS() : "" );
+    if( iid == "cntrReq" && prms.size() >= 1 )
+    {
+	XMLNode req;
+	if( !dynamic_cast<XMLNodeObj*>(prms[0].getO()) ) return string(_("1:Request is not object!"));
+	((XMLNodeObj*)prms[0].getO())->toXMLNode(req);
+	string path = req.attr("path");
+	if( prms.size() < 2 || prms[1].getS().empty() )
+	{
+	    req.setAttr("user",user);
+	    cntrCmd(&req);
+	}
+	else
+	{
+	    req.setAttr("path",prms[1].getS()+"/"+path);
+	    transport().at().cntrIfCmd(req,"xmlCntrReq",user);
+	    req.setAttr("path",path);
+	}
+	((XMLNodeObj*)prms[0].getO())->fromXMLNode(req);
+	return string("0");
+    }
+    if( iid == "time" )
+    {
+	if( prms.empty() ) return (int)time(NULL);
+	long long tm = curTime();
+	prms[0].setI(tm%1000000); prms[0].setModify();
+	return (int)(tm/1000000);
+    }
+    if( iid == "localtime" && prms.size() >= 2 )
+    {
+	time_t tm_t = prms[0].getI();
+	struct tm tm_tm;
+	localtime_r(&tm_t,&tm_tm);
+
+	prms[1].setI(tm_tm.tm_sec); prms[1].setModify();
+	if( prms.size() >= 3 ) { prms[2].setI(tm_tm.tm_min); prms[2].setModify(); }
+	if( prms.size() >= 4 ) { prms[3].setI(tm_tm.tm_hour); prms[3].setModify(); }
+	if( prms.size() >= 5 ) { prms[4].setI(tm_tm.tm_mday); prms[4].setModify(); }
+	if( prms.size() >= 6 ) { prms[5].setI(tm_tm.tm_mon); prms[5].setModify(); }
+	if( prms.size() >= 7 ) { prms[6].setI(1900+tm_tm.tm_year); prms[6].setModify(); }
+	if( prms.size() >= 8 ) { prms[7].setI(tm_tm.tm_wday); prms[7].setModify(); }
+	if( prms.size() >= 9 ) { prms[8].setI(tm_tm.tm_yday); prms[8].setModify(); }
+	if( prms.size() >= 10 ) { prms[9].setI(tm_tm.tm_isdst); prms[9].setModify(); }
+	return 0;
+    }
+    if( iid == "strftime" && !prms.empty() )
+    {
+	time_t tm_t = prms[0].getI();
+	struct tm tm_tm;
+	localtime_r(&tm_t,&tm_tm);
+	char buf[1000];
+	int rez = strftime( buf, sizeof(buf), (prms.size()>=2) ? prms[1].getS().c_str() : "%Y-%m-%d %H:%M:%S", &tm_tm );
+	return (rez>0) ? string(buf,rez) : "";
+    }
+    if( iid == "strptime" && !prms.empty() )
+    {
+	struct tm stm;
+	stm.tm_isdst = -1;
+	strptime( prms[0].getS().c_str(), (prms.size()>=2) ? prms[1].getS().c_str() : "%Y-%m-%d %H:%M:%S", &stm );
+	return (int)mktime(&stm);
+    }
+    if( iid == "cron" && !prms.empty() )
+	return (int)cron( prms[0].getS(), (prms.size()>=2) ? prms[1].getI() : 0 );
+    if( iid == "strFromCharCode" )
+    {
+	string rez;
+	for( int i_p = 0; i_p < prms.size(); i_p++ )
+	    rez += (unsigned char)prms[i_p].getI();
+	return rez;
+    }
+
+    return TCntrNode::objFuncCall(iid,prms,user);
 }
 
 void TSYS::cntrCmdProc( XMLNode *opt )
@@ -941,7 +1324,7 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	{
 	    ctrMkNode("fld",opt,-1,"/gen/id",_("ID"),R_R_R_,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/stat",_("Station"),RWRWR_,"root","root",1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/gen/prog",_("Programm"),0444,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/prog",_("Program"),0444,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/ver",_("Version"),0444,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/host",_("Host name"),0444,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/user",_("System user"),0444,"root","root",1,"tp","str");
@@ -960,7 +1343,7 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	    ctrMkNode("fld",opt,-1,"/gen/savePeriod",_("Save system period"),0664,"root","root",2,"tp","dec",
 		"help",_("Use no zero period (seconds) for periodic saving of changed systems parts to DB."));
 	    ctrMkNode("fld",opt,-1,"/gen/lang",_("Language"),RWRWR_,"root","root",1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/gen/baseLang",_("Variable texts' base language"),RWRWR_,"root","root",4,"tp","str","dest","select","select","/gen/baseLangLs",
+	    ctrMkNode("fld",opt,-1,"/gen/baseLang",_("Text variable's base language"),RWRWR_,"root","root",5,"tp","str","len","2","dest","sel_ed","select","/gen/baseLangLs",
 		"help",_("Multilangual for variable texts support enabling by base language selection."));
 	    if(ctrMkNode("area",opt,-1,"/gen/mess",_("Messages"),0444))
 	    {
@@ -972,9 +1355,23 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/gen/mess/log_arch",_("To archive"),0664,"root","root",1,"tp","bool");
 	    }
 	}
-	if(ctrMkNode("area",opt,-1,"/subs",_("Subsystems")))
+	if( ctrMkNode("area",opt,-1,"/subs",_("Subsystems")) )
 	    ctrMkNode("list",opt,-1,"/subs/br",_("Subsystems"),0444,"root","root",3,"idm","1","tp","br","br_pref","sub_");
-	if(ctrMkNode("area",opt,-1,"/hlp",_("Help")))
+	if( ctrMkNode("area",opt,-1,"/tasks",_("Tasks")) )
+	    if( ctrMkNode("table",opt,-1,"/tasks/tasks",_("Tasks"),R_R___,"root","root") )
+	    {
+		ctrMkNode("list",opt,-1,"/tasks/tasks/path",_("Path"),R_R___,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/tasks/tasks/thrd",_("Thread"),R_R___,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/tasks/tasks/plc",_("Policy"),R_R___,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/tasks/tasks/prior",_("Priority"),R_R___,"root","root",1,"tp","dec");
+	    }
+	if( !cntrEmpty() && ctrMkNode("area",opt,-1,"/cntr",_("Counters")) )
+	    if( ctrMkNode("table",opt,-1,"/cntr/cntr",_("Counters"),R_R___,"root","root") )
+	    {
+		ctrMkNode("list",opt,-1,"/cntr/cntr/id","ID",R_R___,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/cntr/cntr/vl",_("Value"),R_R___,"root","root",1,"tp","real");
+	    }
+	if( ctrMkNode("area",opt,-1,"/hlp",_("Help")) )
 	    ctrMkNode("fld",opt,-1,"/hlp/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","10");
 	return;
     }
@@ -987,11 +1384,7 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	opt->setText(TSYS::strEncode(TUIS::icoGet(id(),&itp),TSYS::base64));
 	opt->setAttr("tp",itp);
     }
-    else if( a_path == "/gen/host" && ctrChkNode(opt) )
-    {
-	utsname ubuf; uname(&ubuf);
-	opt->setText(ubuf.nodename);
-    }
+    else if( a_path == "/gen/host" && ctrChkNode(opt) )	opt->setText(host());
     else if( a_path == "/gen/sys" && ctrChkNode(opt) )
     {
 	utsname ubuf; uname(&ubuf);
@@ -1003,8 +1396,8 @@ void TSYS::cntrCmdProc( XMLNode *opt )
     else if( a_path == "/gen/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/gen/stat" )
     {
-	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEQ_RD) )	opt->setText(name());
-	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEQ_WR) )	setName(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEC_RD) )	opt->setText(name());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEC_WR) )	setName(opt->text());
     }
     else if( a_path == "/gen/frq" && ctrChkNode(opt) )	opt->setText(TSYS::real2str((float)sysClk()/1000000.,6));
     else if( a_path == "/gen/clk_res" && ctrChkNode(opt) )
@@ -1017,43 +1410,43 @@ void TSYS::cntrCmdProc( XMLNode *opt )
     else if( a_path == "/gen/config" && ctrChkNode(opt) )	opt->setText(mConfFile);
     else if( a_path == "/gen/wrk_db" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText(mWorkDB);
-	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEQ_WR) ) setWorkDB(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEC_RD) ) opt->setText(mWorkDB);
+	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEC_WR) ) setWorkDB(opt->text());
     }
     else if( a_path == "/gen/saveExit" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText( int2str(saveAtExit()) );
-	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEQ_WR) ) setSaveAtExit( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEC_RD) ) opt->setText( int2str(saveAtExit()) );
+	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEC_WR) ) setSaveAtExit( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/gen/savePeriod" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText( int2str(savePeriod()) );
-	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEQ_WR) ) setSavePeriod( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEC_RD) ) opt->setText( int2str(savePeriod()) );
+	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEC_WR) ) setSavePeriod( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/gen/workdir" )
     {
-	if( ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(workDir());
-	if( ctrChkNode(opt,"set",0440,"root","root",SEQ_WR) )	setWorkDir(opt->text().c_str());
+	if( ctrChkNode(opt,"get",0440,"root","root",SEC_RD) )	opt->setText(workDir());
+	if( ctrChkNode(opt,"set",0440,"root","root",SEC_WR) )	setWorkDir(opt->text().c_str());
     }
     else if( a_path == "/gen/icodir" )
     {
-	if( ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(icoDir());
-	if( ctrChkNode(opt,"set",0440,"root","root",SEQ_WR) )	setIcoDir(opt->text().c_str());
+	if( ctrChkNode(opt,"get",0440,"root","root",SEC_RD) )	opt->setText(icoDir());
+	if( ctrChkNode(opt,"set",0440,"root","root",SEC_WR) )	setIcoDir(opt->text().c_str());
     }
     else if( a_path == "/gen/moddir" )
     {
-	if( ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(modDir());
-	if( ctrChkNode(opt,"set",0440,"root","root",SEQ_WR) )	setModDir(opt->text().c_str());
+	if( ctrChkNode(opt,"get",0440,"root","root",SEC_RD) )	opt->setText(modDir());
+	if( ctrChkNode(opt,"set",0440,"root","root",SEC_WR) )	setModDir(opt->text().c_str());
     }
     else if( a_path == "/gen/lang" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(Mess->lang());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLang(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(Mess->lang());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	Mess->setLang(opt->text());
     }
     else if( a_path == "/gen/baseLang" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(Mess->lang2CodeBase());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLang2CodeBase(opt->text());
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(Mess->lang2CodeBase());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	Mess->setLang2CodeBase(opt->text());
     }
     else if( a_path == "/gen/baseLangLs" && ctrChkNode(opt) )
     {
@@ -1064,36 +1457,64 @@ void TSYS::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/gen/mess/lev" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TSYS::int2str(Mess->messLevel()));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setMessLevel(atoi(opt->text().c_str()));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText(TSYS::int2str(Mess->messLevel()));
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	Mess->setMessLevel(atoi(opt->text().c_str()));
     }
     else if( a_path == "/gen/mess/log_sysl" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x01)?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x01:Mess->logDirect()&(~0x01) );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText((Mess->logDirect()&0x01)?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x01:Mess->logDirect()&(~0x01) );
     }
     else if( a_path == "/gen/mess/log_stdo" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x02)?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x02:Mess->logDirect()&(~0x02) );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText((Mess->logDirect()&0x02)?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x02:Mess->logDirect()&(~0x02) );
     }
     else if( a_path == "/gen/mess/log_stde" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x04)?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x04:Mess->logDirect()&(~0x04) );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText((Mess->logDirect()&0x04)?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x04:Mess->logDirect()&(~0x04) );
     }
     else if( a_path == "/gen/mess/log_arch" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x08)?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x08:Mess->logDirect()&(~0x08) );
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )	opt->setText((Mess->logDirect()&0x08)?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x08:Mess->logDirect()&(~0x08) );
     }
-    else if( (a_path == "/br/sub_" || a_path == "/subs/br") && ctrChkNode(opt,"get",0444,"root","root",SEQ_RD) )
+    else if( (a_path == "/br/sub_" || a_path == "/subs/br") && ctrChkNode(opt,"get",0444,"root","root",SEC_RD) )
     {
 	vector<string> lst;
 	list(lst);
 	for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 	    opt->childAdd("el")->setAttr("id",lst[i_a])->setText(at(lst[i_a]).at().subName());
     }
-    else if( a_path == "/hlp/g_help" && ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) ) opt->setText(optDescr());
+    else if( a_path == "/tasks/tasks" && ctrChkNode(opt,"get",R_R___,"root","root") )
+    {
+	XMLNode *n_path	= ctrMkNode("list",opt,-1,"/tasks/tasks/path","",R_R___,"root","root");
+	XMLNode *n_thr	= ctrMkNode("list",opt,-1,"/tasks/tasks/thrd","",R_R___,"root","root");
+	XMLNode *n_plc	= ctrMkNode("list",opt,-1,"/tasks/tasks/plc","",R_R___,"root","root");
+	XMLNode *n_prior= ctrMkNode("list",opt,-1,"/tasks/tasks/prior","",R_R___,"root","root");
+
+	ResAlloc res(taskRes,false);
+	for( map<string,STask>::iterator it = mTasks.begin(); it != mTasks.end(); it++ )
+	{
+	    if( n_path )	n_path->childAdd("el")->setText( it->first );
+	    if( n_thr )		n_thr->childAdd("el")->setText( TSYS::uint2str(it->second.thr) );
+	    if( n_plc )		n_plc->childAdd("el")->setText( (it->second.policy==SCHED_RR)?_("Round-robin"):((it->second.policy==SCHED_BATCH)?_("Style \"batch\""):_("Standard")) );
+	    if( n_prior )	n_prior->childAdd("el")->setText( TSYS::int2str(it->second.prior) );
+	}
+    }
+    if( !cntrEmpty() && a_path == "/cntr/cntr" && ctrChkNode(opt,"get",R_R___,"root","root") )
+    {
+	XMLNode *n_id	= ctrMkNode("list",opt,-1,"/cntr/cntr/id","",R_R___,"root","root");
+	XMLNode *n_vl	= ctrMkNode("list",opt,-1,"/cntr/cntr/vl","",R_R___,"root","root");
+
+	ResAlloc res( nodeRes(), false );
+	for( map<string,double>::iterator icnt = mCntrs.begin(); icnt != mCntrs.end(); icnt++ )
+	{
+	    if( n_id )	n_id->childAdd("el")->setText( icnt->first );
+	    if( n_vl )	n_vl->childAdd("el")->setText( TSYS::real2str(icnt->second) );
+	}
+    }
+    else if( a_path == "/hlp/g_help" && ctrChkNode(opt,"get",0440,"root","root",SEC_RD) ) opt->setText(optDescr());
     else TCntrNode::cntrCmdProc(opt);
 }
