@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: ttipdaq.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -61,7 +61,7 @@ class TTipDAQ : public TModule, public TElem
 	TTipParam &tpPrmAt( unsigned id );
 
 	//> Compile functions support API
-	virtual void compileFuncLangs( vector<string> &ls )	{ }
+	virtual bool compileFuncLangs( vector<string> *ls = NULL )	{ return false; }
 	virtual string compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text, const string &usings = "" );
 
 	//> Redundance
