@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: ttransports.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -38,7 +38,7 @@ TTransportS::TTransportS( ) : TSubSYS("Transport","Transports",true)
     el_in.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
     el_in.fldAdd( new TFld("DESCRIPT",_("Description"),TFld::String,TCfg::TransltText,"500") );
     el_in.fldAdd( new TFld("ADDR",_("Address"),TFld::String,TFld::NoFlag,"50") );
-    el_in.fldAdd( new TFld("PROT",_("Transport protocol"),TFld::String,TFld::NoFlag,"20") );
+    el_in.fldAdd( new TFld("PROT",_("Transport protocol"),TFld::String,TFld::NoFlag,"50") );
     el_in.fldAdd( new TFld("START",_("To start"),TFld::Boolean,TFld::NoFlag,"1") );
 
     //> Output transport BD structure
@@ -114,7 +114,7 @@ void TTransportS::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -185,7 +185,7 @@ void TTransportS::load_( )
 	mess_err(nodePath().c_str(),_("Search and create new input transports error."));
     }
 
-    //-- Load external hosts --
+    //>> Load external hosts
     try
     {
 	TConfig c_el(&el_ext);
@@ -318,8 +318,7 @@ string TTransportS::optDescr( )
 {
     char buf[STR_BUF_LEN];
     snprintf(buf,sizeof(buf),_(
-	"======================= Subsystem \"Transports\" options ==================\n"
-	"------------ Parameters of section <%s> in config file -----------\n\n"
+	"======================= Subsystem \"Transports\" options ==================\n\n"
 	),nodePath().c_str());
 
     return(buf);
@@ -443,8 +442,8 @@ void TTransportS::cntrCmdProc( XMLNode *opt )
     if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0444) ) opt->setText(optDescr());
     else if( a_path == "/sub/sysHosts" )
     {
-	if( ctrChkNode(opt,"get",0660,"root","Transport",SEQ_RD) )	opt->setText( TSYS::int2str(sysHost()) );
-	if( ctrChkNode(opt,"set",0660,"root","Transport",SEQ_WR) )	setSysHost( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0660,"root","Transport",SEC_RD) )	opt->setText( TSYS::int2str(sysHost()) );
+	if( ctrChkNode(opt,"set",0660,"root","Transport",SEC_WR) )	setSysHost( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/sub/transps" && ctrChkNode(opt) )
     {
@@ -457,7 +456,7 @@ void TTransportS::cntrCmdProc( XMLNode *opt )
     {
 	TConfig c_el(&el_ext);
 	c_el.cfg("OP_USER").setS(opt->attr("user"));
-	if( ctrChkNode(opt,"get",0666,"root","Transport",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0666,"root","Transport",SEC_RD) )
 	{
 	    XMLNode *n_id	= ctrMkNode("list",opt,-1,"/sub/ehost/id","",0666);
 	    XMLNode *n_nm	= ctrMkNode("list",opt,-1,"/sub/ehost/name","",0666);
@@ -479,11 +478,11 @@ void TTransportS::cntrCmdProc( XMLNode *opt )
 		if( n_pass )	n_pass->childAdd("el")->setText(host.pass.size()?"*******":"");
 	    }
 	}
-	if( ctrChkNode(opt,"add",0666,"root","Transport",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0666,"root","Transport",SEC_WR) )
 	    extHostSet(ExtHost(sysHost()?"*":opt->attr("user"),"newHost","New external host","","",opt->attr("user"),""));
-	if( ctrChkNode(opt,"del",0666,"root","Transport",SEQ_WR) )
+	if( ctrChkNode(opt,"del",0666,"root","Transport",SEC_WR) )
 	    extHostDel(sysHost()?"*":opt->attr("user"),opt->attr("key_id") );
-	if( ctrChkNode(opt,"set",0666,"root","Transport",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0666,"root","Transport",SEC_WR) )
 	{
 	    string col  = opt->attr("col");
 	    ExtHost host = extHostGet(sysHost()?"*":opt->attr("user"),opt->attr("key_id"));
@@ -508,8 +507,8 @@ void TTransportS::cntrCmdProc( XMLNode *opt )
 //************************************************
 TTipTransport::TTipTransport()
 {
-    m_in = grpAdd("in_");
-    m_out = grpAdd("out_");
+    mIn = grpAdd("in_");
+    mOut = grpAdd("out_");
 }
 
 TTipTransport::~TTipTransport()
@@ -521,20 +520,20 @@ TTransportS &TTipTransport::owner( )	{ return (TTransportS&)TModule::owner(); }
 
 void TTipTransport::inAdd( const string &name, const string &idb )
 {
-    if( chldPresent(m_in,name) ) return;
-    chldAdd(m_in,In(name,idb));
+    if( chldPresent(mIn,name) ) return;
+    chldAdd(mIn,In(name,idb));
 }
 
 void TTipTransport::outAdd( const string &name, const string &idb )
 {
-    if( chldPresent(m_out,name) ) return;
-    chldAdd(m_out,Out(name,idb));
+    if( chldPresent(mOut,name) ) return;
+    chldAdd(mOut,Out(name,idb));
 }
 
 void TTipTransport::cntrCmdProc( XMLNode *opt )
 {
     vector<string> list;
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TModule::cntrCmdProc(opt);
@@ -547,37 +546,37 @@ void TTipTransport::cntrCmdProc( XMLNode *opt )
 	}
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/br/in_" || a_path == "/tr/in" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    inList(list);
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",list[i_a])->setText(inAt(list[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    inAdd(vid); inAt(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	inDel(opt->attr("id"),true);
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )	inDel(opt->attr("id"),true);
     }
     else if( a_path == "/br/out_" || a_path == "/tr/out" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    outList(list);
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",list[i_a])->setText(outAt(list[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"add",0664,"root","root",SEC_WR) )
 	{
-	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::oscdID);
 	    outAdd(vid); outAt(vid).at().setName(opt->text());
 	}
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	outDel(opt->attr("id"),true);
+	if( ctrChkNode(opt,"del",0664,"root","root",SEC_WR) )	outDel(opt->attr("id"),true);
     }
     else TModule::cntrCmdProc(opt);
 }
@@ -586,11 +585,11 @@ void TTipTransport::cntrCmdProc( XMLNode *opt )
 //* TTransportIn				 *
 //************************************************
 TTransportIn::TTransportIn( const string &iid, const string &idb, TElem *el ) :
-    TConfig(el), run_st(false), m_db(idb), m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()),
-    m_dscr(cfg("DESCRIPT").getSd()), m_addr(cfg("ADDR").getSd()), m_prot(cfg("PROT").getSd()),
-    m_start(cfg("START").getBd())
+    TConfig(el), run_st(false), mDB(idb), mId(cfg("ID").getSd()), mName(cfg("NAME").getSd()),
+    mDscr(cfg("DESCRIPT").getSd()), mAddr(cfg("ADDR").getSd()), mProt(cfg("PROT").getSd()),
+    mStart(cfg("START").getBd())
 {
-    m_id = iid;
+    mId = iid;
 }
 
 TTransportIn::~TTransportIn()
@@ -606,20 +605,22 @@ TCntrNode &TTransportIn::operator=( TCntrNode &node )
     string tid = id();
     *(TConfig*)this = *(TConfig*)src_n;
     cfg("MODULE").setS(owner().modId());
-    m_id = tid;
-    setDB(src_n->m_db);
+    mId = tid;
+    setDB(src_n->mDB);
 
     return *this;
 }
 
 TTipTransport &TTransportIn::owner( )	{ return *(TTipTransport*)nodePrev(); }
 
-string TTransportIn::name( )		{ return m_name.size()?m_name:m_id; }
+string TTransportIn::name( )		{ return mName.size()?mName:mId; }
 
 string TTransportIn::workId( )		{ return owner().modId()+"."+id(); }
 
 string TTransportIn::tbl( )		{ return owner().owner().subId()+"_in"; }
 
+string TTransportIn::protocol( )	{ return TSYS::strParse(protocolFull(),0,"."); }
+
 void TTransportIn::postDisable(int flag)
 {
     try
@@ -657,73 +658,86 @@ void TTransportIn::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TCntrNode::cntrCmdProc(opt);
-	ctrMkNode("oscada_cntr",opt,-1,"/",(_("Input transport: ")+name()).c_str(),0664,"root","root");
+	ctrMkNode("oscada_cntr",opt,-1,"/",(_("Input transport: ")+name()).c_str(),RWRWR_,"root","Transport");
 	if(ctrMkNode("area",opt,-1,"/prm",_("Transport")))
 	{
 	    if(ctrMkNode("area",opt,-1,"/prm/st",_("State")))
 	    {
-		ctrMkNode("fld",opt,-1,"/prm/st/status",_("Status"),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/st/st",_("Runing"),0664,"root","root",1,"tp","bool");
-		ctrMkNode("fld",opt,-1,"/prm/st/db",_("Transport DB"),0664,"root","root",4,"tp","str","dest","select","select","/db/list",
+		ctrMkNode("fld",opt,-1,"/prm/st/status",_("Status"),R_R_R_,"root","Transport",1,"tp","str");
+		ctrMkNode("fld",opt,-1,"/prm/st/st",_("Runing"),RWRWR_,"root","Transport",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/prm/st/db",_("Transport DB"),RWRWR_,"root","Transport",4,"tp","str","dest","select","select","/db/list",
 		    "help",_("DB address in format [<DB module>.<DB name>].\nFor use main work DB set '*.*'."));
 	    }
 	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Config")))
 	    {
-		ctrMkNode("fld",opt,-1,"/prm/cfg/id",cfg("ID").fld().descr(),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/name",cfg("NAME").fld().descr(),0664,"root","root",2,"tp","str","len","50");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/dscr",cfg("DESCRIPT").fld().descr(),0664,"root","root",3,"tp","str","cols","50","rows","3");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/prot",cfg("PROT").fld().descr(),0664,"root","root",3,"tp","str","dest","select","select","/prm/cfg/p_mod");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/start",cfg("START").fld().descr(),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/id",cfg("ID").fld().descr(),R_R_R_,"root","Transport",1,"tp","str");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/name",cfg("NAME").fld().descr(),RWRWR_,"root","Transport",2,"tp","str","len","50");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/dscr",cfg("DESCRIPT").fld().descr(),RWRWR_,"root","Transport",3,"tp","str","cols","90","rows","3");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),RWRWR_,"root","Transport",1,"tp","str");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/prot",cfg("PROT").fld().descr(),RWRWR_,"root","Transport",3,"tp","str","dest","select","select","/prm/cfg/p_mod");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/start",cfg("START").fld().descr(),RWRWR_,"root","Transport",1,"tp","bool");
 	    }
 	}
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/prm/st/status" && ctrChkNode(opt) )		opt->setText(getStatus());
     else if( a_path == "/prm/st/st" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(run_st?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	atoi(opt->text().c_str())?start():stop();
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(run_st?"1":"0");
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	atoi(opt->text().c_str())?start():stop();
     }
     else if( a_path == "/prm/st/db" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(DB());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setDB(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(DB());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setDB(opt->text());
     }
     else if( a_path == "/prm/cfg/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/prm/cfg/name" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(name());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setName(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(name());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setName(opt->text());
     }
     else if( a_path == "/prm/cfg/dscr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )  	opt->setText(dscr());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setDscr(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(dscr());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setDscr(opt->text());
     }
     else if( a_path == "/prm/cfg/addr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(addr());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setAddr(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(addr());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setAddr(opt->text());
     }
     else if( a_path == "/prm/cfg/prot" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(protocol());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setProtocol(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(protocolFull());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setProtocolFull(opt->text());
     }
     else if( a_path == "/prm/cfg/start" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( toStart() ? "1" : "0" );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setToStart( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( toStart() ? "1" : "0" );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setToStart( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/prm/cfg/p_mod" && ctrChkNode(opt) )
     {
 	vector<string> list;
-	SYS->protocol().at().modList(list);
+	int c_lv = 0;
+	string c_path = "", c_el;
+	for( int c_off = 0; (c_el=TSYS::strSepParse(protocolFull(),0,'.',&c_off)).size(); c_lv++ )
+	{
+	    opt->childAdd("el")->setText(c_path);
+	    c_path += c_lv ? "."+c_el : c_el;
+	}
+	opt->childAdd("el")->setText(c_path);
+	if( c_lv == 0 ) SYS->protocol().at().modList(list);
+	else
+	{
+	    c_path += ".";
+	    SYS->protocol().at().at(protocol()).at().itemListIn(list,protocolFull());
+	}
 	for( unsigned i_a=0; i_a < list.size(); i_a++ )
-	    opt->childAdd("el")->setText(list[i_a]);
+	    opt->childAdd("el")->setText(c_path+list[i_a]);
     }
     else TCntrNode::cntrCmdProc(opt);
 }
@@ -732,11 +746,11 @@ void TTransportIn::cntrCmdProc( XMLNode *opt )
 //* TTransportOut                                *
 //************************************************
 TTransportOut::TTransportOut( const string &iid, const string &idb, TElem *el ) :
-    TConfig(el), m_db(idb), run_st(false), m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()),
-    m_dscr(cfg("DESCRIPT").getSd()), m_addr(cfg("ADDR").getSd()), m_start(cfg("START").getBd()),
-    m_prm1(0), m_prm2(0)
+    TConfig(el), mDB(idb), run_st(false), mId(cfg("ID").getSd()), mName(cfg("NAME").getSd()),
+    mDscr(cfg("DESCRIPT").getSd()), mAddr(cfg("ADDR").getSd()), mStart(cfg("START").getBd()),
+    mPrm1(0), mPrm2(0)
 {
-    m_id = iid;
+    mId = iid;
 }
 
 TTransportOut::~TTransportOut( )
@@ -752,15 +766,15 @@ TCntrNode &TTransportOut::operator=( TCntrNode &node )
     string tid = id();
     *(TConfig*)this = *(TConfig*)src_n;
     cfg("MODULE").setS(owner().modId());
-    m_id = tid;
-    setDB(src_n->m_db);
+    mId = tid;
+    setDB(src_n->mDB);
 
     return *this;
 }
 
 TTipTransport &TTransportOut::owner( )	{ return *(TTipTransport*)nodePrev(); }
 
-string TTransportOut::name()		{ return m_name.size()?m_name:m_id; }
+string TTransportOut::name()		{ return mName.size()?mName:mId; }
 
 string TTransportOut::workId( )		{ return owner().modId()+"."+id(); }
 
@@ -804,38 +818,64 @@ void TTransportOut::messProtIO( XMLNode &io, const string &prot )
     SYS->protocol().at().at(prot).at().outMess( io, *this );
 }
 
+TVariant TTransportOut::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    if( iid == "messIO" && prms.size() >= 1 && prms[0].type() != TVariant::Object )
+    {
+	string rez;
+	char buf[STR_BUF_LEN];
+	try
+	{
+	    int resp_len = messIO( prms[0].getS().data(), prms[0].getS().size(), buf, sizeof(buf), (prms.size()>=2) ? 1e3*prms[1].getR() : 0 );
+	    rez.assign(buf,resp_len);
+	}catch(TError) { }
+
+	return rez;
+    }
+    else if( iid == "messIO" && prms.size() >= 2 && dynamic_cast<XMLNodeObj*>(prms[0].getO()) )
+    {
+	XMLNode req;
+	((XMLNodeObj*)prms[0].getO())->toXMLNode(req);
+	messProtIO(req,prms[1].getS());
+	((XMLNodeObj*)prms[0].getO())->fromXMLNode(req);
+	return 0;
+    }
+    return TCntrNode::objFuncCall(iid,prms,user);
+}
+
 void TTransportOut::cntrCmdProc( XMLNode *opt )
 {
     //> Get page info
     if( opt->name() == "info" )
     {
 	TCntrNode::cntrCmdProc(opt);
-	ctrMkNode("oscada_cntr",opt,-1,"/",(_("Output transport: ")+name()).c_str(),0664,"root","root");
+	ctrMkNode("oscada_cntr",opt,-1,"/",(_("Output transport: ")+name()).c_str(),RWRWR_,"root","Transport");
 	if( ctrMkNode("area",opt,-1,"/prm",_("Transport")) )
 	{
 	    if( ctrMkNode("area",opt,-1,"/prm/st",_("State")) )
 	    {
-		ctrMkNode("fld",opt,-1,"/prm/st/status",_("Status"),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/st/st",_("Runing"),0664,"root","root",1,"tp","bool");
-		ctrMkNode("fld",opt,-1,"/prm/st/db",_("Transport DB"),0664,"root","root",4,"tp","str","dest","select","select","/db/list",
+		ctrMkNode("fld",opt,-1,"/prm/st/status",_("Status"),R_R_R_,"root","Transport",1,"tp","str");
+		ctrMkNode("fld",opt,-1,"/prm/st/st",_("Runing"),RWRWR_,"root","Transport",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/prm/st/db",_("Transport DB"),RWRWR_,"root","Transport",4,"tp","str","dest","select","select","/db/list",
 		    "help",_("DB address in format [<DB module>.<DB name>].\nFor use main work DB set '*.*'."));
 	    }
 	    if( ctrMkNode("area",opt,-1,"/prm/cfg",_("Config")) )
 	    {
-		ctrMkNode("fld",opt,-1,"/prm/cfg/id",cfg("ID").fld().descr(),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/name",cfg("NAME").fld().descr(),0664,"root","root",2,"tp","str","len","50");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/dscr",cfg("DESCRIPT").fld().descr(),0664,"root","root",3,"tp","str","cols","50","rows","3");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/start",cfg("START").fld().descr(),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/id",cfg("ID").fld().descr(),R_R_R_,"root","Transport",1,"tp","str");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/name",cfg("NAME").fld().descr(),RWRWR_,"root","Transport",2,"tp","str","len","50");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/dscr",cfg("DESCRIPT").fld().descr(),RWRWR_,"root","Transport",3,"tp","str","cols","90","rows","3");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),RWRWR_,"root","Transport",1,"tp","str");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/start",cfg("START").fld().descr(),RWRWR_,"root","Transport",1,"tp","bool");
 	    }
 	}
-	if( startStat() && ctrMkNode("area",opt,-1,"/req",_("Request"),RWRW__,"root","root") )
+	if( startStat() && ctrMkNode("area",opt,-1,"/req",_("Request"),RWRW__,"root","Transport") )
 	{
-	    ctrMkNode("fld",opt,-1,"/req/tm",_("Time (ms)"),R_R___,"root","root",1,"tp","real");
-	    ctrMkNode("fld",opt,-1,"/req/mode",_("Mode"),RWRW__,"root","root",4,"tp","dec","dest","select","sel_id","0;1","sel_list",_("Text;Binary"));
-	    ctrMkNode("comm",opt,-1,"/req/send",_("Send"),RWRW__);
-	    ctrMkNode("fld",opt,-1,"/req/req",_("Request"),RWRW__,"root","root",3,"tp","str","cols","90","rows","5");
-	    ctrMkNode("fld",opt,-1,"/req/answ",_("Answer"),R_R___,"root","root",3,"tp","str","cols","90","rows","5");
+	    ctrMkNode("fld",opt,-1,"/req/tm",_("Time (ms)"),R_R___,"root","Transport",1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/req/mode",_("Mode"),RWRW__,"root","Transport",4,"tp","dec","dest","select","sel_id","0;1","sel_list",_("Text;Binary"));
+	    ctrMkNode("fld",opt,-1,"/req/toTmOut",_("Wait timeout"),RWRWR_,"root","Transport",1,"tp","bool");
+	    ctrMkNode("comm",opt,-1,"/req/send",_("Send"),RWRW__,"root","Transport");
+	    ctrMkNode("fld",opt,-1,"/req/req",_("Request"),RWRW__,"root","Transport",3,"tp","str","cols","90","rows","5");
+	    ctrMkNode("fld",opt,-1,"/req/answ",_("Answer"),R_R___,"root","Transport",3,"tp","str","cols","90","rows","5");
 	}
 	return;
     }
@@ -844,71 +884,78 @@ void TTransportOut::cntrCmdProc( XMLNode *opt )
     if( a_path == "/prm/st/status" && ctrChkNode(opt) )		opt->setText(getStatus());
     else if( a_path == "/prm/st/st" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(run_st?"1":"0");
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	atoi(opt->text().c_str())?start():stop();
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(run_st?"1":"0");
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	atoi(opt->text().c_str())?start():stop();
     }
     else if( a_path == "/prm/st/db" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(DB());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setDB(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(DB());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setDB(opt->text());
     }
     else if( a_path == "/prm/cfg/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/prm/cfg/name" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(name());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setName(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(name());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setName(opt->text());
     }
     else if( a_path == "/prm/cfg/dscr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(dscr());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setDscr(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(dscr());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setDscr(opt->text());
     }
     else if( a_path == "/prm/cfg/addr" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(addr());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setAddr(opt->text());
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText(addr());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setAddr(opt->text());
     }
     else if( a_path == "/prm/cfg/start" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( toStart() ? "1" : "0" );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setToStart( atoi(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",RWRWR_,"root","Transport",SEC_RD) )	opt->setText( toStart() ? "1" : "0" );
+	if( ctrChkNode(opt,"set",RWRWR_,"root","Transport",SEC_WR) )	setToStart( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/req/tm" && ctrChkNode(opt,"get",R_R___,"root","root",SEQ_RD) )
+    else if( a_path == "/req/tm" && ctrChkNode(opt,"get",R_R___,"root","Transport",SEC_RD) )
 	opt->setText(TBDS::genDBGet(owner().nodePath()+"ReqTm","0",opt->attr("user")));
     else if( a_path == "/req/mode" )
     {
-	if( ctrChkNode(opt,"get",RWRW__,"root","root",SEQ_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"ReqMode","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",RWRW__,"root","root",SEQ_WR) )	TBDS::genDBSet(owner().nodePath()+"ReqMode",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",RWRW__,"root","Transport",SEC_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"ReqMode","0",opt->attr("user")));
+	if( ctrChkNode(opt,"set",RWRW__,"root","Transport",SEC_WR) )	TBDS::genDBSet(owner().nodePath()+"ReqMode",opt->text(),opt->attr("user"));
+    }
+    else if( a_path == "/req/toTmOut" )
+    {
+	if( ctrChkNode(opt,"get",RWRW__,"root","Transport",SEC_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"ToTmOut","0",opt->attr("user")));
+	if( ctrChkNode(opt,"set",RWRW__,"root","Transport",SEC_WR) )	TBDS::genDBSet(owner().nodePath()+"ToTmOut",opt->text(),opt->attr("user"));
     }
     else if( a_path == "/req/req" )
     {
-	if( ctrChkNode(opt,"get",RWRW__,"root","root",SEQ_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"ReqReq","",opt->attr("user")));
-	if( ctrChkNode(opt,"set",RWRW__,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"get",RWRW__,"root","Transport",SEC_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"ReqReq","",opt->attr("user")));
+	if( ctrChkNode(opt,"set",RWRW__,"root","Transport",SEC_WR) )
 	{
 	    if( atoi(TBDS::genDBGet(owner().nodePath()+"ReqMode","0",opt->attr("user")).c_str()) == 1 )
 		TBDS::genDBSet(owner().nodePath()+"ReqReq",TSYS::strDecode(TSYS::strEncode(opt->text(),TSYS::Bin),TSYS::Bin),opt->attr("user"));
 	    else TBDS::genDBSet(owner().nodePath()+"ReqReq",opt->text(),opt->attr("user"));
 	}
     }
-    else if( a_path == "/req/answ" && ctrChkNode(opt,"get",R_R___,"root","root",SEQ_RD) )
+    else if( a_path == "/req/answ" && ctrChkNode(opt,"get",R_R___,"root","Transport",SEC_RD) )
 	opt->setText(TBDS::genDBGet(owner().nodePath()+"ReqAnsw","",opt->attr("user")));
-    else if( a_path == "/req/send" && startStat() && ctrChkNode(opt,"set",RWRW__,"root","root",SEQ_WR) )
+    else if( a_path == "/req/send" && startStat() && ctrChkNode(opt,"set",RWRW__,"root","Transport",SEC_WR) )
     {
 	string answ;
 	int mode = atoi(TBDS::genDBGet(owner().nodePath()+"ReqMode","0",opt->attr("user")).c_str());
 	string req = TBDS::genDBGet(owner().nodePath()+"ReqReq","",opt->attr("user"));
 
-	if( mode == 1 ) req = TSYS::strEncode(req,TSYS::Bin);
+	if( mode == 0 ) req = TSYS::strEncode(req,TSYS::ShieldSimb);
+	else if( mode == 1 ) req = TSYS::strEncode(req,TSYS::Bin);
 	if( !req.empty() )
 	{
 	    long long stm = TSYS::curTime( );
 	    char buf[STR_BUF_LEN];
-	    int resp_len = messIO(req.data(),req.size(),buf,sizeof(buf),5000);
+	    ResAlloc resN( nodeRes(), true );
+	    int resp_len = messIO(req.data(),req.size(),buf,sizeof(buf),0,true);
 	    answ.assign(buf,resp_len);
 
-	    while( resp_len == sizeof(buf) )
+	    while( resp_len == sizeof(buf) || atoi(TBDS::genDBGet(owner().nodePath()+"ToTmOut","0",opt->attr("user")).c_str()) )
 	    {
-		resp_len = messIO(NULL,0,buf,sizeof(buf),1000);
+		try{ resp_len = messIO(NULL,0,buf,sizeof(buf),0,true); } catch( TError err ) { break; }
 		answ.append(buf,resp_len);
 	    }
 	    TBDS::genDBSet(owner().nodePath()+"ReqTm",TSYS::real2str(1e-3*(TSYS::curTime()-stm)),opt->attr("user"));
