@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tvalue.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -144,7 +144,7 @@ void TValue::cntrCmdProc( XMLNode *opt )
     if( a_path == "/serv/attr"  )		//Attributes access
     {
 	vlList(list_c);
-	if( ctrChkNode(opt,"list",RWRWRW,"root","root",SEQ_RD) )	//Full info attributes list
+	if( ctrChkNode(opt,"list",RWRWRW,"root","root",SEC_RD) )	//Full info attributes list
 	{
 	    AutoHD<TVal> attr;
 	    for( int i_el = 0; i_el < list_c.size(); i_el++ )
@@ -159,59 +159,79 @@ void TValue::cntrCmdProc( XMLNode *opt )
 		    setAttr("names",attr.at().fld().selNames());
 	    }
 	}
-	if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEQ_RD) )		//All attributes values
+	if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEC_RD) )		//All attributes values
 	{
+	    long long vtm;
+	    string svl;
 	    AutoHD<TVal> vl;
-	    bool full = atoi(opt->attr("all").c_str());
-	    long long tm = atoll(opt->attr("tm").c_str());
-	    long long ftm = tm;
-	    for( int i_el = 0; i_el < list_c.size(); i_el++ )
-	    {
-		vl = vlAt(list_c[i_el]);
-		ftm = vmax(ftm,vl.at().time());
-		//>> Get last value
-		if( !tm ||												//Current value request
-			(vl.at().arch( ).freeStat() && (full || vl.at().time() > tm)) ||				//Updated value request
-			(!vl.at().arch( ).freeStat() && (vl.at().time()/vl.at().arch().at().period()-tm/vl.at().arch().at().period()) <= 1) )	//One value diff
-		    opt->childAdd("el")->setAttr("id",list_c[i_el])->setText(vl.at().getS());
-		//>> Get values from archive
-		else if( !vl.at().arch( ).freeStat() )
+	    XMLNode *aNd;
+
+	    //> Put last attribute's values
+	    bool sepReq = atoi(opt->attr("sepReq").c_str());
+	    bool hostTm = atoi(opt->attr("hostTm").c_str());
+	    if( !sepReq )
+		for( int i_el = 0; i_el < list_c.size(); i_el++ )
 		{
-		    AutoHD<TVArchive> arch = vl.at().arch();
-		    long long vper = arch.at().period(BUF_ARCH_NM);
-		    long long reqBeg = (tm/vper+1)*vper;
-		    long long vbeg = vmax(reqBeg,arch.at().begin(BUF_ARCH_NM));
-		    long long vend = arch.at().end(BUF_ARCH_NM);
-
-		    //>>> Longing to equivalent archivators
-		    if( vbeg == arch.at().begin(BUF_ARCH_NM) )
-		    {
-			vector<string> archLs;
-			arch.at().archivatorList(archLs);
-			for( int i_a = 0; i_a < archLs.size(); i_a++ )
-			    if( arch.at().period(archLs[i_a]) == vper )
-				vbeg = vmax(reqBeg,arch.at().begin(archLs[i_a]));
-		    }
-		    XMLNode *aNd = opt->childAdd("el")->setAttr("id",list_c[i_el])->
-			setAttr("tm",TSYS::ll2str(vbeg))->setAttr("per",TSYS::ll2str(vper));
-
-		    TValBuf buf(arch.at().valType(),0,0,false,true);
-		    arch.at().getVals( buf, vbeg, vend, "", (vend-vbeg)/vper, true );
-
-		    bool firstVal = true;
-		    string vl;
-		    for( vbeg = buf.begin(); vbeg <= buf.end(); vbeg++ )
-		    {
-			vl = buf.getS(&vbeg,true);
-			if( firstVal && vl == EVAL_STR ) continue;
-			if( firstVal && vl != EVAL_STR ) { aNd->setAttr("tm",TSYS::ll2str(vbeg)); firstVal = false; }
-			aNd->childAdd("v")->setText(vl);
-		    }
+		    vl = vlAt(list_c[i_el]);
+		    vtm = 0; svl = vl.at().getS(&vtm);
+		    aNd = opt->childAdd("el")->setAttr("id",list_c[i_el])->setText(svl);
+		    if( !hostTm ) aNd->setAttr("tm",TSYS::ll2str(vtm));
 		}
+
+	    //> Archives requests process
+	    for( int i_a = 0; i_a < opt->childSize(); i_a++ )
+	    {
+		aNd = opt->childGet(i_a);
+		if( !sepReq && aNd->name() != "ael" ) break;
+		if( (aNd->name() != "el" && aNd->name() != "ael") || !vlPresent(aNd->attr("id")) ) { opt->childDel(aNd); i_a--; continue; }
+		vl = vlAt(aNd->attr("id"));
+
+		//>> Separated element request
+		if( aNd->name() == "el" )
+		{
+		    vtm = 0; svl = vl.at().getS(&vtm);
+		    aNd->setText(svl);
+		    if( !hostTm ) aNd->setAttr("tm",TSYS::ll2str(vtm));
+		    continue;
+		}
+
+		//>> To archive of element request
+		if( vl.at().arch().freeStat() ) { opt->childDel(aNd); i_a--; continue; }
+
+		AutoHD<TVArchive> arch = vl.at().arch();
+		long long vper = arch.at().period(BUF_ARCH_NM);
+		long long reqBeg = (atoll(aNd->attr("tm").c_str())/vper+1)*vper;
+		long long vbeg = vmax(reqBeg,arch.at().begin(BUF_ARCH_NM));
+		long long vend = arch.at().end(BUF_ARCH_NM);
+
+		//>>> Longing to equivalent archivators
+		if( vbeg == arch.at().begin(BUF_ARCH_NM) )
+		{
+		    vector<string> archLs;
+		    arch.at().archivatorList(archLs);
+		    for( int i_a = 0; i_a < archLs.size(); i_a++ )
+			if( arch.at().period(archLs[i_a]) == vper )
+			    vbeg = vmax(reqBeg,arch.at().begin(archLs[i_a]));
+		}
+		aNd->setAttr("tm",TSYS::ll2str(vbeg))->setAttr("per",TSYS::ll2str(vper));
+
+		TValBuf buf(arch.at().valType(),0,0,false,true);
+		arch.at().getVals( buf, vbeg, vend, "", (vend-vbeg)/vper, true );
+
+		bool firstVal = true;
+		string vl;
+		for( vbeg = buf.begin(); vbeg <= buf.end(); vbeg++ )
+		{
+		    vl = buf.getS(&vbeg,true);
+		    if( firstVal && vl == EVAL_STR ) continue;
+		    if( firstVal && vl != EVAL_STR ) { aNd->setAttr("tm",TSYS::ll2str(vbeg)); firstVal = false; }
+		    aNd->childAdd("v")->setText(vl);
+		}
+
+		if( !aNd->childSize() ) { opt->childDel(aNd); i_a--; }
 	    }
-	    opt->setAttr("tm",TSYS::ll2str(ftm));
 	}
-	if( ctrChkNode(opt,"set",RWRWRW,"root","root",SEQ_WR) )		//Multi attributes set
+	if( ctrChkNode(opt,"set",RWRWRW,"root","root",SEC_WR) )		//Multi attributes set
 	    for( int i_el = 0; i_el < opt->childSize(); i_el++ )
 		vlAt(opt->childGet(i_el)->attr("id")).at().setS(opt->childGet(i_el)->text());
 	return;
@@ -263,14 +283,14 @@ void TValue::cntrCmdProc( XMLNode *opt )
 	    return;
 	}
 	AutoHD<TVal> vl = vlAt(TSYS::pathLev(a_path,1));
-	if( ctrChkNode(opt,"get",(vl.at().fld().flg()&TFld::NoWrite)?0440:0660,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",(vl.at().fld().flg()&TFld::NoWrite)?0440:0660,"root","root",SEC_RD) )
 	{
 	    if( vl.at().fld().flg()&TFld::Selected )	opt->setText(vl.at().getSEL());
 	    else opt->setText( (vl.at().fld().type()==TFld::Real) ?
 		    ((vl.at().getR()==EVAL_REAL) ? EVAL_STR : TSYS::real2str(vl.at().getR(),6)) : 
 		    vl.at().getS() );
 	}
-	if( ctrChkNode(opt,"set",(vl.at().fld().flg()&TFld::NoWrite)?0440:0660,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",(vl.at().fld().flg()&TFld::NoWrite)?0440:0660,"root","root",SEC_WR) )
 	{
 	    if( vl.at().fld().flg()&TFld::Selected )	vl.at().setSEL(opt->text());
 	    else					vl.at().setS(opt->text());
@@ -278,7 +298,7 @@ void TValue::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/arch/arch" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEC_RD) )
 	{
 	    vector<string> list_c2;
 	    //>>> Prepare headers
@@ -304,7 +324,7 @@ void TValue::cntrCmdProc( XMLNode *opt )
 			setText(vlAt(list_c[i_v]).at().arch().freeStat()?"0":(vlAt(list_c[i_v]).at().arch().at().archivatorPresent(c_id)?"1":"0"));
 		}
 	}
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	if( ctrChkNode(opt,"set",0664,"root","root",SEC_WR) )
 	{
 	    bool create = false;	//Create archive
 	    bool v_get = atoi(opt->text().c_str());
@@ -357,7 +377,7 @@ void TValue::cntrCmdProc( XMLNode *opt )
 //*************************************************
 //* TVal                                          *
 //*************************************************
-TVal::TVal( ) : mCfg(false), mTime(0)
+TVal::TVal( ) : mCfg(false), mTime(0), mResB1(false), mResB2(false), mReqFlg(false)
 {
     src.fld = NULL;
 }
@@ -442,7 +462,7 @@ TFld &TVal::fld()
 
 AutoHD<TVArchive> TVal::arch()	{ return mArch; }
 
-void TVal::setArch(const AutoHD<TVArchive> &vl)	{ mArch = vl; }
+void TVal::setArch( const AutoHD<TVArchive> &vl )	{ mArch = vl; }
 
 string TVal::getSEL( long long *tm, bool sys )
 {
@@ -467,6 +487,7 @@ string TVal::getS( long long *tm, bool sys )
 	case TFld::Boolean:
 	{ char vl = getB(tm,sys);	return (vl!=EVAL_BOOL) ? TSYS::int2str((bool)vl) : EVAL_STR; }
 	case TFld::String:
+	    setReqFlg(true);
 	    //> Get from archive
 	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() )
 		return mArch.at().getS(tm);
@@ -494,16 +515,17 @@ int TVal::getI( long long *tm, bool sys )
 	case TFld::Boolean:
 	{ char vl = getB(tm,sys);	return (vl!=EVAL_BOOL) ? (bool)vl : EVAL_INT; }
 	case TFld::Integer:
-	    //- Get from archive -
+	    setReqFlg(true);
+	    //> Get from archive
 	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() ) 
 		return mArch.at().getI(tm);
-	    //- Get value from config -
+	    //> Get value from config
 	    if( mCfg )
 	    {
 		if(tm) *tm = TSYS::curTime();
 		return src.cfg->getI( );
 	    }
-	    //- Get current value -
+	    //> Get current value
 	    if( fld().flg()&TVal::DirRead && !sys )	owner().vlGet( *this );
 	    if( tm ) *tm = time();
 	    return val.val_i;
@@ -521,6 +543,7 @@ double TVal::getR( long long *tm, bool sys )
 	case TFld::Boolean:
 	{ char vl = getB(tm,sys);	return (vl!=EVAL_BOOL) ? (bool)vl : EVAL_REAL; }
 	case TFld::Real:
+	    setReqFlg(true);
 	    //> Get from archive
 	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() )
 		return mArch.at().getR(tm);
@@ -548,16 +571,17 @@ char TVal::getB( long long *tm, bool sys )
 	case TFld::Real:
 	{ double vl = getR(tm,sys);	return (vl!=EVAL_REAL) ? (bool)vl : EVAL_BOOL; }
 	case TFld::Boolean:
-	    //- Get from archive -
+	    setReqFlg(true);
+	    //> Get from archive
 	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() )
 		return mArch.at().getB(tm);
-	    //- Get value from config -
+	    //> Get value from config
 	    if( mCfg )
 	    {
 		if(tm) *tm = TSYS::curTime();
 		return src.cfg->getB( );
 	    }
-	    //- Get current value -
+	    //> Get current value
 	    if( fld().flg()&TVal::DirRead && !sys )	owner().vlGet( *this );
 	    if( tm ) *tm = time();
 	    return val.val_b;
@@ -598,7 +622,8 @@ void TVal::setS( const string &value, long long tm, bool sys )
 	    if( fld().flg()&TVal::DirWrite && !sys )	owner().vlSet( *this, pvl );
 	    //> Set to archive
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
-		mArch.at().setS(value,time());
+		try{ mArch.at().setS(value,time()); }
+		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
     }
 }
 
@@ -627,7 +652,8 @@ void TVal::setI( int value, long long tm, bool sys )
 	    if( fld().flg()&TVal::DirWrite && !sys )	owner().vlSet( *this, pvl );
 	    //> Set to archive
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
-		mArch.at().setI(value,time());
+		try{ mArch.at().setI(value,time()); }
+		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
 	}
     }
 }
@@ -657,7 +683,8 @@ void TVal::setR( double value, long long tm, bool sys )
 	    if( fld().flg()&TVal::DirWrite && !sys )	owner().vlSet( *this, pvl );
 	    //> Set to archive
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
-		mArch.at().setR(value,time());
+		try{ mArch.at().setR(value,time()); }
+		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
 	}
     }
 }
@@ -685,18 +712,69 @@ void TVal::setB( char value, long long tm, bool sys )
 	    if( fld().flg()&TVal::DirWrite && !sys )	owner().vlSet( *this, pvl );
 	    //> Set to archive
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
-		mArch.at().setB(value,time());
+		try{ mArch.at().setB(value,time()); }
+		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
 	}
     }
 }
 
+TVariant TVal::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    if( iid == "get" )
+    {
+	try
+	{
+	    TVariant rez;
+	    long long tm = 0;
+	    if( prms.size() >= 1 ) tm = (long long)prms[0].getI()*1000000;
+	    if( prms.size() >= 2 ) tm += prms[1].getI();
+	    bool isSys = false;
+	    if( prms.size() >= 3 ) isSys = prms[2].getB();
+	    switch( fld().type() )
+	    {
+		case TFld::Boolean:	rez = getB(&tm,isSys);	break;
+		case TFld::Integer:	rez = getI(&tm,isSys);	break;
+		case TFld::Real:	rez = getR(&tm,isSys);	break;
+		case TFld::String:	rez = getS(&tm,isSys);	break;
+	    }
+	    if( prms.size() >= 1 )	{ prms[0].setI(tm/1000000); prms[0].setModify(); }
+	    if( prms.size() >= 2 )	{ prms[1].setI(tm%1000000); prms[1].setModify(); }
+
+	    return rez;
+	}catch(...){ }
+	return EVAL_REAL;
+    }
+    if( iid == "set" && prms.size() >= 1 )
+    {
+	try
+	{
+	    long long tm = 0;
+	    if( prms.size() >= 2 ) tm = (long long)prms[1].getI()*1000000;
+	    if( prms.size() >= 3 ) tm += prms[2].getI();
+	    bool isSys = false;
+	    if( prms.size() >= 4 ) isSys = prms[3].getB();
+	    switch( fld().type() )
+	    {
+		case TFld::Boolean:	setB(prms[0].getB(),tm,isSys);	break;
+		case TFld::Integer:	setI(prms[0].getI(),tm,isSys);	break;
+		case TFld::Real:	setR(prms[0].getR(),tm,isSys);	break;
+		case TFld::String:	setS(prms[0].getS(),tm,isSys);	break;
+	    }
+	    return false;
+	}catch(...){ }
+	return true;
+    }
+
+    return TCntrNode::objFuncCall(iid,prms,user);
+}
+
 void TVal::cntrCmdProc( XMLNode *opt )
 {
     string a_path = opt->attr("path");
     //> Service commands process
     if( a_path == "/serv/val"  )		//Values access
     {
-	if( ctrChkNode(opt,"info",RWRWRW,"root","root",SEQ_RD) )	//Value's data information
+	if( ctrChkNode(opt,"info",RWRWRW,"root","root",SEC_RD) )	//Value's data information
 	{
 	    if( !arch().freeStat() ) arch().at().cntrCmdProc(opt);
 	    else
@@ -705,7 +783,7 @@ void TVal::cntrCmdProc( XMLNode *opt )
 		opt->setAttr("vtp",TSYS::int2str(fld().type()));
 	    }
 	}
-	else if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEQ_RD) )    //Value's data request
+	else if( ctrChkNode(opt,"get",RWRWRW,"root","root",SEC_RD) )    //Value's data request
 	{
 	    if( !atoll(opt->attr("tm_grnd").c_str()) && opt->attr("arch").empty() )
 	    {
