@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: tvalue.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -77,6 +77,14 @@ class TVal : public TCntrNode
 	AutoHD<TVArchive> arch( );
 	void setArch( const AutoHD<TVArchive> &vl );
 
+	bool reqFlg( )	{ return mReqFlg; }
+	bool resB1( )	{ return mResB1; }
+	bool resB2( )	{ return mResB2; }
+
+	bool setReqFlg( bool vl )	{ mReqFlg = vl; }
+	bool setResB1( bool vl )	{ mResB1 = vl; }
+	bool setResB2( bool vl )	{ mResB2 = vl; }
+
 	TValue &owner( );
 	TFld &fld( );
 
@@ -84,6 +92,8 @@ class TVal : public TCntrNode
 	//Methods
 	void cntrCmdProc( XMLNode *opt );
 
+	TVariant objFuncCall( const string &id, vector<TVariant> &prms, const string &user );
+
     private:
 	//Methods
 	string nodeName( )	{ return name(); }
@@ -97,7 +107,10 @@ class TVal : public TCntrNode
 	    char	val_b;		//boolean value
 	} val;
 
-	bool	mCfg;		//Config id
+	unsigned char	mCfg	: 1;		//Config id
+	unsigned char	mReqFlg	: 1;		//Request to attribute flag
+	unsigned char	mResB1	: 1;		//Reserve boolean
+	unsigned char	mResB2	: 1;		//Reserve boolean
 	union
 	{
 	    TFld *fld;
