@@ -1,8 +1,8 @@
 
 //OpenSCADA system file: xml.h
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
- *   rom_as@fromru.com                                                     *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
+ *   rom_as@oscada.org, rom_as@fromru.com                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -40,14 +40,15 @@ class XMLNode
 	//Data
 	enum SaveView
 	{
-	    BrOpenPrev		= 0x01,		//Break preview open tag
-	    BrOpenPast		= 0x02,		//Break past open tag
-	    BrClosePast		= 0x04,		//Break past close tag
-	    BrTextPast		= 0x08,		//Break past text
-	    BrPrcInstrPast	= 0x10,		//Break past process instruction
-	    BrAllPast		= 0x1E,		//Break past all
+	    BrOpenPrev		= 0x01,		//Break before open tag
+	    BrOpenPast		= 0x02,		//Break after open tag
+	    BrClosePast		= 0x04,		//Break after close tag
+	    BrTextPast		= 0x08,		//Break after text
+	    BrPrcInstrPast	= 0x10,		//Break after process instruction
+	    BrAllPast		= 0x1E,		//Break after all
 	    XMLHeader		= 0x20,		//Include XML header
-	    InclNode		= 0x80		//Included node, not root
+	    MissTagEnc		= 0x100,	//Miss tag name encode
+	    MissAttrEnc		= 0x200		//Miss attribute name encode
 	};
 
 	//Methods
@@ -62,7 +63,7 @@ class XMLNode
 	string	text( ) const			{ return mText; }
 	XMLNode* setText( const string &s )	{ mText = s; return this; }
 
-	void	attrList( vector<string> & list ) const;
+	void	attrList( vector<string> &list ) const;
 	XMLNode* attrDel( const string &name );
 	void	attrClear( );
 	string	attr( const string &name ) const;
@@ -78,6 +79,7 @@ class XMLNode
 	string	save( unsigned char flgs = 0 );
 	XMLNode* clear( );
 
+	bool	childEmpty( ) const		{ return mChildren.empty(); }
 	int	childSize( ) const		{ return mChildren.size(); }
 	void	childAdd( XMLNode *nd );
 	XMLNode* childAdd( const string &name = "" );
@@ -91,9 +93,11 @@ class XMLNode
 	XMLNode* childGet( const string &attr, const string &name, bool noex = false ) const;
 
 	XMLNode* parent( )			{ return mParent; }
+
     private:
 	//Methods
-	string encode( const string &s, bool text = false ) const;
+	void save( unsigned char flg, string &xml );
+	void encode( const string &s, string &rez, bool text = false ) const;
 
 	static void start_element( void *data, const char *el, const char **attr );
 	static void end_element( void *data, const char *el );
@@ -108,7 +112,7 @@ class XMLNode
 	vector< pair<string,string> >	mPrcInstr;
 	XMLNode* mParent;
 
-	//- Parse/load XML attributes -
+	//> Parse/load XML attributes
 	vector<XMLNode*> node_stack;
 };
 
